# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from datetime import datetime
from openerp.tools.translate import _


class apac_visitantes(osv.Model):
    _name = 'apac.visitantes'
    _inherit = 'apac.visitantes'

    def apcvis_mostra_pessoa(self, cr, uid, pessoa_ids):
        obj_pessoa = self.pool.get('adm.pessoa')
        dados_pessoa = obj_pessoa.browse(cr, uid, pessoa_ids)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]
        endereco_ids = []
        for dados_endereco in dados_pessoa.endereco_ids:
            enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                         'active': dados_endereco.active,
                         "cep": dados_endereco.cep,
                         "pais_id": dados_endereco.pais_id.id,
                         "uf_id": dados_endereco.uf_id.id,
                         "cidade_id": dados_endereco.cidade_id.id,
                         "bairro_id": dados_endereco.bairro_id.id,
                         "logradouro_id": dados_endereco.logradouro_id.id,
                         "numero": dados_endereco.numero,
                         "complemento": dados_endereco.complemento,
                         "regiao_id": dados_endereco.regiao_id.id,
                         "observacao": dados_endereco.observacao,
            }
            endereco_ids.append((1, dados_endereco.id, enderecos))
        email_ids = []
        for dados_emails in dados_pessoa.email_ids:
            emails = {
                "email_tipo_id": dados_emails.email_tipo_id.id,
                "name": dados_emails.name,
                "observacao": dados_emails.observacao,
            }
            email_ids.append((1, dados_emails.id, emails))
        telefones_ids = []
        for dados_telefones in dados_pessoa.telefones_ids:
            telefones = {
                "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                "ddi": dados_telefones.ddi,
                "ddd": dados_telefones.ddd,
                "numero": dados_telefones.numero,
                "observacoes": dados_telefones.observacoes,
            }
            telefones_ids.append((1, dados_telefones.id, telefones))
        cpf_cnpj = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(dados_pessoa.pessoa_tipo, dados_pessoa.cpf_cnpj,
                                                                     True)
        res = {'value': {'cpf_cnpj': cpf_cnpj,
                         'name': dados_pessoa.name,
                         'fantasia_apelido': dados_pessoa.fantasia_apelido,
                         'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                         'admpes_fis_rg_orgao_emissor': dados_pessoa.admpes_fis_rg_orgao_emissor,
                         'admpes_fis_rg_data_emisssao': dados_pessoa.admpes_fis_rg_data_emisssao,
                         'admpes_fis_cnh_nro': dados_pessoa.admpes_fis_cnh_nro,
                         'admpes_fis_cnh_orgao_emissor': dados_pessoa.admpes_fis_cnh_orgao_emissor,
                         'admpes_fis_cnh_validade': dados_pessoa.admpes_fis_cnh_validade,
                         'admpes_fis_cnh_categoria': dados_pessoa.admpes_fis_cnh_categoria,
                         'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                         'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                         'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                         "foto": dados_pessoa.foto,
                         'endereco_ids': endereco_ids,
                         'email_ids': email_ids,
                         'telefones_ids': telefones_ids,
                         'apcvis_pessoa_id': dados_pessoa.id,
        }
        }
        return res

    def apcvis_onchange_cpf_cnpj(self, cr, uid, ids, pessoa_tipo, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar(pessoa_tipo, cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            if pessoa_tipo == 'F':
                warning['message'] = _('CPF Invalido!  Digite Novamente!')
            else:
                warning['message'] = _('CNPJ Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', pessoa_tipo)],
                                       context=context)
        if pessoa_ids:
            res = self.apcvis_mostra_pessoa(cr, uid, pessoa_ids)
            return res
        return {'warning': warning, 'value': {'cpf_cnpj': cpf_cnpj}}

    def apcvis_onchange_rg(self, cr, uid, ids, rg, context=None):
        """
        Metodo invocado na saida do campo RG da tela.
        """
        context = context and context.copy() or {}
        warning = {}
        if not rg:
            return {'warning': warning, 'value': {}}
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(cr, uid, [('admpes_fis_rg', '=', rg)], context=context)
        if pessoa_ids:
            res = self.apcvis_mostra_pessoa(cr, uid, pessoa_ids)
            return res
        return {'warning': warning, 'value': {}}

    def apcvis_onchange_cnh(self, cr, uid, ids, cnh, context=None):
        """
        Metodo invocado na saida do campo CNH da tela.
        """
        context = context and context.copy() or {}
        warning = {}
        if not cnh:
            return {'warning': warning, 'value': {}}
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(cr, uid, [('admpes_fis_cnh_nro', '=', cnh)], context=context)
        if pessoa_ids:
            res = self.apcvis_mostra_pessoa(cr, uid, pessoa_ids)
            return res
        return {'warning': warning, 'value': {}}

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        result = super(apac_visitantes, self).create(cr, uid, vals, context)
        return result

    def button_saida(self, cr, uid, ids, context=None):
        res = self.write(cr, uid, ids, {'apcvis_data_hora_saida': datetime.now()})
        return res
