# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv
import re
from openerp.tools.translate import _


class asfipo_cadastro(osv.Model):
    _name = 'asfipo.cadastro'
    _inherit = 'asfipo.cadastro'

    def asfcad_mostra_tela(self, cr, uid, asfcad_assistido_id):
        obj_adm_pessoa = self.pool.get('adm.pessoa')
        dados_pessoa = obj_adm_pessoa.browse(cr, uid, asfcad_assistido_id)
        if isinstance(dados_pessoa, list):
            dados_pessoa = dados_pessoa[0]
        endereco_ids = []
        for dados_endereco in dados_pessoa.endereco_ids:
            enderecos = {'endereco_tipo_id': dados_endereco.endereco_tipo_id.id,
                         'active': dados_endereco.active,
                         "cep": dados_endereco.cep,
                         "pais_id": dados_endereco.pais_id.id,
                         "uf_id": dados_endereco.uf_id.id,
                         "cidade_id": dados_endereco.cidade_id.id,
                         "bairro_id": dados_endereco.bairro_id.id,
                         "logradouro_id": dados_endereco.logradouro_id.id,
                         "numero": dados_endereco.numero,
                         "complemento": dados_endereco.complemento,
                         "observacao": dados_endereco.observacao,
            }
            endereco_ids.append((1, dados_endereco.id, enderecos))
        email_ids = []
        for dados_emails in dados_pessoa.email_ids:
            emails = {
                "email_tipo_id": dados_emails.email_tipo_id.id,
                "name": dados_emails.name,
                "observacao": dados_emails.observacao,
            }
            email_ids.append((1, dados_emails.id, emails))
        telefones_ids = []
        for dados_telefones in dados_pessoa.telefones_ids:
            telefones = {
                "telefone_tipo_id": dados_telefones.telefone_tipo_id.id,
                "ddi": dados_telefones.ddi,
                "ddd": dados_telefones.ddd,
                "numero": dados_telefones.numero,
                "observacoes": dados_telefones.observacoes,
            }
            telefones_ids.append((1, dados_telefones.id, telefones))
        _campos = {'cpf_cnpj': dados_pessoa.cpf_cnpj,
                   'name': dados_pessoa.name,
                   'codigo': dados_pessoa.codigo,
                   'fantasia_apelido': dados_pessoa.fantasia_apelido,
                   'admpes_fis_rg': dados_pessoa.admpes_fis_rg,
                   'admpes_fis_sexo': dados_pessoa.admpes_fis_sexo,
                   'admpes_fis_estado_civil': dados_pessoa.admpes_fis_estado_civil,
                   'admpes_fis_data_nascimento': dados_pessoa.admpes_fis_data_nascimento,
                   "foto": dados_pessoa.foto,
                   'endereco_ids': endereco_ids,
                   'email_ids': email_ids,
                   'telefones_ids': telefones_ids,
                   'asssoc_pessoa_id': dados_pessoa.id,
        }
        # Forcar um id qualquer, para que o campo assoc_pessoa_id desapareca da tela
        _campos['id'] = 1
        res = {'value': _campos}
        return res

    def asfcad_onchange_assistido_id(self, cr, uid, ids, asfcad_assistido_id, context=None):
        context = context and context.copy() or {}
        warning = {}
        if not asfcad_assistido_id:
            return {'warning': warning, 'value': {}}
        res = self.asfcad_mostra_tela(cr, uid, asfcad_assistido_id)
        return res

    def asfcad_onchange_cpf_cnpj(self, cr, uid, ids, cpf_cnpj, context=None):
        """
        Metodo invocado na saida do campo cpf/cnpj da tela.
        Valida o cpf/cnpj e caso esteja correto, devolve o cpf/cnpj formatado
        """
        context = context and context.copy() or {}
        warning = {}
        if not cpf_cnpj:
            return {'warning': warning, 'value': {}}
        res = self.pool.get('adm.utils').admutl_cpf_cnpj_testar('F', cpf_cnpj, True)
        if res:
            cpf_cnpj = res
        else:
            warning = {'title': _('Erro na validacao')}
            warning['message'] = _('CPF Invalido!  Digite Novamente!')
            return {'warning': warning, 'value': {'cpf_cnpj': ''}}
        _cnpj_cpf_sem_formatar = re.sub('[^0-9]', '', cpf_cnpj)
        obj_pessoa = self.pool.get('adm.pessoa')
        pessoa_ids = obj_pessoa.search(cr, uid,
                                       [('cpf_cnpj', '=', _cnpj_cpf_sem_formatar), ('pessoa_tipo', '=', 'F')],
                                       context=context)
        if pessoa_ids:
            res = self.asfcad_mostra_tela(cr, uid, pessoa_ids)
        else:
            res = {'value': {'cpf_cnpj': res}}
        return res

    def asfcad_wizard_oficio(self, cr, uid, ids, context=None):
        return {
            'name': 'Imprimir Oficio',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'asfipo.imprimir.oficio',
            'view_id': False,
            'target': 'new',
            'type': 'ir.actions.act_window',
            'context': context,
        }

    def sequence_criar(self, cr, uid, context=None):
        context = context and context.copy() or {}
        company_id = 1
        __code = 'asfipo.nro_entrevista'
        __name = 'Numero da Entrevista'
        vals = {
            'name': __name,
            'code': __code,
            'prefix': '%(year)s-',
            'padding': 6,
            'company_id': company_id,
            'implementation': 'no_gap',
        }
        consulta = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', __code)], context=context)
        if not consulta:
            self.pool.get('ir.sequence').create(cr, uid, {'name': __name, 'code': __code}, context)
            #todo: verificar se e possivel isolar a transacao para apenas comitar no ir.sequence
            cr.commit()
        self.pool.get('ir.sequence').create(cr, uid, vals, context)
        #todo: verificar se e possivel isolar a transacao para apenas comitar no ir.sequence
        cr.commit()
        return True

    def create(self, cr, uid, vals, context=None):
        vals['asfcad_nro_entrevista'] = self.pool.get('adm.utils').get_sequence(cr, uid, self._name,
                                                                                'asfipo.nro_entrevista')
        result = super(asfipo_cadastro, self).create(cr, uid, vals, context)
        return result
