#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel Franca
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

import math
from openerp.osv import osv
import numpy as np
from numpy.linalg import inv
from openerp.tools.translate import _


class Eqvt():
    def __init__(self, B0, B1, B2,dap,ht):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.dap = dap
        self.ht = ht

    def vt_result(self):
        vt=math.exp(self.B0 +self.B1 * math.log(self.dap) + self.B2 * math.log(self.ht))
        return vt


#  Classe para definir altura comercial a partir do menor diametro (que compensa serrar)
class Eqhi():
    def __init__(self, B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite):
        self.B0 = B0f_hi
        self.B1 = B1f_hi
        self.B2 = B2f_hi
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap
        self.di_limite=di_limite

    def hi_result(self):

        if self.di_limite + 0.5 < self.dap:
            # calcula di com casca(di_casc) na altura atual (hi_atual)
            hi = math.sqrt(
                self.B0+self.B1 * (self.di_limite / self.dap) + self.B2 * (self.di_limite / self.dap) ** 2) * self.ht
        else:
            hi = 0

        return hi
# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------


# Formulas
def equacao_volume(dap, ht, b0=0, b1=0, b2=0):
    # equação volume por árvore ILPF GG100 2011
    vt = math.exp(b0 + b1 * math.log(dap) + b2 * math.log(ht))

    # equação volume por árvore ILPF MARAVILHAS 2009
    # vt = math.exp(-10.5312815270227 + 1.7700934601883 * math.log(dap) + 1.29205471199543 * math.log(ht))
    return vt


class Eqdi_casc():
    def __init__(self, B0, B1, B2, hi_atual, hi, ht, dap, di_casc, di_casc2):
        self.B0 = B0
        self.B1 = B1
        self.B2 = B2
        self.hi_atual=hi_atual
        self.hi=hi
        self.ht = ht
        self.dap = dap
        self.di_casc=di_casc
        self.di_casc2=di_casc2

    def di_casc2_result(self):
        # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
        di_casc2 = math.sqrt(self.B0 +self.B1* ((self.hi_atual + self.hi) / self.ht) + self.B2 * (
                    (self.hi_atual + self.hi) / self.ht) ** 2) * self.dap

        return di_casc2

    def di_casc_result(self):

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            self.B0+self.B1 * (self.hi_atual / self.ht) + self.B2 * (self.hi_atual / self.ht) ** 2) * self.dap

        return di_casc


# ----------------------------------------------------------------------------------------------------------------

# Equacao de Taper
def equacao_forma(hi_atual, hi, ht, dap, di_casc, di_casc2, b0=0, b1=0, b2=0):
    # if eqUsuario == 1 and eqForma == 1:

    # 'EQUAÇÃO TAPER (KOZAK)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = Sqr(Range("n2") + Range("n3") * ((hi_atual + hi) / ht) + Range("n4") * ((hi_atual + hi) / ht) ^ 2) * Dap

    # 'calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (Range("n2") + Range("n3") * (hi_atual / ht) + Range("n4") * (hi_atual / ht) ^ 2) * Dap
    # Else

    # EQUAÇÃO TAPER (KOZAK) ILPF GG100 2011
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    try:
        di_casc2 = math.sqrt(
            b0 - (b1 * -1) * ((hi_atual + hi) / ht) + b2 * ((hi_atual + hi) / ht) ** 2) * dap

        # calcula di com casca(di_casc) na altura atual (hi_atual)
        di_casc = math.sqrt(
            b0 - (b1 * -1) * (hi_atual / ht) + b2 * (hi_atual / ht) ** 2) * dap
        return di_casc, di_casc2
    except Exception:
        raise osv.except_osv(_('Atencao'),
                             _('Dados de entrada para calculo incorreto'))

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV1 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)

    # if hi != 0:
    #     di_casc2 = (1.04317139756795 - 1.15251507113576 * ((hi_atual + hi) / ht) + 0.745608873884745 * math.pow((
    #     (hi_atual + hi) / ht), 2)) * dap
    # else:
    #     #calcula di com casca(di_casc) na altura atual (hi_atual)
    #     di_casc = (1.04317139756795 - 1.15251507113576 * (hi_atual / ht) + 0.745608873884745 * math.pow((hi_atual / ht), 2)) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV2 (CITRIODORA GUARITA 2) Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.10041792322808 - 1.46182396963 * ((hi_atual + hi) / ht) + 0.77535958077 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.10041792322808 - 1.46182396963 * (hi_atual / ht) + 0.77535958077 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER MODIFICADA - di/dap sem elevar ao quadrado(KOZAK) DA ÁRVORE ARV3 (CITRIODORA GUARITA 2) s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # di_casc2 = (1.04934898366772 - 1.01123933349793 * ((hi_atual + hi) / ht) - 0.231797825742175 * ((hi_atual + hi) / ht) ^ 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # di_casc = (1.04934898366772 - 1.01123933349793 * (hi_atual / ht) - 0.231797825742175 * (hi_atual / ht) ^ 2) * dap

    # EQUAÇÃO TAPER (KOZAK) MARAVILHAS s/toco Dados cubados
    # calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # if hi!=0:
    # di_casc2 = math.sqrt(1.03698591927493 - 1.45223087205717 * ((hi_atual + hi) / ht) + 0.428220947928028 * ((hi_atual + hi) / ht) ** 2) * dap

    # calcula di com casca(di_casc) na altura atual (hi_atual)
    # else:
    # di_casc = math.sqrt(1.03698591927493 - 1.45223087205717 * (hi_atual / ht) + 0.428220947928028 * (hi_atual / ht) ** 2) * dap

# ----------------------------------------------------------------------------------------------------------------

def formulas(fatia, nes1, nes2, es, es_serra, di, ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s):
    # Formulas metodo Radial

    # angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # 1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)
    # angulos direito, esquerdo (Fatia0), inferior, superior (Fatia1)
    # cordas direita, esquerda (Fatia0), inferior, superior (Fatia1)
    # larguras esquerda, direita, inferior, superior da peça (atualiza com desconto da espessura da peça)

    if fatia == 0:

        ang_e = math.acos(nes1 / (di / 2)) * 180 / math.pi
        ang_d = math.acos((nes1 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_e = di * math.sin(ang_e * math.pi / 180)
        cord_d = di * math.sin(ang_d * math.pi / 180)

        l_e = (cord_e / 2) - nes2
        l_d = (cord_d / 2) - nes2

    elif fatia == 1:

        ang_i = math.asin(nes2 / (di / 2)) * 180 / math.pi
        ang_s = math.asin((nes2 + es-es_serra) / (di / 2)) * 180 / math.pi

        cord_i = di * math.cos(ang_i * math.pi / 180)
        cord_s = di * math.cos(ang_s * math.pi / 180)

        l_i = (cord_i / 2) - nes1
        l_s = (cord_s / 2) - nes1

    return ang_e, ang_d, ang_i, ang_s, l_e, l_d, l_i, l_s


# volume da peça
def calc_v(es_serra, es, l_d, l_e, l_s, l_i, hi):
    if l_s == 0:
        v = ((es - es_serra) / 100) * ((l_d + l_e) / 2) / 100 * hi
    else:
        v = ((es - es_serra) / 100) * ((l_s + l_i) / 2) / 100 * hi
    return v


# SubRotinas
def testa_espessura(es, es_sobra, es_serra, min_es):
    while es > es_sobra:
        es = es - 5
        if es < min_es + es_serra:
            break
    return es


def testa_largura(l_div, l_sobra, l_min, es_serra):
    while l_div > l_sobra:
        l_div = l_div - 5
        if l_div < l_min + es_serra:
            break
    return l_div


def testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra):
    for inv_l in (0, (l_max - l_min) + 1, 5):
        l_div = l_max + es_serra - inv_l

        while l_div > l_sobra:
            l_div = l_div - 5

            if l_div < l_min:
                break

        return l_div


def RelatorioPecaRolica(a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk,
                        codpeca, vr):
    if codpeca == 1:
        a = a + 1
        vra = vra + vr

    elif codpeca == 2:
        b = b + 1
        vrb = vrb + vr

    elif codpeca == 3:
        c = c + 1
        vrc = vrc + vr

    elif codpeca == 4:
        d = d + 1
        vrd = vrd + vr

    elif codpeca == 5:
        e = e + 1
        vre = vre + vr

    elif codpeca == 6:
        f = f + 1
        vrf = vrf + vr

    elif codpeca == 7:
        g = g + 1
        vrg = vrg + vr

    elif codpeca == 8:
        h = h + 1
        vrh = vrh + vr

    elif codpeca == 9:
        k = k + 1
        vrk = vrk + vr

    return a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk


# ----------------------------------------------------------------------------------------------------------------

def RelatorioPeca(a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                  e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk,
                  codpeca, vr, n, n1, v, v1):
    if codpeca == 1:
        na = na + n + n1
        va = va + v + v1

        if vr > 0:
            a = a + 1
            vra = vra + vr
            # relatório por tora
            print('a:{}, vr:{}'.format(a, vr))
            vr = 0

    elif codpeca == 2:
        nb = nb + n + n1
        vb = vb + v + v1

        if vr > 0:
            b = b + 1
            vrb = vrb + vr
            print('b:{}, vr:{}'.format(b, vr))
            vr = 0

    elif codpeca == 3:
        nc = nc + n + n1
        vc = vc + v + v1

        if vr > 0:
            c = c + 1
            vrc = vrc + vr
            print('c:{}, vr:{}'.format(c, vr))
            vr = 0

    elif codpeca == 4:
        nd = nd + n + n1
        vd = vd + v + v1

        if vr > 0:
            d = d + 1
            vrd = vrd + vr
            print('d:{}, vr:{}'.format(d, vr))
            vr = 0

    elif codpeca == 5:
        ne = ne + n + n1
        ve = ve + v + v1

        if vr > 0:
            e = e + 1
            vre = vre + vr
            print('e:{}, vr:{}'.format(e, vr))
            vr = 0

    elif codpeca == 6:
        nf = nf + n + n1
        vf = vf + v + v1

        if vr > 0:
            f = f + 1
            vrf = vrf + vr
            print('f:{}, vr:{}'.format(f, vr))
            vr = 0

    elif codpeca == 7:
        ng = ng + n + n1
        vg = vg + v + v1

        if vr > 0:
            g = g + 1
            vrg = vrg + vr
            print('g:{}, vr:{}'.format(g, vr))
            vr = 0

    elif codpeca == 8:
        nh = nh + n + n1
        vh = vh + v + v1

        if vr > 0:
            h = h + 1
            vrh = vrh + vr
            print('h:{}, vr:{}'.format(h, vrh))
            vr = 0

    elif codpeca == 9:
        nk = nk + n + n1
        vk = vk + v + v1

        if vr > 0:
            k = k + 1
            vrk = vrk + vr
            print('k:{}, vr:{}'.format(k, vr))
            vr = 0

    return a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd, \
           e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk, \
           codpeca, vr


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinalRolica(a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1,
                         vrh1, k1, vrk1, \
                         a, vra, b, vrb, c, vrc, d, vrd, e, vre, f, vrf, g, vrg, h, vrh, k, vrk):
    # transfere as peças contabilizadas da arvore para a variável final,
    # que armazena peças de todas as árvores
    a1 = a1 + a
    vra1 = vra1 + vra
    b1 = b1 + b
    vrb1 = vrb1 + vrb
    c1 = c1 + c
    vrc1 = vrc1 + vrc
    d1 = d1 + d
    vrd1 = vrd1 + vrd
    e1 = e1 + e
    vre1 = vre1 + vre
    f1 = f1 + f
    vrf1 = vrf1 + vrf
    g1 = g1 + g
    vrg1 = vrg1 + vrg
    h1 = h1 + h
    vrh1 = vrh1 + vrh
    k1 = k1 + k
    vrk1 = vrk1 + vrk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, b1, vrb1, c1, vrc1, d1, vrd1, e1, vre1, f1, vrf1, g1, vrg1, h1, vrh1, k1, vrk1, vrt


# ----------------------------------------------------------------------------------------------------------------

def RelatorioFinal(a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1,
                   e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1,
                   k1, vrk1, nk1, vk1,
                   a, vra, na, va, b, vrb, nb, vb, c, vrc, nc, vc, d, vrd, nd, vd,
                   e, vre, ne, ve, f, vrf, nf, vf, g, vrg, ng, vg, h, vrh, nh, vh, k, vrk, nk, vk):
    # transfere peças calculadas da arvore para a variável final, que armazena peças de todas as árvores da amostra
    a1 = a1 + a
    vra1 = vra1 + vra
    na1 = na1 + na
    va1 = va1 + va

    b1 = b1 + b
    vrb1 = vrb1 + vrb
    nb1 = nb1 + nb
    vb1 = vb1 + vb

    c1 = c1 + c
    vrc1 = vrc1 + vrc
    nc1 = nc1 + nc
    vc1 = vc1 + vc

    d1 = d1 + d
    vrd1 = vrd1 + vrd
    nd1 = nd1 + nd
    vd1 = vd1 + vd

    e1 = e1 + e
    vre1 = vre1 + vre
    ne1 = ne1 + ne
    ve1 = ve1 + ve

    f1 = f1 + f
    vrf1 = vrf1 + vrf
    nf1 = nf1 + nf
    vf1 = vf1 + vf

    g1 = g1 + g
    vrg1 = vrg1 + vrg
    ng1 = ng1 + ng
    vg1 = vg1 + vg

    h1 = h1 + h
    vrh1 = vrh1 + vrh
    nh1 = nh1 + nh
    vh1 = vh1 + vh

    k1 = k1 + k
    vrk1 = vrk1 + vrk
    nk1 = nk1 + nk
    vk1 = vk1 + vk

    vrt = vra1 + vrb1 + vrc1 + vrd1 + vre1 + vrf1 + vrg1 + vrh1 + vrk1

    return a1, vra1, na1, va1, b1, vrb1, nb1, vb1, c1, vrc1, nc1, vc1, d1, vrd1, nd1, vd1, \
           e1, vre1, ne1, ve1, f1, vrf1, nf1, vf1, g1, vrg1, ng1, vg1, h1, vrh1, nh1, vh1, k1, vrk1, nk1, vk1, vrt


# ----------------------------------------------------------------------------------------------------------------
# ----------------------------------------------------------------------------------------------------------------

def pecamenor_quadcircun(l_min, l_max, codpeca, hi, es, es_serra, es_sobra, l, l_sobra, n_h,
                         pecas_serradas_preco_dados, es_medula, escolha, cont_medula):
    n1 = 0
    v1 = 0.0000

    l_div = l_min+es_serra

    preco = 0.0000

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco2 = 0.0000
    preco_medula = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:

        # min_es não pode ser menor do que es_serra!
        if l_div > l_sobra or l_div < l_min + es_serra:
            break

        for inv_l in range(0, ((l_max - l_min)+1), 5):
            l_div = l_max + es_serra - inv_l

            # ---------------------------
            # Testa Largura
            l_div = testa_largura(l_div, l_sobra, l_min, es_serra)
            # ---------------------------

            # min_es não pode ser menor do que es_serra!
            if l_div > l_sobra or l_div < l_min + es_serra:
                break

            # numero de peças na vertical
            n_div = math.trunc(l_sobra / l_div)

            # numero de peças total por laço
            n_div1 = n_h * n_div

            # numero de peças total
            n1 = n1 + n_div1

            # volume de peças na horizontal e vertical
            v = (n_h * (es - es_serra) / 1000.0) * (n_div * (l_div - es_serra) / 1000.0) * hi
            v1 = v1 + v

            es_tmp = es
            l_tmp = l_div
            n_tmp = n_div1
            v_tmp = v
            # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
            preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                    hi, v_tmp)

            preco_menor = preco_menor + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
            #       format(codpeca, es, l_div, n_div1, v, l_sobra))

            if es_sobra > es + es_medula and es_medula > es_serra \
                    and cont_medula == 0:
                n_div_medula = 1
                v_medula = (es_medula - es_serra) / 1000 * (
                            n_div_medula * (l_div - es_serra) / 1000) * hi

                v1 = v1 + v_medula
                n1 = n1 + n_div_medula

                # Relatório por peça na MEDULA
                # print('peca_medul.:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}'. \
                #       format(codpeca, es_medula, l_div, 1, v_medula, l_sobra))

                es_tmp = es_medula
                l_tmp = l_div
                n_tmp = n_div_medula
                v_tmp = v_medula
                # -----------------------------------------------------------------
                # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
                preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                        hi, v_tmp)
                # -----------------------------------------------------------------

                preco_medula = preco_medula + preco_peca

            # Relatório por peça
            # print('codpeca:{},es:{}, l_div:{}, n_div1:{}, v:{}, l_sobra:{}, preco_peca:{}, preco:{}' \
            #       .format(codpeca, es, l_div, n_div1, v, l_sobra, preco_peca, preco_medula))

            # largura restante (eixo vertical do bloco)
            l_sobra = l_sobra - (l_div * n_div)

            dados_impressao.append({'l_div': l_div, 'n_div': n_div1, 'v': v, 'l_sobra': l_sobra,
                                    'preco': preco_peca})
            # v=0

            # L sobra
            # print('l_sobra:{}'.format(l_sobra))

    if escolha in ('13', '14'):  # '7' - Gira 180 90
        preco2 = preco_menor + preco_medula
    else:
        preco2 = preco_menor + preco_menor2

    return l_sobra, n1, v1, dados_impressao, preco2


# ---------------------------------------------------------------------------------------------

def pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                         pecas_serradas_preco_dados):

    # ---------------------------
    # Testa Largura
    l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
    # ---------------------------

    # está embutido na função TestaLargura...
    if l_div > l_sobra or l_div < l_min + es_serra:
        n_div = 0
        v = 0
        preco_menor2 = 0
        dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                           'preco': preco_menor2}
        return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2
        # return l_div == 0

    # numero de peças na vertical
    n_div = math.trunc(l_sobra / l_div)

    # numero de peças total
    n1 += n_div

    # volume da peça dividida em subpeças
    v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
    v1 += v

    # largura restante (eixo vertical)
    l_sobra = l_sobra - (l_div * n_div)

    es_tmp = es
    l_tmp = l_div
    n_tmp = n_div
    v_tmp = v
    # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
    preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                            hi, v_tmp)

    preco_menor2 = preco_peca

    # Relatório por peça
    print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'. \
          format(codpeca, es, l_div, n_div, v, l_sobra))
    dados_impressao = {'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                       'preco': preco_peca}
    v = 0

    return l_div, l_sobra, n_div, n1, v1, dados_impressao, preco_menor2

# -------------------------------------------------------------------------------------------------------


def pecamenor_long_1costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,
                                   flecha0,flecha,
                                   ang_e, ang_d, cord_e, cord_d,r_flecha_base,
                                   pecas_serradas_preco_dados, parametro_dados,
                                   flecha_div_r, ang_base, escolha):
    n1 = 0
    v1 = 0.0000
    cord_e1 = 0.00
    dados_impressao = []

    preco_menor = 0.0000
    preco_menor2 = 0.0000
    preco_menor2_14D = 0.0000
    es1 = 0

    # aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h + 1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div > l_sobra or l_div < l_min + es_serra:
            es1 = es
            break

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco_menor = preco_menor + preco_peca

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}, preco_peca:{}, '
              'preco_menor:{}'.format(codpeca, es, l_div, n_div, v, l_sobra, preco_peca,
                                      preco_menor))
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'preco': preco_peca})
        v = 0

        if l_sobra >= l_min + es_serra:
            l_div2 = l_div
            n_div2 = n_div

            # --------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1, impressao, preco_menor2 \
                = pecamenor2_14dcostan(n1, v1, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                                       pecas_serradas_preco_dados)
            dados_impressao.append(impressao)
            # --------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

            preco_menor2_14D = preco_menor2_14D + preco_menor2


        if x <= n_h:

            # incremento da flecha com espessura da nova peça
            flecha = flecha + es

            if x == n_h:
                es1 = es
                es = es_serra

            if escolha in ('13', '14'):  # '7' - Gira 180 90
                cord_e1 = cord_e

            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = formulas_costaneirabase(
                flecha, es, es_serra, di, ang_base, r_flecha_base, escolha, parametro_dados)
            # -----------------------------------------------

            if flecha <= di / 2  or escolha in ('13', '14'):  # '7' - Gira 180 90
                l_sobra = cord_d
            else:
                l_sobra = cord_e

            # 'limite para corte a direita, para giro de 180 graus da tora
            if escolha == '7' and flecha / (di / 2) >= flecha_div_r:
                break

    preco2 = preco_menor + preco_menor2_14D

    es = es1

    return l_div, n_div, l_sobra, n1, v1, flecha, ang_e, ang_d, cord_e, cord_d, \
           r_flecha_base, dados_impressao, preco2, cord_e1


# ---------------------------------------------------------------------------------------------------------

def pecamenor_long_2costaneirabase(l_min, l_max, codpeca, di, hi, es, es_serra,l_sobra, n_h,flecha0,flecha,
                        ang_e, ang_d, cord_e, cord_d,r_flecha_base):
    n1=v1=0
    l_div=0.0000
    n_div=0
    #aplicar volume das peças (v1): volume diferente por peça (vp) (largura aumenta para o centro do circulo)
    for x in range(1, n_h+1):
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        if l_div < l_min + es_serra:
            break

        #numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        #numero de peças total
        n1 += n_div

        #volume da peça dividida em subpeças
        v = ((es - es_serra) / 100) * n_div * (l_div - es_serra) / 100 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        # Relatório por peça
        print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, l_sobra:{}'\
              .format(codpeca, es, l_div, n_div,  v, l_sobra))
        v = 0

        if l_sobra - es_serra >= l_min:
            l_div2 = l_div
            n_div2 = n_div

            #--------------------------------------------------------
            l_div, l_sobra, n_div, n1, v1\
            = pecamenor2_14dcostan(n1,v1,l_min, l_max, codpeca, hi, es, es_serra, l_sobra)
            #--------------------------------------------------------
            l_div = l_div2
            n_div = n_div2

        if flecha+es<=di-flecha0:
            #incremento da flecha com espessura da nova peça
            flecha = flecha + es
        else:
            break

        if x <n_h:
            # -----------------------------------------------
            ang_e, ang_d, cord_e, cord_d = formulas14d(flecha, es, di)
            # -----------------------------------------------
            if flecha <= di / 2:
                l_sobra = cord_d
            else:
                l_sobra = cord_e

    return l_div, n_div, l_sobra ,n1, v1,flecha, ang_e, ang_d, cord_e, cord_d, r_flecha_base

def pecamenor_fatia(fatia, l_min, l_max, codpeca, hi, es, es_serra, l_sobra,
                    pecas_serradas_preco_dados):
    n1 = 0
    v1 = 0.0000
    l_div = 0.0000
    n_div = 0

    preco1 = 0.0000

    dados_impressao = []
    while l_sobra >= l_min + es_serra:
        # ---------------------------
        # Testa Largura
        l_div = testalargura_14dcostaneiras(l_sobra, l_max, l_min, es_serra)
        # ---------------------------

        # numero de peças na vertical
        n_div = math.trunc(l_sobra / l_div)

        # numero de peças total
        n1 += n_div

        # volume da peça dividida em subpeças
        v = ((es - es_serra) / 1000.0) * n_div * (l_div - es_serra) / 1000.0 * hi
        v1 += v

        # largura restante (eixo vertical)
        l_sobra = l_sobra - (l_div * n_div)

        es_tmp = es
        l_tmp = l_div
        n_tmp = n_div
        v_tmp = v
        # Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia
        preco_peca = calc_preco(pecas_serradas_preco_dados, codpeca,
                                hi, v_tmp)

        preco1 = preco1 + preco_peca

        # Relatório por peça
        # print('codpeca:{},es:{}, l_div:{}, n_div:{}, v:{}, fatia:{}, l_sobra:{}' \
        #       .format(codpeca, es, l_div, n_div, v, fatia, l_sobra))
        # v = 0
        #
        # # aviso de troca de fatia (selecionou peça)
        # peca = 1
        dados_impressao.append({'l_div': l_div, 'n_div': n_div, 'v': v, 'l_sobra': l_sobra,
                                'fatia': fatia, 'preco': preco_peca})

    return fatia, l_div, n_div, l_sobra, n1, v1, dados_impressao, preco1


# def formulas14d( flecha, es, es_serra, di):
#
#     # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
#     # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)
#
#     # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
#     ang_e = 180 - math.acos((es-es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
#     ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi
#
#     # 'Cordas esquerda e direita da peça
#     cord_e = di * math.sin(ang_e * math.pi / 180)
#     cord_d = di * math.sin(ang_d * math.pi / 180)
#
#     return  ang_e, ang_d, cord_e, cord_d


def formulas_costaneirabase(flecha, es, es_serra, di, ang_base, r_flecha_base, escolha,
                            parametro_dados):

    # 'angulos (alfa/2) formados com lado esquerdo e direito da peça para comparar com lmin e calcular volume médio
    # '1o angulo do lado esquerdo da 1a peça é 90 (comprimento de l= R = di/2)

    # 'cos(180-alfa) = (es + flecha - di / 2) / (di / 2)
    # 'acos((es + flecha - di / 2) / (di / 2))=180-alfa
    # 'alfa = 180 - acos((es + flecha - di / 2) / (di / 2))

    # 'angulos direito e esquerdo com a nova peça (antes do diametro, usa ang_d, após, usa ang_e
    ang_e = 180 - math.acos((es -es_serra + flecha - di / 2) / (di / 2)) * 180 / math.pi
    ang_d = 180 - math.acos((flecha - di / 2) / (di / 2)) * 180 / math.pi

     # 'Cordas esquerda e direita da peça
    if ang_d < (90 - ang_base) or ang_d > (90 + ang_base):
        cord_d = di * math.sin(ang_d * math.pi / 180)
    else:
        cord_d = di * math.sin(ang_d * math.pi / 180) / 2 + r_flecha_base

    if ang_e < (90 - ang_base) or ang_e > (90 + ang_base):
        cord_e = di * math.sin(ang_e * math.pi / 180)
    else:
        cord_e = di * math.sin(ang_e * math.pi / 180) / 2 + r_flecha_base

    return ang_e, ang_d, cord_e, cord_d


def calcmadeira_get_totais(self, cr, uid, ids, field_name, arg, context=None):
    """
    Apura o rendimento no calculo das madeiras.
    Para a peca rolica, o rendimento e calculado com o volume total da tora divido pelo volume
    total multiplicado por 100.
    Para as demais pecas, o rendimento e calculado com o volume total da peca divido pelo volume
    total da tora multiplicado por 100.
    :param cr: cursor no banco de dados.
    :type cr: object
    :param uid: id da tabela res_users, identificador do usuario.
    :type uid: int
    :param ids: lista de id da tabela do objeto wizard_calcmadeira_calcula.
    :type ids: list
    :param field_name: descricao do atributo funcional.
    :type field_name: str
    :param arg: lista de argumentos de pesquisa.
    :type arg: list
    :param context: dados contextuais.
    :type context: dict
    :return: dados da identificacao do registro e seu respectivo valor.
    :rtype: dict
    """
    context = context and context.copy()
    res = {}
    for calc in self.browse(cr, uid, ids, context):
        # res.update({calc.id: {'vr_rendimento': 0}})
        # volume_peca_total = sum(map(lambda i: i.volpeca or 0, calc.peca_ids))
        # volume_tora_total = sum(map(lambda i: i.voltora or 0, calc.peca_ids))
        # volume_peca_rolica_total = sum(map(lambda i: i.volpeca or 0, calc.peca_rolica_ids))
        # if context.get('tipo_funcao') == '1' and calc.vt1:
        #     res[calc.id].update({'vr_rendimento': (volume_tora_total / calc.vt1) * 100.0})
        # if context.get('tipo_funcao') != '1' and (calc.vt1 or calc.total_vt_vr) \
        #         and volume_tora_total:
        #     res[calc.id].update({'vr_rendimento': (volume_peca_total / volume_tora_total) * 100.0})
        # if context.get('serrada_e_rolica') and (volume_tora_total + volume_peca_total):
        #     res[calc.id].update({
        #         'vr_rendimento': ((calc.vt1 + volume_peca_rolica_total) /
        #                           (volume_tora_total + volume_peca_rolica_total)) * 100.0})
        # preco_total = sum(map(lambda i: i.preco_item or 0, calc.peca_ids))
        # preco_peca = volume_peca_total and preco_total / volume_peca_total or 0
        # preco_tora = volume_tora_total and preco_total / volume_tora_total or 0
        # if calc.tipo_funcao == '1':  # Rolica
        #     preco_tora = 0
        # preco_arvore = calc.vt1 and preco_total / calc.vt1 or 0
        # quantidade_tora = 0
        # if context.get('funcoes_serrada_tora'):
        #     preco_arvore = 0
        #     quantidade_tora = sum(map(lambda i: i.numtora or 0, calc.peca_ids))
        # arvore_processada_hectare = 0
        # falha_hectare = 0
        # total_volume_hectare = 0
        # if context.get('tipo_funcao') in ('1', '2') and calc.narv:
        #     arvore_processada_hectare = int(
        #         (calc.arvore_processada or 0) * (calc.arvore_por_hectare or 0) / float(calc.narv))
        #     falha_hectare = int((calc.falha or 0) * (calc.arvore_por_hectare or 0) /
        #                         float(calc.narv))
        #     total_volume_hectare = (calc.vt1 or 0) * (calc.arvore_por_hectare or 0) / \
        #         float(calc.narv)
        # valor_residual = 0
        # if context.get('serrada_e_rolica'):
        #     valor_residual = calc.vt1 and (calc.vt1 - volume_tora_total - volume_peca_rolica_total)\
        #         or 0
        # valor_residual_percentual = 0
        # if context.get('calculo_rolica') or context.get('serrada_e_rolica'):
        #     volume_peca = volume_peca_total if context.get('calculo_rolica') \
        #         else volume_peca_rolica_total
        #     valor_residual_percentual = volume_tora_total \
        #         and ((volume_tora_total - volume_peca) / volume_tora_total) or 0
        # res[calc.id].update({
        #     'preco_total': preco_total, 'preco_peca': preco_peca, 'preco_tora': preco_tora,
        #     'preco_arvore': preco_arvore, 'quantidade_tora': quantidade_tora,
        #     'arvore_processada_hectare': arvore_processada_hectare, 'falha_hectare': falha_hectare,
        #     'valor_residual_percentual': valor_residual_percentual,
        #     'valor_residual': valor_residual, 'total_volume_hectare': total_volume_hectare,
        #     'qtd_talhao': len(calc.talhao_ids),
        # })
        res.update({calc.id: {
            'preco_total': 0, 'preco_peca': 0, 'preco_tora': 0,
            'preco_arvore': 0, 'quantidade_tora': 0,
            'arvore_processada_hectare': 0, 'falha_hectare': 0,
            'valor_residual_percentual': 0,
            'valor_residual': 0, 'total_volume_hectare': 0,
            'qtd_talhao': len(calc.talhao_ids),
        }})
    return res


# def calcmadeira_check_calculo_receita(self, cr, uid, ids, field_name, arg, context=None):
#     context = context and context.copy()
#     permite_calculo_receita = True
#     if any([context.get('calculo_rolica'), context.get('calculo_laminada_arvore'),
#             context.get('calculo_laminada_tora'), context.get('calculo_fator_forma'),
#             context.get('funcoes_serrada_arvore')]):
#         permite_calculo_receita = False
#     if context.get('serrada_e_rolica'):
#         # O Calculo de 'Madeira Serrada + Rolica' sempre realiza o calculo da receita.
#         # No entanto, atribui o valor falso para 'permite_calculo_receita' para que a opcao de
#         # selecao nao seja exibida ao usuario
#         permite_calculo_receita = False
#     res = {}.fromkeys(ids, permite_calculo_receita)
#     return res


def forma(forma_dados):

    print('Ajusta Equação de Forma')

    # --------------------
    lintaper, ntaper = forma_dados, len(forma_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap = 0

    # a = vetor de '1'
    # 'x1= hi/ht
    # 'x2= (hi/ht)**2
    # 'y= (di_casc/dap)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i = 1
    hi = 0
    hi1 = 1

    while taper < ntaper:

        while hi < hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper == ntaper:
                pass
            else:
                try:
                    dados1 = lintaper[taper]
                    hi1 = float(dados1.get('cmatap_hi'))
                except:
                    print('fim arquivo')
                    hi1 = 0.3
            try:
                arv = int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht = float(dados.get('cmatap_ht'))

                print('hi={}, hi1={}/ ht={}'.format(hi, hi1, ht))

            except:
                print('ainda não leu altura total (ht)')

            if hi == 1.3:
                dap = di_casc

            taper = taper + 1

        for c in range(i - 1, taper - 1, 1):
            dados = lintaper[c]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(hi / ht)
                x2.append((hi / ht) ** 2)
                y.append((di_casc / dap) ** 2)

                print('hi/ht:{}'.format(hi / ht))
            except:
                print('falta algum dado no Taper.txt')

            hi = 0
            hi1 = 1
        i = taper

    dado = ntaper
    dado1 = dado

    a = dado1 * [1]
    print('a=', a)
    print('x1f=', x1)
    print('x2f=', x2)
    print('yf=', y)
    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    return B0, B1, B2, r2


def volume(volume_dados):
    print('Ajusta Equação de Volume')

    # --------------------
    linarv, narv = volume_dados, len(volume_dados)
    # --------------------

    # numero da árvore
    arv = 0
    n_falha = 0
    dado = 0

    # a = vetor de '1'
    # 'x1=ln(dap)
    # 'x2=ln(ht)
    # 'y=ln(vol)

    a = []
    x1 = []
    x2 = []
    y = []

    # Executa por árvore
    while arv < narv:
        try:
            # Acessa arquivo texto, armazena dap e ht por árvore
            dados = linarv[arv]
            dap = float(dados.get('cmacub_diametro'))
            ht = float(dados.get('cmacub_altura'))
            vol = float(dados.get('cmacub_volume'))
        except:
            print('arquivo Arvore.txt com campos incompletos. Vol?')

        if dap == "" or ht == "":
            n_falha = n_falha + 1
        else:
            x1.append(math.log(dap))
            x2.append(math.log(ht))
            y.append(math.log(vol))

        arv = arv + 1

    dado = narv - n_falha

    a = dado * [1]

    # --------------------------
    B, r2 = regressao(dado, a, x1, x2, y)
    # --------------------------

    B0 = B[0]
    B1 = B[1]
    B2 = B[2]

    print('b0={}, b1={}, b2={}'.format(B0, B1, B2))

    # print('x1:{}. x2:{}'.format(x1[1],x2[1]))

    # 'EQUAÇÃO Volume (Schumacher and Hall)
    # 'calcula di com casca (di_casc2) na altura do 2o diâmetro, o menor (hi_atual+hi)
    # volest = math.exp(B0 + B1 * x1[1] + B2 * x2[1])
    # print('Volume: {}'.format(volest))

    return B0, B1, B2, r2


def regressao(dado, a, x1, x2, y):
    Y = np.array(y)
    Xt = np.array([a, x1, x2])
    X = Xt.transpose()

    # dimensoes(matriz):
    tam_Xt = (len(Xt), len(Xt[0]))
    print('{} X {}'.format(tam_Xt[0], tam_Xt[1]))

    tam_X = (len(X), len(X[0]))
    print('{} X {}'.format(tam_X[0], tam_X[1]))

    XtX = Xt.dot(X)
    XtXinv = inv(XtX)
    XtY = Xt.dot(Y)
    B = XtXinv.dot(XtY)
    Bt = B.transpose()
    Yt = Y.transpose()
    YtY = Yt.dot(Y)
    BtXtY = Bt.dot(XtY)

    cte = sum(Y) ** 2 / dado

    SQRegres = BtXtY - cte
    SQTotal = YtY - cte
    SQRes = SQTotal - SQRegres
    print(sum(y))

    if SQTotal != 0:
        r2 = SQRegres / SQTotal
        print('r2: {}'.format(r2))
        # print('B0:{},B1:{}, B2:{}'.format(B[0],B[1],B[2]))

    else:
        r2 = 0
        print('r2: {}'.format(r2))

    # B,r2=Regressao(a,Xt,Y)
    # --------------------------

    return B, r2


def Forma_hi(taper_dados):

    print('Ajusta Equação de Forma com arquivo Taper.txt')

    # --------------------
    lintaper,ntaper=taper_dados, len(taper_dados)
    # --------------------

    # numero da árvore
    dado = 0
    dap=0

    # a = vetor de '1'
    # 'x1= di_casc/dap
    # 'x2= (di_casc/dap)**2
    # 'y= (hi/ht)**2

    a = []
    x1 = []
    x2 = []
    y = []

    # Acessa arquivo texto, armazena dap e ht por arvore
    taper = 1

    # Executa por seção
    i=1
    hi=0
    hi1=1

    while taper <= ntaper:

        while hi<hi1:
            dados = lintaper[taper - 1]
            hi = float(dados.get('cmatap_hi'))

            if taper==ntaper:
                pass
            else:
                dados1 = lintaper[taper]
                hi1 = float(dados1.get('cmatap_hi'))

            try:
                arv=int(dados.get('cmatap_arv'))
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))
                ht=float(dados.get('cmatap_ht'))

            except:
                # print('ainda não leu altura total (ht)')
                pass

            if hi == 1.3:
                dap = di_casc

            taper=taper+1


        for c in range(i,taper,1):
            dados = lintaper[c - 1]
            try:
                hi = float(dados.get('cmatap_hi'))
                di_casc = float(dados.get('cmatap_di_casc'))

            except:
                print('arquivo Taper.txt com campos incompletos')

            try:
                x1.append(di_casc/dap)
                x2.append((di_casc/dap)**2)
                y.append((hi/ht)**2)

                # print('hi/ht:{}'.format(di_casc/dap))
            except:
                print('falta algum dado no Taper.txt')

            hi=0
            hi1=1
        i = taper

    dado = ntaper

    a = dado * [1]
    # print('a=',a)
    # print('x1=',x1)
    # print('x2=',x2)
    # print('y=',y)
    # --------------------------
    B, r2 =regressao(dado, a, x1, x2, y)
    # --------------------------

    B0f_hi = B[0]
    B1f_hi = B[1]
    B2f_hi = B[2]

    # print('b0f_hi={}, b1f_hi={}, b2f_hi={}'.format(B0f_hi, B1f_hi, B2f_hi))

    return B0f_hi,B1f_hi,B2f_hi


def calcula_altura_comercial(arvore_dados, diametro_minimo, parametro_dados, taper_dados):
    # ------------------------------------------------------------------
    # Calcula Altura comercial a partir da Ht do arquivo Arvore.txt
    altura_comercial_dict = {}
    if not diametro_minimo:
        return altura_comercial_dict
    di_limite = diametro_minimo
    falha = 0

    # -----------------------
    linarv, narv = arvore_dados, len(arvore_dados)
    # -----------------------

    # escolha = '1'
    # -------------------------------------------
    # es_serra, hi_atual, hi, hi_min, prop_casc, l_min_pecas, es_min_pecas, di_min, es_perim = parametros(
    #     escolha)
    hi_atual = parametro_dados.get('cmapar_altura_corte')
    hi = parametro_dados.get('cmapar_altura')
    flecha_div_r = parametro_dados.get('cmapar_proporcao_raio_corte_peca')
    es_medula = parametro_dados.get('es_medula')
    # -------------------------------------------

    parc1 = []
    arvore1 = []
    dap1 = []
    ht1 = []
    hc1 = []

    # --------------------------------------
    B0f_hi, B1f_hi, B2f_hi = Forma_hi(taper_dados)
    # --------------------------------------

    for arv in arvore_dados:

        try:
            parc = float(arv.get('cmaarv_linha'))
            arvore = float(arv.get('cmaarv_arvore'))
            dap = float(arv.get('cmaarv_diametro'))
            ht = float(arv.get('cmaarv_altura'))
            hi2 = 0

            if hi <= ht:
                # --------------------------------------
                hi1 = Eqhi(B0f_hi, B1f_hi, B2f_hi, hi_atual, hi, ht, dap, di_limite)

                hi2 = hi1.hi_result()

                if hi2 < hi:
                    hi2 = 0

        except:
            falha += 1

            parc = float(arv.get('cmaarv_linha'))
            arvore = 0
            dap = 0
            ht = 0
            hi2 = 0

        parc1.append(parc)
        arvore1.append(arvore)
        dap1.append(dap)
        ht1.append(ht)
        hc1.append(hi2)

        altura_comercial_dict.update({arv.get('id'): hi2})

        parc = arvore = dap = ht = hi2 = 0

    return altura_comercial_dict


# Ponderação do preço da peça pela sua dimensão em relação a dimensão de referencia


def calc_preco(pecas_serradas_preco_dados, codigopeca, hi, v_tmp):
    preco_peca = 0
    w = 0
    if pecas_serradas_preco_dados:
        preco = pecas_serradas_preco_dados[w]
        while codigopeca != int(preco.get('cpspre_codigo_peca')):
            preco = pecas_serradas_preco_dados[w]
            w = w + 1

        es_ref = float(preco.get('cpspre_espessura'))
        l_ref = float(preco.get('cpspre_largura'))
        preco_ref = float(preco.get('cpspre_preco'))

        v_ref = es_ref / 1000 * l_ref / 1000 * hi
        preco_peca = (v_tmp / v_ref) * preco_ref

    return preco_peca
