import requests
from datetime import datetime
import hashlib
import hmac
import time
import requests


class BlinkTrade(object):
    def __init__(self, ambiente, Broker, Chave, Segredo):
        if ambiente == 'TEST':
            self.url = 'https://api_testnet.blinktrade.com/tapi/v1/message'
        elif ambiente == 'PROD':
            self.url = 'https://api.blinktrade.com/tapi/v1/message'
        else:
            raise ValueError('Ambiente invalido, use TEST ou PROD')
        if Broker not in ('1', '3', '4', '5', '8', '9'):
            raise ValueError(
                'Broker_id invalido! valores validos:\n1-SurBitcoin, \n3-VBTC, \n4-FoxBit, \n5-Testnet, \n8-UrduBit, \n9-ChileBit')
        self.broker = Broker
        self.chave = Chave
        self.segredo = Segredo

    def __send(self, message):
        # Generate a nonce
        dt = datetime.now()
        nonce = str(int((time.mktime(dt.timetuple()) + dt.microsecond / 1000000.0) * 1000000))
        # Sign the nonce with secret using HMAC-SHA256
        signature = hmac.new(self.segredo, nonce, digestmod=hashlib.sha256).hexdigest()
        # Set headers
        headers = {
            'APIKey': self.chave,
            'Nonce': nonce,
            'Signature': signature,
            'Content-Type': 'application/json'
        }
        print self.url, message, headers
        return requests.post(self.url, json=message, verify=True, headers=headers).json()

    def get_balance(self):
        msg = {
            "MsgType": "U2",
            "BalanceReqID": 1,
        }
        return self.__send(msg)

    def get_my_orders(self, id=1, page=0, size=20):
        msg = {
            "MsgType": "U4",
            "OrdersReqID": id,
            "Page": page,
            'PageSize': size,
        }
        return self.__send(msg)

    def set_order(self, id, compra_venda, tipo, preco, qtd, moeda='BTCBRL'):
        msg = {
            "MsgType": "D",
            "ClOrdID": id,
            "Symbol": moeda,
            'Side': compra_venda,
            'OrdType': tipo,
            'Price': preco,
            'OrderQty': qtd,
            'BrokerID': self.broker,
        }
        return self.__send(msg)


def blinktrade_get_ticker(moeda):
    """
    Busca o ticker na foxbit
    :return: dicionario com resultado
     ex: {"buy": 41150.0,
          "last": 41298.49,
          "sell": 41298.49,
    """
    _url = 'https://api.blinktrade.com/api/v1/BRL/ticker?crypto_currency=%s' % moeda
    response = requests.get(_url, {})
    a = response.json()
    """ retorno do ticker  blinktrade
      {"high": 41399.97,
         "vol": 459.41128325,
         "buy": 41150.0,
         "last": 41298.49,
         "low": 37909.0,
         "pair": "BTCBRL",
         "sell": 41298.49,
         "vol_brl": 18066411.69173826}
    """
    return {
        'buy':a.get('buy'),
        'sell': a.get('sell'),
        'last': a.get('last'),
    }

def blinktrade_get_trade(hora):
    """
    Busca o trade na foxbit
    :return: lista de dicionario com resultado
     ex: [{"tid": 2345, "date": 1418293867, "price": 1020.0, "amount": 0.001, "side": "sell"}]
    """
    _url = 'https://api.blinktrade.com/api/v1/BRL/trades?crypto_currency=BTC&since=%s&limit=100' % hora
    response = requests.get(_url, {})
    linhas = []
    for linha in response.json():
        linhas.append({
            'side': linhas.get('side'),
            'data': datetime.fromtimestamp(linha.get('date')).strftime('%Y-%m-%d %H:%M:%S'),
            'tid': linha.get('tid'),
            'amount': linha.get('amount'),
            'price': linha.get('price'),
        })
    return linhas

def blinktrade_get_orderbook(moeda='BTCBRL'):
    _url = 'https://api.blinktrade.com/api/v1/BRL/orderbook"'
    response = requests.get(_url, {})
    for linha in  response.json():
        if linha.get('pair') == moeda:
            return {
                'bids': linha.get('bids'),
                'asks': linha.get('asks'),
            }
    return {'bids':[], 'asks':[]}
