# -*- encoding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 All Rights Reserved. Luiz Fernando Maciel França
#                    www.siginformatica.com.br luiz@siginformatica.com.br
#
##############################################################################

from openerp.osv import osv, fields
import logging


class coin_robot(osv.Model):
    """
    robot para avaliação de oportunidades
    prefixo:coibot
    """
    _name = 'coin.robot'
    _logger = logging.getLogger(_name)
    from funcoes import _coibot_get_dados
    _columns = {
        'id': fields.integer('Id', readonly=True),
        'coibot_broker_id': fields.many2one('coin.brokers','Operador', required=True, select=1,
                                              ondelete='cascade'),
        'coibot_moeda_id': fields.many2one('coin.moedas','Moeda', required=True, select=1,
                                           ondelete='cascade'),
        'create_date': fields.datetime('Data', readonly=True, select=1),
        'coibot_valor': fields.float('Valor disponivel', required=True),
        'coibot_estrategia': fields.selection([('FX', 'Fixo'), ('MX','Maximizado'), ('MM', 'Meio a meio'), ('BA', 'Banheira')], 'Estrategia', required=True,
                                              help='Estratégia de atuação do bot: \rFixo, voce estabelece uma margem para ganho. \rMaximizado, voce tenta ganhar nas variacoes diarias.\rMeio a meio, mantem o equilibrio entre o saldo em coins e em real.\rBanheira, verifica se o valor de venda - o percentual de ganho fica proximo do atual valor de compra'),
        'coibot_ganho': fields.float('Percentual ganho bruto',
                                             help='Percentual para ganho bruco, considerando as taxas das operacoes'),
        'coibot_mx_variacao_sentido': fields.float('Variacao para identificar sentido',
                                                   help='Percentual para identificar o sentido, se a cotacao esta aumentando ou diminuindo'),
        'coibot_mx_variacao_compra': fields.float('Variacao para startar compra',
                                                  help='Percentual para ientificar a necessidade de compra'),
        'coibot_mx_variacao_venda': fields.float('Variacao para startar venda',
                                                 help='Percentual para identificar a necessidade de venda'),
        'coibot_diferenca_outro': fields.float('Diferenca para outro bot mesma estrategia',
                                                  help='Percentual que identifica a diferença minima para compra entre o bot atual e outro bot da mesma estrategia'),
        'coibot_data_compra': fields.datetime('Data da compra'),
        'coibot_valor_compra': fields.float('Valor de compra'),
        'coibot_mm_variacao_para_operacao': fields.float('Variação para operacao',
                                                         help='percentual que identifique a necessidade de compra ou de venda para que os saldos em Real e em coin fique equilibrados'),
        'coibot_quantidade_comprada': fields.float('Quantidade', digits=(16,8), help='Quantidade comprada'),
        'coibot_saldo': fields.float('Saldo em Reais'),
        'coibot_qtd_compras': fields.integer('Qtd. Compras'),
        'coibot_qtd_vendas': fields.integer('Qtd. Vendas'),
        'coibot_horas_media': fields.integer('Horas p/media', help='Tempo em horas para calculo da media para limitar as compras'),
        'coibot_limite_compra': fields.float('Valor limite p/compra',
                                             help='Valor maximo para compra, o robot não comprara se o valor for superior ao definido aqui'),
        'coibot_ativo': fields.boolean('Ativo'),
        'coibot_tempo_execucao': fields.integer('Tempo', help='Tempo em segundos de espera entre execucoes'),
        'coibot_proxima_operacao': fields.selection([('S', 'Sem definicao'),('C','Compra'),('V', 'Venda')], 'Proxima operacao'),
        'coibot_estatisticas_ids': fields.one2many('coin.robot.estatisticas', 'coibte_robot_id', 'Estatisticas'),
        'coibot_ordens_ids': fields.one2many('coin.orders', 'coiord_robot_id', 'Ordens'),
        'coibot_historicos_ids': fields.one2many('coin.robot.historicos', 'coibhi_robot_id', 'Historicos'),
        'coibot_cotacao_compra': fields.function(_coibot_get_dados, method=True, string='Cotacao compra', type='float', store=False,
                                        multi='dados', digits=(8,2)),
        'coibot_cotacao_venda': fields.function(_coibot_get_dados, method=True, string='Cotacao venda', type='float', store=False,
                                        multi='dados', digits=(8,2)),
        'coibot_cotacao_ultima': fields.function(_coibot_get_dados, method=True, string='Cotacao ultima operacao', type='float', store=False,
                                        multi='dados', digits=(8,2)),
        'coibot_saldo_coin': fields.function(_coibot_get_dados, method=True, string='Saldo coins/real', type='float', store=False,
                                        multi='dados', digits=(8,2)),
        'coibot_margem_no_momento':fields.function(_coibot_get_dados, method=True, string='Margem no momento', type='float', store=False,
                                        multi='dados', digits=(8,3)),
    }
    _order = 'create_date desc'
    _defaults = {
        'coibot_proxima_operacao':'C',
        'coibot_horas_media': 24,
    }
