#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc


class ColetorConferenciaBox(object):
    """
    Modulo Conferencia Box.
    """

    # Cores utilizadas
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_ok = termx.fg_white, termx.bg_green,  # Branco/verde
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_list_os = termx.bold, termx.fg_yellow, termx.underline,  # Amarelo claro (sublinhado)
    color_os = termx.fg_yellow,  # Amarelo
    color_msg_ok = termx.fg_green,  # Verde claro
    line_ok = line_error = termx.max_row  # Linha onde sao mostradas as mensagens
    line_info = termx.max_row - 1  # Linha onde sao mostradas as informacoes
    rec_sucess = True  # Conferencia efetuada com sucesso

    def __init__(self):
        self.conferencia_box()

    def conferencia_box(self):
        """
        Lista das ordens de servico (conferencia box)

        :return: o numero da ordem de sevico.
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(2)
        termx.say(2, 1, '- CONFERENCIA  BOX -', *self.color_header)
        ordem_servico = False

        while True:
            ordem_servico = self.ordem_servico_conferencia()  # Selecao da ordem de servico

            if not ordem_servico:
                break

            self.separacao_item(ordem_servico)  # Conferencia/separacao do item

        return ordem_servico

    def ordem_servico_conferencia(self):
        """
        Ordens de servico de conferencia (Box).

        :return: numero da ordem de servico selecionada.
        :rtype: int
        """

        termx.clear(3)
        termx.say(3, 1, 'OS Nr.', *self.color_list_os)
        termx.say(3, 8, '  Data  ', *self.color_list_os)
        termx.say(3, 17, 'Box', *self.color_list_os)

        # Selecao de uma Ordem de Servico
        os_numero = 0
        __ESC = chr(27)

        while True:
            lin_det, lista_os = self.lista_os()  # Mostra a lista de ordens de servico
            lin = 4

            for i in range(0, len(lin_det)):
                termx.say(lin, 1, lin_det[i], *self.color_os)
                lin += 1

                if (lin - 2) > 4:
                    break

            termx.say(8, 1, 'OS Nr. ______')
            os_numero = termx.get(8, 8, 6)  # Aguarda a selecao

            if os_numero == __ESC:
                return False

            if not os_numero:
                continue

            if not os_numero.isdigit():
                continue
            elif not int(os_numero) in lista_os:
                continue

            # Acrescenta usuario como responsavel se nao constar na ordem de servico
            fields = ['responsavel_ids']
            dados_separacao = xmlrpc.execute('separacao.os', 'read', int(os_numero), fields)

            if dados_separacao and isinstance(dados_separacao, list):
                dados_separacao = dados_separacao[0]

            if dados_separacao and isinstance(dados_separacao, dict):
                if xmlrpc.login_id not in dados_separacao.get('responsavel_ids'):
                    dados = {'responsavel_ids': [[6, False, [xmlrpc.login_id]]]}

                    xmlrpc.execute('separacao.os', 'write', int(os_numero), dados)

            break

        return os_numero

    def separacao_item(self, separacao_id):
        """
        Itens de separacao.

        :param separacao_id: o ID da ordem de servico de separacao
        :type separacao_id: int
        :return: verdadeiro se o item foi enderecado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Formulario
        def form_separacao():
            """
            Tela de Separacao

            :return: None
            :rtype: None
            """

            termx.clear(3)
            termx.say(3, 1, 'Nr. do Palete ___    ')
            termx.say(4, 1, 'Cod. _______________ ')
            termx.say(7, 1, 'Qtde. ________       ')

            return None

        form_separacao()
        __ESC = chr(27)

        while True:  # Palete
            # Informa o numero do palete da ordem de servico
            separacao_id = int(separacao_id)
            fields = ['separacao_item_ids']
            separacao_os_dados = xmlrpc.execute('separacao.os', 'read', separacao_id, fields)

            if separacao_os_dados:
                if isinstance(separacao_os_dados, list):
                    separacao_os_dados = separacao_os_dados[0]

                item_ids = separacao_os_dados.get('separacao_item_ids')

                if item_ids:
                    item_id = item_ids[0]
                    wms_separacao_itens_dados = xmlrpc.execute(
                        'wms.separacao.itens', 'read', item_id, ['palete'])
                    palete = wms_separacao_itens_dados.get('palete')
                else:
                    break
            else:
                break

            termx.say(3, 15, palete, *self.color_field)

            # Recebe o codigo do produto (codigo de barra)
            codigo = None

            while True:  # Codigo de barras
                codigo = termx.get(4, 6, 15)

                if codigo == __ESC:
                    break
                elif not codigo:
                    continue
                elif not codigo.isdigit():
                    self.mensagem_erro('ITEM NAO CADASTRADO')

                # Recupera os dados do item/produto
                args = [('codigo_barra', '=', '%014d' % int(codigo))]
                ids = xmlrpc.execute('automacao.codigo.barra', 'search', args)

                if not ids:  # Nao encontrou o codigo do produto
                    self.mensagem_erro('ITEM NAO CADASTRADO.')
                    continue

                produto_id, grade_id, embalagem_id = self.recupera_produto_grade_embalagem(ids)

                # Busca a descricao do item
                fields = ['name']
                dados_item = xmlrpc.execute('estoque.produto', 'read', produto_id, fields)
                descricao = 'ITEM SEM DESCRICAO'
                termx.erase_line(5, 6)

                if dados_item and isinstance(dados_item, list):
                    dados_item = dados_item[0]

                if dados_item and isinstance(dados_item, dict):
                    descricao = dados_item.get('name')[0:40].upper()
                    descricao = '{0:40}'.format(descricao)

                # Descricao do produto conforme parametro do estoque
                fields = ['estpar_descricao_grade']
                adm_utils_dados = xmlrpc.execute(
                    'adm.utils', 'admutl_get_parametros', 'estoque', fields)

                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    fields = ['descricao']
                    dados_grade = xmlrpc.execute('estoque.produto.grade', 'read', grade_id, fields)

                    if dados_grade and isinstance(dados_grade, list):
                        dados_grade = dados_grade[0]

                    descricao = dados_grade.get('descricao') or 'ITEM SEM DESCRICAO.'
                    descricao = '{0:40}'.format(descricao)

                termx.say(5, 1, descricao[0:20], *self.color_field)
                termx.say(6, 1, descricao[20:40], *self.color_field)

                # Confere o produto com os itens do palete
                item_quantidade = self.conferir_item(separacao_id, palete, produto_id, grade_id)

                if item_quantidade == -1:  # Quantidade negativa para itens que nao conferem
                    self.mensagem_erro('ITEM NAO CONFERE.   ')
                    continue

                if item_quantidade == 0:  # Item ja conferido
                    self.mensagem_erro('ITEM JA CONFERIDO!  ')
                    continue

                # Quantidade na embalagem
                qtde_embalagem_contada = 1  # Minimo = 1

                if embalagem_id:
                    fields = ['embalagem_id']
                    dados_embalagem = xmlrpc.execute(
                        'estoque.produto.embalagem', 'read', embalagem_id, fields)

                    if dados_embalagem and isinstance(dados_embalagem, list):
                        dados_embalagem = dados_embalagem[0]

                    if dados_embalagem and isinstance(dados_embalagem, dict):
                        embalagem_contada_id = dados_embalagem.get('embalagem_id')[0]

                        fields = ['quantidade_na_embalagem']
                        dados_embalagem = xmlrpc.execute(
                            'estoque.embalagem', 'read', embalagem_contada_id, fields)

                        if dados_embalagem and isinstance(dados_embalagem, list):
                            dados_embalagem = dados_embalagem[0]

                        if dados_embalagem and isinstance(dados_embalagem, dict):
                            qtde_embalagem_contada = dados_embalagem.get('quantidade_na_embalagem')

                termx.say(7, 16, 'X ' + str(int(qtde_embalagem_contada)))

                # Quantidade contada
                qtde_contada = None

                while True:  # Quantidade
                    qtde_contada = termx.get(7, 7, 8)
                    if not qtde_contada:
                        continue

                    if qtde_contada == __ESC:
                        break
                    if qtde_contada == '*':
                        if self.recontar_item(separacao_id, palete, produto_id, grade_id):
                            self.mensagem_erro('** ITEM RECONTADO **')
                        continue
                    if not qtde_contada.isdigit():
                        continue
                    if not int(qtde_contada):
                        continue
                    qtde_contada = int(qtde_contada) * qtde_embalagem_contada

                    # Registra a quantidade no item (a menor, igual ou a maior)
                    registro = self.registra_contagem(
                        qtde_contada, embalagem_id, separacao_id, palete, produto_id, grade_id)

                    if registro == '>':
                        self.mensagem_erro('QUANTIDADE A MAIOR. ')
                        continue
                    elif registro == '=':
                        self.mensagem_erro('** ITEM CONFERIDO **')
                        termx.get(termx.max_row, termx.max_col, 1)
                        self.mensagem_erro('                    ')
                        break
                    else:
                        break

                # Verifica se todos os itens ja foram conferidos
                if self.conferencia_box_ok(separacao_id):
                    self.mensagem_conferencia_ok()

                    # Fecha a ordem de servico
                    xmlrpc.execute('separacao.os', 'osvosv_fechar_os', separacao_id)
                    return self.rec_sucess

                termx.say(5, 1, '                     ')
                termx.say(6, 1, '                     ')
                termx.say(7, 1, 'Qtde. ________       ')

                if qtde_contada == __ESC:
                    continue

            if codigo == __ESC:
                break

            form_separacao()

    @staticmethod
    def recontar_item(separacao_id, palete, produto_id, grade_id):
        """
        Realiza a recontagem do item de separacao.

        :param separacao_id: ID da ordem de servico de separacao
        :param palete: numero do palete
        :param produto_id: ID do produto
        :param grade_id: ID da grade do produto
        :type separacao_id: int
        :type palete: int
        :type produto_id: int
        :type grade_id: int
        :return: lista com o ID dos itens que sofreram recontagem
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import time

        args = [('os_id', '=', separacao_id), ('palete', '=', str(int(palete)).zfill(3)),
                ('produto_id', '=', produto_id), ('grade_id', '=', grade_id)]
        item_ids = xmlrpc.execute('wms.separacao.itens', 'search', args)

        # Dados do item
        fields = ['produto_embalagem_id', 'qtde_conferida']
        wms_separacao_item_dados = xmlrpc.execute('wms.separacao.itens', 'read', item_ids, fields)
        qtde_na_embalagem_item = 1

        for wms_separacao_item_dado in wms_separacao_item_dados if isinstance(
                wms_separacao_item_dados, list) else [wms_separacao_item_dados]:

            # Quantidade na embalagem do item
            produto_embalagem_id = wms_separacao_item_dado.get('produto_embalagem_id')

            if produto_embalagem_id and isinstance(produto_embalagem_id, (tuple, list)):
                produto_embalagem_id = produto_embalagem_id[0]

            item_qtde_conferida = wms_separacao_item_dado.get('qtde_conferida')
            fields = ['embalagem_id']
            dados_produto_embalagem = xmlrpc.execute(
                'estoque.produto.embalagem', 'read', produto_embalagem_id, fields)

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, list):
                dados_produto_embalagem = dados_produto_embalagem[0]

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, dict):
                embalagem_id = dados_produto_embalagem.get('embalagem_id')

                if embalagem_id and isinstance(embalagem_id, (tuple, list)):
                    embalagem_id = embalagem_id[0]

                fields = ['quantidade_na_embalagem']
                dados_embalagem = xmlrpc.execute(
                    'estoque.embalagem', 'read', embalagem_id, fields)

                if dados_embalagem and isinstance(dados_embalagem, list):
                    dados_embalagem = dados_embalagem[0]

                if dados_embalagem and isinstance(dados_embalagem, dict):
                    qtde_na_embalagem_item = dados_embalagem.get('quantidade_na_embalagem')
                    item_qtde_conferida *= qtde_na_embalagem_item

            if xmlrpc.execute('wms.separacao.itens', 'write', item_ids, {'qtde_conferida': 0}):
                dados = {
                    'wmscit_separacao_id': separacao_id,
                    'wmscit_separacao_item_id': item_ids[0],
                    'wmscit_produto_id': produto_id,
                    'wmscit_grade_id': grade_id,
                    'wmscit_produto_embalagem_id': produto_embalagem_id,
                    'wmscit_conferente_id': xmlrpc.login_id,
                    'wmscit_quantidade': item_qtde_conferida * -1 / qtde_na_embalagem_item,
                    'wmscit_data_conferencia': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'wmscit_status_conferencia': '*'
                }
                xmlrpc.execute('wms.conferencia.itens', 'create', dados)

        return item_ids

    @staticmethod
    def conferencia_box_ok(separacao_id):
        """
        Verifica se todos os itens da ordem de servico estao 'Ok'

        :param separacao_id: ID da ordem de servico,
        :type separacao_id: int
        :return: verdadeiro se todos os itens da Ordem de Servico ja foram enderecados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        args = [('os_id', '=', separacao_id)]
        conferencia_ids = xmlrpc.execute('wms.separacao.itens', 'search', args)
        fields = ['item_conferido']
        dados_conferencia = xmlrpc.execute('wms.separacao.itens', 'read', conferencia_ids, fields)

        if dados_conferencia and isinstance(dados_conferencia, list):
            for item in dados_conferencia:
                if not item.get('item_conferido', False):
                    return False
        elif dados_conferencia and isinstance(dados_conferencia, dict):
            if not dados_conferencia.get('item_conferido', False):
                return False

        return True

    @staticmethod
    def registra_contagem(qtde_contada, embalagem_contagem_id, separacao_id, palete, produto_id,
                          grade_id):
        """
        Efetua o registro da contagem/conferencia do item de separacao.

        :param qtde_contada: quantidade contada no coletor
        :param separacao_id: ID da separacao
        :param embalagem_contagem_id: ID da embalagem utilizada na contagem (produto_embalagem_id)
        :param palete: numero do palete
        :param produto_id: ID do produto
        :param grade_id: ID da grade do produto
        :type qtde_contada: int
        :type embalagem_contagem_id: int
        :type separacao_id: int
        :type palete: int
        :type produto_id: int
        :type grade_id: int
        :return: idicacao da contagem: a maior (">"), a menor ("<") ou igual ("=").
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import time

        status = None
        args = [('os_id', '=', separacao_id), ('palete', '=', str(int(palete)).zfill(3)),
                ('produto_id', '=', produto_id), ('grade_id', '=', grade_id)]
        item_ids = xmlrpc.execute('wms.separacao.itens', 'search', args)

        fields = ['pessoa_id', 'produto_embalagem_id', 'quantidade', 'qtde_conferida',
                  'item_conferido']
        wms_separacao_item_dados = xmlrpc.execute('wms.separacao.itens', 'read', item_ids, fields)
        wms_separacao_item_dados = filter(lambda f: not f.get('item_conferido'),
                                          wms_separacao_item_dados)
        wms_separacao_item_dados = sorted(wms_separacao_item_dados,
                                          key=lambda i: i.get('quantidade'))

        for wms_separacao_item_dado in wms_separacao_item_dados if isinstance(
                wms_separacao_item_dados, list) else [wms_separacao_item_dados]:

            pessoa_id = wms_separacao_item_dado.get('pessoa_id')

            if pessoa_id and isinstance(pessoa_id, (list, tuple)):
                pessoa_id = pessoa_id[0]

            # Quantidade na embalagem do item
            produto_embalagem_id = wms_separacao_item_dado.get('produto_embalagem_id')

            if produto_embalagem_id and isinstance(produto_embalagem_id, (tuple, list)):
                produto_embalagem_id = produto_embalagem_id[0]

            item_quantidade = wms_separacao_item_dado.get('quantidade')
            item_qtde_conferida = wms_separacao_item_dado.get('qtde_conferida')
            qtde_na_embalagem_item = 1
            fields = ['embalagem_id']
            dados_produto_embalagem = xmlrpc.execute(
                'estoque.produto.embalagem', 'read', produto_embalagem_id, fields)

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, list):
                dados_produto_embalagem = dados_produto_embalagem[0]

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, dict):
                embalagem_id = dados_produto_embalagem.get('embalagem_id')

                if embalagem_id and isinstance(embalagem_id, (tuple, list)):
                    embalagem_id = embalagem_id[0]

                fields = ['quantidade_na_embalagem']
                dados_embalagem = xmlrpc.execute(
                    'estoque.embalagem', 'read', embalagem_id, fields)

                if dados_embalagem and isinstance(dados_embalagem, list):
                    dados_embalagem = dados_embalagem[0]

                if dados_embalagem and isinstance(dados_embalagem, dict):
                    qtde_na_embalagem_item = dados_embalagem.get('quantidade_na_embalagem')
                    item_quantidade *= qtde_na_embalagem_item
                    item_qtde_conferida *= qtde_na_embalagem_item

            if qtde_contada + item_qtde_conferida > item_quantidade:
                status = '>'
            elif qtde_contada + item_qtde_conferida < item_quantidade:
                status = '<'
            else:
                status = '='

            # Atualiza os dados do item (quantidade conferida)
            if status != '>':
                qtde_conferida = (qtde_contada + item_qtde_conferida) / qtde_na_embalagem_item
                dados = {'qtde_conferida': qtde_conferida}
                xmlrpc.execute(
                    'wms.separacao.itens', 'write', wms_separacao_item_dado.get('id'), dados)

            # Registra o historico da quantidade contada (conferida)
            qtde_na_embalagem_contada = 1

            # Quantidade na embalagem que foi contada
            fields = ['embalagem_id']
            dados_produto_embalagem = xmlrpc.execute(
                'estoque.produto.embalagem', 'read', embalagem_contagem_id, fields)

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, list):
                dados_produto_embalagem = dados_produto_embalagem[0]

            if dados_produto_embalagem and isinstance(dados_produto_embalagem, dict):
                embalagem_id = dados_produto_embalagem.get('embalagem_id')

                if embalagem_id and isinstance(embalagem_id, (tuple, list)):
                    embalagem_id = embalagem_id[0]

                fields = ['quantidade_na_embalagem']
                dados_embalagem = xmlrpc.execute(
                    'estoque.embalagem', 'read', embalagem_id, fields)

                if dados_embalagem and isinstance(dados_embalagem, list):
                    dados_embalagem = dados_embalagem[0]

                if dados_embalagem and isinstance(dados_embalagem, dict):
                    qtde_na_embalagem_contada = dados_embalagem.get('quantidade_na_embalagem')

            dados = {
                'wmscit_separacao_id': separacao_id,
                'wmscit_separacao_item_id': wms_separacao_item_dado.get('id'),
                'wmscit_pessoa_id': pessoa_id,
                'wmscit_produto_id': produto_id,
                'wmscit_grade_id': grade_id,
                'wmscit_produto_embalagem_id': embalagem_contagem_id,
                'wmscit_conferente_id': xmlrpc.login_id,
                'wmscit_quantidade': qtde_contada / qtde_na_embalagem_contada,
                'wmscit_data_conferencia': time.strftime('%Y-%m-%d %H:%M:%S'),
                'wmscit_status_conferencia': status
            }
            xmlrpc.execute('wms.conferencia.itens', 'create', dados)

        return status

    @staticmethod
    def lista_os():
        """
        Lista o numero das ordens de servico.

        :return: lista com os dados das orden de servico.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import time

        # Ordens de servico
        args = [('state', 'in', ['criada', 'aberta']), ('fracionado', '=', False),
                ('seposv_data_fim_separacao', '!=', False)]
        os_ids = xmlrpc.execute('separacao.os', 'search', args)
        lin_det = []  # Lista com os dados das ordens de servico
        fields = ['data_abertura', 'box', 'state']
        dados_os = sorted(
            xmlrpc.execute('separacao.os', 'read', os_ids, fields), key=lambda s: s.get('id'))

        for os in dados_os if isinstance(dados_os, list) else [dados_os]:
            # Data de criacao ou de abertura da ordem de servico
            os_data = os.get('data_abertura')

            if not os_data:
                os_data = os.get('data_criacao')

                if not os_data:  # Hoje
                    os_data = time.strftime('%Y-%m-%d')

            ano, mes, dia = os_data[2:4], os_data[5:7], os_data[8:10]
            os_nr = os.get('id')
            st = os.get('box')
            lin_det.append(
                '{0:6d} {1:>2s}.{2:>2s}.{3:>2s}  {4:>2s}'.format(os_nr, dia, mes, ano, st))

        return lin_det, os_ids

    def mensagem_ok(self, msg="OK."):
        """
        Mostra mensagem.

        :param msg: mensagem,
        :type msg: str
        :return: a posicao do proximo caracter.
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        return termx.say(self.line_ok, 1, msg, *self.color_ok)

    def mensagem_erro(self, msg="ERRO."):
        """
        Mostra mensagem de erro.

        :param msg: mensagem de erro,
        :type msg: str
        :return: a posicao do proximo caracter.
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        return termx.say(self.line_error, 1, msg, *self.color_error)

    @staticmethod
    def recupera_produto_grade_embalagem(cb_id):
        """
        Recupera os dados do produto pelo ID do codigo de barra.

        :param cb_id: ID do numero do codigo de barra
        :type cb_id: int
        :return: os IDs do produto, da grade e da embalagem.
        :rtype: tuple

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        produto_id = grade_id = embalagem_id = 0
        fields = ['produto_id', 'grade_id', 'produto_embalagem_id']
        dados_produto = xmlrpc.execute('automacao.codigo.barra', 'read', cb_id, fields)

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            produto_id = dados_produto.get('produto_id')[0]
            grade_id = dados_produto.get('grade_id')[0]
            embalagem_id = dados_produto.get('produto_embalagem_id')[0]

        return produto_id, grade_id, embalagem_id

    @staticmethod
    def conferir_item(separacao_id, palete, produto_id, grade_id):
        """
        Realiza a conferencia do item de separacao.

        :param separacao_id: ID da montagem de carga
        :param palete: numero do palete
        :param produto_id: ID do produto/item
        :param grade_id: ID da grade do produto/item
        :type separacao_id: int
        :type palete: int
        :type produto_id: int
        :type grade_id: int
        :return: quantidade a ser conferida ou -1  se o item nao for encontrado.
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = 0
        args = [('os_id', '=', separacao_id), ('palete', '=', str(int(palete)).zfill(3)),
                ('produto_id', '=', produto_id), ('grade_id', '=', grade_id)]
        item_ids = xmlrpc.execute('wms.separacao.itens', 'search', args)
        fields = ['produto_embalagem_id', 'quantidade', 'qtde_conferida', 'qtde_cortada']
        wms_separacao_item_dados = xmlrpc.execute('wms.separacao.itens', 'read', item_ids, fields)
        wms_separacao_item_dados = filter(lambda f: not f.get('item_conferido'),
                                          wms_separacao_item_dados)

        if not wms_separacao_item_dados:
            return -1  # Quantidade negativa para item que nao consta na OS ou no palete

        for wms_separacao_item_dado in wms_separacao_item_dados if isinstance(
                wms_separacao_item_dados, list) else [wms_separacao_item_dados]:

            produto_embalagem_id = wms_separacao_item_dado.get('produto_embalagem_id')
            quantidade = wms_separacao_item_dado.get('quantidade')
            qtde_conferida = wms_separacao_item_dado.get('qtde_conferida')
            qtde_cortada = wms_separacao_item_dado.get('qtde_cortada')

            if produto_embalagem_id and isinstance(produto_embalagem_id, (tuple, list)):
                produto_embalagem_id = produto_embalagem_id[0]
                fields = ['embalagem_id']
                dados_produto_embalagem = xmlrpc.execute(
                    'estoque.produto.embalagem', 'read', produto_embalagem_id, fields)

                if dados_produto_embalagem and isinstance(dados_produto_embalagem, list):
                    dados_produto_embalagem = dados_produto_embalagem[0]

                if dados_produto_embalagem and isinstance(dados_produto_embalagem, dict):
                    embalagem_id = dados_produto_embalagem.get('embalagem_id')

                    if embalagem_id and isinstance(embalagem_id, (tuple, list)):
                        embalagem_id = embalagem_id[0]
                        fields = ['quantidade_na_embalagem']
                        dados_embalagem = xmlrpc.execute(
                            'estoque.embalagem', 'read', embalagem_id, fields)

                        if dados_embalagem and isinstance(dados_embalagem, list):
                            dados_embalagem = dados_embalagem[0]

                        if dados_embalagem and isinstance(dados_embalagem, dict):
                            qtde_na_embalagem = dados_embalagem.get('quantidade_na_embalagem')
                            quantidade *= qtde_na_embalagem
                            qtde_conferida *= qtde_na_embalagem
                            qtde_cortada *= qtde_na_embalagem
                            result += (quantidade - qtde_conferida - qtde_cortada)
        return result

    def mensagem_conferencia_ok(self):
        """
        Mostra a mensagem de que a Conferência Box foi efetuada com sucesso.

        :return: o caracter que foi digitado.
        :rtype: str
        """

        termx.beep()
        termx.clear(3)
        termx.say(3, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.say(4, 1, 'CONFERENCIA'.center(termx.max_col), *self.color_msg_ok)
        termx.say(5, 1, 'EFETUADA'.center(termx.max_col), *self.color_msg_ok)
        termx.say(6, 1, 'COM'.center(termx.max_col), *self.color_msg_ok)
        termx.say(7, 1, 'SUCESSO.'.center(termx.max_col), *self.color_msg_ok)
        termx.say(8, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.beep()
        info = 'Pressione uma tecla.'
        termx.say(self.line_info, 1, info, *self.color_info)

        return termx.get(self.line_info, termx.max_col, 0)  # Aguarda uma tecla ser pressionada

ColetorConferenciaBox()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

