# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc


class ColetorDadosLogisticos(object):
    """
    Dados logisticos. Apresenta os valores referente aos dados logisticos de um
    produto.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Cores
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red  # Branco/vermelho
    color_field = termx.underline, termx.fg_green  # Verde (sublinhado)
    color_msg_warning = termx.fg_yellow,  # Amarelo claro
    color_warning = termx.fg_black, termx.bg_yellow,  # Preto/amarelo
    color_info = termx.reverse, termx.fg_cyan  # Preto/azul claro

    def __init__(self):
        self.dados_logisticos()

    def dados_logisticos(self):
        """
        Dados logisticos do produto.

        @return: {boolean} verdadeiro.
        """

        __ESC = chr(27)
        termx.clear(2)

        while True:
            termx.say(2, 1, '- DADOS LOGISTICOS -', *self.color_header)
            termx.say(4, 1, 'Cod. ______________ ')
            termx.say(5, 1, '                    ')
            codigo = termx.get(4, 6, 14)

            if codigo == __ESC:
                break

            if not codigo:
                continue

            if not codigo.isdigit():
                continue

            produto_id = grade_id = False

            args = [('codigo_barra', '=', '%014d' % int(codigo))]
            produto_ids = xmlrpc.execute('automacao.codigo.barra', 'search', args)

            if produto_ids:
                campos = ['produto_id', 'grade_id']
                dados_produto = xmlrpc.execute(
                    'automacao.codigo.barra', 'read', produto_ids, campos)

                if dados_produto:
                    if isinstance(dados_produto, list):
                        dados_produto = dados_produto[0]

                    if isinstance(dados_produto, dict):
                        produto_id = dados_produto.get('produto_id')[0]
                        grade_id = dados_produto.get('grade_id')[0]
            else:  # Busca pelo codigo interno do produto
                args = [('codigo', '=', '%06d' % int(codigo))]
                produto_ids = xmlrpc.execute('estoque.produto', 'search', args)

                if produto_ids:
                    produto_ids = produto_ids[0]

                    termx.say(5, 1, 'Grade __            ')
                    grade = termx.get(5, 7, 2)
                    args = [('grade', '=', grade.upper() if grade else grade),
                            ('produto_id', '=', produto_ids)]
                    produto_ids = xmlrpc.execute('estoque.produto.grade', 'search', args)
                    campos = ['produto_id', 'id']
                    dados_produto = xmlrpc.execute(
                        'estoque.produto.grade', 'read', produto_ids, campos)

                    if dados_produto and isinstance(dados_produto, list):
                        dados_produto = dados_produto[0]

                    if dados_produto and isinstance(dados_produto, dict):
                        produto_id = dados_produto.get('produto_id')[0]
                        grade_id = dados_produto.get('id')
                else:
                    termx.beep()
                    termx.say(10, 1, 'PROD. NAO CADASTRADO', *self.color_error)
                    continue

            # Descricao do produto e dados logisticos
            campos = ['name']
            dados_produto = xmlrpc.execute('estoque.produto', 'read', produto_id, campos)

            if dados_produto and isinstance(dados_produto, list):
                dados_produto = dados_produto[0]

            if dados_produto and isinstance(dados_produto, dict):
                termx.clear(3)
                descricao = '{0:40}'.format(dados_produto.get('name').upper())

            campos = ['lastro', 'camada', 'norma_palete', 'peso_palete', 'fracionado',
                      'local_armazenagem', 'corte', 'shelf_life', 'restricao_nivel_min',
                      'restricao_nivel_max', 'estrutura_tipo_pulmao_id', 'endereco_tipo_pulmao_id',
                      'endereco_produto_id', 'capacidade_picking', 'ponto_reposicao',
                      'embalagem_master_id', 'cubagem', 'embalagem_expedicao_id', 'produto_tipo_id',
                      'picking_dinamico', 'descricao']

            dados_grade = xmlrpc.execute('estoque.produto.grade', 'read', grade_id, campos)

            if dados_grade and isinstance(dados_grade, list):
                dados_grade = dados_grade[0]

            if dados_grade and isinstance(dados_grade, dict):
                # Descricao do produto conforme parametro do estoque
                campos = ['estpar_descricao_grade']
                adm_utils_dados = xmlrpc.execute(
                    'adm.utils', 'admutl_get_parametros', 'estoque', campos)

                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    descricao = dados_grade.get('descricao') or 'ITEM SEM DESCRICAO.'
                    descricao = '{0:40}'.format(descricao)

                termx.say(3, 1, descricao[:20], *self.color_field)
                termx.say(4, 1, descricao[20:40], *self.color_field)

                pg = 1  # Numeracao da pagina
                dados_alterados = False  # Dados foram alterados

                while True:
                    termx.clear(5)

                    if pg == 4:
                        termx.say(5, 1, 'Tipo Produto')
                        termx.say(7, 1, '- ENDERECO PICKING -')
                        termx.say(9, 1, 'Dinamico    ')

                        produto_tipo_id = dados_grade.get('produto_tipo_id')
                        if produto_tipo_id:
                            dados = '%7s' % produto_tipo_id[1][:7]
                        else:
                            dados = ' ' * 7
                        termx.say(5, 14, '%7s' % dados, *self.color_field)

                        endereco_produto_id = dados_grade.get('endereco_produto_id')
                        if endereco_produto_id:
                            dados = dados_grade.get('endereco_produto_id')[1]
                        else:
                            dados = ' ' * 20
                        termx.say(8, 1, dados, *self.color_field)

                        picking_dinamico = dados_grade.get('picking_dinamico')
                        dados = 'SIM' if picking_dinamico else 'NAO'
                        termx.say(9, 18, dados, *self.color_field)

                    if pg == 3:
                        termx.say(5, 1, 'Tipo Estrut.')
                        termx.say(6, 1, 'Tipo End.   ')
                        termx.say(7, 1, 'Ponto Repos.')
                        termx.say(8, 1, 'Emb. Master ')
                        termx.say(9, 1, 'Emb. Venda  ')

                        estrutura_tipo_pulmao_id = dados_grade.get('estrutura_tipo_pulmao_id')
                        if estrutura_tipo_pulmao_id:
                            dados = '%7s' % estrutura_tipo_pulmao_id[1][:7]
                        else:
                            dados = ' ' * 7
                        termx.say(5, 14, dados, *self.color_field)

                        endereco_tipo_pulmao_id = dados_grade.get('endereco_tipo_pulmao_id')
                        if endereco_tipo_pulmao_id:
                            dados = '%7s' % endereco_tipo_pulmao_id[1][:7]
                        else:
                            dados = ' ' * 7
                        termx.say(6, 14, dados, *self.color_field)

                        ponto_reposicao = dados_grade.get('ponto_reposicao')
                        dados = '%7s' % ponto_reposicao
                        termx.say(7, 14, dados, *self.color_field)

                        embalagem_master_id = dados_grade.get('embalagem_master_id')
                        if embalagem_master_id:
                            dados = '%7s' % embalagem_master_id[1].split('-')[0]
                        else:
                            dados = ' ' * 7
                        termx.say(8, 14, dados, *self.color_field)

                        embalagem_expedicao_id = dados_grade.get('embalagem_expedicao_id')
                        if embalagem_expedicao_id:
                            dados = '%7s' % embalagem_expedicao_id[1].split('-')[0]
                        else:
                            dados = ' ' * 7
                        termx.say(9, 14, dados, *self.color_field)

                    if pg == 2:
                        termx.say(5, 1, 'Corte       ')
                        termx.say(6, 1, 'Cap. Picking')
                        termx.say(7, 1, 'Shelf Life  ')
                        termx.say(8, 1, 'Nivel Min.  ')
                        termx.say(9, 1, 'Nivel Max.  ')

                        corte = dados_grade.get('corte')
                        dados = '%6s%%' % corte
                        termx.say(5, 14, dados, *self.color_field)

                        capacidade_picking = dados_grade.get('capacidade_picking')
                        dados = '%7s' % capacidade_picking
                        termx.say(6, 14, dados, *self.color_field)

                        shelf_life = int(dados_grade.get('shelf_life'))
                        dados = '%6s%%' % shelf_life
                        termx.say(7, 14, dados, *self.color_field)

                        restricao_nivel_min = dados_grade.get('restricao_nivel_min')
                        dados = '%7s' % restricao_nivel_min
                        termx.say(8, 14, dados, *self.color_field)

                        restricao_nivel_max = dados_grade.get('restricao_nivel_max')
                        dados = '%7s' % restricao_nivel_max
                        termx.say(9, 14, dados, *self.color_field)

                    if pg == 1:
                        termx.say(5, 1, 'Lastro      ')
                        termx.say(6, 1, 'Camada      ')
                        termx.say(7, 1, 'Norma Palete')
                        termx.say(8, 1, 'Palete (Kg) ')
                        termx.say(9, 1, 'Fracionado  ')

                        lastro = dados_grade.get('lastro') or 0
                        dados = '%7s' % lastro
                        termx.say(5, 14, dados, *self.color_field)

                        camada = dados_grade.get('camada') or 0
                        dados = '%7s' % camada
                        termx.say(6, 14, dados, *self.color_field)

                        norma = lastro * camada
                        dados = '%7s' % norma
                        termx.say(7, 14, dados, *self.color_field)

                        peso_palete = dados_grade.get('peso_palete') or 0
                        dados = '%7s' % peso_palete
                        termx.say(8, 14, dados, *self.color_field)

                        fracionado = dados_grade.get('fracionado', False)
                        dados = '%7s' % ('NAO', 'SIM')[fracionado]
                        termx.say(9, 14, dados, *self.color_field)

                    termx.say(10, 1, '<=4 6=>             ', *self.color_info)

                    op = termx.get(10, 4, 0, False, True)

                    if op == '4' and pg > 1:
                        pg -= 1
                        continue

                    if op == '6' and pg < 4:
                        pg += 1
                        continue

                    # TODO: Ajustar!
                    if op == '00':
                        termx.say(10, 1, '                    ')
                        dados_alterados = True

                        if pg == 1:  # Edicao da pagina 1
                            termx.say(5, 14, '       ')
                            lastro = termx.get(5, 14, 3)
                            if lastro and lastro.isdigit():
                                dados_grade['lastro'] = int(lastro)

                            termx.say(6, 14, '       ')
                            camada = termx.get(6, 14, 3)
                            if camada and camada.isdigit():
                                dados_grade['camada'] = int(camada)

                            termx.say(9, 14, '       ')
                            termx.say(10, 1, '0=NAO  1=SIM        ', *self.color_warning)
                            fracionado = termx.get(9, 14, 1)
                            if fracionado and fracionado.isdigit():
                                dados_grade['fracionado'] = bool(int(fracionado))
                        elif pg == 2:  # Edicao da pagina 2
                            termx.say(5, 14, '       ')
                            corte = termx.get(5, 14, 3)
                            if corte and corte.isdigit():
                                dados_grade['corte'] = int(corte)

                            termx.say(6, 14, '       ')
                            capacidade = termx.get(6, 14, 3)
                            if capacidade and capacidade.isdigit():
                                dados_grade['capacidade_picking'] = int(capacidade)

                            termx.say(7, 14, '       ')
                            shelflife = termx.get(7, 14, 3)
                            if shelflife and shelflife.isdigit():
                                dados_grade['shelf_life'] = int(shelflife)

                            termx.say(8, 14, '       ')
                            minimo = termx.get(8, 14, 2)
                            if minimo and minimo.isdigit():
                                dados_grade['restricao_nivel_min'] = int(minimo)

                            termx.say(9, 14, '       ')
                            maximo = termx.get(9, 14, 2)
                            if maximo and maximo.isdigit():
                                dados_grade['restricao_nivel_max'] = int(maximo)
                        elif pg == 3:  # Edicao da pagina 3
                            termx.say(5, 14, '       ')
                            sigla = termx.get(5, 14, 2)

                            if sigla:
                                args = [('wmsest_sigla', '=', sigla)]
                                dados_id = xmlrpc.execute('wms.estruturatipo', 'search', args)
                                if dados_id:
                                    campos = ['id', 'name']
                                    dados = xmlrpc.execute(
                                        'wms.estruturatipo', 'read', dados_id, campos)

                                    if dados and isinstance(dados, list):
                                        dados = dados[0]

                                    if dados and isinstance(dados, dict):
                                        dados_grade['estrutura_tipo_pulmao_id'] = [dados.get('id'), dados.get('name')]

                            termx.say(6, 14, '       ')
                            sigla = termx.get(6, 14, 3)

                            if sigla:
                                args = [('wmseti_sigla', '=', sigla)]
                                dados_id = xmlrpc.execute('wms.enderecotipo', 'search', args)

                                if dados_id:
                                    campos = ['id', 'name']
                                    dados = xmlrpc.execute(
                                        'wms.enderecotipo', 'read', dados_id, campos)

                                    if dados and isinstance(dados, list):
                                        dados = dados[0]

                                    if dados and isinstance(dados, dict):
                                        dados_grade['endereco_tipo_pulmao_id'] = [dados.get('id'), dados.get('name')]

                            termx.say(7, 14, '       ')
                            reposicao = termx.get(7, 14, 3)

                            if reposicao and reposicao.isdigit():
                                dados_grade['ponto_reposicao'] = int(reposicao)

                            termx.say(8, 14, '       ')
                            sigla = termx.get(8, 14, 7)

                            if sigla:
                                args = [('sigla', '=', sigla)]
                                dados_id = xmlrpc.execute('estoque.embalagem', 'search', args)

                                if dados_id:
                                    campos = ['id', 'name']
                                    dados_embalagem = xmlrpc.execute(
                                        'estoque.embalagem', 'read', dados_id, campos)

                                    if dados_embalagem and isinstance(dados_embalagem, list):
                                        dados_embalagem = dados_embalagem[0]

                                    if dados_embalagem and isinstance(dados_embalagem, dict):
                                        args = [('produto_id', '=', produto_id),
                                                ('embalagem_id', '=', dados_embalagem.get('id'))]

                                        dados_id = xmlrpc.execute(
                                            'estoque.produto.embalagem', 'search', args)
                                        if dados_id:
                                            dados_grade['embalagem_master_id'] = [dados_id[0], dados_embalagem.get('name')]

                            termx.say(9, 14, '       ')
                            sigla = termx.get(9, 14, 7)

                            if sigla:
                                args = [('sigla', '=', sigla)]
                                dados_id = xmlrpc.execute('estoque.embalagem', 'search', args)

                                if dados_id:
                                    campos = ['id', 'name']
                                    dados_embalagem = xmlrpc.execute('estoque.embalagem', 'read', dados_id, campos)

                                    if dados_embalagem and isinstance(dados_embalagem, list):
                                        dados_embalagem = dados_embalagem[0]

                                    if dados_embalagem and isinstance(dados_embalagem, dict):
                                        args = [('produto_id', '=', produto_id),
                                                ('embalagem_id', '=', dados_embalagem.get('id'))]

                                        dados_id = xmlrpc.execute(
                                            'estoque.produto.embalagem', 'search', args)

                                        if dados_id:
                                            dados_grade['embalagem_expedicao_id'] = [dados_id[0], dados_embalagem.get('name')]
                        elif pg == 4:
                            termx.say(5, 14, '       ')
                            sigla = termx.get(5, 14, 3)

                            if sigla:
                                args = [('wmspti_sigla', '=', sigla)]
                                dados_id = xmlrpc.execute('wms.produtotipo', 'search', args)

                                if dados_id:
                                    campos = ['id', 'name']
                                    dados = xmlrpc.execute(
                                        'wms.produtotipo', 'read', dados_id, campos)

                                    if dados and isinstance(dados, list):
                                        dados = dados[0]

                                    if dados and isinstance(dados, dict):
                                        dados_grade['produto_tipo_id'] = [dados_id[0], dados.get('name')]

                            endereco = termx.get(8, 1, 19)

                            if not endereco:
                                termx.say(9, 14, '       ')
                                termx.say(10, 1, '0=NAO  1=SIM        ', *self.color_warning)
                                dinamico = termx.get(9, 14, 1)

                                if dinamico and dinamico.isdigit():
                                    dados_grade['picking_dinamico'] = bool(int(dinamico))

                            elif endereco and not endereco.isdigit():
                                endereco = '.'.join(map(lambda x: '%03d' % int(x),
                                                        endereco.split('.')))
                                args = [('name', '=', endereco)]
                            else:
                                args = [('id', '=', int(endereco))]

                            endereco_id = xmlrpc.execute('wms.endereco', 'search', args)

                            if endereco_id:
                                campos = ['id', 'bloqueado', 'state']
                                dados_endereco = xmlrpc.execute(
                                    'wms.endereco', 'read', [endereco_id], campos)

                                if dados_endereco and isinstance(dados_endereco, list):
                                    dados_endereco = dados_endereco[0]

                                if dados_endereco and isinstance(dados_endereco, dict):
                                    bloqueado = dados_endereco.get('bloqueado')
                                    status = dados_endereco.get('state')
                                    nome = dados_endereco.get('name')

                                    if not bloqueado and status == 'L':
                                        dados_grade['endereco_produto_id'] = [dados_endereco.get('id'), nome]

                        # Mudanca de pagina na edicao
                        if op in '46':
                            if op == '6' and pg < 4:
                                pg += 1
                            elif op == '4' and pg > 1:
                                pg -= 1

                            continue
                        else:
                            termx.beep()

                    if op == '9' or (op == __ESC and dados_alterados):
                        # Grava os dados alterados na pagina
                        if self.alterar_dados_logisticos(grade_id, dados_grade):
                            # Recupera as informacoes do item apos a edicao
                            campos = ['id', 'bloqueado', 'state']
                            dados_grade = xmlrpc.execute(
                                'estoque.produto.grade', 'read', [grade_id], campos)

                            if dados_grade and isinstance(dados_grade, list):
                                dados_grade = dados_grade[0]

                            if dados_grade and isinstance(dados_grade, dict):
                                dados_alterados = False

                    if op == __ESC:
                        termx.clear(2)
                        break
        return True

    @staticmethod
    def alterar_dados_logisticos(grade_id, dados_grade):
        """
        Grava as alteracoes realizadas nos dados logisticos.

        @param grade_id: {int} ID da grade (dados logisticos),
        @param dados_grade: {dict} dicionario com o nome dos campos e seus respectivos valores.
        @return: {boolean} verdadeiro se houve sucesso na alteracao.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        dados = {}

        for campo in dados_grade:
            if isinstance(dados_grade[campo], list):
                valor = dados_grade[campo][0]
            else:
                valor = dados_grade[campo]

            dados[campo] = valor

        result = xmlrpc.execute('estoque.produto.grade', 'write', [grade_id], dados)

        return result

ColetorDadosLogisticos()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
