#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc


class ColetorEnderecamento(object):
    """
    Modulo de enderecamento

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Cores utilizadas
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_warning = termx.fg_black, termx.bg_yellow,  # Preto/amarelo
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_list_os = termx.bold, termx.fg_yellow, termx.underline,  # Amarelo claro (sublinhado)
    color_os = termx.fg_yellow,  # Amarelo
    color_msg_ok = termx.fg_green,  # Verde claro
    color_msg_wait = termx.fg_green, termx.bold, termx.blink  # Verde claro (pisca)

    line_error = termx.max_row  # Linha onde sao mostradas as mensagens de erro
    line_info = termx.max_row - 1  # Linha onde sao mostradas as informacoes
    rec_sucess = True  # Enderecamento efetuado com sucesso
    rec_continue = False  # O enderecamento continua

    def __init__(self):
        self.parametro = self.parametros_enderecamento()  # Parametros do enderecamento
        self.enderecamento_os()

    @staticmethod
    def parametros_enderecamento():
        """
        Leitura dos parametros utilizados pelo enderecamento.

        :return: {dict} dicionario com os valores dos parametros

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = {}
        parametro_ids = xmlrpc.execute('estoque.parametro', 'search', [])
        campos = ['codigo_numerico', 'estpar_tamanho_codigo']
        dados_parametro = xmlrpc.execute('estoque.parametro', 'read', parametro_ids, campos)

        if dados_parametro and isinstance(dados_parametro, list):
            result = dados_parametro[0]

        if dados_parametro and isinstance(dados_parametro, dict):
            result = dados_parametro

        return result

    def enderecamento_os(self):
        """
        Mostra lista com detalhes das ordens de servico de enderecamento

        :return: o numero da ordem de servico de enderecamento.
        :rtype: int
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(2)
        termx.say(2, 1, ' - ENDERECAMENTO -  ', *self.color_header)
        ordem_servico = False

        while True:
            ordem_servico = self.ordem_servico_enderecamento()  # Selecao da ordem de servico
            if not ordem_servico:
                break
            ordem_servico = int(ordem_servico)
            self.enderecamento_item(ordem_servico)

        return ordem_servico

    def ordem_servico_enderecamento(self):
        """
        Ordens de servico de enderecamento destinadas ao usuario.

        :return: numero da ordem de servico selecionada.
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(3)
        termx.say(3, 1, 'OS Nr.', *self.color_list_os)
        termx.say(3, 8, '  Data  ', *self.color_list_os)
        termx.say(3, 17, 'Box', *self.color_list_os)

        # Selecao de uma ordem de servico
        os_numero = 0
        while True:
            lin_det, lista_os = self.lista_os()  # Mostra a lista de ordens de servico
            lin = 4
            for i in range(0, len(lin_det)):
                termx.say(lin, 1, lin_det[i], *self.color_os)
                lin += 1
                if (lin - 2) > 4:
                    break
            termx.say(8, 1, 'OS Nr. ______')
            os_numero = termx.get(8, 8, 6)  # Aguarda a selecao
            if os_numero == chr(27):
                return False
            if not os_numero:
                continue
            if not os_numero.isdigit():
                continue
            elif not int(os_numero) in lista_os:
                continue
            break

        return os_numero

    def enderecamento_item(self, enderecamento_id):
        """
        Enderecamento dos itens do recebimento

        :param enderecamento_id: ID da ordem de servico.
        :type enderecamento_id: int
        :return: verdadeiro se o item foi enderecado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        :change: 07.08.2013 - pierre.barbosa
        """

        # Formulario
        def form_enderecamento():
            """
            Tela de enderecamento.

            :return: {boolean} verdadeiro.
            """

            termx.clear(3)
            termx.say(3, 1, 'Etiqueta ___________')
            termx.say(4, 1, 'Cod. _______________')
            termx.say(6, 1, 'Endereco de destino:')
            termx.say(7, 1, '000.000.000.000.000 ')
            termx.say(8, 1, '___________________ ')
            termx.say(9, 1, 'Qtde. ______        ')

            return True

        # Mostra o formulario de enderecamento
        form_enderecamento()
        __ESC = chr(27)

        while True:  # Etiqueta do palete
            etiqueta = termx.get(3, 10, 11)  # Codigo de barras do item de enderecamento

            if not etiqueta:
                continue
            elif etiqueta == __ESC:
                break
            elif not etiqueta.isdigit():
                self.mensagem_erro('ETIQUETA INVALIDA!  ')
                continue

            # Recupera os dados da etiqueta do palete
            args = [('id', '=', etiqueta)]
            item_ids = xmlrpc.execute('wms.enderecamento.itens', 'search', args)

            if not item_ids:
                self.mensagem_erro('NAO CONSTA NA O.S.  ')
                continue

            # Item ainda nao enderecado
            dados_item = xmlrpc.execute('wms.enderecamento.itens', 'read', item_ids, [])

            if dados_item and isinstance(dados_item, list):
                dados_item = dados_item[0]

            # Verifica se o item ja foi enderecado
            if dados_item.get('wmseit_enderecado'):
                self.mensagem_erro('ITEM JA ENDERECADO! ')
                continue

            while True:  # Codigo do produto
                codigo = termx.get(4, 6, 15)

                if codigo == __ESC:
                    break
                elif not codigo:
                    continue
                elif not codigo.isdigit():
                    self.mensagem_erro('ITEM NAO CADASTRADO.')
                    continue

                # Recupera os dados do item/produto
                args = [('codigo_barra', '=', '%014d' % int(codigo))]
                ids = xmlrpc.execute('automacao.codigo.barra', 'search', args)
                qtde_embalagem = 1  # Quantidade minima
                produto_id, grade_id, embalagem_id = self.recupera_produto_grade_embalagem(ids)

                if not produto_id:
                    # Recupera os dados do item/produto pelo codigo interno
                    tamanho_codigo = self.parametro.get('estpar_tamanho_codigo', 6)
                    codigo_numerico = self.parametro.get('codigo_numerico')

                    if codigo_numerico:
                        codigo = codigo[tamanho_codigo * -1:]
                    else:
                        codigo = codigo[:tamanho_codigo]

                    args = [('codigo', '=', codigo)]
                    ids = xmlrpc.execute('estoque.produto', 'search', args)

                    if ids and isinstance(ids, list):
                        produto_id = ids[0]

                        # Recupera a grade do produto
                        model = 'estoque.produto.grade'
                        args = [('produto_id', '=', produto_id)]
                        grade_ids = xmlrpc.execute(model, 'search', args)

                        if grade_ids and isinstance(grade_ids, list):
                            grade_id = grade_ids[0]
                        else:
                            grade_id = 0
                else:
                    if embalagem_id:
                        campos = ['embalagem_id']
                        dados_embalagem = xmlrpc.execute(
                            'estoque.produto.embalagem', 'read', [embalagem_id], campos)

                        if dados_embalagem and isinstance(dados_embalagem, list):
                            dados_embalagem = dados_embalagem[0]

                        if dados_embalagem and isinstance(dados_embalagem, dict):
                            embalagem_id = dados_embalagem.get('embalagem_id')

                            if embalagem_id and isinstance(embalagem_id, (list, tuple)):
                                embalagem_id = embalagem_id[0]

                                campos = ['quantidade_na_embalagem']
                                dados_embalagem = xmlrpc.execute('estoque.embalagem', 'read',
                                                                 [embalagem_id], campos)

                                if dados_embalagem and isinstance(dados_embalagem, list):
                                    dados_embalagem = dados_embalagem[0]

                                if dados_embalagem and isinstance(dados_embalagem, dict):
                                    qtde_embalagem = dados_embalagem.get('quantidade_na_embalagem')
                                    qtde_embalagem = int(qtde_embalagem)

                if not produto_id:  # Nao encontrou o codigo do produto
                    self.mensagem_erro('ITEM NAO CADASTRADO.')
                    continue

                # Item nao confere
                if produto_id != dados_item.get('wmseit_produto_id')[0]:
                    self.mensagem_erro('ITEM NAO CONFERE.   ')
                    continue

                # Fator de multiplicacao
                termx.say(9, 14, 'x')
                termx.say(9, 16, str(qtde_embalagem))

                # Descricao do produto conforme parametro do estoque
                descricao = dados_item.get('wmseit_produto_id')[1] or 'ITEM SEM DESCRICAO.'
                campos = ['estpar_descricao_grade']
                adm_utils_dados = xmlrpc.execute(
                    'adm.utils', 'admutl_get_parametros', 'estoque', campos)

                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    campos = ['descricao']
                    dados_grade = xmlrpc.execute('estoque.produto.grade', 'read', grade_id, campos)

                    if dados_grade and isinstance(dados_grade, list):
                        dados_grade = dados_grade[0]
                        descricao = dados_grade.get('descricao') or 'ITEM SEM DESCRICAO.'

                descricao = '{0:40}'.format(descricao)

                termx.erase_line(5, 5)
                termx.say(5, 1, '{0:20s}'.format(descricao), *self.color_field)
                termx.say(7, 1, dados_item.get('wmseit_endereco_id')[1])

                destino = None  # Endereco de destino (enderecamento)
                qtde = 0  # Quantidade a ser enderecada

                while True:  # Endereco de destino
                    destino = termx.get(8, 1, 19)

                    if destino == __ESC:
                        break
                    elif not destino:
                        continue

                    if destino.isdigit():
                        destino = int(destino)

                    if destino not in dados_item.get('wmseit_endereco_id'):
                        # Verifica permissao de sobreposicao
                        if self.permite_sobreposicao(xmlrpc.login_id):
                            args = [('state', '=', 'L'), ('bloqueado', '=', False),
                                    ('tipo_nivel', '!=', 'PK')]

                            if str(destino).isdigit():
                                args.append(('id', '=', destino))
                            else:
                                args.append(('name', '=', destino))

                            endereco_ids = xmlrpc.execute('wms.endereco', 'search', args)
                            if endereco_ids:
                                termx.say(
                                    termx.max_row, 1, '*** SOBREPOSICAO ***', *self.color_warning)
                                endereco_dados = xmlrpc.execute(
                                    'wms.endereco', 'read', endereco_ids)
                                if endereco_dados and isinstance(endereco_dados, list):
                                    endereco_dados = endereco_dados[0]

                                # Troca o endereco na sobreposicao
                                dados_item['wmseit_endereco_id'] = (
                                    endereco_dados.get('id'), endereco_dados.get('name'))
                                dados = dict(wmseit_endereco_id=endereco_dados.get('id'))
                                xmlrpc.execute('wms.enderecamento.itens', 'write', etiqueta, dados)
                            else:
                                self.mensagem_erro('ENDERECO NAO CONFERE')
                                continue
                        else:
                            self.mensagem_erro('ENDERECO NAO CONFERE')
                            continue

                    termx.say(8, 1, dados_item.get('wmseit_endereco_id')[1], *self.color_field)

                    while True:  # Quantidade
                        qtde = termx.get(9, 7, 6)

                        if qtde == __ESC:
                            # Retorna ao endereco original
                            break

                        # Verifica se as quantidades sao iguais
                        if not qtde or not qtde.isdigit():
                            continue

                        qtde = int(qtde)
                        qtde *= qtde_embalagem

                        if dados_item.get('wmseit_quantidade') != qtde:
                            self.mensagem_erro('QTDE. NAO CONFERE.')
                            continue

                        # Marca o item como enderecado
                        dados = {'wmseit_enderecado': True}
                        item_id = dados_item.get('id', 0)
                        xmlrpc.execute('wms.enderecamento.itens', 'write', item_id, dados)

                        # Verifica se todos os itens ja foram conferidos
                        if self.enderecamento_ok(enderecamento_id):
                            self.mensagem_enderecamento_ok()

                            # Fecha a ordem de servico
                            xmlrpc.execute('recebimento.os', 'osvosv_fechar_os', enderecamento_id)

                            return self.rec_sucess

                        break

                    if qtde == __ESC:
                        termx.say(8, 1, 'Qtde. ______')
                        continue

                    break

                if destino == __ESC:
                    termx.erase_line(5, 5)
                    termx.say(8, 1, '___________________ ')
                    termx.say(9, 1, 'Qtde. ______        ')
                    continue

                break

            form_enderecamento()

        return self.rec_continue

    @staticmethod
    def enderecamento_ok(enderecamento_id):
        """
        Verifica se todos os itens ja foram conferidos.

        :param enderecamento_id: ID da ordem de servico de enderecamento.
        :type enderecamento_id: int
        :return: verdadeiro se todos os itens foram enderecados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = False
        campos = ['recosv_enderecamento_ok']
        dados_enderecamento = xmlrpc.execute('recebimento.os', 'read', enderecamento_id, campos)

        if dados_enderecamento and isinstance(dados_enderecamento, list):
            dados_enderecamento = dados_enderecamento[0]

        if dados_enderecamento and isinstance(dados_enderecamento, dict):
            result = dados_enderecamento.get('recosv_enderecamento_ok', False)

        return result

    @staticmethod
    def sobreposicao(endereco):
        """
        Verifica a possibilidade de sobreposicao de endereco .

        :param endereco: ID do endereco a ser reposto,
        :type endereco: int
        :return: dados do endereco de sobreposicao.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = {}  # Dados do endereco de sobreposicao

        # Verifica se o endereco e valido para sobreposicao.
        # Deve ser pulmao nao bloqueado e livre.
        if isinstance(endereco, (list, tuple)):
            endereco = endereco[0]
        else:
            endereco = int(endereco)

        args = [('id', '=', endereco), ('state', '=', 'L'), ('bloqueado', '=', False)]
        endereco_ids = xmlrpc.execute('wms.endereco', 'search', args)

        if endereco_ids:
            campos = ['id', 'name']
            dados_endereco = xmlrpc.execute('wms.endereco', 'read', endereco_ids, campos)

            if dados_endereco and isinstance(dados_endereco, list):
                dados_endereco = dados_endereco[0]

            if dados_endereco and isinstance(dados_endereco, dict):
                result = dados_endereco

        return result

    def lista_os(self):
        """
        Lista das ordens de servico.

        :return: lista com os dados da ordem de servico.
        :rtype: list
        """

        lin_det = []  # Lista com os dados das ordens de servico
        lista_os = []  # Lista com o ID das ordens de servico
        dados_os = []  # Dados da ordem de servico

        termx.say(self.line_info, 1, 'Aguarde...', *self.color_msg_wait)
        termx.say(self.line_info - 1, 11, '')

        # Ordens de Servico
        args = [('state', 'in', ['criada', 'aberta'])]
        ids = xmlrpc.execute('recebimento.os', 'search', args)

        if ids:
            campos = ['data_abertura', 'responsavel_ids', 'box', 'state', 'recosv_recebimento_ok']
            dados_os = xmlrpc.execute('recebimento.os', 'read', ids, campos)

        if dados_os and isinstance(dados_os, list):
            for os in dados_os:
                if self.enderecamento_ok(os.get('id')):
                    continue

                # Somente recebimentos efetuados
                if not os.get('recosv_recebimento_ok'):
                    continue

                lista_os.append(os.get('id'))  # Acrescenta o nr. da ordem de servico a lista

                linefrmt = '{0:6d} {1:>2s}.{2:>2s}.{3:>2s}  {4:>2s}'

                # Data de criacao ou de abertura da ordem de servico
                os_data = os.get('data_abertura')

                if not os_data:
                    os_data = os.get('data_criacao')

                ano = os_data[2:4]
                mes = os_data[5:7]
                dia = os_data[8:10]
                os_nr = os.get('id')
                box = os.get('box')
                linefrmt = linefrmt.format(os_nr, dia, mes, ano, box)

                lin_det.append(linefrmt)

        termx.clear(self.line_info)

        return lin_det, lista_os

    def mensagem_enderecamento_ok(self):
        """
        Mostra a mensagem de que o enderecamento foi efetuado com sucesso.

        :return: uma mensagem.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.beep()
        termx.clear(3)
        termx.say(3, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.say(4, 1, 'ENDERECAMENTO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(5, 1, 'EFETUADO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(6, 1, 'COM'.center(termx.max_col), *self.color_msg_ok)
        termx.say(7, 1, 'SUCESSO.'.center(termx.max_col), *self.color_msg_ok)
        termx.say(8, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.beep()
        info = 'Pressione uma tecla.'
        termx.say(self.line_info, 1, info, *self.color_info)

        return termx.get(self.line_info, termx.max_col, 0)  # Aguarda uma tecla ser pressionada

    def mensagem_erro(self, msg="ERRO."):
        """
        Mostra mensagem de erro.

        :param msg: mensagem de erro,
        :type msg: str
        :return: ultima posicao de impressao
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        result = termx.say(self.line_error, 1, msg, *self.color_error)

        return result

    @staticmethod
    def recupera_produto_grade_embalagem(cb_id):
        """
        Recupera os dados do produto pelo ID do codigo de barra.

        :param cb_id: ID do numero do codigo de barra,
        :type cb_id: int
        :return: os IDs do produto, da grade e da embalagem.
        :rtype: tuple

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        produto_id = grade_id = embalagem_id = 0
        campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
        dados_produto = xmlrpc.execute('automacao.codigo.barra', 'read', cb_id, campos)

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            produto_id = dados_produto.get('produto_id')[0]
            grade_id = dados_produto.get('grade_id')[0]
            embalagem_id = dados_produto.get('produto_embalagem_id')[0]

        result = produto_id, grade_id, embalagem_id

        return result

    @staticmethod
    def permite_sobreposicao(uid):
        """
        Verifica se o pode ou nao realizar sobreposicao de endereco.

        :param uid: ID do usuario para verificacao de perfil,
        :type uid: int
        :return: verdadeiro se o usuario tem permissao de sobrepor enderecos.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        args = [("usuario_id", "=", uid)]
        usuario_id = xmlrpc.execute('usuario.coletor', 'search', args)
        campos = ['sobrepor_endereco']
        dados_usuario = xmlrpc.execute('usuario.coletor', 'read', usuario_id, campos)

        if dados_usuario and isinstance(dados_usuario, list):
            dados_usuario = dados_usuario[0]

        if dados_usuario and isinstance(dados_usuario, dict):
            result = dados_usuario.get('sobrepor_endereco') or False
        else:
            result = False

        return result

    @staticmethod
    def recupera_dados_item(enderecamento_id, grade_id, endereco_id):
        """
        Recupera dados dos itens de enderecamento.

        :param enderecamento_id: ID da ordem de servico de enderecamento,
        :param grade_id: ID da grade do item/produto,
        :param endereco_id: ID do endereco para recuperar os dados,
        :type enderecamento_id: int
        :type grade_id: int
        :type endereco_id: int
        :return: dicionario com os dados do item.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Busca pelo item a ser enderecado
        args = [('recebimento_id', '=', int(enderecamento_id)), ('grade_id', '=', grade_id)]
        item_ids = xmlrpc.execute('recebimento.itens', 'search', args)

        # Recupera dados do item a ser enderecado
        campos = ['wmseit_endereco_id', 'wmseit_quantidade', 'wmseit_enderecado']
        args = [('wmseit_recebimento_item_id', 'in', item_ids),
                ('wmseit_endereco_id', '=', endereco_id)]
        item_ids = xmlrpc.execute('wms.enderecamento.itens', 'search', args)
        result = xmlrpc.execute('wms.enderecamento.itens', 'read', item_ids, campos)

        return result

ColetorEnderecamento()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
