#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Não Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

# noinspection PyUnresolvedReferences
from coletor import termx, xmlrpc


class ColetorInventarioContagem(object):
    """
    Inventario.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Cores utilizadas
    color_input = termx.bold, termx.underline, termx.fg_white  # Branco intenso/sublinhado
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_list_os = termx.bold, termx.fg_yellow, termx.underline,  # Amarelo claro (sublinhado)
    color_os = termx.fg_yellow,  # Amarelo
    color_msg_ok = termx.fg_green,  # Verde claro
    line_error = termx.max_row  # Linha onde sao mostradas as mensagens de erro
    line_info = line_error  # Linha onde sao mostradas as informacoes
    rec_sucess = True  # Enderecamento efetuado com sucesso
    rec_continue = False  # O enderecamento continua

    def __init__(self):
        self.inventario_os()

    def inventario_os(self):
        """
        Mostra lista com detalhes das ordens de servico de inventario

        :return: numero da ordem de servico de inventario
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(2)
        termx.say(2, 1, '   - INVENTARIO -   ', *self.color_header)
        ordem_servico = False

        while True:
            ordem_servico = self.ordem_servico_inventario()  # Selecao da ordem de servico

            if not ordem_servico:
                break

            ordem_servico = long(ordem_servico)
            self.contagem_inventario(ordem_servico)

        return ordem_servico

    def ordem_servico_inventario(self):
        """
        Ordens de servico de inventario destinadas ao usuario.

        :return: numero da ordem de servico selecionada.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(3)
        termx.say(3, 1, 'OS Nr.', *self.color_list_os)
        termx.say(3, 9, 'Ct.', *self.color_list_os)
        termx.say(3, 13, '  %', *self.color_list_os)
        termx.say(3, 18, 'St', *self.color_list_os)

        # Selecao de uma ordem de servico
        os_numero = 0
        __ESC = chr(27)

        while True:
            lin_det, lista_os = self.lista_os()  # Mostra a lista de ordens de servico
            lin = 4

            for i in range(0, len(lin_det)):
                termx.say(lin, 1, lin_det[i], *self.color_os)
                lin += 1
                if (lin - 2) > 4:
                    break
            termx.say(8, 1, 'OS Nr. ______')
            os_numero = termx.get(8, 8, 6)  # Aguarda a selecao

            if os_numero == __ESC:
                return False

            if not os_numero:
                continue

            if not os_numero.isdigit():
                continue
            elif not int(os_numero) in lista_os:
                continue

            # Acrescenta usuario como responsavel se nao constar na ordem de servico
            campos = ['conferente_id']
            dados_inventario = xmlrpc.execute('inventario.os', 'read', int(os_numero), campos)

            if dados_inventario and isinstance(dados_inventario, list):
                dados_inventario = dados_inventario[0]

            if dados_inventario and isinstance(dados_inventario, dict):
                if not dados_inventario.get('conferente_id'):
                    dados = {'conferente_id': xmlrpc.login_id}
                    xmlrpc.execute('inventario.os', 'write', int(os_numero), dados)

            # Abre a ordem de servico se estiver apenas criada
            args = [('id', '=', int(os_numero)), ('state', '=', 'criada')]

            if xmlrpc.execute('inventario.os', 'search', args):
                xmlrpc.execute('inventario.os', 'osvosv_abrir_os', int(os_numero))

            break

        return os_numero

    @staticmethod
    def lista_os():
        """
        Lista das Ordens de servico.

        :return: lista com os dados da ordem de servico.
        :rtype: list

        """

        lin_det = []  # Lista com os dados das ordens de servico
        lista_os = []  # Lista com o ID das ordens de servico
        dados_os = []  # Dados da ordem de servico

        # Ordens de Servico
        args = [('state', 'in', ['criada', 'aberta'])]
        ids = xmlrpc.execute('inventario.os', 'search', args)

        if ids:
            campos = ['conferente_id', 'inventariado_percent', 'state', 'invosv_contagem']
            dados_os = xmlrpc.execute('inventario.os', 'read', ids, campos)

        if dados_os and isinstance(dados_os, list):
            for os in dados_os:
                # verifica se os itens da ordem de servico ja foram todos inventariados
                if os.get('inventariado_percent') >= 100.0:  # todos os intens inventariados (100%)
                    continue

                lista_os.append(os.get('id'))  # acrescenta o nr. da ordem de servico a lista
                linefrmt = '{0:6d} {1:>4s} {2:>3s} {3:>2s}'
                os_nr = os.get('id')
                cont = str(os.get('invosv_contagem', 1))
                percent = '%.0f' % os.get('inventariado_percent', 0.00)

                # ´´status´´ da contagem do inventario
                st = os.get('state').upper()

                linefrmt = linefrmt.format(os_nr, cont, percent, st)
                lin_det.append(linefrmt)

        return lin_det, lista_os

    def contagem_inventario(self, inventario_id):
        """
        Contagem do inventario.

        :param inventario_id:o id da ordem de servico de inventario.
        :return: n/a

        :author: pierre barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from time import strptime

        # Formulario
        def form_enderecamento():
            """
            Formulario (tela) de enderecamento.

            :return: sempre verdadeiro.
            :rtype: bool

            :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
            """

            termx.clear(3)
            termx.say(3, 1, 'Endereco de contagem')
            termx.say(4, 1, '_' * 19)
            termx.say(5, 1, 'Cod.  ______________')
            termx.say(6, 1, 'Grade __')
            termx.say(7, 1, '_' * 20)
            termx.say(8, 1, 'Qtde. ________      ')

            return True

        form_enderecamento()  # Mostra o formulario de contagem
        __ESC = chr(27)

        # Contagem
        args = [('os_id', '=', inventario_id), ('state', '=', 'C')]
        inventario_item_ids = xmlrpc.execute('wms.inventario.itens', 'search', args)
        campos = ['endereco_id', 'wmsiit_produto_id', 'wmsiit_grade_id', 'wmsiit_embalagem_id',
                  'verifica_validade']

        for inventario_item_dados in xmlrpc.execute(
                'wms.inventario.itens', 'read', inventario_item_ids, campos, {}, '_classic_read'):

            endereco = inventario_item_dados.get('endereco_id')
            termx.say(4, 1, endereco[1], *self.color_field)
            verifica_validade = inventario_item_dados.get('verifica_validade', False)
            item_id = inventario_item_dados.get('id')
            qtde = codigo = validade = produto_id = grade_id = qtde_na_embalagem = False
            args = [('id', '=', endereco[0]), ('tipo_nivel', '=', 'PK')]
            picking = xmlrpc.execute('wms.endereco', 'search', args)
            descricao = ''

            while not codigo:  # Item/produto
                termx.say(5, 1, 'Cod.  ______________')
                termx.say(6, 1, 'Grade __')
                termx.say(7, 1, '_' * 20)  # Apaga descricao do produto
                codigo = termx.get(5, 7, 14)

                if not codigo or codigo == __ESC:
                    break

                if codigo and codigo.isdigit():
                    codigo = int(codigo)
                elif not codigo:
                    codigo = 0
                else:
                    codigo = False
                    continue

                if codigo:
                    args = [('codigo_barra', '=', '%014d' % int(codigo))]
                    produto_ids = xmlrpc.execute('automacao.codigo.barra', 'search', args)

                    if produto_ids:
                        campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
                        dados_produto = xmlrpc.execute(
                            'automacao.codigo.barra', 'read', produto_ids, campos, {})

                        if dados_produto:
                            if isinstance(dados_produto, list):
                                dados_produto = dados_produto[0]

                            if isinstance(dados_produto, dict):
                                produto = dados_produto.get('produto_id')[1]
                                produto_id = dados_produto.get('produto_id')[0]
                                grade_id = dados_produto.get('grade_id')[0]
                                produto_embalagem_id = dados_produto.get('produto_embalagem_id')[0]
                                produto = produto.split('-')[1].strip().encode('utf-8')
                                descricao = '{0:20}'.format(produto)

                                # Grades do produto
                                campos = ['grade', 'descricao']
                                modelo = 'estoque.produto.grade'
                                dados_grade = xmlrpc.execute(modelo, 'read', grade_id, campos)

                                if dados_grade and isinstance(dados_grade, list):
                                    dados_grade = dados_grade[0]

                                if dados_grade and isinstance(dados_grade, dict):
                                    termx.say(6, 7, dados_grade.get('grade'))

                                # Descricao do produto conforme parametro do estoque
                                campos = ['estpar_descricao_grade']
                                adm_utils_dados = xmlrpc.execute(
                                    'adm.utils', 'admutl_get_parametros', 'estoque', campos)

                                # Apenas a grade
                                if adm_utils_dados.get('estpar_descricao_grade') == 'G':
                                        descricao = dados_grade.get('descricao') or 'SEM DESCRICAO.'
                                        descricao = '{0:40}'.format(descricao)

                                campos = ['embalagem_id']
                                modelo = 'estoque.produto.embalagem'
                                dados_embalagem = xmlrpc.execute(
                                    modelo, 'read', produto_embalagem_id, campos)

                                if dados_embalagem and isinstance(dados_embalagem, list):
                                    dados_embalagem = dados_embalagem[0]

                                if dados_embalagem and isinstance(dados_embalagem, dict):
                                    embalagem_id = dados_embalagem.get('embalagem_id')[0]

                                    campos = ['quantidade_na_embalagem', 'name']
                                    modelo = 'estoque.embalagem'
                                    dados_embalagem = xmlrpc.execute(
                                        modelo, 'read', embalagem_id, campos)

                                    if dados_embalagem and isinstance(dados_embalagem, list):
                                        dados_embalagem = dados_embalagem[0]

                                    if dados_embalagem and isinstance(dados_embalagem, dict):
                                        qtde_na_embalagem = dados_embalagem.get(
                                            'quantidade_na_embalagem')
                                        termx.say(8, 16, 'X %s' % int(qtde_na_embalagem))
                    else:
                        self.mensagem_erro('COD. PROD. INVALIDO.')
                        continue

                # Descricao do produto e dados logisticos
                termx.say(7, 1, '_' * 20)
                termx.say(7, 1, descricao[:20], *self.color_field)

                while codigo and not qtde:  # Quantidade
                    termx.say(9, 1, '                    ')
                    qtde = termx.get(8, 7, 8)

                    if qtde == __ESC:
                        codigo = False
                        break

                    if qtde and qtde.isdigit():
                        qtde = int(qtde)
                    elif not qtde:
                        qtde = 0
                        termx.say(8, 7, '0       ', *self.color_input)
                    else:
                        continue

                    if verifica_validade or picking:  # Se picking, a validade deve ser informada
                        val_dd = val_mm = val_aa = False

                        while not validade:  # Data de validade
                            termx.say(9, 1, 'Validade __/__/__   ')

                            while not val_dd:  # Dia
                                val_dd = termx.get(9, 10, 2, go=True)

                                if not val_dd:
                                    if picking:
                                        continue  # A validade deve ser informada
                                    else:
                                        break
                                elif val_dd == __ESC:
                                    qtde = False
                                    break

                                if val_dd.isdigit():
                                    val_dd = int(val_dd)
                                    break
                                else:
                                    val_dd = False

                            if val_dd == __ESC or not val_dd:  # Data em branco
                                break

                            while not val_mm:  # Mes
                                termx.say(9, 13, '__')
                                val_mm = termx.get(9, 13, 2, go=True)

                                if val_mm == __ESC:
                                    break

                                if val_mm and val_mm.isdigit():
                                    val_mm = int(val_mm)
                                    break
                                else:
                                    val_mm = False

                            if val_mm == __ESC:
                                qtde = False
                                validade = False
                                break

                            while not val_aa:  # Ano
                                termx.say(9, 16, '__')
                                val_aa = termx.get(9, 16, 2, go=True)

                                if val_aa == __ESC:
                                    break

                                if val_aa and val_aa.isdigit():
                                    val_aa = int(val_aa)
                                    break
                                else:
                                    val_aa = False

                            if val_aa == __ESC:
                                qtde = False
                                validade = False
                                break

                            if val_dd == __ESC or val_mm == __ESC or val_aa == __ESC:
                                continue  # Solicita novamente o codigo do produto
                            else:
                                try:
                                    dt = strptime(
                                        '%02d/%02d/%02d' % (val_dd, val_mm, val_aa), '%d/%m/%y')
                                    validade = '%02d/%02d/%04d' % (dt[2], dt[1], dt[0])
                                    break
                                except:
                                    self.mensagem_erro('DATA INVALIDA.      ')
                                    continue

                        if val_dd == __ESC:
                            qtde = False
                            continue

                if qtde == __ESC:
                    continue

            # Se nao houver produto deve ser feita a confirmacao
            conf = 'X'

            while not codigo and conf not in 'SsNnYy01':
                termx.say(9, 1, 'CONFIRMA? [ ] S/N   ')
                conf = termx.get(9, 12, 1)
                conf = 'X' if not conf else conf
                termx.say(9, 1, '                    ')

            if codigo == __ESC:
                break

            if not codigo and conf not in 'SsYy1':
                continue

            # Atualiza dados do item de contagem de inventario
            dados = dict(wmsiit_produto_id=produto_id, wmsiit_grade_id=grade_id,
                         wmsiit_quantidade=qtde * qtde_na_embalagem, validade=validade,
                         state='O')

            xmlrpc.execute('wms.inventario.itens', 'write', item_id, dados)

            # Verifica se todos os itens ja foram inventariados
            if self.inventario_ok(inventario_id):
                self.mensagem_inventario_ok()

                # Fecha a ordem de servico
                xmlrpc.execute('inventario.os', 'osvosv_fechar_os', inventario_id)

                return self.rec_sucess

            termx.say(5, 1, 'Cod.  ______________')
            termx.say(6, 1, 'Grade __')
            termx.say(7, 1, '_' * 20)  # Apaga descricao do produto
            termx.say(8, 1, 'Qtde. ________      ')
            termx.say(9, 1, '                    ')

    @staticmethod
    def inventario_ok(inventario_id):
        """
        Verifica se todos os itens ja foram conferidos.

        :param inventario_id: ID da ordem de servico de inventario.
        :type inventario_id: int|long

        :return: verdadeiro se todos os itens foram inventariados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = False
        args = [('os_id', '=', inventario_id), ('state', 'in', ['C', 'D'])]
        item_ids = xmlrpc.execute('wms.inventario.itens', 'search', args)

        if not item_ids:
            result = True

        return result

    def mensagem_inventario_ok(self):
        """
        Mensagem de contagem. Todos os itens do inventario ja foram contados.

        :return: a tecla que foi pressionada.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.beep()
        termx.clear(3)
        termx.say(3, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.say(4, 1, 'INVENTARIO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(5, 1, 'EFETUADO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(6, 1, 'COM'.center(termx.max_col), *self.color_msg_ok)
        termx.say(7, 1, 'SUCESSO.'.center(termx.max_col), *self.color_msg_ok)
        termx.say(8, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.beep()
        info = 'Pressione uma tecla.'
        termx.say(9, 1, info, *self.color_info)

        return termx.get(self.line_info, termx.max_col, 0)  # Aguarda uma tecla ser pressionada

    def mensagem_erro(self, msg="ERRO."):
        """
        Mostra mensagem de erro.

        :param msg: mensagem de erro
        :type msg: str

        :return: a posicao do proximo caracter
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        return termx.say(self.line_error, 1, msg, *self.color_error)

ColetorInventarioContagem()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
