#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuição-NãoComercial-Compartilhada Igual 3.0 Não Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc


class ColetorRecebimentoCego(object):
    """
    Classe principal. Recebimento cego de produtos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Cores utilizadas
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_warning = termx.fg_black, termx.bg_yellow,  # Preto/amarelo
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_list_os = termx.bold, termx.fg_yellow, termx.underline,  # Amarelo claro (sublinhado)
    color_os = termx.fg_yellow,  # Amarelo
    color_msg_ok = termx.fg_green,  # Verde claro
    color_msg_warning = termx.fg_yellow,  # Amarelo claro
    color_msg_err = termx.fg_red,  # Vermelho claro
    color_input = termx.bold, termx.underline, termx.fg_white  # Branco intenso/sublinhado
    color_msg_wait = termx.fg_green, termx.bold, termx.blink  # Verde claro (pisca)
    line_error = termx.max_row  # Linha onde sao mostradas as mensagens de erro
    line_info = line_error  # Linha onde sao mostradas as informacoes
    rec_sucess = True  # Recebimento efetuado com sucesso
    rec_continue = False  # O recebimento continua
    rec_falta_dado_logistico = 2

    def __init__(self):
        self.parametro = self.parametros_recebimento()  # Parametros do recebimento
        self.recebimento()

    @staticmethod
    def parametros_recebimento():
        """
        Leitura dos parametros utilizados pelo recebimento cego.

        :return: dicionario com os valores dos parametros

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = {}
        parametro_ids = xmlrpc.execute('estoque.parametro', 'search', [])
        campos = ['codigo_numerico', 'estpar_tamanho_codigo',
                  'estpar_totalmentecego']
        dados_parametro = xmlrpc.execute('estoque.parametro', 'read', parametro_ids, campos)

        if dados_parametro and isinstance(dados_parametro, list):
            result = dados_parametro[0]

        if dados_parametro and isinstance(dados_parametro, dict):
            result = dados_parametro

        return result

    def recebimento(self):
        """
        Mostra a lista das ordens de servico disponiveis para o recebimento.

        :return: os numeros das ordens de sevico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(2)
        termx.say(2, 1, '  - RECEBIMENTO -   ', *self.color_header)
        ordem_servico = False

        while True:
            ordem_servico = self.ordem_servico_recebimento()  # Selecao da ordem de servico
            if not ordem_servico:
                break
            ordem_servico = int(ordem_servico)
            self.recebimento_item(ordem_servico)  # Conferencia/recebimento do item

        return ordem_servico

    def recebimento_item(self, recebimento_id):
        """
        Recebimento/conferencia do item (produto).

        :param recebimento_id: ID da ordem de servico,
        :return: {boolean} verdadeiro se o item foi recebido.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from time import strptime

        # Formulario
        def form_recebimento():
            """
            Formulario de recebimento.

            :return: {boolean} verdadeiro.
            """

            termx.clear(3)
            termx.say(3, 1, 'Cod.  ______________')
            termx.say(4, 1, 'Grade __')
            termx.say(7, 1, 'Qtde. ________      ')
            termx.say(8, 1, 'Qtde Avaria ________')
            termx.say(9, 1, 'Validade   __/__/__ ')

            return True

        form_recebimento()

        __ESC = chr(27)

        while True:  # Codigo do produto
            qtde_embalagem = 1
            codigo = termx.get(3, 7, 14)

            if codigo == chr(27):
                break
            elif not codigo:
                continue
            if codigo.isdigit():
                codigo = '%014d' % int(codigo)

            # Recupera os dados do item/produto pelo codigo de barras
            args = [('codigo_barra', '=', codigo)]
            codigo_barra_ids = xmlrpc.execute('automacao.codigo.barra', 'search', args)

            if codigo_barra_ids:
                campos = ['produto_id', 'grade_id', 'produto_embalagem_id']
                model = 'automacao.codigo.barra'
                dados_cb = xmlrpc.execute(model, 'read', codigo_barra_ids, campos)

                if dados_cb and isinstance(dados_cb, list):
                    dados_cb = dados_cb[0]

                if dados_cb and isinstance(dados_cb, dict):
                    produto_id = dados_cb.get('produto_id')
                    grade_id = dados_cb.get('grade_id')

                    if isinstance(produto_id, (list, tuple)):
                        produto_id = produto_id[0]

                    if isinstance(grade_id, (list, tuple)):
                        grade_id = grade_id[0]

                    produto_embalagem_id = dados_cb.get('produto_embalagem_id')

                    if produto_embalagem_id and isinstance(produto_embalagem_id, (list, tuple)):
                        produto_embalagem_id = produto_embalagem_id[0]
                        campos = ['embalagem_id']
                        dados_embalagem = xmlrpc.execute(
                            'estoque.produto.embalagem', 'read', [produto_embalagem_id], campos)

                        if dados_embalagem and isinstance(dados_embalagem, list):
                            dados_embalagem = dados_embalagem[0]

                        if dados_embalagem and isinstance(dados_embalagem, dict):
                            embalagem_id = dados_embalagem.get('embalagem_id')

                            if embalagem_id and isinstance(embalagem_id, (list, tuple)):
                                embalagem_id = embalagem_id[0]

                                campos = ['quantidade_na_embalagem']
                                dados_embalagem = xmlrpc.execute('estoque.embalagem', 'read',
                                                                 [embalagem_id], campos)

                                if dados_embalagem and isinstance(dados_embalagem, list):
                                    dados_embalagem = dados_embalagem[0]

                                if dados_embalagem and isinstance(dados_embalagem, dict):
                                    qtde_embalagem = dados_embalagem.get('quantidade_na_embalagem', 1)
                                else:
                                    qtde_embalagem = 1

                            else:
                                qtde_embalagem = 1
                        else:
                            qtde_embalagem = 1
                    else:
                        qtde_embalagem = 1
                else:
                    produto_id = grade_id = False
                    qtde_embalagem = 1
            else:
                # Recupera os dados do item/produto pelo codigo interno
                tamanho_codigo = self.parametro.get('estpar_tamanho_codigo', 6)
                codigo_numerico = self.parametro.get('codigo_numerico')
                grade_id = False

                if codigo_numerico:
                    codigo = codigo[tamanho_codigo * -1:]
                else:
                    codigo = codigo[:tamanho_codigo]

                args = [('codigo', '=', codigo)]
                produto_ids = xmlrpc.execute('estoque.produto', 'search', args)

                if produto_ids:
                    produto_id = produto_ids[0]  # Um unico item
                else:
                    produto_id = False

            if not produto_id:
                self.mensagem_erro('ITEM NAO CADASTRADO.')
                continue

            # Busca a descricao do item
            campos = ['name', 'grade_usar']
            dados_item = xmlrpc.execute('estoque.produto', 'read', produto_id, campos)

            if dados_item and isinstance(dados_item, list):
                dados_item = dados_item[0]

            if dados_item and isinstance(dados_item, dict):
                descricao = dados_item.get('name') or 'ITEM SEM DESCRICAO.'
                descricao = '{:<20}'.format(descricao)

                # Faz uso de varias grades ou nao (grade unica)
                grade_usar = dados_item.get('grade_usar')
            else:
                descricao = '{:<20}'.format('ITEM SEM DESCRICAO.')
                grade_usar = False

            termx.say(7, 16, 'X')
            termx.say(7, 18, '{0:<3}'.format(int(qtde_embalagem)), *self.color_input)

            # Grades do produto
            args = [('produto_id', '=', produto_id)]
            grade_ids = xmlrpc.execute('estoque.produto.grade', 'search', args)

            if not grade_id and grade_usar and len(grade_ids) > 1:
                grade = None
                while True:  # Grade do produto
                    grade = termx.get(4, 7, 2)

                    if not grade:
                        continue

                    if grade == __ESC:  # Cancela a digitacao da grade
                        break

                    # Confima a grade
                    args = [('produto_id', '=', produto_id),
                            ('grade', '=', grade.upper())]

                    grade_ids = xmlrpc.execute('estoque.produto.grade', 'search', args)

                    if not grade_ids:
                        termx.say(self.line_error, 1,
                                  'GRADE INVALIDA.     ', *self.color_error)
                        continue

                    grade_id = grade_ids[0]
                    break

                if grade == __ESC:  # Se cancelou a digitacao da grade, retorna ao codigo do produto
                    termx.say(3, 1, 'Cod.  ______________')
                    termx.say(4, 1, 'Grade __            ')
                    continue
            else:
                # Recupera os dados da grade
                campos = ['grade', 'descricao']
                dados_grade = xmlrpc.execute('estoque.produto.grade', 'read', grade_id, campos)

                if dados_grade and isinstance(dados_grade, list):
                    dados_grade = dados_grade[0]

                if dados_grade and isinstance(dados_grade, dict):
                    termx.say(4, 7, dados_grade.get('grade'))

                    # Descricao do produto conforme parametro do estoque
                    campos = ['estpar_descricao_grade']
                    adm_utils_dados = xmlrpc.execute(
                        'adm.utils', 'admutl_get_parametros', 'estoque', campos)

                    if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                        descricao = dados_grade.get('descricao') or 'ITEM SEM DESCRICAO.'
                        descricao = '{:<20}'.format(descricao)

            # Mostra a descricao do item/produto
            termx.say(5, 1, descricao.upper(), *self.color_field)

            # Confirma o item na ordem de servico
            args = [('recebimento_id', '=', recebimento_id),
                    ('produto_id', '=', produto_id),
                    ('grade_id', '=', grade_id)]
            item_ids = xmlrpc.execute('recebimento.itens', 'search', args)

            if not item_ids:  # Item nao consta na ordem de servico
                if not self.parametro.get('estpar_totalmentecego'):
                    self.mensagem_erro('NAO CONSTA DA O.S.  ')
                    continue
            else:
                item_id = item_ids[0]

            # Mostra dados logisticos
            lastro = camada = peso = 0
            campos = ['lastro', 'camada', 'peso_palete',
                      'endereco_produto_id', 'falta_dado_logistico']
            dados_produto = xmlrpc.execute('estoque.produto.grade', 'read', grade_id, campos)

            if dados_produto and isinstance(dados_produto, list):
                dados_produto = dados_produto[0]

            if dados_produto and isinstance(dados_produto, dict):
                lastro = dados_produto.get('lastro', 0)
                camada = dados_produto.get('camada', 0)
                peso = dados_produto.get('peso_palete', 0.0)

            produto = '%d/%d %9.f Kg' % (lastro, camada, peso)
            termx.say(6, 1, produto, *self.color_field)  # Dados logisticos

            # Quantidades e validade
            while True:  # Quantidade contada
                qtde = termx.get(7, 7, 8)

                if qtde == __ESC:
                    termx.say(4, 1, 'Grade __')
                    termx.erase_line(5, 6)  # Apaga descricao do produto
                    termx.say(7, 1, 'Qtde. ________      ')
                    break
                elif qtde == '*':
                    self.efetua_recontagem(item_id)  # Refaz a contagem
                    termx.say(4, 1, 'Grade __')
                    termx.erase_line(5, 6)  # Apaga descricao do produto
                    termx.say(7, 1, 'Qtde. ________      ')
                    break
                elif not qtde:
                    termx.say(7, 7, '0       ', *self.color_input)
                    qtde = 0
                elif qtde.isdigit():
                    qtde = int(qtde)
                else:
                    continue

                # Recupera a quantidade na embalagem
                if not codigo_barra_ids:
                    while True:  # Quantidade na embalagem
                        qtde_embalagem = termx.get(7, 18, 3)

                        if qtde_embalagem == __ESC:
                            break

                        if not qtde_embalagem:
                            termx.say(7, 18, '1  ', *self.color_input)
                            qtde_embalagem = 1
                            break

                        if not qtde_embalagem.isdigit():
                            continue

                        qtde_embalagem = int(qtde_embalagem)

                        if qtde_embalagem < 1:
                            continue

                        break

                    if qtde_embalagem == __ESC:
                        continue

                qtde *= qtde_embalagem

                # Avarias
                qtde_avaria = 0
                termx.say(self.line_info, 1, ' '*20)

                while True:  # Quantidade avariada
                    qtde_avaria = termx.get(8, 13, 8)

                    if qtde_avaria == __ESC:
                        break

                    if not qtde_avaria:
                        termx.say(8, 13, '0       ', *self.color_input)
                        qtde_avaria = 0
                    elif qtde_avaria.isdigit():
                        qtde_avaria = int(qtde_avaria)
                    else:
                        continue

                    qtde_avaria *= qtde_embalagem

                    # Verifica se a quantidade informada (qtde + avaria) excede a quantidade na
                    # nota fiscal
                    campos = ['quantidade', 'qtde_contada', 'qtde_avaria', 'data_validade']
                    dados_item = xmlrpc.execute('recebimento.itens', 'read', item_ids, campos)

                    if dados_item and isinstance(dados_item, list):
                        dados_item = dados_item[0]
                        quantidade = dados_item.get('quantidade', 0)
                        contada = dados_item.get('qtde_contada', 0)
                        avaria = dados_item.get('qtde_avaria', 0)
                        total = contada + qtde + avaria + qtde_avaria

                        if total > quantidade and (qtde or qtde_avaria):
                            termx.say(9, 1, 'Validade   __/__/__ ')
                            self.mensagem_erro('EXCEDE QTDE. DA NOTA')
                            qtde_avaria = __ESC
                            break

                    # Validade do item/produto
                    validade = None
                    while True:  # Data de validade
                        while True:  # Dia
                            val_dd = termx.get(9, 12, 2, go=True)

                            if val_dd == __ESC:
                                break

                            if not val_dd:
                                continue
                            if not val_dd.decode('utf-8').isnumeric():
                                continue

                            val_dd = int(val_dd)
                            break

                        if val_dd == __ESC:
                            validade = '00/00/00'
                            break

                        while True:  # Mes
                            val_mm = termx.get(9, 15, 2, go=True)

                            if val_mm == __ESC:
                                break

                            if not val_mm:
                                continue

                            if not val_mm.decode('utf-8').isnumeric():
                                continue

                            val_mm = int(val_mm)
                            break

                        if val_mm == __ESC:
                            validade = '00/00/00'
                            break

                        while True:  # Ano
                            val_aa = termx.get(9, 18, 2, go=True)

                            if val_aa == __ESC:
                                break

                            if not val_aa:
                                continue

                            if not val_aa.decode('utf-8').isnumeric():
                                continue

                            val_aa = int(val_aa)
                            break

                        if val_aa == __ESC:
                            validade = '00/00/00'
                            break

                        validade = '{0:02d}/{1:02d}/{2:02d}'
                        validade = validade.format(val_dd, val_mm, val_aa)

                        if validade == '00/00/00':
                            break

                        if val_dd < 1 or val_dd > 31:
                            continue

                        if val_mm < 1 or val_mm > 12:
                            continue

                        if val_aa < 1:
                            continue

                        dateformat = '{0:04d}-{1:02d}-{2:02d}'
                        try:
                            dt = strptime(validade, '%d/%m/%y')
                        except Exception:
                            self.mensagem_erro('DATA INVALIDA.')
                            continue

                        validade = dateformat.format(dt.tm_year, dt.tm_mon, dt.tm_mday)

                        # Verifica shelf life
                        result = xmlrpc.execute(
                            'recebimento.itens', 'recite_shelf_life', item_ids, validade)

                        if result == 1:  # Shelf-life incorreto
                            self.mensagem_erro('SHELF LIFE INCORRETO')
                            continue
                        elif result == 2:  # Produto sem validade em dias
                            self.mensagem_erro('SEM VALIDADE EM DIAS')
                            continue
                        elif result == 3:  # Data de validade invalida
                            self.mensagem_erro('DATA INVALIDA.      ')
                            continue

                        # Totaliza a quantidade contada
                        total_contada = qtde + contada
                        total_avaria = qtde_avaria + avaria
                        self.efetua_recebimento(dados_item, total_contada, total_avaria, validade)

                        qtde_avaria = __ESC
                        termx.say(9, 1, 'Validade   __/__/__ ')
                        break

                    # Verifica se todos os itens ja foram conferidos
                    if self.recebimento_ok(recebimento_id):
                        self.mensagem_recebimento_ok()

                        # Aguarda uma tecla ser pressionada
                        termx.get(self.line_info, termx.max_col, 0)

                        # Fecha a contagem e segue para enderecamento
                        xmlrpc.execute('recebimento.os', 'recosv_fechar_contagem', recebimento_id)
                        return self.rec_sucess

                    if validade == '00/00/00':
                        termx.say(9, 1, 'Validade   __/__/__ ')
                    elif qtde_avaria == __ESC:
                        qtde = __ESC
                        break

                if qtde_avaria == __ESC:
                    termx.say(8, 1, 'Qtde Avaria ________')
                    if qtde == __ESC:
                        termx.say(4, 1, 'Grade __')
                        termx.erase_line(5, 6)  # Apaga descricao do produto
                        termx.say(7, 1, 'Qtde. ________      ')
                        break

                    continue
                else:
                    break


        return self.rec_continue

    @staticmethod
    def efetua_recontagem(item_id, context=None):
        """
        Efetua a recontagem do item.

        :param item_id: ID do item,
        :return: {boolean} verdadeiro se a contagem para o item foi zerada.
        """

        context = context and context.copy() or {}

        from datetime import datetime
        from pytz import timezone

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz)).strftime('%Y-%m-%d %H:%M')

        dados = {'qtde_contada': False, 'qtde_avaria': False,  'data_validade': False,
                 'usuario_id': xmlrpc.login_id, 'data_digitacao': agora}

        return xmlrpc.execute('recebimento.itens', 'write', item_id, dados)

    @staticmethod
    def efetua_recebimento(dados_item, qtde, qtde_avaria, validade, context=None):
        """

        :param dados_item: lista de dicionário com os campos e os dados dos itens de recebimento,
        :param qtde: quantidade contada,
        :param qtde_avaria: quantidade de itens avariados,
        :param validade: {date} data de validade do item/produto
        :return: {boolean} verdadeiro se todos os itens ja foram contados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        from datetime import datetime
        from pytz import timezone

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz)).strftime('%Y-%m-%d %H:%M')

        dados = {'qtde_contada': qtde, 'qtde_avaria': qtde_avaria, 'data_validade': validade,
                 'usuario_id': xmlrpc.login_id, 'data_digitacao': agora}
        item_id = dados_item.get('id')

        result = xmlrpc.execute('recebimento.itens', 'write', item_id, dados)

        return result

    @staticmethod
    def recebimento_ok(recebimento_id):
        """
        Verifica se todos os itens da ordem de servico estao 'Ok'

        :param recebimento_id: ID do recebimento,
        :return: {boolean} verdadeiro se todos os itens da ordem de servico ja foram conferidos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = False
        args = [('recebimento_id', '=', recebimento_id)]
        ids = xmlrpc.execute('recebimento.itens', 'search', args)

        # Verifica se todos os itens da ordem de servico estao 'Ok
        if not xmlrpc.execute('recebimento.itens', 'recite_verifica_divergencias', ids):
            result = True

        return result

    def ordem_servico_recebimento(self):
        """
        Lista das ordens de servico de recebimento destinadas ao usuario.

        :return: numero (ID) da ordem de servico que foi selecionada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.clear(3)
        termx.say(3, 1, 'OS Nr.', *self.color_list_os)
        termx.say(3, 8, '  Data  ', *self.color_list_os)
        termx.say(3, 17, 'Box', *self.color_list_os)

        # Selecao de uma ordem de servico
        os_numero = 0
        while True:
            lin_det, lista_os = self.lista_os()  # Mostra a lista de ordens de servico
            lin = 4

            for i in range(0, len(lin_det)):
                termx.say(lin, 1, lin_det[i], *self.color_os)
                lin += 1

                if (lin - 2) > 4:
                    break

            termx.say(8, 1, 'OS Nr. ______')
            os_numero = termx.get(8, 8, 6)  # Aguarda a selecao

            if os_numero == chr(27):
                return False

            if not os_numero:
                continue

            if not os_numero.isdigit():
                continue
            elif not int(os_numero) in lista_os:
                continue

            # Se a ordem de servico estiver criada, abre.
            xmlrpc.execute('recebimento.os', 'osvosv_abrir_os', int(os_numero))

            break

        return os_numero

    def lista_os(self):
        """
        Lista das ordens de servico.

        :return: lista com os dados da ordem de servico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        lin_det = []  # Lista com os dados das ordens de servico
        lista_os = []  # Lista com o ID das ordens de servico
        dados_os = []  # Dados da ordem de servico

        termx.say(self.line_info, 1, 'Aguarde...', *self.color_msg_wait)

        # Ordens de servico
        args = [('state', 'in', ['criada', 'aberta'])]
        ids = sorted(xmlrpc.execute('recebimento.os', 'search', args))

        if ids:
            campos = ['data_criacao', 'data_abertura', 'responsavel_ids', 'box', 'conferente_id']
            dados_os = xmlrpc.execute('recebimento.os', 'read', ids, campos)

        if dados_os and isinstance(dados_os, list):
            for os in dados_os:
                # Somente ordens de servico cujo responsavel e o usuario que efetuou o ´login´
                if xmlrpc.login_id not in os.get('responsavel_ids'):
                    continue

                # Verifica se ja foi atribuido um conferente para a ordem de servico
                if not os.get('conferente_id'):
                    continue

                # Verifica se os itens da ordem de servico ja foram todos conferidos
                if self.recebimento_ok(os.get('id')):
                    continue

                lista_os.append(os.get('id'))  # Acrescenta o nr. da ordem de servico a lista
                linefrmt = '{0:6d} {1:>2s}.{2:>2s}.{3:>2s}  {4:>2s}'

                # Data de criacao ou de abertura da ordem de servico
                os_data = os.get('data_abertura')

                if not os_data:
                    os_data = os.get('data_criacao')

                ano = os_data[2:4]
                mes = os_data[5:7]
                dia = os_data[8:10]
                os_nr = os.get('id')
                st = os.get('box')
                linefrmt = linefrmt.format(os_nr, dia, mes, ano, st)
                lin_det.append(linefrmt)

        termx.clear(self.line_info)

        return lin_det, lista_os

    def mensagem_erro(self, msg="ERRO.", wait=False):
        """
        Mostra mensagem de erro.

        :param msg: mensagem de erro,
        :param wait: {boolean} verdadeiro para aguarda o pressionamento de uma tecla para continuar
        :return: ultmima posicao impressa.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        result = termx.say(self.line_error, 1, msg, *self.color_error)

        if wait:
            termx.get(self.line_error, 20, 0)

        return result

    def mensagem_aviso(self, msg="AVISO."):
        """
        Mostra mensagem de aviso.

        :param msg: mensagem de aviso,
        :return: a mensagem de aviso.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        return termx.say(self.line_error, 1, msg, *self.color_warning)

    def mensagem_recebimento_ok(self):
        """
        Mostra a mensagem de que o recebimento foi efetuado com sucesso.

        :return: a mensagem de que o recebimento foi efetuado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.beep()
        termx.clear(3)
        termx.say(3, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.say(4, 1, 'RECEBIMENTO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(5, 1, 'EFETUADO'.center(termx.max_col), *self.color_msg_ok)
        termx.say(6, 1, 'COM'.center(termx.max_col), *self.color_msg_ok)
        termx.say(7, 1, 'SUCESSO.'.center(termx.max_col), *self.color_msg_ok)
        termx.say(8, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.beep()
        info = 'Pressione uma tecla.'

        return termx.say(9, 1, info, *self.color_info)

ColetorRecebimentoCego()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
