#!/usr/bin/python
# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuição-NãoComercial-Compartilhada Igual 3.0 Não Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from coletor import termx, xmlrpc


class ColetorTransferencia(object):
    """
    Modulo de transferencias .

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    # Cores utilizadas
    color_header = termx.fg_cyan,  # Azul claro
    color_error = termx.fg_white, termx.bg_red,  # Branco/vermelho
    color_field = termx.underline, termx.fg_green,  # Verde (sublinhado)
    color_info = termx.reverse, termx.fg_cyan,  # Preto/azul claro
    color_msg_ok = termx.fg_green,  # Verde claro
    line_error = termx.max_row  # Linha onde são mostradas as mensagens de erro
    line_info = termx.max_row - 1  # Linha onde são mostradas as informações
    rec_continue = False  # A transferencia continua

    def __init__(self):
        # Cores utilizadas
        self.transferencia()

    def transferencia(self):
        """
        Efetua transferência.

        @return: {boolean} verdadeiro se a transferencia foi efetuada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Formulário
        def form_transferencia():
            """
            Formulário de transferência.

            @return: None

            @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
            """
            termx.clear(2)
            termx.say(2, 1, ' - TRANSFERENCIA -  ', *self.color_header)
            termx.say(3, 1, 'Endereco de origem: ')
            termx.say(4, 1, '___________________ ')
            termx.say(5, 1, 'Endereco de destino:')
            termx.say(6, 1, '___________________ ')
            termx.say(7, 1, 'Quantidade ________ ')

            return None

        form_transferencia()
        __ESC = chr(27)

        while True:
            # Endereco de origem
            origem = termx.get(4, 1, 19)

            if not origem:
                continue
            elif origem == __ESC:
                break

            if not origem.isdigit():
                args = [('name', '=', origem)]
                origem_ids = xmlrpc.execute('wms.endereco', 'search', args)

                if not origem_ids:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue
            else:
                origem_ids = int(origem)

            campos = ['name', 'quantidade', 'produto_id', 'grade_id', 'bloqueado', 'validade']
            dados_origem = xmlrpc.execute('wms.endereco', 'read', origem_ids, campos)

            if dados_origem and isinstance(dados_origem, list):
                dados_origem = dados_origem[0]

            if dados_origem and isinstance(dados_origem, dict):
                origem = dados_origem.get('name')
                qtde_origem = dados_origem.get('quantidade')
                grade_origem = dados_origem.get('grade_id')
                produto_origem = dados_origem.get('produto_id')
                bloqueado = dados_origem.get('bloqueado')
                validade_origem = dados_origem.get('validade')

                if qtde_origem < 1 or bloqueado:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue
            else:
                self.mensagem_erro('ENDERECO INVALIDO.')
                continue

            termx.say(4, 1, origem, *self.color_field)

            # Endereço de destino
            destino = None

            while True:
                destino = termx.get(6, 1, 19)

                if not destino:
                    continue
                elif destino == __ESC:
                    break

                if not destino.isdigit():
                    args = [('name', '=', destino)]
                    destino_ids = xmlrpc.execute('wms.endereco', 'search', args)

                    if not destino_ids:
                        self.mensagem_erro('ENDERECO INVALIDO.')
                        continue
                else:
                    destino_ids = int(destino)

                campos = ['name', 'grade_id', 'tipo_nivel', 'picking_dinamico', 'bloqueado',
                          'validade', 'state']
                dados_destino = xmlrpc.execute('wms.endereco', 'read', destino_ids, campos)

                if dados_destino and isinstance(dados_destino, list):
                    dados_destino = dados_destino[0]

                if dados_destino and isinstance(dados_destino, dict):
                    destino = dados_destino.get('name')
                    grade_destino = dados_destino.get('grade_id')
                    tipo_nivel = dados_destino.get('tipo_nivel')
                    dinamico = dados_destino.get('picking_dinamico')
                    bloqueado = dados_destino.get('bloqueado')
                    validade_destino = dados_destino.get('validade')
                else:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue

                # Mesmo endereco?
                if origem_ids == destino_ids:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue

                # Verifica se os enderecos sao compativeis
                if grade_destino and grade_origem != grade_destino:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue

                if tipo_nivel == 'PK' and dinamico:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue

                if bloqueado:
                    self.mensagem_erro('ENDERECO INVALIDO.')
                    continue

                capacidade_picking = 0

                if tipo_nivel == 'PK':
                    campos = ['capacidade_picking', 'embalagem_master_id', 'tipo_nivel']
                    dados_grade = xmlrpc.execute(
                        'estoque.produto.grade', 'read', grade_destino[0], campos)

                    if dados_grade and isinstance(dados_grade, list):
                        dados_grade = dados_grade[0]

                    if dados_grade and isinstance(dados_grade, dict):
                        capacidade = dados_grade.get('capacidade_picking')
                        embalagem_id = dados_grade.get('embalagem_master_id')[0]
                        campos = ['embalagem_id']
                        dados_embalagem = xmlrpc.execute(
                            'estoque.produto.embalagem', 'read', embalagem_id, campos)

                        if dados_embalagem and isinstance(dados_embalagem, list):
                            dados_embalagem = dados_embalagem[0]

                        if dados_embalagem and isinstance(dados_embalagem, dict):
                            embalagem_id = dados_embalagem.get('embalagem_id')[0]
                            campos = ['quantidade_na_embalagem']
                            dados_embalagem = xmlrpc.execute(
                                'estoque.embalagem', 'read', embalagem_id, campos)

                            if dados_embalagem and isinstance(dados_embalagem, list):
                                dados_embalagem = dados_embalagem[0]

                            if dados_embalagem and isinstance(dados_embalagem, dict):
                                qtde_na_embalagem = dados_embalagem.get('quantidade_na_embalagem')
                                capacidade_picking = capacidade * qtde_na_embalagem

                termx.say(6, 1, destino, *self.color_field)

                # Quantidade a ser transferida
                quantidade = 0

                while True:
                    quantidade = termx.get(7, 12, 8)

                    if quantidade == __ESC:
                        break
                    elif not quantidade.isdigit():
                        continue

                    quantidade = int(quantidade)

                    if not quantidade:
                        continue

                    # Verifica a quantidade
                    if quantidade > qtde_origem:
                        self.mensagem_erro('EXCECE SALDO ORIGEM.')
                        continue
                    elif tipo_nivel == 'PK':
                        if quantidade > capacidade_picking:
                            self.mensagem_erro('EXCEDE CAP. PICKING!')
                            continue

                    info = 'OK? [ ] 0=NAO 1=SIM '
                    termx.say(self.line_info, 1, info, *self.color_info)
                    confirma = None

                    while confirma not in ('0', '1'):
                        confirma = termx.get(9, 6, 1)

                    if confirma == '0':  # Nao confirmou as informacoes
                        termx.say(7, 12, '________')
                        continue

                    # Validade
                    if not validade_destino:
                        validade = validade_origem
                    elif validade_destino > validade_origem:
                        validade = validade_origem
                    else:
                        validade = validade_destino

                    dados_transferencia = {
                        'endereco_origem_id': origem_ids,
                        'endereco_destino_id': destino_ids,
                        'produto_id': produto_origem[0],
                        'grade_id': grade_origem[0],
                        'quantidade': quantidade,
                        'validade': validade,
                        'solicitante_id': xmlrpc.login_id,
                        'responsavel_id': xmlrpc.login_id}

                    self.realiza_transferencia(dados_transferencia)
                    return self.mensagem_transferencia_ok()

                if quantidade == __ESC:
                    termx.say(7, 12, '________')
                    destino = None
                else:
                    break

            if destino == __ESC:
                termx.say(6, 1, '___________________ ')

            form_transferencia()  # Limpa o formulario

        return self.rec_continue

    @staticmethod
    def realiza_transferencia(dados_transferencia):
        """
        Realiza a transferecia. Cria, abre e fecha a ordem de servico.

        @param dados_transferencia: {dic} campos com os respectivos valores utilizados na criacao
        da ordem de servico de transferencia,
        @return: {int} ID da ordem de servico.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from time import strftime

        agora = strftime('%Y-%m-%d %H:%M:%S')
        result = []  # Lista com o retorno dos metodos criar, abrir e fechar

        # Nova ordem de servico
        dados_os = {'descricao': 'Tranferência realizada via coletor.'}
        context = {'transferencia': True, 'coletor': True}
        os_id = xmlrpc.execute('transferencia.os', 'create', dados_os)

        # Cria o item de transferencia
        dados_transferencia['transferencia_id'] = os_id
        dados_transferencia['data_transferencia'] = agora
        transferencia_id = xmlrpc.execute('wms.transferencia.itens', 'create', dados_transferencia)

        if transferencia_id:
            dados = {'responsavel_ids': [(4, xmlrpc.login_id)]}
            xmlrpc.execute('transferencia.os', 'write', os_id, dados)

        # Cria a ordem de servico de transferencia
        result.append(xmlrpc.execute('transferencia.os', 'osvosv_criar_os', os_id, context))

        # Abre a ordem de servico
        result.append(xmlrpc.execute('transferencia.os', 'osvosv_abrir_os', os_id, context))

        # Altera o ´´status´´ do item de transferencia
        dados = {'state': '3'}
        result.append(xmlrpc.execute('wms.transferencia.itens', 'write', transferencia_id, dados))

        # Fecha a ordem de servico
        result.append(xmlrpc.execute('transferencia.os', 'osvosv_fechar_os', os_id, context))

        if not all(result):
            xmlrpc.execute('transferencia.os', 'unlink', os_id)
            return False

        return os_id

    def mensagem_erro(self, msg="ERRO."):
        """
        Mostra mensagem de erro.

        @param msg: {char} mensagem de erro,
        @return: {int} a posicao do proximo caracter

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Ajusta a mensagem ao tamanho da linha
        msg = msg[0:termx.max_col + 1]
        msg = msg.ljust(termx.max_col + len(msg))
        termx.beep()

        return termx.say(self.line_error, 1, msg, *self.color_error)

    def mensagem_transferencia_ok(self):
        """
        Mostra a mensagem de que a transferencia foi efetuada com sucesso.
        @return: a mensagem de que a transferencia foi efetuada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        termx.beep()
        termx.clear(3)
        termx.say(3, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.say(4, 1, 'TRANSFERENCIA'.center(termx.max_col), *self.color_msg_ok)
        termx.say(5, 1, 'EFETUADA'.center(termx.max_col), *self.color_msg_ok)
        termx.say(6, 1, 'COM'.center(termx.max_col), *self.color_msg_ok)
        termx.say(7, 1, 'SUCESSO.'.center(termx.max_col), *self.color_msg_ok)
        termx.say(8, 1, '*' * termx.max_col, *self.color_msg_ok)
        termx.beep()
        info = 'Pressione uma tecla.'

        termx.say(9, 1, info, *self.color_info)
        termx.get(7, 15, 0)
        return True

ColetorTransferencia()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
