SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;
SET search_path = public, pg_catalog;
SET default_tablespace = '';
SET default_with_oids = true;
CREATE TABLE rule_group_rel (
    rule_group_id integer NOT NULL,
    group_id integer NOT NULL
);
COMMENT ON TABLE rule_group_rel IS 'RELATION BETWEEN ir_rule AND res_groups';
CREATE INDEX rule_group_rel_group_id_index ON rule_group_rel USING btree (group_id);
CREATE INDEX rule_group_rel_rule_group_id_index ON rule_group_rel USING btree (rule_group_id);
ALTER TABLE ONLY rule_group_rel
    ADD CONSTRAINT rule_group_rel_group_id_fkey FOREIGN KEY (group_id) REFERENCES res_groups(id) ON DELETE CASCADE;
ALTER TABLE ONLY rule_group_rel
    ADD CONSTRAINT rule_group_rel_rule_group_id_fkey FOREIGN KEY (rule_group_id) REFERENCES ir_rule(id) ON DELETE CASCADE;
SET default_with_oids = false;
CREATE TABLE ir_module_web
(
  id serial NOT NULL,
  create_uid integer,
  create_date timestamp without time zone,
  write_date timestamp without time zone,
  write_uid integer,
  website character varying(256), -- Website
  state character varying(16), -- State
  "name" character varying(128) NOT NULL, -- Name
  author character varying(128), -- Author
  module character varying(128) NOT NULL, -- Module
  description text, -- Description
  CONSTRAINT ir_module_web_pkey PRIMARY KEY (id),
  CONSTRAINT ir_module_web_create_uid_fkey FOREIGN KEY (create_uid)
      REFERENCES res_users (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT ir_module_web_write_uid_fkey FOREIGN KEY (write_uid)
      REFERENCES res_users (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT ir_module_web_name_uniq UNIQUE (name)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE ir_module_web;
COMMENT ON TABLE ir_module_web IS 'Web Module';
COMMENT ON COLUMN ir_module_web.website IS 'Website';
COMMENT ON COLUMN ir_module_web.state IS 'State';
COMMENT ON COLUMN ir_module_web."name" IS 'Name';
COMMENT ON COLUMN ir_module_web.author IS 'Author';
COMMENT ON COLUMN ir_module_web.module IS 'Module';
COMMENT ON COLUMN ir_module_web.description IS 'Description';


CREATE TABLE ir_module_web_dependency (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    module_id integer,
    web_module_id integer,
    name character varying(128)
);
COMMENT ON TABLE ir_module_web_dependency IS 'Web Module dependency';
COMMENT ON COLUMN ir_module_web_dependency.module_id IS 'Module';
COMMENT ON COLUMN ir_module_web_dependency.web_module_id IS 'Web Module';
COMMENT ON COLUMN ir_module_web_dependency.name IS 'Name';
CREATE SEQUENCE ir_module_web_dependency_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER SEQUENCE ir_module_web_dependency_id_seq OWNED BY ir_module_web_dependency.id;
ALTER TABLE ir_module_web_dependency ALTER COLUMN id SET DEFAULT nextval('ir_module_web_dependency_id_seq'::regclass);
ALTER TABLE ONLY ir_module_web_dependency
    ADD CONSTRAINT ir_module_web_dependency_pkey PRIMARY KEY (id);
CREATE INDEX ir_module_web_dependency_module_id_index ON ir_module_web_dependency USING btree (module_id);
CREATE INDEX ir_module_web_dependency_web_module_id_index ON ir_module_web_dependency USING btree (web_module_id);
ALTER TABLE ONLY ir_module_web_dependency
    ADD CONSTRAINT ir_module_web_dependency_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_module_web_dependency
    ADD CONSTRAINT ir_module_web_dependency_module_id_fkey FOREIGN KEY (module_id) REFERENCES ir_module_module(id) ON DELETE CASCADE;
ALTER TABLE ONLY ir_module_web_dependency
    ADD CONSTRAINT ir_module_web_dependency_web_module_id_fkey FOREIGN KEY (web_module_id) REFERENCES ir_module_web(id) ON DELETE CASCADE;
ALTER TABLE ONLY ir_module_web_dependency
    ADD CONSTRAINT ir_module_web_dependency_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
SET default_with_oids = true;
CREATE TABLE res_company_users_rel (
    cid integer NOT NULL,
    user_id integer NOT NULL
);
COMMENT ON TABLE res_company_users_rel IS 'RELATION BETWEEN res_company AND res_users';
CREATE INDEX res_company_users_rel_cid_index ON res_company_users_rel USING btree (cid);
CREATE INDEX res_company_users_rel_user_id_index ON res_company_users_rel USING btree (user_id);
ALTER TABLE ONLY res_company_users_rel
    ADD CONSTRAINT res_company_users_rel_cid_fkey FOREIGN KEY (cid) REFERENCES res_company(id) ON DELETE CASCADE;
ALTER TABLE ONLY res_company_users_rel
    ADD CONSTRAINT res_company_users_rel_user_id_fkey FOREIGN KEY (user_id) REFERENCES res_users(id) ON DELETE CASCADE;
SET default_with_oids = false;
CREATE TABLE ir_module_web (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    website character varying(256),
    state character varying(16),
    name character varying(128) NOT NULL,
    author character varying(128),
    module character varying(128) NOT NULL,
    description text
);
COMMENT ON TABLE ir_module_web IS 'Web Module';
COMMENT ON COLUMN ir_module_web.website IS 'Website';
COMMENT ON COLUMN ir_module_web.state IS 'State';
COMMENT ON COLUMN ir_module_web.name IS 'Name';
COMMENT ON COLUMN ir_module_web.author IS 'Author';
COMMENT ON COLUMN ir_module_web.module IS 'Module';
COMMENT ON COLUMN ir_module_web.description IS 'Description';
CREATE SEQUENCE ir_module_web_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER SEQUENCE ir_module_web_id_seq OWNED BY ir_module_web.id;
ALTER TABLE ir_module_web ALTER COLUMN id SET DEFAULT nextval('ir_module_web_id_seq'::regclass);
ALTER TABLE ONLY ir_module_web
    ADD CONSTRAINT ir_module_web_name_uniq UNIQUE (name);
ALTER TABLE ONLY ir_module_web
    ADD CONSTRAINT ir_module_web_pkey PRIMARY KEY (id);
ALTER TABLE ONLY ir_module_web
    ADD CONSTRAINT ir_module_web_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_module_web
    ADD CONSTRAINT ir_module_web_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
SET default_with_oids = false;
CREATE TABLE ir_filters (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    model_id character varying(16) NOT NULL,
    domain text NOT NULL,
    user_id integer,
    name character varying(64) NOT NULL,
    context text NOT NULL
);
COMMENT ON TABLE ir_filters IS 'Filters';
COMMENT ON COLUMN ir_filters.model_id IS 'Model';
COMMENT ON COLUMN ir_filters.domain IS 'Domain Value';
COMMENT ON COLUMN ir_filters.user_id IS 'User';
COMMENT ON COLUMN ir_filters.name IS 'Action Name';
COMMENT ON COLUMN ir_filters.context IS 'Context Value';
CREATE SEQUENCE ir_filters_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER SEQUENCE ir_filters_id_seq OWNED BY ir_filters.id;
ALTER TABLE ir_filters ALTER COLUMN id SET DEFAULT nextval('ir_filters_id_seq'::regclass);
ALTER TABLE ONLY ir_filters
    ADD CONSTRAINT ir_filters_pkey PRIMARY KEY (id);
ALTER TABLE ONLY ir_filters
    ADD CONSTRAINT ir_filters_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_filters
    ADD CONSTRAINT ir_filters_user_id_fkey FOREIGN KEY (user_id) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_filters
    ADD CONSTRAINT ir_filters_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
SET default_with_oids = true;
CREATE TABLE res_groups_action_rel (
    uid integer NOT NULL,
    gid integer NOT NULL
);
COMMENT ON TABLE res_groups_action_rel IS 'RELATION BETWEEN ir_actions_todo AND res_groups';
CREATE INDEX res_groups_action_rel_gid_index ON res_groups_action_rel USING btree (gid);
CREATE INDEX res_groups_action_rel_uid_index ON res_groups_action_rel USING btree (uid);
ALTER TABLE ONLY res_groups_action_rel
    ADD CONSTRAINT res_groups_action_rel_gid_fkey FOREIGN KEY (gid) REFERENCES res_groups(id) ON DELETE CASCADE;
ALTER TABLE ONLY res_groups_action_rel
    ADD CONSTRAINT res_groups_action_rel_uid_fkey FOREIGN KEY (uid) REFERENCES ir_actions_todo(id) ON DELETE CASCADE;
SET default_with_oids = false;
CREATE TABLE multi_company_default (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    name character varying(256) NOT NULL,
    sequence integer,
    expression character varying(256) NOT NULL,
    company_dest_id integer NOT NULL,
    field_id integer,
    company_id integer NOT NULL,
    object_id integer NOT NULL
);
COMMENT ON TABLE multi_company_default IS 'Default multi company';
COMMENT ON COLUMN multi_company_default.name IS 'Name';
COMMENT ON COLUMN multi_company_default.sequence IS 'Sequence';
COMMENT ON COLUMN multi_company_default.expression IS 'Expression';
COMMENT ON COLUMN multi_company_default.company_dest_id IS 'Default Company';
COMMENT ON COLUMN multi_company_default.field_id IS 'Field';
COMMENT ON COLUMN multi_company_default.company_id IS 'Main Company';
COMMENT ON COLUMN multi_company_default.object_id IS 'Object';
CREATE SEQUENCE multi_company_default_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER SEQUENCE multi_company_default_id_seq OWNED BY multi_company_default.id;
ALTER TABLE multi_company_default ALTER COLUMN id SET DEFAULT nextval('multi_company_default_id_seq'::regclass);
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_pkey PRIMARY KEY (id);
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_company_dest_id_fkey FOREIGN KEY (company_dest_id) REFERENCES res_company(id) ON DELETE SET NULL;
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_company_id_fkey FOREIGN KEY (company_id) REFERENCES res_company(id) ON DELETE SET NULL;
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_field_id_fkey FOREIGN KEY (field_id) REFERENCES ir_model_fields(id) ON DELETE SET NULL;
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_object_id_fkey FOREIGN KEY (object_id) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY multi_company_default
    ADD CONSTRAINT multi_company_default_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
SET default_with_oids = false;
CREATE TABLE res_config_users (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    menu_id integer,
    address_id integer,
    config_logo bytea,
    active boolean,
    context_tz character varying(64),
    password character varying(64),
    context_lang character varying(16) NOT NULL,
    name character varying(64) NOT NULL,
    company_id integer NOT NULL,
    signature text,
    progress double precision,
    login character varying(64) NOT NULL,
    email character varying(64),
    action_id integer
);
COMMENT ON TABLE res_config_users IS 'res.config.users';
COMMENT ON COLUMN res_config_users.menu_id IS 'Menu Action';
COMMENT ON COLUMN res_config_users.address_id IS 'Address';
COMMENT ON COLUMN res_config_users.config_logo IS 'Image';
COMMENT ON COLUMN res_config_users.active IS 'Active';
COMMENT ON COLUMN res_config_users.context_tz IS 'Timezone';
COMMENT ON COLUMN res_config_users.password IS 'Password';
COMMENT ON COLUMN res_config_users.context_lang IS 'Language';
COMMENT ON COLUMN res_config_users.name IS 'Name';
COMMENT ON COLUMN res_config_users.company_id IS 'Company';
COMMENT ON COLUMN res_config_users.signature IS 'Signature';
COMMENT ON COLUMN res_config_users.progress IS 'Configuration Progress';
COMMENT ON COLUMN res_config_users.login IS 'Login';
COMMENT ON COLUMN res_config_users.email IS 'E-mail';
COMMENT ON COLUMN res_config_users.action_id IS 'Home Action';
CREATE SEQUENCE res_config_users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER SEQUENCE res_config_users_id_seq OWNED BY res_config_users.id;
ALTER TABLE res_config_users ALTER COLUMN id SET DEFAULT nextval('res_config_users_id_seq'::regclass);
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_login_key UNIQUE (login);
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_pkey PRIMARY KEY (id);
CREATE INDEX res_config_users_name_index ON res_config_users USING btree (name);
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_address_id_fkey FOREIGN KEY (address_id) REFERENCES res_partner_address(id) ON DELETE SET NULL;
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_company_id_fkey FOREIGN KEY (company_id) REFERENCES res_company(id) ON DELETE SET NULL;
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY res_config_users
    ADD CONSTRAINT res_config_users_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
SET default_with_oids = true;
CREATE TABLE ir_act_window_user_rel (
    act_id integer NOT NULL,
    uid integer NOT NULL
);
COMMENT ON TABLE ir_act_window_user_rel IS 'RELATION BETWEEN ir_act_window AND res_users';
CREATE INDEX act_window_action_uid_index ON ir_act_window_user_rel USING btree (act_id, uid);
CREATE INDEX ir_act_window_user_rel_act_id_index ON ir_act_window_user_rel USING btree (act_id);
CREATE INDEX ir_act_window_user_rel_uid_index ON ir_act_window_user_rel USING btree (uid);
ALTER TABLE ONLY ir_act_window_user_rel
    ADD CONSTRAINT ir_act_window_user_rel_act_id_fkey FOREIGN KEY (act_id) REFERENCES ir_act_window(id) ON DELETE CASCADE;
ALTER TABLE ONLY ir_act_window_user_rel
    ADD CONSTRAINT ir_act_window_user_rel_uid_fkey FOREIGN KEY (uid) REFERENCES res_users(id) ON DELETE CASCADE;
    
ALTER TABLE ir_model_fields ADD COLUMN selectable boolean;
ALTER TABLE ir_model_fields ALTER COLUMN selectable SET STORAGE PLAIN;
COMMENT ON COLUMN ir_model_fields.selectable IS 'Selectable';

ALTER TABLE ir_module_module ADD COLUMN web character boolean;
ALTER TABLE ir_module_module ADD COLUMN maintainer character varying(128);
ALTER TABLE ir_module_module ALTER COLUMN maintainer SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_module_module.maintainer IS 'Maintainer';
ALTER TABLE ir_module_module ADD COLUMN contributors text;
ALTER TABLE ir_module_module ALTER COLUMN contributors SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_module_module.contributors IS 'Contributors';

ALTER TABLE ir_actions_todo ADD COLUMN "restart" character varying(16);
ALTER TABLE ir_actions_todo ALTER COLUMN "restart" SET STORAGE EXTENDED;
ALTER TABLE ir_actions_todo ALTER COLUMN "restart" SET NOT NULL;
COMMENT ON COLUMN ir_actions_todo."restart" IS 'Restart';
ALTER TABLE res_company ADD COLUMN account_no character varying(64);
ALTER TABLE res_company ALTER COLUMN account_no SET STORAGE EXTENDED;
COMMENT ON COLUMN res_company.account_no IS 'Account No.';

ALTER TABLE res_lang ADD COLUMN iso_code character varying(5);
ALTER TABLE res_lang ALTER COLUMN iso_code SET STORAGE EXTENDED;
COMMENT ON COLUMN res_lang.iso_code IS 'ISO code';

ALTER TABLE ir_sequence ADD COLUMN company_id integer;
ALTER TABLE ir_sequence ALTER COLUMN company_id SET STORAGE PLAIN;
COMMENT ON COLUMN ir_sequence.company_id IS 'Company';
ALTER TABLE ir_sequence
  ADD CONSTRAINT ir_sequence_company_id_fkey FOREIGN KEY (company_id)
      REFERENCES res_company (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

aLTER TABLE ir_attachment rename link to url;
ALTER TABLE ir_attachment ADD COLUMN "type" character varying(16);
ALTER TABLE ir_attachment ALTER COLUMN "type" SET STORAGE EXTENDED;
ALTER TABLE ir_attachment ALTER COLUMN "type" SET NOT NULL;
COMMENT ON COLUMN ir_attachment."type" IS 'Type';

ALTER TABLE res_users ADD COLUMN email character varying(64);
ALTER TABLE res_users ALTER COLUMN email SET STORAGE EXTENDED;
ALTER TABLE res_users ALTER COLUMN email SET DEFAULT NULL::character varying;

ALTER TABLE ir_act_window ADD COLUMN search_view_id integer;
ALTER TABLE ir_act_window ALTER COLUMN search_view_id SET STORAGE PLAIN;
COMMENT ON COLUMN ir_act_window.search_view_id IS 'Search View Ref.';
ALTER TABLE ir_act_window ADD COLUMN auto_search boolean;
ALTER TABLE ir_act_window ALTER COLUMN auto_search SET STORAGE PLAIN;
COMMENT ON COLUMN ir_act_window.auto_search IS 'Auto Search';
ALTER TABLE ir_act_window ADD COLUMN help text;
ALTER TABLE ir_act_window ALTER COLUMN help SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_act_window.help IS 'Action description';
ALTER TABLE ir_act_window ADD COLUMN menus character varying(4096);
ALTER TABLE ir_act_window ALTER COLUMN menus SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_act_window.menus IS 'Menus';
ALTER TABLE ir_act_window ADD COLUMN filter boolean;
ALTER TABLE ir_act_window ALTER COLUMN filter SET STORAGE PLAIN;
COMMENT ON COLUMN ir_act_window.filter IS 'Filter';
ALTER TABLE ir_act_window
  ADD CONSTRAINT ir_act_window_search_view_id_fkey FOREIGN KEY (search_view_id)
      REFERENCES ir_ui_view (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
ALTER TABLE ir_act_window ADD COLUMN multi boolean;
ALTER TABLE ir_act_window ALTER COLUMN multi SET STORAGE PLAIN;
COMMENT ON COLUMN ir_act_window.multi IS 'Action on Multiple Doc.';
    
ALTER TABLE res_users ADD COLUMN date timestamp without time zone;
ALTER TABLE res_users ALTER COLUMN date SET STORAGE PLAIN;
COMMENT ON COLUMN res_users.date IS 'Last Connection';
ALTER TABLE res_users ADD COLUMN menu_tips boolean;
ALTER TABLE res_users ALTER COLUMN menu_tips SET STORAGE PLAIN;
COMMENT ON COLUMN res_users.menu_tips IS 'Menu Tips';

ALTER TABLE ir_property ADD COLUMN value_text text;
ALTER TABLE ir_property ALTER COLUMN value_text SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_property.value_text IS 'Value';
ALTER TABLE ir_property ADD COLUMN value_float double precision;
ALTER TABLE ir_property ALTER COLUMN value_float SET STORAGE PLAIN;
COMMENT ON COLUMN ir_property.value_float IS 'Value';
ALTER TABLE ir_property ADD COLUMN value_integer bigint;
ALTER TABLE ir_property ALTER COLUMN value_integer SET STORAGE PLAIN;
COMMENT ON COLUMN ir_property.value_integer IS 'Value';
ALTER TABLE ir_property ADD COLUMN value_datetime timestamp without time zone;
ALTER TABLE ir_property ALTER COLUMN value_datetime SET STORAGE PLAIN;
COMMENT ON COLUMN ir_property.value_datetime IS 'Value';
ALTER TABLE ir_property ADD COLUMN value_binary bytea;
ALTER TABLE ir_property ALTER COLUMN value_binary SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_property.value_binary IS 'Value';
ALTER TABLE ir_property ADD COLUMN value_reference character varying(128);
ALTER TABLE ir_property ALTER COLUMN value_reference SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_property.value_reference IS 'Value';
ALTER TABLE ir_property ADD COLUMN "type" character varying(16);
ALTER TABLE ir_property ALTER COLUMN "type" SET STORAGE EXTENDED;
ALTER TABLE ir_property ALTER COLUMN "type" SET NOT NULL;
COMMENT ON COLUMN ir_property."type" IS 'Type';
CREATE INDEX ir_property_type_index
  ON ir_property
  USING btree
  (type);

delete from ir_rule where model_id is null;
ALTER TABLE ir_rule ADD COLUMN model_id integer;
ALTER TABLE ir_rule ALTER COLUMN model_id SET STORAGE PLAIN;
ALTER TABLE ir_rule ALTER COLUMN model_id SET NOT NULL;
COMMENT ON COLUMN ir_rule.model_id IS 'Object';
ALTER TABLE ir_rule ADD COLUMN perm_read boolean;
ALTER TABLE ir_rule ALTER COLUMN perm_read SET STORAGE PLAIN;
COMMENT ON COLUMN ir_rule.perm_read IS 'Apply For Read';
ALTER TABLE ir_rule ADD COLUMN "name" character varying(128);
ALTER TABLE ir_rule ALTER COLUMN "name" SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_rule."name" IS 'Name';
ALTER TABLE ir_rule ADD COLUMN "global" boolean;
ALTER TABLE ir_rule ALTER COLUMN "global" SET STORAGE PLAIN;
COMMENT ON COLUMN ir_rule."global" IS 'Global';
ALTER TABLE ir_rule ADD COLUMN perm_unlink boolean;
ALTER TABLE ir_rule ALTER COLUMN perm_unlink SET STORAGE PLAIN;
COMMENT ON COLUMN ir_rule.perm_unlink IS 'Apply For Delete';
ALTER TABLE ir_rule ADD COLUMN perm_write boolean;
ALTER TABLE ir_rule ALTER COLUMN perm_write SET STORAGE PLAIN;
COMMENT ON COLUMN ir_rule.perm_write IS 'Apply For Write';
ALTER TABLE ir_rule ADD COLUMN perm_create boolean;
ALTER TABLE ir_rule ALTER COLUMN perm_create SET STORAGE PLAIN;
COMMENT ON COLUMN ir_rule.perm_create IS 'Apply For Create';
ALTER TABLE ir_rule
  ADD CONSTRAINT ir_rule_model_id_fkey FOREIGN KEY (model_id)
      REFERENCES ir_model (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
CREATE INDEX ir_rule_model_id_index
  ON ir_rule
  USING btree
  (model_id);
CREATE INDEX ir_rule_name_index
  ON ir_rule
  USING btree
  (name);

ALTER TABLE ir_act_server ADD COLUMN copy_object character varying(256);
ALTER TABLE ir_act_server ALTER COLUMN copy_object SET STORAGE EXTENDED;
COMMENT ON COLUMN ir_act_server.copy_object IS 'Copy Of';

ALTER TABLE res_currency ADD COLUMN date date;
ALTER TABLE res_currency ALTER COLUMN date SET STORAGE PLAIN;
COMMENT ON COLUMN res_currency.date IS 'Date';
ALTER TABLE res_currency ADD COLUMN base boolean;
ALTER TABLE res_currency ALTER COLUMN base SET STORAGE PLAIN;
COMMENT ON COLUMN res_currency.base IS 'Base';
ALTER TABLE res_currency ADD COLUMN company_id integer;
ALTER TABLE res_currency ALTER COLUMN company_id SET STORAGE PLAIN;
COMMENT ON COLUMN res_currency.company_id IS 'Company';
ALTER TABLE res_currency
  ADD CONSTRAINT res_currency_company_id_fkey FOREIGN KEY (company_id)
      REFERENCES res_company (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

ALTER TABLE res_partner ADD COLUMN employee boolean;
ALTER TABLE res_partner ALTER COLUMN employee SET STORAGE PLAIN;
COMMENT ON COLUMN res_partner.employee IS 'Employee';
ALTER TABLE res_partner ADD COLUMN company_id integer;
ALTER TABLE res_partner ALTER COLUMN company_id SET STORAGE PLAIN;
COMMENT ON COLUMN res_partner.company_id IS 'Company';
alter table res_partner drop column title;
ALTER TABLE res_partner ADD COLUMN title integer;
ALTER TABLE res_partner ALTER COLUMN title SET STORAGE PLAIN;
COMMENT ON COLUMN res_partner.title IS 'Title';
ALTER TABLE res_partner
  ADD CONSTRAINT res_partner_title_fkey FOREIGN KEY (title)
      REFERENCES res_partner_title (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
ALTER TABLE res_partner
  ADD CONSTRAINT res_partner_company_id_fkey FOREIGN KEY (company_id)
      REFERENCES res_company (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

alter table res_partner_address drop column function;
ALTER TABLE res_partner_address ADD COLUMN "function" character varying(64);
ALTER TABLE res_partner_address ALTER COLUMN "function" SET STORAGE EXTENDED;
COMMENT ON COLUMN res_partner_address."function" IS 'Function';
ALTER TABLE res_partner_address ADD COLUMN company_id integer;
ALTER TABLE res_partner_address ALTER COLUMN company_id SET STORAGE PLAIN;
COMMENT ON COLUMN res_partner_address.company_id IS 'Company';
alter table res_partner_address drop column title;
ALTER TABLE res_partner_address ADD COLUMN title integer;
ALTER TABLE res_partner_address ALTER COLUMN title SET STORAGE PLAIN;
COMMENT ON COLUMN res_partner_address.title IS 'Title';
ALTER TABLE res_partner_address
  ADD CONSTRAINT res_partner_address_company_id_fkey FOREIGN KEY (company_id)
      REFERENCES res_company (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
ALTER TABLE res_partner_address
  ADD CONSTRAINT res_partner_address_title_fkey FOREIGN KEY (title)
      REFERENCES res_partner_title (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;

update ir_module_module set latest_version= '6.0.1.2';
