﻿--- =================================================
--- Tables in 'teste_7_0' that are not in 'teste_6_1'
--- ================================================= 

SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

SET search_path = public, pg_catalog;

SET default_tablespace = '';

SET default_with_oids = true;


update ir_module_module set demo=False where demo=True;

CREATE TABLE res_partner_res_partner_category_rel (
    category_id integer NOT NULL,
    partner_id integer NOT NULL
);
COMMENT ON TABLE res_partner_res_partner_category_rel IS 'RELATION BETWEEN res_partner_category AND res_partner';
ALTER TABLE ONLY res_partner_res_partner_category_rel
    ADD CONSTRAINT res_partner_res_partner_category_rel_category_id_partner_id_key UNIQUE (category_id, partner_id);
CREATE INDEX res_partner_res_partner_category_rel_category_id_index ON res_partner_res_partner_category_rel USING btree (category_id);
CREATE INDEX res_partner_res_partner_category_rel_partner_id_index ON res_partner_res_partner_category_rel USING btree (partner_id);
ALTER TABLE ONLY res_partner_res_partner_category_rel
    ADD CONSTRAINT res_partner_res_partner_category_rel_category_id_fkey FOREIGN KEY (category_id) REFERENCES res_partner_category(id) ON DELETE CASCADE;
ALTER TABLE ONLY res_partner_res_partner_category_rel
    ADD CONSTRAINT res_partner_res_partner_category_rel_partner_id_fkey FOREIGN KEY (partner_id) REFERENCES res_partner(id) ON DELETE CASCADE;

CREATE TABLE process_process (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    active boolean,
    model_id integer,
    note text,
    name character varying(30) NOT NULL
);
COMMENT ON TABLE process_process IS 'Process';
COMMENT ON COLUMN process_process.active IS 'Active';
COMMENT ON COLUMN process_process.model_id IS 'Object';
COMMENT ON COLUMN process_process.note IS 'Notes';
COMMENT ON COLUMN process_process.name IS 'Name';
CREATE SEQUENCE process_process_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE process_process_id_seq OWNED BY process_process.id;
ALTER TABLE ONLY process_process ALTER COLUMN id SET DEFAULT nextval('process_process_id_seq'::regclass);
ALTER TABLE ONLY process_process
    ADD CONSTRAINT process_process_pkey PRIMARY KEY (id);
ALTER TABLE ONLY process_process
    ADD CONSTRAINT process_process_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_process
    ADD CONSTRAINT process_process_model_id_fkey FOREIGN KEY (model_id) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_process
    ADD CONSTRAINT process_process_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

CREATE TABLE process_node (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    menu_id integer,
    model_id integer,
    kind character varying NOT NULL,
    note text,
    name character varying(30) NOT NULL,
    subflow_id integer,
    process_id integer NOT NULL,
    model_states character varying(128),
    help_url character varying(255),
    flow_start boolean
);
COMMENT ON TABLE process_node IS 'Process Node';
COMMENT ON COLUMN process_node.menu_id IS 'Related Menu';
COMMENT ON COLUMN process_node.model_id IS 'Object';
COMMENT ON COLUMN process_node.kind IS 'Kind of Node';
COMMENT ON COLUMN process_node.note IS 'Notes';
COMMENT ON COLUMN process_node.name IS 'Name';
COMMENT ON COLUMN process_node.subflow_id IS 'Subflow';
COMMENT ON COLUMN process_node.process_id IS 'Process';
COMMENT ON COLUMN process_node.model_states IS 'States Expression';
COMMENT ON COLUMN process_node.help_url IS 'Help URL';
COMMENT ON COLUMN process_node.flow_start IS 'Starting Flow';
CREATE SEQUENCE process_node_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE process_node_id_seq OWNED BY process_node.id;
ALTER TABLE ONLY process_node ALTER COLUMN id SET DEFAULT nextval('process_node_id_seq'::regclass);
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_pkey PRIMARY KEY (id);
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_menu_id_fkey FOREIGN KEY (menu_id) REFERENCES ir_ui_menu(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_model_id_fkey FOREIGN KEY (model_id) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_process_id_fkey FOREIGN KEY (process_id) REFERENCES process_process(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_subflow_id_fkey FOREIGN KEY (subflow_id) REFERENCES process_process(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_node
    ADD CONSTRAINT process_node_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

CREATE TABLE process_transition (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    note text,
    name character varying(32) NOT NULL,
    source_node_id integer NOT NULL,
    target_node_id integer NOT NULL
);
COMMENT ON TABLE process_transition IS 'Process Transition';
COMMENT ON COLUMN process_transition.note IS 'Description';
COMMENT ON COLUMN process_transition.name IS 'Name';
COMMENT ON COLUMN process_transition.source_node_id IS 'Source Node';
COMMENT ON COLUMN process_transition.target_node_id IS 'Target Node';
CREATE SEQUENCE process_transition_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE process_transition_id_seq OWNED BY process_transition.id;
ALTER TABLE ONLY process_transition ALTER COLUMN id SET DEFAULT nextval('process_transition_id_seq'::regclass);
ALTER TABLE ONLY process_transition
    ADD CONSTRAINT process_transition_pkey PRIMARY KEY (id);
ALTER TABLE ONLY process_transition
    ADD CONSTRAINT process_transition_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_transition
    ADD CONSTRAINT process_transition_source_node_id_fkey FOREIGN KEY (source_node_id) REFERENCES process_node(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_transition
    ADD CONSTRAINT process_transition_target_node_id_fkey FOREIGN KEY (target_node_id) REFERENCES process_node(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_transition
    ADD CONSTRAINT process_transition_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

CREATE TABLE process_transition_ids (
    ptr_id integer NOT NULL,
    wtr_id integer NOT NULL
);
COMMENT ON TABLE process_transition_ids IS 'RELATION BETWEEN process_transition AND wkf_transition';
ALTER TABLE ONLY process_transition_ids
    ADD CONSTRAINT process_transition_ids_ptr_id_wtr_id_key UNIQUE (ptr_id, wtr_id);
CREATE INDEX process_transition_ids_ptr_id_index ON process_transition_ids USING btree (ptr_id);
CREATE INDEX process_transition_ids_wtr_id_index ON process_transition_ids USING btree (wtr_id);
ALTER TABLE ONLY process_transition_ids
    ADD CONSTRAINT process_transition_ids_ptr_id_fkey FOREIGN KEY (ptr_id) REFERENCES process_transition(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_transition_ids
    ADD CONSTRAINT process_transition_ids_wtr_id_fkey FOREIGN KEY (wtr_id) REFERENCES wkf_transition(id) ON DELETE CASCADE;

   
   
CREATE TABLE process_condition (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    model_id integer,
    node_id integer NOT NULL,
    model_states character varying(128) NOT NULL,
    name character varying(30) NOT NULL
);
COMMENT ON TABLE process_condition IS 'Condition';
COMMENT ON COLUMN process_condition.model_id IS 'Object';
COMMENT ON COLUMN process_condition.node_id IS 'Node';
COMMENT ON COLUMN process_condition.model_states IS 'Expression';
COMMENT ON COLUMN process_condition.name IS 'Name';
CREATE SEQUENCE process_condition_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE process_condition_id_seq OWNED BY process_condition.id;
ALTER TABLE ONLY process_condition ALTER COLUMN id SET DEFAULT nextval('process_condition_id_seq'::regclass);
ALTER TABLE ONLY process_condition
    ADD CONSTRAINT process_condition_pkey PRIMARY KEY (id);
ALTER TABLE ONLY process_condition
    ADD CONSTRAINT process_condition_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_condition
    ADD CONSTRAINT process_condition_model_id_fkey FOREIGN KEY (model_id) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_condition
    ADD CONSTRAINT process_condition_node_id_fkey FOREIGN KEY (node_id) REFERENCES process_node(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_condition
    ADD CONSTRAINT process_condition_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;
   

CREATE TABLE ir_ui_view_group_rel (
    view_id integer NOT NULL,
    group_id integer NOT NULL
);
COMMENT ON TABLE ir_ui_view_group_rel IS 'RELATION BETWEEN ir_ui_view AND res_groups';
ALTER TABLE ONLY ir_ui_view_group_rel
    ADD CONSTRAINT ir_ui_view_group_rel_view_id_group_id_key UNIQUE (view_id, group_id);
CREATE INDEX ir_ui_view_group_rel_group_id_index ON ir_ui_view_group_rel USING btree (group_id);
CREATE INDEX ir_ui_view_group_rel_view_id_index ON ir_ui_view_group_rel USING btree (view_id);
ALTER TABLE ONLY ir_ui_view_group_rel
    ADD CONSTRAINT ir_ui_view_group_rel_group_id_fkey FOREIGN KEY (group_id) REFERENCES res_groups(id) ON DELETE CASCADE;
ALTER TABLE ONLY ir_ui_view_group_rel
    ADD CONSTRAINT ir_ui_view_group_rel_view_id_fkey FOREIGN KEY (view_id) REFERENCES ir_ui_view(id) ON DELETE CASCADE;

CREATE TABLE ir_model_relation (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    date_init timestamp without time zone,
    date_update timestamp without time zone,
    module integer NOT NULL,
    model integer NOT NULL,
    name character varying(128) NOT NULL
);
CREATE SEQUENCE ir_model_relation_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE ir_model_relation_id_seq OWNED BY ir_model_relation.id;
ALTER TABLE ONLY ir_model_relation ALTER COLUMN id SET DEFAULT nextval('ir_model_relation_id_seq'::regclass);
CREATE INDEX ir_model_relation_model_index ON ir_model_relation USING btree (model);
CREATE INDEX ir_model_relation_module_index ON ir_model_relation USING btree (module);
CREATE INDEX ir_model_relation_name_index ON ir_model_relation USING btree (name);
ALTER TABLE ONLY ir_model_relation
    ADD CONSTRAINT ir_model_relation_model_fkey FOREIGN KEY (model) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_model_relation
    ADD CONSTRAINT ir_model_relation_module_fkey FOREIGN KEY (module) REFERENCES ir_module_module(id) ON DELETE SET NULL;

CREATE TABLE ir_model_constraint (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    date_init timestamp without time zone,
    date_update timestamp without time zone,
    module integer NOT NULL,
    model integer NOT NULL,
    type character varying(1) NOT NULL,
    name character varying(128) NOT NULL
);
CREATE SEQUENCE ir_model_constraint_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE ir_model_constraint_id_seq OWNED BY ir_model_constraint.id;
ALTER TABLE ONLY ir_model_constraint ALTER COLUMN id SET DEFAULT nextval('ir_model_constraint_id_seq'::regclass);
ALTER TABLE ONLY ir_model_constraint
    ADD CONSTRAINT ir_model_constraint_module_name_uniq UNIQUE (name, module);
CREATE INDEX ir_model_constraint_model_index ON ir_model_constraint USING btree (model);
CREATE INDEX ir_model_constraint_module_index ON ir_model_constraint USING btree (module);
CREATE INDEX ir_model_constraint_name_index ON ir_model_constraint USING btree (name);
CREATE INDEX ir_model_constraint_type_index ON ir_model_constraint USING btree (type);
ALTER TABLE ONLY ir_model_constraint
    ADD CONSTRAINT ir_model_constraint_model_fkey FOREIGN KEY (model) REFERENCES ir_model(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_model_constraint
    ADD CONSTRAINT ir_model_constraint_module_fkey FOREIGN KEY (module) REFERENCES ir_module_module(id) ON DELETE SET NULL;

CREATE TABLE ir_fields_converter (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer
);
COMMENT ON TABLE ir_fields_converter IS 'ir.fields.converter';
CREATE SEQUENCE ir_fields_converter_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE ir_fields_converter_id_seq OWNED BY ir_fields_converter.id;
ALTER TABLE ONLY ir_fields_converter ALTER COLUMN id SET DEFAULT nextval('ir_fields_converter_id_seq'::regclass);
ALTER TABLE ONLY ir_fields_converter
    ADD CONSTRAINT ir_fields_converter_pkey PRIMARY KEY (id);
ALTER TABLE ONLY ir_fields_converter
    ADD CONSTRAINT ir_fields_converter_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY ir_fields_converter
    ADD CONSTRAINT ir_fields_converter_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

CREATE TABLE process_transition_group_rel (
    tid integer NOT NULL,
    rid integer NOT NULL
);
COMMENT ON TABLE process_transition_group_rel IS 'RELATION BETWEEN process_transition AND res_groups';
ALTER TABLE ONLY process_transition_group_rel
    ADD CONSTRAINT process_transition_group_rel_tid_rid_key UNIQUE (tid, rid);
CREATE INDEX process_transition_group_rel_rid_index ON process_transition_group_rel USING btree (rid);
CREATE INDEX process_transition_group_rel_tid_index ON process_transition_group_rel USING btree (tid);
ALTER TABLE ONLY process_transition_group_rel
    ADD CONSTRAINT process_transition_group_rel_rid_fkey FOREIGN KEY (rid) REFERENCES res_groups(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_transition_group_rel
    ADD CONSTRAINT process_transition_group_rel_tid_fkey FOREIGN KEY (tid) REFERENCES process_transition(id) ON DELETE CASCADE;


CREATE TABLE res_config_settings (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer
);
COMMENT ON TABLE res_config_settings IS 'res.config.settings';
CREATE SEQUENCE res_config_settings_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE res_config_settings_id_seq OWNED BY res_config_settings.id;
ALTER TABLE ONLY res_config_settings ALTER COLUMN id SET DEFAULT nextval('res_config_settings_id_seq'::regclass);
ALTER TABLE ONLY res_config_settings
    ADD CONSTRAINT res_config_settings_pkey PRIMARY KEY (id);
ALTER TABLE ONLY res_config_settings
    ADD CONSTRAINT res_config_settings_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY res_config_settings
    ADD CONSTRAINT res_config_settings_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

CREATE TABLE process_transition_action (
    id integer NOT NULL,
    create_uid integer,
    create_date timestamp without time zone,
    write_date timestamp without time zone,
    write_uid integer,
    action character varying(64),
    state character varying NOT NULL,
    name character varying(32) NOT NULL,
    transition_id integer NOT NULL
);
COMMENT ON TABLE process_transition_action IS 'Process Transitions Actions';
COMMENT ON COLUMN process_transition_action.action IS 'Action ID';
COMMENT ON COLUMN process_transition_action.state IS 'Type';
COMMENT ON COLUMN process_transition_action.name IS 'Name';
COMMENT ON COLUMN process_transition_action.transition_id IS 'Transition';
CREATE SEQUENCE process_transition_action_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;
ALTER SEQUENCE process_transition_action_id_seq OWNED BY process_transition_action.id;
ALTER TABLE ONLY process_transition_action ALTER COLUMN id SET DEFAULT nextval('process_transition_action_id_seq'::regclass);
ALTER TABLE ONLY process_transition_action
    ADD CONSTRAINT process_transition_action_pkey PRIMARY KEY (id);
ALTER TABLE ONLY process_transition_action
    ADD CONSTRAINT process_transition_action_create_uid_fkey FOREIGN KEY (create_uid) REFERENCES res_users(id) ON DELETE SET NULL;
ALTER TABLE ONLY process_transition_action
    ADD CONSTRAINT process_transition_action_transition_id_fkey FOREIGN KEY (transition_id) REFERENCES process_transition(id) ON DELETE CASCADE;
ALTER TABLE ONLY process_transition_action
    ADD CONSTRAINT process_transition_action_write_uid_fkey FOREIGN KEY (write_uid) REFERENCES res_users(id) ON DELETE SET NULL;

--- =================================================
--- Tables in 'teste_6_1' that are not in 'teste_7_0'
--- ================================================= 

drop table public.res_payterm;

drop TABLE res_widget_user;

drop TABLE res_widget;

drop TABLE res_partner_address;

drop TABLE res_partner_event;

drop TABLE res_partner_category_rel;

drop TABLE maintenance_contract;

drop TABLE wizard_ir_model_menu_create_line;

drop TABLE res_widget_wizard;


drop TABLE partner_massmail_wizard;

drop TABLE partner_clear_ids;

drop TABLE partner_wizard_ean_check;

drop TABLE res_log;

drop TABLE partner_sms_send;

drop TABLE publisher_warranty_contract_wizard;

drop TABLE ir_wizard_screen;

drop TABLE publisher_warranty_contract;

alter table public.ir_actions_todo
   drop column category_id,
   drop constraint ir_actions_todo_action_id_fkey;

drop TABLE ir_actions_todo_category;

--- =================================================================
---  Tables that are in both databases but have different attributes
--- =================================================================
alter table public.ir_actions
  add column help text;
COMMENT ON COLUMN ir_actions.help IS 'Action description';


alter table public.ir_act_client
  add column res_model character varying(64),
  add column context character varying(250) NOT NULL;
COMMENT ON COLUMN ir_act_client.res_model IS 'Destination Model';
COMMENT ON COLUMN ir_act_client.context IS 'Context Value';


alter table public.res_company
   drop column rml_footer1,
   drop column logo,
   add column custom_footer boolean,
   add column rml_footer text;

COMMENT ON COLUMN res_company.rml_footer IS 'Report Footer';
COMMENT ON COLUMN res_company.custom_footer IS 'Custom Footer';


alter table public.ir_rule
  add column active boolean;
COMMENT ON COLUMN ir_rule.active IS 'Active';
update ir_rule set active = True;


alter table public.ir_model_access
  add column active boolean;
COMMENT ON COLUMN ir_model_access.active IS 'Active';

update ir_model_access set active = True;

alter table public.res_partner
  add column function character varying(128), -- Job Position
  add column city character varying(128), -- City
  add column fax character varying(64), -- Fax
  add column image bytea, -- Image
  add column zip character varying(24), -- Zip
  add column tz character varying(64), -- Timezone
  add column mobile character varying(64), -- Mobile
  add column image_medium bytea, -- Medium-sized image
  add column image_small bytea, -- Small-sized image
  add column country_id integer, -- Country
  add column birthdate character varying(64), -- Birthdate
  add column phone character varying(64), -- Phone
  add column use_parent_address boolean, -- Use Company Address
  add column street character varying(128), -- Street
  add column street2 character varying(128), -- Street2
  add column state_id integer, -- State
  add column type character varying, -- Address Type
  add column email character varying(240), -- Email
  add column is_company boolean,
  add column comment text,
  add CONSTRAINT res_partner_country_id_fkey FOREIGN KEY (country_id)
      REFERENCES res_country (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  add CONSTRAINT res_partner_state_id_fkey FOREIGN KEY (state_id)
      REFERENCES res_country_state (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
COMMENT ON COLUMN res_partner.image IS 'Image';
COMMENT ON COLUMN res_partner.use_parent_address IS 'Use Company Address';
COMMENT ON COLUMN res_partner.street IS 'Street';
COMMENT ON COLUMN res_partner.zip IS 'Zip';
COMMENT ON COLUMN res_partner.function IS 'Job Position';
COMMENT ON COLUMN res_partner.country_id IS 'Country';
COMMENT ON COLUMN res_partner.type IS 'Address Type';
COMMENT ON COLUMN res_partner.fax IS 'Fax';
COMMENT ON COLUMN res_partner.street2 IS 'Street2';
COMMENT ON COLUMN res_partner.phone IS 'Phone';
COMMENT ON COLUMN res_partner.tz IS 'Timezone';
COMMENT ON COLUMN res_partner.image_medium IS 'Medium-sized image';
COMMENT ON COLUMN res_partner.mobile IS 'Mobile';
COMMENT ON COLUMN res_partner.image_small IS 'Small-sized image';
COMMENT ON COLUMN res_partner.birthdate IS 'Birthdate';
COMMENT ON COLUMN res_partner.is_company IS 'Is a Company';
COMMENT ON COLUMN res_partner.state_id IS 'State';

alter table public.ir_module_module
   drop column certificate,
   add column   summary character varying(256);


alter table public.ir_filters
  add column is_default boolean;
COMMENT ON COLUMN ir_filters.is_default IS 'Default filter';


alter table public.res_users
  drop column context_tz,
  drop column context_lang,
  drop column name,
  drop column menu_tips,
  drop column date,
  drop column email,
  drop column user_email,
  add column lang character varying(64) DEFAULT ''::character varying,
  add column partner_id integer,
  add column login_date date;
COMMENT ON COLUMN res_users.login_date IS 'Latest connection';
CREATE INDEX res_users_login_date_index
  ON res_users
  USING btree
  (login_date );
update res_users set lang='pt_BR';
update ir_ui_view set arch=replace(arch, '<field name="context_tz"/>', '') where model='res.users' and arch like '%<field name="context_tz"/>%';
update ir_ui_view set arch=replace(arch, '<field name="context_tz" readonly="0"/>', '') where model='res.users' and arch like '%<field name="context_tz" readonly="0"/>%';
update ir_ui_view set arch=replace(arch, '<field name="context_lang" readonly="0"/>', '') where model='res.users' and arch like '%<field name="context_lang" readonly="0"/>%';
update ir_ui_view set arch=replace(arch, '<field name="menu_tips"/>', '') where model='res.users' and arch like '%<field name="menu_tips"/>%';
update ir_ui_view set arch=replace(arch, '<field name="menu_tips" readonly="0" groups="base.group_no_one"/>', '') where model='res.users' and arch like '%<field name="menu_tips" readonly="0" groups="base.group_no_one"/>%';
update ir_ui_view set arch=replace(arch, '<field name="context_lang"/>', '') where model='res.users' and arch like '%<field name="context_lang"/>%';
update res_users set partner_id=1 where partner_id is null;

alter table public.res_country
   add column currency_id integer,
   add CONSTRAINT res_country_currency_id_fkey FOREIGN KEY (currency_id)
      REFERENCES res_currency (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL;
COMMENT ON COLUMN res_country.currency_id IS 'Currency';


alter table public.ir_act_window
   add column   res_id integer;
COMMENT ON COLUMN ir_act_window.res_id IS 'Record ID';   

alter table public.ir_translation
   add column  state character varying, -- Status
   add column   comments text, -- Translation comments
   add column   module character varying;
COMMENT ON COLUMN ir_translation.module IS 'Module';
COMMENT ON COLUMN ir_translation.state IS 'Status';
COMMENT ON COLUMN ir_translation.comments IS 'Translation comments';   

update res_users set partner_id =1 where partner_id is null;
-- delete from wkf_workitem where act_id in (2, 24, 33);
update ir_act_window set auto_search=True;

-- Conversoes para base da SIG
-- delete  from ir_act_window where view_id in (select w.view_id from ir_act_window w left join ir_ui_view v on v.id=w.view_id where view_id > 0 and v.id is null);
-- alter table res_currency_rate alter column rate type numeric;
-- alter table rh_marcacao add column rh_horario numeric;
-- update rh_marcacao set rh_horario=((select extract(hour from rh_hora)) + (select extract(minute from rh_hora) / 60));
-- alter table drop column rh_hora;
--
-- alter table rh_periodo add column rh_hora_entrada numeric;
-- alter table rh_periodo add column rh_hora_saida numeric;
-- update rh_periodo set rh_hora_entrada=((select extract(hour from rh_entrada)) + (select extract(minute from rh_entrada) / 60)),
--                       rh_hora_saida=((select extract(hour from rh_saida)) + (select extract(minute from rh_saida) / 60));
-- alter table rh_periodo drop column rh_entrada;
-- alter table rh_periodo drop column rh_saida;
--
-- UPDATE procedimentos_atividade set responsavel_id = 63 where codigo like 'OP%' OR codigo like 'DOP%';
-- UPDATE procedimentos_atividade set responsavel_id = 3178 where codigo like 'NOR%' OR codigo like 'ADM%';
-- UPDATE procedimentos_atividade set responsavel_id = 63 where responsavel_id in (3, 5);
-- delete from wkf_activity where id in (2, 24);
-- delete from ir_ui_menu;
-- delete from ir_values where name='Menuitem';
-- delete from ir_actions where name ='';
-- alter sequence ir_ui_menu_id_seq RESTART WITH 1;
