CREATE TABLE cotacao_produtos (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  codigo_produto CHAR(15) NOT NULL,
  PRIMARY KEY(id)
)
TYPE=InnoDB;

CREATE TABLE Usuarios (
  id INTEGER UNSIGNED NOT NULL,
  Login CHAR(20) NOT NULL,
  Senha CHAR(20) NOT NULL,
  Nome CHAR(40) NOT NULL,
  adm_email CHAR(40) NOT NULL,
  PRIMARY KEY(id),
  UNIQUE INDEX IDX_Login(Login, Senha)
)
TYPE=InnoDB;

CREATE TABLE adm_empresa (
  id INTEGER UNSIGNED NOT NULL,
  CNPJ CHAR(14) NOT NULL,
  RazaoSocial CHAR(40) NOT NULL,
  Endereco CHAR(40) NULL,
  Telefone CHAR(15) NULL,
  Fax CHAR(15) NULL,
  Bairro CHAR(15) NULL,
  Cidade CHAR(20) NULL,
  Cep CHAR(8) NULL,
  Estado CHAR(2) NULL,
  Dominio CHAR(40) NOT NULL,
  PRIMARY KEY(id)
)
TYPE=InnoDB;

CREATE TABLE fornecedor_fornecedor (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  adm_empresa_id INTEGER UNSIGNED NOT NULL,
  CodFornecedor CHAR(14) NOT NULL,
  Nome CHAR(40) NOT NULL,
  adm_email CHAR(40) NULL,
  PRIMARY KEY(id),
  INDEX FK_Pessoa_Empresa(adm_empresa_id),
  UNIQUE INDEX UK_EmpresaFornecedor(adm_empresa_id, CodFornecedor),
  FOREIGN KEY(adm_empresa_id)
    REFERENCES adm_empresa(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE cotacao_produtos_grades (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  produto_id INTEGER UNSIGNED NOT NULL,
  grade CHAR(3) NOT NULL,
  nome CHAR(40) NOT NULL,
  PRIMARY KEY(id),
  INDEX cotacao_grade_FKIndex1(produto_id),
  FOREIGN KEY(produto_id)
    REFERENCES cotacao_produtos(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE cotacao_cotacoes (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  fornecedor_id INTEGER UNSIGNED NOT NULL,
  nro_cotacao INTEGER UNSIGNED NOT NULL,
  data_importacao DATE NULL,
  data_cotacao DATE NULL,
  situacao ENUM('I', 'E', 'C', 'X') NULL DEFAULT "I",
  prazo_entrega DATE NULL,
  validade INTEGER UNSIGNED NULL,
  prazo_pagto1 INTEGER UNSIGNED NULL,
  prazo_pagto2 INTEGER UNSIGNED NULL,
  prazo_pagto3 INTEGER UNSIGNED NULL,
  prazo_pagto4 INTEGER UNSIGNED NULL,
  prazo_pagto5 INTEGER UNSIGNED NULL,
  prazo_pagto6 INTEGER UNSIGNED NULL,
  comprador SMALLINT UNSIGNED NULL,
  cif_fob ENUM('C', 'F') NULL,
  transportadora CHAR(20) NULL,
  PRIMARY KEY(id),
  INDEX cotacao_cotacoes_FKIndex1(fornecedor_id),
  FOREIGN KEY(fornecedor_id)
    REFERENCES fornecedor_fornecedor(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE cotacao_cotacoes_algoritmos (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  cotacao_id INTEGER UNSIGNED NOT NULL,
  codigo CHAR(8) NOT NULL,
  PRIMARY KEY(id),
  INDEX cotacao_algoritmo_FKIndex1(cotacao_id),
  FOREIGN KEY(cotacao_id)
    REFERENCES cotacao_cotacoes(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE adm_email (
  adm_empresa_id INTEGER UNSIGNED NOT NULL,
  SmtpServer CHAR(40) NOT NULL,
  ContaEmail CHAR(40) NOT NULL,
  Senha CHAR(20) NOT NULL,
  Porta INTEGER UNSIGNED NULL,
  PRIMARY KEY(adm_empresa_id),
  INDEX FK_Email_Empresa(adm_empresa_id),
  FOREIGN KEY(adm_empresa_id)
    REFERENCES adm_empresa(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE Empresa_has_Usuarios (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  Usuarios_id INTEGER UNSIGNED NOT NULL,
  adm_empresa_id INTEGER UNSIGNED NOT NULL,
  Administrador ENUM('S', 'N') NOT NULL DEFAULT "N",
  PRIMARY KEY(id),
  INDEX FK_Empresas(adm_empresa_id),
  UNIQUE INDEX UK_EmpresaUsuarios(adm_empresa_id),
  INDEX Empresa_has_Usuarios_FKIndex2(Usuarios_id),
  FOREIGN KEY(adm_empresa_id)
    REFERENCES adm_empresa(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(Usuarios_id)
    REFERENCES Usuarios(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
);

CREATE TABLE cotacao_cotacoes_itens (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  cotacao_id INTEGER UNSIGNED NOT NULL,
  grade_id INTEGER UNSIGNED NOT NULL,
  valor NUMERIC(12,2) NULL,
  frete NUMERIC(12,2) NULL,
  outros NUMERIC(12,2) NULL,
  descontos NUMERIC(12,2) NULL,
  encargos NUMERIC(12,2) NULL,
  depesas_acessorias NUMERIC(12,2) NULL,
  icms SMALLINT UNSIGNED NULL,
  ipi SMALLINT UNSIGNED NULL,
  iss SMALLINT UNSIGNED NULL,
  garantia_item CHAR(20) NULL,
  PRIMARY KEY(id),
  INDEX cotacao_itens_FKIndex1(cotacao_id),
  INDEX cotacao_itens_FKIndex2(grade_id),
  FOREIGN KEY(cotacao_id)
    REFERENCES cotacao_cotacoes(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(grade_id)
    REFERENCES cotacao_produtos_grades(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
)
TYPE=InnoDB;

CREATE TABLE cotacao_produtos_fornecedores (
  id INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,
  fornecedor_id INTEGER UNSIGNED NOT NULL,
  produto_id INTEGER UNSIGNED NOT NULL,
  PRIMARY KEY(id),
  INDEX cotacao_fornecedor_produto_FKIndex1(fornecedor_id),
  INDEX cotacao_fornecedor_produto_FKIndex2(produto_id),
  FOREIGN KEY(fornecedor_id)
    REFERENCES fornecedor_fornecedor(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  FOREIGN KEY(produto_id)
    REFERENCES cotacao_produtos(id)
      ON DELETE CASCADE
      ON UPDATE CASCADE
);


