SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

CREATE SCHEMA IF NOT EXISTS `mydb` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci ;
USE `mydb` ;

-- -----------------------------------------------------
-- Table `mydb`.`estoquepedido_pedidos_capa`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`estoquepedido_pedidos_capa` (
  `id` INT NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`fiscal_nf_capa`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`fiscal_nf_capa` (
  `id` INT NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`entrega_vendas_capa`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`entrega_vendas_capa` (
  `id` INT NOT NULL ,
  `entvec_pedido_id` INT NULL ,
  `entvec_nf_id` INT NULL ,
  `entvec_data_prevista` DATE NOT NULL ,
  INDEX `fk_entrega_vendas_capa_table1_idx` (`entvec_pedido_id` ASC) ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_entrega_vendas_capa_fiscal_nf_capa1_idx` (`entvec_nf_id` ASC) ,
  CONSTRAINT `fk_entrega_vendas_capa_table1`
    FOREIGN KEY (`entvec_pedido_id` )
    REFERENCES `mydb`.`estoquepedido_pedidos_capa` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT,
  CONSTRAINT `fk_entrega_vendas_capa_fiscal_nf_capa1`
    FOREIGN KEY (`entvec_nf_id` )
    REFERENCES `mydb`.`fiscal_nf_capa` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT)
ENGINE = InnoDB
COMMENT = 'Tabela de entregas, contem os dados do que tem que ser/foi e /* comment truncated */ /*ntregue
Prefixo: entvec_*/';


-- -----------------------------------------------------
-- Table `mydb`.`estoquepedido_pedidos_item`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`estoquepedido_pedidos_item` (
  `id` INT NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`estoque_movimento_itens`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`estoque_movimento_itens` (
  `id` INT NOT NULL ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`entrega_vendas_item`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`entrega_vendas_item` (
  `id` INT NOT NULL ,
  `entvei_capa_id` INT NOT NULL ,
  `entvei_pedidos_item_id` INT NULL ,
  `entvei_nf_item_id` INT NULL ,
  `entvei_qtd_pedida` FLOAT NOT NULL ,
  `entvei_qtd_separada` FLOAT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_entrega_vendas_item_entrega_vendas_capa1_idx` (`entvei_capa_id` ASC) ,
  INDEX `fk_entrega_vendas_item_estoquepedido_pedidos_item1_idx` (`entvei_pedidos_item_id` ASC) ,
  INDEX `fk_entrega_vendas_item_fiscal_nf_item1_idx` (`entvei_nf_item_id` ASC) ,
  CONSTRAINT `fk_entrega_vendas_item_entrega_vendas_capa1`
    FOREIGN KEY (`entvei_capa_id` )
    REFERENCES `mydb`.`entrega_vendas_capa` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT `fk_entrega_vendas_item_estoquepedido_pedidos_item1`
    FOREIGN KEY (`entvei_pedidos_item_id` )
    REFERENCES `mydb`.`estoquepedido_pedidos_item` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE,
  CONSTRAINT `fk_entrega_vendas_item_fiscal_nf_item1`
    FOREIGN KEY (`entvei_nf_item_id` )
    REFERENCES `mydb`.`estoque_movimento_itens` (`id` )
    ON DELETE CASCADE
    ON UPDATE CASCADE)
ENGINE = InnoDB
COMMENT = 'Tabela de itens a serem/foram entregues\n_inherits = {\'entreg /* comment truncated */ /*a_vendas_capa': 'entvei_capa_id'}
Prefixo:entvei_*/';


-- -----------------------------------------------------
-- Table `mydb`.`entrega_heranca_veiculos_veiculo`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`entrega_heranca_veiculos_veiculo` (
  `id` INT NOT NULL ,
  `veivei_disponivel` TINYINT(1) NULL DEFAULT True ,
  PRIMARY KEY (`id`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `mydb`.`entrega_vendas_item_entregues`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`entrega_vendas_item_entregues` (
  `id` INT NOT NULL ,
  `entvie_item_id` INT NOT NULL ,
  `entvie_veiculos_id` INT NOT NULL ,
  `entvie_qtd_entregue` FLOAT NOT NULL ,
  `entvie_data_entrega` DATE NOT NULL DEFAULT date ,
  `entvie_hora_entrega` FLOAT NULL ,
  `entvie_quem_recebeu` VARCHAR(45) NULL ,
  `entvie_obs` TEXT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_entrega_vendas_item_entregues_entrega_vendas_item1_idx` (`entvie_item_id` ASC) ,
  INDEX `fk_entrega_vendas_item_entregues_entrega_heranca_veiculo_ve_idx` (`entvie_veiculos_id` ASC) ,
  CONSTRAINT `fk_entrega_vendas_item_entregues_entrega_vendas_item1`
    FOREIGN KEY (`entvie_item_id` )
    REFERENCES `mydb`.`entrega_vendas_item` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT,
  CONSTRAINT `fk_entrega_vendas_item_entregues_entrega_heranca_veiculo_veic1`
    FOREIGN KEY (`entvie_veiculos_id` )
    REFERENCES `mydb`.`entrega_heranca_veiculos_veiculo` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT)
ENGINE = InnoDB
COMMENT = 'Tabela de itens entregues\nPrefixo: entvie_';


-- -----------------------------------------------------
-- Table `mydb`.`entrega_viagem`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS `mydb`.`entrega_viagem` (
  `id` INT NOT NULL ,
  `entprg_veiculos_id` INT NOT NULL ,
  `entprg_data` DATE NOT NULL ,
  `entprg_capa_id` INT NOT NULL ,
  `entprg_volume` FLOAT NULL ,
  `entprg_peso` FLOAT NULL ,
  PRIMARY KEY (`id`) ,
  INDEX `fk_entrega_viagem_entrega_heranca_veiculo_veiculos1_idx` (`entprg_veiculos_id` ASC) ,
  INDEX `fk_entrega_viagem_entrega_vendas_capa1_idx` (`entprg_capa_id` ASC) ,
  CONSTRAINT `fk_entrega_viagem_entrega_heranca_veiculo_veiculos1`
    FOREIGN KEY (`entprg_veiculos_id` )
    REFERENCES `mydb`.`entrega_heranca_veiculos_veiculo` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT,
  CONSTRAINT `fk_entrega_viagem_entrega_vendas_capa1`
    FOREIGN KEY (`entprg_capa_id` )
    REFERENCES `mydb`.`entrega_vendas_capa` (`id` )
    ON DELETE RESTRICT
    ON UPDATE RESTRICT)
ENGINE = InnoDB
COMMENT = 'Tabela de Programação de entregas\nPrefixo: entprg_';

USE `mydb` ;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;
