#!/usr/bin/python
# -*- encoding: utf-8 -*-

from openerp.sql_db import db_connect
from openerp.osv import osv
from traceback import format_exc as traceback_format_exc
from ..integracao.modulos import cre2open, cpg2open, cax2open, ccb2open, sis2open
from ..integracao.modulos import est2open, fis2open, gei2open, ven2open, ctb2open, chq2open
from ..integracao.registros.integ_registros import Registros
from ..model.integracao_descricao_coluna import IntegracaDescricaoColunas
from logging import getLogger
from ..integracao.funcoes.integ_funcoes import CalcTempo
import time

READ_LOAD = '_classic_write'
_logger = getLogger("[LEGADO INTEGRACAO]")


class legado_logs_control(osv.Model):
    
    _inherit = 'legado.logs'

    def write(self, cr, uid, ids, values, context=None):
        context = context and context.copy() or {}
        if 'lelg_conteudo' in values:
            values['lelg_erro_registro'] = False
        res = super(legado_logs_control, self).write(cr, uid, ids, values, context=context)
        return res

    def leglog_get_empresa(self, db_registro, legado_empresa, tipo_empresa, context=None):
        """
        Ao utilizar esta funcao, trata-la com try-except. Informar no except o rollback
        """
        context = context and context.copy() or {}
        args = [('empresa_legado', '=', legado_empresa), ('tipo', '=', tipo_empresa)]
        leg_empresa_dados = db_registro.select(table='legado_empresa', fields=['company_id'], where=args)
        if not leg_empresa_dados:
            str_tipo = 'Com NF' if tipo_empresa == 'C' else 'Sem NF'
            raise osv.except_osv(
                'SigERP',
                'Nenhuma correlacao de empresa criada para a Empresa: %s - Tipo - %s' % (legado_empresa, str_tipo)
            )
        else:
            company_id = leg_empresa_dados[0]['company_id']
            return company_id

    def leglog_importar_registros(self, cr, uid, context=None):
        """
        1º - CRIA UMA NOVA CONEXAO COM O BANCO PARA INSERIR OS DADOS.
        """
        context = context and context.copy() or {}
        lelg_empresa_legado = context.get('empresa_legado')
        empresa_cron = lelg_empresa_legado
        cron_principal = False
        if not lelg_empresa_legado:
            obj_ir_cron = self.pool.get('ir.cron')
            args_ir_cron = [('active', '=', True),
                            ('model', '=', 'legado.logs'),
                            ('function', '=', 'leglog_importar_registros')]
            ir_cron_ids = obj_ir_cron.search(cr, uid, args_ir_cron, context=context)
            dados_cron = obj_ir_cron.read(cr, uid, ir_cron_ids, ['args'], context=context)
            empresa_cron = []
            for dados in dados_cron:
                if dados.get('args'):
                    empresa = eval(str(dados.get('args')).replace('(', '').replace(')', '').replace(',', ''))
                    empresa_cron.append(empresa.get('empresa_legado'))
            cron_principal = True
        cr_integracao = False
        db_registro_erp = Registros(cr, uid)
        legado_log_ids = self.lelog_get_legado_log_ids(db_registro_erp, empresa_cron,
                                                       cron_principal)
        if legado_log_ids:
            try:
                cursor_dbname = filter(lambda i: i.startswith('dbname='), cr.connection.dsn.split(' '))
                dbname = cursor_dbname[0].split('=')[1]
                # CRIANDO CONEXAO EXTERNA COM O BANCO DE DADOS PARA INSERCAO DOS DADOS DA INTEGRACAO.
                # A CONEXAO EXTERNA É NECESSARIA PARA NAO HAVER CONCORRENCIA DE DADOS, POIS ESTAVA GERANDO ERRO.
                _logger.info("CONEXAO COM A BASE DE DADOS: %s" % dbname)
                conn_integracao = db_connect(dbname)
                cr_integracao = conn_integracao.cursor()
                db_registro_integracao = Registros(cr_integracao, uid)

                # FUNCAO PARA EXECUTAR A IMPORTACAO
                self.lelog_executar_importacao(cr_integracao, uid, db_registro_integracao, legado_log_ids, context)

            finally:
                if cr_integracao:
                    cr_integracao.close()
                if db_registro_integracao.cr_integracao:
                    db_registro_integracao.cr_integracao.close()
                    _logger.info("FECHANDO CONEXAO")
        return True

    def lelog_get_legado_log_ids(self, db_registro, lelg_empresa_legado, cron_principal):
        """
        2º - VERIFICA SE EXISTEM REGISTRO PARA SEREM IMPORTADOS
        """
        args_order_by = """
                        id IN(SELECT id
                                FROM   legado_logs
                                WHERE  lelg_arquivo_legado IN ( 'est01', 'est05', 'est10',
                                                                'est17',
                                                                'est55' )
                                       AND lelg_operacao = 'W'
                                ORDER  BY id) DESC
                        """
        if cron_principal:
            if not lelg_empresa_legado:
                where = [('lelg_erro_registro', '=', False)]
            else:
                where = [('lelg_erro_registro', '=', False),
                         ('lelg_empresa_legado', 'not in', tuple(lelg_empresa_legado))]
        else:
            where = [('lelg_erro_registro', '=', False),
                     ('lelg_empresa_legado ', '=', lelg_empresa_legado)]
        legado_log_dados_ids = db_registro.select(
            table='legado_logs', 
            fields=['id'],
            where=where,
            order_by=['id'],
            #order_by=[args_order_by,
            #          'lelg_arquivo_legado = \'fis01\' desc',
                      #'lelg_conteudo ilike \'%*|*E*|*N*|*A*%\' desc',
            #          'id'],
            limit=10000
        )
        if legado_log_dados_ids:
            ids = tuple(map(lambda i: i['id'], legado_log_dados_ids))
        else:
            ids = []
        return ids

    def lelog_executar_importacao(self, cr, uid, db_registro, legado_log_ids, context=None):
        """
        3º - REALIZA A IMPORTACAO DOS DADOS
        """
        context = context and context.copy() or {}
        if legado_log_ids:
            args_order_by = """
                        id IN(SELECT id
                                FROM   legado_logs
                                WHERE  lelg_arquivo_legado IN ( 'est01', 'est05', 'est10',
                                                                'est17',
                                                                'est55' )
                                       AND lelg_operacao = 'W'
                                ORDER  BY id) DESC """
            fields_to_read = ['id', 'lelg_operacao', 'lelg_conteudo', 'lelg_arquivo_legado',
                              'lelg_empresa_legado']

            _logger.info("REGISTROS AGUARDANDO IMPORTACAO: %d" % legado_log_ids.__len__())

            legado_dados = db_registro.select(
                table='legado_logs', fields=fields_to_read, where=[('id', 'in', legado_log_ids)],
                order_by=['id']
            )
            # OBJETOS
            obj_nf_capa = self.pool.get('fiscal.nf.capa')
            obj_adm_empresa = self.pool.get('adm.empresa.objeto')
            obj_estoque_produto = self.pool.get('estoque.produto')
            obj_tesouraria_saldo = self.pool.get('financeiro.tesouraria.saldos')
            obj_tesouraria_conta = self.pool.get('financeiro.tesouraria.contas')
            obj_estoque_categoria = self.pool.get('estoque.categoria')
            obj_estoque_parametro = self.pool.get('estoque.parametro')
            obj_embalagem_unidade = self.pool.get('estoque.embalagem.unidade')
            obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
            obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')
            obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
            obj_ir_cron = self.pool.get('ir.cron')
            obj_adm_utils = self.pool.get('adm.utils')

            est_parametro_dados = db_registro.select(table='estoque_parametro', fields=['company_id', 'codigo_numerico'])
            dict_company_usar_digito = False
            if not est_parametro_dados:
                _logger.error('Parametro do Estoque nao cadastrado')
                db_registro.cr.connection.rollback()
            else:
                dict_company_usar_digito = dict(map(lambda i: (i['company_id'], i['codigo_numerico']), est_parametro_dados))

            # CONTEXTO PARA OS MODULOS
            context.update({
                'ctx_obj_tesouraia_saldo': obj_tesouraria_saldo,
                'ctx_obj_tesouraia_conta': obj_tesouraria_conta,
                'ctx_ccb02_obj_tesouraia_saldo': obj_tesouraria_saldo,
                # EST
                'ctx_est_obj_estoque_produto': obj_estoque_produto,
                'ctx_est_utilizar_codigo_numerico': dict_company_usar_digito,
                'ctx_obj_nf_capa': obj_nf_capa,
                'ctx_est_obj_estoque_categoria': obj_estoque_categoria,
                'ctx_est_obj_embalagem_unidade': obj_embalagem_unidade,
                'ctx_funct_getEmpresa': obj_adm_empresa.getEmpresa,
                'ctx_est_obj_estoque_parametro': obj_estoque_parametro,
                'ctx_estped_obj_estoquepedidos_pedido_item': obj_estoquepedidos_pedido_item,
                'ctx_est_obj_estoque_local_locacao': obj_estoque_local_locacao,
                'ctx_est_obj_estoque_produto_grade': obj_estoque_produto_grade,
                'ctx_obj_ir_cron': obj_ir_cron,
                'ctx_obj_adm_utils': obj_adm_utils
            })
            cont = 0
            estatisticas = {}
            if legado_dados:
                _logger.info("IMPORTANDO ......")
                for i in legado_dados:
                    _i = time.time()
                    legado_id = i['id']
                    legado_operacao = i.get('lelg_operacao')
                    legado_conteudo = i.get('lelg_conteudo')
                    legado_arquivo = (i.get('lelg_arquivo_legado') or '').lower()
                    legado_empresa = i.get('lelg_empresa_legado')

                    try:
                        company_com_nf_id = self.leglog_get_empresa(db_registro, legado_empresa, 'C', context)
                        company_sem_nf_id = self.leglog_get_empresa(db_registro, legado_empresa, 'S', context)
                    except:
                        db_registro.cr.connection.rollback()
                        msg_erro = traceback_format_exc().replace("'", '"')
                        db_registro.registro_gravar_erro(legado_id, msg_erro)
                        cr.commit()
                        continue

                    modulo = legado_arquivo.capitalize()
                    if modulo:
                        function = 'gerar%s' % modulo
                        # CRE
                        if legado_arquivo.startswith('cre'):
                            gerar_registro = getattr(cre2open, function)
                        # CPG
                        elif legado_arquivo.startswith('cpg'):
                            gerar_registro = getattr(cpg2open, function)
                        # CAX
                        elif legado_arquivo.startswith('cax'):
                            gerar_registro = getattr(cax2open, function)
                        # CCB
                        elif legado_arquivo.startswith('ccb'):
                            gerar_registro = getattr(ccb2open, function)
                        # SIS
                        elif legado_arquivo.startswith('sis'):
                            gerar_registro = getattr(sis2open, function)
                        # EST
                        elif legado_arquivo.startswith('est'):
                            gerar_registro = getattr(est2open, function)
                        # FIS
                        elif legado_arquivo.startswith('fis'):
                            gerar_registro = getattr(fis2open, function)
                        # GEI
                        elif legado_arquivo.startswith('gei'):
                            gerar_registro = getattr(gei2open, function)
                        # VEN
                        elif legado_arquivo.startswith('ven'):
                            gerar_registro = getattr(ven2open, function)
                        # ctb
                        elif legado_arquivo.startswith('ctb'):
                            gerar_registro = getattr(ctb2open, function)
                        # chq
                        elif legado_arquivo.startswith('chq'):
                            gerar_registro = getattr(chq2open, function)
                        else:
                            continue
                        a = CalcTempo()
                        registro_importado = gerar_registro(
                            db_registro=db_registro,
                            legado_log_id=legado_id,
                            dados=legado_conteudo,
                            company_com_nf_id=company_com_nf_id,
                            company_sem_nf_id=company_sem_nf_id,
                            empresa_legado=legado_empresa,
                            operacao=legado_operacao.lower(),
                            context=context
                        )
                        a.apurar('Importar arquivo: %s' % legado_arquivo)
                        if registro_importado:
                            cont += 1


                            if modulo in estatisticas.keys():
                                estatisticas[modulo] += 1
                            else:
                                estatisticas[modulo] = 1

                            db_registro.delete(table='legado_logs', where=[('id', '=', legado_id)])
                            db_registro.cr_integracao.connection.commit()
                            cr.commit()
                        else:
                            db_registro.cr_integracao.connection.rollback()

            modulos_importados = filter(lambda i: i[1], estatisticas.items())
            if modulos_importados:
                for k, v in modulos_importados:
                    _logger.info('%s - %d' % (k, v))

            _logger.info("REGISTROS IMPORTADOS: (TOTAL) %d" % cont)

        return True

    def lelg_habilitar_todos(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        log_dados = self.read(cr, uid, ids, ['lelg_arquivo_legado'], context=context, load=READ_LOAD)
        arquivo_legado = filter(None, map(lambda i: i.get('lelg_arquivo_legado') or False, log_dados))
        if arquivo_legado:
            log_ids = self.search(
                cr, uid,
                [('lelg_arquivo_legado', 'in', arquivo_legado), ('lelg_erro_registro', '=', True)],
                context=context
            )
            if log_ids:

                self.write(cr, uid, log_ids, {'lelg_erro_registro': False}, context)
        return True

    def lelg_unlink_log_conteudo(self, cr, uid, log_conteudo_ids, context=None):
        context = context and context.copy() or {}
        if log_conteudo_ids:
            context['lelgcon_force_unlink'] = True
            self.pool.get('legado.logs.conteudo').unlink(cr, uid, log_conteudo_ids, context)
        return True

    def lelg_create_conteudo_detalhado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        obj_log_conteudo = self.pool.get('legado.logs.conteudo')

        legado_log_dados = self.read(
            cr, uid, ids, ['lelg_arquivo_legado', 'lelg_conteudo', 'lelg_conteudo_ids'], context=context, load=READ_LOAD
        )

        coluna_modulos = IntegracaDescricaoColunas()

        for i in legado_log_dados:
            lelg_id = i['id']
            lelg_arquivo_legado = i.get('lelg_arquivo_legado')
            lelg_conteudo = i.get('lelg_conteudo')
            lelg_conteudo_ids = i.get('lelg_conteudo_ids')
            if lelg_conteudo:

                if lelg_conteudo_ids:
                    self.lelg_unlink_log_conteudo(cr, uid, lelg_conteudo_ids, context)

                lelg_conteudo = lelg_conteudo.replace('\n', '')
                # AO FAZER O SPLIT, GERA UMA POSICAO A MAIS. POR ISSO É UTILIZADO O [:-1]
                list_conteudo = lelg_conteudo.split('*|*')[:-1]

                for index, value in enumerate(list_conteudo, start=0):

                    value = value.replace('#', ' ').strip()

                    try:
                        info_coluna = getattr(coluna_modulos, lelg_arquivo_legado)
                    except Exception:
                        info_coluna = {}

                    vals_conteudo = {
                        'legcon_legado_log_id': lelg_id,
                        'legcon_posicao': index,
                        'legcon_valor': value,
                        'legcon_descricao': info_coluna.get(index) or '** Descricao nao encontrada para o valor **'
                    }
                    obj_log_conteudo.create(cr, uid, vals_conteudo, context)
        return True

    def lelg_unlink_conteudo_detalhado(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        legado_log_dados = self.read(
            cr, uid, ids, ['lelg_conteudo_ids'], context=context, load=READ_LOAD
        )
        for i in legado_log_dados:
            lelg_conteudo_ids = i.get('lelg_conteudo_ids')
            if lelg_conteudo_ids:
                self.lelg_unlink_log_conteudo(cr, uid, lelg_conteudo_ids, context=context)
        return True

    def lelg_gerar_novo_conteudo(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if isinstance(ids, (int, long)):
            ids = [ids]

        obj_log_conteudo = self.pool.get('legado.logs.conteudo')
        log_conteudo_ids = obj_log_conteudo.search(cr, uid, [('legcon_legado_log_id', 'in', ids)], context=context)
        if log_conteudo_ids:
            log_conteudo_dados = obj_log_conteudo.read(
                cr, uid, log_conteudo_ids, ['legcon_valor'], context=context, load=READ_LOAD
            )
            new_value = ''
            for i in log_conteudo_dados:
                new_value += '%s*|*' % (i.get('legcon_valor') or '')

            if new_value:
                self.write(cr, uid, ids, {'lelg_conteudo': new_value}, context)
            self.lelg_unlink_conteudo_detalhado(cr, uid, ids, context)
        return True

