#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import datetime
import psycopg2
from openerp.tools import config


class IntegracaoConnection:

    def __init__(self, cr, uid=False):
        self.cr = cr
        self.uid = uid
        self.cr_integracao = self.newConnection()

    def newConnection(self):
        cursor_dbname = filter(lambda i: i.startswith('dbname='), self.cr.connection.dsn.split(' '))
        dbname = cursor_dbname[0].split('=')[1]
        dbuser = config.options.get('db_user')
        dbpass = config.options.get('db_password') or None
        dbhost = config.options.get('db_host') or '127.0.0.1'
        dbport = config.options.get('db_port') or '5432'
        conexao = psycopg2.connect(database=dbname, user=dbuser, password=dbpass, host=dbhost,
                                   port=dbport, connect_timeout=3)
        _cr = conexao.cursor()
        return _cr

    def newConnection(self):
        cursor_dbname = filter(lambda i: i.startswith('dbname='), self.cr.connection.dsn.split(' '))
        dbname = cursor_dbname[0].split('=')[1]
        dbuser = config.options.get('db_user')
        dbpass = config.options.get('db_password') or None
        dbhost = config.options.get('db_host') or '127.0.0.1'
        dbport = config.options.get('db_port') or '5432'
        conexao = psycopg2.connect(database=dbname, user=dbuser, password=dbpass, host=dbhost,
                                   port=dbport, connect_timeout=3)
        _cr = conexao.cursor()
        return _cr

    def dictfetchall(self, cr):
        """Retorna todas as linhas de um cursor como um dicionario"""
        desc = cr.description
        return [dict(zip([col[0] for col in desc], row)) for row in cr.fetchall()]

    def select(self, table, fields=[], where=[], order_by=[], limit=0):

        #_cr = self.cr
        _cr = self.cr_integracao
        if not fields:
            sql_fields_from = "* from"
        else:
            fields = tuple(fields)
            str_fields = "%s, " * fields.__len__()
            sql_fields_from = str_fields % fields

            sql_fields_from += "from"
            sql_fields_from = sql_fields_from.replace(", from", " from")

        sql_where = ""
        if where:
            for i in where:
                column = i[0]
                operator = i[1]
                value = i[2]

                if isinstance(value, tuple):
                    str_value = "%s" % str(value).replace(',)', ')')
                else:
                    str_value = "'%s'" % str(value)
                sql_where += "%s %s %s and " % (column, operator, str_value)

        sql_order_by = ""
        if order_by:
            orders = tuple(order_by)
            str_order = "%s, " * orders.__len__()
            sql_order_by = str_order % orders

        sql_sentenca = "select %s %s" % (sql_fields_from, table)
        # WHERE
        if sql_where:
            sql_where += ";"
            sql_sentenca += " where %s" % sql_where.replace(" and ;", ";")
        else:
            sql_sentenca += ";"
        # ORDER BY
        if sql_order_by:
            sql_order_by += ';'
            sql_sentenca = sql_sentenca.replace(';', ' order by %s' % sql_order_by.replace(', ;', ';'))

        if limit:
            sql_sentenca = sql_sentenca.replace(';', ' limit %d;' % limit)

        try:
            _cr.execute(sql_sentenca)
            return self.dictfetchall(_cr)
        except:
            return False

    def insert(self, table, vals, many2many=False):
        """
        @param table: Tabela que recebera a insercao
        @type table: string
        @param vals: Colunas e respectivos valores a serem inseridos
        @type vals: dict
        @param many2many: Indica que a insercao sera feita em uma tabela m2m
        @type many2many: bool
        @return: ID da insercao ou True se a tabela for m2m
        @rtype: integer or bool
        """

        #_cr = self.cr
        _cr = self.cr_integracao
        if not many2many:
            if not 'create_date' in vals:
                vals['create_date'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            if self.uid and not 'create_uid' in vals:
                vals['create_uid'] = self.uid

        fields = tuple(vals.keys())
        values = tuple(vals.values())

        str_fields = "%s, " * fields.__len__()
        str_values = "'%s', " * values.__len__()

        sql_fields = str_fields % fields
        sql_values = str_values % values

        sql_sentenca = "insert into %s (%s) values (%s) " % (table, sql_fields, sql_values)
        sql_sentenca = sql_sentenca.replace(", )", ")")

        if not many2many:
            sql_sentenca += "returning id;"
        else:
            sql_sentenca += ";"

        sql_sentenca = sql_sentenca.replace("'null'", "null").replace("'Null'", "null")

        _cr.execute(sql_sentenca)

        if many2many:
            return True
        else:
            return (_cr.fetchone() or [False])[0]

    def update(self, table, values={}, where=[], many2many=False):
        """
        @param table: Tabela a ser atualizada
        @type table: string'
        @param values: Colunas e respectivos valores
        @type values: dicionario
        @param where: Clausulas para atualizacao
        @type where: lista de tuplas
        @return: True or False
        @rtype: boolean
        """

        #_cr = self.cr
        _cr = self.cr_integracao
        if not many2many:
            values['write_date'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')

        sql_fields_values = ""
        for column, value in values.items():
            sql_fields_values += "%s = '%s', " % (column, str(value))

        sql_where = ""
        if where:
            for i in where:
                column = i[0]
                operator = i[1]
                value = i[2]

                if isinstance(value, tuple):
                    str_value = "%s" % str(value).replace(',)', ')')
                else:
                    str_value = "'%s'" % value
                sql_where += "%s %s %s and " % (column, operator, str_value)

        sql_sentenca = "update %s set %s" % (table, sql_fields_values)
        if sql_where:
            sql_where += ";"
            sql_sentenca += "where %s" % sql_where.replace(" and ;", ";")
            sql_sentenca = sql_sentenca.replace(', where', ' where')
        else:
            sql_sentenca += ";"
            sql_sentenca = sql_sentenca.replace(', ;', ';')

        sql_sentenca = sql_sentenca.replace("'null'", "null").replace("'Null'", "null")

        _cr.execute(sql_sentenca)

        return True

    def delete(self, table, where=[]):
        """
        @param table: Tabela a ser deletada
        @type table: string
        @param where: Condicoes para a delecao
        @type where: lista de tuplas
        @return: True or False
        @rtype: bool
        """

        #_cr = self.cr
        _cr = self.cr_integracao
        sql_where = ""
        if where:
            for i in where:
                column = i[0]
                operator = i[1]
                value = i[2]

                if isinstance(value, tuple):
                    str_value = "%s" % str(value).replace(',)', ')')
                else:
                    str_value = "'%s'" % value
                sql_where += "%s %s %s and " % (column, operator, str_value)

        sql_sentenca = "delete from %s" % table

        if sql_where:
            sql_where += ";"
            sql_sentenca += " where %s" % sql_where.replace(" and ;", ";")
            sql_sentenca = sql_sentenca.replace(', where', ' where')
        else:
            sql_sentenca += ";"

        _cr.execute(sql_sentenca)

        return True


