#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from unicodedata import normalize
from logging import getLogger
import time
from datetime import datetime
from datetime import timedelta

_logger = getLogger("[LEGADO INTEGRACAO]")


class CalcTempo(object):
    def __init__(self):
        self.i = time.time()
    def apurar(self, mens):
        f = time.time()
        d = f - self.i
        _logger.info(str(mens) + '-Tempo:%s' % d)
        return d

# FUNCAO SERVE PARA VALIDAR O VALOR DOS CAMPOS QUE VEM DO FISCAL.
# NO EST03 (MOVIMENTO), EXISTEM VALORES QUE ESTAO VINDO COM 202020, QUE REPRESENTA ESPAÇO.
# SE NO VALOR ESTIVER COM 202020 ELE SERÁ ZERADO.
def funct_fis_validade_value(value):
    str_value = str(value).replace('.', '').replace(',', '')
    if '202020' in str_value:
        return 0
    else:
        return value


def funct_capitalize_all(texto):
    lista = texto.lower().split()
    lista_capitalize = []
    for a in lista:
        lista_capitalize.append(a.capitalize())
    b = ' '
    return b.join(lista_capitalize)


def funct_get_fonema(string, tamanho_fonema=5):

    def _calc_fonema(nome, _tam):
        nome += '     '
        inicio = ''
        fim = ''
        #Tratar inicio
        if nome[:1] in ('A', 'E', 'I', 'O', 'U'):
            inicio = nome[:1]
            nome = nome[1:]
        else:
            if nome[:2] in ('HA', 'HE', 'HI', 'HO', 'HU'):
                inicio = nome[1:2]
                nome = nome[2:]
            else:
                if nome[:1] == 'H':
                    nome = nome[1:]
                else:
                    if nome[:3] in ('EX', 'EXE', 'EXI', 'EXO', 'EXHU'):
                        inicio = 'Z'
                        nome = nome[3:]
                        # Tratar segunda posicao
        if not inicio:
            if nome[1:2] == 'R' and nome[2] not in ('A', 'E', 'I', 'O', 'U'):
                nome = nome[:1] + 'R' + nome[3:]
            else:
                if nome[1:3] in ('EXE', 'EXI', 'EXA', 'EXO', 'EXU'):
                    nome = nome[:1] + 'X' + nome[4:]
        nome = nome.strip()
        # Tratar ultima posicao
        if nome[-1:] == 'C':
            fim = 'K'
            nome = nome[:-1]
        else:
            if nome[-1:] == 'N':
                fim = 'M'
                nome = nome[:-1]
            else:
                if nome[-1:] == 'Z':
                    fim = 'S'
                    nome = nome[:-1]
                else:
                    if nome[-1:] == 'R':
                        fim = '2'
                        nome = nome[:-1]
                    else:
                        if nome[-1:] == 'X':
                            fim = 'X'
                            nome = nome[:-1]
                            # Tratar penultima posicao
        if not fim:
            if nome[-3:-1] == 'EX':
                nome = nome [:-3] + 'ks' + nome[-1:]

        nome = re_sub('SC[AOU]', 'sk', nome)
        nome = re_sub('C[AOU]', 'k', nome)
        nome = re_sub('SC[EI]', 's', nome)
        nome = re_sub('C[EI]', 's', nome)
        nome = nome.replace('CHR', 'k')
        nome = nome.replace('CH', 'x')
        nome = re_sub('C[^AEIOU]', 'k', nome) # C com consoante
        nome = re_sub('G[AOU]', 'g', nome)
        nome = re_sub('G[EI]', 'j', nome)
        nome = re_sub('GH[^AEIOU]', 'g', nome) # GH com consoante
        nome = nome.replace('LH', '1')
        nome = re_sub('L[AEIOU]', 'l', nome)
        nome = nome.replace('NH', '3')
        nome = nome.replace('PH', 'f')
        nome = nome.replace('Q', 'k')
        nome = nome.replace('RR', '2')
        nome = re_sub('[AEIOU]R[AEIOU]', 'r', nome) # vogal com R com vogal
        nome = re_sub('[^AEIOU]R[AEIOU]', 'r', nome) # consoante com R com vogal
        nome = nome.replace('SS', 's')
        nome = nome.replace('SH', 's')
        nome = nome.replace('TH', 't')
        nome = re_sub('W[AEIOU]', 'v', nome)
        nome = re_sub('W[^AEIOU]', '', nome)
        nome = re_sub('EX[EI]', 'x', nome)
        nome = re_sub('EX[AOU]', 'ks', nome)
        nome = re_sub('[AEIOU][CKGLRX]X', 's', nome)
        nome = re_sub('[DFMNPQSTVZ][AIOU]X', 'ks', nome)
        nome = nome.replace('SI', 'zi')
        nome = nome.replace('ZI', 'zi')
        nome = nome.replace('SA', 'za')
        nome = nome.replace('ZA', 'za')
        nome = nome.replace('Y', 'i')
        nome = nome.replace('H', '')
        # translate alpha chars in nome to soundex digits
        nome = nome.strip(' ')
        nome = inicio + nome + fim
        tam = len(nome)
        if tam > _tam:
            nome = nome[:_tam]
        else:
            if tam < _tam:
                nome = nome + ('0' * (_tam - tam))
        return nome.upper()

    try:
        string = normalize('NFKD', string.decode('utf-8')).encode('ASCII', 'ignore')
    except:
        string = normalize('NFKD', string).encode('ASCII', 'ignore')
    string = string.upper()
    string = string.replace('.', ' ')
    string = string.replace(' DE ', ' ').replace(' DA ', ' ').replace(' DAS ', ' ').replace(' DO ', ' '). \
        replace(' DOS ', ' ').replace(' E ', ' ').replace(' & ', ' ').replace(' LTDA', ' ')
    snd = ''
    inc = 0
    lst = string.split(' ')
    if lst:
        lst.sort()
    for x in lst:
        if len(x.strip()) < 2:  #nao gerar fonema para apenas uma letra
            continue
        snd += _calc_fonema(x, tamanho_fonema)
        inc += 1
        if not inc == len(lst):
            snd += ';'
    return snd


def funct_acha_numero(endereco):
    if endereco == '':
        return ['', '']
    _tam = len(endereco)
    _end = endereco
    _nro = ''
    while True:
        _tam = _tam - 1
        if _tam == 0:
            break
        _byte = endereco[_tam:_tam + 1]
        if _byte in (' ', ',', '/', '.', ';', '-'):
            _end = endereco[:_tam].strip()
            if _end[len(_end) - 1:] == ',':
                _end = _end[:len(_end) - 1]
            _nro = endereco[_tam + 1:].strip()
            break
        if (len(endereco) - _tam) > 5:
            _end = endereco[:-5]
            _nro = endereco[-5:]
            break
    return [_end, _nro]

# TODO: VERIFICAR FORMA DE RETIRAR ACENTOS
def funct_retira_acentos(letters):
    if isinstance(letters, basestring):
        nova_string = ''
        for x in letters:
            if ord(x) <= 127 and x != "'" and ord(x) != 92:
                nova_string = nova_string + x
        return nova_string
    return letters


def funct_formata_cpf_cnpj(cpf_cnpj, tipo):
    _cpf_cnpj = cpf_cnpj.lstrip('0')
    if _cpf_cnpj.__len__() < 5:
        _cpf_cnpj = ''
    else:
        if tipo == 'F':
            _cpf_cnpj = _cpf_cnpj.rjust(11, '0')
            from SigUtils import cpf
            a = cpf.Cpf()
            if not a.validar(_cpf_cnpj):
                _cpf_cnpj = ''
        else:
            from SigUtils import cnpj
            _cpf_cnpj = _cpf_cnpj.rjust(14, '0')
            a = cnpj.Cnpj()
            if not a.validar(_cpf_cnpj):
                _cpf_cnpj = ''
    return _cpf_cnpj


def funct_get_cfop_pai_sinteticos(cfop):

    cfop = str(cfop)
    digito_1 = cfop[0]
    digito_2 = cfop[1]
    digito_3 = cfop[2]

    sintetico_1 = '%s000' % digito_1
    sintetico_2 = '%s%s00' % (digito_1, digito_2)
    sintetico_3 = '%s%s%s0' % (digito_1, digito_2, digito_3)

    return filter(lambda i: i != cfop, tuple(set([sintetico_3, sintetico_2, sintetico_1])))


def funct_get_categoria_pai(categoria):
    # retira o unicode
    categoria = str(categoria).ljust(12, '0')

    grupo_1 = categoria[:3]
    grupo_2 = categoria[3:6]
    grupo_3 = categoria[6:9]

    categ_1 = '%s000000000' % grupo_1
    categ_2 = '%s%s000000' % (grupo_1, grupo_2)
    categ_3 = '%s%s%s000' % (grupo_1, grupo_2, grupo_3)

    return filter(lambda i: i != categoria, tuple(set([categ_3, categ_2, categ_1])))


def funct_converte_csosn(csosn):
    cst_csosn = {
        '101': '00',
        '102': '00',
        '103': '00',
        '203': '30',
        '201': '10',
        '202': '10',
        '500': '60',
        '900': '90',
        '300': '41',
        '400': '40'
    }
    cst = cst_csosn.get(csosn)
    return cst


def funct_time_2_float(hora, formato):
    """
    @Parametro: hora: string da hora
                formato: formato da hora, ex: '%H:%M'
    @Funcao:  Converte a hora passada como parametro para o formato datetime
              posteriormente converte a hora utilizando o timedelta
    @Retorno: Retorna a hora convertida quando hora e minuto validos,
              se nao retorna False
    """
    hora = datetime.strptime(hora, formato)
    if hora.hour > 23 or hora.minute > 59:
        return False
    else:
        hora_segundos = timedelta(hours=hora.hour, minutes=hora.minute)
        hora_convertida = (hora_segundos.total_seconds() / 60) / 60
    return hora_convertida

def funct_get_empresa(db_registro, tabela=None, args=None):
    company_id = db_registro.select(
            table='res_users', fields=['company_id'], where=[('id', '=', db_registro.uid)]
    )
    if not tabela:  # se nao tem tabela, nao interessa args. Retornar a empresa default
        return company_id
    if args: # Tendo args:1) se inteiro, usar a empresa, 2)tendo company_id em args, usar a empresa que veio no args. formato args ex:  '2' ou '[('company_id', '=', 2)]'
        try:
            company_id = int(args)
        except:
            for arg in args:
                if arg[0] == 'company_id':
                    company_id = arg[2]
                    break
    result = company_id
    res = db_registro.select(
            table='ir_model', fields=['id'], where=[('model', '=', tabela)]
    )
    if not res:  # Tabela nao localizada
        return result
    tabelaId = res[0]['id']
    if isinstance(company_id, (int, long)):
        company_id = (company_id,)
    res = db_registro.select(
            table='adm_empresa_objeto', fields=['id', 'empresa_a_usar_id'],
            where=[('empresa_escolhida_id', 'in', company_id), ('ir_model_id', '=', tabelaId)]
    )

    if res:
        result = res[0]['empresa_a_usar_id']
    if not args:  # retornar apenas o numero da empresa
        return result
    try:
        integer = int(args)
        args1 = result
    except:
        if not args: #Retornar lista com argumentos(args)
            args1 = [('company_id', '=', result)]
        else:
            args1 = [('company_id', '=', result)]
            for arg in args:
                if arg[0] != 'company_id':
                    args1.append(arg)
    return args1
