#!/usr/bin/python
# -*- encoding: utf-8 -*-

import signal
import psycopg2
import psycopg2.extras
from os import listdir as os_listdir, remove as os_remove, path as os_path
from csv import DictReader
from time import sleep
from psutil import process_iter
from SigUtils.geralogs import geraLogs
from traceback import format_exc as traceback_format_exc


DIRETORIO = './'

qted_integra = 0
encerrar = False

try:
    # VERIFICA SE EXISTE OUTRO PROCESSO integra.py EM EXECUCAO
    processos_ativos = filter(lambda i: i.is_running() and 'python' in i.name, process_iter())
    for i in processos_ativos:
        for p in i.cmdline:
            if 'integra.py' in p:
                qted_integra += 1
                if qted_integra > 1:
                    encerrar = True
                    break
except:
    pass

if encerrar:
    exit()

continuar = True


#daemon.daemonize()
# controle para parar o programa via kill -10
def handler(signum, frame):
    global continuar
    continuar = False

signal.signal(signal.SIGUSR1, handler)


# FUNCAO PARA GERAR ARQUIVO DE TEXTO COM ERRO
def integra_gravar_log(texto):
    geraLogs('integra.log',  texto, 'S', '../arquivos/logs/')


# DADOS DO ARQUIVO DE CONFIGURACAO
ark_config = False
try:
    arquivo_config = '../objetos/sigerp.cfg'
    ark_config = open(arquivo_config)
    arquivo = DictReader(ark_config)

    for i in arquivo:
        db = i['database']
        dbhostname = i['dbhostname']
        dbuser = i['dbuser']
        dbpasswd = i['dbpasswd']
except:
    gerar_importacao = False
    arquivo_config = False
    erro = traceback_format_exc()
    integra_gravar_log(erro)
finally:
    if ark_config:
        ark_config.close()

# FUNCAO PARA INSERIR OS REGISTROS NA TABELA
def integra_insert(cr, vals):

    table = 'legado_logs'
    _cr = cr

    fields = tuple(vals.keys())
    values = tuple(vals.values())

    str_fields = "%s, " * fields.__len__()
    str_values = "'%s', " * values.__len__()

    sql_fields = str_fields % fields
    sql_values = str_values % values

    sql_sentenca = "insert into %s (%s) values (%s);" % (table, sql_fields, sql_values)
    sql_sentenca = sql_sentenca.replace(", )", ")")


    try:
        _cr.execute(sql_sentenca)
        return True
    except:
        cr.connection.rollback()
        erro = traceback_format_exc()
        integra_gravar_log(erro)
        return False

while continuar:
    conn = False
    try:
        all_files = os_listdir('../arquivos/int/') or []
        list_dir = filter(lambda i: '.tmp' not in i, all_files)
        diretorio = '../arquivos/int/'
        all_files = os_listdir('../arquivos/int/') or []
        arquivos = filter(lambda i: '.tmp' not in i, all_files)
        data_arquivo = []
        list_dir = []
        """
        Abaixo e gerado uma lista de tuplas contendo o datetime no formato float da ultima data
        de modificacao do log (metodo getmtime) e o nome dos arquivos, apos isso e
        utilizado o sort para ordernar pela data dos arquivos"""
        for arquivo in arquivos:
            data_arquivo.append((os_path.getmtime(diretorio+arquivo), arquivo))
        data_arquivo.sort()
        for arq in data_arquivo:
            list_dir.append(arq[1])
        if list_dir:
            # CONEXAO COM O BANCO DE DADOS
            conn = psycopg2.connect(database=db, user=dbuser, password=dbpasswd, host=dbhostname,
                                    connect_timeout=3)
            cr = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

            list_dir.sort(key=lambda i: i[6:23])

            for i in list_dir:
                empresa = i[24:27]
                operacao = i[28:29].upper()
                modulo_legado = i[:5]

                try:
                    ark = open('../arquivos/int/' + i, 'r')
                    registros = ark.readlines()
                except:
                    erro = traceback_format_exc()
                    integra_gravar_log(erro)
                    continue

                data = i[6:14]
                hora = i[15:23]

                if operacao == 'X':
                    operacao = 'W'

                vals_legado_logs = {
                    'lelg_hora': hora,
                    'lelg_data': data,
                    'lelg_operacao': operacao,
                    'lelg_erro_registro': False,
                    'lelg_empresa_legado': empresa,
                    'lelg_arquivo_legado': modulo_legado,
                }

                for dados in registros:
                    dados = dados.replace("'", ' ')
                    dados = unicode(dados, 'utf-8', errors='ignore')
                    vals_legado_logs['lelg_conteudo'] = dados

                    integra_insert(cr, vals=vals_legado_logs)

                # TODAS AS OPERACOES SAO GRAVADAS EFETIVAMENTE NO BANCO DE DADOS
                cr.connection.commit()

                # FECHA O ARQUIVO QUE FOI ABERTO
                ark.close()

                # APAGA O ARQUIVO DE TEXTO
                if registros:
                    os_remove('../arquivos/int/%s' % i)
    except:
        erro = traceback_format_exc()
        integra_gravar_log(erro)
    finally:
        if conn:
            # FECHA A CONEXAO COM O BANCO DE DADOS
            conn.close()
    sleep(1)
