#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from SigUtils.datas import sigDatas
from dateutil.parser import parse
from psycopg2.extensions import TransactionRollbackError
from ..funcoes.integ_funcoes import funct_retira_acentos
from traceback import format_exc as traceback_format_exc
from ..funcoes.integ_funcoes import funct_get_empresa


def cax02_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip()
    # lista_dados[1] = lista_dados[1]
    # lista_dados[2] = lista_dados[2]
    lista_dados[3] = lista_dados[3].strip()
    # lista_dados[4] = lista_dados[4]
    # lista_dados[5] = lista_dados[5]
    lista_dados[6] = sigDatas(lista_dados[6]).getDataNoFormato('I', 'S', '-')
    # lista_dados[7] = lista_dados[7]
    lista_dados[8] = funct_retira_acentos(lista_dados[8].strip())
    lista_dados[9] = '%s.%s' % (lista_dados[9][:13], lista_dados[9][13:])
    lista_dados[10] = lista_dados[10]
    # lista_dados[11] = lista_dados[11]
    # lista_dados[12] = lista_dados[12]

    return lista_dados


# TESOURARIA MOVIMENTO ________________
def gerarCax02(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = cax02_format_data(dados)

    dados_origem = lista_dados[0]
    dados_cli_fornec = lista_dados[1]
    dados_controle = lista_dados[2]
    dados_serie = lista_dados[3]
    dados_sequencia = lista_dados[4]
    dados_operador = lista_dados[5]
    dados_data_movimento = lista_dados[6]
    dados_conta = lista_dados[7]
    dados_historico = lista_dados[8]
    dados_valor = lista_dados[9]
    dados_empresa_tipo = lista_dados[10].upper()
    # dados_operacao = lista_dados[11]
    # dados_centro_custo = lista_dados[12]

    obj_tesouraria_saldo = context.get('ctx_obj_tesouraia_saldo')

    try:

        try:
            date_data_movimento = parse(dados_data_movimento, yearfirst=True)
            dados_data_movimento = date_data_movimento.date().strftime('%Y-%m-%d')
        except:
            raise Exception('Cax02 - Data de Movimento invalida: %s' % dados_data_movimento)

        if dados_empresa_tipo != 'S':
            dados_empresa_tipo = 'C'

        tabela_legado_cax03 = 'cax03%s%s' % (dados_operador, dados_empresa_tipo)

        # PROCURA CONTA DO CAX03
        legado_cax03_conta_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=dados_operador,
            reg_tabela_legado=tabela_legado_cax03,
            reg_check_table='financeiro.tesouraria.contas',
            reg_empresa_tipo=dados_empresa_tipo
        )

        if not legado_cax03_conta_id:
            raise Exception(
                'Cax02: Conta nao encontrada na legado_controle para efeturar lancamento. '
                'Chave legado - %s, Tabela legado - %s' % (dados_operador, tabela_legado_cax03)
            )

        chave_cax02 = '%s%s%s%s%s' % (dados_origem, dados_cli_fornec, dados_controle, dados_serie, dados_sequencia)

        legado_cax02_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_cax02,
            reg_tabela_legado='cax02',
            reg_check_table='financeiro.tesouraria.movimento',
            reg_empresa_tipo=dados_empresa_tipo
        )

        if operacao in ('d', 'r'):
            if not legado_cax02_id:
                if operacao == 'd':
                    label_operacao = 'deletar'
                else:
                    label_operacao = 'alterar'
                raise Exception(
                    'Cax02 - Movimento nao encontrado na legado_controle para %s a Conta - %s' %
                    (label_operacao, chave_cax02)
                )

            else:
                fields_to_read = ['conta_id', 'data_movimento']
                tesouraria_mov_dados = db_registro.select(
                    table='financeiro_tesouraria_movimento', fields=fields_to_read, where=[('id', '=', legado_cax02_id)]
                )
                if tesouraria_mov_dados:
                    tesouraria_mov_dados = tesouraria_mov_dados[0]

                    mov_conta_id = tesouraria_mov_dados['conta_id']
                    mov_data_movimento = tesouraria_mov_dados['data_movimento']

                    # REMOVE O MOVIMENTO DO LEGADO, PORQUE VAI INSERIR NOVAMENTE
                    db_registro.delete(table='financeiro_tesouraria_movimento', where=[('id', '=', legado_cax02_id)])
                    db_registro.delete(
                        table='legado_controle', where=[('empresa', '=', empresa_legado),
                                                        ('chave_legado', '=', chave_cax02),
                                                        ('tabela_legado', '=', 'cax02')])
                    # RECALCULA O VALOR DO SALDO - METODO OPENERP
                    obj_tesouraria_saldo.fintsa_recalcular_saldos(
                        db_registro.cr, db_registro.uid, mov_conta_id, mov_data_movimento, context=context
                    )

                    if operacao == 'd':
                        return True
                    else:
                        legado_cax02_id = False

        if operacao in ('w', 'r'):

            if not legado_cax02_id:
                # CREDITO OU DEBITO
                credito_ou_debito = dados_valor[:1]
                if credito_ou_debito == '+':
                    credito_ou_debito = 'C'
                else:
                    credito_ou_debito = 'D'

                ano = date_data_movimento.year
                mes = date_data_movimento.month

                vals_tesouraria_movimento = {
                    'conta_id': legado_cax03_conta_id,
                    'data_movimento': date_data_movimento.strftime('%Y-%m-%d'),
                    'mes_competencia': mes,
                    'ano_competencia': ano,
                    'valor': dados_valor[1:],
                    'historico': dados_historico,
                    'conciliado': False,
                    'debito_credito': credito_ou_debito,
                    'state': 'A',
                    'conta_caixa_operacao': dados_conta
                }

                # CRIA UM MOVIMENTO NA TESOURARIA
                tesouraria_mov_id = db_registro.insert(
                    table='financeiro_tesouraria_movimento', vals=vals_tesouraria_movimento
                )

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='cax02',
                    reg_chave_legado=chave_cax02,
                    reg_chave_sigerp=tesouraria_mov_id,
                    reg_check_table='financeiro_tesouraria_movimento',
                    reg_empresa_tipo=dados_empresa_tipo
                )

                args_tesouraria_saldo = [
                    ('conta_id', '=', legado_cax03_conta_id), ('data_movimento', '<=', dados_data_movimento)
                ]

                tesouraria_saldo_dados = db_registro.select(
                    table='financeiro_tesouraria_saldos', fields=['id'], where=args_tesouraria_saldo,
                    order_by=['data_movimento desc'])

                if not tesouraria_saldo_dados:
                    raise Exception(
                        'Cax02 - Atualizacao de Saldo nao realizada. Saldo da tesouraria nao encontrado para a '
                        'conta_id - %s e data_movimento <= %s' % (legado_cax03_conta_id, dados_data_movimento))
                else:
                    # REALIZA A ATUALIZAO DE SALDO - METODO OPENERP
                    obj_tesouraria_saldo.atualizar_saldo(
                        db_registro.cr, db_registro.uid, legado_cax03_conta_id, dados_data_movimento, credito_ou_debito,
                        dados_valor[1:]
                    )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


def cax03_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip()
    lista_dados[1] = '%s.%s' % (lista_dados[1][:13], lista_dados[1][13:])
    lista_dados[2] = '%s.%s' % (lista_dados[2][:13], lista_dados[2][13:])
    lista_dados[3] = lista_dados[3].strip()
    lista_dados[4] = '%s.%s' % (lista_dados[4][:13], lista_dados[4][13:])
    lista_dados[5] = '%s.%s' % (lista_dados[5][:13], lista_dados[5][13:])
    lista_dados[6] = lista_dados[6].strip()
    lista_dados[7] = '%s.%s' % (lista_dados[7][:13], lista_dados[7][13:])
    lista_dados[8] = '%s.%s' % (lista_dados[8][:13], lista_dados[8][13:])
    lista_dados[9] = lista_dados[9].strip()
    lista_dados[10] = '%s.%s' % (lista_dados[10][:13], lista_dados[10][13:])
    lista_dados[11] = '%s.%s' % (lista_dados[11][:13], lista_dados[11][13:])
    lista_dados[12] = lista_dados[12].strip()
    lista_dados[13] = '%s.%s' % (lista_dados[13][:13], lista_dados[13][13:])
    lista_dados[14] = '%s.%s' % (lista_dados[14][:13], lista_dados[14][13:])
    lista_dados[15] = lista_dados[15].strip()
    lista_dados[16] = '%s.%s' % (lista_dados[16][:13], lista_dados[16][13:])
    lista_dados[17] = '%s.%s' % (lista_dados[17][:13], lista_dados[17][13:])
    lista_dados[18] = lista_dados[18].strip()
    lista_dados[19] = '%s.%s' % (lista_dados[19][:13], lista_dados[19][13:])
    lista_dados[20] = '%s.%s' % (lista_dados[20][:13], lista_dados[20][13:])
    lista_dados[21] = lista_dados[21].strip()
    lista_dados[22] = '%s.%s' % (lista_dados[22][:13], lista_dados[22][13:])
    lista_dados[23] = '%s.%s' % (lista_dados[23][:13], lista_dados[23][13:])
    lista_dados[24] = lista_dados[24].strip()
    lista_dados[25] = '%s.%s' % (lista_dados[25][:13], lista_dados[25][13:])
    lista_dados[26] = '%s.%s' % (lista_dados[26][:13], lista_dados[26][13:])
    # lista_dados[27] = lista_dados[27]

    return lista_dados


def gerarCax03(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = cax03_format_data(dados)

    dados_nome1 = lista_dados[0]
    dados_valor_c1 = lista_dados[1]
    dados_valor_s1 = lista_dados[2]
    dados_nome2 = lista_dados[3]
    dados_valor_c2 = lista_dados[4]
    dados_valor_s2 = lista_dados[5]
    dados_nome3 = lista_dados[6]
    dados_valor_c3 = lista_dados[7]
    dados_valor_s3 = lista_dados[8]
    dados_nome4 = lista_dados[9]
    dados_valor_c4 = lista_dados[10]
    dados_valor_s4 = lista_dados[11]
    dados_nome5 = lista_dados[12]
    dados_valor_c5 = lista_dados[13]
    dados_valor_s5 = lista_dados[14]
    dados_nome6 = lista_dados[15]
    dados_valor_c6 = lista_dados[16]
    dados_valor_s6 = lista_dados[17]
    dados_nome7 = lista_dados[18]
    dados_valor_c7 = lista_dados[19]
    dados_valor_s7 = lista_dados[20]
    dados_nome8 = lista_dados[21]
    dados_valor_c8 = lista_dados[22]
    dados_valor_s8 = lista_dados[23]
    dados_nome9 = lista_dados[24]
    dados_valor_c9 = lista_dados[25]
    dados_valor_s9 = lista_dados[26]
    dados_encerramento_mes_ano = lista_dados[27]

    obj_tesouraria_conta = context.get('ctx_obj_tesouraia_conta')

    try:

        if dados_encerramento_mes_ano == '000000':
            dados_encerramento_mes_ano = '012000'

        data_inclusao = sigDatas('01%s' % dados_encerramento_mes_ano).getDataNoFormato('N', 'S', '/')
        operacao_lower = operacao.lower()

        # VERIFICANDO AS EMPRESAS
        tesouraria_conta_com_nf_company_id = funct_get_empresa(db_registro,
                                                               'financeiro.tesouraria.contas',
                                                               company_com_nf_id)
        if company_com_nf_id != company_sem_nf_id:
            tesouraria_conta_sem_nf_company_id = funct_get_empresa(db_registro,
                                                                   'financeiro.tesouraria.contas',
                                                                   company_sem_nf_id)

        else:
            tesouraria_conta_sem_nf_company_id = tesouraria_conta_com_nf_company_id

        if operacao_lower == 'w':
            conta_ids = []

            def cax03_registrar_conta(conta_numero, conta_nome, conta_valor_com_nf, conta_valor_sem_nf):
                if conta_nome:
                    ## ____________________________CONTA PARA A EMPRESA QUE UTILIZA NF__________________________________
                    conta_com_nf_cadastrada_dados = db_registro.select(
                        table='financeiro_tesouraria_contas', fields=['id'], where=[('apelido', 'ilike', conta_nome)]
                    )
                    # CONTA JA ESTA CADASTRADA NO SISTEMA
                    if conta_com_nf_cadastrada_dados:
                        tesouraria_nf_conta_id = conta_com_nf_cadastrada_dados[0]['id']
                    else:

                        # CRIA UMA CONTA NA TESOURARIA_CONTAS
                        vals_tesouraria_conta_nf = {
                            'banco_caixa': 'C',
                            'company_id': tesouraria_conta_com_nf_company_id,
                            'apelido': conta_nome,
                            'active': True
                        }
                        tesouraria_nf_conta_id = db_registro.insert(
                            table='financeiro_tesouraria_contas', vals=vals_tesouraria_conta_nf
                        )

                    conta_ids.append(tesouraria_nf_conta_id)

                    new_empresa_legado_C = db_registro.get_empresa_legado_relacionada(
                        reg_empresa_legado=empresa_legado, reg_objeto_nome='financeiro.tesouraria.contas'
                    )

                    args_legado_controle = [
                        ('empresa', '=', new_empresa_legado_C), ('tabela_legado', '=', 'cax03%sC' % conta_numero),
                        ('chave_legado', '=', conta_numero)
                    ]
                    legado_controle_dados = db_registro.select(
                        table='legado_controle', fields=['id'], where=args_legado_controle)

                    if legado_controle_dados:
                        legado_controle_id = legado_controle_dados[0]['id']
                        # ATUALIZA A LEGADO CONTROLE
                        db_registro.update(
                            table='legado_controle',
                            values={'chave_sigerp': tesouraria_nf_conta_id, 'empresa': new_empresa_legado_C},
                            where=[('id', '=', legado_controle_id)]
                        )
                    else:
                        # CRIA A LEGADO_CONTROLE
                        db_registro.create_legado_controle(
                            reg_empresa_legado=empresa_legado,
                            reg_tabela_legado='cax03%sC' % conta_numero,
                            reg_chave_legado=conta_numero,
                            reg_chave_sigerp=tesouraria_nf_conta_id,
                            reg_check_table='financeiro.tesouraria.contas',
                        )

                    # VERIFICA SE O SALDO JA ESTA CRIADO
                    tesousaria_saldo_dados = db_registro.select(
                        table='financeiro_tesouraria_saldos', fields=['id'],
                        where=[('conta_id', '=', tesouraria_nf_conta_id), ('data_movimento', '=', data_inclusao)]
                    )
                    if not tesousaria_saldo_dados:
                        # CRIA O SALDO PARA A CONTA
                        vals_tesouraria_saldo_nf = {
                            'conta_id': tesouraria_nf_conta_id,
                            'data_movimento': data_inclusao,
                            'fintsa_saldo_inicial': conta_valor_com_nf,
                            'fintsa_saldo_final': conta_valor_com_nf
                        }
                        db_registro.insert(table='financeiro_tesouraria_saldos', vals=vals_tesouraria_saldo_nf)

                    ## __________________________CONTA PARA A EMPRESA QUE NAO UTILIZA NF________________________________
                    conta_sem_nf_cadastrada_dados = db_registro.select(
                        table='financeiro_tesouraria_contas', fields=['id'],
                        where=[('apelido', 'ilike', '%s(S)' % conta_nome)]
                    )

                    if conta_sem_nf_cadastrada_dados:
                        tesouraria_sem_nf_conta_id = conta_sem_nf_cadastrada_dados[0]['id']
                    else:
                        # CRIA UMA CONTA NA TESOURARIA_CONTAS
                        vals_tesouraria_conta_sem_nf = {
                            'banco_caixa': 'C',
                            'company_id': tesouraria_conta_sem_nf_company_id,
                            'apelido': '%s(S)' % conta_nome,
                            'active': True
                        }
                        tesouraria_sem_nf_conta_id = db_registro.insert(
                            table='financeiro_tesouraria_contas', vals=vals_tesouraria_conta_sem_nf
                        )

                    conta_ids.append(tesouraria_sem_nf_conta_id)

                    new_empresa_legado_S = db_registro.get_empresa_legado_relacionada(
                        reg_empresa_legado=empresa_legado, reg_objeto_nome='financeiro.tesouraria.contas',
                        reg_empresa_tipo='S'
                    )

                    args_legado_controle_sem_nf = [
                        ('empresa', '=', new_empresa_legado_S), ('tabela_legado', '=', 'cax03%sS' % conta_numero),
                        ('chave_legado', '=', conta_numero)
                    ]
                    sem_nf_legado_controle_dados = db_registro.select(
                        table='legado_controle', fields=['id'], where=args_legado_controle_sem_nf)

                    if sem_nf_legado_controle_dados:
                        sem_nf_legado_controle_id = sem_nf_legado_controle_dados[0]['id']
                        # ATUALIZA A LEGADO CONTROLE
                        db_registro.update(
                            table='legado_controle',
                            values={'chave_sigerp': tesouraria_sem_nf_conta_id, 'empresa': new_empresa_legado_S},
                            where=[('id', '=', sem_nf_legado_controle_id)]
                        )
                    else:
                        # CRIA A LEGADO_CONTROLE
                        db_registro.create_legado_controle(
                            reg_empresa_legado=empresa_legado,
                            reg_tabela_legado='cax03%sS' % conta_numero,
                            reg_chave_legado=conta_numero,
                            reg_chave_sigerp=tesouraria_sem_nf_conta_id,
                            reg_check_table='financeiro.tesouraria.contas',
                            reg_empresa_tipo='S'
                        )

                    # VERIFICA SE O SALDO JA ESTA CRIADO
                    tesousaria_saldo_dados2 = db_registro.select(
                        table='financeiro_tesouraria_saldos', fields=['id'],
                        where=[('conta_id', '=', tesouraria_sem_nf_conta_id), ('data_movimento', '=', data_inclusao)]
                    )
                    if not tesousaria_saldo_dados2:
                        # CRIA O SALDO PARA A CONTA
                        vals_tesouraria_saldo_sem_nf = {
                            'conta_id': tesouraria_sem_nf_conta_id,
                            'data_movimento': data_inclusao,
                            'fintsa_saldo_inicial': conta_valor_sem_nf,
                            'fintsa_saldo_final': conta_valor_sem_nf,
                        }

                        db_registro.insert(table='financeiro_tesouraria_saldos', vals=vals_tesouraria_saldo_sem_nf)

            # CRICAO DE CONTAS
            cax03_registrar_conta('1', dados_nome1, dados_valor_c1, dados_valor_s1)
            cax03_registrar_conta('2', dados_nome2, dados_valor_c2, dados_valor_s2)
            cax03_registrar_conta('3', dados_nome3, dados_valor_c3, dados_valor_s3)
            cax03_registrar_conta('4', dados_nome4, dados_valor_c4, dados_valor_s4)
            cax03_registrar_conta('5', dados_nome5, dados_valor_c5, dados_valor_s5)
            cax03_registrar_conta('6', dados_nome6, dados_valor_c6, dados_valor_s6)
            cax03_registrar_conta('7', dados_nome7, dados_valor_c7, dados_valor_s7)
            cax03_registrar_conta('8', dados_nome8, dados_valor_c8, dados_valor_s8)
            cax03_registrar_conta('9', dados_nome9, dados_valor_c9, dados_valor_s9)

            if conta_ids:
                db_registro.cr_integracao.connection.commit()

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False