#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from SigUtils.datas import sigDatas
from dateutil.parser import parse
from psycopg2.extensions import TransactionRollbackError
from ..funcoes.integ_funcoes import funct_retira_acentos
from traceback import format_exc as traceback_format_exc
from ..funcoes.integ_funcoes import funct_get_empresa

def ccb01_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip()
    lista_dados[1] = lista_dados[1].strip()
    # lista_dados[2] = lista_dados[2]
    lista_dados[3] = lista_dados[3].strip()
    lista_dados[4] = '%s.%s' % (lista_dados[4][:13], lista_dados[4][13:])
    lista_dados[5] = '%s.%s' % (lista_dados[5][:13], lista_dados[5][13:])
    lista_dados[6] = '%s.%s' % (lista_dados[6][:13], lista_dados[6][13:])
    # lista_dados[7] = lista_dados[7]
    lista_dados[8] = lista_dados[8].upper()
    lista_dados[9] = lista_dados[9].strip()
    lista_dados[10] = lista_dados[10].strip()
    lista_dados[11] = lista_dados[11].strip()
    # lista_dados[12] = lista_dados[12]
    # lista_dados[13] = lista_dados[13]
    # lista_dados[14] = lista_dados[14]

    return lista_dados


# TESOURARIA CONTAS
def gerarCcb01(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = ccb01_format_data(dados)

    dados_apelido = lista_dados[0]
    dados_conta = lista_dados[1]
    dados_banco = lista_dados[2]
    dados_agencia = lista_dados[3]
    dados_saldo = lista_dados[4]
    dados_limite_especial = lista_dados[5]
    # dados_juros = lista_dados[6]
    dados_csnf = lista_dados[7]
    dados_integra = lista_dados[8]
    # dados_gerente = lista_dados[9]
    # dados_telefone = lista_dados[10]
    # dados_celular = lista_dados[11]
    # dados_pagina = lista_dados[12]
    # dados_integra_disponibilidade = lista_dados[13]
    # dados_conta_ctb = lista_dados[14]

    try:
        if dados_csnf != 'S':
            dados_csnf = 'C'
        if dados_csnf == 'S':
            company_id = company_sem_nf_id
        else:
            company_id = company_com_nf_id

        if dados_integra in ('S', 'Y'):
            dados_integra = True
        else:
            dados_integra = False

        tesouraria_conta_dados = db_registro.select(
            table='financeiro_tesouraria_contas', fields=['id'], where=[('apelido', 'ilike', dados_apelido)]
        )

        # BANCO
        adm_banco_dados = db_registro.select(
            table='adm_banco', fields=['id'], where=[('admbco_codigo_bc', '=', dados_banco)]
        )
        if not adm_banco_dados:
            raise Exception('Ccb01: Banco nao cadastrado. Codigo no Banco Central - %s' % dados_banco)

        banco_id = adm_banco_dados[0]['id']

        # AGENCIA
        if not dados_agencia:
            agencia_id = False
        else:
            adm_agencia_dados = db_registro.select(
                table='adm_banco_agencia', fields=['id'], where=[('name', 'ilike', dados_agencia)]
            )
            if adm_agencia_dados:
                # AGENCIA ENCONTRADA
                agencia_id = adm_agencia_dados[0]['id']
            else:
                numero_agencia = re_sub('[^0-9]', '', dados_agencia)
                vals_agencia = {
                    'admbag_banco_id': banco_id,
                    'name': dados_agencia
                }
                if numero_agencia:
                    vals_agencia['admbag_codigo_agencia'] = numero_agencia
                    # CRIA UMA AGENCIA
                agencia_id = db_registro.insert(table='adm_banco_agencia', vals=vals_agencia)

        # VERIFICANDO AS EMPRESAS
        tesouraria_conta_company_id = funct_get_empresa(db_registro, 'financeiro.tesouraria.contas',
                                                        company_com_nf_id)

        operacao_lower = operacao.lower()
        if operacao_lower == 'w':

            if tesouraria_conta_dados:
                return True

            vals_tesouraria_conta = {
                'banco_caixa': 'B',
                'banco_id': banco_id,
                'company_id': tesouraria_conta_company_id,
                'apelido': dados_apelido,
                'nro_conta': dados_conta,
                'limite_especial': dados_limite_especial,
                'integra_fluxo': dados_integra,
                'active': True
            }
            if agencia_id:
                vals_tesouraria_conta['agencia_id'] = agencia_id

            tesouraria_conta_id = db_registro.insert(table='financeiro_tesouraria_contas', vals=vals_tesouraria_conta)

            vals_tesouraria_saldo = {
                'conta_id': tesouraria_conta_id,
                'data_movimento': '2000-01-01',
                'fintsa_saldo_inicial': dados_saldo,
                'fintsa_saldo_final': dados_saldo
            }

            db_registro.insert(table='financeiro_tesouraria_saldos', vals=vals_tesouraria_saldo)

        elif operacao_lower in ('r', 'd'):
            if not tesouraria_conta_dados:
                raise Exception('Ccb01: Conta nao encontrada para alterar. Apelido - %s' % dados_apelido)

            conta_id = tesouraria_conta_dados[0]['id']

            # ATUALIZA OS DADOS DA CONTA
            if operacao_lower == 'r':
                values = {
                    'banco_caixa': 'B',
                    'banco_id': banco_id,
                    'company_id': tesouraria_conta_company_id,
                    'apelido': dados_apelido,
                    'nro_conta': dados_conta,
                    'limite_especial': dados_limite_especial,
                    'integra_fluxo': dados_integra,
                    'active': True
                }
                if agencia_id:
                    values['agencia_id'] = agencia_id

                db_registro.update(table='financeiro_tesouraria_contas', values=values, where=[('id', '=', conta_id)])

            # DELETA A CONTA
            elif operacao_lower == 'd':
                db_registro.delete(table='financeiro_tesouraria_contas', where=[('id', '=', conta_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


def ccb02_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip()
    # lista_dados[1] = lista_dados[1]
    # lista_dados[2] = lista_dados[2]
    lista_dados[3] = lista_dados[3].strip()
    # lista_dados[4] = lista_dados[4]
    lista_dados[5] = lista_dados[5].strip()
    # lista_dados[6] = lista_dados[6]
    lista_dados[7] = sigDatas(lista_dados[7]).getDataNoFormato('I', 'S', '-')
    # lista_dados[8] = lista_dados[8]
    lista_dados[9] = '%s.%s' % (lista_dados[9][:13], lista_dados[9][13:])
    # lista_dados[10] = lista_dados[10]
    # lista_dados[11] = lista_dados[11]
    lista_dados[12] = funct_retira_acentos(lista_dados[12].strip())

    return lista_dados


# TESOURARIA MOVIMENTO
def gerarCcb02(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = ccb02_format_data(dados)

    dados_origem = lista_dados[0]
    dados_cliente_fornecedor = lista_dados[1]
    dados_controle = lista_dados[2]
    dados_serie = lista_dados[3]
    dados_sequencia = lista_dados[4]
    dados_apelido = lista_dados[5]
    # dados_documento = lista_dados[6]
    dados_data_movimento = lista_dados[7]
    # dados_tipo = lista_dados[8]
    dados_valor = lista_dados[9]
    dados_conferencia = lista_dados[10]
    dados_operacao = lista_dados[11]
    dados_historico = lista_dados[12]

    # self.pool.get('financeiro.tesouraria.saldos')
    obj_tesouraria_saldo = context.get('ctx_ccb02_obj_tesouraia_saldo')

    try:

        try:
            date_data_movimento = parse(dados_data_movimento, yearfirst=True)
        except:
            raise Exception('Ccb02 - Data de Movimento invalida: %s' % dados_data_movimento)

        # BUSCA A CONTA POR MEIO DO APELIDO
        tesouraria_conta_dados = db_registro.select(
            table='financeiro_tesouraria_contas', fields=['id', 'company_id'], where=[('apelido', '=', dados_apelido)]
        )
        if not tesouraria_conta_dados:
            raise Exception('Ccb02: Conta nao encontrada para efetuar lancamento. Apelido - %s' % dados_apelido)

        chave_ccb02 = '%s%s%s%s%s' % \
                      (dados_origem, dados_cliente_fornecedor, dados_controle, dados_serie, dados_sequencia)

        legado_ccb02_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_ccb02,
            reg_tabela_legado='ccb02',
            reg_check_table='financeiro_tesouraria_movimento',
        )

        operacao_lower = operacao.lower()

        if operacao_lower in ('d', 'r'):
            if not legado_ccb02_id:
                if operacao_lower == 'd':
                    label_operacao = 'deletar'
                else:
                    label_operacao = 'alterar'
                raise Exception(
                    'Ccb02: Chave %s nao encontrada na legado_controle para %s' % (chave_ccb02, label_operacao)
                )
            tesouraria_movimento_dados = db_registro.select(table='financeiro_tesouraria_movimento',
                                                            fields=['conta_id', 'data_movimento'],
                                                            where=[('id', '=', legado_ccb02_id)])
            if not tesouraria_movimento_dados:
                raise Exception(
                    'Nao foi possivel alterar/deletar o movimento. '
                    'ID - %d nao encontrado na tabela financeiro_tesouraria_movimento' % legado_ccb02_id)

            mov_conta_id = tesouraria_movimento_dados[0]['conta_id']
            mvo_data_movimento = tesouraria_movimento_dados[0]['data_movimento']

            # REMOVE O MOVIMENTO DO LEGADO, PORQUE VAI INSERIR NOVAMENTE
            db_registro.delete(table='financeiro_tesouraria_movimento', where=[('id', '=', legado_ccb02_id)])
            # ATUALIZA O LEGADO CONTROLE
            args_legado_controle = [
                ('empresa', '=', empresa_legado), ('chave_legado', '=', chave_ccb02), ('tabela_legado', '=', 'ccb02')
            ]
            db_registro.delete_legado_controle(
                reg_args=args_legado_controle,
                reg_check_table='financeiro_tesouraria_movimento',
            )

            # RECALCULA O VALOR DO SALDO - METODO OPENERP
            obj_tesouraria_saldo.fintsa_recalcular_saldos(
                db_registro.cr, db_registro.uid, mov_conta_id, mvo_data_movimento, context=context
            )
            legado_ccb02_id = False
            if operacao_lower == 'd':
                return True

        if operacao_lower in ('w', 'r'):
            if not legado_ccb02_id:

                if dados_valor[:1] == '+':
                    credito_ou_debito = 'C'
                else:
                    credito_ou_debito = 'D'

                if dados_conferencia in ('S', 'Y', 'C'):
                    dados_conferencia = True
                else:
                    dados_conferencia = False

                conta_id = tesouraria_conta_dados[0]['id']

                mes = date_data_movimento.month
                ano = date_data_movimento.year

                vals_movimento = {
                    'conta_id': conta_id,
                    'data_movimento': dados_data_movimento,
                    'mes_competencia': mes,
                    'ano_competencia': ano,
                    'valor': dados_valor[1:],
                    'historico': dados_historico,
                    'conciliado': dados_conferencia,
                    'debito_credito': credito_ou_debito,
                    'state': 'A',
                    'conta_caixa_operacao': dados_operacao
                }
                # CRIA UM MOVIMENTO NA TESOURARIA
                movimento_id = db_registro.insert(table='financeiro_tesouraria_movimento', vals=vals_movimento)

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='ccb02',
                    reg_chave_legado=chave_ccb02,
                    reg_chave_sigerp=movimento_id,
                    reg_check_table='financeiro_tesouraria_movimento',
                )

                args_tesouraria_saldo = [
                    ('conta_id', '=', conta_id), ('data_movimento', '<=', dados_data_movimento)
                ]

                tesouraria_saldo_dados = db_registro.select(
                    table='financeiro_tesouraria_saldos', fields=['id'], where=args_tesouraria_saldo,
                    order_by=['data_movimento desc'])

                if not tesouraria_saldo_dados:
                    raise Exception('Ccb02: Saldo da tesouraria nao encontrado para a conta_id - %s e movimento <= %s'
                                    % (conta_id, dados_data_movimento))
                else:
                    # REALIZA A ATUALIZAO DE SALDO - METODO OPENERP
                    obj_tesouraria_saldo.atualizar_saldo(
                        db_registro.cr, db_registro.uid, conta_id, dados_data_movimento, credito_ou_debito,
                        dados_valor[1:]
                    )
        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False
