#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from SigUtils.datas import sigDatas
from psycopg2.extensions import TransactionRollbackError
from ..funcoes.integ_funcoes import funct_retira_acentos
from traceback import format_exc as traceback_format_exc
from ..funcoes.integ_funcoes import funct_get_empresa


def chq02_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    # lista_dados[0] = lista_dados[0]
    # lista_dados[1] = lista_dados[1]
    lista_dados[2] = lista_dados[2]
    lista_dados[3] = lista_dados[3]
    lista_dados[4] = sigDatas(lista_dados[4]).getDataNoFormato('I', 'S', '-')
    lista_dados[5] = lista_dados[5]
    lista_dados[6] = lista_dados[6]
    lista_dados[7] = '%s.%s' % (lista_dados[7][:10], lista_dados[7][10:])
    lista_dados[8] = sigDatas(lista_dados[8]).getDataNoFormato('I', 'S', '-')
    # lista_dados[9] = '%s.%s' % (lista_dados[9][:10], lista_dados[9][10:])
    # lista_dados[10] = sigDatas(lista_dados[10]).getDataNoFormato('I', 'S', '-')
    lista_dados[11] = lista_dados[11]
    # lista_dados[12] = lista_dados[12]
    # lista_dados[13] = lista_dados[13].strip()
    lista_dados[14] = '%s.%s' % (lista_dados[14][:10], lista_dados[14][10:])

    return lista_dados


# CHEQUES _____________________________________
def gerarChq02(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id,
               empresa_legado, operacao, context=None):

    try:

        lista_dados = chq02_format_data(dados)

        # dados_data = lista_dados[0]
        # dados_hora = lista_dados[1]
        dados_cliente_cpf = lista_dados[2]
        dados_duplicata = lista_dados[3]
        dados_emissao = lista_dados[4]
        dados_banco = lista_dados[5]
        dados_agencia = lista_dados[6]
        dados_valor = lista_dados[7]
        dados_vencimento = lista_dados[8]
        # dados_devolucao = lista_dados[9]
        # dados_apelido_deposito = lista_dados[10]
        dados_loja = lista_dados[11]
        # dados_cheque_negociado = lista_dados[12]
        # dados_pdv = lista_dados[13]

        documento = '%s-%s' % (str(dados_loja), str(dados_duplicata))

        if dados_emissao == '0000-00-00':
            raise Exception(
                'Data Emissao invalida: %s' % dados_emissao
            )

        if dados_vencimento == '0000-00-00':
            raise Exception(
                'Data Vencimento invalida: %s' % dados_vencimento
            )

        if not float(dados_valor):
            raise Exception('Valor zerado: %s' % dados_valor)

        dados_csnf = 'C'

        pessoa_id = False
        cpf_cnpj_pessoa = dados_cliente_cpf.lstrip('0')
        if len(cpf_cnpj_pessoa) == 1:
            cpf_cnpj_pessoa = dados_cliente_cpf
        cpf_cnpj_pessoa = '%' + cpf_cnpj_pessoa
        pessoa_dados = db_registro.select(
            table='adm_pessoa', fields=['id'],
            where=[('cpf_cnpj', 'like', cpf_cnpj_pessoa)])
        if pessoa_dados:
            pessoa_id = pessoa_dados[0].get('id')

        if not pessoa_id:
            raise Exception('Chq02 - Cliente nao cadastrado no SIGERP. CPF/CNPJ: %s' %
                            dados_cliente_cpf)

        documento_dados = db_registro.select(table='documento_documento_tipo', fields=['id'],
                                             where=[('sigla', '=', 'CHQ')])
        if not documento_dados:
            raise Exception('Tipo de Documento com sigla CHQ nao encontrado')

        doc_tipo_id = documento_dados[0]['id']

        titulo_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=dados_cliente_cpf + documento,
            reg_tabela_legado='chq02',
            reg_check_table='financeiro_titulo',
            reg_empresa_tipo=dados_csnf
        )

        local_cobranca_dados = db_registro.select(
            table='financeiro_local_cobranca', fields=['id'], where=[('tipo', '=', 'C')]
        )
        if not local_cobranca_dados:
            raise Exception('Chq02 - Nenhum local de cobranca do tipo Carteira encontrado')
        else:
            local_cobranca_id = local_cobranca_dados[0]['id']

        DICT_NATUREZA = {'1': 'Cobranca simples', '2': 'Desconto', '3': 'Caucao'}

        # BUSCA MOEDA DO TIPO R$
        moeda_dados = db_registro.select(table='adm_moeda', fields=['id'],
                                         where=[('sigla', '=', 'R$')])
        if not moeda_dados:
            raise Exception('Chq02 - Moeda com sigla R$ nao cadastrada')
        else:
            moeda_id = moeda_dados[0]['id']

        if operacao != 'w':
            if not titulo_id:
                raise Exception('Chq02 - Cheque nao cadastrado: %s. Cliente: %s' %
                                (documento, dados_cliente_cpf))
        elif operacao == 'd':
            # TODO: NAO IMPLEMENTADO AINDA. O TITULO NAO DEVE SER EXLCUIDO E SIM CANCELADO.
            # TODO: AGUARDAR O CRE03 PARA SABER
            return True

        # VERIFICANDO AS EMPRESAS
        titulo_company_id = funct_get_empresa(db_registro, 'financeiro.titulo', company_com_nf_id)
        try:
            if float(dados_valor) < 0:
                tipo_titulo = 'P'
                dados_valor = float(dados_valor).__abs__()
            else:
                tipo_titulo = 'R'
        except ValueError:
            raise Exception('Chq02 - Valor do cheque invalido')

        vals_titulo = {
            'company_id': titulo_company_id,
            'doc_tipo_id': doc_tipo_id,
            'pessoa_id': pessoa_id,
            'documento': documento,
            'parcela': 0,
            'local_cobranca_id': local_cobranca_id,
            'tipo_titulo': '%s' % tipo_titulo,
            'state': 'A',
            'data_emissao': dados_emissao,
            'data_vencimento': dados_vencimento,
            'data_prorrogacao': dados_vencimento,
            'tipo_sacado': 'C',
            'vr_original': dados_valor,
            'saldo': dados_valor,
            'adm_moeda_id': moeda_id,
            'tipo_lancamento': 'M',
            'fintit_chq_banco': dados_banco,
            'fintit_chq_agencia': dados_agencia,
            'fintit_chq_emitente': dados_cliente_cpf,
            'fintit_cheque': True
        }

        # NATUREZA DE COBRANCA
        natureza_cob_dados = db_registro.select(
            table='financeiro_natureza_cobranca', fields=['id'],
            where=[('name', 'ilike', DICT_NATUREZA['1'])]
        )
        if natureza_cob_dados:
            natureza_cob_id = natureza_cob_dados[0]['id']
            vals_titulo['natureza_cobranca_id'] = natureza_cob_id

        if not titulo_id:
            if operacao == 'w':
                # CRIA UM TITULO
                titulo_id = db_registro.insert(table='financeiro_titulo', vals=vals_titulo)

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='chq02',
                    reg_chave_legado=dados_cliente_cpf + documento,
                    reg_chave_sigerp=titulo_id,
                    reg_check_table='financeiro_titulo',
                    reg_empresa_tipo=dados_csnf
                )
        else:
            # ATUALIZA OS DADOS DO TITULO
            # o titulo já existindo, o status não deve ser mudado
            x = vals_titulo.pop('state')
            db_registro.update(table='financeiro_titulo', values=vals_titulo,
                               where=[('id', '=', titulo_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False
