#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from traceback import format_exc as traceback_format_exc
from dateutil.parser import parse
from psycopg2.extensions import TransactionRollbackError


# INDEXADOR
def gerarGei01(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

        dados_moeda_sigla = (lista_dados[0] or '').upper().replace(' ', '')
        dados_moeda_data_movimento = lista_dados[1]
        dados_moeda_valor = lista_dados[2].replace(' ', '0')

        dados_moeda_valor = '%s.%s' % (dados_moeda_valor[:-2], dados_moeda_valor[-2:])

        try:
            str_data_movimento = parse(dados_moeda_data_movimento, yearfirst=True).strftime('%Y-%m-%d')
        except:
            raise Exception('Gei01: Data Movimento Invalida: %s' % dados_moeda_data_movimento)

        adm_moeda_dados = db_registro.select(
            table='adm_moeda', fields=['id'], where=[('sigla', 'ilike', dados_moeda_sigla)]
        )
        if adm_moeda_dados:
            moeda_id = adm_moeda_dados[0]['id']
        else:
            moeda_id = False

        if operacao in ('w', 'r'):
            # SE A MOEDA NAO EXISTIR É CRIADA
            if not moeda_id:
                vals_moeda = {
                    'active': True,
                    'name': dados_moeda_sigla,
                    'sigla': dados_moeda_sigla,
                }
                moeda_id = db_registro.insert(table='adm_ moeda', vals=vals_moeda)

            args_moeda_valor = [('moeda_id', '=', moeda_id), ('data_movimento', '=', str_data_movimento)]
            adm_moeda_valor_dados = db_registro.select(table='adm_moeda_valor', fields=['id'], where=args_moeda_valor)

            # SE A MOEDA VALOR EXISTIR É ATUALIZADA
            if adm_moeda_valor_dados:
                moeda_valor_id = adm_moeda_valor_dados[0]['id']
                values_moeda_valor = {
                    'valor': dados_moeda_valor
                }
                db_registro.update(table='adm_moeda_valor', values=values_moeda_valor, where=[('id', '=', moeda_valor_id)])
            # SENAO EH CRIADA
            else:
                vals_moeda_valor = {
                    'moeda_id': moeda_id,
                    'data_movimento': str_data_movimento,
                    'valor': dados_moeda_valor,
                }
                db_registro.insert(table='adm_moeda_valor', vals=vals_moeda_valor)

        elif operacao == 'd':
            if not moeda_id:
                raise Exception(
                    'Gei01: Moeda nao encontrada para realizar a exclusao do valor. Sigla - %s' % dados_moeda_sigla
                )
            args_moeda_valor = [('moeda_id', '=', moeda_id), ('data_movimento', '=', str_data_movimento)]
            db_registro.delete(table='adm_moeda_valor', where=args_moeda_valor)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


