#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from traceback import format_exc as traceback_format_exc
from psycopg2.extensions import TransactionRollbackError
from ..funcoes.integ_funcoes import funct_get_cfop_pai_sinteticos


def sis09_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    # lista_dados[0] = lista_dados[0]
    lista_dados[1] = lista_dados[1].strip()

    return lista_dados


# LOCAL DE COBRANCA
def gerarSis09(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = sis09_format_data(dados)

    dados_numero_banco = lista_dados[0]
    dados_nome = lista_dados[1]

    try:
        adm_banco_dados = db_registro.select(
            table='adm_banco', fields=['admbco_pessoa_id'], where=[('admbco_codigo_bc', '=', dados_numero_banco)]
        )
        if adm_banco_dados:
            pessoa_id = adm_banco_dados[0]['admbco_pessoa_id']
        else:
            # CRIA UM PARTNER
            vals_partner = {
                'name': dados_nome,
                'notification_email_send': 'comment'
            }
            partner_id = db_registro.insert(table='res_partner', vals=vals_partner)
            # CRIA UMA PESSOA
            vals_pessoa = {
                'nome': dados_nome,
                'pessoa_tipo': 'J',
                'active': True,
                'admpes_jur_tipo_empresa': 'B',
                'admpes_jur_tipo_atividade': 'F',
                'partner_id': partner_id
            }
            # BUSCA O CODIGO PARA A PESSOA
            ir_sequence_dados = db_registro.select(
                table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
            )
            sequence_id = ir_sequence_dados[0]['id']
            sequence_codigo = ir_sequence_dados[0]['number_next']
            sequence_proximo_codigo = sequence_codigo + 1
            # ATUALIZA O IR_SEQUENCE
            db_registro.update(
                table='ir_sequence', values={'number_next': sequence_proximo_codigo}, where=[('id', '=', sequence_id)]
            )
            vals_pessoa['codigo'] = sequence_codigo

            pessoa_id = db_registro.insert(table='adm_pessoa', vals=vals_pessoa)

        operacao_lower = operacao.lower()

        if operacao_lower == 'w':
            if adm_banco_dados:
                return True

            if dados_numero_banco == '000':
                return True

            vals_banco = {
                'admbco_pessoa_id': pessoa_id,
                'admbco_codigo_bc': dados_numero_banco
            }
            # CRIA UM BANCO
            db_registro.insert(table='adm_banco', vals=vals_banco)

            vals_finan_local_cobranca = {
                'pessoa_id': pessoa_id,
                'tipo': 'B',
                'descricao': dados_nome,
                'active': True
            }
            # CRIA LOCAL DE COBRANCA
            db_registro.insert(table='financeiro_local_cobranca', vals=vals_finan_local_cobranca)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


def sis26_format_data(dados):

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = str(lista_dados[0])
    lista_dados[1] = lista_dados[1].strip()
    lista_dados[2] = lista_dados[2].strip()
    # lista_dados[3] = lista_dados[3]
    # lista_dados[4] = lista_dados[4]
    # lista_dados[5] = lista_dados[5]
    # lista_dados[6] = lista_dados[6]

    return lista_dados


# CFOP
def gerarSis26(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    lista_dados = sis26_format_data(dados)

    dados_cfop = lista_dados[0]
    # dados_natureza = lista_dados[1]
    dados_descricao = lista_dados[2]
    dados_entrada_saida = lista_dados[3]
    dados_analitico_sintetico = lista_dados[4]
    dados_tipo_movimento = lista_dados[5]
    dados_transacao_tipo = lista_dados[6]

    try:
        if dados_cfop[:1] == '0':
            raise Exception('Sis26: CFOP comeca com Zero - %s' % dados_cfop)

        if dados_tipo_movimento == 'A':
            movimento_tipo = 'C'
        elif dados_tipo_movimento == 'B':
            movimento_tipo = 'T'
        elif dados_tipo_movimento == 'C':
            movimento_tipo = 'D'
        else:
            movimento_tipo = 'U'

        if dados_transacao_tipo in ('C', 'I', 'M', 'S', 'O'):
            pass
        else:
            dados_transacao_tipo = 'C'

        operacao_lower = operacao.lower()

        fiscal_cfop_dados = db_registro.select(table='fiscal_cfop', fields=['id'], where=[('codigo', '=', dados_cfop)])
        # CFOP JA ENCONTRA-SE CADASTRADO NA BASE
        if fiscal_cfop_dados:
            return True

        # CFOP NAO ESTA CADASTRADO NA BASE
        elif operacao_lower == 'w':

            vals_cfop = {
                'descricao': dados_descricao,
                'tipo': dados_entrada_saida,
                'analitico_sintetico': dados_analitico_sintetico,
                'movimento_tipo': movimento_tipo,
                'transacao_tipo': dados_transacao_tipo,
                'codigo': dados_cfop,
                #OBS - No gestao nao existe este campo. Aqui sera definido como Mercadoria / Produto (P).
                # Apos fazer a importacao, tera que ser ajustado manualmente.
                'produto_servico_tipo': 'P'
            }

            list_sinteticos = funct_get_cfop_pai_sinteticos(dados_cfop)
            if list_sinteticos:
                args_cfop = [('codigo', 'in', list_sinteticos), ('analitico_sintetico', '=', 'S')]
                sintetico_dados = db_registro.select(
                    table='fiscal_cfop', fields=['id'], where=args_cfop, order_by=['codigo desc']
                )
                if sintetico_dados:
                    vals_cfop['parent_id'] = sintetico_dados[0]['id']

            db_registro.insert(table='fiscal_cfop', vals=vals_cfop)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False
