#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from traceback import format_exc as traceback_format_exc
from dateutil.parser import parse
from psycopg2.extensions import TransactionRollbackError
from ..exceptions import EstoqueParametroError
from ..modulos.est2open import est_format_produto_codigo
from SigUtils.datas import calcVencimento
from datetime import datetime
from ..funcoes.integ_funcoes import funct_get_empresa


def gerarVen13_format_data(dados, context=None):
    if not isinstance(context, dict):
        context = {}

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip().rjust(10, '0')
    lista_dados[1] = lista_dados[1].strip() or None
    lista_dados[2] = lista_dados[2].strip()
    lista_dados[3] = lista_dados[3].strip()
    lista_dados[4] = lista_dados[4].strip()
    lista_dados[5] = lista_dados[5].strip()
    #lista_dados[6] = lista_dados[6].strip()
    #lista_dados[7] = lista_dados[7].strip()
    lista_dados[8] = '%s.%s' % (lista_dados[8][:-3], lista_dados[8][-3:])
    lista_dados[9] = '%s.%s' % (lista_dados[9][:-3], lista_dados[9][-3:])
    lista_dados[10] = lista_dados[10].strip()
    lista_dados[11] = '%s.%s' % (lista_dados[11][:-2], lista_dados[11][-2:])
    #lista_dados[12] = lista_dados[12].strip()
    #lista_dados[13] = lista_dados[13].strip()
    lista_dados[14] = lista_dados[14].strip()
    lista_dados[15] = '%s.%s' % (lista_dados[15][:-2], lista_dados[15][-2:])

    return lista_dados


# PEDIDOS
def gerarVen13(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao,
               context=None):

    if not isinstance(context, dict):
        context = {}

    try:
        lista_dados = gerarVen13_format_data(dados, context)

        dados_numero_pedido = lista_dados[0]
        dados_data_emissao = lista_dados[1]
        dados_cliente_codigo = lista_dados[2]
        dados_situacao = lista_dados[3]  # (A = A Faturar, C = Cancelado, F = Faturado)
        dados_vendedor_codigo = lista_dados[4]
        dados_vendedor_externo_codigo = lista_dados[5]
        #dados_cfop = lista_dados[6]
        #dados_natureza = lista_dados[7]
        dados_desconto = lista_dados[8]
        dados_valor_total = lista_dados[9]
        dados_observacao = lista_dados[10]
        dados_despesas_acessorias = lista_dados[11]
        #dados_transportador_codigo = lista_dados[12]
        #dados_transportador_placa = lista_dados[13]
        dados_frete_tipo = lista_dados[14]  # (1 - Emitente, 2 - Destinatario)
        dados_frete_valor = lista_dados[15]

        # VERIFICA SE O PEDIDO ESTA CRIADO. SE ESTIVER, RETORNA O ID DO PEDIDO NO ERP.
        legado_ven13_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=dados_numero_pedido,
            reg_tabela_legado='ven13',
            reg_check_table='estoquepedidos_pedido_capa',
        )
        if legado_ven13_id:
            args_estado_pedido = [('id', '=', legado_ven13_id)]
            state = db_registro.select(table='estoquepedidos_pedido_capa', fields=['state'],
                                       where=args_estado_pedido)
            if state[0].get('state') == 'F':
                raise Exception('Ven13 - Pedido faturado nao pode ser alterado!')
        if operacao in ('w', 'r'):
            str_data_emissao = ''
            if dados_data_emissao:
                try:
                    str_data_emissao = parse(dados_data_emissao, yearfirst=True).strftime('%Y-%m-%d')
                except Exception:
                    pass

            if not str_data_emissao:
                raise Exception('Ven13 - Data de Emissao invalida. Data - %s' % dados_data_emissao)

            pessoa_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=dados_cliente_codigo,
                reg_tabela_legado='cre01',
                reg_check_table='adm_pessoa',
            )
            if not pessoa_id:
                raise Exception(
                    'Ven13 - Cliente nao encontrado na legado_controle. Chave_legado - %s, Tabela_legado - cre01' %
                    dados_cliente_codigo
                )

            # STATUS DO PEDIDO
            if dados_situacao == 'C':
                pedido_state = 'C'
            elif dados_situacao == 'F':
                pedido_state = 'F'
            else:
                pedido_state = 'V'

            # COMPANY DO PEDIDO
            pedido_company_id = funct_get_empresa(db_registro, 'estoquepedidos.pedido.capa',
                                           company_com_nf_id)

            # TIPO DE FRETE DO PEDIDO
            if dados_frete_tipo == '2':
                pedido_frete_tipo = 'F'
            else:
                pedido_frete_tipo = 'C'

            # ID DO VENDEDOR E VENDEDOR EXTERNO
            vendedor_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=dados_vendedor_codigo,
                reg_tabela_legado='est11',
                reg_check_table='vendas_vendedor',
            )

            vendedor_externo_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=dados_vendedor_externo_codigo,
                reg_tabela_legado='est11',
                reg_check_table='vendas_vendedor',
            )

            vals_pedido = {
                'tipo': 'V',
                'name': dados_numero_pedido,
                'state': pedido_state,
                'frete': pedido_frete_tipo,
                'vr_frete': dados_frete_valor,
                'vr_seguro': 0,
                'pessoa_id': pessoa_id,
                'observacao': dados_observacao,
                'company_id': pedido_company_id,
                'vr_desconto': dados_desconto,
                'valor_total': dados_valor_total,
                'vr_acrescimo': 0,
                'numero_pedido': dados_numero_pedido,
                'exportado_frente': False,
                'vr_despesas_acessorias': dados_despesas_acessorias,
            }
            if not legado_ven13_id:
                vals_pedido['data_emissao'] = str_data_emissao
            if vendedor_id:
                vals_pedido['vendedor_id'] = vendedor_id
            if vendedor_externo_id:
                vals_pedido['vendedor_externo_id'] = vendedor_externo_id

            if not legado_ven13_id:
                pedido_id = db_registro.insert(table='estoquepedidos_pedido_capa', vals=vals_pedido)
            else:
                pedido_id = legado_ven13_id
                args_pedido_id = [('id', '=', legado_ven13_id)]

                db_registro.update(table='estoquepedidos_pedido_capa', values=vals_pedido, where=args_pedido_id)

            # CRIACAO/ATUALIZACAO
            if not legado_ven13_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='ven13',
                    reg_chave_legado=dados_numero_pedido,
                    reg_chave_sigerp=pedido_id,
                    reg_check_table='estoquepedidos_pedido_capa',
                )
            else:
                args_ven13 = [
                    ('tabela_legado', '=', 'ven13'), ('empresa', '=', empresa_legado),
                    ('chave_legado', '=', dados_numero_pedido)
                ]

                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': pedido_id},
                    reg_args=args_ven13,
                    reg_check_table='estoquepedidos_pedido_capa',
                )
        elif operacao == 'd':

            if not legado_ven13_id:
                raise Exception(
                    'Ven13 - Pedido nao encontrado na legado controle. Chave_legado (Codigo Cliente Gestao) - %s, '
                    'Tabela_legado - ven13' % dados_numero_pedido
                )
            # DELETA O PEDIDO
            db_registro.delete(table='estoquepedidos_pedido_capa', where=[('id', '=', legado_ven13_id)])

            # DELETE A LEGADO CONTROLE
            args_ven13 = [
                ('tabela_legado', '=', 'ven13'), ('empresa', '=', empresa_legado),
                ('chave_legado', '=', dados_numero_pedido)
            ]
            db_registro.delete_legado_controle(
                reg_args=args_ven13, reg_check_table='estoquepedidos_pedido_capa'
            )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


def gerarVen14_format_data(db_registro, dados, empresa_legado, company_id, context=None):
    if not isinstance(context, dict):
        context = {}

    dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
    try:
        _codNumerico = dict_company_utilizar_cod_numerico[company_id]
    except KeyError:
        raise EstoqueParametroError(value=empresa_legado)

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip().rjust(10, '0')
    lista_dados[1] = lista_dados[1].strip()
    lista_dados[2] = lista_dados[2].strip()

    if _codNumerico:
        lista_dados[3] = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[3],
                                                   context=context, company_com_nf_id=company_id)
    else:
        lista_dados[3] = lista_dados[3].strip()
    print lista_dados[3]

    lista_dados[4] = lista_dados[4].strip()
    lista_dados[5] = lista_dados[5].strip()
    lista_dados[6] = lista_dados[6].strip()
    #lista_dados[7] = lista_dados[7]
    #lista_dados[8] = lista_dados[8]
    #lista_dados[9] = lista_dados[9]
    lista_dados[10] = '%s.%s' % (lista_dados[10][:-3], lista_dados[10][-3:])
    lista_dados[11] = '%s.%s' % (lista_dados[11][:-3], lista_dados[11][-3:])
    lista_dados[12] = '%s.%s' % (lista_dados[12][:-3], lista_dados[12][-3:])
    lista_dados[13] = '%s.%s' % (lista_dados[13][:-3], lista_dados[13][-3:])

    return lista_dados


# PEDIDO ITENS
def gerarVen14(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao,
               context=None):
    """
    delete from legado_logs
    where substring(lelg_conteudo from 0 for 11)::int < numero do pedido

    sql para deletar movimentos do pedido que nao tenham mais capa.
    Na base do santa helena há mais de 1 milhao de registros
    """


    if not isinstance(context, dict):
        context = {}

    try:
        lista_dados = gerarVen14_format_data(db_registro, dados, empresa_legado, company_com_nf_id, context)

        dados_numero_pedido = lista_dados[0]
        dados_sequencia = lista_dados[1]
        dados_kit_sequencia = lista_dados[2]
        dados_produto_codigo = lista_dados[3]
        dados_grade = lista_dados[4]
        dados_local = lista_dados[5]
        dados_quantidade = lista_dados[6]
        #dados_tributacao_codigo = lista_dados[7]
        #dados_tributacao_tipo = lista_dados[8]
        #dados_cfop = lista_dados[9]
        dados_valor_liquido = lista_dados[10]
        dados_desconto = lista_dados[11]
        #dados_despesas_acessorias = lista_dados[12]
        #dados_frete = lista_dados[13]

        chave_ven14 = '%s%s%s' % (dados_numero_pedido, dados_sequencia, dados_kit_sequencia)

        # VERIFICA SE O PEDIDO ESTA CRIADO. SE ESTIVER, RETORNA O ID DO PEDIDO NO ERP.
        legado_ven13_pedido_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=dados_numero_pedido,
            reg_tabela_legado='ven13',
            reg_check_table='estoquepedidos_pedido_capa',
        )

        if not legado_ven13_pedido_id:
            raise Exception(
                'Ven14 - Pedido nao encontrado na legado_controle. Chave_legado (Numero Pedido) - %s, '
                'Tabela_legado - ven13' % dados_numero_pedido
            )

        # PEDITO_ITEM_ID (LEGADO CONTROLE)
        legado_ven14_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_ven14,
            reg_tabela_legado='ven14',
            reg_check_table='estoquepedidos_pedido_item',
        )

        if operacao in ('w', 'r'):
            # COMPANY DO PEDIDO
            pedido_item_company_id = funct_get_empresa(db_registro, 'estoquepedidos.pedido.item',
                                                  company_com_nf_id)
            # VALIDACAO DA QUANTIDADE INFORMADA
            try:
                float_quantidade = float(dados_quantidade) / 1000
            except:
                raise Exception('Ven14 - Quantidade invalida. Quantidade - %s' % dados_quantidade)

            if not float_quantidade:
                raise Exception('Ven14 - Quantidade igual deve ser maior que zero. Quantidade - %s' % dados_quantidade)

            """
            Quando o dav e feito por transferencia no gestao, e gerado um registro w para cada item
            com todos os valores zerados, posteriormente e gerado um registro r para cada item
            contendo os valores reais dos itens, com isto quando a operacao for w e o item
            nao tiver preco liquido o codigo retorna True para nao acomular logs de erro.
            """
            if not float(dados_valor_liquido):
                if operacao == 'w':
                    return True

            valor_unitario = float(dados_valor_liquido) / float_quantidade

            # LOCAL_ID
            local_company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)
            local_dados = db_registro.select(
                table='estoque_local',
                fields=['id'],
                where=[('numero', '=', dados_local), ('company_id', '=', local_company_id)]
            )
            if not local_dados:
                raise Exception(
                    'Est03: Local nao cadastrado. Numero - %s, Company_id - %s' % (str(dados_local), str(local_company_id))
                )
            local_id = local_dados[0]['id']

            # PRODUTO_ID
            produto_company_id = funct_get_empresa(db_registro, 'estoque.produto',
                                                   company_com_nf_id)
            produto_dados = db_registro.select(
                table='estoque_produto',
                fields=['id'],
                where=[('company_id', '=', produto_company_id), ('codigo', '=', dados_produto_codigo)]
            )
            if produto_dados:
                produto_id = produto_dados[0]['id']
            else:
                raise Exception(
                    'Ven14 - Item nao cadastrado. Codigo - %s, Company_id - %s' %
                    (str(dados_produto_codigo), str(pedido_item_company_id))
                )

            # GRADE_ID
            if not dados_grade:
                dados_grade = 'UN'

            args_grade = [('produto_id', '=', produto_id), ('grade', '=', dados_grade)]
            grade_dados = db_registro.select(table='estoque_produto_grade', fields=['id'], where=args_grade)
            if not grade_dados:
                raise Exception(
                    'Est03: Grade nao cadastrada. Produto_id - %s, Grade - %s'
                    % (str(produto_id), str(dados_grade))
                )

            grade_id = grade_dados[0]['id']

            # PRODUTO_EMBALAGEM_ID
            local_locacao_company_id = funct_get_empresa(
                db_registro, 'estoque.local.locacao', company_com_nf_id
            )

            args_produto_embalagem = [('produto_id',  '=', produto_id),
                                      ('local_id', '=', local_id),
                                      ('company_id', '=', local_locacao_company_id),
                                      ('grade_id', '=', grade_id)]

            estoque_local_locacao = db_registro.select(
                table='estoque_local_locacao',
                fields=['estllc_produto_embalagem_id'],
                where=args_produto_embalagem)
            estpro_embalagem_id = estoque_local_locacao[0]['estllc_produto_embalagem_id']
            produto_embalagem_dados = db_registro.select(
                table='estoque_produto_embalagem', fields=['id', 'embalagem_id'],
                where=[('id', '=', estpro_embalagem_id)]
            )
            if not produto_embalagem_dados:
                raise Exception(
                    'Ven14 - Nenhuma embalagem encontra-se cadastrada para o produto.'
                    'Produto_id - %s, Codigo Produto - %s. Tabela - estoque_produto_embalagem'
                    % (str(produto_id), dados_produto_codigo)
                )

            tuple_id_embalagem_id = map(lambda i: (i['id'], i['embalagem_id']), produto_embalagem_dados)

            embalagem_ids = tuple(map(lambda i: i[1], tuple_id_embalagem_id))

            embalagem_dados = db_registro.select(
                table='estoque_embalagem',
                fields=['id'],
                where=[('id', 'in', embalagem_ids)],
                order_by=['quantidade_na_embalagem desc'],
                limit=1
            )
            embalagem_id = embalagem_dados[0]['id']
            produto_embalagem_id = filter(lambda i: i[1] == embalagem_id, tuple_id_embalagem_id)[0][0]

            # DADOS PARA CRIACAO OU ATUALIZACAO
            vals_pedido_item = {
                'pedido_id': legado_ven13_pedido_id,
                'qtde_pedida': float_quantidade,
                'qtde_atendida': float_quantidade,
                'vr_unitario': valor_unitario,
                'vr_desconto': dados_desconto,
                'produto_id': produto_id,
                'grade_id': grade_id,
                'local_id': local_id,
                'produto_embalagem_id': produto_embalagem_id,
                'valor_total': dados_valor_liquido,
                'pei_preco_venda': valor_unitario,
            }

            if not legado_ven14_id:
                pedido_item_id = db_registro.insert(table='estoquepedidos_pedido_item', vals=vals_pedido_item)
            else:
                pedido_item_id = legado_ven14_id

                args_ven14 = [('id', '=', legado_ven14_id)]
                db_registro.update(table='estoquepedidos_pedido_item', values=vals_pedido_item, where=args_ven14)
            obj_estoquepedidos_pedido_item = context.get('ctx_estped_obj_estoquepedidos_pedido_item')
            db_registro.cr_integracao.connection.commit()
            obj_estoquepedidos_pedido_item.write(
                db_registro.cr, db_registro.uid, [pedido_item_id],
                {'write_date': datetime.now().strftime('%Y-%m-%d %H:%M:%S')},
                context=context
            )

            # CRIACAO/ATUALIZACAO
            if not legado_ven14_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='ven14',
                    reg_chave_legado=chave_ven14,
                    reg_chave_sigerp=pedido_item_id,
                    reg_check_table='estoquepedidos_pedido_item',
                )
            else:
                args_ven14 = [
                    ('tabela_legado', '=', 'ven14'), ('empresa', '=', empresa_legado),
                    ('chave_legado', '=', chave_ven14)
                ]

                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': pedido_item_id},
                    reg_args=args_ven14,
                    reg_check_table='estoquepedidos_pedido_item',
                )
        elif operacao == 'd':

            if not legado_ven14_id:
                raise Exception(
                    'Ven13 - Pedido Item nao encontrado na legado controle. Chave_legado - %s, '
                    'Tabela_legado - ven14' % chave_ven14
                )
            # DELETA O PEDIDO
            db_registro.delete(table='estoquepedidos_pedido_item', where=[('id', '=', legado_ven14_id)])

            # DELETE A LEGADO CONTROLE
            args_ven13 = [
                ('tabela_legado', '=', 'ven14'), ('empresa', '=', empresa_legado),
                ('chave_legado', '=', chave_ven14)
            ]
            db_registro.delete_legado_controle(
                reg_args=args_ven13, reg_check_table='estoquepedidos_pedido_item'
            )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


def gerarVen18_format_data(dados, context=None):
    if not isinstance(context, dict):
        context = {}

    lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

    lista_dados[0] = lista_dados[0].strip().rjust(10, '0')
    lista_dados[1] = lista_dados[1].strip()
    lista_dados[2] = lista_dados[2].strip()
    lista_dados[3] = lista_dados[3].strip().upper()
    lista_dados[4] = lista_dados[4].strip().upper()
    lista_dados[5] = lista_dados[5]
    lista_dados[6] = lista_dados[6]
    lista_dados[7] = lista_dados[7]
    lista_dados[8] = lista_dados[8]
    lista_dados[9] = lista_dados[9]
    lista_dados[10] = lista_dados[10]
    lista_dados[11] = lista_dados[11]
    lista_dados[12] = lista_dados[12]
    lista_dados[13] = lista_dados[13]
    lista_dados[14] = lista_dados[14]
    lista_dados[15] = lista_dados[15]
    lista_dados[16] = lista_dados[16]
    lista_dados[17] = lista_dados[17]
    lista_dados[18] = lista_dados[18]
    lista_dados[19] = lista_dados[19]
    lista_dados[20] = lista_dados[20]
    lista_dados[21] = lista_dados[21]
    lista_dados[22] = lista_dados[22]
    lista_dados[23] = lista_dados[23]
    lista_dados[24] = lista_dados[24]
    lista_dados[25] = lista_dados[25]
    lista_dados[26] = lista_dados[26]
    lista_dados[27] = lista_dados[27]
    lista_dados[28] = lista_dados[28]

    return lista_dados


# PEDIDO PARCELAS
def gerarVen18(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao,
               context=None):

    if not isinstance(context, dict):
        context = {}

    try:

        lista_dados = gerarVen18_format_data(dados, context)

        dados_numero_pedido = lista_dados[0]
        dados_sequencia = lista_dados[1]
        dados_nome_finalizadora = lista_dados[2]  # nome da finalizadora
        dados_avista_prazo = lista_dados[3]  # V = avista, P = a prazo, S = sem condicoes
        dados_prazo_tipo = lista_dados[4]
        dados_parcela = {'1': [lista_dados[5], lista_dados[6]],
                         '2': [lista_dados[7], lista_dados[8]],
                         '3': [lista_dados[9], lista_dados[10]],
                         '4': [lista_dados[11], lista_dados[12]],
                         '5': [lista_dados[13], lista_dados[14]],
                         '6': [lista_dados[15], lista_dados[16]],
                         '7': [lista_dados[17], lista_dados[18]],
                         '8': [lista_dados[19], lista_dados[20]],
                         '9': [lista_dados[21], lista_dados[22]],
                         '10': [lista_dados[23], lista_dados[24]],
                         '11': [lista_dados[25], lista_dados[26]],
                         '12': [lista_dados[27], lista_dados[28]]}

        chave_ven18 = '%s%s' % (dados_numero_pedido, dados_sequencia)

        # VERIFICA SE O PEDIDO ESTA CRIADO. SE ESTIVER, RETORNA O ID DO PEDIDO NO ERP.
        legado_ven13_pedido_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=dados_numero_pedido,
            reg_tabela_legado='ven13',
            reg_check_table='estoquepedidos_pedido_capa',
        )

        if not legado_ven13_pedido_id:
            raise Exception(
                'Ven14 - Pedido nao encontrado na legado_controle. Chave_legado (Numero Pedido) - %s, '
                'Tabela_legado - ven13' % dados_numero_pedido
            )

        if not dados_prazo_tipo:
                dados_prazo_tipo = 'DD'
        # BUSCA A DATA DE EMISSAO DO PEDIDO
        args_pedido_capa = [
            ('id', '=', legado_ven13_pedido_id)
        ]
        pedido_capa_dados = db_registro.select(table='estoquepedidos_pedido_capa', fields=['data_emissao'],
                                               where=args_pedido_capa)
        data_emissao = pedido_capa_dados[0]['data_emissao']
        # dict_prazo_valor: dicionario cuja chave eh a data de vencimento e seu conteudo o valor da parcela
        dict_prazo_valor = {}
        total_condicao = 0
        for dado in dados_parcela:
            try:
                prazo = int(dados_parcela.get(dado)[0])
                parcela_valor = float(dados_parcela.get(dado)[1]) / 100
            except:
                parcela_valor = 0
                prazo = 0
            if parcela_valor:
                data_vencimento = calcVencimento(data_emissao, prazo, tipo_prazo=dados_prazo_tipo,
                                                 diaUtil='N', diaFechamento=0)
                dict_prazo_valor[data_vencimento] = [dado, parcela_valor]
                total_condicao += parcela_valor
        # PEDIDO_CONDICAO_ID (LEGADO CONTROLE)
        legado_ven18_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_ven18,
            reg_tabela_legado='ven18',
            reg_check_table='estoquepedidos_pedido_condicao',
        )

        if operacao in ('w', 'r'):
            if operacao == 'w' and total_condicao == 0.00:
                return True
            if dados_avista_prazo == 'C':
                dados_avista_prazo = 'S'
            if dados_avista_prazo == 'P':
                dados_avista_prazo = 'C'

            # BUSCA A CONDICAO DE PAGAMENTO POR MEIO DO TIPO DE PRAZO E TIPO
            args_adm_condicao = [
                ('tipo_prazo', '=', dados_prazo_tipo), ('tipo', '=', dados_avista_prazo),
                ('name', 'ilike', dados_nome_finalizadora)  # TODO: QNTDE DE PARCELAS
            ]
            adm_condicao_dados = db_registro.select(table='adm_condicao', fields=['id'], where=args_adm_condicao)
            if not adm_condicao_dados:
                raise Exception(
                    'Ven18 - Condicao de pagamento nao encontrada para os criterios: '
                    'Tipo Prazo - %s, Tipo - %s, Qtde de Parcelas - %d ' % (dados_prazo_tipo, dados_avista_prazo, 0)
                )
            adm_condicao_id = adm_condicao_dados[0]['id']

            # BUSCA A FORMA DE PAGAMENTO POR MEIO DO NOME DA FINALIZADORA
            args_forma_pgto = [('name', 'ilike', dados_nome_finalizadora)]
            adm_forma_pgto_dados = db_registro.select(table='adm_forma_pgto', fields=['id'], where=args_forma_pgto)
            if not adm_forma_pgto_dados:
                raise Exception('Ven18 - Forma de pagamento nao encontrada. Name - %s' % dados_nome_finalizadora)

            adm_forma_pgto_id = adm_forma_pgto_dados[0]['id']

            # BUSCA A ASSOCIACAO DA CONDICAO COM A A FORMA DE PAGAMENTO
            args_condicao_forma = [
                ('adm_condicao_id', '=', adm_condicao_id), ('forma_pagamento_id', '=', adm_forma_pgto_id)
            ]
            condicao_forma_dados = db_registro.select(table='adm_condicao_forma', fields=['id'],
                                                      where=args_condicao_forma)
            if not condicao_forma_dados:
                raise Exception('Ven18 - Condicao Pagamento nao encontra-se associada para a Forma de Pagamento. ')

            condicao_forma_id = condicao_forma_dados[0]['id']

            # DADOS PARA CRIACAO OU ATUALIZACAO
            vals_pedido_condicao = {
                'capa_id': legado_ven13_pedido_id,
                'valor': total_condicao,
                'condicao_forma_id': condicao_forma_id,
                'forma_pagto_id': adm_forma_pgto_id,
                'pedcdc_chq_numero': '',
                'pedcdc_chq_banco': 0,
                'pedcdc_chq_agencia': 0,
                'pedcdc_chq_emitente': '',
                'pedcdc_chq_conta': '',
                'pedcdc_flag_pgto_chq': False,
            }

            # CRIA OU ATUALIZA A CONDICAO DO PEDIDO
            if not legado_ven18_id:
                pedido_condicao_id = db_registro.insert(table='estoquepedidos_pedido_condicao',
                                                        vals=vals_pedido_condicao)
            else:
                pedido_condicao_id = legado_ven18_id

                args_ven18 = [('id', '=', legado_ven18_id)]
                db_registro.update(table='estoquepedidos_pedido_condicao', values=vals_pedido_condicao,
                                   where=args_ven18)

            # CRIA OU ATUALIZA A LEGADO CONTROLE PARA A CONDICAO
            if not legado_ven18_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='ven18',
                    reg_chave_legado=chave_ven18,
                    reg_chave_sigerp=pedido_condicao_id,
                    reg_check_table='estoquepedidos_pedido_condicao',
                )
            else:
                args_ven18 = [
                    ('tabela_legado', '=', 'ven18'), ('empresa', '=', empresa_legado),
                    ('chave_legado', '=', chave_ven18)
                ]

                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': pedido_condicao_id},
                    reg_args=args_ven18,
                    reg_check_table='estoquepedidos_pedido_condicao',
                )

            # CRIA OU ATUALIZA AS PARCELAS
            for prazo_valor in dict_prazo_valor:
                data_vencimento = parse(prazo_valor, yearfirst=True).strftime('%Y-%m-%d')
                vals_pedido_parcela = {'pedido_condicao_id': pedido_condicao_id,
                                       'vr_parcela': float(dict_prazo_valor.get(prazo_valor)[1]),
                                       'data_vencimento': data_vencimento,
                                       'pedido_id': legado_ven13_pedido_id,
                                       }
                chave_ven18_parcela = '%s%s%s' % (pedido_condicao_id,
                                                  dict_prazo_valor.get(prazo_valor)[0], prazo_valor)

                # PEDIDO_PARCELA_ID (LEGADO CONTROLE)
                legado_ven18_parcela_id = db_registro.controle_get_chave_sigerp(
                    reg_empresa_legado=empresa_legado,
                    reg_chave=chave_ven18_parcela,
                    reg_tabela_legado='ven18',
                    reg_check_table='estoquepedidos_pedido_parcela',
                )

                # CRIA OU ATUALIZA AS PARCELAS DA CONDICAO DO PEDIDO
                if not legado_ven18_parcela_id:
                    pedido_parcela_id = db_registro.insert(table='estoquepedidos_pedido_parcela',
                                                           vals=vals_pedido_parcela)
                else:
                    pedido_parcela_id = legado_ven18_parcela_id

                    args_ven18_parcela = [('id', '=', legado_ven18_parcela_id)]
                    db_registro.update(table='estoquepedidos_pedido_parcela', values=vals_pedido_parcela,
                                       where=args_ven18_parcela)

                # CRIA OU ATUALIZA A LEGADO CONTROLE PARA A PARCELA
                if not legado_ven18_parcela_id:
                    db_registro.create_legado_controle(
                        reg_empresa_legado=empresa_legado,
                        reg_tabela_legado='ven18',
                        reg_chave_legado=chave_ven18_parcela,
                        reg_chave_sigerp=pedido_parcela_id,
                        reg_check_table='estoquepedidos_pedido_parcela',
                    )
                else:
                    args_ven18_parcela = [
                        ('tabela_legado', '=', 'ven18'), ('empresa', '=', empresa_legado),
                        ('chave_legado', '=', chave_ven18_parcela)
                    ]

                    db_registro.update_legado_controle(
                        reg_values={'chave_sigerp': pedido_parcela_id},
                        reg_args=args_ven18_parcela,
                        reg_check_table='estoquepedidos_pedido_parcela',
                    )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False
