#!/usr/bin/python
# -*- encoding: utf-8 -*-

from datetime import date
from re import sub as re_sub
from dateutil.parser import parse
from SigUtils.uf import Uf as sigutils_uf
from ..connection.integ_db import IntegracaoConnection
from ..funcoes.integ_funcoes import funct_capitalize_all, funct_get_fonema


class Registros(IntegracaoConnection):

    def __init__(self, cr, uid=False):
        IntegracaoConnection.__init__(self, cr, uid)

    def registro_gravar_erro(self, reg_leg_log_id, msg_erro):
        dados_lelg = self.select(table='legado_logs', fields=['lelg_arquivo_legado',
                                                         'lelg_contador_importacao'],
                            where=[('id', '=', reg_leg_log_id)])
        _modulo = dados_lelg[0].get('lelg_arquivo_legado')
        _qttentativas = dados_lelg[0].get('lelg_contador_importacao')
        if not _qttentativas:
            _qttentativas = 0
        args_cont_modulo = [('lelgco_modulo', 'ILIKE', _modulo)]
        dados_lelgco = self.select(table='legado_logs_contador',
                                         fields=['lelgco_qtd_tentativas'],
                                         where=args_cont_modulo)
        values_legado_log = {'lelg_erro': msg_erro, 'lelg_erro_registro': True}
        if dados_lelgco:
            _qt_max_tentativas = dados_lelgco[0].get('lelgco_qtd_tentativas')
            if _qt_max_tentativas > _qttentativas:
                _qttentativas += 1
                values_legado_log = {'lelg_erro': msg_erro,
                                     'lelg_erro_registro': False,
                                     'lelg_contador_importacao': _qttentativas}
        res = self.update(table='legado_logs', values=values_legado_log, where=[('id', '=', reg_leg_log_id)])
        self.cr_integracao.connection.commit()
        return res

    def check_table_has_company_id(self, reg_table):

        reg_table = reg_table.replace('.', '_')

        tabela_dados = self.select(
            table='information_schema.columns',
            fields=['column_name'],
            where=[('table_name', '=', reg_table), ('column_name', '=', 'company_id')]
        )
        if tabela_dados and tabela_dados[0].get('column_name'):
            res = True
        else:
            res = False
        return res

    def get_empresa_legado_relacionada(self, reg_empresa_legado, reg_objeto_nome, reg_empresa_tipo='C'):

        res = reg_empresa_legado

        if reg_empresa_tipo != 'S':
            reg_empresa_tipo = 'C'

        args_empresa = [('empresa_legado', '=', reg_empresa_legado), ('tipo', '=', reg_empresa_tipo)]
        # BUSCA NA LEGADO_EMPRESA O COMPANY_ID PARA O LEGADO
        legado_empresa_dados = self.select(table='legado_empresa', fields=['company_id'], where=args_empresa)
        if legado_empresa_dados:
            company_id = legado_empresa_dados[0]['company_id']

            # BUSCA O ID NA IR_MODEL
            str_objeto = reg_objeto_nome.replace('_', '.')
            model_dados = self.select(table='ir_model', fields=['id'], where=[('model', '=', str_objeto)])
            if model_dados:
                model_id = model_dados[0]['id']

                # BUSCA NA ADM_EMPRESA_OBJETO QUAL EMPRESA QUE DEVERA SER UTILIZADA PARA O MODELO INFORMADO
                args_objeto = [('empresa_escolhida_id', '=', company_id), ('ir_model_id', '=', model_id)]
                empresa_objeto_dados = self.select(
                    table='adm_empresa_objeto', fields=['empresa_a_usar_id'], where=args_objeto
                )
                if empresa_objeto_dados:
                    company_usar_id = empresa_objeto_dados[0]['empresa_a_usar_id']

                    # BUSCA NA LEGADO_EMPRESA QUAL A EMPRESA DO LEGADO RELACIONADA A NOVA EMPRESA
                    args_empresa2 = [('company_id', '=', company_usar_id), ('tipo', '=', reg_empresa_tipo)]
                    empresa_dados = self.select(table='legado_empresa', fields=['empresa_legado'], where=args_empresa2)
                    if empresa_dados:
                        res = empresa_dados[0]['empresa_legado']
        return res

    def controle_get_chave_sigerp(self, reg_empresa_legado, reg_chave, reg_tabela_legado, reg_many_ids=False,
                                  reg_check_table='', reg_empresa_tipo=''):

        args = [('chave_legado', '=', reg_chave), ('tabela_legado', '=', reg_tabela_legado)]

        # BUSCA A EMPRESA DO LEGADO DE ACORDO COM A CORRELACAO DE CLASSES NO ERP
        new_empresa_legado = self.get_empresa_legado_relacionada(
            reg_empresa_legado=reg_empresa_legado,
            reg_objeto_nome=reg_check_table,
            reg_empresa_tipo=reg_empresa_tipo
        )

        args.append(('empresa', '=', new_empresa_legado))

        if reg_check_table:
            # VERIFICA SE A TABELA INFORMADA POSSUI A COLUNA COMPANY_ID
            table_has_column = self.check_table_has_company_id(reg_table=reg_check_table)
            if not table_has_column:
                # SE A TABELA QUE FOI VERIFICADA NAO POSSUIR COMPANY_ID,
                # O ARGUMENTO PARA FILTRAR POR EMPRESA É RETIRADO.
                args = filter(lambda i: 'empresa' not in i[0], args)

        if reg_many_ids:
            chave_erp = []
        else:
            chave_erp = False

        legado_controle_dados = self.select(
            table='legado_controle', fields=['chave_sigerp'], where=args, order_by=['id']
        )

        if legado_controle_dados:
            if reg_many_ids:
                chave_erp = tuple(map(lambda i: i['chave_sigerp'], legado_controle_dados))
            else:
                chave_erp = legado_controle_dados[0]['chave_sigerp']

        return chave_erp

    def create_legado_controle(self, reg_empresa_legado, reg_tabela_legado, reg_chave_legado, reg_chave_sigerp,
                               reg_m2m=False, reg_check_table='', reg_empresa_tipo=''):

        new_empresa_legado = self.get_empresa_legado_relacionada(reg_empresa_legado, reg_check_table, reg_empresa_tipo)

        vals_legado_controle = {
            'empresa': new_empresa_legado,
            'tabela_legado': reg_tabela_legado,
            'chave_legado': reg_chave_legado,
            'chave_sigerp': reg_chave_sigerp
        }

        controle_id = self.insert(table='legado_controle', vals=vals_legado_controle, many2many=reg_m2m)
        return controle_id

    def update_legado_controle(self, reg_values, reg_args, reg_m2m=False, reg_check_table='', reg_empresa_tipo=''):

        if not isinstance(reg_args, list):
            reg_values = []

        args_sem_empresa = filter(lambda i: 'empresa' not in i[0], reg_args)
        args_empresa = filter(lambda i: 'empresa' in i[0], reg_args)

        if args_empresa:
            empresa_legado = args_empresa[0][2]
            new_empresa_legado = self.get_empresa_legado_relacionada(empresa_legado, reg_check_table, reg_empresa_tipo)
            args_empresa = [('empresa', '=', new_empresa_legado)]
            reg_values['empresa'] = new_empresa_legado

        reg_args = args_sem_empresa
        if args_empresa:
            reg_args.extend(args_empresa)

        res = self.update(table='legado_controle', values=reg_values, where=reg_args, many2many=reg_m2m)
        return res

    def delete_legado_controle(self, reg_args=None, reg_check_table='', reg_empresa_tipo=''):

        if not isinstance(reg_args, list):
            reg_args = []

        args_sem_empresa = filter(lambda i: 'empresa' not in i[0], reg_args)
        args_empresa = filter(lambda i: 'empresa' in i[0], reg_args)

        if args_empresa:
            empresa_legado = args_empresa[0][2]
            new_empresa_legado = self.get_empresa_legado_relacionada(empresa_legado, reg_check_table, reg_empresa_tipo)
            args_empresa = [('empresa', '=', new_empresa_legado)]

        reg_args = args_sem_empresa
        if args_empresa:
            reg_args.extend(args_empresa)

        res = self.delete(table='legado_controle', where=reg_args)
        return res

    def select_legado_controle(self, reg_fields=None, reg_args=None, reg_order_by=None, reg_limit=0, reg_check_table='',
                               reg_empresa_tipo=''):

        if not isinstance(reg_fields, list):
            reg_fields = []
        if not isinstance(reg_args, list):
            reg_args = []

        args_sem_empresa = filter(lambda i: 'empresa' not in i[0], reg_args)
        args_empresa = filter(lambda i: 'empresa' in i[0], reg_args)

        if args_empresa:
            empresa_legado = args_empresa[0][2]
            new_empresa_legado = self.get_empresa_legado_relacionada(empresa_legado, reg_check_table, reg_empresa_tipo)
            args_empresa = [('empresa', '=', new_empresa_legado)]

        reg_args = args_sem_empresa
        if args_empresa:
            reg_args.extend(args_empresa)

        legado_dados = self.select(
            table='legado_controle', fields=reg_fields, where=reg_args, order_by=reg_order_by, limit=reg_limit
        )
        return legado_dados

    def registro_pessoa(self, dados):

        dados_sexo = dados.get('sexo') or ''
        dados_nome = dados.get('nome') or ''
        dados_ativo = dados.get('ativo') or False
        dados_email = dados.get('email') or ''
        dados_suframa = dados.get('suframa') or ''
        dados_cpf_cnpj = dados.get('cpf_cnpj') or ''
        dados_pais_nome = dados.get('pais_nome') or ''
        dados_identidade = dados.get('identidade') or ''
        dados_cliente_uf = dados.get('cliente_uf') or ''
        dados_endcli_rua = dados.get('endcli_rua') or ''
        dados_pessoa_tipo = dados.get('pessoa_tipo') or ''
        dados_observacoes = dados.get('observacoes') or ''
        dados_cliente_cep = dados.get('cliente_cep') or ''
        dados_empresa_tipo = dados.get('empresa_tipo') or ''
        dados_data_fundacao = dados.get('fundacao_data') or ''
        dados_endcli_numero = dados.get('end_cli_numero') or ''
        dados_produtor_rural = dados.get('produtor_rural') or False
        dados_cliente_cidade = dados.get('cliente_cidade') or ''
        dados_cliente_bairro = dados.get('cliente_bairro') or ''
        dados_data_nascimento = dados.get('data_nascimento') or None
        dados_fantasia_apelido = dados.get('fantasia_apelido') or ''
        dados_inscricao_estadual = dados.get('ie') or ''
        dados_tipo_atividade = dados.get('admpes_jur_tipo_atividade') or ''
        dados_regime_tributacao = dados.get('admpes_jur_regime_tributario') or ''

        TABLE = 'adm_pessoa'
        pessoa_id = False

        if dados_cpf_cnpj.__len__() > 6:
            args_pessoa = [('cpf_cnpj', '=', dados_cpf_cnpj)]
        else:
            args_pessoa = [('nome', '=', dados_nome.upper(), ('pessoa_tipo', '=', dados_pessoa_tipo))]

        pessoa_dados = self.select(table=TABLE, fields=['id'], where=args_pessoa)
        if pessoa_dados:
            pessoa_id = pessoa_dados[0]['id']
        else:
            vals_pessoa = {
                'nome': dados_nome.upper(),
                'fantasia_apelido': dados_fantasia_apelido,
                'pessoa_tipo': dados_pessoa_tipo,
                'cpf_cnpj': dados_cpf_cnpj,
                'active': dados_ativo,
            }

            if dados_pessoa_tipo == 'J':
                if dados_data_fundacao != '':
                    try:
                        data_fundacao = parse(dados_data_fundacao, yearfirst=True).date().strftime('%Y-%m-%d')
                    except:
                        data_fundacao = False
                    if data_fundacao:
                        vals_pessoa['admpes_jur_data_fundacao'] = dados_data_fundacao
                if dados_empresa_tipo:
                    vals_pessoa['admpes_jur_tipo_empresa'] = dados_empresa_tipo
                if dados_suframa:
                    vals_pessoa['admpes_jur_suframa'] = dados_suframa
                if dados_regime_tributacao:
                    vals_pessoa['admpes_jur_regime_tributario'] = dados_regime_tributacao
                if dados_tipo_atividade:
                    vals_pessoa['admpes_jur_tipo_atividade'] = dados_tipo_atividade
                if dados_inscricao_estadual != '':
                    vals_pessoa['admpes_jur_ie'] = dados_inscricao_estadual
            else:
                if dados_identidade != '':
                    vals_pessoa['admpes_fis_rg'] = dados_identidade
                vals_pessoa['admpes_fis_sexo'] = dados_sexo

                nascimento_data = False
                try:
                    nascimento_data = parse(dados_data_nascimento, yearfirst=True).date().strftime('%Y-%m-%d')
                except:
                    pass

                if nascimento_data:
                    vals_pessoa['admpes_fis_data_nascimento'] = dados_data_nascimento
                vals_pessoa['admpes_fis_produtor_rural'] = dados_produtor_rural

            vals_pessoa['endereco'] = dados_endcli_rua + ', ' + dados_endcli_numero

            logradouro_id = self.registro_logradouro(dados_endcli_rua)
            if logradouro_id:
                logradouro_dados = self.select(
                    table='adm_logradouro', fields=['titulo', 'descricao'], where=[('id', '=', logradouro_id)]
                )
                if logradouro_dados:
                    lograd_titulo = logradouro_dados[0]['titulo']
                    lograd_descricao = logradouro_dados[0]['descricao']

                    vals_pessoa['endereco'] = '%s %s, %s' % (lograd_titulo, lograd_descricao, dados_endcli_numero)

            vals_pessoa['bairro'] = dados_cliente_bairro
            vals_pessoa['cidade'] = dados_cliente_cidade
            vals_pessoa['uf'] = dados_cliente_uf
            vals_pessoa['pais'] = dados_pais_nome
            vals_pessoa['cep'] = dados_cliente_cep
            vals_pessoa['email'] = dados_email
            vals_pessoa['observacoes'] = dados_observacoes

            vals_pessoa['soundex'] = funct_get_fonema(vals_pessoa['nome'] + ' ' + dados_fantasia_apelido)

            partner_id = self.insert(table='res_partner', vals={'name': dados_nome, 'notification_email_send': 'comment'})
            if partner_id:
                vals_pessoa['partner_id'] = partner_id
                sequence_dados = self.select(
                    table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
                )
                if sequence_dados:
                    sequence_id = sequence_dados[0]['id']
                    sequence_codigo = int(sequence_dados[0]['number_next'])
                    sequence_proximo_codigo = sequence_codigo + 1
                    self.update(
                        table='ir_sequence', values={'number_next': sequence_proximo_codigo}, where=[('id', '=', sequence_id)]
                    )
                    vals_pessoa['codigo'] = sequence_codigo
                    pessoa_id = self.insert(table=TABLE, vals=vals_pessoa)
        return pessoa_id

    def registro_cliente(self, reg_company_id, reg_pessoa_id, reg_dados):

        TABLE = 'financeiro_cliente'
        cliente_id = False

        vals_cliente = {}

        # LEITURA DOS DADOS
        reg_cli_condicao_pgto = reg_dados.get('condicao_pgto')
        reg_cli_data_inclusao = reg_dados.get('data_inclusao')
        reg_cli_bonus_dia_mes = reg_dados.get('bonus_dia_mes')
        reg_cli_fidelidade_codigo = reg_dados.get('fidelidade_codigo')
        reg_cli_limite_credito = reg_dados.get('limite_credito') or 0
        reg_cli_ativo = reg_dados.get('ativo') or False

        if reg_cli_condicao_pgto == '000000000':
            vls_condicoes = ''
        else:
            vls_condicoes = reg_dados.get('condicao_pgto')

        if reg_cli_data_inclusao == '0000-00-00':
            vls_data_incluscao = date.today().strftime('%Y-%m-%d')
        else:
            vls_data_incluscao = reg_cli_data_inclusao

        module_fidelidade_instalado = self.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'fidelidade'), ('state', '=', 'installed')]
        )
        if module_fidelidade_instalado:
            vals_cliente['codigo_fidelidade'] = reg_cli_fidelidade_codigo
            if reg_cli_bonus_dia_mes:
                vals_cliente['dia_mes_vencimento_pontos'] = reg_cli_bonus_dia_mes

        vals_cliente['data_inclusao'] = vls_data_incluscao
        vals_cliente['pessoa_id'] = reg_pessoa_id
        vals_cliente['company_id'] = reg_company_id
        vals_cliente['limite_credito'] = reg_cli_limite_credito
        vals_cliente['active'] = reg_cli_ativo

        args_fin_cli = [('pessoa_id', '=', reg_pessoa_id), ('company_id', '=', reg_company_id)]
        financeiro_cliente_dados = self.select(table=TABLE, fields=['id'], where=args_fin_cli)
        if financeiro_cliente_dados:
            cliente_id = financeiro_cliente_dados[0]['id']
            self.update(
                table=TABLE, values=vals_cliente, where=[('id', '=', cliente_id)]
            )
        else:
        # INCLUI O CLIENTE NA BASE DADOS
            cliente_id = self.insert(table=TABLE, vals=vals_cliente)
        return cliente_id

    def registro_pessoa_dependente(self, reg_nome, reg_fis_jur):
        TABLE = 'adm_pessoa'
        dependente_id = False

        args_pessoa = [('nome', '=', reg_nome), ('pessoa_tipo', '=', reg_fis_jur)]
        adm_pessoa_dados = self.select(table=TABLE, fields=['id'], where=args_pessoa)
        if adm_pessoa_dados:
            dependente_id = adm_pessoa_dados[0]['id']
        else:
            vals_pessoa = {'nome': reg_nome, 'pessoa_tipo': reg_fis_jur, 'active': True}

            partner_id = self.insert(
                table='res_partner', vals={'name': reg_nome, 'notification_email_send': 'comment', 'active': True}
            )
            if partner_id:
                vals_pessoa['partner_id'] = partner_id
                sequence_dados = self.select(
                    table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
                )
                if sequence_dados:
                    sequence_id = sequence_dados[0]['id']
                    sequence_codigo = int(sequence_dados[0]['number_next'])
                    sequence_proximo_codigo = sequence_codigo + 1

                    self.update(
                        table='ir_sequence', values={'number_next': sequence_proximo_codigo}, where=[('id', '=', sequence_id)]
                    )
                    vals_pessoa['codigo'] = sequence_codigo
                    dependente_id = self.insert(table=TABLE, vals=vals_pessoa)
        return dependente_id

    def registro_pessoa_relacionamento_tipo(self, reg_titularidade, reg_fis_jur):
        TABLE = 'adm_relacionamento_tipo'

        args_relacionamento = [('pessoa_tipo', '=', reg_fis_jur)]
        if reg_titularidade in ('Pai', 'Mae'):
            args_relacionamento .append(('sigla', '=', 'PM'))
        else:
            args_relacionamento .append(('sigla', '=', reg_titularidade))

        adm_rel_tipo_dados = self.select(table=TABLE, fields=['id'], where=args_relacionamento)
        if adm_rel_tipo_dados:
            titulo_id = adm_rel_tipo_dados[0]['id']
        else:
            vals_adm_rel_tipo = {'name': reg_titularidade, 'pessoa_tipo': reg_fis_jur}
            titulo_id = self.insert(table=TABLE, vals=vals_adm_rel_tipo)

        return titulo_id

    def registro_dependente(self, reg_pessoa_id, reg_relat_tipo_id, reg_dependente_id):
        TABLE = 'adm_pessoa_relacionamento'

        relacionamento_tipo_dados = self.select(
            table='adm_relacionamento_tipo', fields=['relacionamento_inverso_id'], where=[('id', '=', reg_relat_tipo_id)]
        )

        if relacionamento_tipo_dados:
            inverso_id = relacionamento_tipo_dados[0]['relacionamento_inverso_id']
        else:
            inverso_id = False

        args_relacionamento = [
            ('dependente_id', '=', reg_dependente_id), ('relacionamento_tipo_id', '=', reg_relat_tipo_id),
            ('pessoa_id', '=', reg_pessoa_id)
        ]
        relacionamento_dados = self.select(table=TABLE, fields=['id'], where=args_relacionamento)
        if relacionamento_dados:
            relat_id = relacionamento_dados[0]['id']
        else:
            vals_relat = {
                'dependente_id': reg_dependente_id,
                'relacionamento_tipo_id': reg_relat_tipo_id,
                'pessoa_id': reg_pessoa_id
            }

            relat_id = self.insert(table=TABLE, vals=vals_relat)

        return relat_id

    def registro_endereco(self, reg_endereco_nro, reg_cep, reg_pessoa_id, reg_tipo_id, reg_pais_id,
                          reg_uf_id, reg_cidade_id, reg_bairro_id, reg_logradouro_id, dados={}):

        TABLE = 'adm_pessoa_endereco'

        args_endereco = [('pessoa_id', '=', reg_pessoa_id), ('endereco_tipo_id', '=', reg_tipo_id)]
        endereco_dados = self.select(table=TABLE, fields=['id'], where=args_endereco)
        if endereco_dados:
            endereco_id = endereco_dados[0]['id']
        else:
            try:
                numero = int(re_sub('[^0-9]', '', reg_endereco_nro))
            except:
                numero = 0
            complemento = re_sub('[^A-Z]', '', reg_endereco_nro.upper())
            if numero > 999910:
                if complemento == '':
                    complemento = str(numero)
                    numero = ''
                else:
                    numero = int(str(numero)[:5])

            vals_endereco = {
                'pessoa_id': reg_pessoa_id,
                'numero': numero,
                'complemento': complemento,
                'active': True,
            }

            if reg_pais_id:
                vals_endereco['pais_id'] = reg_pais_id
            if reg_uf_id:
                vals_endereco['uf_id'] = reg_uf_id
            if reg_cidade_id:
                vals_endereco['cidade_id'] = reg_cidade_id
            if reg_bairro_id:
                vals_endereco['bairro_id'] = reg_bairro_id
            if reg_tipo_id:
                vals_endereco['endereco_tipo_id'] = reg_tipo_id

            if reg_logradouro_id:
                vals_endereco['logradouro_id'] = reg_logradouro_id
            if reg_cep != '00000000':
                vals_endereco['cep'] = reg_cep

            endereco_id = self.insert(table=TABLE, vals=vals_endereco)
            if endereco_id and dados.get('tipo') == 'normal':
                modulo_fiscal_instalado = self.select(
                    table='ir_module_module', fields=['id'], where=[('name', '=', 'fiscal'), ('state', '=', 'installed')]
                )
                if modulo_fiscal_instalado:
                    self.update(table='adm_pessoa', values={'endereco_nf_id': endereco_id}, where=[('id', '=', reg_pessoa_id)])

        return endereco_id

    def registro_email_tipo(self, reg_tipo):
        TABLE = 'adm_email_tipo'

        email_tipo_dados = self.select(table=TABLE, fields=['id'], where=[('name', 'ilike', reg_tipo)])
        if email_tipo_dados:
            email_tipo_id = email_tipo_dados[0]['id']
        else:
            vals_email = {'name': reg_tipo, 'bloqueio': True}
            email_tipo_id = self.insert(table=TABLE, vals=vals_email)
        return email_tipo_id

    def registro_email(self, reg_pessoa_id, reg_email, reg_email_tipo_id):
        TABLE = 'adm_pessoa_email'

        args_email = [('name', '=', reg_email), ('pessoa_id', '=', reg_pessoa_id)]
        email_dados = self.select(table=TABLE, fields=['id'], where=args_email)
        if email_dados:
            email_id = email_dados[0]['id']
        else:
            vals_email = {'name': reg_email, 'pessoa_id': reg_pessoa_id, 'email_tipo_id': reg_email_tipo_id}
            email_id = self.insert(table=TABLE, vals=vals_email)
        return email_id

    def registro_telefone_tipo(self, reg_tel_tipo):
        TABLE = 'adm_telefone_tipo'

        tel_tipo_dados = self.select(table=TABLE, fields=['id'], where=[('name', 'ilike', reg_tel_tipo)])
        if tel_tipo_dados:
            tel_tipo_id = tel_tipo_dados[0]['id']
        else:
            vals_tel_tipo = {'name': reg_tel_tipo, 'bloqueio': True}
            tel_tipo_id = self.insert(table=TABLE, vals=vals_tel_tipo)

        return tel_tipo_id

    def registro_telefone(self, reg_pessoa_id, reg_telefone, reg_telefone_tipo_id):

        def acha_ddd_telefone(reg_telefone):
            _ddd = ''
            _telefone = ''
            if len(reg_telefone) < 10:
                _telefone = reg_telefone
            else:
                posI = reg_telefone.find('(')  # ddd entre ()
                if posI != -1:
                    posF = reg_telefone.find(')')
                    if posF > posI:
                        _ddd = reg_telefone[posI + 1:posF]
                        _telefone = reg_telefone[posF + 1:].strip()
                else:
                    posF = reg_telefone.find(' ')  # ddd separado por espaco
                    if posF != -1 and posF < 4:
                        _ddd = reg_telefone[:posF].strip()
                        _telefone = reg_telefone[posF + 1:].strip()
                    else:
                        posF = reg_telefone.find('-')  # ddd separado por "-"
                        if posF != -1 and posF < 4:
                            _ddd = reg_telefone[:posF].strip()
                            _telefone = reg_telefone[posF + 1:].strip()
                        else:
                            posF = reg_telefone.find(')')  # ddd separado por "-"
                            if posF != -1 and posF < 5:
                                _ddd = reg_telefone[:posF].strip()
                                _telefone = reg_telefone[posF + 1:].strip()
                            else:
                                if len(reg_telefone) > 25:
                                    _ddd = reg_telefone[:len(reg_telefone) - 1].strip()
                                    _telefone = reg_telefone[len(reg_telefone) - 25:].strip()
            return _ddd, _telefone

        TABLE = 'adm_pessoa_telefone'

        if reg_telefone.upper() == 'NAO TEM':
            return True

        ddd, telefone = acha_ddd_telefone(reg_telefone)
        ddd = re_sub('[^0-9]', '', ddd)
        try:
            a = int(ddd)
            if not a:
                ddd = ''
            else:
                ddd = '%03d' % a
        except:
            pass

        telefone = re_sub('[^0-9]', '', telefone)
        try:
            a = int(telefone)
            if a == 0:
                telefone = ''
        except:
            pass

        if telefone == '' or telefone.__len__() < 4:
            return True

        args_tel = [('numero', '=', telefone), ('pessoa_id', '=', reg_pessoa_id)]
        telefone_dados = self.select(table=TABLE, fields=['id'], where=args_tel)
        if telefone_dados:
            telefone_id = telefone_dados[0]['id']
        else:
            vals_telefone = {
                'numero': telefone, 'ddd': ddd, 'pessoa_id': reg_pessoa_id, 'telefone_tipo_id': reg_telefone_tipo_id
            }
            telefone_id = self.insert(table=TABLE, vals=vals_telefone)

        return telefone_id

    def registro_categoria_tipo(self, reg_categ_name):
        TABLE = 'adm_pessoa_categoria_nome'

        categ_dados = self.select(table=TABLE, fields=['id'], where=[('name', '=', reg_categ_name)])
        if categ_dados:
            categ_id = categ_dados[0]['id']
        else:
            vals_categ_nome = {'name': reg_categ_name}
            categ_id = self.insert(table=TABLE, vals=vals_categ_nome)

        return categ_id

    def registro_categoria_pessoa(self, reg_pessoa_id, reg_categoria_id):
        TABLE = 'pessoa_categoria'  # tabela m2m

        args_categ = [('pessoa_id', '=', reg_pessoa_id), ('categoria_id', '=', reg_categoria_id)]
        categ_dados = self.select(table=TABLE, fields=[], where=args_categ)
        if not categ_dados:
            vals_categ = {'pessoa_id': reg_pessoa_id, 'categoria_id': reg_categoria_id}
            categ_res = self.insert(table=TABLE, vals=vals_categ, many2many=True)
        else:
            categ_res = True
        return categ_res

    def registro_mensagem_nome(self, reg_msg_cod, reg_msg_name, reg_msg_tipo):
        TABLE = 'financeiro_clientes_mensagem_nome'
        if reg_msg_name:
            reg_msg_name = reg_msg_name
        args_msg = [('codigo', '=', reg_msg_cod)]
        cli_msg_nome_dados = self.select(table=TABLE, fields=['id'], where=args_msg)
        if cli_msg_nome_dados:
            cli_msg_nome_id = cli_msg_nome_dados[0]['id']
        else:
            if reg_msg_name:
                reg_msg_name = reg_msg_name[:50]
            vals_cli_msg_nome = {'codigo': reg_msg_cod, 'name': reg_msg_name,
                                 'tipo_mensagem': reg_msg_tipo}
            cli_msg_nome_id = self.insert(table=TABLE, vals=vals_cli_msg_nome)
        return cli_msg_nome_id

    def registro_cliente_mensagem(self, reg_msg_id, reg_cliente_id, reg_data):
        TABLE = 'financeiro_clientes_mensagem'

        args = [('mensagens_id', '=', reg_msg_id), ('cliente_id', '=', reg_cliente_id)]
        cli_msg_dados = self.select(table=TABLE, fields=['id'], where=args)
        if cli_msg_dados:
            cli_msg_id = cli_msg_dados[0]['id']
        else:
            vals_cli_msg = {
                'mensagens_id': reg_msg_id,
                'cliente_id': reg_cliente_id,
                'modo_exibicao': 'I',
                'data_inclusao': date.today().strftime('%Y-%m-%d')
            }
            cli_msg_id = self.insert(table=TABLE, vals=vals_cli_msg)

        return cli_msg_id

    def registro_pais(self, reg_pais_cod, reg_pais_nome):
        TABLE = 'adm_pais'

        pais_dados = self.select(table=TABLE, fields=['id'], where=[('name', 'ILIKE', reg_pais_nome)])
        if pais_dados:
            pais_id = pais_dados[0]['id']
        else:
            vals_pais = {'name': reg_pais_nome, 'codigo_bc': reg_pais_cod}
            pais_id = self.insert(table=TABLE, vals=vals_pais)
        return pais_id

    def registro_uf(self, reg_uf_sigla, reg_uf_pais_id):
        TABLE = 'adm_uf'

        args_uf = [('sigla', '=', reg_uf_sigla), ('pais_id', '=', reg_uf_pais_id)]
        uf_dados = self.select(table=TABLE, fields=['id'], where=args_uf)
        if uf_dados:
            uf_id = uf_dados[0]['id']
        else:
            uf_sigla = sigutils_uf(reg_uf_sigla)
            try:
                uf_nome = funct_capitalize_all(uf_sigla.getDescricao())
            except:
                raise Exception("UF NAO ENCONTRADO NA TABELA %s: %s" % (TABLE, reg_uf_sigla))

            vals_uf = {'sigla': reg_uf_sigla, 'pais_id': reg_uf_pais_id, 'name': uf_nome}
            uf_id = self.insert(table=TABLE, vals=vals_uf)
        return uf_id

    def registro_cidade(self, reg_cod_ibge, reg_cidade_nome,  reg_uf_id, reg_uf_sigla):

        if reg_cidade_nome == 'Araai':
            reg_cidade_nome = 'Aracai'
        elif reg_cidade_nome == 'Baldin':
            reg_cidade_nome = 'Baldim'
        elif reg_cidade_nome[:6] == 'Belo H' or reg_cidade_nome[:2] == 'B.' or reg_cidade_nome[:6] in ('B Hori', 'Helo H', 'Bwelo ',):
            reg_cidade_nome = 'Belo Horizonte'
        elif reg_cidade_nome == 'Brasilan.minas' or reg_cidade_nome == 'Brazilia De Minas' or (reg_cidade_nome == 'Brasilia' and reg_uf_sigla == 'MG'):
            reg_cidade_nome = 'Brasilia De Minas'
        elif reg_cidade_nome[:7] == 'Cach.da' or reg_cidade_nome[:7] == 'Cach Da' or reg_cidade_nome[:8] == 'Cach. Da' or reg_cidade_nome[:10] in ('Cachoeiora', 'Cachoeiera') or reg_cidade_nome in ('Cachoeira Da Prato', 'Cachoeira Prata', 'Ca Da Prata', 'C.da Prata', 'C Da Prata', 'Cachoeira', 'Cachoira Da Prata', 'Cha Prata', 'Ch.da Prata', 'Cachoeria Da Prata'):
            reg_cidade_nome = 'Cachoeira Da Prata'
        elif reg_cidade_nome == 'Caetanapolis' or reg_cidade_nome == 'Caetanoplis' or reg_cidade_nome == 'Caetanpolis':
            reg_cidade_nome = 'Caetanopolis'
        elif reg_cidade_nome in ('Cerdisburgo', 'Codisburgo', 'Codisbrugo', 'Cosdisburgo') or reg_cidade_nome[:6]=='Cordis':
            reg_cidade_nome = 'Cordisburgo'
        elif reg_cidade_nome in ('Divinapolis','Divinpolis'):
            reg_cidade_nome = 'Divinopolis'
        elif reg_cidade_nome == 'Esmeralda':
            reg_cidade_nome = 'Esmeraldas'
        elif reg_cidade_nome == 'Farropilha':
            reg_cidade_nome = 'Farroupilha'
        elif reg_cidade_nome in ('Felixlandia', 'Fexlandia', 'Fexlandia'):
            reg_cidade_nome = 'Felislandia'
        elif reg_cidade_nome[:6] == 'Ferraz':
            reg_cidade_nome = 'Ferraz De Vasconcelos'
        elif reg_cidade_nome in ('Fortuna', 'Fortunas De Minas'):
            reg_cidade_nome = 'Fortuna De Minas'
        elif reg_cidade_nome == 'Inhuma':
            reg_cidade_nome = 'Inhauma'
        elif reg_cidade_nome in ('Jaboticatuva', 'Jaboticatuba'):
            reg_cidade_nome = 'Jaboticatubas'
        elif reg_cidade_nome in ('Jeauitiba', 'Jequetiba', 'Jequiba', 'Jequitia', 'Jewuitiba', 'Jquitiba'):
            reg_cidade_nome = 'Jequitiba'
        elif reg_cidade_nome == 'J.de Fora':
            reg_cidade_nome = 'Juiz De Fora'
        elif reg_cidade_nome == 'Joinville Sc':
            reg_cidade_nome = 'Joinville'
        elif reg_cidade_nome == 'L.santa':
            reg_cidade_nome = 'Lagoa Santa'
        elif reg_cidade_nome == 'Luziana':
            reg_cidade_nome = 'Luziania'
        elif reg_cidade_nome == 'Matozinhoss':
            reg_cidade_nome = 'Matozinhos'
        elif reg_cidade_nome in ('M.claros', 'Montes Claro'):
            reg_cidade_nome = 'Montes Claros'
        elif reg_cidade_nome == 'Montrreal':
            reg_cidade_nome = 'Montreal'
        elif reg_cidade_nome == 'Morada Nova':
            reg_cidade_nome = 'Morada Nova De Minas'
        elif reg_cidade_nome == 'Papagaio':
            reg_cidade_nome = 'Papagaios'
        elif reg_cidade_nome in ('Paraobepa', 'Paraopepa'):
            reg_cidade_nome = 'Paraopeba'
        elif reg_cidade_nome == 'Para Minas':
            reg_cidade_nome = 'Para De Minas'
        elif reg_cidade_nome in ('P. De Moraes', 'P-de Morais', 'P.de Morais', 'P Moraes', 'P.morais', 'Pridente De Morais', 'Prrudente De Morais', 'Prude.de Morais', 'Prud. De Morais', 'Pru.de Moraes', 'Pru. De Morais', 'Prudente De Moaris', 'Prudente De Moraes', 'Prudente D.moraes', 'Prudente Moraes', 'Prudente Morais', 'Prud Moraes', 'Prud. Moraes', 'Prud. Morais'):
            reg_cidade_nome = 'Prudente De Morais'
        elif reg_cidade_nome in ('P.leopoldo', 'P.leopolso'):
            reg_cidade_nome = 'Pedro Leopoldo'
        elif reg_cidade_nome in ('Pocos De Calda', 'Pocos De Caldass', 'Poos De Caldas'):
            reg_cidade_nome = 'Pocos De Caldas'
        elif reg_cidade_nome == '.porto Alegre':
            reg_cidade_nome = 'Porto Alegre'
        elif reg_cidade_nome == 'Ribeirao Des Neves':
            reg_cidade_nome = 'Ribeirao Das Neves'
        elif reg_cidade_nome == 'Saubara':
            reg_cidade_nome = 'Sabara'
        elif reg_cidade_nome in ('Santana Do Pirapama', 'Santana Pirapama', 'Santan De Pirapama', 'Santan Pirapama', 'Sant.de Pirapama', 'Sant. De Pirapama', 'Satana De Pirapama', 'Satana Pirapama', 'S.de Pirapama', 'S Pirapama', 'Sta Pirapama', 'Sta De Pirapama', 'St De Pirapama', 'Pirapama'):
            reg_cidade_nome = 'Santana De Pirapama'
        elif reg_cidade_nome == 'S.b.campo':
            reg_cidade_nome = 'Sao Bernardo Do Campo'
        elif reg_cidade_nome == 'So Bento Do Sul':
            reg_cidade_nome = 'Sao Bento Do Sul'
        elif reg_cidade_nome in ('Sao Vicente Baldim', 'Sao Vicente / Baldim', 'Sao Vicente De Minas'):
            reg_cidade_nome = 'Sao Vicente'
        elif reg_cidade_nome[1:5] == 'sete' or reg_cidade_nome[:3] in ('See',) or reg_cidade_nome[:4] in ('Sset', 'Dete', 'Aete', 'Sate', 'Sere', 'Seta', 'Sete', 'Set ', 'Sety', 'Seyt', 'Ste ', 'Sets','Stee') or reg_cidade_nome in ('Este Lagoas', '.ete Lagoas', 'Ete Lagoas', 'Faz.velha 7lagoas', 'Serte Lagoas', 'Seste Lagoas','Slagoas', 'S/lagoas', 'S.lagoas', 'S Lagoas', 'Lagoas', 'Algoas') :
            reg_cidade_nome = 'Sete Lagoas'
        elif reg_cidade_nome == 'So Paulo':
            reg_cidade_nome = 'Sao Paulo'
        elif reg_cidade_nome == 'Susano':
            reg_cidade_nome = 'Suzano'
        elif reg_cidade_nome == 'Tres Maria':
            reg_cidade_nome = 'Tres Marias'
        elif reg_cidade_nome == 'Vargem Gde. Paulista':
            reg_cidade_nome = 'Vargem Grande Paulis'

        TABLE = 'adm_cidade'

        if not reg_cod_ibge.isdigit() or len(reg_cod_ibge) != 7:
            reg_cod_ibge = 0
            args_cidade = [('name', '=', reg_cidade_nome), ('uf_id', '=', reg_uf_id)]
            cidade_dados = self.select(table=TABLE, fields=['id'], where=args_cidade)
        else:
            args_cidade = [('codigo_ibge', '=', reg_cod_ibge), ('uf_id', '=', reg_uf_id)]
            cidade_dados = self.select(table=TABLE, fields=['id'], where=args_cidade)
        if cidade_dados:
            cidade_id = cidade_dados[0]['id']
        elif len(reg_cidade_nome) < 3:
            return False
        else:
            args_cidade = [('name', '=', reg_cidade_nome), ('uf_id', '=', reg_uf_id)]
            cidade_dados = self.select(table=TABLE, fields=['id'], where=args_cidade)
            if cidade_dados:
                cidade_id = cidade_dados[0]['id']
            else:
                vals_cidade = {'uf_id': reg_uf_id, 'name': reg_cidade_nome, 'codigo_ibge': reg_cod_ibge}
                cidade_id = self.insert(table=TABLE, vals=vals_cidade)

        return cidade_id

    def registro_bairro(self, reg_bairro_nome):
        TABLE = 'adm_bairro'

        bairro_dados = self.select(table=TABLE, fields=['id'], where=[('name', '=', reg_bairro_nome)])
        if bairro_dados:
            bairro_id = bairro_dados[0]['id']
        else:
            vals_bairro = {'name': reg_bairro_nome}

            bairro_id = self.insert(table=TABLE, vals=vals_bairro)
        return bairro_id

    def registro_logradouro(self, reg_rua_nome):

        def acha_titulo(endereco):
            _endereco = endereco.upper()
            dic_titulo = {
                'RUA': 'Rua',
                'R': 'Rua',
                'ROD': 'Rodovia',
                'TR': 'Travessa',
                'PC': u'Praca',
                'PRACA': u'Praca',
                'BECO': 'Beco',
                'BC': 'Beco',
                'AV': 'Avenida'
            }
            _pos = 0
            _tit = ''
            _end = ''
            for x in dic_titulo:
                _tam = len(x)
                if _endereco[:_tam] == x:
                    _pos = _tam
                    _fim = _tam
                    _tit = dic_titulo[x]
                    break
            if _pos == 0:
                _end = endereco.strip()
            else:
                a = endereco[_pos:]
                _fim = _pos
                for x in a:
                    _fim += 1
                    if x in ' .-/:;=':
                        break
                _end = endereco[_fim:]
                _end = _end.strip()
                if _end[:1] == '.' or _end[:1] == ',' or _end[:1] == ';' or _end[:1] == ':':
                    _end = _end[1:]
                    _end = _end.strip()
            a = [_tit, _end]
            return a

        TABLE = 'adm_logradouro'

        titulo, nome_rua = acha_titulo(reg_rua_nome)

        args_logradouro = [('descricao', '=', nome_rua), ('titulo', '=', titulo)]
        logradouro_dados = self.select(table=TABLE, fields=['id'], where=args_logradouro)
        if logradouro_dados:
            logradouro_id = logradouro_dados[0]['id']
        else:

            vals_logradouro = {'descricao': nome_rua, 'titulo': titulo}

            logradouro_id = self.insert(table=TABLE, vals=vals_logradouro)

        return logradouro_id

    def registro_endereco_tipo(self, reg_end_name):
        TABLE = 'adm_endereco_tipo'

        end_tipo_dados = self.select(table=TABLE, fields=['id'], where=[('name', '=', reg_end_name)])
        if end_tipo_dados:
            endereco_tipo_id = end_tipo_dados[0]['id']
        else:
            vals_end_tipo = {'name': reg_end_name, 'bloqueio': True}
            endereco_tipo_id = self.insert(table=TABLE, vals=vals_end_tipo)

        return endereco_tipo_id

    def registro_fornecedor(self, reg_company_id, reg_pessoa_id, reg_ativo):
        TABLE = 'financeiro_fornecedor'

        args_fornecedor = [('pessoa_id', '=', reg_pessoa_id), ('company_id', '=', reg_company_id)]
        fornecedor_dados = self.select(table=TABLE, fields=['id'], where=args_fornecedor)
        if fornecedor_dados:
            fornecedor_id = fornecedor_dados[0]['id']
        else:
            vals_fornecedor = {
                'pessoa_id': reg_pessoa_id,
                'company_id': reg_company_id,
                'active': reg_ativo
            }
            fornecedor_id = self.insert(table=TABLE, vals=vals_fornecedor)

        return fornecedor_id

