#-*- encoding: utf-8 -*-
"""
API Python Rastreio - Rastreamento de Encomendas
Copyright (C) 2012  Leandro T. de Souza <leandrotoledo@member.fsf.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from zeep import Client, Transport
from requests import Session

from datetime import datetime
from __init__ import Encomendas

class SoapClient(Client):
    def __init__(self, wsdl, cert=None, verify=True, timeout=8, **kwargs):
        session = Session()
        session.cert = cert
        session.verify = verify
        session.timeout = timeout
        session.headers.update({'Content-Type': 'text/xml;charset=UTF-8'})

        transport = Transport(
            operation_timeout=timeout,
            session=session
        )

        super(SoapClient, self).__init__(wsdl=wsdl, transport=transport, **kwargs)

class Correios(object):

    def __init__(self, username, password, timeout=8):
        websro_url = 'https://webservice.correios.com.br/service/rastro/Rastro.wsdl'

        self.username = username
        self.password = password
        self.timeout = timeout

        self.websro_client = SoapClient(websro_url, timeout=self.timeout)
        self.websro = self.websro_client.service

    def get_tracking_code_events(self, tracking_list):

        response = self.websro.buscaEventos(self.username, self.password, "L", "T", "101", tracking_list)
        return response.objeto


def correios_get_rastreio(lista_codigo):
    corr = Correios('9912415012', 'P@XR3D<W<M')
    if not isinstance(lista_codigo, (tuple, list)):
        lista_codigo = [lista_codigo]
    encomendas = Encomendas()
    for codigo in lista_codigo:
        args = codigo.upper() if len(codigo) == 13 else ''
        if args:
           encomendas.setEncomenda(args)
    lista_codigo = encomendas.getEncomendaCodigos()
    qtd_encomendas = len(lista_codigo)
    i = 0
    while i < qtd_encomendas:
        args = ''
        for x in range(50): #Limite de codigos para rastreamento nos correios
            args += lista_codigo[i]
            i += 1
            if i == qtd_encomendas:
                break
        for res_corr in corr.get_tracking_code_events(args):
            if res_corr.erro:
                continue
            for x in res_corr.evento:
                _d = '%s %s' % (x.data, x.hora)
                __atualizacao = datetime.strptime(_d, '%d/%m/%Y %H:%M')
                __cidade = x.cidade
                __estado = x.uf
                __agencia = x.local
                __pais = 'BRASIL'
                __situacao = x.descricao
                __observacao = ''
                _origem = x.local or ''
                _origem += ('-' + x.cidade) if x.cidade else ''
                _origem += ('/' +x.uf) if x.uf else ''
                _destino = ''
                if x.destino and x.destino[0] != None:
                    _destino += x.destino[0].local if x.destino[0].local else ''
                    _destino += ('-' +x.destino[0].cidade) if x.destino[0].cidade else ''
                    _destino += ('/' +x.destino[0].uf) if x.destino[0].uf else ''
                __observacao = x.descricao
                if _origem and _destino:
                    __observacao += ' de ' + _origem + ' para ' + _destino
                else:
                    if _origem:
                        __observacao += ' ' + _origem
                    else:
                        __observacao += ' ' + _destino
                if x.detalhe:
                    __observacao += ' ' + x.detalhe
                if x.endereco:
                    _end = x.endereco.logradouro if x.endereco.logradouro else ''
                    _end += (' ' +x.endereco.numero) if x.endereco.numero else ''
                    _end += (' em ' + x.endereco.localidade) if x.endereco.localidade else ''
                    _end += ('/' + x.endereco.uf) if x.endereco.uf else ''
                    if _end:
                        __observacao += ' ' + _end
                if x.tipo in ('BDE', 'BDI', 'BDR') and x.status in ('00', '01'):
                    __fim = True
                else:
                    __fim = False
                encomendas.setEvento(res_corr.numero, __atualizacao,__pais, __estado, __cidade, __agencia, __situacao, __observacao, __fim)
    return encomendas.getEncomendas()


# a = correios_get_rastreio(('PT107167277BR','PT130368701BR','PT141698023BR'))
# pass