#-*- encoding: utf-8 -*-
"""
API Python Rastreio - Rastreamento de Encomendas
Copyright (C) 2012  Leandro T. de Souza <leandrotoledo@member.fsf.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


import re
from HTMLParser import HTMLParser
from datetime import datetime
import requests


class Status(object):

    def __init__(self, *args):
        args = [x.strip().replace('Origem:', 'De').replace('Destino:', 'Para').replace('Local:', '') for x in args]
        try:
            self.atualizacao = datetime.strptime(args[0], '%d/%m/%Y %H:%M')
        except:
            return
        if 'atrás' in args[len(args)-1]:
            args.pop(len(args)-1)
        self.situacao = args[1]
        self.agencia = self.cidade = self.estado = self.pais = None
        try:
            self.observacao = args[1] + ' ' + args[2] + ' ' + args[3]
        except:
            try:
                self.observacao = args[1] + ' ' + args[2]
            except:
                self.observacao = args[1]
        # try:
        #     if str(args[2].find('(')) != '-1':
        #         lista = args[2].split('-')
        #         lista = (args[4][args[4].rfind('(')+1:args[4].rfind(')')]).split('/')
        #         self.cidade = lista[0].strip()
        #         self.estado = lista[1].strip()
        # except:
        #     pass

class Encomenda(object):
    def __init__(self, identificador):
        self.identificador = identificador
        self.status = []

    def adicionar_status(self, status):
        self.status.append(status)
        self.status.sort(key=lambda status: status.atualizacao)

    def __str__(self):
        return repr(self.identificador)

class EncomendaParser(HTMLParser):
    def __init__(self, html):
        HTMLParser.__init__(self)
        self.in_td = False
        self.data = []
        self.feed(html)
    def handle_starttag(self, tag, attrs):
        if tag == 'td':
            self.in_td = True

    def handle_data(self, data):
        if self.in_td:
            self.data.append(data)

    def handle_endtag(self, tag):
        self.in_td = False

    # def handle_starttag(self, tag, attrs):
    #     if tag == 'ul':
    #         for x in attrs:
    #             if x == ('class', 'res'):
    #                 self.in_ul = True
    #     if self.in_ul and tag == 'li':
    #         self.in_li = True
    #
    # def handle_data(self, data):
    #     if self.in_ul:
    #         if self.in_li:
    #             self.data.append(data)
    #
    # def handle_endtag(self, tag):
    #     if tag == 'li':
    #         self.in_li = False
    #     if tag =='ul':
    #         self.in_ul = False


class Rastreio(object):
    URL = 'https://www.linkcorreios.com.br/%s'
    RE_DATE = re.compile(r'\d{2}/\d{2}/\d{4} \d{2}:\d{2}')

    def _get_encomenda_status(self, html):
        status = []
        if not '<tbody>' in html:
            return []
        a = html[html.index('<tbody>'):html.index('</tbody>') + 8]
        b = a.replace('\n',' ').replace('\r',' ').replace('\t',' ').replace('<br />','').replace('<strong>','').replace('</strong>','').replace('&nbsp;', ' ').replace('<label style="text-transform:capitalize;">', '')
        parser = EncomendaParser(b)
        data = parser.data
        parser.close()

        if not data:
            # raise RastreioException('Object not found')
            return status

        i = 0
        intervals = []
        for x in data:
            if Rastreio.RE_DATE.match(x):
                intervals.append(i)
            i = i+1
        # intervals = [data.index(x) for x in data if Rastreio.RE_DATE.match(x)]
        intervals.append(len(data))

        for x in intervals:
            try:
                status.append(Status(*data[x:intervals[intervals.index(x)+1]]))
            except IndexError:
                break

        if not status:
            # raise RastreioException('Status not found')
            status = []

        return status

    @staticmethod
    def get_encomenda(identificador):
        try:
            _url = Rastreio.URL % identificador
            request = requests.get(_url)
        except:
            return False
        html = request.content
        request.close()

        if html:
            encomenda = Encomenda(identificador)
            [encomenda.adicionar_status(status) for status in Rastreio()._get_encomenda_status(html)]

            return encomenda
        else:
            pass


class RastreioException(Exception):
    def __init__(self, code):
        self.code = code

    def __str__(self):
        return repr(self.code)

def rastreamentocorreios_get_rastreio(codigo):
    args = codigo.upper() if len(codigo) == 13 else ''
    res = {'status': False, 'status_erro': 'Sem definicao', 'rastro': []}
    if len(args) == 0:
        res['status_erro'] = 'Código de encomenda inválido\nO código deve ter 13 dígitos'
        return res
    a = Rastreio()
    b = a.get_encomenda(args)
    if not b:
        return False
    for e in b.status:
        _dict_rastro = {
            'data_hora': str(e.atualizacao),
            'pais': e.pais,
            'estado': e.estado,
            'cidade': e.cidade,
            'agencia': e.agencia,
            'situacao': e.situacao,
            'observacao': e.observacao,
        }
        res['rastro'].append(_dict_rastro)
        res['status'] = True
        res['status_erro'] = ''
    return res



