# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class wms_conferencia_itens(osv.Model):
    """
    Itens de conferencia.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.conferencia.itens'  # wmscit
    _inherit = 'wms.conferencia.itens'  # wms/conferencia/model/conferencia_itens_data

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Converte a quantidade expressa na embalagem em unidades.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param values: {dict} dicionario com o nome dos campos e seus respectivos valores,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} item de conferencia criado com a quantidade ajustada.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        qtde_na_embalagem = 0

        if 'wmscit_produto_embalagem_id' in values:
            embalagem_id = values['wmscit_produto_embalagem_id']
            dados_embalagem = obj_produto_embalagem.browse(cr, uid, embalagem_id)

            if dados_embalagem:
                qtde_na_embalagem = dados_embalagem.embalagem_id.quantidade_na_embalagem

            if 'wmscit_quantidade' in values:
                values['wmscit_quantidade'] *= qtde_na_embalagem

        result = super(wms_conferencia_itens, self).create(cr, uid, values, context)

        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Converte a quantidade expressa na embalagem em unidades

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param fields: {dict} dicionario com o nome dos campos e seus respectivos valores,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @param load:
        @return: {dict} dicionario com o nome dos campos e seus respectivos valores.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        if not 'wmscit_produto_embalagem_id' in fields:
            fields.append('wmscit_produto_embalagem_id')

        results = super(wms_conferencia_itens, self).read(
            cr, uid, ids, fields, context=context, load='_classic_write')

        for result in results:
            embalagem_id = result.get('wmscit_produto_embalagem_id')
            qtde_na_embalagem = 1
            dados_embalagem = obj_produto_embalagem.browse(cr, uid, embalagem_id)

            if dados_embalagem:
                qtde_na_embalagem = dados_embalagem.embalagem_id.quantidade_na_embalagem

            if 'wmscit_quantidade' in result:
                result['wmscit_quantidade'] /= qtde_na_embalagem

        return results

    def write(self, cr, uid, ids, values, context=None):
        """
        Converte a quantidade expressa na embalagem em unidades.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista de IDs,
        @param values: {dict} dicionario com os campos e seus respectivos valores,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} item de conferencia valor da quantidade ajustado em funcao da embalagem.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        qtde_na_embalagem = 0

        if 'wmscit_produto_embalagem_id' in values:
            embalagem_id = values.get('wmscit_produto_embalagem_id')
            dados_embalagem = obj_produto_embalagem.browse(cr, uid, embalagem_id)

            if dados_embalagem:
                qtde_na_embalagem = dados_embalagem.embalagem_id.quantidade_na_embalagem

            if 'wmscit_quantidade' in values:
                values['wmscit_quantidade'] *= qtde_na_embalagem

        result = super(wms_conferencia_itens, self).write(cr, uid, ids, values, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
