# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class separacao_os(osv.Model):
    """
    Cria os campos necessarios para a conferencia de itens.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'separacao.os'  # seposv
    _inherit = 'separacao.os'  # wms/separacao/inherit/os_data

    def seposv_onchange_codigo_barra(self, cr, uid, ids, codigo, separacao_itens, fator=1,
                                     context=None):
        """

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param codigo: {char} codigo de parra do produto/item,
        :param separacao_itens: {list} itens de separacao,
        :param fator: {int} fator de multiplicacao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} dados do produto ou mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from time import strftime

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if not ids or not codigo:
            return {}

        separacao_itens = self._seposv_format_itens(cr, uid, separacao_itens)

        result = {}

        # ___________________________ Objects ________________________________
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')
        obj_wms_conferencia_itens = self.pool.get('wms.conferencia.itens')
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        # Verifica se o fator de multiplicacao esta no codigo
        if '*' in codigo:
            fator = codigo[:codigo.find('*')].replace(',', '.', 1)
            try:
                fator = eval(fator)
            except:
                fator = 1

            codigo = codigo[codigo.find('*') + 1:]

            if not codigo:
                return result
        else:
            fator = 1

        # Verifica se o codigo de barras e valido
        args = [('codigo_barra', '=', codigo)]
        codigo_barra_ids = obj_automacao_codigo_barra.search(cr, uid, args, context=context)

        if not codigo_barra_ids:  # Produto nao cadastrado
            msg = _('Codigo de produto nao cadastrado. (%s)' % codigo)
            return {
                'value': {
                    'wmscit_codigo_barra': None,
                    'wmscit_fator': 1
                },
                'warning': {
                    'title': _('Aviso.'),
                    'message': msg
                }
            }
        else:
            codigo_barra_ids = codigo_barra_ids[0]

        # Verifica se a quantidade confere com a solicitada na OS
        brw_automacao_codigo_barra = obj_automacao_codigo_barra.browse(
            cr, uid, codigo_barra_ids, context)
        grade_id = brw_automacao_codigo_barra.grade_id.id
        produto = brw_automacao_codigo_barra.produto_id.name
        embalagem = brw_automacao_codigo_barra.produto_embalagem_id
        qtde_embalagem = embalagem.embalagem_id.quantidade_na_embalagem

        itens = filter(lambda i: i[2].get('grade_id') == grade_id
                                 and not i[2].get('item_conferido'), separacao_itens)

        if not itens:
            # Produto nao pertence a lista ou o item ja foi conferido
            msg = _('O produto %s (codigo %s) NAO pertence a lista de itens e deve ser '
                    'separado.' % (produto, codigo))

            return {
                'value': {
                    'wmscit_codigo_barra': None
                },
                'warning': {
                    'title': _('Aviso.'), 'message': msg
                }
            }
        else:
            item = itens[0]
            item_id = item[1]

        # Acresce a nova quantidade a ja conferida
        produto_embalagem_id = item[2].get('produto_embalagem_id')
        brw_estoque_produto_embalagem = obj_estoque_produto_embalagem.browse(
            cr, uid, produto_embalagem_id, context)
        item_qtde_na_embalagem = brw_estoque_produto_embalagem.embalagem_id.quantidade_na_embalagem
        item_quantidade = item[2].get('quantidade', 0) * item_qtde_na_embalagem
        qtde_conferida = item[2].get('wmssit_qtde_conferida_unidade', 0) + fator * qtde_embalagem
        qtde_cortada = item[2].get('wmssit_qtde_cortada_unidade', 0)

        # Verifica se a quantidade separada supera a quantidade que foi solicitada
        if qtde_conferida + qtde_cortada > item_quantidade:
            # A quantidade nao sera acrescentada as ja conferidas
            msg = _('A quantidade informada para o produto %s (%s), supera a quantidade '
                    'solicitada.' % (produto, codigo))
            status_conferencia = '>'
            result['warning'] = {'title': _('Aviso.'), 'message': msg}
        elif qtde_conferida + qtde_cortada == item_quantidade:
            status_conferencia = '='
            item[2].update({'item_conferido': True})
        else:
            status_conferencia = '<'

        # Atualiza os valores dos itens de separacao
        if status_conferencia != '>':
            item[2].update({'wmssit_qtde_conferida_unidade': qtde_conferida,
                            'wmssit_qtde_cortada_unidade': qtde_cortada,
                            'qtde_conferida': qtde_conferida / item_qtde_na_embalagem,
                            'qtde_cortada': qtde_cortada / item_qtde_na_embalagem,
                            'conferente_id': uid,
                            'data_conferencia': strftime('%Y-%m-%d %H:%M:%S')})

            for item_separacao in list(separacao_itens):
                if item_separacao[1] == item_id:  # Atualiza dados do item de separacao
                    item_separacao[2] = item[2]

                    separacao_itens[separacao_itens.index(item_separacao)] = item_separacao

                    break

        # Cria novo lancamento (historico)
        dados_conferencia = {
            'wmscit_separacao_id': ids[0],
            'wmscit_separacao_item_id': item_id,
            'wmscit_pessoa_id': item[2].get('pessoa_id'),
            'wmscit_produto_id': item[2].get('produto_id'),
            'wmscit_grade_id': item[2].get('grade_id'),
            'wmscit_conferente_id': uid,
            'wmscit_quantidade': fator,
            'wmscit_produto_embalagem_id': embalagem.id,
            'wmscit_data_conferencia': strftime('%Y-%m-%d %H:%M:%S'),
            'wmscit_status_conferencia': status_conferencia
        }

        obj_wms_conferencia_itens.create(cr, uid, dados_conferencia, context)

        # Atualiza os dados do formulario
        result['value'] = {
            'wmscit_codigo_barra': None,
            'wmscit_fator': 1,
            'separacao_item_ids': separacao_itens
        }

        return result

    def seposv_onchange_fator(self, cr, uid, ids, fator, context=None):
        """
        Ajuste do fator de multiplicacao da quantidade do produto e limpeza do campo codigo de
        barra do produto.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param fator: {int} fator de multiplicacao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} fator de multiplicacao ajustado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        try:
            fator = int(fator)
        except ValueError:
            fator = 1

        result = {
            'value': {
                'wmscit_fator': fator,
                'wmscit_codigo_barra': None
            }
        }

        return result

    def seposv_button_trocar_conferente_checkout(self, cr, uid, ids, context=None):
        """
        Troca o usuario que realiza a conferencia da ordem de servico de separacao (checkout)

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a troca de do conferente (checkout) foi realizada ou
        mensagen de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import time

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_separacao_os_historico = self.pool.get('separacao.os.historico')

        # Verifica se o usuario esta como responsavel pela ordem de servico para autorizar a troca
        # do conferente
        dados = {'conferente_id': uid}

        for separacao in self.browse(cr, uid, ids, context):
            if separacao.state == 'criada':  # Abre a ordem de servico
                dados['data_abertura'] = time.strftime('%Y-%m-%d %H:%M:%S')
                dados['state'] = 'aberta'

                # Registra a abertura da ordem de servico
                acao = _('Abrir ordem de servico')
                obj_separacao_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context)

            if uid not in separacao.responsavel_ids:
                dados['responsavel_ids'] = [[6, 0, [uid]]]

        # Registra o historico
        acao = 'Troca de usuario (checkout)'
        obj_separacao_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context)

        result = self.write(cr, uid, ids, dados, context)

        return result

    def seposv_button_reconferir(self, cr, uid, ids, context=None):
        """
        Refaz a conferencia dos itens. Apaga todos os lancamentos de conferencia e zera a
        quantidade conferida de todos os itens da ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se a lista de historio de conferencia foi excluida.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_separacao_os_historico = self.pool.get('separacao.os.historico')
        obj_wms_conferencia_itens = self.pool.get('wms.conferencia.itens')
        obj_wms_separacao_itens = self.pool.get('wms.separacao.itens')

        args = [('wmscit_separacao_id', 'in', ids)]
        result = False

        # Itens de conferencia
        item_ids = obj_wms_conferencia_itens.search(cr, uid, args, context=context)

        if item_ids:
            # Exclui todos os itens de conferencia da ordem de servico
            if obj_wms_conferencia_itens.unlink(cr, uid, item_ids, context):
                args = [('os_id', 'in', ids)]

                item_ids = obj_wms_separacao_itens.search(cr, uid, args, context=context)   # Itens de separacao
                if item_ids:
                    dados = {
                        'qtde_conferida': False,
                        'item_conferido': False,
                        'wmssit_qtde_conferida_unidade': False,
                    }

                    # Zera a quantidade conferida dos itens de separacao
                    if obj_wms_separacao_itens.write(cr, uid, item_ids, dados, context):
                        # Registra no historico a reconferencia
                        acao = 'Reconferir itens'
                        obj_separacao_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context)
                        result = True

        return result

    def _seposv_format_itens(self, cr, uid, listas, context=None):
        """
        Formatacao dos itens de separacao. Inclusao de campos necessarios a conferencia e ajuste
        das quantidades para unidades simples

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param listas: {list} lista com os dados dos itens de separacao
        :return: {list} lista dos itens de separacao com os dados de separacao atualizadaos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_separacao_itens = self.pool.get('wms.separacao.itens')

        for lista in list(listas):
            campos = ['conferente_id', 'data_conferencia', 'produto_embalagem_id', 'grade_id',
                      'qtde_conferida', 'wmssit_qtde_conferida_unidade', 'item_conferido',
                      'wmssit_qtde_cortada_unidade', 'quantidade', 'pessoa_id', 'produto_id',
                      'endereco_id']
            dados_item = obj_wms_separacao_itens.read(
                cr, uid, lista[1], campos, context=context, load='_classic_write')

            if dados_item and isinstance(dados_item, list):
                dados_item = dados_item[0]

            if dados_item and isinstance(dados_item, dict):
                lista_dados = {
                    'conferente_id': dados_item.get('conferente_id'),
                    'data_conferencia': dados_item.get('data_conferencia'),
                    'produto_embalagem_id': dados_item.get('produto_embalagem_id'),
                    'produto_id': dados_item.get('produto_id'),
                    'grade_id': dados_item.get('grade_id'),
                    'quantidade': dados_item.get('quantidade'),
                    'qtde_conferida': dados_item.get('qtde_conferida'),
                    'wmssit_qtde_conferida_unidade': dados_item.get('wmssit_qtde_conferida_unidade'),
                    'wmssit_qtde_cortada_unidade': dados_item.get('wmssit_qtde_cortada_unidade'),
                    'endereco_id': dados_item.get('endereco_id'),
                    'item_conferido': dados_item.get('item_conferido'),
                    'pessoa_id': dados_item.get('pessoa_id')
                }

                if lista[0] == 4:  # Atualiza dados do indice da lista (item de separacao)
                    listas[listas.index(lista)] = [1, lista[1], lista_dados]
                else:
                    lista_dados.update(lista[2])
                    listas[listas.index(lista)] = [1, lista[1], lista_dados]

        return listas

    # _________________________ ORM Methods __________________________________
    def fields_get(self, cr, uid, fields=None, context=None):
        """
        Acrescenta o campos extras necessarios na conferencia.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas estruturas de
        construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return:

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(separacao_os, self).fields_get(cr, uid, fields, context)
        result['wmscit_codigo_barra'] = {
            'selectable': True,
            'type': 'char',
            'size': 20,
            'string': 'Codigo do produto'
        }
        result['wmscit_fator'] = {
            'selectable': True,
            'type': 'char',
            'size': 3,
            'string': 'Fator'
        }

        return result

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False,
                        submenu=False):
        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')

        # Contexto para mostrar ou nao a quantidade
        campos = ['estpar_mostra_quantidade']
        dados_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)

        # Remove o campo "quantidade" de acordo com o parametro
        context['checkout_mostrar_quantidade'] = dados_parametro.get('estpar_mostra_quantidade')

        result = super(separacao_os, self).fields_view_get(
            cr, uid, view_id, view_type, context, toolbar, submenu)

        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param fields: {dict} dicionario com o nome dos campos e seus respectivos valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :param load:
        :return: {dict} nome dos campos e seus respectivos valores.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        results = super(separacao_os, self).read(cr, uid, ids, fields, context=context, load=load)

        for result in results if isinstance(results, list) else [results]:
            if not 'wmscit_fator' in result:
                result['wmscit_fator'] = 1

        return results

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
