# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br luiz@siginformatica.com.br
"""

from openerp.osv import osv, fields


class wms_conferencia_corte_itens(osv.Model):
    """
    Historico de corte dos itens.

    @author: Pierre Barbosa
    """

    _name = 'wms.conferencia.corte.itens'  # corite

    def corite_fields_function(self, cr, uid, ids, name, arg, context=None):
        """
        Manipulacao de campos. Valores de campos pre-definidos.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param name: {list} lista com o nome dos campos,
        @param arg: {dict} dicionario com os argumentos e valores,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} valores ajustados dos campos.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {}

        for item in self.browse(cr, uid, ids, context):
            try:
                if name == 'corite_grade':
                    valor = item.corite_grade_id.grade
                elif name == 'corite_embalagem':
                    valor = item.corite_produto_embalagem_id.embalagem_id.sigla
                elif name == 'corite_item':
                    valor = item.corite_item_separacao_id.id
                else:
                    valor = None
            except Exception as e:
                print '[erro] %s' % e
                valor = None

            result[item.id] = valor

        return result

    _columns = {
        # ___________________________ Many2One _______________________________
        'corite_separacao_id': fields.many2one('separacao.os', 'Mapa de separacao'),
        'corite_item_separacao_id': fields.many2one('wms.separacao.itens',  'Item de separacao'),
        'corite_pessoa_id': fields.many2one('adm.pessoa', 'Cliente'),  # Cliente ou loja
        'corite_produto_id': fields.many2one('estoque.produto', 'Produto', select=True),
        'corite_grade_id': fields.many2one('estoque.produto.grade', 'Grade', select=True),
        'corite_produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem'),

        # Usuario responsavel pelo corte
        'corite_supervisor_id': fields.many2one('res.users', 'Supervisor'),

        # ___________________________ Function _______________________________
        'corite_grade': fields.function(corite_fields_function, method=True, type='char', size=50,
                                        string='Grade'),
        'corite_embalagem': fields.function(corite_fields_function, method=True, type='char',
                                            size=30, string='Embalagem'),
        'corite_item': fields.function(corite_fields_function, method=True, type='integer',
                                       string='Item'),

        # ___________________________ Float __________________________________
        'corite_quantidade': fields.float('Quantidade', digits=(9, 3)),  # Quantidade cortada

        # ___________________________ Datetime _______________________________
        'corite_data_corte': fields.datetime('Data do corte'),

        # ___________________________ Boolean ________________________________
        'active': fields.boolean('Ativo')
    }

    _defaults = {
        'active': lambda *args: True
    }


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
