# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
import subprocess
import unicodedata
import base64
import tempfile


# noinspection PyPep8Naming
class wms_endereco(osv.Model):
    """
    Endreco de armazenamento.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.endereco'  # wmsend
    _inherit = 'wms.endereco'  # wms/control/endereco

    def wmsend_ocupar(self, cr, uid, ids, produto_id, grade_id, context=None):
        """
        Ocupa um endereco com um determinado produto/grade.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param produto_id: ID do produto,
        :param grade_id: ID da grade do produto,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type produto_id: int|long
        :type grade_id: int|long
        :type context: dict
        :return: verdadeiro se o endereco foi ocupado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Ocupa os enderecos
        dados = dict(produto_id=produto_id, grade_id=grade_id, state='O')
        result = self.write(cr, uid, ids, dados, context)

        return result

    def wmsend_liberar(self, cr, uid, ids, context=None):
        """
        Libera um endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o endereco foi liberado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        dados = dict(produto_id=False, grade_id=False, validade=False, state='L', bloqueado=False)
        result = self.write(cr, uid, ids, dados, context)

        return result

    def wmsend_valida_endereco(self, cr, uid, values, context=None):
        """
        Valida a criacao de um endereco

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param values: dicionario com os campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type values: dict
        :type context: dict
        :return: verdadeiro se o endereco e valido.
        :rtype: bool
        :raise: mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        virtual = context.get('virtual')
        deposito = values.get('deposito')
        rua = values.get('rua')
        predio = values.get('predio')
        nivel = values.get('nivel')
        apartamento = values.get('apartamento')
        title = _('Atencao!')

        if not deposito and not virtual:
            raise osv.except_osv(title, _('Deposito zero nao pode ser criado.'))

        if not rua and not virtual:
            raise osv.except_osv(title, _('Rua zero nao pode ser criada.'))

        if not predio and not virtual:
            raise osv.except_osv(title, _('Predio zero nao pode ser criado.'))

        if not apartamento and not virtual:
            raise osv.except_osv(title, _('Apartamento zero nao pode ser criado.'))

        # Lado
        values['lado'] = str(2 - predio % 2)

        # Verifica se o endereco ja foi criado
        args = [('deposito', '=', deposito), ('rua', '=', rua), ('predio', '=', predio),
                ('nivel', '=', nivel), ('apartamento', '=', apartamento)]

        if self.search(cr, uid, args, context=context):  # Nao recria o endereco
            endereco = '%03d.%03d.%03d.%03d.%03d' % (deposito, rua, predio, nivel, apartamento)

            if not context.get('Faixa'):
                raise osv.except_osv('Atencao!', _('O endereco %s ja existe.' % endereco))

            return False

        if (context.get('picking') and values.get('tipo_nivel') != 'PK') and not virtual:
            raise osv.except_osv(_('Atencao!'), _('E permitido somente enderecos do tipo picking.'))

        result = super(wms_endereco, self).create(cr, uid, values, context)

        return result

    def wmsend_imprimir_etiqueta(self, cr, uid, ids, context=None):
        """
        Imprime etiqueta do endereco (picking/pulmao).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se a etiqueta foi impressa.
        :rtype: bool
        :raise: mensagem de aviso informando sobre a configuracao da impressora (geralmente uma
        impressora termica, propria para impressao de etiquetas).

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')

        # Leitura dos parametros: impressora e modelo de formulario.
        campos = ['estpar_impressora_picking_id', 'estpar_impressora_pulmao_id',
                  'estpar_impressora_volume_id', 'modelo_etiqueta_picking',
                  'modelo_etiqueta_pulmao', 'estpar_descricao_grade']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)
        title = _('Atencao!')

        if not adm_utils_dados.get('modelo_etiqueta_picking'):
            raise osv.except_osv(title, _('O modelo de etiqueta para impressao nao foi definido.'))

        if not adm_utils_dados.get('modelo_etiqueta_pulmao'):
            raise osv.except_osv(title, _('O modelo de etiqueta para impressao nao foi definido.'))

        arquivo_etiqueta_pk = tempfile.NamedTemporaryFile(mode='w', delete=False)
        arquivo_etiqueta_pu = tempfile.NamedTemporaryFile(mode='w', delete=False)
        modelo_picking = adm_utils_dados.get('modelo_etiqueta_picking')  # Modelo etiqueta picking
        modelo_picking = base64.decodestring(modelo_picking)
        modelo_pulmao = adm_utils_dados.get('modelo_etiqueta_pulmao')  # Modelo etiqueta pulmao
        modelo_pulmao = base64.decodestring(modelo_pulmao)

        # Impressao das etiquetas de endereco
        impressora_id = adm_utils_dados.get('estpar_impressora_picking_id') or 0
        impressora_pk = self._get_label_printer(cr, uid, impressora_id)
        impressora_id = adm_utils_dados.get('estpar_impressora_pulmao_id') or 0
        impressora_pu = self._get_label_printer(cr, uid, impressora_id)

        if not impressora_pk:
            raise osv.except_osv(title, _('Nome da impressora para impressao de etiquetas de '
                                          'picking nao foi definido.'))

        if not impressora_pu:
            raise osv.except_osv(title, _('Nome da impressora para impressao de etiquetas de '
                                          'pulmao nao foi definido.'))

        for brw_endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            endereco_numero = brw_endereco.id  # Identificador {__endereco_numero__}
            endereco_nome = brw_endereco.name  # Identificador {__endereco_nome__}
            capacidade = brw_endereco.capacidade  # Identificador {__capacidade__}
            deposito = brw_endereco.deposito  # Identificador {__deposito__}
            rua = brw_endereco.rua  # Identificador {__rua__}
            predio = brw_endereco.predio  # Identificador {__predio__}
            nivel = brw_endereco.nivel  # Identificador {__nivel__}
            apartamento = brw_endereco.apartamento  # Identificador {__apartamento__}
            produto = brw_endereco.produto_id
            produto_codigo = produto.codigo  # Identificador {__produto_codigo__}
            tipo_endereco = brw_endereco.tipo_nivel

            if adm_utils_dados.get('estpar_descricao_grade') == 'G':
                produto_descricao = brw_endereco.grade_id.descricao or u''
            else:
                produto_descricao = brw_endereco.produto_id.name or u''

            produto = unicodedata.normalize('NFKD', produto_descricao)
            produto = produto.encode('ascii', 'ignore')  # Identificador {__produto__}
            grade = brw_endereco.grade_id
            norma_palete = grade.norma_palete  # Identificador {__norma_palete__}
            embalagem_master = grade.embalagem_master_id

            if produto and embalagem_master:
                emb_master = embalagem_master.embalagem_id.sigla  # Identificador {__emb_master__}
                embalagem_venda = grade.embalagem_expedicao_id

                if embalagem_venda:
                    emb_venda = embalagem_venda.embalagem_id.sigla  # Identificador {__emb_venda__}
                else:
                    emb_venda = ''
            else:
                emb_venda = emb_master = ''

            codigo_ean = grade.estpgr_ean  # Identificador {__codigo_ean__}
            codigo_dun = grade.estpgr_dun  # Identificador {__codigo_dun__}

            # Tipo de etiqueta
            if tipo_endereco == 'PK':
                etiqueta = modelo_picking
            else:
                etiqueta = modelo_pulmao

            # Formatacao da etiqueta
            etiqueta %= {
                '__endereco_numero__': endereco_numero,
                '__endereco_nome__': endereco_nome,
                '__capacidade__': capacidade,
                '__deposito__': deposito,
                '__rua__': rua,
                '__predio__': predio,
                '__nivel__': nivel,
                '__apartamento__': apartamento,
                '__produto__': produto,
                '__produto_codigo__': produto_codigo,
                '__norma_palete__': norma_palete,
                '__emb_master__': emb_master,
                '__emb_venda__': emb_venda,
                '__codigo_ean__': codigo_ean,
                '__codigo_dun__': codigo_dun
            }

            if tipo_endereco == 'PK':
                arquivo_etiqueta_pk.write(etiqueta)
            else:
                arquivo_etiqueta_pu.write(etiqueta)

        arquivo_etiqueta_pk.close()
        arquivo_etiqueta_pu.close()

        cmd = 'lp -d %s %s' % (impressora_pk, arquivo_etiqueta_pk.name)
        subprocess.Popen(cmd, shell=True)  # Impressao das etiquetas de picking

        cmd = 'lp -d %s %s' % (impressora_pu, arquivo_etiqueta_pu.name)
        subprocess.Popen(cmd, shell=True)  # Impressao das etiquetas de pulmao

        return True

    def _get_label_printer(self, cr, uid, printer, context=None):
        """
        Recupera o nome da impressora a ser utilizada para impressao da etiqueta.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param printer: ID da impressora
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type printer: int|long
        :type context: dict
        :return: nome da impressora.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        printer = [printer] if printer and isinstance(printer, (int, long)) else printer
        result = self.pool.get('wms.impressora.etiqueta').read(
            cr, uid, printer, ['name'], context=context)
        result = result[0].get('name') or None

        return result

    # noinspection PyMethodMayBeStatic
    def wmsend_onchange_deposito(self, cr, uid, ids, deposito, tipo, context=None):
        """
        Ajusta o endereco de acordo com o deposito e tipo do nivel.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param deposito: identificador do deposito que faz parte do endereco,
        :param tipo: identificador do tipo de nivel do endereco,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: valor corrigido do deposito.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if tipo not in ('PK', 'PU'):
            deposito = 99

        result = dict(value=dict(deposito=deposito))

        return result

    # noinspection PyMethodMayBeStatic
    def wmsend_onchange_rua(self, cr, uid, ids, rua, tipo, context=None):
        """
        Ajusta o endereco de acordo com a rua e o tipo do nivel.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param rua: identificador da rua que faz parte do endereco,
        :param tipo: identificador do tipo de nivel do endereco,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type rua: str
        :type tipo: str
        :type context: dict
        :return: valor corrigido da rua.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if tipo not in ('PK', 'PU'):
            rua = 99

        result = dict(value=dict(rua=rua))

        return result

    # noinspection PyMethodMayBeStatic
    def wmsend_onchange_predio(self, cr, uid, ids, predio, tipo, context=None):
        """
        Ajusta o endereco de acordo com o predio e o tipo do nivel.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param predio: identificador do predio que faz parte do endereco,
        :param tipo: identificador do tipo de nivel do endereco,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type predio: str
        :type tipo: str
        :type context: dict
        :return: valor corrigido do predio.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if tipo not in ('PK', 'PU'):
            predio = 99

        result = dict(value=dict(predio=predio))

        return result

    # noinspection PyMethodMayBeStatic
    def wmsend_onchange_nivel(self, cr, uid, ids, nivel, tipo, context=None):
        """
        Ajusta o endereco de acordo com nivel e o tipo do nivel.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param nivel: identificador do nivel que faz parte do endereco,
        :param tipo: identificador do tipo de nivel do endereco,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type nivel: str
        :type tipo: str
        :type context: dict
        :return: valor corrigido do nivel.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if tipo not in ('PK', 'PU'):
            nivel = 99

        result = dict(value=dict(nivel=nivel))

        return result

    def wmsend_onchange_tipo_nivel(self, cr, uid, ids, nivel, tipo, context=None):
        """
        Avisa se nivel criado esta fora da faixa padrao para picking (nivel = 0) ou se fora da
        faixa padrao para pulmao (nivel diferente de 0)

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param nivel: nivel a ser verificado,
        :param tipo: tipo a ser verificado. 'PK' = picking, 'PU' pulmao ou 'VT' = Virtual,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type nivel: int
        :type tipo: str
        :type context: dict
        :return: dados do endereco de armazenagem.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = self.wmsend_verifica_tipo_nivel(cr, uid, ids, nivel, tipo, context)

        return result

    def wmsend_verifica_tipo_nivel(self, cr, uid, ids, nivel, tipo, context=None):
        """
        Verifica se nivel esta dentro ou fora da faixa padrao de picking/Pulmao

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param nivel: nivel a ser verificado,
        :param tipo: tipo a ser verificado. 'PK' = picking, 'PU' pulmao ou 'VT' = Virtual,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type nivel: int
        :type tipo: str
        :type context: dict
        :return: valores corrigidos do nivel e do tipo.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Ajusta endereco (avaria/virtual)
        deposito = 1

        if tipo == 'VT':
            # Pesquisa pelo maior endereco de apartamento.
            rua = predio = nivel = 99
            args = [('rua', '=', 99), ('predio', '=', 99),
                    ('nivel', '=', 99)]
            ordem = 'apartamento desc'
        else:
            rua = 1
            predio = 1
            nivel = 0 if tipo == 'PK' else 1
            args = [('tipo_nivel', '=', tipo)]
            ordem = 'deposito, rua, predio, nivel, apartamento desc'

        endereco_ids = self.search(cr, uid, args, order=ordem, limit=1, context=context)

        if not endereco_ids:
            apartamento = 1
        else:
            endereco = self.browse(cr, uid, endereco_ids, context)[0]
            deposito = endereco.deposito
            rua = endereco.rua
            predio = endereco.predio
            nivel = endereco.nivel
            apartamento = endereco.apartamento + 1

        result = dict(value=dict(deposito=deposito, rua=rua, predio=predio, nivel=nivel,
                                 apartamento=apartamento))

        return result

    def wmsend_bloquear(self, cr, uid, ids, context=None):
        """
        Bloqueio de enderecos de armazenagem.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se conseguiu realizar o bloqueio.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Por padrao nao bloqueia enderecos do tipo picking (PK)
        if not context.get('lock_picking'):
            args = [('id', 'in' if isinstance(ids, list) else '=', ids), ('tipo_nivel', '!=', 'PK')]
            ids = self.search(cr, uid, args, context=context)

        result = self.write(cr, uid, ids, {'bloqueado': True}, context=context)

        return result

    def wmsend_desbloquear(self, cr, uid, ids, context=None):
        """
        Desbloqueia um determinado endereco

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se conseguiu realizar o desbloqueio.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids, {'bloqueado': False}, context=context)

        return result

    def wmsend_criar_virtual(self, cr, uid, virtual=99, reaproveita=True, context=None):
        """
        Criar um endereco virtual.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param virtual: numero do virtual,
        :param reaproveita: reaproveita enderecos virtuais ja criados e livres,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: integer
        :type virtual: integer
        :type reaproveita: bool
        :type context: dict
        :return: o ID do novo endereco virtual que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        ordem = 'deposito, rua, predio, nivel, apartamento'  # Ordem crescente dos enderecos

        # Procura por endereco virtual vago ou nao
        if reaproveita:  # Reaproveita: utiliza um endereco ja criado (livre)
            args = [('deposito', '=', virtual), ('bloqueado', '=', False), ('state', '=', 'L')]
            endereco_ids = self.search(cr, uid, args, limit=1, order=ordem, context=context)

            if endereco_ids:
                return endereco_ids[0]

        # Se nao encontrou endereco livre cria um novo
        args = [('deposito', '=', virtual), ('rua', '=', virtual),
                ('predio', '=', virtual), ('nivel', '=', virtual)]
        endereco_ids = self.search(cr, uid, args, order=ordem, context=context)
        apto = 1

        if endereco_ids:
            dados_endereco = self.read(cr, uid, endereco_ids[-1], ['apartamento'], context=context)

            if dados_endereco and isinstance(dados_endereco, list):
                dados_endereco = dados_endereco[0]

            if dados_endereco and isinstance(dados_endereco, dict):
                apto = dados_endereco.get('apartamento') or 0
                apto += 1

        dados = dict(deposito=virtual, rua=virtual, predio=virtual, nivel=virtual, apartamento=apto,
                     tipo_nivel='VT')
        context['virtual'] = True  # Para permitir a criacao do endereco virtual
        result = self.create(cr, uid, dados, context=context)  # Cria o novo endereco

        return result

    def wmsend_saldo(self, cr, uid, ids, context=None):
        """
        Calcula o saldo do endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: valor total do saldo.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = 0

        # Nao contabiliza os enderecos de avaria
        args = [('id', 'in' if isinstance(ids, list) else '=', ids), ('deposito', '!=', 98)]
        ids = self.search(cr, uid, args, context=context)

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            result += endereco.quantidade + endereco.reserva_credito - endereco.reserva_debito

        return result

    def wmsend_subtrair_saldo(self, cr, uid, ids, valor, context=None):
        """
        Subtrai o valor da reserva debito e tambem da quantidade.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param valor: valor a ser subtraido da quantidade no endereco,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: lista dos enderecos.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []  # Lista com o ID resultado das alteracoes no saldo

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            debito = endereco.reserva_debito - valor
            quantidade = endereco.quantidade - valor
            credito = endereco.reserva_credito
            saldo = quantidade + credito - debito
            dados = {'quantidade': quantidade}

            if context.get('debito', True):  # Extorna ou nao o valor do debito
                dados['reserva_debito'] = debito

            # Verifica a possibilidade de liberaracao do endereco. Faz o reaproveitamento de
            # enderecos de pulmao e virtuais
            if saldo <= 0:
                dados['validade'] = False

                if endereco.tipo_nivel != 'PK':
                    dados.update(dict(produto_id=False, grade_id=False, state='L', bloqueado=False))
                else:  # Picking. Libera se for dinamico ou for movimentacao de picking para picking
                    if endereco.picking_dinamico or context.get('release'):
                        # Verifica se existe saldo do produto para liberar ou nao o picking
                        args = [('grade_id', '=', endereco.grade_id.id)]
                        wms_endereco_ids = self.search(cr, uid, args, context=context)
                        wms_endereco_dados = self.read(
                            cr, uid, wms_endereco_ids, ['saldo'], context=context)

                        # Verifica se existe saldo do produto
                        if not sum(map(lambda m: m.get('saldo', 0),
                                       wms_endereco_dados)) or context.get('release'):
                            dados.update(dict(produto_id=False, grade_id=False, state='L',
                                              bloqueado=False))

            self.write(cr, uid, endereco.id, dados, context)

            result.append(dict(id=endereco.id, saldo_anterior=endereco.saldo, saldo_atual=saldo))

        return result

    def wmsend_somar_saldo(self, cr, uid, ids, valor, validade=None, context=None):
        """
        Subtrai o valor da reserva e o acrescenta a quantidade.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param valor: valor a ser somado a quantidade do endereco,
        :param validade: {date} data de validade,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type valor: int
        :type validade: date
        :type context: dict
        :return: verdadeiro se todos os valores de reserva foram realizados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            dados = dict(reserva_credito=endereco.reserva_credito - valor,
                         quantidade=endereco.quantidade + valor)

            # Tenta a troca da validade
            if validade:
                if not endereco.validade or endereco.validade > validade:
                    dados['validade'] = validade

            result.append(self.write(cr, uid, endereco.id, dados, context))

        return result

    def wmsend_atualizar_saldo(self, cr, uid, ids, context=None):
        """
        Atualiza o saldo do endereco. Move o saldo para a quantidade e zera as
        reservas (credito e debito).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se todos saldos foram atualizados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        liberados = []

        for endereco in self.browse(cr, uid, ids, context):
            credito = endereco.reserva_credito
            debito = endereco.reserva_debito
            quantidade = endereco.quantidade + credito - debito
            dados = dict(reserva_credito=0, reserva_debito=0, quantidade=quantidade)

            # Enderecos para serem liberados
            if credito - debito == 0:
                if endereco.tipo_nivel == 'PK':
                    # Se for picking dinamico e o saldo for zero, libera o endereco
                    if endereco.picking_dinamico:
                        liberados.append(endereco.id)
                else:
                    liberados.append(endereco.id)  # O endereco sera liberado

            # Atualiza o saldo
            self.write(cr, uid, endereco.id, dados, context)

        # Libera os enderecos com saldo igual a zero
        result = self.wmsend_liberar(cr, uid, liberados, context)

        return result

    def wmsend_reserva_credito(self, cr, uid, ids, produto_id, grade_id, reserva=0, validade=False,
                               context=None):
        """
        Reserva a quantidade a ser somada ao saldo do picking.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param produto_id: ID do produto,
        :param validade: {date} data de validade do produto
        :param grade_id: ID da grade do produto,
        :param reserva: valor a ser reservado para credito,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type produto_id: long
        :type validade: date
        :type grade_id: long
        :type reserva: int
        :type context: dict
        :return: verdadeiro se todos os valores de credito foram realizados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            # Somente se produto e grade sao os mesmos
            if endereco.produto_id and endereco.produto_id.id != produto_id:
                continue

            if endereco.grade_id and endereco.grade_id.id != grade_id:
                continue

            credito = endereco.reserva_credito + reserva

            # Ocupa o endereco
            dados = dict(produto_id=produto_id, grade_id=grade_id, state='O',
                         reserva_credito=credito)

            if validade:  # Somente se a validade for informada!
                # Menor validade
                if not endereco.validade or endereco.validade > validade:
                    dados['validade'] = validade
            else:  # Assume a data de hoje se a validade for forcada
                if context.get('push_date'):
                    import time

                    dados['validade'] = time.strftime('%Y-%m-%d')

            result.append(self.write(cr, uid, endereco.id, dados, context))

            # Verifica a possibilidade para liberar o endereco. Faz o reaproveitamento de
            # enderecos
            if credito <= 0 and (endereco.tipo_nivel != 'PK' or endereco.picking_dinamico):
                if (endereco.saldo + reserva) <= 0:
                    self.wmsend_liberar(cr, uid, endereco.id, context)

        return all(result)

    def wmsend_reserva_debito(self, cr, uid, ids, reserva=0, context=None):
        """
        Reserva a quantidade a ser subtraidada do saldo do picking.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param reserva: valor a ser reservado para debito,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type reserva: int
        :type context: dict
        :return: verdadeiro se todos os valores de debito foram realizados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            dados = dict(reserva_debito=endereco.reserva_debito + reserva)

            result.append(self.write(cr, uid, endereco.id, dados, context))

        return all(result)

    def wmsend_disponivel(self, cr, uid, ids, context=None):
        """
        Informa a disponibilidade (para uso) de um determinado endereco. Para
        estar disponivel o endereco deve estar com saldo = 0, state = 'L', blo-
        queado = False.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o endereco estiver disponivel.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        enderecos = filter(lambda e: e.quantidade != 0 or e.reserva_credito != 0
                           or e.reserva_debito != 0 or e.bloqueado,
                           self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context))

        result = not any(map(lambda e: e.id, enderecos))

        return result

    def wmsend_acha_pulmao_livre(self, cr, uid, deposito, rua, predio, nivel_min, nivel_max, tipo,
                                 estrutura, endereco_utilizado=None, limite=1):
        """
        Procura por endereco livre para armazenamento

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param deposito: numero do deposito utilizado na busca,
        :param rua: numero da rua utilizado na busca,
        :param predio: numero do predio utilizado na busca,
        :param nivel_min: nivel minimo para um endereco valido na busca,
        :param nivel_max: nivel maximo para um endereco valido na busca,
        :param tipo: ID que identifica o tipo de endereco,
        :param estrutura: ID que identifica o tipo de estrutura,
        :param endereco_utilizado: lista com os IDs dos enderecos a serem descartados. Ja
        utilizados,
        :param limite: a quantidade limite de IDs que serao retornados pela busca,
        :type cr: cursor
        :type uid: int
        :type deposito: int
        :type rua: int
        :type predio: int
        :type nivel_min: int
        :type nivel_max: int
        :type tipo: int|long
        :type estrutura: int|long
        :type endereco_utilizado: list[int|long]
        :return: lista com os IDs dos enderecos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if isinstance(endereco_utilizado, (int, long)):
            endereco_utilizado = [endereco_utilizado]
        elif not isinstance(endereco_utilizado, list):
            endereco_utilizado = []

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        args = [('deposito', '=', deposito), ('rua', '=', rua), ('predio', '=', predio),
                ('tipo_nivel', '=', 'PU'), ('endereco_tipo_id', '=', tipo),
                ('estrutura_tipo_id', '=', estrutura), ('state', '=', 'L'),
                ('bloqueado', '=', False)]

        if endereco_utilizado:
            args.append(('id', 'not in', endereco_utilizado))

        # Restricoes de nivel
        if nivel_min:
            args.append(('nivel', '>=', nivel_min))

        if nivel_max:
            args.append(('nivel', '<=', nivel_max))

        ordem = 'rua, predio, nivel, apartamento'
        endereco_ids = obj_wms_endereco.search(cr, uid, args, limit=limite, order=ordem, context=context)

        if not endereco_ids:  # Nao achou endereco Virtual livre
            return False

        return endereco_ids

    def wmsend_alterar_tipo_estrutura(self, cr, uid, ids, tipo_id, context=None):
        """
        Atera o tipo de estrutura de um endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs dos enderecos,
        :param tipo_id: ID do tipo de estrutura. Ex. Porta palete, escaninho, etc.,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type tipo_id: int|long
        :type context: dict
        :return: verdadeiro se o tipo de estrutura do endereco foi alterada.
        :rtype: bool


        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids, {'estrutura_tipo_id': tipo_id}, context)

        return result

    def wmsend_alterar_tipo_endereco(self, cr, uid, ids, tipo_id, context=None):
        """
        Atera o tipo de endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs dos enderecos,
        :param tipo_id: ID do tipo de endereco. Ex. Inteiro, metade, etc.,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type tipo_id: int|long
        :type context: dict
        :return: verdadeiro se o tipo de endereco foi alterado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids, {'endereco_tipo_id': tipo_id}, context)

        return result

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Sobescreve o metodo 'create' para validar a criacao de um endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param values: dicionario com os campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type values: dict
        :type context: dict
        :return: verdadeiro se sucesso na criacao do endereco.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        return self.wmsend_valida_endereco(cr, uid, values, context)

    def name_search(self, cr, uid, name='', args=None, operator='ilike', context=None, limit=10):
        """
        Procura o endereco pelo ID  ou pelo nome/descricao (name).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param name: nome do objeto utilizado na busca,
        :param args: lista de argumentos utilizado na busca,
        :param operator: operador utilizado pelo criterio de busca,
        :param context: um dicionario padrao para valores contextuais,
        :param limit: quantidade maxima de IDs que serao retornados,
        :type cr: cursor
        :type uid: int
        :type name: str
        :type operator: str
        :type context: dict
        :return: lista com os IDs que satisfizeram os parametros da busca.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        args = [] if not args else args
        ids = False

        if name.isdigit():
            args += [['id', '=', int(name)]]
            ids = self.search(cr, uid, args, limit=limit, context=context)

        if not ids:
            args += [['name', '>=', name]]
            ids = self.search(cr, uid, args, limit=limit, context=context)

        result = self.name_get(cr, uid, ids, context)

        return result

    def unlink(self, cr, uid, ids, context=None):
        """
        Verifica a condicao de exclusao do endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se o registro excluido.
        :rtype: bool
        :raise: mensagem de aviso se o tipo de endereco for dinamico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        for endereco in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            # Nao exclui enderecos ocupados, bloqueados ou com saldo
            if endereco.state == 'O' or endereco.bloqueado or endereco.saldo > 0:
                ids.remove(endereco.id)
                continue

            if endereco.picking_dinamico:  # Informa a exclusao de endereco dinamico (saldo zero)
                msg = 'Endereco dinamico %s foi excluido.' % endereco.name
                self.log(cr, uid, endereco.id, msg)

        result = super(wms_endereco, self).unlink(cr, uid, ids, context)

        return result

    def wmsend_norma_quantidade(self, cr, uid, grade_id, quantidade, context=None):
        """
        Retorna a lista dos enderecos com saldo iqual a norma palete de um produto.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param grade_id: ID da grade do produto,
        :param quantidade: quantidade limite para equivalencia da norma,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type grade_id: int|long
        :type quantidade: int
        :type context: dict
        :return: lista com ID dos enderecos com saldo igual a norma.
        :rtype: list[long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')

        # Norma palete do produto
        qtde_na_norma = 0
        brw_produto_grade = obj_estoque_produto_grade.browse(cr, uid, grade_id, context)

        if brw_produto_grade:
            master = brw_produto_grade.embalagem_master_id
            qtde_na_master = master.embalagem_id.quantidade_na_embalagem
            norma = brw_produto_grade.lastro * brw_produto_grade.camada
            qtde_na_norma = norma * qtde_na_master   # Quantidade em unidades simples
        else:
            return []

        # Enderecos
        args = [('tipo_nivel', '!=', 'PK'), ('bloqueado', '=', False), ('state', '=', 'O'),
                ('grade_id', '=', grade_id), ('deposito', '!=', 98)]
        ordem = 'validade, name'
        endereco_ids = self.search(cr, uid, args, order=ordem, context=context)
        campos = ['id', 'saldo']
        dados_endereco = self.read(cr, uid, endereco_ids, campos, context=context)
        result = []

        if dados_endereco and isinstance(dados_endereco, list):
            result = map(lambda i: i.get('id'), filter(
                lambda d: d.get('saldo') == qtde_na_norma, dados_endereco))
        elif dados_endereco and isinstance(dados_endereco, dict):
            result = [dados_endereco.get('id', False)]

        # Maximo de enderecos baseado na quantidade/norma
        result = result[:int(quantidade/qtde_na_norma)]

        return result

    def wmsend_virtuais_ocupados(self, cr, uid, context=None):
        """
        Retorna os enderecos virtuais ocupados.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type context: dict
        :return: lista com o ID dos enderecos virtuais.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        args = [('deposito', '=', 99), ('rua', '=', 99), ('predio', '=', 99), ('nivel', '=', 99),
                ('bloqueado', '=', False), ('state', '=', 'O'), ('reserva_credito', '=', 0),
                ('reserva_debito', '=', 0)]
        ordem = 'rua, predio, nivel, apartamento'

        # lista dos enderecos virtuais ocupados
        result = self.search(cr, uid, args, order=ordem, context=context)

        return result

    def wmsend_pulmao_livre(self, cr, uid, context=None):
        """
        Retorna os enderecos de pulmao livre.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type cr: cursor
        :type uid: int
        :type context: dict
        :return: a lista dos ID's dos enderecos ocupados.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        args = [('tipo_nivel', '=', 'PU'), ('bloqueado', '=', False), ('state', '=', 'L')]
        ordem = 'rua, predio, nivel, apartamento'

        # lista de enderecos de pulmao livre encontrados
        result = self.search(cr, uid, args, order=ordem, context=context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
