# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


# noinspection PyPep8Naming
class pgview_wms_controle_ordem_servico(osv.Model):

    _name = "view.wms.controle.ordem.servico"
    _auto = False
    _columns = {
        # ___________________________ Integer ________________________________
        'id': fields.integer('id'),

        # ___________________________ Char ___________________________________
        'wmscos_numero': fields.char('Nr', size=11),
        'wmscos_descricao': fields.char('Descricao', size=60),
        'wmscos_state': fields.char('Situacao', size=15),
        'wmscos_tipo': fields.char('Tipo', size=40),
        'wmscos_responsavel': fields.char('Responsavel/conferente', size=64),

        # ___________________________ Datetime _______________________________
        'wmscos_data_criacao': fields.datetime('Data de criacao'),
        'wmscos_data_abertura': fields.datetime('Data de abertura'),
        'wmscos_data_fechamento': fields.datetime('Data de fechamento'),
        'wmscos_data_cancelamento': fields.datetime('Data de cancelamento'),
    }

    @staticmethod
    def init(cr):
        """
        Cria a ´view´ que lista todas as ordens de servico.

        :param cr: a linha atual do cursor do banco de dados
        :type cr: Cursor
        :return: N/A

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        drop_view_if_exists(cr, 'view_wms_controle_ordem_servico')
        cr.execute(
            """
            CREATE VIEW view_wms_controle_ordem_servico AS (
                /* PSQL view schema
                 * Todas as ordens de servico no WMS.
                 * (C) 2014 - by Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br
                 *
                 * Return:
                 *  id                          bigint
                 *  wmscos_numero               char(11)    // Numero da ordem de servico
                 *  wmscos_descricao            char(60)
                 *  wmscos_state                char(15)    // Situacao
                 *  wmscos_tipo                 char(40)    // Tipo da ordem de servico
                 *  wmscos_responsavel          char(64)    // Responsavel ou conferente
                 *  wmscos_data_criacao         datetime
                 *  wmscos_data_abertura        datetime
                 *  wmscos_data_fechamento      datetime
                 *  wmscos_data_cancelamento    datetime
                 *
                 */
                SELECT  row_number() OVER ()    AS id,
                        *
                FROM    (
                SELECT  FORMAT('%s-%s', EXTRACT(YEAR FROM osr.data_criacao),
                        LPAD(osr.id::text, 6, '0')) AS wmscos_numero,
                        osr.descricao               AS wmscos_descricao,
                        CASE WHEN osr.state         = 'fechada'     THEN 'Fechada'
                             WHEN osr.state         = 'aberta'      THEN 'Aberta'
                             WHEN osr.state         = 'criada'      THEN 'Criada'
                             WHEN osr.state         = 'cancelada'   THEN 'Cancelada'
                             ELSE                   'Esboco'
                        END                         AS wmscos_state,
                        tos.name                    AS wmscos_tipo,
                        pes.nome                    AS wmscos_responsavel,
                        osr.data_criacao            AS wmscos_data_criacao,
                        osr.data_abertura           AS wmscos_data_abertura,
                        osr.data_fechamento         AS wmscos_data_fechamento,
                        osr.data_cancelamento       AS wmscos_data_cancelamento
                FROM    recebimento_os              AS osr
                    INNER JOIN  os_tipo             AS tos  ON tos.id = osr.tipo_id
                    INNER JOIN  res_users           AS usr  ON usr.id = osr.conferente_id
                    INNER JOIN  adm_pessoa          AS pes  ON pes.id = usr.pessoa_id
                UNION
                SELECT  FORMAT('%s-%s', EXTRACT(YEAR FROM ost.data_criacao),
                        LPAD(ost.id::text, 6, '0')) AS wmscos_numero,
                        ost.descricao               AS wmscos_descricao,
                        CASE WHEN ost.state         = 'fechada'     THEN 'Fechada'
                             WHEN ost.state         = 'aberta'      THEN 'Aberta'
                             WHEN ost.state         = 'criada'      THEN 'Criada'
                             WHEN ost.state         = 'cancelada'   THEN 'Cancelada'
                             ELSE                   'Esboco'
                        END                         AS wmscos_state,
                        tos.name                    AS wmscos_tipo,
                        pes.nome                    AS wmscos_responsavel,
                        ost.data_criacao            AS wmscos_data_criacao,
                        ost.data_abertura           AS wmscos_data_abertura,
                        ost.data_fechamento         AS wmscos_data_fechamento,
                        ost.data_cancelamento       AS wmscos_data_cancelamento
                FROM    transferencia_os            AS ost
                    INNER JOIN  os_tipo             AS tos  ON tos.id = ost.tipo_id
                    INNER JOIN  res_users           AS usr  ON usr.id = ost.conferente_id
                    INNER JOIN  adm_pessoa          AS pes  ON pes.id = usr.pessoa_id
                UNION
                SELECT  FORMAT('%s-%s', EXTRACT(YEAR FROM oss.data_criacao),
                        LPAD(oss.id::text, 6, '0')) AS wmscos_numero,
                        oss.descricao               AS wmscos_descricao,
                        CASE WHEN oss.state         = 'fechada'     THEN 'Fechada'
                             WHEN oss.state         = 'aberta'      THEN 'Aberta'
                             WHEN oss.state         = 'criada'      THEN 'Criada'
                             WHEN oss.state         = 'cancelada'   THEN 'Cancelada'
                             ELSE                   'Esboco'
                        END                         AS wmscos_state,
                        tos.name                    AS wmscos_tipo,
                        pes.nome                    AS wmscos_responsavel,
                        oss.data_criacao            AS wmscos_data_criacao,
                        oss.data_abertura           AS wmscos_data_abertura,
                        oss.data_fechamento         AS wmscos_data_fechamento,
                        oss.data_cancelamento       AS wmscos_data_cancelamento
                FROM    separacao_os                AS oss
                    INNER JOIN  os_tipo             AS tos  ON tos.id = oss.tipo_id
                    INNER JOIN  res_users           AS usr  ON usr.id = oss.conferente_id
                    INNER JOIN  adm_pessoa          AS pes  ON pes.id = usr.pessoa_id
                UNION
                SELECT  FORMAT('%s-%s', EXTRACT(YEAR FROM osi.data_criacao),
                        LPAD(osi.id::text, 6, '0')) AS wmscos_numero,
                        osi.descricao               AS wmscos_descricao,
                        CASE WHEN osi.state         = 'fechada'     THEN 'Fechada'
                             WHEN osi.state         = 'aberta'      THEN 'Aberta'
                             WHEN osi.state         = 'criada'      THEN 'Criada'
                             WHEN osi.state         = 'cancelada'   THEN 'Cancelada'
                             ELSE                   'Esboco'
                        END                         AS wmscos_state,
                        tos.name                    AS wmscos_tipo,
                        pes.nome                    AS wmscos_responsavel,
                        osi.data_criacao            AS wmscos_data_criacao,
                        osi.data_abertura           AS wmscos_data_abertura,
                        osi.data_fechamento         AS wmscos_data_fechamento,
                        osi.data_cancelamento       AS wmscos_data_cancelamento
                FROM    inventario_os               AS osi
                    INNER JOIN  os_tipo             AS tos  ON tos.id = osi.tipo_id
                    INNER JOIN  res_users           AS usr  ON usr.id = osi.conferente_id
                    INNER JOIN  adm_pessoa          AS pes  ON pes.id = usr.pessoa_id
            )                                       AS view_wms_controle_os_tmp)
            """
        )

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4

