# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.report import report_sxw
from datetime import datetime
from pytz import timezone
from openerp.tools.translate import _


# noinspection PyPep8Naming
class report_controle_ordem_servico(report_sxw.rml_parse):
    """
    Relacao das ordens de servico.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context=None):
        """
        Atribui valores as funcoes utilizada no relatorio.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param name: nome do modelo utilizado,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: integer
        :type name: char|list
        :return: impressao do relatorio.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        super(report_controle_ordem_servico, self).__init__(cr, uid, name, context)

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))

        # Periodo inicial e final. Assume-se a menor data de criacao e e maior data de fechamento/
        # cancelamento
        args = [('id', 'in', context.get('active_ids'))]
        menor_data_id = self.pool.get('view.wms.controle.ordem.servico').search(
            cr, uid, args, order='wmscos_data_criacao', limit=1, context=context)
        menor_data_dados = self.pool.get('view.wms.controle.ordem.servico').read(
            cr, uid, menor_data_id, ['wmscos_data_criacao'], context=context)

        if menor_data_dados and isinstance(menor_data_dados, list):
            menor_data_dados = menor_data_dados[0]

        menor_data = menor_data_dados.get('wmscos_data_criacao')

        # Maior data de fechamento
        args += [('wmscos_data_fechamento', '!=', False)]
        data_fechamento_id = self.pool.get('view.wms.controle.ordem.servico').search(
            cr, uid, args, order='wmscos_data_fechamento desc', limit=1, context=context)
        maior_data_dados = self.pool.get('view.wms.controle.ordem.servico').read(
            cr, uid, data_fechamento_id, ['wmscos_data_fechamento'], context=context)

        if maior_data_dados and isinstance(maior_data_dados, list):
            maior_data_dados = maior_data_dados[0]

        if isinstance(maior_data_dados, dict):
            data_fechamento = maior_data_dados.get('wmscos_data_fechamento')
        else:
            data_fechamento = None

        # Maior data de cancelamento
        args.pop()
        args += [('wmscos_data_cancelamento', '!=', False)]
        data_cancelamento_id = self.pool.get('view.wms.controle.ordem.servico').search(
            cr, uid, args, order='wmscos_data_cancelamento desc', limit=1, context=context)
        maior_data_dados = self.pool.get('view.wms.controle.ordem.servico').read(
            cr, uid, data_cancelamento_id, ['wmscos_data_cancelamento'], context=context)

        if maior_data_dados and isinstance(maior_data_dados, list):
            maior_data_dados = maior_data_dados[0]

        if isinstance(maior_data_dados, dict):
            data_cancelamento = maior_data_dados.get('wmscos_data_cancelamento')
        else:
            data_cancelamento = None

        # Define a maior data entre fechamento e cancelamento
        maior_data = data_fechamento if data_fechamento > data_cancelamento else data_cancelamento
        maior_data = menor_data if not maior_data else maior_data

        # Periodo
        menor_data = map(lambda m: int(m), menor_data[:10].split('-'))
        maior_data = map(lambda m: int(m), maior_data[:10].split('-'))
        periodo = datetime(menor_data[0], menor_data[1], menor_data[2]).strftime('%x'),\
                  datetime(maior_data[0], maior_data[1], maior_data[2]).strftime('%x')
        periodo = '%s a %s' % tuple(map(lambda m: m.replace('-', '/'), periodo))

        self.localcontext.update(dict(datetime=datetime, agora=agora, periodo=periodo))

        self.context = context

report_sxw.report_sxw('report.controle.ordem.servico', 'view.wms.controle.ordem.servico',
                      'addons/wms/controle_os/report/controle_os.rml',
                      parser=report_controle_ordem_servico, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
