# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


# noinspection PyPep8Naming
class wizard_inventario(osv.TransientModel):
    """
    Transferencia manual e automatica.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.inventario'  # wizinv
    _inherit = 'wizard.inventario'  # wms/inventario/wizard/inventario_data

    def wizinv_verifica_quebra_tipo_nivel(self, cr, uid, ids, tipo_nivel, deposito, context=None):
        """
        Verifica se tipo de nivel e virtual e crossdocking

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: ID do formulario de planejamento de inventario
        :param context: um dicionario padrao para valores contextuais
        :param tipo_nivel: tipo do nivel do endereco de inventario (PK, PU ou VT)
        :param deposito: numero do deposito (formacao do nome do endereco)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: integer
        :type tipo_nivel: str
        :type deposito: int
        :type context: dict
        :return: verdadeiro se o tipo do nivel for virtual e deposito for 99 ou se tipo do nivel
        for avaria e o deposito for 98
        :rtype: bool
        """

        context = context and context.copy() or {}
        result = super(wizard_inventario, self).wizinv_verifica_quebra_tipo_nivel(
            cr, uid, ids, tipo_nivel, deposito, context=context)

        # Dados do inventario
        dados_inventario = self.read(cr, uid, ids, [], context=context)

        if dados_inventario and isinstance(dados_inventario, list):
            dados_inventario = dados_inventario[0]

        if dados_inventario and isinstance(dados_inventario, dict):
            considera_crossdocking = dados_inventario.get('wizinv_crossdocking')

            if tipo_nivel == 'VT' and considera_crossdocking and deposito == '97':
                result = True

        return result

    def wizinv_considerar_virtual(self, inventario):
        """
        Retorna a lista dos argumentos de validacao dos enderecos de inventario sem considerar
        enderecos de crossdocking

        :param inventario: dicionario com os dados do planejamento de inventario
        :type inventario: dict
        :return: argumentos de pesquisa para validacao dos enderecos de inventario
        :rtype: list
        """

        result = super(wizard_inventario, self).wizinv_considerar_virtual(inventario)

        if not inventario.get('wizinv_crossdocking'):
            result.append(('deposito', '!=', 97))  # Nao considera enderecos de crossdocking

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
