# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class recebimento_os(osv.Model):
    """
    Enderecamento. Ordens de armazenagem. Nao realiza reposicao se item estiver com a ´´flag´´ de
    crosdocking marcada. (heranca)

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.os'  # recosv
    _inherit = 'recebimento.os'  # recebimento/inherit/os

    def recosv_ressuprimento_picking(self, cr, uid, os, item, context=None):
        """
        Ressuprimento do picking. (heranca)

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} dados da ordem de servico,
        :param item: {browser} dados do item de recebimento,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {int} zero se o ´flag´ do produto estiver marcado como crossdocking ou o valor que
        foi reposto no picking.

        :attention: Nao realiza o ressuprimento do picking se o produto estiver com a ´´flag´´ de
        crossdocking marcada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = False  # Nao realiza a reposicao

        if not item.produto_id.estpro_crossdocking:  # Se nao for crossdocking...
            # ...procedimento normal
            result = super(recebimento_os, self).recosv_ressuprimento_picking(cr, uid, os, item)

        return result

    def recosv_enderecamento_pulmao(self, cr, uid, os, item, restante, context=None):
        """
        Movimentacao horizontal. Enderecamento pulmao. (heranca)

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} os dados da ordem de servico,
        :param item: {browser} os dados do item de recebimento,
        :param restante: {int} a quantidade restantde do que foi recebido subtraida da quantidade
        reposta no picking,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} lista com o ID dos enderecos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if not item.produto_id.estpro_crossdocking:  # Se nao for crossdocking...
            # ...procedimento normal
            return super(recebimento_os, self).recosv_enderecamento_pulmao(
                cr, uid, os, item, restante)

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_grade = self.pool.get('estoque.produto.grade')

        result = []

        # Dados do endereco
        brw_grade = obj_grade.browse(cr, uid, item.grade_id.id)

        # Dados Logisticos
        norma = brw_grade.lastro * brw_grade.camada  # Norma palete nao muda
        estrutura_tipo_pulmao_id = brw_grade.estrutura_tipo_pulmao_id.id
        endereco_tipo_pulmao_id = brw_grade.endereco_tipo_pulmao_id.id

        while restante > 0:
            # Cria endereco crossdocking
            endereco_id = obj_wms_endereco.wmsend_criar_virtual(cr, uid, 97)
            result.append(endereco_id)

            # Enderecamento
            restante -= self.recosv_enderecamento(cr, uid, os, item, endereco_id, restante, norma)

        # Verifica se o endereco de picking para o crossdocking ja foi atribuido. Se nao, o primeiro
        # endereco de crossdocking sera o do picking
        if not item.grade_id.endereco_produto_id.id and result:
            obj_grade.write(cr, uid, item.grade_id.id, {'endereco_produto_id': result[0]}, context)

            # Atualiza o tipo de endereco e o tipo de estrutura conforme dados logisticos.
            dados = {
                'tipo_nivel': 'PK',
                'endereco_tipo_id': endereco_tipo_pulmao_id,
                'estrutura_tipo_id': estrutura_tipo_pulmao_id
            }

            obj_wms_endereco.write(cr, uid, result[0], dados, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
