# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class wms_endereco(osv.Model):
    """
    Endreco de armazenamento. (heranca)

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.endereco'  # wmsend
    _inherit = 'wms.endereco'  # wms/control/endereco

    def wmsend_bloquear(self, cr, uid, ids, context=None):
        """
        Bloqueia um determinado endereco. Mas nao bloqueia endereco de crossdocking.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista com ID dos enderecos que nao sao de crossdocking.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if not isinstance(ids, list):
            ids = [ids]

        args = [('id', 'in', ids), ('deposito', '!=', 97)]

        ids = self.search(cr, uid, args, context=context)  # O endereco de crossdocking nao e bloqueado

        return super(wms_endereco, self).wmsend_bloquear(cr, uid, ids, context)

    def wmsend_norma_quantidade(self, cr, uid, grade_id, quantidade, context=None):
        """
        Retorna a lista dos enderecos com saldo iqual a norma palete do produto excluindo-se os
        enderecos de crossdocking.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param grade_id: {int} ID da grade do produto,
        @param quantidade: {int} quantidade limite para equivalencia da norma,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista com os ids dos enderecos com saldo igual a norma.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_estoque_produto = self.pool.get('estoque.produto')

        result = super(wms_endereco, self).wmsend_norma_quantidade(
            cr, uid, grade_id, quantidade, context)

        # Filtra os enderecos de crossdocking se o produto NAO FOR crossdocking
        campos = ['produto_id']
        dados_grade = obj_estoque_produto_grade.read(
            cr, uid, grade_id, campos, context=context, load='_classic_write')

        if dados_grade and isinstance(dados_grade, list):
            dados_grade = dados_grade[0]

        if dados_grade and isinstance(dados_grade, dict):
            produto_id = dados_grade.get('produto_id')

            campos = ['estpro_crossdocking']
            dados_produto = obj_estoque_produto.read(cr, uid, produto_id, campos, context=context)

            if dados_produto and isinstance(dados_produto, list):
                dados_produto = dados_produto[0]

            if dados_produto and isinstance(dados_produto, dict):
                if not dados_produto.get('estpro_crossdocking', False):  # Nao e crossdocking
                    args = [('id', 'in', result), ('deposito', '!=', 97)]
                    result = self.search(cr, uid, args, context=context)

        return result

    def wmsend_disponivel(self, cr, uid, ids, context=None):
        """
        Informa a disponibilidade (para uso) de um determinado Endereco. Para
        estar disponivel o Endereco deve estar com saldo = 0, state = 'L', blo-
        queado = False.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {boolean} verdadeiro se o endereco estiver disponivel.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Ignora os enderecos de crossdocking
        endereco_ids = map(lambda e: e.id, filter(lambda e: e.deposito != 97,
                                                  self.browse(cr, uid, ids, context)))

        result = super(wms_endereco, self).wmsend_disponivel(cr, uid, endereco_ids, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
