# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuição-Não Comercial-Compartilhada Igual 3.0 Não Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


# noinspection PyPep8Naming
class wms_enderecamento_itens(osv.Model):
    """
    Itens de movimentacao (Enderecamento).

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.enderecamento.itens'  # wmseit
    _inherit = 'wms.enderecamento.itens'  # wms/enderecamento/model/enderecamento_itens_data

    def wmseit_button_enderecar(self, cr, uid, ids, context=None):
        """
        Endereca o item de recebimento.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista dos IDs dos enderecos.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = self.wmseit_enderecar(cr, uid, ids, context)

        return result

    def wmseit_enderecar(self, cr, uid, ids, context=None):
        """
        Enderecar um item de recebimento.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista dos IDs dos enderecos.
        :rtype: list
        :attention: A alteracao do saldo do endereco sera efetivada no fechamento da ordem de
        servico de enderecamento.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        import time

        context = context and context.copy() or {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        dados = {
            'wmseit_usuario_mov_id': uid,
            'wmseit_data_movimentacao': time.strftime('%Y-%m-%d %H:%M:%S'),
            'wmseit_enderecado': True
        }

        result = self.write(cr, uid, ids, dados, context)  # Marca o item como enderecado

        return result

    @staticmethod
    def wmseit_imprimir_etiqueta_palete(cr, uid, ids, context=None):
        """
        Imprimir etiqueta de palete.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: relatorio de etiquetas.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        dados = {
            'ids': ids,
            'model': 'recebimento.os',
            'form': {}
        }

        result = {
            'type': 'ir.actions.report.xml',
            'report_name': 'etiqueta.palete',
            'datas': dados,
            'context': context
        }

        return result

    def write(self, cr, uid, ids, values, context=None):
        """
        Verifica a possibilidade de sobreposicao de um endereco

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param values: o nome dos campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: verdadeiro se o registro foi alterado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        # Sobreposicao
        if 'wmseit_endereco_id' in values:
            # Recupera dados do endereco original
            campos = ['wmseit_endereco_id', 'wmseit_quantidade', 'wmseit_produto_id',
                      'wmseit_grade_id', 'wmseit_data_validade']
            dados_item = self.read(
                cr, uid, ids if isinstance(ids, list) else [ids], campos, context=context,
                load='_classic_write')

            if dados_item and isinstance(dados_item, list):
                dados_item = dados_item[0]

            if dados_item and isinstance(dados_item, dict):
                endereco_id = dados_item.get('wmseit_endereco_id')

                if endereco_id != values['wmseit_endereco_id']:
                    quantidade = dados_item.get('wmseit_quantidade') * -1  # Estorno da quantidade
                    produto_id = dados_item.get('wmseit_produto_id')
                    grade_id = dados_item.get('wmseit_grade_id')
                    validade = dados_item.get('wmseit_data_validade')

                    obj_wms_endereco.wmsend_reserva_credito(
                        cr, uid, endereco_id, produto_id, grade_id, quantidade, False, context=context)
                    obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_id)
                    obj_wms_endereco.wmsend_reserva_credito(
                        cr, uid, values.get('wmseit_endereco_id'), produto_id, grade_id,
                        quantidade * -1, validade)
                    obj_wms_endereco.wmsend_bloquear(cr, uid, values.get('wmseit_endereco_id'))

        result = super(wms_enderecamento_itens, self).write(cr, uid, ids, values, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
