# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
# noinspection PyProtectedMember
from openerp.tools.translate import _
from dateutil.parser import parse


# noinspection PyPep8Naming
class recebimento_os(osv.Model):
    """
    Enderecamento. Ordens de Armazenagem.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.os'  # recosv
    _inherit = 'recebimento.os'  # recebimento/inherit/os

    def recosv_fechar_contagem(self, cr, uid, ids, context=None):
        """
        Fecha contagem de recebimento. Verifica se todos os itens contados estao Ok para realizar o
        enderecamento.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} lita com os IDs das Ordens de Servico que sofreram enderecamento.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = super(recebimento_os, self).recosv_fechar_contagem(cr, uid, ids, context)

        # Endereca os itens (se houver)
        for os in self.browse(cr, uid, ids, context):
            # Fix: As ordens de servico fechadas sem o modulo WMS nao podem ser enderecadas
            if os.contagem_status == 'F' and not os.recosv_enderecamento_item_ids:
                self.recosv_enderecamento_ordemservico(cr, uid, ids, context)

        return result

    def recosv_enderecamento_ordemservico(self, cr, uid, ids, context=None):
        """
        Enderecamento dos itens de recebimento.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} ID's das Ordens de Servico,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} os ID's das ordens de servico que foram enderecadas.
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Somente os ID's das ordens de servico com recebimento ok
        ids = map(lambda os: os.id, filter(lambda os: os.recosv_recebimento_ok, self.browse(
            cr, uid, ids, context)))

        for os in self.browse(cr, uid, ids, context):
            for item in os.recebimento_item_ids:
                # Reposicao picking - Movimentacao vertical
                reposicao = self.recosv_ressuprimento_picking(cr, uid, os, item)

                # Armazenamento pulmao - Movimentacao horizontal
                restante = item.qtde_contada - reposicao
                if restante > 0:
                    self.recosv_enderecamento_pulmao(cr, uid, os, item, restante)

                # Enviar itens avariados para endereco virtual [98]
                if item.qtde_avaria:
                    self.recosv_endereca_produto_avariado(cr, uid, os, item)

        return ids

    def recosv_ressuprimento_picking(self, cr, uid, os, item, context=None):
        """
        Ressuprimento do picking.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} dados da ordem de servico,
        :param item: {browser} dados do item de recebimento,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {int} o valor que foi reposto no picking.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_enderecamento_itens = self.pool.get('wms.enderecamento.itens')
        obj_grade = self.pool.get('estoque.produto.grade')

        # Armazenamento do produto no picking (reposicao).
        endereco_id = item.grade_id.endereco_produto_id.id
        produto_id = item.produto_id.id
        grade_id = item.grade_id.id
        data_validade = item.data_validade

        # Se nao houver endereco assume picking dinamico
        if not endereco_id:
            estrutura_tipo_pulmao_id = item.grade_id.estrutura_tipo_pulmao_id.id
            endereco_tipo_pulmao_id = item.grade_id.endereco_tipo_pulmao_id.id
            args = [('picking_dinamico', '=', True), ('state', '=', 'L'),
                    ('estrutura_tipo_id', '=', estrutura_tipo_pulmao_id),
                    ('bloqueado', '=', False)]
            endereco_id = obj_wms_endereco.search(
                cr, uid, args, limit=1, order='name', context=context)
            if not endereco_id:  # Nao encontrou um endereco de picking dinamico liberado
                # Cria um endereco virtual
                endereco_id = obj_wms_endereco.wmsend_criar_virtual(cr, uid, 99)

                # Ajusta endereco para tipo ´´picking``
                dados = {'tipo_nivel': 'PK'}

                # Atualiza o tipo de endereco e o tipo de estrutura conforme dados logisticos.
                dados.update({
                    'endereco_tipo_id': endereco_tipo_pulmao_id,
                    'estrutura_tipo_id': estrutura_tipo_pulmao_id
                })

                obj_wms_endereco.write(cr, uid, endereco_id, dados, context)
            else:
                endereco_id = endereco_id[0]

            # Atribui o endereco de picking
            dados = {'endereco_produto_id': endereco_id}
            obj_grade.write(cr, uid, item.grade_id.id, dados, context)

        validade_picking = item.grade_id.endereco_produto_id.validade

        # Saldo do endereco
        saldo = obj_wms_endereco.wmsend_saldo(cr, uid, endereco_id)

        # Diferenca em dias das datas de validade
        if not validade_picking:
            validade_picking = item.data_validade

        try:
            delta = parse(item.data_validade, yearfirst=True).date()
            delta -= parse(validade_picking, yearfirst=True).date()
        except Exception as e:
            msg = _('Data de validade invalida para o item %s - %s' % (item.id, item.grade_id.name))

            raise osv.except_osv(_('Atencao!'), msg)

        # Parametro de data de validade.
        # Se o parametro nao estiver ativo nao verificar validade e sempre repor estoque do picking,
        # substituindo a validade pela validade do produto se esta for maior.
        parametros = ['estpar_verifica_data_validade']
        dados_parametro = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', parametros)

        # Completar picking se data de validade do produto e menor ou parametro de reposicao indicar
        qtde_na_master = item.grade_id.embalagem_master_id.embalagem_id.quantidade_na_embalagem
        norma_palete = item.grade_id.norma_palete * qtde_na_master
        capacidade = item.grade_id.capacidade_picking * qtde_na_master

        if delta.days <= 0 or not dados_parametro.get('estpar_verifica_data_validade', False):
            reposicao = capacidade - saldo

            # Zera reposicao negativa para repor quantidade fracionada
            if reposicao < 0:  # Reposicao negativa (picking alem da capacidade)
                reposicao = 0

            # Reposicao de quantidade fracionada independente da capacidade do picking
            qtde_contada_master = int(item.qtde_contada / qtde_na_master) * qtde_na_master
            reposicao += item.qtde_contada - qtde_contada_master

            # Verifica se a quantidade contada e menor que a capacidade do picking
            if reposicao > item.qtde_contada:
                reposicao = item.qtde_contada  # Se sim, repoe toda a quantidade
        else:
            # Forca a reposicao de quantidade fracionada mesmo se o picking estiver cheio!
            qtde_contada_master = int(item.qtde_contada / qtde_na_master) * qtde_na_master
            reposicao = item.qtde_contada - qtde_contada_master

        # Paletetizacao da quantidade de reposicao
        qtde_paletizacao = reposicao

        while qtde_paletizacao > 0:
            if qtde_paletizacao > norma_palete:
                qtde_reposicao = norma_palete
            else:
                qtde_reposicao = qtde_paletizacao

            dados = {'wmseit_recebimento_os_id': os.id, 'wmseit_recebimento_item_id': item.id,
                     'wmseit_quantidade': qtde_reposicao, 'wmseit_produto_id': produto_id,
                     'wmseit_grade_id': grade_id, 'wmseit_data_validade': data_validade,
                     'wmseit_endereco_id': endereco_id}

            context = {'enderecamento': True}

            obj_enderecamento_itens.create(cr, uid, dados, context)
            obj_wms_endereco.wmsend_reserva_credito(
                cr, uid, endereco_id, item.produto_id.id, item.grade_id.id, qtde_reposicao)

            qtde_paletizacao -= qtde_reposicao

        return reposicao  # Retorna a quantidade reposta

    def recosv_enderecamento_pulmao(self, cr, uid, os, item, restante, context=None):
        """
        Movimentacao horizontal. Enderecamento pulmao.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} os dados da ordem de servico,
        :param item: {browser} os dados do item de recebimento,
        :param restante: {int} a quantidade restantde do que foi recebido subtraida da quantidade
        reposta no picking,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} sempre verdadeiro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_wms_enderecotipo = self.pool.get('wms.enderecotipo')
        obj_grade = self.pool.get('estoque.produto.grade')

        # Dados do endereco
        brw_grade = obj_grade.browse(cr, uid, item.grade_id.id, context)

        if brw_grade.endereco_produto_id:
            deposito = brw_grade.endereco_produto_id.deposito
            rua = brw_grade.endereco_produto_id.rua
            predio = brw_grade.endereco_produto_id.predio
            lado = str(abs(predio % 2 - 2))
            tipo_endereco = brw_grade.endereco_tipo_pulmao_id.id
            prioridade = brw_grade.endereco_tipo_pulmao_id.sequence
            tipo_estrutura = brw_grade.estrutura_tipo_pulmao_id.id
        else:
            deposito = rua = predio = 99
            lado = '1'  # Impar
            tipo_endereco = prioridade = tipo_estrutura = False

        # Dados Logisticos
        RESTRICAO_NIVEL_MIN = brw_grade.restricao_nivel_min
        RESTRICAO_NIVEL_MAX = brw_grade.restricao_nivel_max
        NORMA_PALETE = brw_grade.lastro * brw_grade.camada  # Norma palete nao muda

        # Controle
        rua_verificada = []
        predio_verificado = []
        endereco_utilizado = []
        endereco_tipo_seq = [tipo_endereco]  # Tipos de endereco utilizados
        rua_inicial = rua
        predio_inicial = predio
        troca_lado = False

        while restante > 0 and deposito != 99:
            # Procura pelo endereco de pulmao mais proximo do picking
            # TODO: Transformar a busca em funcao prevendo intervencao necessaria de herancas
            args = [('deposito', '=', deposito), ('rua', '=', rua), ('predio', '=', predio),
                    ('tipo_nivel', '!=', 'PK'), ('endereco_tipo_id', '=', tipo_endereco),
                    ('estrutura_tipo_id', '=', tipo_estrutura), ('state', '=', 'L'),
                    ('bloqueado', '=', False)]

            if endereco_utilizado:
                args.append(('id', 'not in', endereco_utilizado))

            if RESTRICAO_NIVEL_MIN:
                args.append(('nivel', '>=', RESTRICAO_NIVEL_MIN))

            if RESTRICAO_NIVEL_MAX:
                args.append(('nivel', '<=', RESTRICAO_NIVEL_MAX))

            ordem = 'deposito, rua, predio, nivel, apartamento'
            endereco_id = obj_wms_endereco.search(
                cr, uid, args, limit=1, order=ordem, context=context)

            if endereco_id:  # Se encontrou, armazena a quantidade no endereco
                restante -= self.recosv_enderecamento(
                    cr, uid, os, item, endereco_id, restante, NORMA_PALETE)

                # Acrescenta o endereco na lista de enderecos utilizados
                endereco_utilizado.extend(endereco_id)
                continue  # Restantante da quantidade (se houver) em novo endereco
            else:  # Nao encontrou.
                #  Busca por outro endereco
                predio_verificado.append(predio)  # Todos os enderecos do predio foram verificados

                # Procura predios anteriores e posteriores alternando os lados
                # TODO: Transformar a busca em funcao prevendo intervencao necessaria de herancas
                args = [('deposito', '=', deposito), ('rua', '=', rua), ('lado', '=', lado),
                        ('predio', '<', predio_inicial), ('predio', 'not in', predio_verificado),
                        ('predio', '>', 0), ('state', '=', 'L'), ('bloqueado', '=', False),
                        ('tipo_nivel', '!=', 'PK'), ('endereco_tipo_id', '=', tipo_endereco),
                        ('estrutura_tipo_id', '=', tipo_estrutura)]
                ordem = 'deposito, rua, predio desc, nivel, apartamento'

                # Predio anterior
                anterior_ids = obj_wms_endereco.search(
                    cr, uid, args, limit=1, order=ordem, context=context)

                if anterior_ids:  # Achou predio anterior
                    campos = ['predio']
                    dados_anterior = obj_wms_endereco.read(cr, uid, anterior_ids, campos, context=context)

                    if dados_anterior and isinstance(dados_anterior, list):
                        dados_anterior = dados_anterior[0]

                    if dados_anterior and isinstance(dados_anterior, dict):
                        predio = dados_anterior.get('predio')

                        if predio:
                            continue

                args = [('rua', '=', rua), ('lado', '=', lado), ('predio', '>', predio_inicial),
                        ('predio', 'not in', predio_verificado), ('state', '=', 'L'),
                        ('bloqueado', '=', False), ('tipo_nivel', '!=', 'PK'),
                        ('endereco_tipo_id', '=', tipo_endereco),
                        ('estrutura_tipo_id', '=', tipo_estrutura)]
                ordem = 'deposito, rua, predio, nivel, apartamento'

                # Predio posterior
                posterior_ids = obj_wms_endereco.search(
                    cr, uid, args, limit=1, order=ordem, context=context)

                if posterior_ids:  # Achou predio posterior
                    campos = ['predio']
                    dados_posterior = obj_wms_endereco.read(cr, uid, posterior_ids, campos, context=context)

                    if dados_posterior and isinstance(dados_posterior, list):
                        dados_posterior = dados_posterior[0]

                    if dados_posterior and isinstance(dados_posterior, dict):
                        predio = dados_posterior.get('predio')

                        if predio:
                            continue

                if not troca_lado:
                    lado = str(abs(int(lado) - 3))
                    troca_lado = True
                    continue

                # Todos os predios da rua foram verificados
                rua_verificada.append(rua)
                predio = predio_inicial  # Mesmo predio de outra rua
                predio_verificado = []  # Predios de outra rua
                lado = str(abs(predio % 2 - 2))  # Lado do predio inicial

                # Busca ruas anteriores e posteriores
                args = [('rua', '<', rua_inicial), ('rua', '>', 0),
                        ('rua', 'not in', rua_verificada)]
                ordem = 'deposito, rua desc, predio, nivel, apartamento'

                # Rua anterior
                anterior_ids = obj_wms_endereco.search(
                    cr, uid, args, limit=1, order=ordem, context=context)

                if anterior_ids:  # Achou rua anterior
                    campos = ['rua']
                    dados_anterior = obj_wms_endereco.read(cr, uid, anterior_ids, campos, context=context)

                    if dados_anterior and isinstance(dados_anterior, list):
                        dados_anterior = dados_anterior[0]

                    if dados_anterior and isinstance(dados_anterior, dict):
                        rua = dados_anterior.get('rua')
                        troca_lado = False
                        continue

                args = [('rua', '>', rua_inicial), ('rua', 'not in', rua_verificada)]
                ordem = 'deposito, rua, predio, nivel, apartamento'

                # Rua posterior
                posterior_ids = obj_wms_endereco.search(
                    cr, uid, args, limit=1, order=ordem, context=context)

                if posterior_ids:  # Achou rua posterior
                    campos = ['rua']
                    dados_posterior = obj_wms_endereco.read(cr, uid, posterior_ids, campos, context=context)

                    if dados_posterior and isinstance(dados_posterior, list):
                        dados_posterior = dados_posterior[0]

                    if dados_posterior and isinstance(dados_posterior, dict):
                        rua = dados_posterior.get('rua')
                        troca_lado = False
                        continue

                # Todas as ruas foram verificadas.
                # Se ainda nao encontrou endereco vago, verifica pelo tipo de endereco baseado na
                # prioridade maior definida no cadastro de tipos.
                args = [('id', 'not in', endereco_tipo_seq), ('sequence', '<', prioridade)]
                tipo_ids = obj_wms_enderecotipo.search(
                    cr, uid, args, limit=1, order='sequence', context=context)

                if tipo_ids:
                    campos = ['id', 'sequence']
                    dados_tipo = obj_wms_enderecotipo.read(cr, uid, tipo_ids, campos, context=context)

                    if dados_tipo and isinstance(dados_tipo, list):
                        dados_tipo = dados_tipo[0]

                    if dados_tipo and isinstance(dados_tipo, dict):
                        tipo_endereco = dados_tipo.get('id')  # Novo tipo de endereco

                        endereco_tipo_seq.append(tipo_ids[0])
                        prioridade = dados_tipo.get('sequence')  # Nova prioridade

                        # Retorna ao endereco de picking buscando novo tipo de endereco
                        # Novos enderecos a verificar para novo tipo de endereco
                        rua = rua_inicial
                        predio = predio_inicial
                        rua_verificada = []
                        predio_verificado = []

                        continue
                else:  # Todas as prioridades verificadas
                    break
            break

        # O restante, se houver (por falta de endereco), sera armazenado em um endereco virtual
        while restante > 0:
            # Cria novo endereco virtual
            endereco_id = obj_wms_endereco.wmsend_criar_virtual(cr, uid, 99)

            # Enderecamento
            restante -= self.recosv_enderecamento(
                cr, uid, os, item, endereco_id, restante, NORMA_PALETE)

        return True

    def recosv_endereca_produto_avariado(self, cr, uid, os, item, context=None):
        """
        Enderecamento de produto avariado.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} os dados da ordem de servico,
        :param item: {browser} os dados do item de recebimento,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} sempre verdadeiro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        # obj_enderecamento_itens = self.pool.get('wms.enderecamento.itens')

        restante = item.qtde_avaria
        NORMA_PALETE = item.grade_id.lastro * item.grade_id.camada

        while restante > 0:
            # Cria novo endereco virtual (avaria)
            endereco_id = obj_wms_endereco.wmsend_criar_virtual(cr, uid, 98)

            # Enderecamento
            restante -= self.recosv_enderecamento(
                cr, uid, os, item, endereco_id, restante, NORMA_PALETE)

        return True

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Ao fechar a ordem de servico efetivar o enderecamento.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} ´´status´´ da ordem de servico fechada ou mensagem de
        erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        result = super(recebimento_os, self).osvosv_fechar_os(cr, uid, ids, context)

        for os in self.browse(cr, uid, ids, context):
            if os.tipo_id.codigo == 'REC':
                # Verifica se tudo esta correto na ordem de servico para efetivar o enderecamento.
                msg = None

                if not os.recosv_recebimento_ok:
                    msg = _('Ordem de servico nr %s nao pode ser fechada, pois o recebimento ainda '
                            'nao foi finalizado.')

                # Verifica itens de enderecamento somente de ordens de servico do modulo WMS
                if os.recosv_enderecamento_item_ids and not os.recosv_enderecamento_ok:
                    msg = _('Ordem de servico nr %s nao pode ser fechada. Existem itens que nao '
                            'foram enderecados.' % os.numero)

                if msg:
                    raise osv.except_osv(_('Atencao!'), msg)

            # Efetiva o enderecamento atualizando o saldo
            for recebimento_item in os.recebimento_item_ids:
                validade = recebimento_item.data_validade

                for enderecamento_item in os.recosv_enderecamento_item_ids:
                    if enderecamento_item.wmseit_recebimento_item_id.id != recebimento_item.id:
                        continue

                    endereco_id = enderecamento_item.wmseit_endereco_id.id
                    quantidade = enderecamento_item.wmseit_quantidade
                    obj_wms_endereco.wmsend_somar_saldo(cr, uid, endereco_id, quantidade, validade)

                    # Desbloqueia o endereco
                    obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_id)

        return result

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} status da ordem de servico cancelada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        for os in self.browse(cr, uid, ids, context):
            if os.tipo_id.codigo == 'REC':
                # Reverte os enderecamentos
                for recebimento_item in os.recebimento_item_ids:
                    enderecamento_itens = filter(
                        lambda i: i.wmseit_recebimento_item_id.id == recebimento_item.id,
                        os.recosv_enderecamento_item_ids)

                    endereco_ids = []
                    for enderecamento_item in enderecamento_itens:
                        endereco_id = enderecamento_item.wmseit_endereco_id.id
                        quantidade = enderecamento_item.wmseit_quantidade
                        produto_id = enderecamento_item.wmseit_endereco_id.produto_id.id
                        grade_id = enderecamento_item.wmseit_endereco_id.grade_id.id

                        ajuste = quantidade * -1  # Credito negativo!
                        obj_wms_endereco.wmsend_reserva_credito(
                            cr, uid,  endereco_id, produto_id, grade_id, ajuste)
                        endereco_ids.append(endereco_id)

                    # Desbloqueia os enderecos
                    obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_ids)

        result = super(recebimento_os, self).osvosv_cancelar_os(cr, uid, ids, context)

        return result

    # noinspection PyMethodMayBeStatic
    def recosv_onchange_enderecamento_item_ids(self, cr, uid, ids, item_ids, context=None):
        """
        Verifica se existem enderecos duplicados (sobreposicao).

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param item_ids: {dict} itens de enderecamento,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return:

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = {}

        # Verifica enderecos duplicados
        enderecos = [item[2].get('wmseit_endereco_id') for item in filter(
            lambda i: isinstance(i[2], dict), item_ids)]
        duplicados = set([endereco for endereco in enderecos if enderecos.count(endereco) > 1])

        if len(list(duplicados)):
            msg = _('O enderecamento nao sera efetuado para itens com enderecos duplicados.')
            result['warning'] = {'title': _('Aviso.'), 'message': msg}

        return result

    def recosv_enderecamento(self, cr, uid, os, item, endereco, quantidade, norma, context=None):
        """
        Efetua o enderecamento do produto.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param os: {browser} os dados da ordem de servico,
        :param item: {browser} os dados do item de recebimento,
        :param endereco: {long} o ID do endereco utilizado para no enderecamento,
        :param quantidade: {int} a quantidade a ser armazenada no endereco,
        :param norma: {int} o valor da Norma palete que sera utilizada para o calculo da quantidade
        a ser enderecada,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {int} a quantidade que foi armazenada (enderecada) ou a mensagem de que o endereco
        utilizado nao pode ser bloqueado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if isinstance(endereco, (list, tuple)):
            endereco = endereco[0]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_enderecamento_itens = self.pool.get('wms.enderecamento.itens')

        master = item.grade_id.embalagem_master_id
        qtde_na_master = master.embalagem_id.quantidade_na_embalagem
        produto_id = item.produto_id.id
        grade_id = item.grade_id.id
        data_validade = item.data_validade

        # Calculo da quantidade a ser armazenada no endereco
        if quantidade > norma * qtde_na_master:
            qtde_armazenada = norma * qtde_na_master
        else:
            qtde_armazenada = quantidade  # Quantidade menor que a Norma

        dados = {'wmseit_recebimento_os_id': os.id, 'wmseit_recebimento_item_id': item.id,
                 'wmseit_quantidade': qtde_armazenada, 'wmseit_endereco_id': endereco,
                 'wmseit_produto_id': produto_id, 'wmseit_grade_id': grade_id,
                 'wmseit_data_validade': data_validade}

        obj_enderecamento_itens.create(cr, uid, dados, context)  # Registra dados no endereco

        # Bloqueia o endereco utilizado
        obj_wms_endereco.wmsend_bloquear(cr, uid, endereco)

        # Reserva a quantidade (credito)
        obj_wms_endereco.wmsend_reserva_credito(
            cr, uid, endereco, item.produto_id.id, item.grade_id.id, qtde_armazenada)

        result = int(qtde_armazenada)  # Retorna a quantidade armazenada

        return result

    def recosv_button_enderecar_itens(self, cr, uid, ids, context=None):
        """
        Realiza o enderecamento de todos os itens da ordem de servico.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {boolean} verdadeiro se os itens foram enderecados.

        :note: Botao "Enderecar todos" da aba "Itens de enderecamento"
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_enderecamento_itens = self.pool.get('wms.enderecamento.itens')

        args = [('wmseit_recebimento_os_id', 'in', ids)]
        item_ids = obj_enderecamento_itens.search(cr, uid, args, context=context)
        result = obj_enderecamento_itens.wmseit_enderecar(cr, uid, item_ids, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

