# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.report import report_sxw
from datetime import datetime
from pytz import timezone
from openerp.tools.translate import _


class report_enderecamento(report_sxw.rml_parse):
    """
    Relatorio de Ordem de Enderecamento.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context=None):
        """
        Atribui valores as funcoes utilizada no relatorio.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param name: {char} nome do modelo utilizado,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: None

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        super(report_enderecamento, self).__init__(cr, uid, name, context=context)

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))
        self.localcontext.update({'datetime': datetime, 'agora': agora})

        self.context = context

        # Verifica se existe item de enderecamento
        obj_enderecamento_itens = self.pool.get('wms.enderecamento.itens')
        args = [('wmseit_recebimento_os_id', '=', context.get('active_id'))]

        if not obj_enderecamento_itens.search(cr, uid, args, context=context):
            msg = _('Relatorio de ordem de servico de enderecamento nao pode ser impresso. Nao '
                    'ha itens de enderecamento.')

            raise osv.except_osv(_('Atencao!'), msg)

report_sxw.report_sxw('report.enderecamento.os', 'recebimento.os',
                      'addons/wms/enderecamento/report/enderecamento.rml',
                      parser=report_enderecamento, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
