# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class estoque_local_locacao(osv.Model):
    """
    Heranca

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.local.locacao'  # estllc
    _inherit = 'estoque.local.locacao'  # estoque/control/local_locacao

    # _________________________ ORM Methods __________________________________
    def write(self, cr, uid, ids, values, context=None):
        """
        Atualiza a embalagem de expedicao no dado logistico do produto (grade) em funcao da
        embalagem na locacao.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista de IDs,
        :param values: dicionario com os campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :return: verdadeiro se a alteracao foi efetuada.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_adm_utils = self.pool.get('adm.utils')

        result = super(estoque_local_locacao, self).write(cr, uid, ids, values, context)

        # Atualiza a embalagem de expedicao (venda) do dado logistico
        if 'grade_id' in values and 'estllc_produto_embalagem_id':
            # Verifica se a alteracao da embalagem esta sendo realizada no local cadastrado no
            # parametro do estoque.
            campos = ['estpar_local_id', 'estpar_empresa_id']
            adm_utils_dados = obj_adm_utils.admutl_get_parametros(
                cr, uid, 'estoque', campos, context=context)
            estpar_local_id = adm_utils_dados.get('estpar_local_id')

            if estpar_local_id and isinstance(estpar_local_id, (list, tuple)):
                estpar_local_id = estpar_local_id[0]

            estpar_empresa_id = adm_utils_dados.get('estpar_empresa_id')

            if estpar_empresa_id and isinstance(estpar_empresa_id, (list, tuple)):
                estpar_empresa_id = estpar_empresa_id[0]

            local_id = company_id = False

            if 'local_id' in values and 'company_id' in values:
                local_id = values['local_id']
                company_id = values['company_id']
            else:
                fields = ['local_id', 'company_id']
                estoque_local_locacao_dados = self.read(
                    cr, uid, ids[0], fields, context=context, load='_classic_write')

                if estoque_local_locacao and isinstance(estoque_local_locacao_dados, list):
                    estoque_local_locacao_dados = estoque_local_locacao_dados[0]

                if estoque_local_locacao and isinstance(estoque_local_locacao_dados, dict):
                    local_id = estoque_local_locacao_dados.get('local_id')
                    company_id = estoque_local_locacao_dados.get('company_id')

            if local_id and isinstance(local_id, (list, tuple)):
                local_id = local_id[0]

            if company_id and isinstance(company_id, (list, tuple)):
                company_id = company_id[0]

            if local_id == estpar_local_id and company_id == estpar_empresa_id:
                self.pool.get('estoque.produto.grade').write(cr, uid, values.get('grade_id'), {
                    'embalagem_expedicao_id': values.get('estllc_produto_embalagem_id')
                })

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
