# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class estoque_produto_embalagem(osv.Model):
    """
    Calculo do volume (metros cubicos).

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.produto.embalagem'  # estpem
    _inherit = 'estoque.produto.embalagem'  # estoque/control/produto_embalagem

    def estpem_onchange_altura(self, cr, uid, ids, altura=0, largura=0, profundidade=0):
        """
        Calculo do volume do palete.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} a altura maxima (em metros.) suportada pelo Palete,
        @param largura: {float} a largura (em metros.) do palete,
        @param profundidade: {float} a profundidade (em metros) do palete,
        @return: {float} o volume do palete.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = self.estpem_calcular_cubagem(cr, uid, ids, altura, largura, profundidade)

        return result

    def estpem_onchange_largura(self, cr, uid, ids, altura=0, largura=0, profundidade=0):
        """
        Calculo do volume do palete.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} a altura maxima (em metros.) suportada pelo Palete,
        @param largura: {float} a largura (em metros.) do palete,
        @param profundidade: {float} a profundidade (em metros) do palete,
        @return: {float} o volume do palete.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = self.estpem_calcular_cubagem(cr, uid, ids, altura, largura, profundidade)

        return result

    def estpem_onchange_profundidade(self, cr, uid, ids, altura=0, largura=0, profundidade=0):
        """
        Calculo do volume do palete.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} a altura maxima (em metros.) suportada pelo Palete,
        @param largura: {float} a largura (em metros.) do palete,
        @param profundidade: {float} a profundidade (em metros) do palete,
        @return: {float} o volume do palete.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = self.estpem_calcular_cubagem(cr, uid, ids, altura, largura, profundidade)

        return result

    @staticmethod
    def estpem_calcular_cubagem(cr, uid, ids, altura=0, largura=0, profundidade=0):
        """
        Calculo do volume. produto entre a altura, largura e profundidade.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} a altura maxima (em metros.) suportada pelo palete,
        @param largura: {float} a largura (em metros.) do palete,
        @param profundidade: {float} a profundidade (em metros) do palete,
        @return: {float} o volume em metros cubicos.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        volume = altura * largura * profundidade / 1000000.0  # Volume em m3

        result = {'value': {'cubagem': volume}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
