# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class estoque_produto_grade(osv.Model):
    """
    Inclusao do campo ´falta_dado_logistico´ na classe ´estoque.produto.grade'.
    (heranca)

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.produto.grade'  # estgrd
    _inherit = ['estoque.produto.grade',  # estoque/control/produto_grade
                'ir.needaction_mixin']  # openerp/addons/base/ir/ir.needaction

    def _needaction_domain_get(self, cr, uid, context=None):
        """
        Mostra a quantidade de Ordens de Servico com ´´status´´ igual a "aberta"
        no menu "Ordem de servico".

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} dominio utilizado para contar os produtos quem tem dado logistico faltando.

        :see: openerp/addons/base/ir/ir.needaction (class ir_needaction_mixin)
        :copyright: OpenERP SA.
        """

        context = context and context.copy() or {}
        return []

    def estpgr_falta_dado_logistico(self, cr, uid, ids, context=None):
        """
        Verifica se faltam dados logisticos para o produto (grade).

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,

        :attention: A verificacao de endereco de picking assume que se um endereco nao for
        atribuido o picking este sera um endereco dinamico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        falta = ''
        campos = ['lastro', 'camada', 'local_armazenagem', 'corte', 'shelf_life',
                  'restricao_nivel_max', 'restricao_blocado_max', 'capacidade_picking',
                  'embalagem_master_id', 'embalagem_expedicao_id', 'produto_tipo_id', 'produto_id',
                  'estrutura_tipo_pulmao_id', 'endereco_tipo_pulmao_id', 'endereco_produto_id',
                  'dinamico']
        dados_produto = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            if not dados_produto.get('endereco_produto_id') and not dados_produto.get('dinamico'):
                falta += 'endereco de picking, '
            if not dados_produto.get('lastro'):
                falta += 'lastro, '
            if not dados_produto.get('camada'):
                falta += 'camada, '
            if not dados_produto.get('local_armazenagem'):
                falta += 'local de armazenagem, '
            if not dados_produto.get('corte'):
                falta += '% corte, '
            if not dados_produto.get('shelf_life'):
                falta += 'shelf life, '
            if not dados_produto.get('capacidade_picking'):
                falta += 'capacidade de picking, '
            if not dados_produto.get('embalagem_master_id'):
                falta += 'embalagem master, '
            else:
                campos = ['peso_liquido', 'peso_bruto', 'altura', 'largura',
                          'profundidade']
                master_id = dados_produto.get('embalagem_master_id')
                dados_master = obj_produto_embalagem.read(
                    cr, uid, master_id, campos, context=context, load='_classic_write')
                if dados_master and isinstance(dados_master, list):
                    dados_master = dados_master[0]

                if dados_master and isinstance(dados_master, dict):
                    if not dados_master.get('peso_liquido'):
                        falta += 'peso liquido (embalagem master), '
                    if not dados_master.get('peso_bruto'):
                        falta += 'peso bruto (embalagem master), '
                    if not dados_master.get('altura'):
                        falta += 'altura (embalagem master), '
                    if not dados_master.get('largura'):
                        falta += 'largura (embalagem master), '
                    if not dados_master.get('profundidade'):
                        falta += 'profundidade (embalagem master), '
            if not dados_produto.get('embalagem_expedicao_id'):
                falta += 'embalagem expedicao, '
            else:
                campos = ['peso_liquido', 'peso_bruto', 'altura', 'largura', 'profundidade']
                expedicao_id = dados_produto.get('embalagem_expedicao_id')
                dados_expedicao = obj_produto_embalagem.read(cr, uid, expedicao_id, campos, context=context)
                if dados_expedicao and isinstance(dados_expedicao, list):
                    dados_expedicao = dados_expedicao[0]

                if dados_expedicao and isinstance(dados_expedicao, dict):
                    if not dados_expedicao.get('peso_liquido'):
                        falta += 'peso liquido (embalagem de expedicao), '
                    if not dados_expedicao.get('peso_bruto'):
                        falta += 'peso bruto (embalagem de expedicao), '
                    if not dados_expedicao.get('altura'):
                        falta += 'altura (embalagem de expedicao), '
                    if not dados_expedicao.get('largura'):
                        falta += 'largura (embalagem de expedicao), '
                    if not dados_expedicao.get('profundidade'):
                        falta += 'profundidade (embalagem de expedicao), '
            if not dados_produto.get('produto_tipo_id'):
                falta += 'tipo de produto, '
            if not dados_produto.get('estrutura_tipo_pulmao_id'):
                falta += 'tipo de estrutura pulmao, '
            if not dados_produto.get('endereco_tipo_pulmao_id'):
                falta += 'tipo de endereco pulmao, '

            # A validade em dias verificada em estoque.produto
            produto_id = dados_produto.get('produto_id')
            if produto_id:
                campos = ['validade']
                dados_produto = obj_estoque_produto.read(cr, uid, produto_id, campos, context=context)

                if dados_produto and isinstance(dados_produto, list):
                    dados_produto = dados_produto[0]
                if dados_produto and isinstance(dados_produto, dict):
                    if not dados_produto.get('validade'):
                        falta += 'validade (em dias), '
        if falta:
            if falta.count(', ') > 1:
                falta = ' e '.join(falta[:-2].rsplit(', ', 1))
            else:
                falta = falta[:-2]

        return falta.capitalize()

    def estpgr_onchange_dinamico(self, cr, uid, ids, dinamico, endereco_id, context=None):
        """
        Modifica o endereco de picking.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param dinamico: {boolean} verdadeiro se o picking for dinamico
        :param endereco_id: {int} ID do endereco de armazenagem,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: ´´Flag´´ ligado/desligado de picking_dinamico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ____________________________ Objects _______________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        # Se for dinamico, verifica se o endereco e valido
        args = [('id', '=', endereco_id), ('picking_dinamico', '=', dinamico)]
        endereco_ids = obj_wms_endereco.search(cr, uid, args, context=context)

        if endereco_ids:
            endereco_id = endereco_ids[0]
        else:
            endereco_id = False

        result = {'value': {'endereco_produto_id': endereco_id}}

        return result

    def estpgr_onchange_lastro(self, cr, uid, ids, lastro=0,camada=0, embalagem_id=False):
        """
        Calcula a norma palete.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param lastro: {int} valor definido para o lastro,
        :param camada: {int} valor definido para a camada,
        :param embalagem_id: {int} ID da embalagem master,
        :return: Valor da norma palete.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Verifica a embalagem
        if lastro < 1:
            msg = _('O valor do lastro nao e valido. Deve ser igual ou maior que 1 (um).')

            raise osv.except_osv(_('Atencao!'), msg)

        result = self.estpgr_norma_cubagem(cr, uid, ids, lastro, camada, embalagem_id)

        return result

    def estpgr_onchange_camada(self, cr, uid, ids, lastro=0, camada=0, embalagem_id=False):
        """
        Calcula a norma palete.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param lastro: {int} valor definido para o lastro,
        :param camada: {int} valor definido para a camada,
        :param embalagem_id: {int} ID da embalagem master,
        :return:
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Verifica a embalagem
        if lastro < 1:
            msg = _('O valor da camada nao e valido. Deve ser igual ou maior que 1 (um).')

            raise osv.except_osv(_('Atencao!'), msg)

        result = self.estpgr_norma_cubagem(cr, uid, ids, lastro, camada, embalagem_id)

        return result

    def estpgr_onchange_embalagem_master(self, cr, uid, ids, lastro=0, camada=0, master_id=False,
                                         venda_id=False, context=None):
        """
        Calcula a norma palete e verifica os dados da embalagem.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param lastro: {int} valor definido para o lastro,
        :param camada: {int} valor definido para a camada,
        :param master_id: {int} ID da embalagem master,
        :param venda_id: {int} ID da embalagem de venda (expedicao),
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: Valores calculados da Norma, peso e volume do palete.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ____________________________ Objects _______________________________
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        # Calculo da norma palete e volume
        result = self.estpgr_norma_cubagem(cr, uid, ids, lastro, camada, master_id)

        # Verifica se as quantidades nas embalagens sao diferentes para alterar o estado de
        # fracionado e embalado
        qtde_embalagem_master = qtde_embalagem_venda = 0

        if master_id:
            brw_embalagem_master = obj_estoque_produto_embalagem.browse(cr, uid, master_id, context)
            qtde_embalagem_master = brw_embalagem_master.embalagem_id.quantidade_na_embalagem

        if venda_id:
            brw_embalagem_venda = obj_estoque_produto_embalagem.browse(cr, uid, venda_id, context)
            qtde_embalagem_venda = brw_embalagem_venda.embalagem_id.quantidade_na_embalagem

        if qtde_embalagem_master != qtde_embalagem_venda:
            result['value']['fracionado'] = True
            result['value']['embalado'] = False
        else:
            result['value']['fracionado'] = False

        # Compara dados das embalagens Master e Venda
        msg = None

        if venda_id:
            brw_venda = obj_estoque_produto_embalagem.browse(cr, uid, venda_id, context)
            qtde_venda = brw_venda.embalagem_id.quantidade_na_embalagem

            if master_id:
                brw_master = obj_estoque_produto_embalagem.browse(cr, uid, master_id, context)
                qtde_master = brw_master.embalagem_id.quantidade_na_embalagem

                if qtde_venda > qtde_master:
                    msg = 'Embalagem master (recebimento) nao pode ser menor que a embalagem de ' \
                          'venda (expedicao).'
                peso_bruto = brw_master.peso_bruto
                altura = brw_master.altura
                largura = brw_master.largura
                profundidade = brw_master.profundidade
                result['value']['peso_palete'] = peso_bruto * lastro * camada
                volume = altura * largura * profundidade / 1000000.0
                result['value']['cubagem'] = volume

        if msg:
            titulo = 'Aviso.'
            return {'warning': {'title': titulo, 'message': msg},
                    'value': {'embalagem_master_id': False, 'peso': 0.0, 'cubagem': 0.0}}

        return result

    def estpgr_onchange_embalagem_venda(self, cr, uid, ids, lastro=0, camada=0, master_id=False,
                                        venda_id=False, context=None):
        """
        Verifica os dados da embalagem de venda (expedicao).

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param master_id: {int} ID da embalagem master (recebimento),
        :param venda_id: {int} ID da embalagem de venda (expedicao),
        :param lastro: {int} quantidade no lastro,
        :param camada: {int} quantidade de camadas,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} sem o valor da embalagem master.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ____________________________ Objects _______________________________
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        # Verifica se as quantidades nas embalagens sao diferentes para alterar o estado de
        # fracionado e embalado
        qtde_embalagem_master = qtde_embalagem_venda = 0

        if master_id:
            brw_embalagem_master = obj_estoque_produto_embalagem.browse(cr, uid, master_id, context)
            qtde_embalagem_master = brw_embalagem_master.embalagem_id.quantidade_na_embalagem

        if venda_id:
            brw_embalagem_venda = obj_estoque_produto_embalagem.browse(cr, uid, venda_id, context)
            qtde_embalagem_venda = brw_embalagem_venda.embalagem_id.quantidade_na_embalagem

        if qtde_embalagem_master != qtde_embalagem_venda:
            result = {'value': {'fracionado': True, 'embalado': False}}
        else:
            result = {'value': {'fracionado': False}}

        # Verifica se os dados de altura, largura, profundidade e peso estao corretos
        msg = None
        titulo = 'Aviso.'

        if venda_id:
            brw_venda = obj_estoque_produto_embalagem.browse(cr, uid, venda_id, context)
            qtde_venda = brw_venda.embalagem_id.quantidade_na_embalagem

            if master_id:
                brw_master = obj_estoque_produto_embalagem.browse(cr, uid, master_id, context)
                qtde_master = brw_master.embalagem_id.quantidade_na_embalagem

                if qtde_venda > qtde_master:
                    result['value']['embalagem_expedicao_id'] = False
                    msg = 'Embalagem master (recebimento) nao pode ser menor que a embalagem de ' \
                          'venda (expedicao).'
            if msg:
                result['warning'] = {'title': titulo, 'message': msg}

        return result

    def estpgr_norma_cubagem(self, cr, uid, ids, lastro=0, camada=0, embalagem_id=False,
                             context=None):
        """
        Calculo da norma palete: quantidade maxima de produtos suportada pelo
        palete. E representado pelo produto entre o volume e peso da embalagem
        Master.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param lastro: {int} valor definido para o lastro,
        :param camada: {int} valor definido para a camada,
        :param embalagem_id: ID da embalagem master,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: A norma palete o peso total do palete (Kg) e volume em metros
        cubicos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ____________________________ Objects _______________________________
        obj_embalagem = self.pool.get('estoque.produto.embalagem')

        norma = lastro * camada if lastro and camada else 0
        peso = volume = 0.0

        result = {'value': {'norma_palete': norma, 'peso_palete': peso, 'cubagem': volume}}

        # Peso e volume em funcao da embalagem
        if embalagem_id:
            campos = ['altura', 'largura', 'profundidade', 'peso_bruto']
            dados_embalagem = obj_embalagem.read(cr, uid, embalagem_id, campos, context=context)

            if dados_embalagem and isinstance(dados_embalagem, list):
                dados_embalagem = dados_embalagem[0]

            if dados_embalagem and isinstance(dados_embalagem, dict):
                altura = dados_embalagem.get('altura', 0.0)
                largura = dados_embalagem.get('largura', 0.0)
                profundidade = dados_embalagem.get('profundidade', 0.0)
                peso_bruto = dados_embalagem.get('peso_bruto', 0.0)

                if not all([altura, largura, profundidade, peso_bruto]):
                    titulo = 'Aviso.'
                    msg = _('Dados da embalagem master (recebimento) estao incorretos. Os valores '
                            'para altura, largura, profundidade e o peso bruto da embalagem devem '
                            'ser maior que zero.')
                    result.update({'warning': {'title': titulo, 'message': msg}})

                result['value']['peso_palete'] = norma * peso_bruto  # Peso em Kg
                volume = altura * largura * profundidade / 1000000.0
                result['value']['cubagem'] = volume  # Volume em m3

        return result

    # _________________________ ORM Methods __________________________________
    def fields_get(self, cr, uid, fields=None, context=None, write_access=True):
        """
        Acrescenta o campo validade (em dias) de estoque/estoque.produto

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas estruturas de
        construcao,
        :param write_access: {boolean} N/A,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} campo "validade em dias" acrescentado a lista de campos.
        """

        context = context and context.copy() or {}
        fields = [] if not fields else fields

        result = super(estoque_produto_grade, self).fields_get(cr, uid, fields, context)

        result['validade'] = {'selectable': True, 'type': 'integer', 'string': 'Validade (em dias)'}

        return result

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Insere a validade (em dias) no campo validade

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param fields: {dict} dicionario com o nome dos campos e seus respectivos valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :param load:
        :return: {dict} dicionario com o nome dos campos e seus respectivos valores.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoque = self.pool.get('estoque.produto')

        results = super(estoque_produto_grade, self).read(cr, uid, ids, fields, context=context, load=load)

        for result in results if isinstance(results, list) else [results]:
            if 'produto_id' in result and result.get('produto_id'):
                produto_id = result.get('produto_id')

                if isinstance(produto_id, tuple):
                    produto_id = produto_id[0]

                campos = ['validade']
                dados_estoque = obj_estoque.read(cr, uid, [produto_id], campos, context=context, load=load)

                if dados_estoque and isinstance(dados_estoque, list):
                    dados_estoque = dados_estoque[0]

                if dados_estoque and isinstance(dados_estoque, dict):
                    result['validade'] = dados_estoque.get('validade')

        return results

    def write(self, cr, uid, ids, values, context=None):
        """
        Realiza ou nao a troca de endereco de picking e registra a validade em dias.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista de IDs,
        :param values: {dict} dicionario com os campos e seus respectivos valores,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {super} validade em dias com valor ajustado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # teste implementao para agilizar o write por causa da integração
        # como o codigo abaixo só interessa se endereco_produto_id e/ou validade estiverem sendo alterados
        # já saio para melhorar o tempo do write
        if not 'endereco_produto_id' in values and not 'validade' in values:
            result = super(estoque_produto_grade, self).write(cr, uid, ids, values, context)
            return result

        # ____________________________ Objects _______________________________
        obj_estoque_produto = self.pool.get('estoque.produto')
        obj_wms_endereco = self.pool.get('wms.endereco')

        brw_estoque_produto_grade = self.browse(cr, uid, ids, context)
        for grade in brw_estoque_produto_grade if isinstance(brw_estoque_produto_grade, list) \
                else [brw_estoque_produto_grade]:
            # Realiza a troca somente se envolver o endereco do produto
            if 'endereco_produto_id' in values:

                endereco_anterior_id = grade.endereco_produto_id.id
                endereco_novo_id = values.get('endereco_produto_id')
                endereco_ids = filter(lambda e: e, [endereco_anterior_id, endereco_novo_id])

                # Verifica se a mudanca de endereco pode ser realizada
                if not obj_wms_endereco.wmsend_disponivel(cr, uid, endereco_ids, context):
                    if endereco_novo_id:
                        msg = _('O endereco de picking nao pode ser alterado.')
                    else:
                        msg = _('O endereco de picking nao pode ser removido.')

                    if not context.get('transferencia'):  # Nao critica se for uma transferencia
                        raise osv.except_osv(_('Atencao'), msg)

                # Libera o endereco anterior
                obj_wms_endereco.wmsend_liberar(cr, uid, endereco_anterior_id, context)

                # Ocupa o novo endereco
                obj_wms_endereco.wmsend_ocupar(
                    cr, uid, endereco_novo_id, grade.produto_id.id, grade.id, context)

            # Grava a validade em dias
            if 'validade' in values:
                validade = values.get('validade')
                produto_id = grade.produto_id.id
                dados = {'validade': validade}
                obj_estoque_produto.write(cr, uid, produto_id, dados, context)

        result = super(estoque_produto_grade, self).write(cr, uid, ids, values, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
