# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields

TIPO_LOCAL_ARMAZENAGEM = [('S', 'Seco'), ('R', 'Resfriado'), ('C', 'Congelado')]


class estoque_produto_grade(osv.Model):
    """
    Parametros para o WMS: norma palete, peso do palete e volume.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.produto.grade'  # estpgr
    _inherit = 'estoque.produto.grade'  # estoque/model/produto_grade_data

    from funcoes import estpgr_fields_function, estpgr_function_falta_dado_logistico
    from funcoes import estpgr_constraint_estrutura_pulmao, estpgr_constraint_estrutura_pulmao_msg

    _columns = {
        # ___________________________ Many2One _______________________________
        'embalagem_master_id': fields.many2one('estoque.produto.embalagem',
                                               'Embalagem de recebimento (master)',
                                               ondelete='restrict'),
        'embalagem_expedicao_id': fields.many2one('estoque.produto.embalagem',
                                                  'Embalagem de expedicao (venda)',
                                                  ondelete='restrict'),
        'produto_tipo_id': fields.many2one('wms.produtotipo', 'Tipo do produto',
                                           ondelete='restrict'),
        'endereco_produto_id': fields.many2one('wms.endereco', 'Endereco de picking', select=True,
                                               ondelete='restrict'),
        'estrutura_tipo_pulmao_id': fields.many2one('wms.estruturatipo', 'Tipo de estrutura pulmao',
                                                    ondelete='restrict'),
        'endereco_tipo_pulmao_id': fields.many2one('wms.enderecotipo', 'Tipo de endereco pulmao',
                                                   ondelete='restrict'),

        # ___________________________ Integer ________________________________
        'lastro': fields.integer('Lastro'),
        'camada': fields.integer('Camada'),
        'corte': fields.integer('% de corte'),
        'shelf_life': fields.integer('% shelf life'),
        'restricao_nivel_min': fields.integer('Restricao nivel minimo'),
        'restricao_nivel_max': fields.integer('Restricao nivel maximo.'),
        'capacidade_picking': fields.integer('Capacidade do picking (master)'),
        'ponto_reposicao': fields.integer('Ponto de reposicao'),

        # ___________________________ Function _______________________________
        'cubagem': fields.function(estpgr_fields_function, method=True, digits=(9, 6),
                                   store=True, string="Volume (m3)"),
        'falta_dado_logistico': fields.function(estpgr_function_falta_dado_logistico, method=True,
                                                type='text', string='Falta dado logistico'),
        'norma_palete': fields.function(estpgr_fields_function, method=True, type='integer',
                                        store=True, string='Norma palete'),
        'peso_palete': fields.function(estpgr_fields_function, method=True, digits=(9, 3),
                                       store=True, string='Peso do palete (Kg)'),
        'estpgr_wms_uso': fields.function(estpgr_fields_function, method=True, type='boolean',
                                          string='Utilizado pelo WMS'),
        'estpgr_produto': fields.function(estpgr_fields_function, method=True, type='char',
                                          store=True, size=60, string='Produto'),
        'estpgr_ean': fields.function(estpgr_fields_function, method=True, type='char',
                                      store=True, size=14, string='EAN'),
        'estpgr_dun': fields.function(estpgr_fields_function, method=True, type='char',
                                      store=True, size=14, string='DUN'),

        # ___________________________ Selection ______________________________
        'local_armazenagem': fields.selection(TIPO_LOCAL_ARMAZENAGEM, 'Local de armazenagem',
                                              size=1),

        # ___________________________ Boolean ________________________________
        # Forca checkout. Sempre ira para separacao no checkout (nunca vai para o box)
        'embalado': fields.boolean('Embalado'),
        'perigoso': fields.boolean('Produto perigoso'),
        'fracionado': fields.boolean('Fracionado'),
        'dinamico': fields.boolean('Picking dinamico'),
    }

    _defaults = {
        'corte': lambda *arg: 100,
        'capacidade_picking': lambda *args: 0,
        'ponto_reposicao': lambda *args: 1,
        'fracionado': lambda *args: False,
        'dinamico': lambda *args: False
    }

    _constraints = [(estpgr_constraint_estrutura_pulmao,
                     lambda self, *a, **kw: self.estpgr_constraint_estrutura_pulmao_msg(*a, **kw),
                     ['Tipo de estrutura (pulmao)'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
