# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class recebimento_os(osv.Model):
    """
    Heranca em Recebimento. Verificacao de dado logistico faltando.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.os'  # recosv
    _inherit = 'recebimento.os'  # wms/inherit/recebimento_os_data

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Ao fechar a ordem de servico verificar se algum item de recebimento tem
        dado logistico faltando.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} status da ordem de servico fechada ou mensagem de erro.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = super(recebimento_os, self).osvosv_fechar_os(cr, uid, ids, context)

        for os in self.browse(cr, uid, ids):
            if os.tipo_id.codigo == 'REC':
                # Verifica todos os itens da ordem de servico
                # Se houver item faltando dado logistico a ordem de servico nao e fechada.
                for item in os.recebimento_item_ids:
                    if item.recite_falta_dado_logistico and item.state != 'A':
                        msg = _('Ordem de servico nr %s nao pode ser fechada. Existem produtos '
                                'com dado logistico faltando.' % os.numero)

                        raise osv.except_osv(_('Atencao!'), msg)

        return result

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Ao abrir uma ordem de servico verificar se algum item de recebimento tem
        dado logistico faltando.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} status da ordem de servico aberta ou mensagem de erro.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        for os in filter(lambda os: os.state == 'criada', self.browse(cr, uid, ids)):
            if os.tipo_id.codigo == 'REC':
                # Verifica todos os itens da O.S.
                # Se houver item faltando dado logistico a ordem de servico nao e fechada.
                for item in os.recebimento_item_ids:
                    if item.grade_id.falta_dado_logistico and item.state != 'A':
                        msg = _('Ordem de servico nr %s nao pode ser aberta. Existem produtos '
                                'com dado logistico faltando.' % os.numero)

                        raise osv.except_osv(_('Atencao!'), msg)

        return super(recebimento_os, self).osvosv_abrir_os(cr, uid, ids, context)

    def recosv_imprimir_dados_logisticos(self, cr, uid, ids, context=None):
        """
        Impressao do relatório de dados logisticos.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {super} o relatorio com a lista dos itens com dado logistico faltando.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from datetime import datetime
        from pytz import timezone

        # Hora Brasil (padrao)
        TIMEZONE = 'Brazil/East'

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_recebimento_itens = self.pool.get('recebimento.itens')

        # Produtos com dado logistico faltando
        args = [('recite_falta_dado_logistico', '=', True), ('recebimento_id', 'in', ids)]
        item_ids = obj_recebimento_itens.search(cr, uid, args, context=context)
        divergente_ids = map(lambda i: i.grade_id.id, obj_recebimento_itens.browse(
            cr, uid, item_ids))
        dados = {'ids': divergente_ids, 'model': 'estoque.produto.grade', 'form': {}}
        agora = datetime.now(timezone('UTC')).astimezone(timezone(TIMEZONE))
        agora = agora.strftime('%d/%m/%Y - %Hh%M')

        result = {'type': 'ir.actions.report.xml', 'report_name': 'lista.produto.dados_logisticos',
                  'datas': dados, 'context': {'subtitulo': 'LISTA DE PRODUTOS COM DADO LOGISTICO '
                                                           'FALTANDO', 'agora': agora}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
