# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
from datetime import datetime


# noinspection PyPep8Naming
class wms_inventario(osv.Model):
    """
    Inventario.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.inventario'  # wmsinv
    _inherit = 'wms.inventario'  # wms/inventario/model/inventario_data

    def wmsinv_fechar_inventario(self, cr, uid, ids, context=None):
        """
        Fecha a contagem de inventario e atualiza os dados dos enderecos.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com ID do inventario fechado.
        :rtype: list
        :raise: mensagem de erro se existir ordem de servico ´aberta´ ou ´fechada´ com divergencia.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Mesnagem padrao
        msg = _('Nao foi possivel fechar o inventario. ')

        # Verifica pendencias de ordens de servico criadas
        if self._wmsinv_verificar_pendencia_os_criada(cr, uid, ids, context):
            msg += _('Existem ordens de servico em aberto.')
            raise osv.except_osv(_('Atencao'), msg)

        # Verifica pendencias de ordens de servico abertas
        if self._wmsinv_verificar_pendencia_os_aberta(cr, uid, ids, context):
            msg += _('Existem ordens de servico em aberto.')
            raise osv.except_osv(_('Atencao'), msg)

        # Verifica pendencias de ordens de servico fechadas com divergencia
        if self._wmsinv_verificar_pendencia_os_fechada(cr, uid, ids, context):
            msg += _('Existem ordens de servico com divergencia de contagem.')
            raise osv.except_osv(_('Atencao'), msg)

        # Atualiza, libera e desbloqueia os enderecos utilizados pelo inventario
        _sql = """SELECT  ii.endereco_id,
                          ii.wmsiit_produto_id,
                          ii.wmsiit_grade_id,
                          ii.wmsiit_quantidade,
                          ii.validade,
                          e.tipo_nivel
                  FROM    wms_inventario_itens  AS ii
                    INNER JOIN wms_endereco     AS e      ON e.id = ii.endereco_id
                  WHERE   active = True
                    AND   os_id in (SELECT id
                                    FROM   inventario_os
                                    WHERE  inventario_id  IN (%s)
                                   )"""
        cr.execute(_sql % tuple(ids),)

        picking_ids = []  # Lista com ID dos enderecos de picking com quantidade zero

        for wms_inventario_item_dados in cr.dictfetchall():
            agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            wms_endereco_dados = dict(produto_id=wms_inventario_item_dados.get('wmsiit_produto_id'),
                                      grade_id=wms_inventario_item_dados.get('wmsiit_grade_id'),
                                      quantidade=wms_inventario_item_dados.get('wmsiit_quantidade'),
                                      validade=wms_inventario_item_dados.get('validade'),
                                      state='O', bloqueado=False, write_date=agora, write_uid=uid)

            # Libera o endereco se a quantidade for zero
            if wms_inventario_item_dados.get('wmsiit_quantidade') == 0:
                wms_endereco_dados = dict(wms_endereco_dados, validade=None, state='L',
                                          grade_id=None, produto_id=None, capacidade=0)

                # Se for picking deve ser removido da grade (endereco de picking do produto)
                if wms_inventario_item_dados.get('tipo_nivel') == 'PK':
                    picking_ids.append(wms_inventario_item_dados.get('endereco_id'))

            _sql = """UPDATE wms_endereco SET %s WHERE id = %s"""
            _values = ', '.join(map(lambda m: '%s=%%s' % m, wms_endereco_dados.keys()))
            _params = wms_endereco_dados.values()
            cr.execute(_sql % (_values, wms_inventario_item_dados.get('endereco_id')), _params)

        # Remove os enderecos de picking vazios da grade
        if picking_ids:
            _sql = """UPDATE  estoque_produto_grade
                      SET     endereco_produto_id   = NULL
                      WHERE   endereco_produto_id   IN (%s)"""
            cr.execute(_sql % tuple(picking_ids),)

        # Fecha o inventario
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        wms_inventario_dados = dict(state='F', write_date=agora, write_uid=uid,
                                    wmsinv_data_fechamento=agora)

        self.write(cr, uid, ids, wms_inventario_dados, context)

        return ids

    # noinspection PyMethodMayBeStatic
    def wmsinv_cancelar_inventario(self, cr, uid, ids, context=None):
        """
        Cancela a contagem de inventario e todas as ordens de servico relacionadas.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com ID do inventario cancelado.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Lista de inventarios
        inventario_ids = ids if isinstance(ids, list) else [ids]

        # Lista de ordens de servico de inventario
        _sql = """SELECT  id FROM inventario_os WHERE active = True AND inventario_id IN (%s)"""
        cr.execute(_sql % tuple(inventario_ids),)
        inventario_os_ids = map(lambda m: m[0], cr.fetchall())

        # Cancela as ordens de servico
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        inventario_os_dados = dict(data_cancelamento=agora, state='cancelada', write_uid=uid,
                                   write_date=agora)
        _sql = """UPDATE inventario_os SET %s WHERE id IN %s"""
        _values = ', '.join(map(lambda m: '%s=%%s' % m, inventario_os_dados.keys()))
        _params = inventario_os_dados.values()
        cr.execute(_sql % (_values, tuple(inventario_os_ids + [0])), _params)

        # Cancela o inventario
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        wms_inventario_dados = dict(wmsinv_data_cancelamento=agora, state='C', write_uid=uid,
                                    write_date=agora)
        _sql = """UPDATE wms_inventario SET %s WHERE id IN %s"""
        _values = ', '.join(map(lambda m: '%s=%%s' % m, wms_inventario_dados.keys()))
        _params = wms_inventario_dados.values()
        cr.execute(_sql % (_values, tuple(inventario_ids + [0])), _params)

        # Desbloqueio dos enderecos
        _sql = """UPDATE wms_endereco
                  SET bloqueado = False
                  WHERE id IN (SELECT  endereco_id
                               FROM    wms_inventario_itens
                               WHERE   active = True
                                 AND   os_id in (SELECT id
                                                 FROM   inventario_os
                                                 WHERE  inventario_id in (%s)
                                                 )
                              )"""
        cr.execute(_sql % tuple(inventario_ids),)

        return ids

    def wmsinv_button_recontagem(self, cr, uid, ids, context=None):
        """
        Recontagem de inventario. Abre uma nova contagem de inventario.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs do inventario,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com o ID das ordens de servico que sofreram recontagem.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_inventario_os = self.pool.get('inventario.os')

        # Ordens de servico (abertas)
        args = [('inventario_id', 'in' if isinstance(ids, list) else '=', ids),
                ('state', '=', 'fechada')]
        os_ids = obj_inventario_os.search(cr, uid, args, context=context)

        result = self._wmsinv_recontagem(cr, uid, os_ids, context)

        return result

    def _wmsinv_recontagem(self, cr, uid, ids, context=None):
        """
        Recontagem de inventario. Abre uma nova contagem de inventario.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com ID das ordens de servico de inventario,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com os IDs das ordens de servico que sofreram recontagem.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if not ids:
            return []

        # ___________________________ Objects ________________________________
        obj_inventario_itens = self.pool.get('wms.inventario.itens')

        inventario_os_ids = []  # Lista com o ID das ordens de servicos de inventario
        args = [('state', '!=', 'I'), ('os_id', 'in' if isinstance(ids, list) else '=', ids)]
        inventario_item_ids = obj_inventario_itens.search(cr, uid, args, context=context)

        if not inventario_item_ids:  # Nao existe item de inventario com divergencia
            return []

        _keys = 'os_id, endereco_id, verifica_validade, contagem, validade, wmsiit_produto_id,' \
                'wmsiit_grade_id'
        _sql = """SELECT %s FROM wms_inventario_itens WHERE id IN %s"""
        cr.execute(_sql % (_keys, tuple(inventario_item_ids + [0])))

        # Cria os novos itens de inventario para a nova contagem
        for inventario_item_dados in cr.dictfetchall():
            # Divergencias sao trocadas para contagem no novo item
            agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            contagem = inventario_item_dados.get('contagem', 0) + 1
            inventario_item_dados = dict(inventario_item_dados, state='C', wmsiit_quantidade=0,
                                         active=True, write_date=agora, write_uid=uid,
                                         create_date=agora, create_uid=uid, contagem=contagem)
            _keys = ', '.join(inventario_item_dados.keys())
            _values = ', '.join(['%s'] * len(inventario_item_dados.values()))
            _params = inventario_item_dados.values()
            _sql = """INSERT INTO wms_inventario_itens (%s) VALUES (%s)"""
            cr.execute(_sql % (_keys, _values), _params)

            # Somente as ordens de servico que tiveram itens divergentes
            os_id = inventario_item_dados.get('os_id')
            inventario_os_ids.append(os_id) if os_id and os_id not in inventario_os_ids else None  # Nao duplica o ID!

        # Desativa os itens da contagem anterior
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        item_anterior_dados = dict(write_uid=uid, write_date=agora, active=False)
        _sql = """UPDATE wms_inventario_itens SET %s WHERE id IN %s"""
        _values = ', '.join(map(lambda m: '%s=%%s' % m, item_anterior_dados.keys()))
        _params = item_anterior_dados.values()
        cr.execute(_sql % (_values, tuple(inventario_item_ids + [0])), _params)

        # Reabre a ordem de servico com a nova contagem
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        inventario_os_dados = dict(state='aberta', data_fechamento=None, write_date=agora,
                                   write_uid=uid, data_abertura=agora)
        _sql = """UPDATE  inventario_os
                  SET     invosv_contagem=invosv_contagem + 1, %s
                  WHERE   id IN %s"""
        _values = ', '.join(map(lambda m: '%s=%%s' % m, inventario_os_dados.keys()))
        _params = inventario_os_dados.values()
        cr.execute(_sql % (_values, tuple(inventario_os_ids + [0])), _params)

        # Registro do historico
        acao = _('Abertura da ordem de servico para recontagem de itens.')

        for inventario_os_id in inventario_os_ids:
            agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            # TODO: ´inventario_id´ deve ser trocado para ´inventario_os_id´ na classe inventario_os_historico para evitar confusoes
            inventario_os_historico_dados = dict(inventario_id=inventario_os_id, usuario_id=uid,
                                                 data_historico=agora, acao=acao, create_date=agora,
                                                 write_date=agora, write_uid=uid, create_uid=uid,
                                                 active=True)
            _sql = """INSERT INTO inventario_os_historico (%s) VALUES (%s)"""
            _keys = ', '.join(inventario_os_historico_dados.keys())
            _values = ', '.join(['%s'] * len(inventario_os_historico_dados.values()))
            _params = inventario_os_historico_dados.values()
            cr.execute(_sql % (_keys , _values), _params)

        return inventario_os_ids

    def _wmsinv_verificar_pendencia_os_aberta(self, cr, uid, ids, context=None):
        """
        Verifica se existem pendencias de ordens de servico com status ´aberta´.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com o ID dos inventarios,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com ID das ordens de servico de inventarios com pendencia (abertas).
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_inventario_os = self.pool.get('inventario.os')

        # Verifica pendencias de ordens de servico abertas
        args = [('inventario_id', 'in' if isinstance(ids, list) else '=', ids),
                ('state', '=', 'aberta')]
        result = obj_inventario_os.search(cr, uid, args, context=context)

        return result

    def _wmsinv_verificar_pendencia_os_fechada(self, cr, uid, ids, context=None):
        """
        Verifica se existem pendencias de ordens de servico com status ´fechada´ com divergencia.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com o ID dos inventarios,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com ID dos itens de inventario com divergencia.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_inventario_os = self.pool.get('inventario.os')
        obj_wms_inventario_itens = self.pool.get('wms.inventario.itens')

        # Verifica pendencias de ordens de servico fechadas com itens divergentes
        args = [('inventario_id', 'in' if isinstance(ids, list) else '=', ids),
                ('state', '=', 'fechada')]
        os_ids = obj_inventario_os.search(cr, uid, args, context=context)
        args = [('os_id', 'in', os_ids), ('state', '!=', 'I')]
        result = obj_wms_inventario_itens.search(cr, uid, args, context=context)

        return result

    def _wmsinv_verificar_pendencia_os_criada(self, cr, uid, ids, context=None):
        """
        Verifica se existem pendencias de ordens de servico com status ´criada´.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com o ID dos inventarios,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com ID das ordens de servico de inventarios com pendencia (criada).
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_inventario_os = self.pool.get('inventario.os')

        # Verifica pendencias de ordens de servico abertas
        args = [('inventario_id', 'in' if isinstance(ids, list) else '=', ids),
                ('state', '=', 'criada')]
        result = obj_inventario_os.search(cr, uid, args, context=context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
