# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class inventario_itens(osv.Model):
    """
    Itens de inventario. (heranca)

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.inventario.itens'  # wmsiit
    _inherit = 'wms.inventario.itens'  # wms/inventario/model/inventario_itens_data

    # noinspection PyMethodMayBeStatic
    def wmsiit_onchange_validade(self, cr, uid, ids, validade, context=None):
        """
        Altera o ´´status´´ do item em funcao da alteracao da data de validade.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista de IDs,
        :param validade: data de validade informada na contagem,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: long|list[long]
        :type validade: date
        :type context: dict
        :return: valor do campo "usuario_digitacao_id" ajustado.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = dict(value=dict(usuario_digitacao_id=uid, state='O'))

        return result

    def wmsiit_onchange_produto(self, cr, uid, ids, produto_id, context=None):
        """
        Quando o item for modificado atribui a grade se o item tiver apenas uma.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista de IDs,
        :param produto_id: ID do item/produto,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list|int
        :type produto_id: int
        :type context: dict
        :return: grade do produto se este tiver apenas uma.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')

        result = dict(value=dict(wmsiit_grade_id=False))

        # Verifica grade unica
        args = [('produto_id', '=', produto_id)]
        grade_ids = obj_estoque_produto_grade.search(cr, uid, args, context=context)

        if len(grade_ids) == 1:  # Atribui a grade se o item tiver apenas uma
            result['value']['wmsiit_grade_id'] = grade_ids[0]

        return result

    # _________________________ ORM Methods __________________________________
    def unlink(self, cr, uid, ids, context=None):
        """
        Nao permite a exclusao de um item de inventario. Um item de inventario so pode ser excluido
        pelo sistema e nao pelo usuario.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista de IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list|int
        :type context: dict
        :return: exclusao do registro ou mensagem de impossibilidade de exclusao do
        registro.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Nao permite excluir itens de inventario
        if not context.get('inventario'):
            msg = _('Item de inventario nao pode ser excluido.')

            raise osv.except_osv(_('Atencao!'), msg)

        result = super(inventario_itens, self).unlink(cr, uid, ids, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
