# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class inventario_os(osv.Model):
    """
    Enderecamento. Ordens de Armazenagem.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'inventario.os'  # invosv
    _inherit = 'inventario.os'  # wms/inventario/inherit/os

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Abrir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: verdadeiro se a ordem de servico foi aberta.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('inventario.os.historico')

        for os in self.browse(cr, uid, ids, context):
            if os.tipo == 'INV':  # Ordem de servico - Inventario
                # Se nao existir responsavel a ordem de servico nao podera ser aberta
                if not os.conferente_id:
                    msg = _('Informe o nome do responsavel pela ordem de servico.')

                    raise osv.except_osv(_('Atencao!'), msg)

            acao = 'Abrir ordem de servico'
            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

        return super(inventario_os, self).osvosv_abrir_os(cr, uid, ids, context)

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: verdadeiro se a ordem de servico foi cancelada.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_os_historico = self.pool.get('inventario.os.historico')

        # Itens do enderecamento
        for os in self.browse(cr, uid, ids, context):
            if os.tipo_id.codigo == 'INV':
                endereco_ids = []  # Lista de enderecos a serem desbloqueados

                for item in os.inventario_item_ids:
                    endereco_ids.append(item.endereco_id.id)

                # Desbloqueia os enderecos
                obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_ids, context)

            acao = 'Cancelar ordem de servico'
            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

        return super(inventario_os, self).osvosv_cancelar_os(cr, uid, ids, context)

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Fechar ordem de servico. Verifica se ha divergencia nas contagens e registra a contagem.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: verdadeiro se a ordem de servico foi fechada.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('inventario.os.historico')

        # Verifica se todos os itens ja foram inventariados para fechar a OS
        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo == 'INV':

                # Verifica divergencias das contagens
                self.invosv_registra_divergencia(cr, uid, os.id, context)

                acao = 'Fechar ordem de servico'
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

        result = super(inventario_os, self).osvosv_fechar_os(cr, uid, ids, context)

        return result

    def invosv_registra_divergencia(self, cr, uid, ids, context=None):
        """
        Verifica se ha divengencia entre as contagens e realiza o registro.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com o ID dos itens de contagem divergente.
        :rtype: list
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_inventario_itens = self.pool.get('wms.inventario.itens')

        result = []

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            # Verifica se existe divergencia para alterar o status do item.
            for item in os.inventario_item_ids:
                if item.state != 'I':
                    quantidade = item.wmsiit_quantidade

                    # Itens de contagens anteriores
                    args = [('os_id', '=', os.id), ('endereco_id', '=', item.endereco_id.id),
                            ('wmsiit_quantidade', '=', quantidade), ('active', '=', False)]

                    if item.verifica_validade:  # Considera tambem a data de validade
                        args.append(('validade', '=', item.validade))

                    if obj_inventario_itens.search(cr, uid, args, context=context):
                        _state = 'I' if item.contagem != 1 else 'D'  # Inventariado
                        result.append(item.id)
                    else:
                        _state = 'D'  # Com divergencia

                    dados = {'state': _state, 'wmsiit_quantidade': quantidade}
                    obj_inventario_itens.write(cr, uid, item.id, dados, context)

        return result

    def osvosv_imprimir_os(self, cr, uid, ids, context=None):
        """
        Imprimir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: verdadeiro se a ordem de servico foi impressa.
        :rtype: bool
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('inventario.os.historico')

        for os in self.browse(cr, uid, ids, context):
            if os.tipo == 'INV':
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'imprimir', context)

                dados = {'impressao': os.impressao + 1}
                self.write(cr, uid, os.id, dados, context)

                return {'type': 'ir.actions.report.xml', 'report_name': 'inventario.os',
                        'datas': {'ids': [os.id], 'model': 'inventario.os'}}

        return super(inventario_os, self).osvosv_imprimir_os(cr, uid, ids, context)

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Ordens de servico de inventario devem ser criadas pela opcao de 'Planejamento'

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param values: dicionario com os campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type context: dict
        :return: ID (numero) da nova ordem de servico
        :rtype: int

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(inventario_os, self).create(cr, uid, values, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
