# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""


# _______________________ wms.inventario______________________________________
def wmsinv_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :type cr: object
    :type uid: int
    :type ids: long|list[long]
    :type arg: dict
    :type context: dict
    :param context: um dicionario padrao para valores contextuais,
    :return: valores ajustados dos campos.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for brw_wms_inventario in self.browse(cr, uid, ids, context):
        try:
            if name == 'name':
                value = 'Inventario nr %05d' % brw_wms_inventario.id
            elif name == 'wmsinv_numero':
                value = '%05d' % brw_wms_inventario.id
            elif name == 'wmsinv_inventario_ok':
                value = True
                for os in brw_wms_inventario.wmsinv_os_ids:
                    if os.state == 'aberta':
                        value = False
                        break
                    elif os.state == 'fechada' and os.invosv_divergencia:
                        value = False
                        break
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_wms_inventario.id] = value

    return result


# _______________________ wms.inventario.itens________________________________
def wmsiit_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :type cr: object
    :type uid: int
    :type ids: long|list[long]
    :type arg: dict
    :type context: dict
    :return: valores ajustados dos campos.
    :rtype: dict

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    # ___________________________ Objects ________________________________
    obj_adm_utils = self.pool.get('adm.utils')

    result = {}

    # Contexto para mostrar produto baseado no parametro de descricao da grade
    campos = ['estpar_descricao_grade']
    adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)

    for brw_wms_inventario_item in self.browse(
            cr, uid, ids if isinstance(ids, list) else [ids], context):
        try:
            if name == 'wmsiit_endereco_nome':
                # Ordena por dep+rua+pred+niv+apto
                value = '%03d%03d%03d%03d%03d' % (brw_wms_inventario_item.endereco_id.deposito,
                                                  brw_wms_inventario_item.endereco_id.rua,
                                                  brw_wms_inventario_item.endereco_id.predio,
                                                  brw_wms_inventario_item.endereco_id.nivel,
                                                  brw_wms_inventario_item.endereco_id.apartamento)
            elif name == 'wmsiit_endereco_tipo':
                tipo_nivel = brw_wms_inventario_item.endereco_id.tipo_nivel
                value = tipo_nivel if tipo_nivel == 'PK' else 'PU'
            elif name == 'wmsiit_produto':
                codigo = brw_wms_inventario_item.wmsiit_produto_id.codigo or ''
                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    produto = brw_wms_inventario_item.wmsiit_grade_id.descricao or ''
                else:  # Produto + grade
                    produto = brw_wms_inventario_item.wmsiit_produto_id.name or ''
                value = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)
            elif name == 'wmsiit_grade':
                value = brw_wms_inventario_item.wmsiit_grade_id.grade
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_wms_inventario_item.id] = value

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
