# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields

STATUS_INVENTARIO = [('C', 'Contagem'), ('O', 'Contado'), ('D', 'Divergencia'),
                     ('I', 'Inventariado')]


# noinspection PyPep8Naming
class wms_inventario_itens(osv.Model):
    """
    Itens de inventario.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    from funcoes import wmsiit_fields_function

    _name = 'wms.inventario.itens'  # wmsiit
    _inherits = {'wms.endereco': 'endereco_id'}

    _columns = {
        # ___________________________ Many2One _______________________________
        'endereco_id': fields.many2one('wms.endereco', 'Endereco', required=True,
                                       ondelete='restrict'),
        'usuario_contagem_id': fields.many2one('res.users', 'Contador',
                                               help='Usuario responsavel pela contagem'),
        'usuario_digitacao_id': fields.many2one('res.users', 'Digitador',
                                                help='Usuario responsavel pela digitacao.'),
        'wmsiit_produto_id': fields.many2one('estoque.produto', 'Produto', select=True),
        'wmsiit_grade_id': fields.many2one('estoque.produto.grade', 'Grade', select=True),

        # ___________________________ Float __________________________________
        'wmsiit_quantidade': fields.float('Quantidade contada', digits=(12, 3)),

        # ___________________________ Integer ________________________________
        'contagem': fields.integer('Contagem'),  # Numero da contagem

        # ___________________________ Date ___________________________________
        'data_contagem': fields.date('Data da contagem'),
        'validade': fields.date('Validade',
                                help='Data de validade informada para ser comparada (validade do produto no endereco).'),

        # ___________________________ Boolean ________________________________
        # Verdadeiro se a validade deve ser comparada
        'verifica_validade': fields.boolean('Verifica validade'),
        'active': fields.boolean('Ativo'),

        # ___________________________ Selection ______________________________
        'state': fields.selection(STATUS_INVENTARIO, 'status', size=1),

        # ___________________________ Function _______________________________
        'wmsiit_endereco_tipo': fields.function(wmsiit_fields_function, type='char', size=2,
                                                method=True, store=True, string='Tipo de endereco'),
        'wmsiit_endereco_nome': fields.function(wmsiit_fields_function, type='char', size=15,
                                                method=True, store=True, string='Endereco'),
        'wmsiit_produto': fields.function(wmsiit_fields_function, type='char', size=60, method=True,
                                          string='Produto'),
        'wmsiit_grade': fields.function(wmsiit_fields_function, type='char', size=2, method=True,
                                        string='Grade')
    }

    _order = 'wmsiit_endereco_nome'

    _defaults = {
        'active': lambda *args: True,
        'state': lambda *args: 'C',
        'contagem': 1
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
