# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report import report_sxw
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class wms_relatorio_inventario_contagem(report_sxw.rml_parse):
    """
    Relatorio de contagem do inventario.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context):
        """
        Atribui valores as funcoes utilizada no relatorio.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param name: nome do modelo utilizado,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type name: str
        :type context: dict
        :return: relatorio de inventario.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from datetime import datetime
        from pytz import timezone

        context = context and context.copy() or {}

        super(wms_relatorio_inventario_contagem, self).__init__(cr, uid, name, context=context)

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))

        self.localcontext.update({'datetime': datetime, 'agora': agora, 'name': name})
        self.context = context

        # ___________________________ Objects ________________________________
        obj_inventario_os_historico = self.pool.get('inventario.os.historico')
        obj_inventario_os = self.pool.get('inventario.os')

        ids = context.get('active_ids')

        # Registra o numero de impressoes das ordens de servico
        for active_id in context.get('active_ids'):
            # Quantidade de impressao
            inventario_os_dados = obj_inventario_os.read(cr, uid, active_id, ['impressao'], context=context)

            if inventario_os_dados and isinstance(inventario_os_dados, list):
                inventario_os_dados = inventario_os_dados[0]

            if inventario_os_dados and isinstance(inventario_os_dados, dict):
                impressao = inventario_os_dados.get('impressao', 0) + 1
                obj_inventario_os.write(cr, uid, active_id, {'impressao': impressao}, context=context)

        # Registra no historico
        acao = _('Imprimir contagem de inventario.')
        obj_inventario_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context=context)

report_sxw.report_sxw('report.inventario.contagem', 'inventario.os',
                      'addons/wms/inventario/report/inventario_contagem.rml',
                      parser=wms_relatorio_inventario_contagem,  header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
