# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _
import locale


class wizard_inventario_exportar_dados(osv.TransientModel):
    """
    Exporta dados do inventario para arquivo que sera utilizado no SigGestao.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.inventario.exportar.dados'  # wizied
    _inherit = 'wizard.inventario.exportar.dados'  # wms/inventario/wizard/exportar_dados_data

    def wizied_button_exportar(self, cr, uid, ids, context=None):
        """
        Exporta os dados do inventario

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {object} formulario de exportacao.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        campos = ['wizied_layout', 'wizied_arquivo_nome']
        dados_form = self.read(cr, uid, ids, campos, context=context, load='_classic_write')

        if dados_form and isinstance(dados_form, list):
            dados_form = dados_form[0]

        if dados_form and isinstance(dados_form, dict):
            inventario_ids = context.get('active_ids')
            layout = dados_form.get('wizied_layout')
            arquivo = dados_form.get('wizied_arquivo_nome')

            self.wizied_exportar_dados_inventario(
                cr, uid, inventario_ids, arquivo, layout, context=context)

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(
            cr, uid, 'wms', 'view_wizard_inventario_exportar_dados_form')
        result = {'view_type': 'form', 'view_mode': 'form', 'view_id': [view and view[1] or False],
                  'res_model': 'wizard.inventario.exportar.dados', 'type': 'ir.actions.act_window',
                  'res_id': ids[0], 'name': _('Exportar dados do inventario'),  'target': 'new',
                  'context': context}

        return result

    def wizied_exportar_dados_inventario(self, cr, uid, ids, filename, layout=715, context=None):
        """
        Exporta os dados do inventario.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs dos inventarios,
        :param layout: {int} tipo de layout do arquivo de importacao/exportacao,
        :param filename: {char} nome do arquivo,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {object} formulario de exportacao.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_inventario = self.pool.get('wms.inventario')

        try:
            filename = open(filename, 'w')
        except Exception as error:
            print error
            raise osv.except_osv(_('Erro!'),
                                 _('Nao foi possivel criar/abrir o arquivo % s.\n%s' %
                                   (str(filename), error)))

        brw_inventario = obj_wms_inventario.browse(cr, uid, ids, context)

        # Consolida os itens inventariados
        item_dados = {}

        for inventario in brw_inventario:
            for os in inventario.wmsinv_os_ids:
                for item in filter(lambda i: i.produto_id, os.inventario_item_ids):
                    if item.id not in item_dados:
                        produto = item.produto_id
                        grade = item.grade_id
                        embalagem = grade.embalagem_expedicao_id

                        # Codigo de barras da embalagem de venda (menor unidade)
                        codigo_barras = self._wizied_embalagem_codigo_barra(
                            cr, uid, produto.id, grade.id, embalagem.id)

                        if not codigo_barras:  # Tenta codigo de barras da master
                            embalagem = grade.embalagem_master_id
                            codigo_barras = self._wizied_embalagem_codigo_barra(
                                cr, uid, produto.id, grade.id, embalagem.id)

                            if not codigo_barras:  # Nenhum codigo de barras
                                codigo_barras = [u'00000000000000']

                        codigo_barras = codigo_barras[0]
                        qtde_na_embalagem = embalagem.embalagem_id.quantidade_na_embalagem

                        preco_venda = produto.preco_venda or 0.0
                        saldo = grade.endereco_produto_id.saldo or 0.0
                        item_dados[item.id] = {'descricao': item.produto_id.name.upper(),
                                               'preco_venda': preco_venda, 'saldo': saldo,
                                               'quantidade': item.quantidade,  # quantidade contada
                                               'codigo_barras': codigo_barras[1:],
                                               'qtde_na_embalagem': qtde_na_embalagem}
                    else:
                        item_dados[item.id]['quantidade'] += item.quantidade

        for item in item_dados:
            if layout == '705':
                detalhe = self._wizied_layout_705(item_dados[item])
            else:
                detalhe = self._wizied_layout_715(item_dados[item])

            filename.write(detalhe)

        filename.close()

        result = filename.name

        return result

    # TODO: embalagem_codigo_barra deve ser um atribuito da classe automacao.codigo.barra
    def _wizied_embalagem_codigo_barra(self, cr, uid, produto_id, grade_id, embalagem_id,
                                       context=None):
        """
        Codigos de barras do produto/grade/embalagem

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param produto_id: {int} ID do produto,
        :param grade_id: {int} ID da grade,
        :param embalagem_id: {int} ID da embalagem (classe: estoque.produto.embalagem),
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} lista com os codigos de barras dos produtos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        if not all((produto_id, grade_id, embalagem_id)):
            return result
        # ___________________________ Objects ________________________________
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')


        args = [('produto_id', '=', produto_id), ('grade_id', '=', grade_id),
                ('produto_embalagem_id', '=', embalagem_id)]
        codigo_barra_ids = obj_automacao_codigo_barra.search(cr, uid, args, context=context)

        campos = ['codigo_barra']
        dados_cb = obj_automacao_codigo_barra.read(cr, uid, codigo_barra_ids, campos, context=context)

        if dados_cb and isinstance(dados_cb, list):
            result = map(lambda i: i.get('codigo_barra'),
                         filter(lambda f: f.get('codigo_barra'), dados_cb))

        if dados_cb and isinstance(dados_cb, dict):
            result = dados_cb.get('codigo_barra', [])

            if result and not isinstance(result, list):
                result = [result]

        return result

    @staticmethod
    def _wizied_layout_715(dados):
        """
        Formata os dados do item de inventario conforme definido no layout de exportacao est715.

        layout
        ======
                 1          2         3         4         5
        123456789012345678901234567890123456789012345678901
        INVE        9999999999999                  99999999
        `+-´        `-----+-----´                  `-+-´`+´
         |                |                          |   |
         `--> Fixo        `--> Codigo de barras      |   `--> Casas decimais
                                                     `--> Qtde. contada (3 bytes finais

        :param dados: {dict} dicionario com os valores do item,
        :return: {char} dado formatado no padrao est715.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        result = 'INVE        {0:<13}                  {1}\n'.format(
            str(dados.get('codigo_barras'))[:13],
            str('%.3f' % dados.get('quantidade')).replace('.', '')[-8:])

        return result

    @staticmethod
    def _wizied_layout_705(dados):
        """
        Formata os dados do item de inventario conforme definido no layout de importacao para
        coloter est705

        layout
        ======
                 1          2         3         4         5
        12345678901234567890123456789012345678901234567890123
        9999999999999XXXXXXXXXXXXXXXXXXXX999.999,9999.999,999
        `-----+-----´`---------+--------´`----+---´`----+---´
              |                |              |         |
              `--> Codigo de barras           |         `--> Saldo do item
                               |              `--> Preco de venda do item
                               `--> Nome do item

        :param dados: {dict} dicionario com os valores do item,
        :return: {char} dado formatado no padrao est705.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        locale.setlocale(locale.LC_ALL, 'pt_BR.utf8')
        preco_venda = locale.currency(dados.get('preco_venda'), symbol=None, grouping=True)
        saldo = locale.currency(dados.get('saldo'), symbol=None, grouping=True)

        result = '{0:>13}{1:<20}{2:>10}{3:>10}\n'.format(
            str(dados.get('codigo_barras'))[:13],
            dados.get('descricao', ''), preco_venda, saldo)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
