# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
from datetime import datetime


# noinspection PyPep8Naming
class wizard_inventario(osv.TransientModel):
    """
    Transferencia manual e automatica.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.inventario'  # wizinv
    _inherit = 'wizard.inventario'  # wms/inventario/wizard/inventario_data

    def wizinv_button_gerar_contagem_inventario(self, cr, uid, ids, context=None):
        """
        Gerar as ordens de servico com a contagem de inventario.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID do planejamento de inventario
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: int|list
        :type context: dict
        :return: lista com o ID das ordens de servico de contagem.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_adm_utils = self.pool.get('adm.utils')

        # Dados do inventario
        dados_inventario = self.read(cr, uid, ids, [], context=context)

        if dados_inventario and isinstance(dados_inventario, list):
            dados_inventario = dados_inventario[0]

        descricao = 'Inventario %s.' % ('rotativo' if context.get('active_ids') else 'total')
        descricao = dados_inventario.get('wizinv_descricao', descricao)

        if not dados_inventario or not isinstance(dados_inventario, dict):
            # Se nao houver dados de inventario...
            return []  # ... nao cria nenhuma ordem de servico

        # Por padrao nao verifica a data de validade
        verifica_validade = dados_inventario.get('verifica_validade', False)

        # Por padrao o saldo sempre sera importado na 1a. contagem
        importar_saldo = dados_inventario.get('importar_saldo', True)

        # Quantidade maxima de predios em cada ordem de servico
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', 'estpar_qtde_predios', context=context)

        args = [('id', 'in', context.get('active_ids'))] if context.get('active_ids') else []

        # Enderecos a serem inventariados de acordo com os parametros
        args.extend(self._wizinv_considerar_virtual(dados_inventario))
        endereco_ids = obj_wms_endereco.search(cr, uid, args, context=context)

        # Verifica pendencias para inventario. Nao pode haver movimentacao nos enderecos
        args = [('id', 'in', endereco_ids), '|', '|', ('reserva_debito', '!=', 0),
                ('reserva_credito', '!=', 0), ('bloqueado', '=', True)]

        if obj_wms_endereco.search(cr, uid, args, context=context):
            msg = _('Nao foi possivel gerar a contagem de inventario. Verifique se existe algum '
                    'endereco bloqueado, alguma transacao pendente para qualquer endereco ou se os '
                    'parametros informados consideram enderecos para inventariar.')
            raise osv.except_osv(_('Atencao!'), msg)

        # Cria as ordens de servico de inventario e respectivos itens
        result = []  # Lista das ordens de servico
        rua = 1  # Primeira ordem de quebra
        lado = 1  # Segunda ordem de quebra
        nivel = 0  # Terceira ordem de quebra
        tipo_nivel = 'PK'  # Picking. Unica ordem de quebra para inventario rotativo
        contagem = 2 if importar_saldo else 1
        produto_ids = []  # Lista de produtos utilizada no inventario rotativo
        predios = []  # Lista dos predios. Controla a quantidade de predios por ordem de servico
        inventario_itens = []  # Lista dos itens de inventario
        primeira_vez = True  # Controle da rotina de primeira vez
        predio = 1  # Controle dos predios para ´quebra´ da ordem de servico
        tipo = 'R' if context.get('active_ids') else 'T'

        if tipo == 'R':
            ordem = 'tipo_nivel, deposito, rua, predio, nivel, apartamento'
        else:
            ordem = 'deposito, rua, lado, nivel, predio, apartamento'

        # Cria o inventario
        inventario_id = self._wizinv_criar_inventario(cr, uid, tipo, descricao, context)

        # Todos os enderecos para inventario. Cria as ordens de servico e os itens de inventario
        _sql = """SELECT   id, deposito, rua, lado, predio, nivel, tipo_nivel, grade_id, produto_id,
                           quantidade + reserva_credito - reserva_debito AS saldo, name
                  FROM     wms_endereco
                  WHERE    id in %s
                  ORDER BY %s"""
        cr.execute(_sql % (str(tuple([0] + endereco_ids)), ordem))

        for dados_wms_endereco in cr.dictfetchall():
            endereco_id = dados_wms_endereco.get('id')
            endereco_deposito = dados_wms_endereco.get('deposito')
            endereco_rua = dados_wms_endereco.get('rua')
            endereco_lado = dados_wms_endereco.get('lado')
            endereco_predio = dados_wms_endereco.get('predio')
            endereco_nivel = dados_wms_endereco.get('nivel')
            endereco_tipo_nivel = dados_wms_endereco.get('tipo_nivel')
            endereco_produto_id = dados_wms_endereco.get('produto_id')
            endereco_grade_id = dados_wms_endereco.get('grade_id')
            endereco_saldo = dados_wms_endereco.get('saldo')

            # Controle dos predios para ´quebra´ da ordem de servico (parametro estoque/wms)
            predios.append(endereco_predio) if endereco_predio not in predios else None

            # Produtos utilizados no inventario rotativo
            if endereco_produto_id and endereco_produto_id not in produto_ids:
                produto_ids.append(endereco_produto_id)

            if primeira_vez:  # Rotina de primeira vez!  (controle de ´quebras´)
                rua = endereco_rua
                lado = endereco_lado  # 1 = Impar e 2 = Par
                predio = endereco_predio
                nivel = endereco_nivel
                tipo_nivel = 'PU' if endereco_tipo_nivel != 'PK' else endereco_tipo_nivel
                primeira_vez = False

            # Marca a "quebra" das ordens de servico
            if endereco_rua != rua and tipo != 'R':  # "Quebra" por rua
                if inventario_itens:
                    os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
                    self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
                    inventario_itens = []
                    result.append(os_id)

                rua = endereco_rua
                predios = [endereco_predio]
                predio = endereco_predio
                lado = endereco_lado
                nivel = endereco_nivel

            if endereco_lado != lado and tipo != 'R':  # "Quebra" por lado
                if inventario_itens:
                    os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
                    self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
                    inventario_itens = []
                    result.append(os_id)

                rua = endereco_rua
                predios = [endereco_predio]
                predio = endereco_predio
                lado = endereco_lado
                nivel = endereco_nivel

            if endereco_nivel != nivel and tipo != 'R':  # "Quebra" por nivel
                if inventario_itens:
                    os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
                    self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
                    inventario_itens = []
                    result.append(os_id)

                rua = endereco_rua
                predios = [endereco_predio]
                predio = endereco_predio
                lado = endereco_lado
                nivel = endereco_nivel

            if predio != endereco_predio and tipo != 'R':  # "Quebra" por qtde. de predios
                if len(predios) > adm_utils_dados.get('estpar_qtde_predios'):
                    if inventario_itens:
                        os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
                        self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
                        inventario_itens = []
                        result.append(os_id)

                    predios = [endereco_predio]

                predio = endereco_predio
                rua = endereco_rua
                lado = endereco_lado
                nivel = endereco_nivel

            if endereco_tipo_nivel != tipo_nivel and tipo == 'R':  # Somente rotativo
                if self._wizinv_verifica_quebra_tipo_nivel(
                        cr, uid, ids, endereco_tipo_nivel, endereco_deposito, context):
                    if inventario_itens:
                        os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
                        self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
                        inventario_itens = []
                        result.append(os_id)

                tipo_nivel = endereco_tipo_nivel

            #  Item da 1a. contagem assumido como contado
            if importar_saldo:
                inventario_itens.append(
                    dict(endereco_id=endereco_id, wmsiit_produto_id=endereco_produto_id,
                         wmsiit_grade_id=endereco_grade_id, contagem=1, state='O',
                         verifica_validade=verifica_validade, wmsiit_quantidade=endereco_saldo,
                         active=False)
                )
            else:
                if endereco_tipo_nivel != 'PK':
                    endereco_produto_id = False
                    endereco_grade_id = False

            # Item em contagem
            inventario_itens.append(
                dict(endereco_id=endereco_id, wmsiit_produto_id=endereco_produto_id,
                     wmsiit_grade_id=endereco_grade_id, contagem=contagem,
                     verifica_validade=verifica_validade, state='C', active=True)
            )

        # Ordem de servico com os ultimos itens de inventario ou se nao houve "quebra"
        if inventario_itens:
            os_id = self._wizinv_criar_nova_os(cr, uid, inventario_id, contagem)
            self._wizinv_criar_itens_inventario(cr, uid, os_id, inventario_itens)
            result.append(os_id)

        # Bloqueia os enderecos
        cr.execute("""UPDATE  wms_endereco
                      SET     bloqueado = TRUE
                      WHERE   id IN %s""" % str(tuple([0] + endereco_ids)))

        # Cria a relacao entre inventario e produtos utilizados
        for produto_id in produto_ids:
            agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
            wms_inventario_produto_dados = dict(wmsipr_produto_id=produto_id,
                                                wmsipr_inventario_id=inventario_id,
                                                write_uid=uid, write_date=agora, create_uid=uid,
                                                create_date=agora)
            _keys = ', '.join(wms_inventario_produto_dados.keys())
            _values = ', '.join(['%s'] * len(wms_inventario_produto_dados.values()))
            _params = wms_inventario_produto_dados.values()
            _sql = """INSERT INTO wms_inventario_produto (%s) VALUES (%s)"""
            cr.execute(_sql % (_keys, _values), _params)

        return result

    @staticmethod
    def _wizinv_considerar_virtual(inventario):
        """
        Retorna a lista dos argumentos de validacao dos enderecos de inventario

        :param inventario: dicionario com os dados do planejamento de inventario
        :type inventario: dict
        :return: argumentos de pesquisa para validacao dos enderecos de inventario
        :rtype: list
        """

        result = []

        if not inventario.get('wizinv_virtual'):
            result.append(('deposito', '!=', 99))  # Nao considera enderecos virtuais

        if not inventario.get('wizinv_avaria'):
            result.append(('deposito', '!=', 98))  # Nao considera enderecos de avaria

        return result

    def _wizinv_verifica_quebra_tipo_nivel(self, cr, uid, ids, tipo_nivel, deposito, context=None):
        """
        Verifica se tipo de nivel e virtual ou avaria

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: ID do formulario de planejamento de inventario
        :param context: um dicionario padrao para valores contextuais
        :param tipo_nivel: tipo do nivel do endereco de inventario (PK, PU ou VT)
        :param deposito: numero do deposito (formacao do nome do endereco)
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: integer
        :type tipo_nivel: str
        :type deposito: int
        :type context: dict
        :return: verdadeiro se o tipo do nivel for virtual e deposito for 99 ou se tipo do nivel
        for avaria e o deposito for 98
        """

        context = context and context.copy() or {}
        result = False

        # Dados do inventario
        dados_inventario = self.read(cr, uid, ids, [], context=context)

        if dados_inventario and isinstance(dados_inventario, list):
            dados_inventario = dados_inventario[0]

        if dados_inventario and isinstance(dados_inventario, dict):
            considera_virtual = dados_inventario.get('wizinv_virtual')
            considera_avaria = dados_inventario.get('wizinv_avaria')

            if tipo_nivel == 'VT':
                if considera_virtual and deposito == 99:
                    result = True
                elif considera_avaria and deposito == 98:
                    result = True
            else:
                result = True

        return result

    def _wizinv_criar_inventario(self, cr, uid, tipo, descricao, context=None):
        """
        Cria o inventario.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param tipo: tipo do inventario: Total ou Rotativo
        :param descricao: descricao do inventario.
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type tipo: str
        :type descricao: str
        :type context: dict
        :return: o ID do inventario que foi criado.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = {} if not context else context

        # noinspection PyProtectedMember
        cr.execute("SELECT nextval('" + self.pool.get('wms.inventario')._sequence + "')")
        inventario_id = cr.fetchone()[0]
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        inventario_dados = dict(id=inventario_id, wmsinv_tipo=tipo, wmsinv_descricao=descricao,
                                create_uid=uid, state='E', create_date=agora, write_uid=uid,
                                write_date=agora, wmsinv_data_criacao=agora, active=True)
        _keys = ', '.join(inventario_dados.keys())
        _values = ', '.join(['%s'] * len(inventario_dados.values()))
        _params = inventario_dados.values()
        cr.execute("""INSERT INTO wms_inventario (%s) VALUES (%s)""" % (_keys, _values), _params)

        return inventario_id

    def _wizinv_criar_nova_os(self, cr, uid, inventario_id, contagem, descricao='', context=None):
        """
        Cria nova ordem de servico de inventario.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param inventario_id: ID do inventario
        :param contagem: contagem inicial do inventario
        :param descricao: descricao da ordem de servico de inventario
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type inventario_id: long
        :type contagem: int
        :type descricao: str
        :return: ID da ordem de servico.
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        tz = context.get('tz') or 'Brazil/East'

        tipo = 'rotativo' if context.get('active_ids') else 'total'
        descricao = descricao or _('Inventario %s nr %03d.') % (tipo, inventario_id)

        # Cria a cordem de servico de inventario
        cr.execute("""SELECT id FROM os_tipo WHERE codigo = 'INV'""")
        tipo_os_id = cr.fetchone()[0]  # INV = Inventario

        # noinspection PyProtectedMember
        cr.execute("SELECT nextval('" + self.pool.get('inventario.os')._sequence + "')")
        inventario_os_id = cr.fetchone()[0]
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        os_numero = '%s-%06d' % (datetime.now().year, inventario_os_id)
        inventario_os_dados = dict(id=inventario_os_id, descricao=descricao, numero=os_numero,
                                   data_criacao=agora, inventario_id=inventario_id,
                                   invosv_contagem=contagem, tipo_id=tipo_os_id, state='criada',
                                   active=True, write_uid=uid, create_uid=uid, write_date=agora,
                                   create_date=agora)
        _keys = ', '.join(inventario_os_dados.keys())
        _values = ', '.join(['%s'] * len(inventario_os_dados.values()))
        _params = inventario_os_dados.values()
        cr.execute("""INSERT INTO inventario_os (%s) VALUES (%s)""" % (_keys, _values), _params)

        # Registro do historico
        acao = _('Criar ordem de servico.')
        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
        # Fix: ´inventario_id´ deve ser trocado para ´inventario_os_id´ na classe inventario_os_historico para evitar confusoes
        inventario_os_historico_dados = dict(inventario_id=inventario_os_id, usuario_id=uid,
                                             data_historico=agora, acao=acao, active=True,
                                             create_date=agora, write_date=agora, create_uid=uid,
                                             write_uid=uid)
        _sql = """INSERT INTO inventario_os_historico (%s) VALUES (%s)"""
        _keys = ', '.join(inventario_os_historico_dados.keys())
        _values = ', '.join(['%s'] * len(inventario_os_historico_dados.values()))
        _params = inventario_os_historico_dados.values()
        cr.execute(_sql % (_keys, _values), _params)

        return inventario_os_id

    @staticmethod
    def _wizinv_criar_itens_inventario(cr, uid, os, itens=None, context=None):
        """
        Cria os itens da ordem de servico de inventario.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param os: ID da ordem de servico de inventario
        :param itens: lista com os dados do item de inventario
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type os: long
        :type itens: list[dict]
        :return: verdadeiro se os itens foram criados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = {} if not context else context

        # Cria os itens da ordem de servico de inventario
        for item in itens:
            _keys = ', '.join(item.keys()) + ', os_id'  # Acrescenta o ID da ordem de servico
            _values = ', '.join(['%s'] * len(item.values())) + ', %s'
            _params = item.values() + [os]
            _sql = """INSERT INTO wms_inventario_itens (%s) VALUES (%s)"""
            cr.execute(_sql % (_keys, _values), _params)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
