# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class wms_endereco(osv.Model):
    """
    Endrecos de armazenamento.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.endereco'  # wmsend

    from funcoes import wmsend_fields_function
    from funcoes import wmsend_fields_selection_tipo_nivel  # Tipos de nivel
    from funcoes import wmsend_fields_selection_tipo_lado  # Lado (par/impar)
    from funcoes import wmsend_fields_selection_status  # Status do endereco
    from funcoes import wmsend_fields_selection_sentido_rua  # Sentido da rua

    _columns = {
        # ___________________________ Many2One _______________________________
        'estrutura_tipo_id': fields.many2one('wms.estruturatipo', 'Tipo de estrutura',
                                             ondelete='restrict'),
        'endereco_tipo_id': fields.many2one('wms.enderecotipo', 'Tipo de endereco',
                                            ondelete='restrict'),
        'produto_id': fields.many2one('estoque.produto', 'Produto', ondelete='restrict', select=True),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', ondelete='restrict', select=True),

        # ___________________________ Integer ________________________________
        'id': fields.integer('Nr', readonly=True),
        'deposito': fields.integer('Deposito', required=True),
        'rua': fields.integer('Rua', required=True),
        'predio': fields.integer('Predio', required=True),
        'nivel': fields.integer('Nivel', required=True),
        'apartamento': fields.integer('Apartamento', required=True),
        'quantidade': fields.integer('Estoque atual', write=['wms.group_wms_manager',
                                                             'wms.group_wms_supervisor'],
                                     help='Quantidade real no endereco.'),
        'reserva_credito': fields.integer('Entrada', help='Valor para acrescentar na quantidade.'),
        'reserva_debito': fields.integer('Saida', help='Valor para diminuir na quantidade.'),

        # ___________________________ Date ___________________________________
        'validade': fields.date('Validade'),

        # ___________________________ Function _______________________________
        'name': fields.function(wmsend_fields_function, store=True, method=True, type='char',
                                string='Endereco'),
        'utilizado': fields.function(wmsend_fields_function, type='integer', method=True,
                                     store=True, string='Utilizado',
                                     help='Percentual de utilizacao do endereco.'),
        'capacidade_percent': fields.function(wmsend_fields_function, type='integer', method=True,
                                              string='utilizacao do endereco',
                                              help='Percentual de utilizacao do endereco.'),
        'capacidade': fields.function(wmsend_fields_function, type='integer', method=True,
                                      store=True, string='Capacidade do picking'),
        'quantidade_descricao': fields.function(wmsend_fields_function, type='char', size=50,
                                                method=True, string='Estoque atual'),
        'capacidade_descricao': fields.function(wmsend_fields_function, type='char', size=50,
                                                method=True, string='Capacidade do picking'),
        'saldo': fields.function(wmsend_fields_function, type='integer', method=True,
                                 string='Estoque final'),
        'reposicao': fields.function(wmsend_fields_function, type='char', size=50, method=True,
                                     string='Ponto de reposicao'),
        'repor': fields.function(wmsend_fields_function, type='boolean', method=True,
                                 string='Necessidade de reposicao'),
        'fracionado': fields.function(wmsend_fields_function, type='boolean', method=True,
                                      string='Fracionado'),
        'wmsend_codigo': fields.function(wmsend_fields_function, type='char', method=True,
                                         string='Codigo'),
        'wmsend_produto': fields.function(wmsend_fields_function, type='char', size=60, method=True,
                                          string='Produto'),
        'wmsend_grade_sigla': fields.function(wmsend_fields_function, type='char', size=2,
                                              method=True, string='Grade'),

        # ___________________________ Selection ______________________________
        # Lado da rua (2=Par, 1=Impar)
        'lado': fields.selection(wmsend_fields_selection_tipo_lado, 'Lado', size=1),
        # Tipo do nivel (Picking, pulmao, virtual, etc)
        'tipo_nivel': fields.selection(wmsend_fields_selection_tipo_nivel, 'Tipo do nivel', size=2),
        # Crescente ou decrescente
        'rua_sentido': fields.selection(wmsend_fields_selection_sentido_rua, 'Sentido da rua',
                                        size=1),
        'state': fields.selection(wmsend_fields_selection_status, 'Status'),

        # ___________________________ Boolean ________________________________
        'bloqueado': fields.boolean('Bloqueado'),
        'picking_dinamico': fields.boolean('Dinamico')
    }

    _order = 'deposito, rua, predio, nivel, apartamento'

    _defaults = {
        'deposito': lambda *args: 1,
        'rua': lambda *args: 1,
        'rua_sentido': lambda *args: 'C',  # Padrao e crescente
        'predio': lambda *args: 1,
        'nivel': lambda *args: 0,  # picking por padrao
        'tipo_nivel': lambda *args: 'PK',
        'apartamento': lambda *args: 1,
        'state': lambda *args: 'L',
        'bloqueado': lambda *args: False,  # Para uso do sistema
        'reserva_credito': lambda *args: 0,
        'reserva_debito': lambda *args: 0,
        'quantidade': lambda *args: 0
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
