# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class wms_enderecotipo(osv.Model):
    """
    Tipo de endereco picking/pulmao.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.enderecotipo'  # wmseti

    from funcoes import wmseti_fields_function, wmseti_nova_sequencia
    from funcoes import wmseti_verifica_volume

    _rec_name = 'descricao'
    _columns = {
        # ___________________________ Char ___________________________________
        # Descricao/nome do tipo de endereco.
        'descricao': fields.char('Descricao', size=45),

        # ___________________________ Integer ________________________________
        'sequence': fields.integer('Prioridade', required=True),  # Prioridade da ordem de contagem

        # ___________________________ Float __________________________________
        # Largura (em metros) utilizada no calculo do volume
        'largura': fields.float('Largura (m)', digits=(5, 2)),
        # Altura (em metros) utilizada no calculo do volume
        'altura': fields.float('Altura (m)', digits=(5, 2)),
        # Profundidade do endereco (em metros) utilizada no calculo do volume
        'profundidade': fields.float('Profundidade (m)', digits=(5, 2)),
        # Capacidade maxima (em Kg) do tipo de endereco
        'capacidade': fields.float('Capacidade (Kg)', digits=(12, 3)),

        # ___________________________ Boolean ________________________________
        # Marcacao (logica) de registro ativo ou inativo. O padrao e 1 (ativo)
        'active': fields.boolean('Ativo'),

        # ___________________________ Function _______________________________
        'cubagem': fields.function(wmseti_fields_function, method=True, digits=(15, 6),
                                   string='Volume (m3)'),
        'name': fields.function(wmseti_fields_function, method=True, type='char', size=50,
                                string='Tipo de endereco')
    }

    # Valores padrao
    _defaults = {
        'active': lambda *args: 1,
        'sequence': wmseti_nova_sequencia
    }

    _order = 'sequence'

    _constraints = [(wmseti_verifica_volume, 'O volume deve ser maior que zero.',
                     ['Altura', 'Largura', 'Profundidade'])]

    _sql_constraints = [
        ('descricao', 'unique(descricao)', 'Tipo de endereco ja cadastrado.')]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
