# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""


# _______________________ wms.enderecotipo ___________________________________

def wmseti_verifica_volume(self, cr, uid, ids, context=None):
    """
    Verifica se os valores informados para altura, largura e profundidade estao corretos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param context: um dicionario padrao para valores contextuais,
    :return: {boolean} verdadeiro se o volume for maior que zero.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = False

    # Dados do tipo de endereco
    campos = ['cubagem']
    dados_endereco_tipo = self.read(cr, uid, ids, campos, context)

    if dados_endereco_tipo and isinstance(dados_endereco_tipo, list):
        dados_endereco_tipo = dados_endereco_tipo[0]

    if dados_endereco_tipo and isinstance(dados_endereco_tipo, dict):
        volume = dados_endereco_tipo.get('cubagem', 0.0)

        result = True if volume > 0 else False

    return result


def wmseti_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores das chaves representando registros divergentes e
    nao divergentes.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for brw_wms_enderecotipo in self.browse(cr, uid, ids, context):
        try:
            altura = brw_wms_enderecotipo.altura
            largura = brw_wms_enderecotipo.largura
            profundidade = brw_wms_enderecotipo.profundidade

            if name == 'cubagem':
                value = altura * largura * profundidade
            elif name == 'name':
                value = '%s - %s' % (brw_wms_enderecotipo.id, brw_wms_enderecotipo.descricao)
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_wms_enderecotipo.id] = value

    return result


def wmseti_nova_sequencia(self, cr, uid, context=None):
    """
    Define a nova sequencia dos tipos de produto.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :return: nova sequencia

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}
    sequencia_id = self.search(
        cr, uid, [], context=context, limit=1, order='sequence desc')  # Busca pela maior sequencia

    sequencia = 0
    if sequencia_id:
        dados_sequencia = self.read(cr, uid, sequencia_id, ['sequence'], context=context)

        if dados_sequencia and isinstance(dados_sequencia, list):
            dados_sequencia = dados_sequencia[0]

        if dados_sequencia and isinstance(dados_sequencia, dict):
            sequencia = dados_sequencia.get('sequence') or 0

    sequencia += 1

    return sequencia


# _______________________ wms.endereco _______________________________________
def wmsend_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores ajustados dos campos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    context['retorno_sem_codigo'] = True   # Nao retorna o codigo antes da descricao

    # ___________________________ Objects ________________________________
    obj_adm_utils = self.pool.get('adm.utils')

    result = {}

    # Contexto para mostrar produto baseado no parametro de descricao da grade
    campos = ['estpar_descricao_grade']
    adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)

    for brw_wms_endereco in self.browse(cr, uid, ids, context=context):
        value = None

        try:
            if name == 'name':
                value = '%03d.%03d.%03d.%03d.%03d' % (
                    brw_wms_endereco.deposito, brw_wms_endereco.rua, brw_wms_endereco.predio,
                    brw_wms_endereco.nivel, brw_wms_endereco.apartamento)
            elif name == 'capacidade':
                value = brw_wms_endereco.grade_id.capacidade_picking
            elif name == 'capacidade_percent' or name == 'utilizado':
                quantidade = float(brw_wms_endereco.quantidade or 0)
                capacidade = float(brw_wms_endereco.grade_id.capacidade_picking or 1)
                master = brw_wms_endereco.grade_id.embalagem_master_id
                if master:
                    qtde_master = master.embalagem_id.quantidade_na_embalagem
                    value = quantidade / (capacidade * qtde_master) * 100
                else:
                    value = 0
            elif name == 'capacidade_descricao':
                capacidade = brw_wms_endereco.grade_id.capacidade_picking
                master = brw_wms_endereco.grade_id.embalagem_master_id
                sigla = master.embalagem_id.sigla
                qtde_na_embalagem = master.embalagem_id.quantidade_na_embalagem
                capacidade_total = capacidade * qtde_na_embalagem
                value = '%s %s (%s)' % (capacidade, sigla, capacidade_total)
            elif name == 'reposicao':
                master = brw_wms_endereco.grade_id.embalagem_master_id
                sigla = master.embalagem_id.sigla
                qtde_na_embalagem = master.embalagem_id.quantidade_na_embalagem
                reposicao = brw_wms_endereco.grade_id.ponto_reposicao
                value = '%s %s (%s)' % (reposicao, sigla, reposicao * qtde_na_embalagem)
            elif name == 'repor':
                if brw_wms_endereco.tipo_nivel == 'PK':
                    if brw_wms_endereco.grade_id.embalagem_master_id:
                        master = brw_wms_endereco.grade_id.embalagem_master_id
                        qtde_na_embalagem = master.embalagem_id.quantidade_na_embalagem
                        capacidade = brw_wms_endereco.grade_id.capacidade_picking * qtde_na_embalagem
                        value = brw_wms_endereco.quantidade < capacidade
                else:
                    value = False
            elif name == 'fracionado':
                value = brw_wms_endereco.grade_id.fracionado
            elif name == 'saldo':
                value = brw_wms_endereco.quantidade + brw_wms_endereco.reserva_credito
                value -= brw_wms_endereco.reserva_debito
            elif name == 'wmsend_grade_sigla':  # Sigla da grade
                value = brw_wms_endereco.grade_id.grade
            elif name == 'wmsend_produto':
                codigo = brw_wms_endereco.produto_id.codigo or ''
                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    produto = brw_wms_endereco.grade_id.descricao or ''
                else:  # Produto + grade
                    produto = brw_wms_endereco.produto_id.name or ''
                value = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)
            elif name == 'wmsend_codigo':
                value = brw_wms_endereco.produto_id.codigo
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_wms_endereco.id] = value

    return result


def wmsend_fields_selection_tipo_nivel(self, cr, uid, context=None):
    """
    Tipos de nivel de enderecos de armazenamento

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores de selecao do tipo de nivel.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    # Tipos de Nivel
    result = [('PK', 'Picking'), ('PU', 'Pulmao'), ('VT', 'Virtual')]

    return result


def wmsend_fields_selection_tipo_lado(self, cr, uid, context=None):
    """
    Os lados (do predio) que os enderecos de armazenamento podem ter

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores de selecao do tipo de lado.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    # Lado (par/impar)
    result = [('1', 'Impar'), ('2', 'Par')]

    return result


def wmsend_fields_selection_status(self, cr, uid, context=None):
    """
    O ´´status´´ do endereco.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores de selecao do status.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    # Status do Endereco
    result = [('L', 'Livre'), ('O', 'Ocupado'), ('R', 'Reservado')]

    return result


def wmsend_fields_selection_sentido_rua(self, cr, uid, context=None):
    """
    Os sentidos das ruas.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores de selecao do tipo de lado.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    # Sentido da Rua
    result = [('C', 'Crescente'), ('D', 'Decrescente')]

    return result


# _______________________ wms.estruturatipo __________________________________
def wmsest_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores das chaves representando registros divergentes e nao divergentes.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for item in self.browse(cr, uid, ids, context):
        try:
            if name == 'name':
                value = '%s - %s' % (item.id, item.descricao)
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[item.id] = value

    return result


# _______________________ wms.produtotipo __________________________________
def wmspti_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores das chaves representando registros divergentes e nao divergentes.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if ids and isinstance(ids, (int, long)):
        ids = [ids]

    result = {}

    for item in self.browse(cr, uid, ids, context):
        try:
            if name == 'name':
                value = '%s - %s' % (item.id, item.descricao)
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[item.id] = value

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
