# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _

# Dados de validacao da carga
EXCEDE_CAPACIDADE = 1
EXCEDE_CUBAGEM = 2
TUDO_OK = 0

# Identificacao das chaves
QTDE = 0
PESO = 1
VOLUME = 2
PRODUTO = 0
GRADE = 1
TIPO = 2


# noinspection PyPep8Naming
class wms_montagemcarga(osv.Model):
    """
    Montagem de Carga.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.montagemcarga'  # wmsmcg
    _inherit = 'wms.montagemcarga'  # wms/montagemcarga/model/montagemcarga_data

    def wmsmcg_button_finalizar_montagemcarga(self, cr, uid, ids, context=None):
        """
        Finaliza a montagem de carga.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a montagem de carga foi finalizada ou mensagem de erro.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_separacao_os = self.pool.get('separacao.os')
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_wms_separacao_itens = self.pool.get('wms.separacao.itens')

        args = [('montagemcarga_id', 'in', ids if isinstance(ids, list) else [ids]),
                ('state', '=', 'fechada')]
        separacao_os_ids = obj_separacao_os.search(cr, uid, args, context=context)
        args = [('os_id', 'in', separacao_os_ids)]
        separacao_item_ids = obj_wms_separacao_itens.search(cr, uid, args, context=context)
        brw_wms_separacao_itens = obj_wms_separacao_itens.browse(
            cr, uid, separacao_item_ids, context=context)

        # Atualiza o saldo dos enderecos
        for brw_wms_separacao_item in brw_wms_separacao_itens:
            embalagem = brw_wms_separacao_item.produto_embalagem_id.embalagem_id
            qtde_na_embalagem = embalagem.quantidade_na_embalagem
            endereco_id = brw_wms_separacao_item.endereco_id.id
            qtde_conferida = brw_wms_separacao_item.qtde_conferida * qtde_na_embalagem
            qtde_cortada = brw_wms_separacao_item.qtde_cortada * qtde_na_embalagem

            obj_wms_endereco.wmsend_subtrair_saldo(
                cr, uid, endereco_id, qtde_conferida, context)

            # Ajuste de corte
            obj_wms_endereco.wmsend_reserva_debito(
                cr, uid, endereco_id, qtde_cortada * -1, context)

        # Finaliza a montagem de carga
        result = self.write(cr, uid, ids, {'state': 'finalizada'}, context)

        return result

    def wmsmcg_button_criar_montagemcarga(self, cr, uid, ids, context=None):
        """
        Cria a carga. Itens consolidados.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a montagem de carga foi criada ou mensagem de erro.
        :rtype: bool
        :raise: mensagem de erro da montagem de carga.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        for carga_id in ids if isinstance(ids, list) else [ids]:
            campos = ['pedido_item_ids', 'veiculo_id']
            wms_montagemcarga_dados = self.read(
                cr, uid, carga_id, campos, context, load='_classic_write')

            # Nao cria a carga se algum item estiver com dado logistico faltando
            grade_ids = list(set(map(
                lambda g: g.get('grade_id'), self.pool.get('estoquepedidos.pedido.item').read(
                    cr, uid, wms_montagemcarga_dados.get('pedido_item_ids'),
                    ['grade_id'], context=context, load='_classic_write'))))

            if filter(lambda i: i.get('falta_dado_logistico'),
                      self.pool.get('estoque.produto.grade').read(
                              cr, uid, grade_ids, ['falta_dado_logistico'], context=context)):
                msg = _('A montagem de carga nao pode ser criada. Existem itens com dado '
                        'logistico faltando.')

                raise osv.except_osv(_('Atencao!'), msg)

            # Nao cria a montagem de carga se hover item com picking dinamico sem endereco definido
            brw_estoquepedidos_pedido_item = self.pool.get('wms.montagemcarga').browse(
                cr, uid, carga_id, context).pedido_item_ids
            brw_estoquepedidos_pedido_item = filter(lambda f: not f.grade_id.endereco_produto_id.id,
                                                    brw_estoquepedidos_pedido_item)
            if brw_estoquepedidos_pedido_item:
                grade = brw_estoquepedidos_pedido_item[0].grade_id.grade
                produto = brw_estoquepedidos_pedido_item[0].produto_id.name
                codigo = brw_estoquepedidos_pedido_item[0].produto_id.codigo
                produto = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)
                msg = _('A montagem de carga nao pode ser criada. Endereco dinamico indefinido '
                        'para o produto %s (%s).') % (produto, grade)

                raise osv.except_osv(_('Atencao!'), msg)

            # Clientes
            self._wmsmcg_consolidar_clientes(
                cr, uid, carga_id, wms_montagemcarga_dados.get('pedido_item_ids'), context)

            # Carga
            self._wmsmcg_consolidar_carga(
                cr, uid, carga_id, wms_montagemcarga_dados.get('pedido_item_ids'), context)

            # Marca veiculo "em viagem"
            dados = {'montagemcarga_id': carga_id, 'veivei_veiculo_viagem': True}
            veiculo_id = wms_montagemcarga_dados.get('veiculo_id')

            self.pool.get('veiculos.veiculo').write(cr, uid, veiculo_id, dados, context)

            # Grava o nr. da montagem de carga nos pedidos
            pedido_item_ids = wms_montagemcarga_dados.get('pedido_item_ids')
            if pedido_item_ids:
                if len(pedido_item_ids) > 1:
                    cr.execute("UPDATE estoquepedidos_pedido_item "
                               "SET    montagemcarga_id = %s "
                               "WHERE  id IN %s" % (carga_id, tuple(map(int, pedido_item_ids)),))
                else:
                    cr.execute("UPDATE estoquepedidos_pedido_item "
                               "SET    montagemcarga_id = %s "
                               "WHERE  id = %s" % (carga_id, pedido_item_ids[0]))

        # Altera o ´´status´´ da ordem de servico
        self.write(cr, uid, ids, {'state': 'criada'}, context)

        return True

    def _wmsmcg_consolidar_carga(self, cr, uid, carga_id, pedido_item_ids, context=None):
        """
        Consolida os itens de pedido (carga).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param carga_id: ID da montagem de carga,
        :param pedido_item_ids: lista com o ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type carga_id: int
        :type pedido_item_ids: list
        :return: lista com ID dos clientes.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []
        itens = {}

        estoquepedidos_pedido_itens = self.pool.get('estoquepedidos.pedido.item').browse(
            cr, uid, pedido_item_ids if isinstance(pedido_item_ids, list) else [pedido_item_ids],
            context)

        for brw_estoquepedidos_pedido_item in estoquepedidos_pedido_itens:
            qtde_pedida = brw_estoquepedidos_pedido_item.qtde_pedida
            embalagem = brw_estoquepedidos_pedido_item.produto_embalagem_id
            qtde_na_embalagem = embalagem.embalagem_id.quantidade_na_embalagem
            embalagem_master = brw_estoquepedidos_pedido_item.grade_id.embalagem_master_id
            qtde_na_master = embalagem_master.embalagem_id.quantidade_na_embalagem
            embalagem_expedicao = brw_estoquepedidos_pedido_item.grade_id.embalagem_expedicao_id
            qtde_na_expedicao = embalagem_expedicao.embalagem_id.quantidade_na_embalagem
            qtde_pedida_expedicao = qtde_pedida * qtde_na_embalagem / qtde_na_expedicao
            peso = embalagem_expedicao.peso_bruto * qtde_pedida_expedicao
            volume = embalagem_expedicao.cubagem * qtde_pedida_expedicao
            fracionado = brw_estoquepedidos_pedido_item.grade_id.fracionado
            embalado = brw_estoquepedidos_pedido_item.grade_id.embalado
            perigoso = brw_estoquepedidos_pedido_item.grade_id.perigoso

            # Verifica se o peso e o volume das embalagem sao os mesmos se as quantidades na
            # embalagem tambem forem as mesmas
            if qtde_na_master == qtde_na_expedicao:
                msg = None
                if embalagem_expedicao.peso_bruto != embalagem_master.peso_bruto:
                    msg = _('Existe divergencia de peso entre as embalagens de recebimento '
                            '(master) e expedicao (venda) para o produdo %s (%s).')
                elif embalagem_expedicao.cubagem != embalagem_master.cubagem:
                    msg = _('Existe divergencia de volume entre as embalagens de recebimento '
                            '(master) e expedicao (venda) para o produdo %s (%s).')

                if msg:
                    msg %= ('%s - %s' % (brw_estoquepedidos_pedido_item.produto_id.codigo,
                                         brw_estoquepedidos_pedido_item.produto_id.name),
                            brw_estoquepedidos_pedido_item.grade_id.grade)

                    raise osv.except_osv(_('Atencao!'), msg)

            # Verifica proporcionalidade das embalagens do pedido e expedicao
            if qtde_pedida * qtde_na_embalagem % qtde_na_expedicao:
                msg = _('A montagem de carga nao pode ser criada. A embalagem do produdo '
                        '%s-%s (%s) no pedido nao e compativel com a embalagem de expedicao.')
                msg %= (brw_estoquepedidos_pedido_item.produto_id.codigo,
                        brw_estoquepedidos_pedido_item.produto_id.name,
                        brw_estoquepedidos_pedido_item.grade_id.grade)

                raise osv.except_osv(_('Atencao!'), msg)

            chave = (carga_id, brw_estoquepedidos_pedido_item.produto_id.id,
                     brw_estoquepedidos_pedido_item.grade_id.id,
                     brw_estoquepedidos_pedido_item.grade_id.produto_tipo_id.id)

            if chave in itens:
                itens[chave][0] += qtde_pedida * qtde_na_embalagem
                itens[chave][1] += peso
                itens[chave][2] += volume
            else:
                itens[chave] = [qtde_pedida * qtde_na_embalagem, peso, volume, fracionado, embalado,
                                perigoso]

        # Criar itens de carga
        for item in itens:
            dados = dict(montagemcarga_id=item[0], wmsmci_produto_id=item[1],
                         wmsmci_grade_id=item[2], wmsmci_produto_tipo_id=item[3],
                         wmsmci_quantidade=itens[item][0], wmsmci_peso=itens[item][1],
                         wmsmci_volume=itens[item][2], wmsmci_fracionado=itens[item][3],
                         wmsmci_embalado=itens[item][4], wmsmci_perigoso=itens[item][5])

            self.pool.get('wms.montagemcarga.itens').create(cr, uid, dados, context)

        return result

    def _wmsmcg_consolidar_clientes(self, cr, uid, carga_id, pedido_item_ids, context=None):
        """
        Consolida os clientes.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param carga_id: ID da montagem de carga,
        :param pedido_item_ids: lista com o ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type carga_id: int
        :type pedido_item_ids: list
        :type context: dict
        :return: lista com ID dos clientes.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        campos = ['pedido_id']
        estoquepedidos_pedido_item_dados = self.pool.get('estoquepedidos.pedido.item').read(
            cr, uid, pedido_item_ids, campos, context, load='_classic_write')
        pedido_ids = list(set(map(lambda p: p.get('pedido_id'), estoquepedidos_pedido_item_dados)))

        campos = ['pessoa_id']
        estoquepedidos_pedido_capa_dados = self.pool.get('estoquepedidos.pedido.capa').read(
            cr, uid, pedido_ids, campos, context, load='_classic_write')
        pessoa_ids = list(set(map(lambda p: p.get('pessoa_id'), estoquepedidos_pedido_capa_dados)))

        # Dados do cliente e rota de entrega
        args = [('pessoa_id', 'in', pessoa_ids)]
        cliente_ids = self.pool.get('financeiro.cliente').search(cr, uid, args, context=context)
        brw_financeiro_clientes = self.pool.get('financeiro.cliente').browse(
            cr, uid, cliente_ids, context)

        for brw_financeiro_cliente in brw_financeiro_clientes:
            dados = dict(wmsmcc_pessoa_id=brw_financeiro_cliente.pessoa_id.id,
                         wmsmcc_cpf_cnpj=brw_financeiro_cliente.pessoa_id.cpf_cnpj,
                         wmsmcc_nome=brw_financeiro_cliente.pessoa_id.nome,
                         wmsmcc_endereco=brw_financeiro_cliente.pessoa_id.endereco,
                         wmsmcc_montagemcarga_id=carga_id,
                         wmsmcc_rota_id=brw_financeiro_cliente.fincli_rota.id,
                         wmsmcc_sequencia_entrega=brw_financeiro_cliente.sequence)

            self.pool.get('wms.montagemcarga.clientes').create(cr, uid, dados, context)

            result.append(brw_financeiro_cliente.id)

        return result

    def wmsmcg_button_cancelar_montagemcarga(self, cr, uid, ids, context=None):
        """
        Cancelar a carga.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: ID ou lista com o ID das cargas,
        :param context: um dicionario padrao para valores contextuais
        :type cr: Cursor
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a montagem de carga foi cancelada.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')
        obj_pedido_item = self.pool.get('estoquepedidos.pedido.item')

        args = [('montagemcarga_id', 'in' if isinstance(ids, list) else '=', ids)]

        # Libera veiculo
        veiculo_ids = obj_veiculos_veiculo.search(cr, uid, args, context=context)
        obj_veiculos_veiculo.write(cr, uid, veiculo_ids, {
            'montagemcarga_id': False
        }, context)

        # Remove o nr. da montagem de carga nos pedidos (libera os pedidos)
        pedido_item_ids = obj_pedido_item.search(cr, uid, args, context=context)
        _sql = """UPDATE estoquepedidos_pedido_item SET montagemcarga_id = NULL """

        if len(pedido_item_ids) == 1:
            _sql += """WHERE  id = %s""" % pedido_item_ids[0]
        else:
            _sql += """WHERE  id IN %s""" % str(tuple(pedido_item_ids))

        cr.execute(_sql)

        # Cancela a montagem de carga.
        _sql = """UPDATE wms_montagemcarga SET state = 'cancelada' """

        if len(ids)  == 1:
            _sql += """WHERE id = %s""" % ids[0]
        else:
            _sql += """WHERE id IN %s""" % str(tuple(ids))

        cr.execute(_sql)

        return True

    def wmsmcg_button_atualizar_montagemcarga(self, cr, uid, ids, context=None):
        """
        Atualizar dados da montagem de carga.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se os dados da montagem de carga foram atualizados.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids if isinstance(ids, list) else [ids], {}, context)

        return result

    def wmsmcg_button_esboco_montagemcarga(self, cr, uid, ids, context=None):
        """
        Retorna a montagem de carga a condicao de esboco.

        :param cr:  a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se o ´´status´´ da montagem de carga foi alterado.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_montagemcarga_itens = self.pool.get('wms.montagemcarga.itens')
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')
        obj_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_montagemcarga_clientes = self.pool.get('wms.montagemcarga.clientes')

        # Remove os clientes
        cliente_ids = obj_montagemcarga_clientes.search(cr, uid, [], context=context)
        obj_montagemcarga_clientes.unlink(cr, uid, cliente_ids, context)

        args = [('montagemcarga_id', 'in' if isinstance(ids, list) else '=', ids)]
        item_ids = obj_wms_montagemcarga_itens.search(cr, uid, args, context=context)

        obj_wms_montagemcarga_itens.unlink(cr, uid, item_ids, context)

        args = [('montagemcarga_id', 'in' if isinstance(ids, list) else '=', ids)]

        # Desmarca veiculo "em viagem"
        veiculo_ids = obj_veiculos_veiculo.search(cr, uid, args, context=context)
        obj_veiculos_veiculo.write(cr, uid, veiculo_ids, {
            'montagemcarga_id': False
        }, context)

        # Remove o nr. da montagem de carga nos pedidos
        pedido_item_ids = obj_pedido_item.search(cr, uid, args, context=context)
        if pedido_item_ids:
            if len(pedido_item_ids) > 1:
                cr.execute("UPDATE estoquepedidos_pedido_item "
                           "SET    montagemcarga_id = NULL "
                           "WHERE  id IN %s" % (tuple(map(int, pedido_item_ids)),))
            else:
                cr.execute("UPDATE estoquepedidos_pedido_item "
                           "SET    montagemcarga_id = NULL "
                           "WHERE  id = %s" % pedido_item_ids[0])

        result = self.write(cr, uid, ids, {'state': 'esboco'}, context)

        return result

    def wmsmcg_onchange_veiculo(self, cr, uid, ids, veiculo_id, item_ids, context=None):
        """
        Critica se carga nao couber no veiculo sem travar o processo de montagem de carga.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param veiculo_id: ID do veiculo,
        :param item_ids: lista com ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type veiculo_id: int
        :type item_ids: list
        :type context: dict
        :return: {dict} dicionario com os dados do veiculo.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_veiculo = self.pool.get('veiculos.veiculo')

        # Dados do veiculo
        motorista_id = marca_id = modelo_id = qtde_palete = False
        carga_maxima_peso = cubagem = 0.00
        ano_fabricacao = ano_modelo = combustivel = carroceria_tipo = msg = None
        veivei_tipo_veiculo = None

        item_ids = item_ids[0][2]  # Lista com o ID dos pedidos
        result = {}

        # Dados do veiculo
        campos = ['placa', 'motorista_id', 'qtde_palete', 'carga_maxima_peso', 'cubagem',
                  'marca_id', 'modelo_id', 'ano_fabricacao', 'ano_modelo', 'combustivel',
                  'carroceria_tipo', 'veivei_tipo_veiculo']

        if veiculo_id:
            dados_veiculo = obj_veiculo.read(
                cr, uid, veiculo_id, campos, context, load='_classic_write')
        else:
            dados_veiculo = []

        if dados_veiculo and isinstance(dados_veiculo, list):
            dados_veiculo = dados_veiculo[0]

        if dados_veiculo and isinstance(dados_veiculo, dict):
            motorista_id = dados_veiculo.get('motorista_id')
            qtde_palete = dados_veiculo.get('qtde_palete')
            carga_maxima_peso = dados_veiculo.get('carga_maxima_peso')
            cubagem = dados_veiculo.get('cubagem')
            marca_id = dados_veiculo.get('marca_id')
            modelo_id = dados_veiculo.get('modelo_id')
            ano_fabricacao = dados_veiculo.get('ano_fabricacao')
            ano_modelo = dados_veiculo.get('ano_modelo')
            combustivel = dados_veiculo.get('combustivel')
            carroceria_tipo = dados_veiculo.get('carroceria_tipo')
            veivei_tipo_veiculo = dados_veiculo.get('veivei_tipo_veiculo')

            # Excedeu capacidade (Kg)
            if not self.wmsmcg_verifica_peso(cr, uid, veiculo_id, item_ids, context):
                msg = _('Total do peso excede capacidade maxima do veiculo')
            # Excedeu cubagem (m3)
            elif not self.wmsmcg_verifica_volume(cr, uid, veiculo_id, item_ids, context):
                msg = _('Total do volume excede cubagem do veiculo.')
            # Excedeu o numero de paletes
            elif not self.wmsmcg_verifica_palete(cr, uid, veiculo_id, item_ids, context):
                msg = _('Numero de paletes ultrapassa o suportado pelo veiculo.')

        valor = dict(motorista_id=motorista_id, wmsmcg_veiculo_palete=qtde_palete,
                     wmsmcg_veiculo_carga=carga_maxima_peso, cubagem=cubagem, marca_id=marca_id,
                     modelo_id=modelo_id, ano_fabricacao=ano_fabricacao, ano_modelo=ano_modelo,
                     combustivel=combustivel, carroceria_tipo=carroceria_tipo,
                     veivei_tipo_veiculo=veivei_tipo_veiculo, mensagem='')
        result['value'] = valor

        if item_ids and msg:
            msg = _('Aviso. ') + msg
            result['value']['mensagem'] = msg

        return result

    def wmsmcg_onchange_item_pedido(self, cr, uid, ids, veiculo_id, item_ids, context=None):
        """
        Critica se carga nao couber no veiculo sem travar o processo de montagem de carga.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param veiculo_id: ID do veiculo,
        :param item_ids: lista com ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: list
        :type veiculo_id: int
        :type item_ids: list
        :type context: dict
        :return: dicionario com os dados dos itens.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_estoque_local_locacao = self.pool.get('estoque.local.locacao')

        item_ids = item_ids[0][2]  # Lista com o ID dos pedidos
        peso_total = cubagem_total = 0.0
        qtde_volumes = 0
        itens = []
        result = {}
        item_embalagem_divergente = []  # Itens com embalagem diferente do local/locacao
        grades_divergentes = []  # Lista com ID das grades dos itens com divergencia
        msg = None

        for item in obj_estoquepedidos_pedido_item.browse(cr, uid, item_ids, context):
            peso_total += item.pedite_peso or 0
            cubagem_total += item.pedite_volume or 0
            embalagem = item.produto_embalagem_id.embalagem_id
            qtde_volumes += item.qtde_pedida / embalagem.quantidade_na_embalagem

            if item.produto_id.id not in itens:
                itens.append(item.produto_id.id)

            # Embalagem do item difere da embalagem do local/locacao
            args = [('local_id', '=', item.local_id.id),
                    ('estllc_produto_embalagem_id', '=', item.produto_embalagem_id.id)]

            if not obj_estoque_local_locacao.search(cr, uid, args, context=context):
                if item.grade_id.id not in grades_divergentes:
                    item_embalagem_divergente.append('%s (%s)' % (item.produto_id.name,
                                                                  item.grade_id.grade))
                    grades_divergentes.append(item.grade_id.id)

        result['value'] = dict(qtde_volumes=qtde_volumes, qtde_itens=len(itens),
                               peso_total=peso_total, cubagem_total=cubagem_total, mensagem='')

        # Excedeu capacidade (Kg)
        if not self.wmsmcg_verifica_peso(cr, uid, veiculo_id, item_ids, context):
            if veiculo_id:
                msg = _('Total do peso excede capacidade maxima do veiculo.')
        # Excedeu cubagem (m3)
        elif not self.wmsmcg_verifica_volume(cr, uid, veiculo_id, item_ids, context):
            if veiculo_id:
                msg = _('Total do volume excede cubagem do veiculo.')
        # Excedeu o numero de paletes
        elif not self.wmsmcg_verifica_palete(cr, uid, veiculo_id, item_ids, context):
            if veiculo_id:
                msg = _('Numero de paletes ultrapassa o suportado pelo veiculo.')
        # Embalagens com divergencia na locacao
        elif item_embalagem_divergente:
            if item_embalagem_divergente.__len__() > 1:
                msg = _('Os produtos %s apresentam embalagem diferente na locacao.')
            else:
                msg = _('O produto %s apresenta embalagem diferente na locacao.')

            msg %= ', '.join(item_embalagem_divergente)

        if itens and msg:
            msg = _('Aviso. ') + msg
            result['value']['mensagem'] = msg

        return result

    def wmsmcg_verifica_peso(self, cr, uid, veiculo_id, item_ids, context=None):
        """
        Verifica se o peso total dos itens selecionados supera o peso maximo suportado pelo veiculo.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param veiculo_id: ID do veiculo,
        :param item_ids: lista com ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type veiculo_id: int
        :type item_ids: list
        :type context: dict
        :return: verdadeiro se o peso dos itens for maior que a carga do veiculo
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')

        # Verifica se o veiculo foi informado
        if not veiculo_id:
            return False

        # Dados do veiculo
        if not veiculo_id:
            msg = _('Um veiculo deve ser selecionado.')
            return {'warning': {'title': _('Aviso.'), 'message': msg}}

        campos = ['carga_maxima_peso']
        carga_maxima_peso = peso_total = 0.0
        dados_veiculo = obj_veiculos_veiculo.read(cr, uid, veiculo_id, campos, context)

        if dados_veiculo and isinstance(dados_veiculo, list):
            dados_veiculo = dados_veiculo[0]

        if dados_veiculo and isinstance(dados_veiculo, dict):
            carga_maxima_peso = dados_veiculo.get('carga_maxima_peso')

        # Dados dos itens
        campos = ['peso']
        dados_pedido = obj_estoquepedidos_pedido_item.read(cr, uid, item_ids, campos, context)

        if dados_pedido and isinstance(dados_pedido, list):
            peso_total = sum(map(lambda i: i.get('peso', 0.0), dados_pedido))

        if dados_pedido and isinstance(dados_pedido, dict):
            peso_total = dados_pedido.get('peso')

        result = carga_maxima_peso > peso_total

        return result

    def wmsmcg_verifica_volume(self, cr, uid, veiculo_id, item_ids, context=None):
        """
        Verifica se o volume total dos itens selecionados supera a cubagem do veiculo.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param veiculo_id: ID do veiculo,
        :param item_ids: lista com ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type veiculo_id: int
        :type item_ids: list
        :type context: dict
        :return: verdadeiro se o volume superar a cubagem do veiculo.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')

        # Verifica se o veiculo foi informado
        if not veiculo_id:
            return False

        # Dados do veiculo
        campos = ['cubagem']
        cubagem = volume_total = 0.0
        dados_veiculo = obj_veiculos_veiculo.read(cr, uid, veiculo_id, campos, context)

        if dados_veiculo and isinstance(dados_veiculo, list):
            dados_veiculo = dados_veiculo[0]

        if dados_veiculo and isinstance(dados_veiculo, dict):
            cubagem = dados_veiculo.get('cubagem')

        # Dados dos itens
        dados_pedido = obj_estoquepedidos_pedido_item.read(cr, uid, item_ids, campos, context)

        if dados_pedido and isinstance(dados_pedido, list):
            volume_total = sum(map(lambda i: i.get('cubagem', 0.0), dados_pedido))

        if dados_pedido and isinstance(dados_pedido, dict):
            volume_total = dados_pedido.get('cubagem')

        result = cubagem > volume_total

        return result

    def wmsmcg_verifica_palete(self, cr, uid, veiculo_id, item_ids, context=None):
        """
        Verifica se o numero de paletes utilizado pelos clientes supera o numero de paletes levado
        pelo veiculo.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param veiculo_id: ID do veiculo,
        :param item_ids: lista com ID dos itens de pedido,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type veiculo_id: int
        :type item_ids: list
        :type context: dict
        :return: verdadeiro se o numero de paletes utilizado pelos clientes superar o numero de
        paletes levado pelo veiculo.
        :rtype: bool
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        from math import ceil

        context = context and context.copy() or {}

        # ____________________________ Objects _______________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')

        # Verifica se o veiculo foi informado
        if not veiculo_id or not item_ids:
            return False

        # Dados do veiculo
        campos = ['carga_maxima_peso', 'cubagem', 'qtde_palete']
        qtde_palete = 2  # Minimo para quantidade de paltes
        carga_maxima_peso = cubagem = 0.0
        dados_veiculo = obj_veiculos_veiculo.read(cr, uid, veiculo_id, campos, context)

        if dados_veiculo and isinstance(dados_veiculo, list):
            dados_veiculo = dados_veiculo[0]

        if dados_veiculo and isinstance(dados_veiculo, dict):
            qtde_palete = dados_veiculo.get('qtde_palete', qtde_palete)
            carga_maxima_peso = dados_veiculo.get('carga_maxima_peso', 0.0)
            cubagem = dados_veiculo.get('cubagem', 0.0)

        peso_palete = carga_maxima_peso / qtde_palete
        volume_palete = cubagem / qtde_palete

        # Dados dos itens
        campos = ['peso', 'cubagem', 'pessoa_id']
        dados_pedido = obj_estoquepedidos_pedido_item.read(cr, uid, item_ids, campos, context)
        peso_total = {}  # Peso total do cliente
        volume_total = {}  # Volume total do cliente

        # Consolida os totais de peso e volume (cubagem) por cliente
        if dados_pedido and isinstance(dados_pedido, list):
            for pedido in dados_pedido:
                cliente_id = pedido.get('pessoa_id')

                if cliente_id:
                    cliente_id = cliente_id[0]

                    if cliente_id not in peso_total:
                        peso_total[cliente_id] = pedido.get('peso', 0.0)
                        volume_total[cliente_id] = pedido.get('cubagem', 0.0)
                    else:
                        peso_total[cliente_id] += pedido.get('peso', 0.0)
                        volume_total[cliente_id] += pedido.get('cubagem', 0.0)

        if dados_pedido and isinstance(dados_pedido, dict):
            cliente_id = dados_pedido.get('pessoa_id')

            if cliente_id:
                cliente_id = cliente_id[0]
                peso_total[cliente_id] = dados_pedido.get('peso', 0.0)
                volume_total[cliente_id] = dados_pedido.get('cubagem', 0.0)

        # Calculo da quantidade de paletes
        paletes_total = 0.0

        if len(peso_total) > qtde_palete:
            result = False
        else:
            for cliente in peso_total:
                paletes_peso = ceil(peso_total[cliente] / peso_palete)
                paletes_volume = ceil(volume_total[cliente] / volume_palete)

                if paletes_peso > paletes_volume:
                    paletes_total += paletes_peso
                else:
                    paletes_total += paletes_volume

            result = paletes_total <= qtde_palete

        return result

    def wmsmcg_resumo_carga(self, cr, uid, ids, context=None):
        """
        Resumo da carga com o total de itens, peso e volume total.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: valores informando os dados (resumidos) da carga.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = []

        for montagem in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            peso_total = 0.0
            cubagem_total = 0.0

            for pedido in montagem.pedido_item_ids:  # Calcula os totais
                peso_total += pedido.peso
                cubagem_total += pedido.cubagem

            dados = {
                'id': montagem.id,
                'peso_total': peso_total,
                'cubagem_total': cubagem_total
            }

            result.append(dados)

        return result

    def wmsmcg_button_remover_itens_pedido(self, cr, uid, ids, context=None):
        """
        Remove todos os itens de pedido.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se itens de pedido foram removidos.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = self.write(cr, uid, ids if isinstance(ids, list) else [ids], {
            'pedido_item_ids': [[6, False, []]],
            'mensagem': None
        }, context)

        return result

    # _________________________ ORM Methods __________________________________
    def read(self, cr, user, ids, fields=None, context=None, load='_classic_read'):
        """
        Verifica se existem valores da carga que superam os valores do veiculo em peso, volume e
        quantidade de paletes.

        :param cr: a linha atual do cursor do banco de dados,
        :param user: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param fields: dicionario com os nome dos campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais
        :param load:
        :type cr: object
        :type user: int
        :type ids: list
        :type fields: list
        :type context: dict
        :type load: str
        :return: valores com mensagem de aviso se a carga esta acima do volume ou do peso suportado
        pelo veiculo.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        results = super(wms_montagemcarga, self).read(cr, user, ids, fields, context, load)

        for result in results if isinstance(results, list) else [results]:
            if result.get('peso_total') > result.get('wmsmcg_veiculo_carga'):
                result['mensagem'] = _('Aviso. O peso da carga esta acima do maximo suportado pelo '
                                       'veiculo.')
            elif result.get('cubagem_total') > result.get('wmsmcg_veiculo_volume'):
                result['mensagem'] = _('Aviso. O volume da carga esta acima do maximo suportado '
                                       'pelo veiculo')

        return results

    def unlink(self, cr, uid, ids, context=None):
        """
        Permitir exclusao de montagem de carga apenas se o status for "esboco".

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a montagem de carga pode ser excluida
        :rtype: bool
        :raise: uma mensagem informando que a montagem de carga nao pode ser excluida.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if filter(lambda f: f.state != 'esboco', self.browse(
                cr, uid, ids if isinstance(ids, list) else [ids], context)):
                msg = _('Montagem de carga nao pode ser excluida, apenas cancelada.')

                raise osv.except_osv(_('Atencao!'), msg)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

