# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields

# ´´Status´´ da montagem de carga
STATUS_MONTAGEM = [('esboco', 'Esboco'), ('criada', 'Criada'), ('separacao', 'Separacao/Contagem'),
                   ('separada', 'Separada'), ('finalizada', 'Finalizada'),
                   ('cancelada', 'Cancelada')]


# noinspection PyPep8Naming
class wms_montagemcarga(osv.Model):
    """
    Montagem de carga.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.montagemcarga'  # wmsmcg

    from funcoes import wmsmcg_fields_function

    _columns = {
        # ___________________________ Many2Many ______________________________
        'pedido_item_ids': fields.many2many('estoquepedidos.pedido.item',  # Itens do Pedido
                                            'montagemcarga_m2m_estoquepedidos',
                                            'os_id', 'item_id',
                                            'Item de pedido'),

        # ____________________________ One2Many ______________________________
        'montagemcarga_item_ids': fields.one2many('wms.montagemcarga.itens',
                                                  'montagemcarga_id',
                                                  'Item da carga'),
        'wmsmcg_montagemcarga_cliente_ids': fields.one2many('wms.montagemcarga.clientes',
                                                            'wmsmcc_montagemcarga_id',
                                                            'Cliente'),
        # ____________________________ Many2One ______________________________
        'veiculo_id': fields.many2one('veiculos.veiculo', 'Veiculo',
                                      required=True, ondelete='restrict'),
        'motorista_id': fields.many2one("veiculos.motorista", 'Motorista', select=True),

        # ___________________________ Char ___________________________________
        'descricao': fields.char('Descricao', size=100, required=True),
        'box': fields.char('Box', size=2, required=True),  # Box (local) de expedicao

        # ___________________________ Datetime _______________________________
        'create_date': fields.datetime('Data de Criacao'),

        # ___________________________ Function _______________________________
        'peso_total': fields.function(wmsmcg_fields_function, digits=(9, 3), method=True,
                                      store=True, string='Peso Total (Kg)'),
        'cubagem_total': fields.function(wmsmcg_fields_function, method=True, digits=(15, 6),
                                         store=True, string='Volume Total (m3)'),
        'qtde_volumes': fields.function(wmsmcg_fields_function, type='integer', method=True,
                                        store=True, string='Total de volumes'),
        'qtde_itens': fields.function(wmsmcg_fields_function, type='integer', method=True,
                                      store=True, string='Total de itens'),
        'name': fields.function(wmsmcg_fields_function, method=True, type='char', size=80,
                                string='Nome'),
        'wmsmcg_veiculo_palete': fields.function(wmsmcg_fields_function, method=True,
                                                 type='integer', size=80,
                                                 string='Quantidade de paletes'),
        'wmsmcg_veiculo_carga': fields.function(wmsmcg_fields_function, method=True,
                                                string='Carga maxima (Kg)'),
        'wmsmcg_veiculo_volume': fields.function(wmsmcg_fields_function, method=True,
                                                 string='Volume maximo (m3)'),

        # ___________________________ Selection ______________________________
        'state': fields.selection(STATUS_MONTAGEM, 'Status'),

        # ___________________________ Text ___________________________________
        'dados_veiculo': fields.text('Dados do veiculo', readonly=True),

        # ___________________________ Char ___________________________________
        'mensagem': fields.char('Aviso', size=255, readonly=True),
    }

    _defaults = {
        'state': lambda *args: 'esboco'
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
