# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _
from openerp.report import report_sxw
from datetime import datetime
from pytz import timezone


class report_dadoslogisticos_faltando(report_sxw.rml_parse):
    """
    Relatorio de produtos sem dados logisticos.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context):
        super(report_dadoslogisticos_faltando, self).__init__(cr, uid, name, context=context)

        # ___________________________ Objects ________________________________
        obj_wms_montagemcarga = self.pool.get('wms.montagemcarga')
        brw_wms_montagemcarga = obj_wms_montagemcarga.browse(cr, uid, context.get('active_id'))

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))

        # Itens com dado logistico faltando
        estoque_produto_ids = []  # Lista de produtos com dado logistico faltando
        pedido_item_ids = []  # Lista dos pedidos com dado logistico faltando

        for brw_pedido_item in brw_wms_montagemcarga.pedido_item_ids:
            if brw_pedido_item.grade_id.falta_dado_logistico:
                if brw_pedido_item.produto_id.id not in estoque_produto_ids:
                    estoque_produto_ids.append(brw_pedido_item.produto_id.id)
                    pedido_item_ids.append(brw_pedido_item.id)

        dados = {
            'datetime': datetime,
            'agora': agora,
            'subtitulo': context.get('subtitulo', ''),
            'pedido_item_ids': pedido_item_ids
        }

        self.localcontext.update(dados)

        self.context = context

        # Verifica se existe item com dado logistico faltando

        if not filter(lambda item: item.grade_id.falta_dado_logistico,
                      brw_wms_montagemcarga.pedido_item_ids):
            msg = _('Nao existem produtos sem dado logistico.')

            raise osv.except_osv(_('Atencao!'), msg)

report_sxw.report_sxw('report.dadoslogisticos_faltando', 'wms.montagemcarga',
                      'addons/wms/montagemcarga/report/dadoslogisticos_faltando.rml',
                      parser=report_dadoslogisticos_faltando, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
