# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2015 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2015, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report import report_sxw
from datetime import datetime
from pytz import timezone


# noinspection PyPep8Naming
class report_pedidos_pendentes(report_sxw.rml_parse):
    """
    Relacao de pedidos pendentes. Pedidos que ainda nao foram expedidos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context=None):
        """
        Atribui valores as funcoes utilizada no relatorio.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param name: nome do modelo utilizado,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: integer
        :type name: char|list
        :return: impressao do relatorio.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        super(report_pedidos_pendentes, self).__init__(cr, uid, name, context)

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))

        # Periodo inicial e final. Menor e maior data de emissao
        args = [('id', 'in', context.get('active_ids'))]
        pedido_item_ids = self.pool.get('estoquepedidos.pedido.item').search(
            cr, uid, args, order='pedite_data_emissao', limit=1, context=context)
        menor_data_dados = self.pool.get('estoquepedidos.pedido.item').read(
            cr, uid, pedido_item_ids, ['pedite_data_emissao'], context)

        if menor_data_dados and isinstance(menor_data_dados, list):
            menor_data_dados = menor_data_dados[0]

        menor_data = menor_data_dados.get('pedite_data_emissao')

        # Maior data de emissao
        pedido_item_ids = self.pool.get('estoquepedidos.pedido.item').search(
            cr, uid, args, order='pedite_data_emissao desc', limit=1, context=context)
        maior_data_dados = self.pool.get('estoquepedidos.pedido.item').read(
            cr, uid, pedido_item_ids, ['pedite_data_emissao'], context)

        if maior_data_dados and isinstance(maior_data_dados, list):
            maior_data_dados = maior_data_dados[0]

        if isinstance(maior_data_dados, dict):
            maior_data = maior_data_dados.get('pedite_data_emissao')
        else:
            maior_data = None

        # Periodo
        menor_data = map(lambda m: int(m), menor_data[:10].split('-'))
        maior_data = map(lambda m: int(m), maior_data[:10].split('-'))
        periodo = datetime(menor_data[0], menor_data[1], menor_data[2]).strftime('%x'),\
                  datetime(maior_data[0], maior_data[1], maior_data[2]).strftime('%x')
        periodo = '%s a %s' % tuple(map(lambda m: m.replace('-', '/'), periodo))

        self.localcontext.update(dict(datetime=datetime, agora=agora, periodo=periodo))

        self.context = context

report_sxw.report_sxw('report.pedidos.pendentes', 'estoquepedidos.pedido.item',
                      'addons/wms/controle_os/report/pedidos_pendentes.rml',
                      parser=report_pedidos_pendentes, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
