# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from openerp.tools import drop_view_if_exists


# noinspection PyPep8Naming
class pgview_wms_estoque_picking(osv.Model):

    _name = "view.wms.estoque.picking"  # wmsepk
    _auto = False
    _columns = {
        # ___________________________ Integer ________________________________
        'id': fields.integer('id'),
        'wmsepk_deposito': fields.integer('Deposito'),
        'wmsepk_rua': fields.integer('Rua'),
        'wmsepk_predio': fields.integer('Predio'),
        'wmsepk_nivel': fields.integer('Nivel'),
        'wmsepk_apartamento': fields.integer('Apartamento'),
        'wmsepk_capacidade_picking': fields.integer('Capacidade'),
        'wmsepk_ponto_reposicao': fields.integer('Ponto de reposicao'),
        'wmsepk_saldo': fields.integer('Saldo'),

        # ___________________________ Float __________________________________
        'wmsepk_reposicao_percent': fields.float('% reposicao', digits=(5, 2)),
        'wmsepk_capacidade_percent': fields.float('% capacidade', digits=(5, 2)),
        'wmsepk_qtde_embalagem': fields.float('Quantidade na embalagem', digits=(12, 3)),

        # ___________________________ Char ___________________________________
        'wmsepk_codigo': fields.char('Codigo do produto', size=20),
        'wmsepk_produto': fields.char('Produto', size=60),
        'wmsepk_grade': fields.char('Grade', size=2),
        'wmsepk_endereco': fields.char('Endereco', size=20),
        'wmsepk_situacao': fields.char('Situacao', size=1,
                                       help='Situacao do picking: ocupado, livre, etc.'),
        'wmsepk_reposicao_situacao': fields.char('Situacao da reposicao', size=6,
                                                 help='Situacao da reposicao: "acima", "abaixo" ou '
                                                      '"igual". Diferenca ente saldo e ponto de '
                                                      'reposicao do picking.'),

        # ___________________________ Boolean ________________________________
        'wmsepk_bloqueado': fields.boolean('Bloqueado')
    }

    _order = 'wmsepk_endereco'

    # noinspection PyMethodMayBeStatic
    def init(self, cr):
        drop_view_if_exists(cr, 'view_wms_estoque_picking')
        cr.execute(
            """
            CREATE VIEW view_wms_estoque_picking AS (
            SELECT  ROW_NUMBER() OVER ()                                                                  AS id,
                    ep.codigo                                                                             AS wmsepk_codigo,
                    ep.name                                                                               AS wmsepk_produto,
                    epg.grade                                                                             AS wmsepk_grade,
                    e.deposito                                                                            AS wmsepk_deposito,
                    e.rua                                                                                 AS wmsepk_rua,
                    e.predio                                                                              AS wmsepk_predio,
                    e.nivel                                                                               AS wmsepk_nivel,
                    e.apartamento                                                                         AS wmsepk_apartamento,
                    e.name                                                                                AS wmsepk_endereco,
                    e.bloqueado                                                                           AS wmsepk_bloqueado,
                    e.state                                                                               AS wmsepk_situacao,
                    epg.capacidade_picking * ee.quantidade_na_embalagem                                   AS wmsepk_capacidade_picking,
                    epg.ponto_reposicao * ee.quantidade_na_embalagem                                      AS wmsepk_ponto_reposicao,
                    ee.quantidade_na_embalagem                                                            AS wmsepk_qtde_embalagem,
                    (e.quantidade + e.reserva_credito - e.reserva_debito)                                 AS wmsepk_saldo,
                    CASE WHEN epg.capacidade_picking != 0 THEN (
                        100.0 * epg.ponto_reposicao / epg.capacidade_picking) ELSE 0 END                  AS wmsepk_reposicao_percent,
                    CASE WHEN epg.capacidade_picking != 0 THEN (100.0 *
                        (e.quantidade + e.reserva_credito - e.reserva_debito) / epg.capacidade_picking)
                      ELSE 0 END                                                                          AS wmsepk_capacidade_percent,
                    CASE WHEN ( -- SALDO > REPOSICAO --
                        (e.quantidade + e.reserva_credito - e.reserva_debito) > epg.ponto_reposicao)      THEN 'acima'
                         WHEN ( -- SALDO < REPOSICAO --
                        (e.quantidade + e.reserva_credito - e.reserva_debito) < epg.ponto_reposicao)      THEN 'abaixo'
                         WHEN (  -- SALDO > 0 E SEM REPOSICAO --
                        (e.quantidade + e.reserva_credito - e.reserva_debito) > 0)
                              AND epg.ponto_reposicao IS NULL                                             THEN 'acima'
                         WHEN (  -- SALDO < 0 E SEM REPOSICAO --
                        (e.quantidade + e.reserva_credito - e.reserva_debito) < 0)
                              AND epg.ponto_reposicao IS NULL                                             THEN 'abaixo'
                      ELSE 'igual' END                                                                    AS wmsepk_reposicao_situacao
            FROM    wms_endereco                                                                          AS e
                LEFT JOIN estoque_produto_grade                                                           AS epg   ON epg.id = e.grade_id
                LEFT JOIN estoque_produto                                                                 AS ep    ON ep.id = epg.produto_id
                LEFT JOIN estoque_produto_embalagem                                                       AS epe   ON epe.id = epg.embalagem_master_id
                LEFT JOIN estoque_embalagem                                                               AS ee    ON ee.id = epe.embalagem_id
            WHERE   e.tipo_nivel                                                                          = 'PK' -- Somente enderecos de picking
            );
            """
        )

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
