# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

import StringIO
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4
from openerp.tools.translate import _
from datetime import datetime
from pytz import timezone


class ReportPDF(object):
    __doc__ = """Report PDF
    by Pierre Barbosa  <barbosa.pierre[at]gmail.com>
    """

    def __init__(self):
        self.title = 'Report'
        self.pagesize = A4
        self.margin_left, self.margin_top, self.margin_right, self.margin_bottom = 30, 50, 30, 50
        self.font_name, self.font_size, self.font_leading = 'Helvetica', 10, 12
        self.pdf_string = StringIO.StringIO()
        self.__canvas = canvas.Canvas(self.pdf_string, self.pagesize)
        self.tz = 'Brazil/East'

        # Header & footer
        self.__header = []
        self.last_row_header = self.pagesize[1] - self.margin_top
        self.__footer = [dict(col=self.margin_left, row=self.margin_bottom, title=self.title)]
        self.last_row_footer = self.margin_bottom

    def header(self, col, row, title, fontname=None, fontsize=0, fontleading=0, align='left'):
        """
        Adiciona um novo titulo ao cabecalho.

        :param row:
        :param col:
        :param title:
        :param fontname:
        :param fontsize:
        :param fontleading:
        :return:
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        self.__header.append(dict(col=col, row=row, title=title, fontname=fontname,
                                  fontsize=fontsize, fontleading=fontleading, align=align))

    def footer(self, col, row, title, fontname=None, fontsize=0, fontleading=0, align='left'):
        """
        Adiciona um novo titulo ao rodape.

        :param row:
        :param col:
        :param title:
        :param fontname:
        :param fontsize:
        :param fontleading:
        :return:
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        self.__footer.append(dict(col=col, row=row, title=title, fontname=fontname,
                                  fontsize=fontsize, fontleading=fontleading, align=align))

    def set_title(self, name, updateheader=False, updatefooter=False):
        """
        Define o titulo do relatorio.

        :param name:
        :param updateheader:
        :return:
        """
        # Update header title
        if updateheader:
            newheader = []

            for header in self.__header:
                if header.get('title') == self.title:
                    header = dict(header, title=name)

                newheader.append(header)

            self.__header = newheader

        # Update footer title
        if updatefooter:
            newfooter = []

            for footer in self.__footer:
                if footer.get('title') == self.title:
                    footer = dict(footer, title=name)

                newfooter.append(footer)

            self.__footer = newfooter

        self.title = name
        self.__canvas.setTitle(name)

    def start_page(self, separatorheader=1, separatorfooter=0):
        """
        Inicia uma nova pagina com cabecalho e rodape no relatorio.

        :return:
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # Header
        self.draw_string(self.margin_left, self.pagesize[1] - self.margin_top, self.title,
                         fontname='Helvetica-Bold', size=18*self.font_size/10)
        report_date = datetime.now(
            timezone('UTC')).astimezone(timezone(self.tz)).strftime('%d/%m/%Y')
        report_time = datetime.now(timezone('UTC')).astimezone(timezone(self.tz)).strftime('%Hh%M')
        self.draw_string(self.pagesize[0] - self.margin_right, self.pagesize[1] - self.margin_top,
                         report_date, align='right')
        self.draw_string(self.pagesize[0] - self.margin_right,
                         self.pagesize[1] - self.margin_top - self.font_leading, report_time,
                         align='right')
        self.last_row_header = self.pagesize[1] - self.margin_top - self.font_leading

        for draw in self.__header:
            col = draw.get('col', self.margin_left)
            row = draw.get('row', self.last_row_header)
            title = draw.get('title', '')
            fontname = draw.get('fontname', self.font_name)
            fontsize = draw.get('fontsize', self.font_size)
            fontleading = draw.get('fontleading', self.font_leading)
            align = draw.get('align', 'left')
            self.draw_string(col, row, title, None, fontname, fontsize, fontleading, align)

            self.last_row_header = row if row < self.last_row_header else self.last_row_header

        # Line separator
        x1 = self.margin_left
        y1 = self.last_row_header - self.font_leading / 2
        x2 = self.pagesize[0] - self.margin_right
        y2 = self.last_row_header - self.font_leading / 2
        self.draw_line(x1, y1, x2, y2, separatorheader)

        # Footer
        for draw in self.__footer:
            col = draw.get('col', self.margin_left)
            row = draw.get('row', self.last_row_footer)
            title = draw.get('title', '')
            fontname = draw.get('fontname', self.font_name)
            fontsize = draw.get('fontsize', self.font_size)
            fontleading = draw.get('fontleading', self.font_leading)
            align = draw.get('align', 'left')
            self.draw_string(col, row, title, None, fontname, fontsize, fontleading, align)

            self.last_row_footer = row if row > self.last_row_footer else self.last_row_footer

        page = _('Página: ') + str(self.__canvas._pageNumber)
        self.draw_string(
            self.pagesize[0] - self.margin_right, self.margin_bottom, page,
            align='right')

        # Line separator
        x1 = self.margin_left
        y1 = self.margin_bottom + self.font_leading
        x2 = self.pagesize[0] - self.margin_right
        y2 = self.margin_bottom + self.font_leading
        self.draw_line(x1, y1, x2, y2, separatorfooter)

    def set_color(self, r, g, b, alfa=None):
        self.__canvas.setFillColorRGB(r, g, b, alfa)

    def set_font(self, name, size, leading=None):
        """
        Define a fonte de caracteres do relatorio.

        :param name:
        :param size:
        :param leading:
        :return:

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        leading = size * 1.2 if not leading else leading
        self.__canvas.setFont(name, size, leading)
        self.font_name = name
        self.font_size = size
        self.font_leading = leading

    def set_scale(self, x=1, y=1):
        self.__canvas.scale(x, y)

    def set_page_size(self, pagesize):
        self.__canvas.setPageSize(pagesize)
        self.pagesize = pagesize

    def set_margin_left(self, size):
        self.margin_left = size

    def set_margin_top(self, size):
        self.margin_top = size

    def set_margin_right(self, size):
        self.margin_right = size

    def set_margin_bottom(self, size):
        self.margin_bottom = size

    def draw_string(self, x, y, text, mode=None, fontname=None, size=0, leading=0, align='left'):
        """
        Desenha uma ´string´ na linha do relatorio.

        :param x:
        :param y:
        :param text:
        :param mode:
        :param fontname:
        :param size:
        :param leading:
        :param align:
        :return:
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        fontname = self.font_name if not fontname else fontname
        size = self.font_size if not size else size
        leading = size * 1.2 if not leading else leading
        self.__canvas.setFont(fontname, size, leading)

        if align == 'right':
            self.__canvas.drawRightString(x, y, text, mode)
        elif align == 'center':
            self.__canvas.drawCentredString(x, y, text, mode)
        else:
            self.__canvas.drawString(x, y, text, mode)

        self.__canvas.setFont(self.font_name, self.font_size, self.font_leading)

    def draw_line(self, x1, y1, x2, y2, thickness=0):
        """
        Desenha uma linha.

        :param x1:0
        :param y1:
        :param x2:
        :param y2:
        :param thickness:
        :return:
        """
        self.__canvas.setLineWidth(thickness)
        self.__canvas.line(x1, y1, x2, y2)

    def draw_rect(self, x, y, width, height, stroke=1, fill=0):
        self.__canvas.rect(x, y, width, height, stroke=stroke, fill=fill)

    def page_break(self):
        self.__canvas.showPage()

    def show(self):
        self.__canvas.save()

    def get_string_width(self, text, fontname, size):
        return self.__canvas.stringWidth(text, fontname, size)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
