# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report.interface import report_int
from openerp.report.render import render
from reportlab.lib.pagesizes import A4, landscape
from openerp.tools.translate import _
from ReportPDF import ReportPDF


class external_pdf(render):
    def __init__(self, pdf):
        render.__init__(self)
        self.pdf = pdf
        self.output_type = 'pdf'

    def _render(self):
        return self.pdf


# noinspection PyPep8Naming
class report_custom(report_int):

    def create(self, cr, uid, ids, datas, context=None):
        context = context and context.copy() or {}

        # Datas
        _sql = """ SELECT * FROM view_wms_estoque_picking WHERE id IN %s ORDER BY wmsepk_endereco"""
        cr.execute(_sql % str(tuple(datas.get('endereco_ids') + [0])))
        result = cr.dictfetchall()

        # Report's data
        fields = dict(wmsepk_codigo=dict(width=70, alias=_('Codigo'), order=1, align='center'),
                      wmsepk_produto=dict(width=280, alias=_('Produto'), order=2, align='left'),
                      wmsepk_grade=dict(width=35, alias=_('Grade'), order=3, align='center'),
                      wmsepk_endereco=dict(width=130, alias=_('Endereco'), order=4),
                      wmsepk_capacidade_picking=dict(width=40, alias=_('Cap. Picking'), order=5,
                                                     align='right'),
                      wmsepk_ponto_reposicao=dict(width=60, alias=_('Ponto Rep.'), align='right',
                                                  order=6),
                      wmsepk_capacidade_percent=dict(width=80, alias=_('% Capacidade'), order=7,
                                                     align='right'),
                      wmsepk_reposicao_percent=dict(width=80, alias=_('% Reposicao'), order=8,
                                                    align='right'))

        report = ReportPDF()
        report.set_page_size(landscape(A4))
        report.set_font('Helvetica', 10)
        report.set_title(u'RELATÓRIO DE ESTOQUE (PICKING)', True, True)

        # Show filters name in header
        args = dict(map(lambda m: (m[0], m[2]), datas.get('args')))

        deposito = args.get('wmsepk_deposito') or 'Todos'
        row = report.pagesize[1] - report.margin_top
        report.header(450, row, u'Depósito:', align='right')
        report.header(455, row, str(deposito))

        rua = args.get('wmsepk_rua') or 'Todas'
        rua = '%s a %s' % (min(rua) or 1, max(rua)) if isinstance(rua, list) else rua
        row -= report.font_leading
        report.header(450, row, u'Rua:', align='right')
        report.header(455, row, str(rua))

        predio = args.get('wmsepk_predio') or 'Todos'
        predio = '%s ao %s' % (
            min(predio) or 1, max(predio)) if isinstance(predio, list) else predio
        row -= report.font_leading
        report.header(450, row, u'Prédio:', align='right')
        report.header(455, row, str(predio))

        nivel = args.get('wmsepk_nivel') or 'Todos'
        nivel = '%s ao %s' % (min(nivel), max(nivel)) if isinstance(nivel, list) else nivel
        row -= report.font_leading
        report.header(450, row, u'Nível:', align='right')
        report.header(455, row, str(nivel))

        apto = args.get('wmsepk_apartamento') or 'Todos'
        apto = '%s ao %s' % (min(apto) or 1, max(apto)) if isinstance(apto, list) else apto
        row -= report.font_leading
        report.header(450, row, u'Apartamento:', align='right')
        report.header(455, row, str(apto))

        situacao = dict(O='Ocupado', L='Livre', R='Reservado').get(args.get('wmsepk_situacao'))
        row -= report.font_leading
        report.header(450, row, u'Situação do endereço:', align='right')
        report.header(455, row, situacao or 'Todas')

        codigo = args.get('wmsepk_codigo') or 'Todos'
        row -= report.font_leading
        report.header(450, row, u'Produto:', align='right')
        report.header(455, row, codigo)

        reposicao_situacao = args.get('wmsepk_reposicao_situacao') or 'Todos'
        row -= report.font_leading
        report.header(450, row, u'Estado do ponto de reposição:', align='right')
        report.header(455, row, reposicao_situacao.capitalize())

        # Start page
        report.start_page()

        # Column's name
        col = report.margin_left
        row = report.last_row_header - report.font_leading * 2

        for column in sorted(fields, key=lambda k: fields[k].get('order', 0)):
            align = fields[column].get('align') or 'left'
            dif = 0

            if align == 'right':
                dif = fields[column].get('width')
            elif align == 'center':
                dif = fields[column].get('width') / 2

            report.draw_string(
                col + dif, row, fields[column].get('alias') or column, 0, fontname='Helvetica-Bold',
                align=align)
            col += fields[column].get('width')

        row -= report.font_leading
        report.draw_line(report.margin_left, row,
                         report.pagesize[0] - report.margin_right, row, 1)
        report.set_color(0, 0, 0, .3)
        report.draw_rect(report.margin_left,
                         report.pagesize[1] - report.margin_top - report.font_size * 12,
                         report.pagesize[0] - report.margin_left - report.margin_right,
                         30 * report.font_leading / 12, 0, fill=1)
        report.set_color(0, 0, 0, 1)

        # Detail
        row = report.last_row_header - report.font_leading * 4

        for item in result:
            col = report.margin_left

            if row <= report.last_row_footer + report.font_leading * 2:
                report.page_break()
                report.start_page()
                row = report.last_row_header - report.font_leading * 2

                # Column's name
                for column in sorted(fields, key=lambda k: fields[k].get('order', 999)):
                    align = fields[column].get('align') or 'left'
                    dif = 0

                    if align == 'right':
                        dif = fields[column].get('width')
                    elif align == 'center':
                        dif = fields[column].get('width') / 2

                    report.draw_string(
                        col + dif, row, fields[column].get('alias'), 0, fontname='Helvetica-Bold',
                        align=align)
                    col += fields[column].get('width')

                row -= report.font_leading
                report.draw_line(
                    report.margin_left, row, report.pagesize[0] - report.margin_left, row, 1)
                col = report.margin_left
                report.set_color(0, 0, 0, .3)
                report.draw_rect(report.margin_left,
                                 report.pagesize[1] - report.margin_top - report.font_size * 12,
                                 report.pagesize[0] - report.margin_left - report.margin_right,
                                 30 * report.font_leading / 12, 0, fill=1)
                report.set_color(0, 0, 0, 1)
                row -= report.font_leading

            # Field's content
            for field in sorted(fields, key=lambda k: fields[k].get('order', 999)):
                align = fields[field].get('align') or 'left'
                content = str(item.get(field)) if item.get(field) else ''
                f_width = report.get_string_width(content, report.font_name, report.font_size) or 1
                c_width = fields[field].get('width') or 1
                width = ((c_width - f_width) * len(content)) / f_width
                width = c_width if width > 0 else int(width)
                dif = 0

                if align == 'right':
                    dif = fields[field].get('width')
                elif align == 'center':
                    dif = fields[field].get('width') / 2

                report.draw_string(col + dif, row, content[:width], 0, align=align)
                col += c_width

            row -= report.font_leading

        report.page_break()
        report.show()

        # Show report
        self.obj = external_pdf(report.pdf_string.getvalue())
        self.obj.render()
        del report

        return self.obj.pdf, 'pdf'

report_custom('report.wms.estoque.picking')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

