# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.report import report_sxw
from datetime import datetime
from pytz import timezone


# noinspection PyPep8Naming
class report_wms_posicao_nf(report_sxw.rml_parse):
    """
    Relatorio de posicao das notas ficais. Mostra lista com as notas fiscais que fazem referencia
    ou nao as ordens de servico de recebimento.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    def __init__(self, cr, uid, name, context):
        super(report_wms_posicao_nf, self).__init__(cr, uid, name, context=context)

        tz = context.get('tz') or 'Brazil/East'
        agora = datetime.now(timezone('UTC')).astimezone(timezone(tz))
        dados = dict(datetime=datetime, agora=agora)

        # ___________________________ Objects ________________________________
        obj_wms_posicao_nf = self.pool.get('wizard.wms.posicao.nf')

        campos = ['wizpnf_periodo_inicio', 'wizpnf_periodo_fim']
        wms_posicao_nf_dados = obj_wms_posicao_nf.read(
            cr, uid, context.get('active_id'), campos, context)

        if wms_posicao_nf_dados and \
                isinstance(wms_posicao_nf_dados, list):
            wms_posicao_nf_dados = wms_posicao_nf_dados[0]

        if wms_posicao_nf_dados and \
                isinstance(wms_posicao_nf_dados, dict):
            inicio = wms_posicao_nf_dados.get('wizpnf_periodo_inicio')
            fim = wms_posicao_nf_dados.get('wizpnf_periodo_fim')

            if inicio and fim:
                inicio = datetime.strptime(inicio, '%Y-%m-%d').strftime('%d/%m/%Y')
                fim = datetime.strptime(fim, '%Y-%m-%d').strftime('%d/%m/%Y')
                if inicio == fim:
                    dados['periodo'] = 'Em %s' % inicio or fim
                else:
                    dados['periodo'] = 'De %s a %s' % (inicio, fim)
            elif inicio and not fim:
                inicio = datetime.strptime(inicio, '%Y-%m-%d').strftime('%d/%m/%Y')
                dados['periodo'] = 'Desde %s' % inicio
            elif not inicio and fim:
                fim = datetime.strptime(fim, '%Y-%m-%d').strftime('%d/%m/%Y')
                dados['periodo'] = 'Ate %s' % fim
            else:
                dados['periodo'] = ''

            dados = dict(dados, inicio=inicio, fim=fim)   # Periodo: inicial e final

        self.localcontext.update(dados)

        self.context = context

report_sxw.report_sxw('report.wms.posicao.nf', 'view.wms.posicao.nf',
                      'addons/wms/report/posicao_nf.rml',
                      parser=report_wms_posicao_nf, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

