# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
from time import strftime


# noinspection PyPep8Naming
class wms_separacao_itens(osv.Model):
    """
    Itens de separacao.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.separacao.itens'  # wmssit
    _inherit = 'wms.separacao.itens'  # wms/separacao/model/separacao_itens_data

    def wmssit_conferencia_checkout(self, cr, uid, ids, context=None):
        """
        Separacao de mercadorias. Realiza a conferencia e registra o coferente e data da contagem

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se o registro da conferencia foi efetuada com sucesso.
        :rtype: bool
        :raise: mesagem de erro se a ordem de servico nao estiver aberta.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = False

        # Permite separacao apenas para ordem de servico com status "aberta"
        for item in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if item.os_id.state != 'aberta':
                msg = _('Alteracao nos dados do item so e permitida se a srdem de servico estiver '
                        'aberta.')

                raise osv.except_osv(_('Atencao!'), msg)

            # Usuario que realizou a conferencia no checkout
            dados = {'conferente_id': uid, 'data_checkout': strftime('%Y-%m-%d %H:%M:%S')}

            result = self.write(cr, uid, item.id, dados, context)

        return result

    def wmssit_onchance_qtde_conferida(self, cr, uid, ids, quantidade, qtde_cortada, qtde_conferida,
                                       qtde_conferida_unidade, context=None):
        """

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param quantidade: quantidade a ser separada baseada na embalagem do item,
        :param qtde_cortada: quantidade a ser cortada baseada na embalagem do item,
        :param qtde_conferida: quantidade a ser conferida baseada na embalagem do item,
        :param qtde_conferida_unidade: quantidade a ser conferida (em unidades simples),
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type quantidade: float
        :type qtde_conferida: float
        :type qtde_cortada: float
        :type qtde_conferida_unidade: int
        :type context: dict
        :return: dados do item com a quantidade a ser cortada.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        brw_wms_separacao_itens = self.browse(
            cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        produto_embalagem = brw_wms_separacao_itens.produto_embalagem_id
        qtde_na_embalagem = produto_embalagem.embalagem_id.quantidade_na_embalagem

        if qtde_conferida + qtde_cortada > quantidade:
            result = {
                'value': {'qtde_conferida': qtde_conferida_unidade / qtde_na_embalagem},
                'warning': {
                    'title': _('Aviso.'),
                    'message': _('A quantidade conferida supera a quantidade solicitada.')
                }
            }
        elif qtde_conferida + qtde_cortada < 1:
            result = {
                'value': {'qtde_conferida': qtde_conferida_unidade / qtde_na_embalagem},
                'warning': {
                    'title': _('Aviso.'),
                    'message': _('A quantidade conferida nao e valida.')
                }
            }
        else:
            result = {
                'value': {
                    'wmssit_qtde_conferida_unidade': qtde_conferida * qtde_na_embalagem,
                    'wmssit_qtde_cortada_unidade': qtde_cortada * qtde_na_embalagem
                }
            }

        # Item conferido
        if qtde_conferida + qtde_cortada == quantidade:
            result['value']['item_conferido'] = True

        return result

    def wmssit_onchange_qtde_cortada(self, cr, uid, ids, quantidade, qtde_cortada, qtde_conferida,
                                     grade_id, context=None):
        """

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param quantidade: quantidade a ser separada,
        :param qtde_cortada: quantidade a ser cortada,
        :param qtde_conferida: quantidade a ser conferida,
        :param grade_id: ID da grade,
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type quantidade: float
        :type qtde_cortada: float
        :type qtde_conferida: float
        :type grade_id: int
        :type context: dict
        :return: dados do item com a quantidade a ser cortada.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = {}

        # Valida a quantidade a ser cortada (% de corte)
        brw_wms_separacao_itens = self.browse(
            cr, uid, ids[0] if isinstance(ids, list) else ids, context)
        produto_embalagem = brw_wms_separacao_itens.produto_embalagem_id
        qtde_na_embalagem = produto_embalagem.embalagem_id.quantidade_na_embalagem

        titulo = _('Aviso.')
        msg = _('Valor invalidao para quantidade cortada.')

        result['value'] = {
            'qtde_cortada': qtde_cortada,
            'wmssit_qtde_cortada_unidade': qtde_cortada * qtde_na_embalagem,
            'produto_embalagem_id': produto_embalagem.id
        }

        corte = brw_wms_separacao_itens.grade_id.corte  # Percentual (%) de corte
        corte = round(quantidade * corte / 100.0, 0)

        if corte < qtde_cortada:
            result['warning'] = {
                'title': titulo,
                'message': msg
            }
            result['value'] = {
                'qtde_cortada': 0.0,
                'wmssit_qtde_contada_unidade': 0,
                'produto_embalagem_id': produto_embalagem.id
            }
        elif qtde_cortada < 0 or qtde_cortada + qtde_conferida > quantidade:
            result['warning'] = {
                'title': titulo,
                'message': msg
            }
            result['value'] = {
                'qtde_cortada': 0.0,
                'wmssit_qtde_contada_unidade': 0,
                'produto_embalagem_id': produto_embalagem.id
            }

        # Item conferido
        if qtde_cortada + qtde_conferida == quantidade:
            result['value']['item_conferido'] = True

        return result

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Converte a quantidade expressa na embalagem em unidades.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param values: dicionario com os campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type values: dict
        :type context: dict
        :return: dados do registro com a quantidade expressa em unidades simples
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')
        obj_estoque_embalagem = self.pool.get('estoque.embalagem')
        obj_wms_endereco = self.pool.get('wms.endereco')

        qtde_na_embalagem = 0
        produto_embalagem_id = values.get('produto_embalagem_id')

        if produto_embalagem_id:
            fields = ['embalagem_id']
            estoque_produto_embalagem_dados = obj_estoque_produto_embalagem.read(
                cr, uid, produto_embalagem_id, fields, context, load='_classic_write')
            embalagem_id = estoque_produto_embalagem_dados.get('embalagem_id')
            fields = ['quantidade_na_embalagem']
            estoque_embalagem_dados = obj_estoque_embalagem.read(
                cr, uid, embalagem_id, fields, context)
            qtde_na_embalagem = estoque_embalagem_dados.get('quantidade_na_embalagem')

        if 'quantidade' in values:
            values['quantidade'] *= qtde_na_embalagem

            # Debito da quantidade no endereco
            if 'endereco_id' in values:
                obj_wms_endereco.wmsend_reserva_debito(
                    cr, uid, values['endereco_id'], values['quantidade'])

        if 'qtde_conferida' in values:
            values['qtde_conferida'] *= qtde_na_embalagem

        if 'qtde_cortada' in values:
            values['qtde_cortada'] *= qtde_na_embalagem

        return super(wms_separacao_itens, self).create(cr, uid, values, context)

    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Converte a quantidade expressa na embalagem em unidades.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs
        :param fields: lista com os nome dos campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type fields: list
        :type context: dict
        :return: dados do registro com a quantidade expressa em unidades da embalagem
        :rtype: dict|list[dict]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoque_produto_embalagem = self.pool.get('estoque.produto.embalagem')

        if not 'produto_embalagem_id' in fields:
            fields.append('produto_embalagem_id')

        results = super(wms_separacao_itens, self).read(cr, uid, ids, fields, context, load)

        for result in results if isinstance(results, list) else [results]:
            produto_embalagem_id = result.get('produto_embalagem_id', 0)

            if produto_embalagem_id:
                if isinstance(produto_embalagem_id, (list, tuple)):
                    produto_embalagem_id = produto_embalagem_id[0]

                dados_embalagem = obj_estoque_produto_embalagem.browse(
                    cr, uid, produto_embalagem_id, context)

                try:
                    qtde_na_embalagem = dados_embalagem.embalagem_id.quantidade_na_embalagem
                except:
                    qtde_na_embalagem = 1
            else:
                qtde_na_embalagem = 1

            if 'quantidade' in result:
                result['quantidade'] /= qtde_na_embalagem

            if 'qtde_conferida' in result:
                result['qtde_conferida'] /= qtde_na_embalagem

            if 'qtde_cortada' in result:
                result['qtde_cortada'] /= qtde_na_embalagem

        return results

    def write(self, cr, uid, ids, values, context=None):
        """
        Converte a quantidade expressa na embalagem em unidades.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs
        :param values: dicionario com os nome dos campos e seus respectivos valores,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type values: dict
        :type context: dict
        :return: verdadeiro se a gravacao dos dados foi efetuada com sucesso
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        brw_wms_separacao_itens = self.browse(
            cr, uid, ids[0] if isinstance(ids, list) else ids, context)

        try:
            embalagem = brw_wms_separacao_itens.produto_embalagem_id.embalagem_id
            qtde_na_embalagem = embalagem.quantidade_na_embalagem
        except:
            qtde_na_embalagem = 1

        if 'quantidade' in values:
            values['quantidade'] *= qtde_na_embalagem

        if 'qtde_conferida' in values:
            values['qtde_conferida'] *= qtde_na_embalagem

        if 'qtde_cortada' in values:
            values['qtde_cortada'] *= qtde_na_embalagem

        result = super(wms_separacao_itens, self).write(cr, uid, ids, values, context)

        return result

    def unlink(self, cr, uid, ids, context=None):
        """
        Nao permite que itens de separacao sejam excluidos.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type context: dict
        :return: verdadeiro se o item foi excluido
        :rtype: bool
        :raise: mensagem de erro se o item nao foi excluido.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if not context.get('separacao'):
            msg = _('Item de separacao nao pode ser excluido.')

            raise osv.except_osv(_('Atencao!'), msg)

        return super(wms_separacao_itens, self).unlink(cr, uid, ids, context)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
