# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class estoque_parametro(osv.Model):
    """
    Calculo da cubabem (volume) da caixa de fracionado. (heranca)

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.parametro'  # estpar
    _inherit = 'estoque.parametro'  # estoque/control/parametro

    @staticmethod
    def estpar_caixa_fracionado_cubagem(cr, uid, ids, altura, largura, profundidade):
        """
        Calculo da cubagem da caixa de separacao para produtos fracionados.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} altura da caixa de francionado,
        @param largura: {float} largura da caixa de francionado,
        @param profundidade: {float} profundidade da caixa de francionado,
        @return: {dict} valores ajustados dos campos.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        cubagem = altura * largura * profundidade / 1000000.0  # Volume em metros cubicos (m3)
        result = {'value': {'caixa_fracionado_volume': cubagem}}

        return result

    @staticmethod
    def estpar_palete_cubagem(cr, uid, ids, altura, largura, profundidade):
        """
        Calculo do volume (cubagem) do palete.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param altura: {float} altura do palete,
        @param largura: {float} largura do palete,
        @param profundidade: {float} profundidade do palete,
        @return: {dict} valores ajustados dos campos.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        cubagem = altura * largura * profundidade
        result = {'value': {'volume_palete': cubagem}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
