# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class estoque_parametro(osv.Model):
    """
    Parametros do modulo de montagem de carga. (heranca)

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'estoque.parametro'  # estpar
    _inherit = 'estoque.parametro'  # estoque/control/parametro

    from funcoes import estpar_fields_function, estpar_constraint_verifica_peso
    from funcoes import estpar_constraint_verifica_quebra, estpar_constraint_verifica_volume

    _columns = {
        # ___________________________ Boolean ________________________________
        'quebra_peso': fields.boolean('Quebra por peso',
                                      help='Parametro de quebra para separacao. Se esta opcao estiver marcada, a separacao dos produtos nos paletes sera em funcao do peso maximo suportado pelo palete.'),
        'quebra_volume': fields.boolean('Quebra por volume',
                                        help='Parametro de quebra para separacao. Se esta opcao estiver marcada, a separacao dos produtos nos paletes sera em funcao do volume (cubagem) maxima suportada pelo palete.'),

        'palete_extra': fields.boolean('Palete extra para produtos perigosos',
                                       help='Se esta opcao estiver marcada, paletes extras serao criados para suportar os produtos marcados como perigosos.'),

        # ___________________________ Float __________________________________
        'peso_palete': fields.float('Peso (Kg)', digits=(9, 3),
                                    help='Peso maximo (Kg) suportado pelo palete.'),
        'altura_palete': fields.float('Altura do palete (m)', digits=(9, 2),
                                      help='Altura maxima permitida no empilhamento de produtos sobre o palete.'),
        'largura_palete': fields.float('Largura do palete (m)', digits=(9, 2),
                                       help='Larguram maxima permitida para a disposicao de produtos no palete.'),
        'profundidade_palete': fields.float('Profundidade do palete (m)', digits=(9, 2),
                                            help='Profundidade maxima permitida para a disposicao de produtos no palete.'),
        'caixa_fracionado_altura': fields.float('Altura da caixa (cm)', digits=(5, 2),
                                                help='Altura maxima da caixa destinada a separacao de produtos fracionados.'),
        'caixa_fracionado_largura': fields.float('Largura da caixa (cm)', digits=(5, 2),
                                                 help='Largura maxima da caixa destinada a separacao de produtos fracionados.'),
        'caixa_fracionado_profundidade': fields.float('Profundidade da caixa (cm)', digits=(5, 2),
                                                      help='Profundidade maxima da caixa destinada a separacao de produtos fracionados.'),

        # ___________________________ Char ___________________________________
        'descricao_palete': fields.char('Palete', size=30),
        'caixa_fracionado_descricao': fields.char('Caixa para fracionado', size=30),

        # ___________________________ Integer ________________________________
        'caixas_fracionado_carrinho': fields.integer('Caixas por carrinho',
                                                     help='Numero maximo de caixas suportadas no E.T. (Carrinho/Elemento de transporte).'),

        # ___________________________ Function _______________________________
        'volume_palete': fields.function(estpar_fields_function, method=True, digits=(15, 9),
                                         string='Volume do palete (m3)',
                                         help='Volume (cubagem) do palete. Produto entre altura, largura e profundidade (m3).'),
        'caixa_fracionado_volume': fields.function(estpar_fields_function, digits=(15, 9),
                                                   method=True, string='Volume (m3)',
                                                   help='Volume (cubagem) da caixa de separacao de fracionado. Produto entre altura, largura e profundidade (m3).')
    }

    _defaults = {
        'quebra_peso': lambda *args: True,
        'quebra_volume': lambda *args: True,
        'palete_extra': lambda *args: False,
        'peso_palete': lambda *args: 1200.0,
        'altura_palete': lambda *args: 1.6,
        'largura_palete': lambda *args: 1.0,
        'descricao_palete': lambda *args: 'Palete padrao (PBR)',
        'profundidade_palete': lambda *args: 1.2,
        'caixa_fracionado_altura': lambda *args: 10,
        'caixa_fracionado_largura': lambda *args: 10,
        'caixa_fracionado_profundidade': lambda *args: 10,
        'caixa_fracionado_descricao': lambda *args: 'CAIXA 10x10x10',
        'caixas_fracionado_carrinho': lambda *args: 10
    }

    # Verificacao
    _constraints = [(estpar_constraint_verifica_quebra, 'Ao menos um tipo de quebra deve ser '
                                                        'definido.',
                     ['Quebra por peso', 'Quebra por volume']),
                    (estpar_constraint_verifica_peso, 'Peso do palete nao pode ser zero ou menor '
                                                      'que zero.',
                     ['Peso do palete (Kg)']),
                    (estpar_constraint_verifica_volume, 'Volume do palete nao pode ser zero ou '
                                                        'menor que zero.',
                     ['Volume do palete (m3)'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
