# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time


# noinspection PyPep8Naming
class separacao_os(osv.Model):
    """
    Separacao. Separacao de mercadorias (box e checkout).

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'separacao.os'  # seposv
    _inherit = 'separacao.os'  # wms/separacao/inherit/os_data

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Abrir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a ordem de servico foi aberta
        :raise: mensagem de erro se necessaria a inclusao de um responsavel.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('separacao.os.historico')

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo_id.codigo in ('SPC', 'SPR'):
                # Se nao existir responsavel a OS nao podera ser aberta
                if not os.conferente_id:
                    msg = _('Informe o nome do separador.')

                    raise osv.except_osv(_('Atencao!'), msg)

            acao = _('Abrir ordem de servico')
            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

        return super(separacao_os, self).osvosv_abrir_os(cr, uid, ids, context)

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico de separacao. Somente havera cancelamento da ordem de servico se
        a montagem de carga for cancelada.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a ordem de servico foi cancelada.
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_os_historico = self.pool.get('separacao.os.historico')
        obj_wms_separacao_itens = self.pool.get('wms.separacao.itens')

        # Itens de separacao
        args = [('os_id', 'in' if isinstance(ids, list) else '=', ids)]
        wms_separacao_item_ids = obj_wms_separacao_itens.search(cr, uid, args, context=context)
        brw_wms_separacao_itens = obj_wms_separacao_itens.browse(
            cr, uid, wms_separacao_item_ids, context=context)

        for brw_wms_separacao_item in brw_wms_separacao_itens:
            embalagem = brw_wms_separacao_item.produto_embalagem_id.embalagem_id
            qtde_na_embalagem = embalagem.quantidade_na_embalagem
            quantidade_cancelada = brw_wms_separacao_item.quantidade * qtde_na_embalagem * -1
            obj_wms_endereco.wmsend_reserva_debito(
                cr, uid, brw_wms_separacao_item.endereco_id.id, quantidade_cancelada, context)

        # Desbloqueio dos enderecos utilizados na separacao
        endereco_ids = map(lambda m: m.endereco_id.id, brw_wms_separacao_itens)
        obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_ids, context)

        # Cancela as ordens de servico de separacao
        acao = _('Cancelamento da ordem de servico')

        obj_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context)

        result = super(separacao_os, self).osvosv_cancelar_os(cr, uid, ids, context)

        return result

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Fechar ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se a ordem de sevico foi fechada.
        :rtype: bool
        :raise: mensagem de erro se a ordem de servico nao puder ser fechada.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        # obj_wms_endereco = self.pool.get('wms.endereco')
        obj_os_historico = self.pool.get('separacao.os.historico')
        obj_wms_montagemcarga = self.pool.get('wms.montagemcarga')

        result = False

        # Verifica se todos os itens ja foram separados para fechar a ordem de servico
        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo_id.codigo in ('SPC', 'SPR'):
                if not os.seposv_conferencia_ok:
                    msg = _('Ordem de servico nr %s nao pode ser fechada. Existem itens que ainda '
                            'nao foram conferidos.' % os.numero)

                    raise osv.except_osv(_('Atencao!'), msg)

                acao = _('Fechar ordem de servico')
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

                super(separacao_os, self).osvosv_fechar_os(cr, uid, ids, context)

                # Verifica se todas as ordens de servico relacionadas a carga ja foram fechadas
                # para alterar o ´´status´´ da montagem de carga para ´´separada``.
                args = [('montagemcarga_id', '=', os.montagemcarga_id.id),
                        ('state', 'not in', ['fechada', 'cancelada'])]

                # Todas as ordens de servico estao fechadas/canceladas
                if not self.search(cr, uid, args, context=context):
                    dados = {'state': 'separada'}
                    carga_id = os.montagemcarga_id.id

                    result = obj_wms_montagemcarga.write(cr, uid, carga_id, dados, context)

        return result

    def osvosv_imprimir_os(self, cr, uid, ids, context=None):
        """
        Imprimir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o relatorio da ordem de servico.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('separacao.os.historico')

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo in ('SPC', 'SPR'):
                acao = _('Imprimir ordem de servico')
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, acao, context)

                mapa = {
                    'type': 'ir.actions.report.xml',
                    'datas': {
                        'ids': [os.id],
                        'model': 'separacao.os'}
                }

                if os.tipo == 'SPC':
                    # Mapa de separacao paletizada por carga
                    mapa['report_name'] = 'separacao.paletizada.carga'
                elif os.tipo == 'SPR':
                    # Mapa de separacao paletizada por rua
                    mapa['report_name'] = 'separacao.paletizada.rua'

                return mapa

        return super(separacao_os, self).osvosvimprimir_os(cr, uid, ids, context)

    def seposv_button_separar(self, cr, uid, ids, context=None):
        """
        Registro do historico da separacao.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: verdadeiro se o historico foi registrado.
        :rtype: bool
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('separacao.os.historico')

        dados = {}
        acao = None

        if context.get('status_separacao') == 'inicio':
            # Informa o inicio da separacao no historico
            acao = _('Inicio da separacao')
            dados['seposv_data_inicio_separacao'] = time.strftime('%Y-%m-%d %H:%M:%S')
        elif context.get('status_separacao') == 'fim':
            # Informa o termino da separacao no historico
            acao = _('Termino da separacao')
            dados['seposv_data_fim_separacao'] = time.strftime('%Y-%m-%d %H:%M:%S')

        if acao:
            obj_os_historico.osvhis_registra_acao(cr, uid, ids, acao, context)

        result = self.write(cr, uid, ids, dados, context)

        return result

    def seposv_onchange_montagemcarga(self, cr, uid, ids, carga_id, context=None):
        """
        Mostra os valores da carga, calcula e atualiza valores de peso e volume para quebra.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param carga_id: ID da ordem de servico de Montagem de Carga,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type carga_id: int
        :type context: dict
        :return: valores atualizados de peso e volume.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        # ___________________________ Objects ________________________________
        obj_carga = self.pool.get('wms.montagemcarga')
        obj_veiculo = self.pool.get('veiculos.veiculo')

        peso_quebra = 0
        volume_quebra = 0
        peso_total = 0
        volume_total = 0
        paletes = 0

        campos = ['veiculo_id', 'peso_total', 'volume_total']
        dados_carga = obj_carga.read(cr, uid, carga_id, campos, context, load='_classic_write')

        if dados_carga and isinstance(dados_carga, list):
            dados_carga = dados_carga[0]

        if dados_carga and isinstance(dados_carga, dict):
            veiculo_id = dados_carga.get('veiculo_id')
            peso_total = dados_carga.get('peso_total')
            volume_total = dados_carga.get('volume_total')

            if veiculo_id:
                campos = ['qtde_palete']
                dados_veiculo = obj_veiculo.read(cr, uid, veiculo_id, campos, context)

                if dados_veiculo and isinstance(dados_veiculo, list):
                    dados_veiculo = dados_veiculo[0]

                if dados_veiculo and isinstance(dados_veiculo, dict):
                    paletes = dados_veiculo.get('qtde_palete')

            if paletes:
                peso_quebra = peso_total / paletes
                volume_quebra = volume_total / paletes

        result = {'value': {'peso': peso_quebra, 'volume': volume_quebra, 'qtde_paletes': paletes,
                            'carga_total': peso_total, 'volume_total': volume_total}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
