# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


class separacao_os(osv.Model):
    """
    Ordem de servico - Mapa de separacao.
    Comum a SPC - Separacao paletizada por carga, SPR - Separacao paletizada por rua.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'separacao.os'  # seposv
    _inherit = 'os'  # recebimento/os/model/os_data

    from funcoes import seposv_fields_function

    _columns = {
        # ___________________________ Many2Many ______________________________
        # Responsaveis pela execucao da ordem de servico
        'responsavel_ids': fields.many2many('res.users', 'separacao_m2m_user', 'os_id', 'user_id',
                                            'Responsaveis'),

        # ___________________________ Many2One _______________________________
        'montagemcarga_id': fields.many2one('wms.montagemcarga', 'Carga'),

        # ___________________________ One2Many _______________________________
        'separacao_item_ids': fields.one2many('wms.separacao.itens', 'os_id', 'Item de separacao'),

        # ___________________________ Char ___________________________________
        'rua': fields.char('Rua', size=2),  # Numero da rua se for separacao de carga por rua
        'box': fields.char('Box', size=2),

        # ___________________________ Datetime _______________________________
        'seposv_data_inicio_separacao': fields.datetime('Inicio da separacao'),
        'seposv_data_fim_separacao': fields.datetime('Fim da separacao'),

        # ___________________________ Boolenan _______________________________
        # Verdadeiro se existir item fracionado para a ordem de servico
        'fracionado': fields.boolean('Fracionado'),

        # ___________________________ Function _______________________________
        'conferente': fields.function(seposv_fields_function, method=True, type='char', size=60,
                                      string='Conferente'),
        'qtde_volumes': fields.function(seposv_fields_function, method=True, type='integer',
                                        string='Total de volumes'),
        'qtde_itens': fields.function(seposv_fields_function, method=True, type='integer',
                                      string='Total de itens'),  # Quantidade de itens (unicos)
        'peso_total': fields.function(seposv_fields_function, method=True, digits=(12, 3),
                                      string='Peso total (Kg)'),
        'cubagem_total': fields.function(seposv_fields_function, method=True, digits=(12, 3),
                                         string='Cubagem total (m3)'),
        # Verdadeiro se o usuario for o mesmo que esta realizando a separacao
        'mesmo_conferente': fields.function(seposv_fields_function, method=True, type='boolean',
                                            string='Mesmo separador'),
        # Verdadeiro para mostrar a quantidade a ser separada
        'seposv_mostra_quantidade': fields.function(seposv_fields_function, method=True,
                                                    type='boolean', string='Mostrar quantidade'),
        'seposv_conferencia_ok': fields.function(seposv_fields_function, method=True,
                                                 type='boolean', string="Itens confereridos")
    }

    _defaults = {
        'fracionado': lambda *args: False
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
