# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields


# noinspection PyPep8Naming
class wms_separacao_itens(osv.Model):
    """
    Itens de separacao.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.separacao.itens'  # wmssit
    _inherits = {'estoque.produto.grade': 'grade_id'}

    from funcoes import wmssit_fields_function

    _columns = {
        # ___________________________ Many2One _______________________________
        'endereco_id': fields.many2one('wms.endereco', 'Endereco', ondelete='restrict',
                                       help='Endereco do produto (picking ou pulmao)'),
        'pessoa_id': fields.many2one('adm.pessoa', 'Cliente/loja', ondelete='restrict'),
        'produto_id': fields.many2one('estoque.produto', 'Produto', required=True, select=True,
                                      ondelete='restrict'),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', required=True, select=True,
                                    ondelete='restrict'),
        'produto_embalagem_id': fields.many2one('estoque.produto.embalagem', 'Embalagem',
                                                ondelete='restrict'),
        'produto_tipo_id': fields.many2one('wms.produtotipo', 'Tipo de produto',
                                           help='Tipo de produto (ex.: grandeza, leveza, etc.)'),
        'pedido_id': fields.many2one('estoquepedidos.pedido.item', 'Pedido', ondelete='restrict'),
        'separador_id': fields.many2one('res.users', 'Separador', ondelete='restrict',
                                        help='Usuario responsavel pela separacao'),
        'conferente_id': fields.many2one('res.users', 'Conferente', ondelete='restrict',
                                         help='Usuario responsavel pela conferencia no checkout'),
        'supervisor_id': fields.many2one('res.users', 'Supervisor', ondelete='restrict',
                                         help='Usuario (supervisor) responsavel pelo corte'),

        # ___________________________ Function _______________________________
        'rua': fields.function(wmssit_fields_function, method=True, type='integer', string='Rua'),
        # Sequencia da rota. Necessario para ordenar mapa de separacao
        'seq_rota': fields.function(wmssit_fields_function, method=True, store=True, type='integer',
                                    string='Sequencia na rota'),
        # Sequencia do tipo de produto. Necessario para ordenar mapa de separacao
        'seq_tipo_produto': fields.function(wmssit_fields_function, method=True, store=True,
                                            type='integer', string='Ordem de separacao'),
        'embalagem': fields.function(wmssit_fields_function, method=True, type='char', size=30,
                                     string='Embalagem'),
        'codigo_barra': fields.function(wmssit_fields_function, method=True, type='char', size=15,
                                        string='Codigo de barra'),
        'item_conferido': fields.function(wmssit_fields_function, method=True, type='boolean',
                                          string='Item conferido'),
        'wmssit_endereco': fields.function(wmssit_fields_function, method=True, type='char',
                                           size=20, store=True, string='Endereco'),
        'wmssit_produto': fields.function(wmssit_fields_function, method=True, type='char',
                                          size=60, string='Produto'),

        # ___________________________ Float __________________________________
        'quantidade': fields.float('Quantidade solicitada', digits=(9, 3)),
        'qtde_conferida': fields.float('Quantidade conferida', digits=(9, 3)),
        'qtde_cortada': fields.float('Quantidade cortada', digits=(9, 3)),

        # ___________________________ Integer ________________________________
        'wmssit_qtde_conferida_unidade': fields.integer(string='Quantidade conferida'),
        'wmssit_qtde_cortada_unidade': fields.integer(string='Quantidade cortada'),

        # ___________________________ Char ___________________________________
        'palete': fields.char('Palete', size=3),
        'et': fields.char('E.T.', size=3),  # Carrinho / Elemento de Transferencia / E.T.
        'caixa': fields.char('Caixa', size=3),  # Numero da caixa (fracionado)

        # ___________________________ Datetime _______________________________
        'data_separacao': fields.datetime('Data da separacao'),
        'data_conferencia': fields.datetime('Data da conferencia'),
        'data_corte': fields.datetime('Data do corte'),

        # ___________________________ Boolean ________________________________
        'active': fields.boolean('Ativo')
    }

    _order = 'palete, wmssit_endereco'

    _defaults = {
        'active': lambda *args: True,
        'item_conferido': lambda *args: False
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
