# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
import base64
import unicodedata
import subprocess
import logging


# noinspection PyPep8Naming
class wizard_etiqueta_volume(osv.TransientModel):
    """
    Impressao de etiqueta de volume.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.etiqueta.volume'  # wiziev
    _inherit = 'wizard.etiqueta.volume'  # wms/separacao/wizard/etiqueta_volume_data

    def wiziev_imprimir_etiqueta(self, cr, uid, ids, context=None):
        """
        Imprime etiqueta de volume.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista dos IDs
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: integer
        :type ids: integer|list
        :type context: dict
        :return: verdadeiro se a etiqueta foi impressa ou mensagem de aviso informando
        sobre a configuracao da impressora (geralmente uma impressora termica, propria para
        impressao de etiquetas).
        :rtype: bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_parametro = self.pool.get('estoque.parametro')
        obj_separacao_os = self.pool.get('separacao.os')

        # Leitura dos parametros: impressora e modelo de formulario.
        parametro_ids = obj_estoque_parametro.search(cr, uid, [], context=context)

        titulo = _('Atencao!')
        if parametro_ids:
            parametro_id = parametro_ids[0]
            dados_parametro = obj_estoque_parametro.browse(cr, uid, parametro_id, context)

            # Modelo etiqueta de volume
            modelo_volume = base64.decodestring(dados_parametro.modelo_etiqueta_volume)
        else:
            msg = _('Nenhum parametro definido.')
            raise osv.except_osv(titulo, msg)

        # Dados da ordem de servico de separacao
        brw_separacao = obj_separacao_os.browse(cr, uid, context.get('active_id'), context)
        os_numero = brw_separacao.numero  # Identificador {1}
        data_criacao = brw_separacao.data_criacao[:10]  # Identificador {2}
        os_id = str(brw_separacao.id).zfill(6)  # Identificadores {1} e {3}
        pessoa = brw_separacao.separacao_item_ids[0].pessoa_id
        cliente = pessoa.name  # Identificador {4}
        pedido = brw_separacao.montagemcarga_id.pedido_item_ids[0]
        pedido_numero = pedido.pedite_numero_pedido  # Identificador {5}
        box = brw_separacao.box  # Identificador {6}
        palete_numero = brw_separacao.separacao_item_ids[0].palete  # Identificador {7}
        rota = brw_separacao.separacao_item_ids[0].seq_rota  # Identificador {8}
        endereco_cliente = pessoa.endereco   # Identificador {9}
        hora_criacao = brw_separacao.data_criacao[11:]  # Identificador {10}

        # Remove acentuacao
        if cliente:
            cliente = unicodedata.normalize('NFKD', cliente)
            cliente = cliente.encode('ascii', 'ignore')

        # Impressao das etiquetas de volume
        dados_volume = self.read(cr, uid, ids, context=context)

        if dados_volume and isinstance(dados_volume, list):
            dados_volume = dados_volume[0]

        if dados_volume and isinstance(dados_volume, dict):
            volumes = dados_volume.get('wiziev_qtde_volumes')
            impressora = dados_volume.get('wiziev_impressora_volume_id')[1]

            arquivo_etiqueta = open('/tmp/wms-etq.imp', 'w')
            for volume in range(1, volumes + 1):
                etiqueta = modelo_volume.format('%02d de %02d' % (volume, volumes), os_id,
                                                data_criacao, os_id, cliente, pedido_numero, box,
                                                palete_numero, rota, endereco_cliente, hora_criacao)

                arquivo_etiqueta.write(etiqueta)

            arquivo_etiqueta.close()

            if impressora:
                cmd = 'lp -d %s %s' % (impressora, arquivo_etiqueta.name)

                # Envia arquivo para impressao
                logging.getLogger(__name__).info(cmd)
                subprocess.Popen(cmd, shell=True)

        return True

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
