# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time
from math import ceil

# Indices do item de separacao
PESSOA = 0  # Loja/Cliente
PRODUTO = 1
GRADE = 2

# Indice dos totais do item de separacao
QUANTIDADE = 0
QTDE_MASTER = 1  # Unidades na embalagem Master
NORMA = 2
PICKING = 3
TIPO_PRODUTO = 4

ressuprimento_picking = {}  # Enderecos de picking a serem ressupridos


# noinspection PyPep8Naming
class wizard_mapa_separacao(osv.TransientModel):
    """
    Criar Mapa de separacao.
    """

    _name = 'wizard.mapa.separacao'  # wizmap
    _inherit = 'wizard.mapa.separacao'  # wms/separacao/wizard/mapaseparacao_data

    # noinspection PyMethodMayBeStatic
    def wizmap_onchange_separacao_tipo(self, cr, uid, ids, tipo, context=None):
        """
        Habilita o tipo de calculo (carga ou veiculo) se o tipo de separacao for por carga. Se for
        por rua, o campo do tipo de calculo sera desabilitado.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista de IDs,
        :param tipo: ID do tipo de separacao,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type tipo: int
        :type context: dict
        :return: um dicionario com os valores do codigo de separacao e o tipo de calculo para ser
        substituido nos campos do formulario.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if tipo != 'C':
            result = {
                'value': {
                    'wizmap_calculo_veiculo': False
                }
            }
        else:
            result = {}

        return result

    def wizmap_button_gerar_mapa_separacao(self, cr, uid, ids, context=None):
        """
        Cria o mapa de separacao.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: ID do mapa de separacao (form wizard)
        :param context: um dicionario padrao para valores contextuais
        :type cr object
        :type uid int
        :type ids int
        :return: verdadeiro se o mapa de separacao foi criado.
        :rtype bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        wms_separacao_dados = self._wizmap_obter_dados_separacao(cr, uid, ids, context)
        calculo_veiculo = wms_separacao_dados.get('wizmap_calculo_veiculo')
        por_rua = wms_separacao_dados.get('wizmap_separacao_tipo') == 'SPR' and not calculo_veiculo
        result = self._wizmap_separacao_carga(cr, uid, calculo_veiculo, por_rua, context)

        return result

    def _wizmap_separacao_carga(self, cr, uid, veiculo=False, rua=False, context=None):
        """
        Realiza a separacao por carga - Paletizado por carga/veiculo ou rua.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param veiculo: verdadeiro se utilizar dados do veiculo: carga e numero de paletes
        :param rua: verdadeiro se a separacao for por rua
        :param context: um dicionario padrao para valores contextuais
        :type cr object
        :type uid int
        :type context dict
        :raise mensagem de erro se a separacao nao for efetuada.
        :return: verdadeiro se gerou o mapa de separacao.
        :rtype bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = False

        # ___________________________ Objects ________________________________
        obj_wms_montagemcarga = self.pool.get('wms.montagemcarga')
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_estoquepedidos_pedido_capa = self.pool.get('estoquepedidos.pedido.capa')
        obj_wms_montagemcarga_clientes = self.pool.get('wms.montagemcarga.clientes')
        obj_adm_utils = self.pool.get('adm.utils')
        obj_veiculos_veiculo = self.pool.get('veiculos.veiculo')

        carga_ids = context.get('active_ids')  # Lista com o ID da montagem de carga

        # Verifica o saldo dos itens. Se houver item com saldo divergente a separacao nao pode ser
        # realizada
        if not self._wizmap_validar_carga_item(cr, uid, carga_ids, context):
            msg = _('A separacao nao pode ser efetuada. Existem itens com saldo insuficiente.')
            raise osv.except_osv(_('Atencao!'), msg)

        # Dados do palete padrao
        fields = ['peso_palete', 'volume_palete']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', fields, context=context)
        palete_padrao_peso = adm_utils_dados.get('peso_palete') or 1000.0
        palete_padrao_volume = adm_utils_dados.get('volume_palete') or 1.92

        # Caixas de fracionado
        fields = ['caixas_fracionado_carrinho', 'caixa_fracionado_volume']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', fields, context=context)
        caixa_fracionado_volume = adm_utils_dados.get('caixa_fracionado_volume')
        numero_caixas_et = adm_utils_dados.get('caixas_fracionado_carrinho')  # Numero de caixas

        # Separacao de produtos perigosos (separa palete extra para produtos perigosos)
        fields = ['palete_extra']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', fields, context=context)
        palete_extra = adm_utils_dados.get('palete_extra')

        for carga_id in carga_ids:
            # Dados da carga
            fields = ['wmsmcg_montagemcarga_cliente_ids', 'pedido_item_ids', 'veiculo_id',
                      'peso_total', 'cubagem_total', 'box']
            wms_montagemcarga_dados = obj_wms_montagemcarga.read(
                cr, uid, carga_id, fields, context, load='_classic_write')

            peso_total = wms_montagemcarga_dados.get('peso_total')  # Peso total da carga
            volume_total = wms_montagemcarga_dados.get('cubagem_total')  # Volume total da carga

            # Dados do veiculo
            veiculo_id = wms_montagemcarga_dados.get('veiculo_id')
            fields = ['qtde_palete']
            veiculos_veiculo_dados = obj_veiculos_veiculo.read(cr, uid, veiculo_id, fields, context)
            qtde_paletes_veiculo = veiculos_veiculo_dados.get('qtde_palete') or 2

            if veiculo:  # Se a separacao utilizar dados do veiculo...
                if palete_extra:
                    pedido_item_ids = wms_montagemcarga_dados.get('pedido_item_ids', [])
                    peso_total -= self._wizmap_calcular_peso_carga_perigoso(
                        cr, uid, pedido_item_ids, context)
                    volume_total -= self._wizmap_calcular_volume_carga_perigoso(
                        cr, uid, pedido_item_ids, context)

                # Utiliza a quantidade de paletes do veiculo para calculo do peso e volume do
                # palete padrao
                palete_padrao_peso = peso_total / qtde_paletes_veiculo
                palete_padrao_volume = volume_total / qtde_paletes_veiculo

            # Ordena os clientes baseado na sequencia de entrega da rota
            args = [('wmsmcc_montagemcarga_id', '=', carga_id), ]
            cliente_ids = obj_wms_montagemcarga_clientes.search(
                cr, uid, args, order='wmsmcc_sequencia_entrega desc', context=context)
            fields = ['wmsmcc_pessoa_id']
            wms_montagemcarga_clientes_dados = obj_wms_montagemcarga_clientes.read(
                cr, uid, cliente_ids, fields, context, load='_classic_write')
            cliente_ids = map(lambda p: p.get('wmsmcc_pessoa_id'), wms_montagemcarga_clientes_dados)

            # Verifica se a relacao de cliente x empresa esta correta
            if not cliente_ids:
                msg = _('A separacao nao pode ser efetuada. Nao ha cliente cadastrado. Verifique '
                        'a correlacao Clientes x Empresa.')
                raise osv.except_osv(_('Atencao!'), msg)

            palete = 1  # Palete inicial
            total_paletes = 0  # Quantidade total de paletes
            et = caixa_et = 1  # Valores iniciais de ET e caixa para produtos fracionados
            separacao_itens = []  # Itens de separacao

            # Pedidos (consolidads por cliente)
            for cliente_id in cliente_ids:
                args = [('pessoa_id', '=', cliente_id)]
                pedido_capa_ids = obj_estoquepedidos_pedido_capa.search(
                    cr, uid, args, context=context)

                # Ordena os itens de pedido do cliente pelo endereco de armazenagem
                pedido_item_ids = wms_montagemcarga_dados.get('pedido_item_ids', [])
                args = [('id', 'in', pedido_item_ids), ('pedido_id', 'in', pedido_capa_ids)]
                pedido_item_ids = obj_estoquepedidos_pedido_item.search(
                    cr, uid, args, order='pedite_endereco', context=context)

                # Filtra os itens nao perigosos se a opcao "Palete extra para produtos perigosos"
                # estiver marcada e consolida os pedidos
                if palete_extra:
                    pedido_item_dados = filter(lambda i: not i.get('grade').get('perigoso'),
                                               self._wizmap_consolidar_pedidos(
                                                   cr, uid, pedido_item_ids, context))
                else:
                    pedido_item_dados = self._wizmap_consolidar_pedidos(
                        cr, uid, pedido_item_ids, context)

                # Separa os produtos pulmao/box
                pedido_dados = self._wizmap_separar_produto_pulmao_box(
                    cr, uid, carga_id, pedido_item_dados, palete, context)
                palete = pedido_dados.get('palete')
                total_paletes += len(pedido_dados.get('separacao_itens'))

                # Itens de separacao (pulmao box)
                separacao_itens.extend(pedido_dados.get('separacao_itens'))

                # Ajusta a carga do cliente (peso e volume)
                peso = sum(map(lambda i: i.get('peso'), pedido_dados.get('pedido_itens')))
                volume = sum(map(lambda i: i.get('volume'), pedido_dados.get('pedido_itens')))

                # Quantidade de paletes utilizado pelo cliente
                if veiculo:
                    paletes_por_peso = round(peso / palete_padrao_peso, 3)
                    paletes_por_volume = round(volume / palete_padrao_volume, 6)
                else:
                    paletes_por_peso = ceil(peso / palete_padrao_peso)
                    paletes_por_volume = ceil(volume / palete_padrao_volume)

                if paletes_por_peso > paletes_por_volume:
                    qtde_paletes = paletes_por_peso
                else:
                    qtde_paletes = paletes_por_volume

                # Se for o ultimo cliente e a quantidade total de paletes for impar, um novo
                # palete deve ser acrescentado
                if not veiculo and cliente_id == cliente_ids[len(cliente_ids) - 1]:
                    if (total_paletes + qtde_paletes) % 2:
                        qtde_paletes += 1

                total_paletes += qtde_paletes

                # Verifica se tudo ja foi separado
                if not qtde_paletes:
                    continue  # Proximo cliente

                # Palete do cliente
                peso_palete = peso / qtde_paletes
                volume_palete = volume / qtde_paletes

                # Caixa fechada (embalagem master). Itens nao fracionados e reducoes
                for pedido_item_dados in pedido_dados.get('pedido_itens'):
                    grade_dados = pedido_item_dados.get('grade')
                    embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                    qtde_na_expedicao = embalagem_expedicao_dados.get('qtde_na_embalagem')
                    embalagem_master_dados = grade_dados.get('embalagem_master')
                    qtde_na_master = embalagem_master_dados.get('qtde_na_embalagem')
                    embalagem_master_id = embalagem_master_dados.get('id')
                    qtde_master = embalagem_master_dados.get('quantidade')
                    qtde_expedicao = embalagem_expedicao_dados.get('quantidade')
                    constante = qtde_na_master / qtde_na_expedicao
                    fracionado = grade_dados.get('fracionado')

                    # Peso e volume da master em funcao da embalagem de expedicao
                    peso_master = embalagem_expedicao_dados.get('peso') * constante
                    volume_master = embalagem_expedicao_dados.get('volume') * constante

                    # Endereco de picking
                    endereco_produto_id = pedido_item_dados.get('produto').get('endereco_id')

                    # Nao embalado e reducao
                    if not qtde_master and not fracionado and qtde_expedicao:  # Trata as Reducoes
                        # Se houver quantidade de embalagens de expedicao mas o produto nao for
                        # fracionado, o item deve ser tratado como reducao. Tudo deve ir para o
                        # palete e nao para o ´´checkout``. Os dados da master tornam-se os
                        # dados da expedicao
                        grade_dados = pedido_item_dados.get('grade')
                        embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                        embalagem_master_id = embalagem_expedicao_dados.get('id')
                        peso_master = embalagem_expedicao_dados.get('peso')
                        volume_master = embalagem_expedicao_dados.get('volume')
                        qtde_na_master = embalagem_expedicao_dados.get('qtde_na_embalagem')
                        qtde_master = embalagem_expedicao_dados.get('quantidade')
                        embalagem_expedicao_dados['quantidade'] = 0
                        reducao = True
                    else:
                        reducao = False

                    while qtde_master and not grade_dados.get('embalado'):
                        qtde_por_peso = ceil(peso_palete / peso_master)
                        qtde_por_volume = ceil(volume_palete / volume_master)

                        if qtde_master < qtde_por_volume and qtde_master < qtde_por_peso:
                            quantidade = qtde_master  # Cabe toda a quantidade no palete
                        else:
                            if qtde_por_peso < qtde_por_volume:
                                quantidade = qtde_master if qtde_por_peso < 1 else qtde_por_peso
                            else:
                                quantidade = qtde_master if qtde_por_volume < 1 else qtde_por_volume

                        # Cria os itens de separacao
                        separacao_itens.append({
                            'pessoa_id': pedido_item_dados.get('pessoa_id'),  # Cliente
                            'produto_id': pedido_item_dados.get('produto').get('id'),
                            'grade_id': pedido_item_dados.get('grade').get('id'),
                            'palete': '%03d' % palete,
                            'produto_embalagem_id': embalagem_master_id,
                            'quantidade': quantidade,
                            'endereco_id': endereco_produto_id,  # Endereco de picking do produto
                            'produto_tipo': pedido_item_dados.get('produto').get('tipo'),
                            'wmssit_carga_id': carga_id,
                            'pedido_id': pedido_item_dados.get('pedido').get('id')
                        })

                        # Prepara ressuprimento
                        qtde_unidade = quantidade * qtde_na_master
                        if endereco_produto_id not in ressuprimento_picking:
                            ressuprimento_picking[endereco_produto_id] = qtde_unidade
                        else:
                            ressuprimento_picking[endereco_produto_id] += qtde_unidade

                        # Ajusta valores do palete e da carga do cliente (peso e volume)
                        qtde_master -= quantidade
                        peso_palete -= quantidade * peso_master
                        peso -= quantidade * peso_master
                        volume_palete -= quantidade * volume_master
                        volume -= quantidade * volume_master

                        if peso_palete <= 0 or volume_palete <= 0:
                            # Controle da quantidade de paletes restante
                            qtde_paletes -= 1  # Cliente
                            qtde_paletes_veiculo -= 1  # Veiculo

                            if qtde_paletes > 0:
                                # Novo palete com valores de peso e volume ajustados
                                palete += 1
                                peso_palete = peso / qtde_paletes
                                volume_palete = volume / qtde_paletes
                        elif not reducao and not fracionado and qtde_expedicao:  # Trata as Reducoes
                            # Se houver quantidade de embalagens de expedicao mas o produto nao for
                            # fracionado, o item deve ser tratado como reducao. Tudo deve ir para o
                            # palete e nao para o ´´checkout``. Os dados da master tornam-se os
                            # dados da expedicao
                            grade_dados = pedido_item_dados.get('grade')
                            embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                            embalagem_master_id = embalagem_expedicao_dados.get('id')
                            peso_master = embalagem_expedicao_dados.get('peso')
                            volume_master = embalagem_expedicao_dados.get('volume')
                            qtde_na_master = embalagem_expedicao_dados.get('qtde_na_embalagem')
                            qtde_master = embalagem_expedicao_dados.get('quantidade')
                            embalagem_expedicao_dados['quantidade'] = 0
                            reducao = True

                # Caixa de ET (fracionado)
                volume_caixa_et = caixa_fracionado_volume
                usou_caixa = False

                # Tratamento dos itens fracionados e embalados
                for pedido_item_dados in pedido_dados.get('pedido_itens'):
                    grade_dados = pedido_item_dados.get('grade')
                    embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')

                    # Se for embalado, os dados da expedicao tornam-se os dados da master e Tudo
                    # deve ir para ´´checkout``. Oposto de reduzido.
                    if grade_dados.get('embalado'):
                        embalagem_expedicao_dados = grade_dados.get('embalagem_master')

                    embalagem_expedicao_id = embalagem_expedicao_dados.get('id')
                    qtde_na_expedicao = embalagem_expedicao_dados.get('qtde_na_embalagem')
                    qtde_expedicao = embalagem_expedicao_dados.get('quantidade')
                    volume_expedicao = embalagem_expedicao_dados.get('volume')

                    if not qtde_expedicao:
                        continue

                    # Verifica se o volume de uma unidade supera o volume de uma caixa do ET
                    if grade_dados.get('fracionado') or grade_dados.get('embalado'):
                        if volume_expedicao > caixa_fracionado_volume:
                            produto = pedido_item_dados.get('produto').get('descricao')
                            msg = _('Separacao nao pode ser realizada. Volume de uma fracao do '
                                    'item %s supera o volume da caixa do E.T.')
                            raise osv.except_osv(_('Atencao!'), msg % produto)

                    # Distribui as fracoes no ultimo palete do cliente
                    while qtde_expedicao:
                        usou_caixa = True
                        qtde_por_volume = int(volume_caixa_et / volume_expedicao)

                        if not qtde_por_volume:  # volume insuficiente
                            caixa_et += 1

                            if caixa_et > numero_caixas_et:
                                et += 1
                                caixa_et = 1

                            volume_caixa_et = caixa_fracionado_volume  # Volume padrao da caixa
                            continue

                        if qtde_por_volume > qtde_expedicao:
                            quantidade = qtde_expedicao  # Cabe tudo na caixa
                        else:
                            quantidade = qtde_por_volume

                        # Cria os itens de separacao
                        separacao_itens.append({
                            'pessoa_id': pedido_item_dados.get('pessoa_id'),  # Cliente
                            'produto_id': pedido_item_dados.get('produto').get('id'),
                            'grade_id': pedido_item_dados.get('grade').get('id'),
                            'palete': '%03d' % palete,
                            'et': '%03d' % et,
                            'caixa': '%03d' % caixa_et,
                            'produto_embalagem_id': embalagem_expedicao_id,
                            'quantidade': quantidade,
                            'endereco_id': pedido_item_dados.get('produto').get('endereco_id'),
                            'produto_tipo': pedido_item_dados.get('produto').get('tipo'),
                            'wmssit_carga_id': carga_id,
                            'pedido_id': pedido_item_dados.get('pedido').get('id')
                        })

                        # Prepara ressuprimento
                        qtde_unidade = quantidade * qtde_na_expedicao
                        if endereco_produto_id not in ressuprimento_picking:
                            ressuprimento_picking[endereco_produto_id] = qtde_unidade
                        else:
                            ressuprimento_picking[endereco_produto_id] += qtde_unidade

                        # Ajusta o volume da caixa
                        qtde_expedicao -= quantidade
                        volume_caixa_et -= quantidade * volume_expedicao

                palete += 1  # Palete do proximo cliente ou dos produtos perigosos

                # Caixas diferentes para clientes diferentes
                if usou_caixa:
                    caixa_et += 1

                    if caixa_et > numero_caixas_et:
                        et += 1
                        caixa_et = 1

            # Trata os produtos perigosos se a opcao "Palete extra para produtos perigosos" estiver
            # marcada
            if adm_utils_dados.get('palete_extra'):
                peso_palete = adm_utils_dados.get('peso_palete') or 1000.0
                volume_palete = adm_utils_dados.get('volume_palete') or 1.92

                for cliente_id in cliente_ids:
                    args = [('pessoa_id', '=', cliente_id)]
                    pedido_capa_ids = obj_estoquepedidos_pedido_capa.search(
                        cr, uid, args, context=context)

                    # Ordena os itens de pedido do cliente pelo endereco de armazenagem
                    pedido_item_ids = wms_montagemcarga_dados.get('pedido_item_ids', [])
                    args = [('id', 'in', pedido_item_ids), ('pedido_id', 'in', pedido_capa_ids)]
                    pedido_item_ids = obj_estoquepedidos_pedido_item.search(
                        cr, uid, args, order='pedite_endereco', context=context)

                    # Consolida item perigoso
                    pedido_item_dados = filter(lambda i: i.get('grade').get('perigoso'),
                                               self._wizmap_consolidar_pedidos(
                                                   cr, uid, pedido_item_ids, context))

                    # Separa os produtos pulmao/box
                    pedido_dados = self._wizmap_separar_produto_pulmao_box(
                        cr, uid, carga_id, pedido_item_dados, palete, context)
                    palete = pedido_dados.get('palete')
                    total_paletes += len(pedido_dados.get('separacao_itens'))

                    # Itens de separacao (pulmao box)
                    separacao_itens.extend(pedido_dados.get('separacao_itens'))

                    # Ajusta a carga (peso e volume)
                    peso = sum(map(lambda i: i.get('peso'), pedido_dados.get('pedido_itens')))
                    volume = sum(map(lambda i: i.get('volume'), pedido_dados.get('pedido_itens')))

                    # Caixa fechada (embalagem master). Itens nao fracionados e reducoes
                    for pedido_item_dados in pedido_dados.get('pedido_itens'):
                        grade_dados = pedido_item_dados.get('grade')
                        embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                        qtde_na_expedicao = embalagem_expedicao_dados.get('qtde_na_embalagem')
                        embalagem_master_dados = grade_dados.get('embalagem_master')
                        qtde_na_master = embalagem_master_dados.get('qtde_na_embalagem')
                        embalagem_master_id = embalagem_master_dados.get('id')
                        qtde_master = embalagem_master_dados.get('quantidade')
                        qtde_expedicao = embalagem_expedicao_dados.get('quantidade')
                        constante = qtde_na_master / qtde_na_expedicao
                        fracionado = grade_dados.get('fracionado')

                        # Peso e volume da master em funcao da embalagem de expedicao
                        peso_master = embalagem_expedicao_dados.get('peso') * constante
                        volume_master = embalagem_expedicao_dados.get('volume') * constante

                        # Endereco de picking
                        endereco_produto_id = pedido_item_dados.get('produto').get('endereco_id')

                        # Nao embalado e reducao
                        if not qtde_master and not fracionado and qtde_expedicao:
                            # Trata as Reducoes
                            # Se houver quantidade de embalagens de expedicao mas o produto nao for
                            # fracionado, o item deve ser tratado como reducao. Tudo deve ir para o
                            # palete e nao para o ´´checkout``. Os dados da master tornam-se os
                            # dados da expedicao
                            grade_dados = pedido_item_dados.get('grade')
                            embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                            embalagem_master_id = embalagem_expedicao_dados.get('id')
                            peso_master = embalagem_expedicao_dados.get('peso')
                            volume_master = embalagem_expedicao_dados.get('volume')
                            qtde_na_master = embalagem_expedicao_dados.get('qtde_na_embalagem')
                            qtde_master = embalagem_expedicao_dados.get('quantidade')
                            embalagem_expedicao_dados['quantidade'] = 0
                            reducao = True
                        else:
                            reducao = False

                        while qtde_master and not grade_dados.get('embalado'):
                            qtde_por_peso = ceil(peso_palete / peso_master)
                            qtde_por_volume = ceil(volume_palete / volume_master)

                            if qtde_master < qtde_por_volume and qtde_master < qtde_por_peso:
                                quantidade = qtde_master  # Cabe toda a quantidade no palete
                            else:
                                if qtde_por_peso < qtde_por_volume:
                                    quantidade = qtde_master if qtde_por_peso < 1 else qtde_por_peso
                                else:
                                    quantidade = qtde_master if qtde_por_volume < 1 else qtde_por_volume

                            # Cria os itens de separacao
                            separacao_itens.append({
                                'pessoa_id': pedido_item_dados.get('pessoa_id'),  # Cliente
                                'produto_id': pedido_item_dados.get('produto').get('id'),
                                'grade_id': pedido_item_dados.get('grade').get('id'),
                                'palete': '%03d' % palete,
                                'produto_embalagem_id': embalagem_master_id,
                                'quantidade': quantidade,
                                'endereco_id': endereco_produto_id,  # Endereco de picking
                                'produto_tipo': pedido_item_dados.get('produto').get('tipo'),
                                'wmssit_carga_id': carga_id,
                                'pedido_id': pedido_item_dados.get('pedido').get('id')
                            })

                            # Prepara ressuprimento
                            qtde_unidade = quantidade * qtde_na_master
                            if endereco_produto_id not in ressuprimento_picking:
                                ressuprimento_picking[endereco_produto_id] = qtde_unidade
                            else:
                                ressuprimento_picking[endereco_produto_id] += qtde_unidade

                            # Ajusta valores do palete e da carga do cliente (peso e volume)
                            qtde_master -= quantidade
                            peso_palete -= quantidade * peso_master
                            peso -= quantidade * peso_master
                            volume_palete -= quantidade * volume_master
                            volume -= quantidade * volume_master

                            if peso_palete <= 0 or volume_palete <= 0:
                                # Novo palete com valores de peso e volume ajustados
                                palete += 1
                                total_paletes += 1
                                peso_palete = adm_utils_dados.get('peso_palete') or 1000.0
                                volume_palete = adm_utils_dados.get('volume_palete') or 1.92
                            elif not reducao and not fracionado and qtde_expedicao:
                                # Trata as reducoes
                                # Se houver quantidade de embalagens de expedicao mas o produto nao
                                # for fracionado, o item deve ser tratado como reducao. Tudo deve
                                # ir para o palete e nao para o ´´checkout``. Os dados da master
                                # tornam-se os dados da expedicao
                                grade_dados = pedido_item_dados.get('grade')
                                embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')
                                embalagem_master_id = embalagem_expedicao_dados.get('id')
                                peso_master = embalagem_expedicao_dados.get('peso')
                                volume_master = embalagem_expedicao_dados.get('volume')
                                qtde_na_master = embalagem_expedicao_dados.get('qtde_na_embalagem')
                                qtde_master = embalagem_expedicao_dados.get('quantidade')
                                embalagem_expedicao_dados['quantidade'] = 0
                                reducao = True

                    # Caixa de ET (fracionado)
                    volume_caixa_et = caixa_fracionado_volume
                    usou_caixa = False

                    # Tratamento dos itens fracionados e embalados
                    for pedido_item_dados in pedido_dados.get('pedido_itens'):
                        grade_dados = pedido_item_dados.get('grade')
                        embalagem_expedicao_dados = grade_dados.get('embalagem_expedicao')

                        # Se for embalado, os dados da expedicao tornam-se os dados da master e Tudo
                        # deve ir para ´´checkout``. Oposto de reduzido.
                        if grade_dados.get('embalado'):
                            embalagem_expedicao_dados = grade_dados.get('embalagem_master')

                        embalagem_expedicao_id = embalagem_expedicao_dados.get('id')
                        qtde_expedicao = embalagem_expedicao_dados.get('quantidade')
                        volume_expedicao = embalagem_expedicao_dados.get('volume')

                        if not qtde_expedicao:
                            continue

                        # Verifica se o volume de uma unidade supera o volume de uma caixa do ET
                        if grade_dados.get('fracionado') or grade_dados.get('embalado'):
                            if volume_expedicao > caixa_fracionado_volume:
                                produto = pedido_item_dados.get('produto').get('descricao')
                                msg = _('Separacao nao pode ser realizada. Volume de uma fracao do '
                                        'item %s supera o volume da caixa do E.T.')
                                raise osv.except_osv(_('Atencao!'), msg % produto)

                        qtde_na_expedicao = embalagem_expedicao_dados.get('qtde_na_embalagem')

                        while qtde_expedicao:
                            usou_caixa = True
                            qtde_por_volume = int(volume_caixa_et / volume_expedicao)

                            if not qtde_por_volume:  # volume insuficiente
                                caixa_et += 1

                                if caixa_et > numero_caixas_et:
                                    et += 1
                                    caixa_et = 1

                                volume_caixa_et = caixa_fracionado_volume  # Volume padrao da caixa
                                continue

                            if qtde_por_volume > qtde_expedicao:
                                quantidade = qtde_expedicao  # Cabe tudo na caixa
                            else:
                                quantidade = qtde_por_volume

                            # Cria os itens de separacao
                            separacao_itens.append({
                                'pessoa_id': pedido_item_dados.get('pessoa_id'),  # Cliente
                                'produto_id': pedido_item_dados.get('produto').get('id'),
                                'grade_id': pedido_item_dados.get('grade').get('id'),
                                'palete': '%03d' % palete,
                                'et': '%03d' % et,
                                'caixa': '%03d' % caixa_et,
                                'produto_embalagem_id': embalagem_expedicao_id,
                                'quantidade': quantidade,
                                'endereco_id': pedido_item_dados.get('produto').get('endereco_id'),
                                'produto_tipo': pedido_item_dados.get('produto').get('tipo'),
                                'wmssit_carga_id': carga_id,
                                'pedido_id': pedido_item_dados.get('pedido').get('id')
                            })

                            # Prepara ressuprimento
                            qtde_unidade = quantidade * qtde_na_expedicao
                            if endereco_produto_id not in ressuprimento_picking:
                                ressuprimento_picking[endereco_produto_id] = qtde_unidade
                            else:
                                ressuprimento_picking[endereco_produto_id] += qtde_unidade

                            # Ajusta valores do palete e da carga do cliente (quantidade e volume)
                            qtde_expedicao -= quantidade
                            volume_caixa_et -= quantidade * volume_expedicao
                            volume_palete -= quantidade * volume_expedicao

                            if volume_palete <= 0:
                                # Novo palete com valores de peso e volume ajustados
                                palete += 1
                                total_paletes + 1
                                peso_palete = adm_utils_dados.get('peso_palete') or 1000.0
                                volume_palete = adm_utils_dados.get('volume_palete') or 1.92

                    # Caixas diferentes para clientes diferentes
                    if usou_caixa:
                        caixa_et += 1

                        if caixa_et > numero_caixas_et:
                            et += 1
                            caixa_et = 1

            # Ordens de servico de separacao
            separacao_os = self._wizmap_criar_ordem_servico_separacao(
                cr, uid, wms_montagemcarga_dados, separacao_itens, False, context)

            # Ordens de servico de movimentacao - ressuprimento do picking
            movimentacao_os = self._wizmap_criar_ordem_servico_movimentacao(
                cr, uid, carga_id, ressuprimento_picking, context)

            # Cria todas as ordens de servico e atualiza o ´´status`` da montagem de carga
            result = obj_wms_montagemcarga.write(
                cr, uid, carga_id, {
                    'separacao_os_ids': separacao_os.get('separacao_os_ids'),
                    'wmsmcg_et_ids': separacao_os.get('wmsmcg_et_ids'),
                    'movimentacao_os_ids': movimentacao_os.get('movimentacao_os_ids'),
                    'state': 'separacao'  # (´´Em separacao``)
                }, context)

            # Atualiza os itens de separacao
            cr.execute("UPDATE wms_separacao_itens         AS si "
                       "SET    wmssit_et_id = sfe.id "
                       "FROM   wms_separacao_fracionado_et AS sfe "
                       "WHERE  si.wmssit_carga_id = sfe.wmssfe_montagemcarga_id  "
                       "  AND  si.et = sfe.wmssfe_numero_et "
                       "  AND  sfe.wmssfe_montagemcarga_id = %s;" % carga_id)

        return result

    def _wizmap_calcular_volume_carga_perigoso(self, cr, uid, item_ids, context=None):
        """
        Calcula o volume da carga somente de produtos marcados como perigoso.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param item_ids: ID ou lista dos IDs dos itens de pedido que formam a carga
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type item_ids: int|list[int|long]
        :type context: dict
        :return: somatorio do peso dos itens marcados como perigoso.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')

        if item_ids and isinstance(item_ids, (long, int)):
            item_ids = [item_ids]

        brw_estoquepedidos_pedido_item = filter(lambda f: f.grade_id.perigoso,
                                                obj_estoquepedidos_pedido_item.browse(
                                                    cr, uid, item_ids, context))

        # Somatorio do volume dos itens perigosos
        result = sum(map(lambda m:  m.pedite_volume, brw_estoquepedidos_pedido_item))

        return result

    def _wizmap_calcular_peso_carga_perigoso(self, cr, uid, item_ids, context=None):
        """
        Calcula o peso da carga somente de produtos marcados como perigoso.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param item_ids: ID ou lista dos IDs dos itens de pedido que formam a carga
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type item_ids: int|list[int|long]
        :type context: dict
        :return: somatorio do peso dos itens marcados como perigoso.
        :rtype: float

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')

        if item_ids and isinstance(item_ids, (long, int)):
            item_ids = [item_ids]

        brw_estoquepedidos_pedido_item = filter(lambda f: f.grade_id.perigoso,
                                                obj_estoquepedidos_pedido_item.browse(
                                                    cr, uid, item_ids, context))

        # Somatorio do peso dos itens perigosos
        result = sum(map(lambda m:  m.pedite_peso, brw_estoquepedidos_pedido_item))

        return result

    def _wizmap_criar_ordem_servico_separacao(self, cr, uid, carga, itens, rua, context=None):
        """
        Cria as ordens de sevico de separacao em funcao dos itens de separacao.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param carga: dados da montagem de carga
        :param itens: lista de dicionarios com os dados dos itens de separacao
        :param rua: verdadeiro se a separacao for por rua
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type carga: dict
        :type itens: list
        :type rua: bool
        :type context: dict
        :return: dicionario com os dados das ordens de servivo de separacao
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_tipo = self.pool.get('os.tipo')

        separacao_os_ids = []  # Ordens de servico de separacao
        lista_et = []  # Lista dos elementos de transporte (ET)
        key = et = False

        # Tipo das ordens de servico de separacao
        if rua:
            args = [('codigo', '=', 'SPR')]
            descricao_separacao = _('Separacao paletizada por rua - %s')
        else:
            args = [('codigo', '=', 'SPC')]
            descricao_separacao = _('Separacao paletizada por carga - %s')

        os_tipo_id = obj_os_tipo.search(cr, uid, args, context=context)

        if os_tipo_id and isinstance(os_tipo_id, list):
            os_tipo_id = os_tipo_id[0]

        for item in sorted(itens, key=lambda k: (k.get('palete'), k.get('et'), k.get('caixa'))):
            if key != [item.get('palete'), item.get('et'), item.get('caixa')]:
                key = [item.get('palete'), item.get('et'), item.get('caixa')]
                descricao = descricao_separacao % item.get('produto_tipo').get('descricao')
                item['produto_tipo_id'] = item.get('produto_tipo').get('id')
                del item['produto_tipo']

                if item.get('caixa'):
                    descricao += ' / Fracionado'

                dados = {
                    'descricao': descricao,
                    'montagemcarga_id': carga.get('id'),
                    'tipo_id': os_tipo_id,
                    'state': 'criada',
                    'box': carga.get('box'),
                    'data_criacao': time.strftime('%Y-%m-%d %H:%M:%S'),
                    'fracionado': True if item.get('caixa') else False
                }
                separacao_os_ids.append((0, 0, dados))
                index = len(separacao_os_ids) - 1
                separacao_os_ids[index][2]['separacao_item_ids'] = [(0, 0, item)]

                # Registro dos historicos da ordem de servico: esboco e criada
                separacao_os_ids[index][2]['historico_ids'] = [
                    (0, 0, {'acao': _('Criar ordem de servico')}),
                    (0, 0, {'acao': _('Esboco')})]
            else:
                item['produto_tipo_id'] = item.get('produto_tipo').get('id')
                del item['produto_tipo']
                index = len(separacao_os_ids) - 1
                separacao_os_ids[index][2]['separacao_item_ids'].append((0, 0, item))

            if item.get('et') and et != item.get('et'):
                et = item.get('et')
                lista_et.append((0, 0, {
                    'wmssfe_montagemcarga_id': carga.get('id'),
                    'wmssfe_numero_et': et
                }))

        result = {
            'separacao_os_ids': separacao_os_ids,
            'wmsmcg_et_ids': lista_et
        }

        return result

    def _wizmap_separar_produto_pulmao_box(self, cr, uid, carga, pedidos, palete, context=None):
        """
        Separa os paltes dos itens baseado na norma palete.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param carga: ID da montagem de carga
        :param pedidos: dicionario com os dados dos pedidos consolidados
        :param palete: numero do palete
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type carga: int
        :type pedidos: dict
        :type palete: int
        :type context: dict
        :return: dicionario com os dados dos pedidos, dados dos itens de separacao e o numero do
        proximo palete.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = {'separacao_itens': []}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        for pedido_dados in pedidos if isinstance(pedidos, list) else [pedidos]:
            # Quantidade de norma palete em funcao da quantidade pedida (master)
            grade_dados = pedido_dados.get('grade')
            norma_palete = grade_dados.get('norma_palete')
            qtde_na_master = grade_dados.get('embalagem_master').get('qtde_na_embalagem')
            qtde_norma = int(pedido_dados.get('qtde_pedida') / qtde_na_master / norma_palete)

            if not qtde_norma:
                continue

            qtde_na_expedicao = grade_dados.get('embalagem_expedicao').get('qtde_na_embalagem')
            constante = qtde_na_master / qtde_na_expedicao

            # Peso e volume para ajuste da carga em funcao da embalagem de expedicao
            peso_master = grade_dados.get('embalagem_expedicao').get('peso') * constante
            volume_master = grade_dados.get('embalagem_expedicao').get('volume') * constante

            # Pulmoes utilizados
            args = [
                ('bloqueado', '=', False),
                ('state', '=', 'O'),
                ('grade_id', '=', pedido_dados.get('grade').get('id')),
                ('deposito', '!=', 98),  # Nao considera avaria
                ('reserva_debito', '=', 0),
                ('quantidade', '=', norma_palete * qtde_na_master),
                ('tipo_nivel', '!=', 'PK')
            ]
            ordem = 'validade, quantidade, deposito, rua, predio, nivel, apartamento'
            endereco_pulmao_ids = obj_wms_endereco.search(cr, uid, args, order=ordem, context=context)

            if endereco_pulmao_ids:
                # A quantidade de pulmoes encontrado tambem pode ser inferior a quantidade de noma
                # palete. Neste caso, o restante sera subtraido do picking
                endereco_pulmao_ids = endereco_pulmao_ids[:qtde_norma]
                qtde_pulmao_box = len(endereco_pulmao_ids)

                # Cria os itens de separacao pulmao/box
                for endereco_pulmao_id in endereco_pulmao_ids:
                    result['separacao_itens'].append({
                        'wmssit_carga_id': carga,
                        'pessoa_id': pedido_dados.get('pessoa_id'),
                        'produto_id': pedido_dados.get('produto').get('id'),
                        'grade_id': pedido_dados.get('grade').get('id'),
                        'palete': '%03d' % palete,
                        'produto_embalagem_id': pedido_dados.get('grade').get(
                            'embalagem_master').get('id'),
                        'quantidade': norma_palete,
                        'endereco_id': endereco_pulmao_id,
                        'produto_tipo': pedido_dados.get('produto').get('tipo'),
                        'pedido_id': pedido_dados.get('pedido').get('id')
                    })

                    # Proximo palete
                    palete += 1

                # Ajusta os valores do pedido em funcao da separacao pulmao/box
                pedido_dados['qtde_pedida'] -= (qtde_pulmao_box * norma_palete * qtde_na_master)
                pedido_dados['peso'] -= (qtde_pulmao_box * norma_palete * peso_master)
                pedido_dados['volume'] -= (qtde_pulmao_box * norma_palete * volume_master)
                qtde_master = qtde_pulmao_box * norma_palete  # Quantidade de master no pulmao/box
                pedido_dados['grade']['embalagem_master']['quantidade'] -= qtde_master

                # Bloqueio dos pulmoes utilizados
                obj_wms_endereco.wmsend_bloquear(cr, uid, endereco_pulmao_ids, context)

        result['palete'] = palete
        result['pedido_itens'] = filter(lambda f: f.get('peso') and f.get('volume'), pedidos)

        return result

    def _wizmap_consolidar_pedidos(self, cr, uid, ids, context=None):
        """
        Consolida a quantidade o peso e o volume dos itens de pedido.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs dos itens de pedido
        :param context: um dicionario padrao para valores contextuais
        :type cr: object
        :type uid: int
        :type ids: list
        :type context: dict
        :return: lista de dicionario com os dados dos pedidos consolidados.
        :rtype: list|dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_estoquepedidos_pedido_item = self.pool.get('estoquepedidos.pedido.item')
        obj_adm_utils = self.pool.get('adm.utils')

        result = []
        pedido_item = {}

        # Parametros do estoque
        fields = ['estpar_descricao_grade']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(
            cr, uid, 'estoque', fields, context=context)

        for brw_estoquepedidos_pedido_item in obj_estoquepedidos_pedido_item.browse(
                cr, uid, ids, context):

            key = brw_estoquepedidos_pedido_item.grade_id.id
            produto_id = brw_estoquepedidos_pedido_item.produto_id.id
            pessoa_id = brw_estoquepedidos_pedido_item.pedido_id.pessoa_id.id
            embalagem = brw_estoquepedidos_pedido_item.produto_embalagem_id.embalagem_id
            qtde_na_embalagem = embalagem.quantidade_na_embalagem
            qtde_pedida = brw_estoquepedidos_pedido_item.qtde_pedida * qtde_na_embalagem
            grade = brw_estoquepedidos_pedido_item.grade_id
            embalagem_expedicao = grade.embalagem_expedicao_id
            peso_expedicao = embalagem_expedicao.peso_bruto
            volume_expedicao = embalagem_expedicao.cubagem
            qtde_na_expedicao = embalagem_expedicao.embalagem_id.quantidade_na_embalagem
            embalagem_master = grade.embalagem_master_id
            peso_master = embalagem_master.peso_bruto
            qtde_na_master = embalagem_master.embalagem_id.quantidade_na_embalagem
            volume_master = embalagem_master.cubagem
            peso = brw_estoquepedidos_pedido_item.grade_id.embalagem_expedicao_id.peso_bruto
            peso *= qtde_pedida / qtde_na_expedicao  # Peso em funcao da embalagem de expedicao
            volume = brw_estoquepedidos_pedido_item.grade_id.embalagem_expedicao_id.cubagem
            volume *= qtde_pedida / qtde_na_expedicao  # Volume em funcao da embalagem de expedicao
            fracionado = brw_estoquepedidos_pedido_item.grade_id.fracionado
            embalado = brw_estoquepedidos_pedido_item.grade_id.embalado
            perigoso = brw_estoquepedidos_pedido_item.grade_id.perigoso
            endereco_produto_id = grade.endereco_produto_id.id  # Endereco de picking
            endereco_produto_nome = grade.endereco_produto_id.name
            norma_palete = grade.norma_palete
            pedido_id = brw_estoquepedidos_pedido_item.pedido_id.id

            # descricao do produto
            codigo = brw_estoquepedidos_pedido_item.produto_id.codigo or ''

            if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                produto = grade.descricao or ''
            else:  # Produto + grade
                produto = brw_estoquepedidos_pedido_item.produto_id.name or ''

            descricao = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)

            # Sequencia do tipo de produto (gradeze, leveza, ...) utilizado na ordenacao dos itens
            seq_tipo_produto = grade.produto_tipo_id.sequence
            tipo_produto_id = grade.produto_tipo_id.id
            tipo_produto = grade.produto_tipo_id.descricao

            if key not in pedido_item:
                qtde_master = int(qtde_pedida / qtde_na_master)
                qtde_expedicao = (qtde_pedida - (qtde_master * qtde_na_master)) / qtde_na_expedicao
                pedido_item[key] = [pessoa_id, qtde_pedida, peso, volume, fracionado, embalado,
                                    perigoso, seq_tipo_produto, peso_expedicao, volume_expedicao,
                                    qtde_expedicao, peso_master, volume_master, qtde_master,
                                    endereco_produto_id, produto_id, norma_palete, qtde_na_master,
                                    qtde_na_expedicao, embalagem_master.id, embalagem_expedicao.id,
                                    descricao, tipo_produto, endereco_produto_nome, tipo_produto_id,
                                    pedido_id]
            else:  # Somatorio dos itens
                pedido_item[key][1] += qtde_pedida  # Totaliza a quantidade do item
                pedido_item[key][2] += peso  # Totaliza o peso do item
                pedido_item[key][3] += volume  # Totaliza o volume do item
                qtde_master = int(pedido_item[key][1] / qtde_na_master)
                qtde_expedicao = (pedido_item[key][1] - (qtde_master * qtde_na_master))
                qtde_expedicao /= qtde_na_expedicao
                pedido_item[key][10] = qtde_expedicao
                pedido_item[key][13] = qtde_master

        # Dicionario com os dados dos pedidos consolidados
        # Ordem: tipo de produto (grandeza, leveza, etc.), endereco
        if pedido_item:
            result = map(lambda k: {
                'pedido': {
                    'id': k[1][25]
                },
                'grade': {
                    'id': k[0],
                    'fracionado': k[1][4],
                    'embalado': k[1][5],
                    'perigoso': k[1][6],
                    'embalagem_master': {
                        'id': k[1][19],
                        'quantidade': k[1][13],
                        'qtde_na_embalagem': k[1][17],
                        'peso': k[1][11],
                        'volume': k[1][12]
                    },
                    'embalagem_expedicao': {
                        'id': k[1][20],
                        'quantidade': k[1][10],
                        'qtde_na_embalagem': k[1][18],
                        'peso': k[1][8],
                        'volume': k[1][9]
                    },
                    'norma_palete': k[1][16],
                    },
                'produto': {
                    'id': k[1][15],
                    'descricao': k[1][21],
                    'tipo': {
                        'id': k[1][24],
                        'descricao': k[1][22]
                    },
                    'endereco_id': k[1][14],
                    'endereco_nome': k[1][23]
                },
                'pessoa_id': k[1][0],
                'qtde_pedida': k[1][1],
                'peso': k[1][2],
                'volume': k[1][3]
            }, sorted(pedido_item.items(), key=lambda i: (i[1][7], i[1][23])))

        return result

    def _wizmap_validar_carga_item(self, cr, uid, ids, context):
        """
        Verifica se exitem itens com saldo menor que a quantidade solicitada.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs da montagem de carga
        :param context: um dicionario padrao para valores contextuais
        :type cr object
        :type uid int
        :type ids list
        :type context dict
        :return: verdadeiro se todos os itens da carga tiverem saldo suficiente.
        :rtype bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = True

        # ___________________________ Objects ________________________________
        obj_wms_montagemcarga_itens = self.pool.get('wms.montagemcarga.itens')

        args = [('montagemcarga_id', 'in', ids)]
        fields = ['wmsmci_quantidade', 'wmsmci_saldo']
        carga_item_ids = obj_wms_montagemcarga_itens.search(cr, uid, args, context=context)
        wms_montagemcarga_itens_dados = obj_wms_montagemcarga_itens.read(
            cr, uid, carga_item_ids, fields, context)

        if filter(lambda i: i.get('wmsmci_quantidade') > i.get('wmsmci_saldo'),
                  wms_montagemcarga_itens_dados):
            result = False

        return result

    def _wizmap_validar_carga_veiculo(self, cr, uid, ids, context):
        """
        Verifica se a quantidade de paletes a ser utilizada pela carga nao excede a quantidade de
        paletes do veiculo.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista de IDs da montagem de carga
        :param context: um dicionario padrao para valores contextuais
        :type cr object
        :type uid int
        :type ids list
        :type context dict
        :return: verdadeiro se a quantidade de paletes utilizada pela carga nao exceder a quantidade
         de paletes utilizada pelo veiculo.
        :rtype bool

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_montagemcarga = self.pool.get('wms.montagemcarga')
        obj_adm_utils = self.pool.get('adm.utils')

        # Dados da carga
        fields = ['wmsmcg_veiculo_palete', 'peso_total', 'cubagem_total']
        wms_montagemdecarga_dados = obj_wms_montagemcarga.read(cr, uid, ids, fields, context=context)

        # Parametros do WMS
        fields = ['peso_palete']
        adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', fields, context=context)
        peso_palete_padrao = adm_utils_dados.get('peso_palete')

        # Verifica se a quantidade de paletes em funcao da carga excede a quantidade de paletes do
        # veiculo
        result = map(lambda m:
                     m.get('peso_total') / peso_palete_padrao > m.get('wmsmcg_veiculo_palete'),
                     wms_montagemdecarga_dados if isinstance(wms_montagemdecarga_dados,
                                                             list) else [wms_montagemdecarga_dados])

        return not any(result)

    def _wizmap_obter_dados_separacao(self, cr, uid, ids, context):
        """
        Obtem os dados da separacao. Qual o tipo de separacao a ser realizada, se os dados do
        veiculo serao utilizados no processo de calculo da carga e quais as cargas.

        :param cr: cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: ID da separacao (wizard form)
        :param context: um dicionario padrao para valores contextuais
        :type cr object
        :type uid int
        :type ids int|list
        :type context dict
        :return: dicionario com os dados da separacao.
        :rtype dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        fields = ['wizmap_separacao_tipo', 'wizmap_calculo_veiculo']
        wms_mapaseparacao_dados = self.read(cr, uid, ids, fields, context)

        if wms_mapaseparacao_dados and isinstance(wms_mapaseparacao_dados, list):
            wms_mapaseparacao_dados = wms_mapaseparacao_dados[0]

        return wms_mapaseparacao_dados

    def _wizmap_criar_ordem_servico_movimentacao(self, cr, uid, carga_id, reposicoes, context=None):
        """
        Cria as ordens servico de movimentacao - ressuprimento de picking.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param carga_id: {int} ID da montagem de carga,
        :param reposicoes: {dict} ID e quantidade a ser reposta de cada picking,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} valores das reposicoes.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        result = {'movimentacao_os_ids': []}
        item_movimentacao_ids = []

        def quantidade_reposta(endereco, repos):
            """

            :param endereco: dados do endereco de picking
            :param repos: dados da reposicao
            :type repos: dict
            :type endereco: browse
            :return: quantidade a ser reposta (ressuprida)
            :rtype: integer
            """

            embalagem_master = endereco.grade_id.embalagem_master_id
            qtde_master = embalagem_master.embalagem_id.quantidade_na_embalagem
            ponto_reposicao = endereco.grade_id.ponto_reposicao * qtde_master
            capacidade = endereco.capacidade * qtde_master

            if (endereco.saldo - repos[endereco.id]) < ponto_reposicao:
                res = capacidade - endereco.saldo + repos[endereco.id]
            else:
                res = 0

            return res

        # Cria os itens de transferencia/movimentacao
        endereco_ids = map(lambda i: i, filter(lambda r: r, reposicoes))
        brw_wms_enderecos = [e for e in obj_wms_endereco.browse(
            cr, uid, endereco_ids, context) if quantidade_reposta(e, reposicoes)]

        for brw_wms_endereco in brw_wms_enderecos if isinstance(brw_wms_enderecos, list) \
                else [brw_wms_enderecos]:
            # Busca de pulmoes com saldo
            endereco_id = brw_wms_endereco.id
            produto_id = brw_wms_endereco.produto_id.id
            grade_id = brw_wms_endereco.grade_id.id
            pulmao_ids = self._wizmap_busca_pulmao(cr, uid, grade_id, False, context)
            wms_endereco_dados = obj_wms_endereco.read(cr, uid, pulmao_ids, ['saldo'], context)
            qtde_reposta = quantidade_reposta(brw_wms_endereco, reposicoes)

            for wms_endereco_dado in wms_endereco_dados if isinstance(wms_endereco_dados, list) \
                    else [wms_endereco_dados]:
                if wms_endereco_dado.get('saldo') < qtde_reposta:
                    quantidade = wms_endereco_dado.get('saldo')
                else:
                    quantidade = qtde_reposta

                # Cria o item de transferencia (reabastecimento)
                if quantidade > 0:
                    dados = {
                        'endereco_origem_id': wms_endereco_dado.get('id'),
                        'endereco_destino_id': endereco_id,
                        'produto_id': produto_id,
                        'grade_id': grade_id,
                        'quantidade': quantidade
                    }

                    item_movimentacao_ids.append((0, 0, dados))

                qtde_reposta -= quantidade

                # Toda a quantidade foi reposta
                if not qtde_reposta:
                    break

        # Ordem de servico de movimentacao se existirem itens de movimentacao
        if item_movimentacao_ids:
            result['movimentacao_os_ids'].append((0, 0, {
                'descricao': 'Ressuprimento picking.',
                'montagemcarga_id': carga_id,
                'state': 'criada',
                'data_criacao': time.strftime('%Y-%m-%d %H:%M:%S'),
                'transferencia_item_ids': item_movimentacao_ids
            }))

        return result

    def _wizmap_busca_pulmao(self, cr, uid, grade_id, saldo=0, context=None):
        """
        Busca por pulmoes.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param grade_id: {int} o ID da grade,
        :param saldo: {int} pulmoes com saldo igual ou maior,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} uma lista com os ID's dos Pulmoes que foram encontrados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        result = []

        # Nao contabiliza avaria
        args = [('tipo_nivel', '!=', 'PK'), ('nivel', '!=', 98), ('grade_id', '=', grade_id),
                ('state', '=', 'O'), ('bloqueado', '=', False), ('validade', '!=', False)]
        ordem = 'validade, quantidade, name'

        # Pulmoes com saldo
        pulmao_ids = obj_wms_endereco.search(cr, uid, args, order=ordem, context=context)

        campos = ['saldo']
        dados_pulmao = obj_wms_endereco.read(cr, uid, pulmao_ids, campos, context)

        if dados_pulmao and isinstance(dados_pulmao, list):
            result = map(lambda i: i.get('id'),
                         filter(lambda d: d.get('saldo') >= saldo, dados_pulmao))
        elif dados_pulmao and isinstance(dados_pulmao, dict):
            if dados_pulmao.get('saldo') >= saldo:
                result = [dados_pulmao.get('id', False)]
            else:
                result = []

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

