# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-NaoComercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from time import strftime
# noinspection PyProtectedMember
from openerp.tools.translate import _

# ´Status´ do item de transferencia
CRIADA = '1'
PENDENTE = '2'
EFETUADA = '3'


# noinspection PyPep8Naming
class wms_transferencia_itens(osv.Model):
    """
    Itens de transferencia.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.transferencia.itens'  # wmstit
    _inherit = 'wms.transferencia.itens'  # wms/transferencia/model/transferencia_itens_data

    def wmstit_onchange_quantidade(self, cr, uid, ids, quantidade, origem_id, destino_id,
                                   context=None):
        """
        Nao permite que a quantidade a ser transferida seja maior que a quantidade no endereco de
        origem e maior que a capacidade se o destino for o picking.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param quantidade: quantidade a ser transferida
        :param origem_id: ID do endereco de origem,
        :param destino_id: ID do endereco de destino,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type quantidade: float
        :type origem_id: long
        :type destino_id: long
        :type context: dict
        :return: ID do produto e da grade.
        :param quantidade quantidade a ser transferida
        :param origem_id: ID do endereco de origem,
        :param destino_id: ID do endereco de destino,
        :param context: um dicionario padrao para valores contextuais,
        :return: ID do produto e da grade.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = {}

        if not all((origem_id, destino_id, quantidade)):
            return result

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        # Se a transferencia nao for possivel, retorna 1 (um) para a quantidade
        if quantidade < 1:
            result['value'] = {'quantidade': 1}

        brw_wms_endereco_origem = obj_wms_endereco.browse(cr, uid, origem_id, context)
        tipo_nivel_origem = brw_wms_endereco_origem.tipo_nivel
        brw_wms_endereco_destino = obj_wms_endereco.browse(cr, uid, destino_id, context)
        tipo_nivel_destino = brw_wms_endereco_destino.tipo_nivel

        # Verifica se a quantidade informada excede o saldo da origem
        if quantidade > brw_wms_endereco_origem.quantidade:
            result['warning'] = {'title': _('Aviso.'),
                                 'message': _('Quantidade ajustada para nao ultrapassar o '
                                              'saldo do endereco de origem.')}
            quantidade = brw_wms_endereco_origem.quantidade

        # Verifica se a quantidade informada excede a capacidade do picking ou se a quantidade
        if tipo_nivel_destino == 'PK':
            try:  # O endereco de destino pode nao ter grade, embalagem_master ou capacidade
                grade = brw_wms_endereco_destino.grade_id
                embalagem_master = grade.embalagem_master_id.embalagem_id
                quantidade_na_embalagem = embalagem_master.quantidade_na_embalagem
                capacidade = grade.capacidade_picking * quantidade_na_embalagem
                saldo = brw_wms_endereco_destino.saldo

                if capacidade < quantidade + saldo:
                    result['warning'] = {'title': _('Aviso.'),
                                         'message': _('Quantidade ajustada para nao '
                                                      'ultrapassar a capacidade do picking.')}
                    quantidade = capacidade - saldo
            except:
                pass

            # Se a transferencia for de picking para picking a quantidade deve ser total
            if tipo_nivel_origem == tipo_nivel_destino == 'PK':
                if quantidade != brw_wms_endereco_origem.quantidade:
                    quantidade = brw_wms_endereco_origem.quantidade

                result['warning'] = {'title': _('Aviso.'),
                                     'message': _('Quantidade ajustada para total do picking de '
                                                  'origem.')}

        result['value'] = {'quantidade': quantidade}

        return result

    # noinspection PyMethodMayBeStatic
    def wmstit_onchange_grade_id(self, cr, uid, ids, context=None):
        """
        Refaz os enderecos de origem e de destino

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: valores vazios para os enderecos de origem e destino.
        :rtype: dict
        :return: valores vazios para os enderecos de origem e destino.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = dict(value=dict(endereco_origem_id=False, endereco_destino_id=False))

        return result

    # noinspection PyMethodMayBeStatic
    def wmstit_onchange_produto_id(self, cr, uid, ids, context=None):
        """
        Refaz os enderecos de origem e de destino

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: valores vazios para os enderecos de origem e destino.
        :rtype: dict
        :return: valores vazios para os enderecos de origem e destino.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = dict(value=dict(endereco_origem_id=False, endereco_destino_id=False,
                                 grade_id=False))

        return result

    def wmstit_onchange_responsavel_id(self, cr, uid, ids, data_transferencia, responsavel_id,
                                       context=None):
        """
        Altera o ´status´ do item de transferencia em funcao da data

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param data_transferencia: {date} data da efetivacao da transferencia,
        :param responsavel_id: ID do responsavel pela transferencia,
        :return: o valor do ´´status´´ alterado em funcao da data e do responsavel pela
        transferencia.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        status = self.wmstit_status_item_transferencia(data_transferencia, responsavel_id)

        result = dict(value=dict(state=status))

        return result

    def wmstit_onchange_endereco_origem(self, cr, uid, ids, tipo_movimentacao, endereco_id,
                                        context=None):
        """
        Ajusta a quantidade a ser movimentada se a movimentacao for do tipo 4: picking para picking.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param tipo_movimentacao: tipo de movimentacao,
        :param endereco_id: ID do endereco de origem,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type tipo_movimentacao: int
        :type endereco_id: long
        :type context: dict
        :return: valores com a quantidade alterada para o total da origem (picking).
        :rtype: dict
        :return: a quantidade alterada para o total da origem (picking).

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        result = {}

        if tipo_movimentacao == 4:  # Movimentacao de picking para picking
            brw_wms_endereco_origem = obj_wms_endereco.browse(cr, uid, endereco_id, context)

            result['warning'] = dict(title=_('Aviso.'),
                                     message=_('Quantidade ajustada para nao ultrapassar o saldo '
                                               'do endereco de origem.'))
            result.update(dict(value=dict(quantidade=brw_wms_endereco_origem.quantidade)))

        return result

    def wmstit_onchange_data_transferencia(self, cr, uid, ids, data_transferencia, responsavel_id,
                                           context=None):
        """
        Altera o ´status´ do item de transferencia em funcao da data

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param data_transferencia: {date} data da efetivacao da transferencia,
        :param responsavel_id: ID do responsavel pela transferencia,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: o valor do ´´status´´ alterado em funcao da data e do responsavel pela
        :return: o valor do ´´status´´ alterado em funcao da data e do responsavel pela
        transferencia.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        status = self.wmstit_status_item_transferencia(data_transferencia, responsavel_id, context)

        result = dict(value=dict(state=status))

        return result

    # noinspection PyMethodMayBeStatic
    def wmstit_status_item_transferencia(data_transferencia, responsavel_id, context=None):
        """
        Retorna o ´status´ do item de transferencia em funcao da data

        :param data_transferencia: data da efetivacao da transferencia,
        :param responsavel_id: ID do responsavel pela transferencia,
        :type data_transferencia: date
        :type responsavel_id: long
        :return: o valor do ´status´ alterado em funcao da data e do responsavel pela transferencia.
        :param responsavel_id: ID do responsavel pela transferencia,
        :return: o valor do ´´status´´ alterado em funcao da data e do responsavel pela
        transferencia.
        """

        context = context and context.copy() or {}

        if data_transferencia and responsavel_id:
            status = EFETUADA
        else:
            status = PENDENTE

        return status

    def wmstit_button_movimentar_item(self, cr, uid, ids, context=None):
        """
        Altera o status do item quanto a movimentacao da quantidade entre os enderecos.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param data_transferencia: data da efetivacao da transferencia,
        :param responsavel_id: ID do responsavel pela transferencia,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type data_transferencia: date
        :type responsavel_id: long
        :type context: dict
        :return: o valor do ´status´ alterado em funcao da data e do responsavel pela transferencia.
        :param context: um dicionario padrao para valores contextuais,
        :return: verdadeiro se a o item foi movimentado.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        agora = strftime('%Y-%m-%d %H:%M:%S')
        dados = dict(state=EFETUADA, data_transferencia=agora, responsavel_id=uid)
        result = self.write(cr, uid, ids, dados, context)

        return result

    def wmstit_movimentacao_ok(self, cr, uid, ids, context=None):
        """
        Verifica se totos os itens foram movimentados

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int|long
        :type ids: long|list[long]
        :type context: dict
        :return: verdadeiro se a o item foi movimentado.
        :rtype: bool
        :return: verdadeiro se todos os itens foram movimentados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        campos = ['state']
        dados_item_movimentacao = self.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context)

        # Verifica se existem itens pendentes de movimentacao
        item_ids = map(lambda m: m.get('id', False),
                       filter(lambda f: f.get('state') != EFETUADA, dados_item_movimentacao))

        result = not any(item_ids)

        return result

    # noinspection PyMethodMayBeStatic
    def wmstit_onchange_tipo_movimentacao(self, cr, uid, ids, context=None):
        """
        Limpa os campos de endereco (origem e destino).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: Valores dos campos de endereco como falso (sem endereco).
        :rtype: dict
        :return: Valores dos campos de endereco como falso (sem endereco).

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Movimentacao de pulmao para pulmao (padrao)
        result = dict(value=dict(endereco_origem_id=False, endereco_destino_id=False))

        return result

    # _________________________ ORM Methods __________________________________
    def create(self, cr, uid, values, context=None):
        """
        Realiza a movimentacao das quantidades.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type context: dict
        :return: verdadeiro se todos os itens foram movimentados.
        :rtype: bool
        :param values: valores do item de movimentacao,
        :param context: um dicionario padrao para valores contextuais,
        :return: verdadeiro se o item foi criado

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        endereco_origem_id = values.get('endereco_origem_id')
        endereco_destino_id = values.get('endereco_destino_id')
        produto_id = values.get('produto_id')
        grade_id = values.get('grade_id')
        quantidade = values.get('quantidade')

        # Retira a quantidade da origem
        if endereco_origem_id:
            obj_wms_endereco.wmsend_reserva_debito(cr, uid, endereco_origem_id, quantidade)

        # Acrescenta a quantidade no destino (picking)
        obj_wms_endereco.wmsend_reserva_credito(
            cr, uid, endereco_destino_id, produto_id, grade_id, quantidade)

        result = super(wms_transferencia_itens, self).create(cr, uid, values, context)

        return result

    def unlink(self, cr, uid, ids, context=None):
        """
        Desfaz a movimentacao das quantidades.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param context: um dicionario padrao para valores contextuais,
        :param ids: lista dos IDs,
        :return: verdadeiro se o item foi criado

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        fields = ['endereco_origem_id', 'endereco_destino_id', 'produto_id', 'grade_id',
                  'quantidade']
        wms_transferencia_item_dados = self.read(cr, uid, ids, fields, context)

        for wms_transferencia_item_dado in wms_transferencia_item_dados if isinstance(
                wms_transferencia_item_dados, list) else [wms_transferencia_item_dados]:
            endereco_origem_id = wms_transferencia_item_dado.get('endereco_origem_id')
            endereco_destino_id = wms_transferencia_item_dado.get('endereco_destino_id')
            produto_id = wms_transferencia_item_dado.get('produto_id')
            grade_id = wms_transferencia_item_dado.get('grade_id')
            quantidade = wms_transferencia_item_dado.get('quantidade')

            # Remove o credito no endereco de destino
            obj_wms_endereco.wmsend_reserva_credito(
                cr, uid, endereco_destino_id, produto_id, grade_id, quantidade, False, context)

            # Remove o debido no endereco de origem
            obj_wms_endereco.wmsend_reserva_debito(cr, uid, endereco_origem_id, quantidade, context)

        result = super(wms_transferencia_itens, self).unlink(cr, uid, ids, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
