# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _

# Status dos itens de transferencia
PENDENTE = '2'
CANCELADA = '9'
EFETUADA = '3'


# noinspection PyPep8Naming
class transferencia_os(osv.Model):
    """
    ordem de servico de transferencia.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'transferencia.os'  # trfosv
    _inherit = 'transferencia.os'  # wms/transferencia/model/os_data

    def osvosv_imprimir_os(self, cr, uid, ids, context=None):
        """
        Imprimir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: o relatorio da ordem de servico.
        :rtype: object
        :return: {super} o relatorio da ordem de servico.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('transferencia.os.historico')

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo == 'TRF':
                obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'imprimir', context)

                return {'type': 'ir.actions.report.xml', 'report_name': 'transferencia.os',
                        'datas': {'ids': [os.id], 'model': 'transferencia.os'}}

        return super(transferencia_os, self).osvosv_imprimir_os(cr, uid, ids, context)

    def osvosv_abrir_os(self, cr, uid, ids, context=None):
        """
        Abrir ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: status da ordem de servico aberta.
        :rtype: str
        :raise: mensagem de erro se nao existir responsavel para a ordem de servico
        :return: {super} status da ordem de servico aberta ou mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_transferencia_itens = self.pool.get('wms.transferencia.itens')
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_os_historico = self.pool.get('transferencia.os.historico')

        # Altera o ´status´ dos itens para "Pendente" (state = ´2´)
        for os in self.browse(cr, uid, ids, context):
            if os.tipo == 'TRF':
                # Se nao existir responsavel a ordem de servico nao podera ser aberta
                if not os.responsavel_ids:
                    msg = _('Informe ao menos um responsavel pela ordem de servico.')
                    raise osv.except_osv(_('Atencao!'), msg)

                endereco_ids = []  # Enderecos a serem bloqueados

                # Altera o ´status´ do item
                for item in os.transferencia_item_ids:
                    dados = {'state': PENDENTE}
                    context['transferencia'] = True

                    obj_transferencia_itens.write(cr, uid, item.id, dados, context)
                    endereco_ids.extend([item.endereco_origem_id.id, item.endereco_destino_id.id])

                # Bloqueia os enderecos
                obj_wms_endereco.wmsend_bloquear(cr, uid, endereco_ids)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'abrir',  context)

        return super(transferencia_os, self).osvosv_abrir_os(cr, uid, ids, context)

    def osvosv_cancelar_os(self, cr, uid, ids, context=None):
        """
        Cancelar ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: status da ordem de servico cancelada.
        :rtype: str
        :return: {super} status da ordem de servico cancelada ou mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_transferencia = self.pool.get('wms.transferencia.itens')
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_os_historico = self.pool.get('transferencia.os.historico')

        endereco_ids = []  # Enderecos a serem desbloqueados

        # Altera o status dos itens para "Cancelado" (state = ´9´)
        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo == 'TRF':
                for item in os.transferencia_item_ids:
                    # Reverte a movimentacao
                    origem = item.endereco_origem_id
                    destino = item.endereco_destino_id
                    produto = item.endereco_origem_id.produto_id
                    grade = item.endereco_origem_id.grade_id
                    quantidade = item.quantidade * -1  # Remove a quantidade

                    # Remove o credito no endereco de destino
                    obj_wms_endereco.wmsend_reserva_credito(
                        cr, uid, destino.id, produto.id, grade.id, quantidade, False, context)

                    # Remove o debido no endereco de origem
                    obj_wms_endereco.wmsend_reserva_debito(cr, uid, origem.id, quantidade, context)

                    dados = {'state': CANCELADA}
                    context['transferencia'] = True

                    obj_transferencia.write(cr, uid, item.id, dados, context)  # Cancela
                    endereco_ids.extend([origem.id, destino.id])

                # Desbloqueia os enderecos
                obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_ids)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'cancelar', context)

        return super(transferencia_os, self).osvosv_cancelar_os(cr, uid, ids, context)

    def osvosv_fechar_os(self, cr, uid, ids, context=None):
        """
        Fechar ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: status da ordem de servico.
        :rtype: str
        :raise: mensagem de erro se a ordem de servico nao foi fechada.
        :return: {super} status da ordem de servico fechada ou mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_transferencia_os_historico = self.pool.get('transferencia.os.historico')
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_wms_transferencia_itens = self.pool.get('wms.transferencia.itens')

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo == 'TRF':
                # Verifica se existem transferencias nao efetuadas. Se existir, nao permite
                # fechamento da OS
                args = [('state', 'not in', [CANCELADA, EFETUADA]),
                        ('transferencia_id', '=', os.id)]

                if obj_wms_transferencia_itens.search(cr, uid, args, context=context):
                    msg = _('A ordem de servico nr %s nao pode ser fechada. Existem '
                            'transferencias que ainda nao foram efetuadas.' % os.numero)

                    raise osv.except_osv(_('Atencao!'), msg)

                endereco_ids = []  # Enderecos para desbloqueio

                # Efetiva a movimentacao
                for item in os.transferencia_item_ids:
                    origem = item.endereco_origem_id
                    destino = item.endereco_destino_id
                    quantidade = item.quantidade
                    validade = origem.validade
                    movimentacao_tipo = item.wmstit_tipo_movimentacao  # ´4´ = picking para picking

                    # Libera picking de origem na troca de saldo
                    context['release'] = True if movimentacao_tipo == '4' else False

                    # Atualisa as quantidades nos enderecos
                    obj_wms_endereco.wmsend_somar_saldo(cr, uid, destino.id, quantidade, validade)
                    obj_wms_endereco.wmsend_subtrair_saldo(cr, uid, origem.id, quantidade, context)

                    # Se a transferencia for de picking para picking havera a troca dos enderecos
                    if movimentacao_tipo == '4':
                        args = [('endereco_produto_id', '=', origem.id)]
                        grade_ids = obj_estoque_produto_grade.search(cr, uid, args, context=context)

                        dados = {'endereco_produto_id': destino.id}
                        context['transferencia'] = True  # Nao verifica disponibilidade
                        obj_estoque_produto_grade.write(cr, uid, grade_ids, dados, context)

                    endereco_ids.extend([origem.id, destino.id])

                # Desbloqueia os enderecos
                obj_wms_endereco.wmsend_desbloquear(cr, uid, endereco_ids, context)

            obj_transferencia_os_historico.osvhis_registra_acao(cr, uid, os.id, 'fechar', context)

        result = super(transferencia_os, self).osvosv_fechar_os(cr, uid, ids, context)

        return result

    def osvosv_criar_os(self, cr, uid, ids, context=None):
        """
        Criar ordens de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: o ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: status da ordem de servico criada.
        :rtype: str
        :raise: mensagem de erro se a ordem de servico nao foi criada.
        :return: {super} status da ordem de servico criada ou mensagem de erro.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_os_historico = self.pool.get('transferencia.os.historico')

        for os in self.browse(cr, uid, ids if isinstance(ids, list) else [ids], context):
            if os.tipo == 'TRF':
                # Se nao existir item de transferencia a OS nao podera ser criada
                if not os.transferencia_item_ids:
                    msg = _('Informe ao menos uma transferencia a ser realizada.')

                    raise osv.except_osv(_('Atencao!'), msg)

            obj_os_historico.osvhis_registra_acao(cr, uid, os.id, 'criar', context)

        return super(transferencia_os, self).osvosv_criar_os(cr, uid, ids, context)

    def trfosv_button_movimentar_itens(self, cr, uid, ids, context=None):
        """
        Realiza a movimentacao de todos os itens da ordem de servico.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: int
        :type ids: long|list[long]
        :return: verdadeiro se os itens foram movimentados.
        :rtype: bool

        :note: Botao "Movimentar todos" da aba "Movimentacoes"
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_transferencia_itens = self.pool.get('wms.transferencia.itens')

        args = [('transferencia_id', 'in' if isinstance(ids, list) else '=', ids)]
        item_ids = obj_wms_transferencia_itens.search(cr, uid, args, context=context)
        result = obj_wms_transferencia_itens.wmstit_button_movimentar_item(
            cr, uid, item_ids, context)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
