# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv


class wms_endereco(osv.Model):
    """
    Enderecos de armazenagem. (heranca)

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.endereco'  # wmsend
    _inherit = 'wms.endereco'  # wms/control/endereco

    def wmsend_fields_selection_tipo_nivel(self, cr, uid, context=None):
        """
        Tipos de nivel de endereco de armazenamento.
        Ajusta os tipos de nivel conforme contexto (heranca).

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} valores de selecao do tipo de nivel.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = super(wms_endereco, self).wmsend_fields_selection_tipo_nivel(cr, uid, context)

        # Tipos de Nivel
        result.extend([('PK', 'Picking'), ('PU', 'Pulmao'), ('VT', 'Virtual')])

        return result

    # _________________________ ORM Methods __________________________________
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Filtra os enderecos de acordo com o tipo de movimentacao.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param args: {list} lista com os parametros da pesquisa,
        :param offset: {int} numero inicial do registro,
        :param limit: numero maximo de registros para retornar,
        :param order: {char} ordem atribuida para a sequencia de registros,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :param count: {boolean} verdadeiro para retornar apenas a quantidade de registros,
        :return: {list} lista com os dados da pesquisa.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if not args or not isinstance(args, list):
            args = []

        if context.get('movimentacao_itens'):
            endereco = context.get('movimentacao_itens')[0]
            tipo_movimentacao = context.get('movimentacao_itens')[1]
            produto_id = context.get('movimentacao_itens')[2]

            if endereco == 'origem':
                if tipo_movimentacao == 2:  # Movimentacao de picking para pulmao
                    args.append(['tipo_nivel', '=', 'PK'])
                elif tipo_movimentacao == 3:  # Movimentacao de pulmao para picking
                    args.append(['tipo_nivel', '!=', 'PK'])
                elif tipo_movimentacao == 4:  # Movimentacao de picking para picking
                    args.extend((['tipo_nivel', '=', 'PK'], ['state', '=', 'O']))
                else:  # Assume movimentacao de pulmao para pulmao (padrao)
                    args.append(['tipo_nivel', '!=', 'PK'])
            else:
                if tipo_movimentacao == 2:  # Movimentacao de picking para pulmao
                    args.extend(
                        (['tipo_nivel', '!=', 'PK'], '|', ['produto_id', '=', produto_id],
                         ['produto_id', '=', False])
                    )
                elif tipo_movimentacao == 3:  # Movimentacao de pulmao para picking
                    args.extend(
                        (['tipo_nivel', '=', 'PK'], ['produto_id', '=', produto_id])
                    )
                elif tipo_movimentacao == 4:  # Movimentacao de picking para picking
                    args.extend((['tipo_nivel', '=', 'PK'], ['state', '=', 'L']))
                else:  # Assume movimentacao de pulmao para pulmao (padrao)
                    args.extend(
                        (['tipo_nivel', '!=', 'PK'], '|', ['produto_id', '=', produto_id],
                         ['produto_id', '=', False])
                    )

        result = super(wms_endereco, self).search(
            cr, uid, args, offset, limit, order, context, count)

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
