# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.tools.translate import _


# _______________________ wms.transferencia.itens ____________________________
def wmstit_fields_function(self, cr, uid, ids, name, arg, context=None):
    """
    Manipulacao de campos. Valores de campos pre-definidos.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: o ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :param name: lista com o nome dos campos,
    :param arg: dicionario com os argumentos e valores,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores ajustados dos campos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    if isinstance(ids, (int, long)):
        ids = [ids]

    # ___________________________ Objects ________________________________
    obj_adm_utils = self.pool.get('adm.utils')

    context = context and context.copy() or {}

    result = {}

    # Contexto para mostrar produto baseado no parametro de descricao da grade
    campos = ['estpar_descricao_grade']
    adm_utils_dados = obj_adm_utils.admutl_get_parametros(cr, uid, 'estoque', campos, context=context)

    for brw_transferencia_item in self.browse(cr, uid, ids):
        try:
            if name == 'embalagem':
                value = brw_transferencia_item.endereco_origem_id.grade_id.embalagem_master_id.embalagem_id.name
            elif name == 'peso_bruto':
                value = brw_transferencia_item.endereco_origem_id.grade_id.embalagem_master_id.peso_bruto
            elif name == 'wmstit_grade':
                value = brw_transferencia_item.endereco_origem_id.grade_id.grade
            elif name == 'produto':
                codigo = brw_transferencia_item.endereco_origem_id.produto_id.codigo or ''
                if adm_utils_dados.get('estpar_descricao_grade') == 'G':  # Apenas a grade
                    produto = brw_transferencia_item.endereco_origem_id.grade_id.descricao or ''
                else:  # Produto + grade
                    produto = brw_transferencia_item.endereco_origem_id.produto_id.name or ''
                value = "%s%s%s" % (codigo, ' - ' if codigo else '', produto)
            else:
                value = None
        except Exception as e:
            print '[ERRO] - %s: Atributo %s: %s' % (__name__, name, e)
            value = None

        result[brw_transferencia_item.id] = value

    return result


def wmstit_field_selection_state(self, cr, uid, context=None):
    """
    ´´Status´´ do item de movimentacao.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: valores de selecao do ´´status´´.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    result = [('1', 'Criado'), ('2', 'Pendente'), ('3', 'Efetuado'), ('9', 'Cancelado')]

    return result


def wmstit_constraint_validar_quantidade(self, cr, uid, ids):
    """
    Validar a quantidade a ser transferida. A quantidade deve sempre ser maior que zero.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: o ID do usuario utilizado para a verificacao de perfil,
    :param ids: lista dos IDs,
    :return: {boolean} verdadeiro se a quantidade for maior que zero.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    if isinstance(ids, (int, long)):
        ids = [ids]

    result = True

    for item in self.browse(cr, uid, ids):
        if item.quantidade < 1:
            result = False

    return result


def wmstit_field_selection_tipo_movimentacao(self, cr, uid, context=None):
    """
    Tipos possiveis de movimentacao.

    :param cr: a linha atual do cursor do banco de dados,
    :param uid: ID do usuario utilizado para a verificacao de perfil,
    :param context: um dicionario padrao para valores contextuais,
    :return: lista com os tipos possiveis de movimentacao.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    context = context and context.copy() or {}

    result = [(1, _('Pulmao para pulmao')), (2, _('Picking para pulmao')),
              (3, _('Pulmao para picking')), (4, _('Picking para picking'))]

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4
