# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv, fields
from time import strftime


class transferencia_itens(osv.Model):
    """
    Itens de transferencia.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wms.transferencia.itens'  # wmstit
    _inherits = {'wms.endereco': 'endereco_origem_id'}

    from funcoes import wmstit_fields_function
    from funcoes import wmstit_constraint_validar_quantidade
    from funcoes import wmstit_field_selection_state
    from funcoes import wmstit_field_selection_tipo_movimentacao

    _columns = {
        # ___________________________ Many2One _______________________________
        'endereco_origem_id': fields.many2one('wms.endereco', 'Endereco de origem', required=True,
                                              help='Endereco de origem.',
                                              ondelete='restrict'),
        'endereco_destino_id': fields.many2one('wms.endereco', 'Endereco de destino', required=True,
                                               help='Endereco de destino.',
                                               ondelete='restrict'),
        'solicitante_id': fields.many2one('res.users', 'Solicitante',
                                          help='Usuario que solicitou a movimentacao.'),
        'responsavel_id': fields.many2one('res.users', 'Responsavel',
                                          help='Usuario responsavel pela movimentacao.'),
        'produto_id': fields.many2one('estoque.produto', 'Produto', ondelete='restrict', select=True,
                                      help='Descricao do produto.'),
        'grade_id': fields.many2one('estoque.produto.grade', 'Grade', ondelete='restrict', select=True,
                                    help='Grade do produto.'),

        # ___________________________ Integer ________________________________
        'quantidade': fields.integer('Quantidade', help='Quantidade movimentada.'),

        # ___________________________ Date ___________________________________
        'validade': fields.date('Validade'),

        # ___________________________ Datetime _______________________________
        'data_solicitacao': fields.datetime('Data da solicitacao',
                                            help='Data da solicitacao da movimentacao.'),
        'data_transferencia': fields.datetime('Data da movimentacao',
                                              help='Data da movimentacao (efetiva).'),

        # ___________________________ Function _______________________________
        'produto': fields.function(wmstit_fields_function, type='char', size=60, method=True,
                                   help=' Descricao do produto.', string='Produto'),
        'embalagem': fields.function(wmstit_fields_function, type='char', size=30, method=True,
                                     help='Embalagem utilizada na movimentacao (sempre master).',
                                     string='Embalagem'),
        'wmstit_grade': fields.function(wmstit_fields_function, type='char', size=2, method=True,
                                        string='Grade'),

        # ___________________________ Selection_______________________________
        'state': fields.selection(wmstit_field_selection_state, 'Status', size=1,
                                  help='Status da movimentacao.'),
        # ___________________________ Selection ______________________________
        'wmstit_tipo_movimentacao': fields.selection(wmstit_field_selection_tipo_movimentacao,
                                                     'Tipo de movimentacao', type='integer',
                                                     help='Tipos possiveis de movimentacao.')
    }

    _defaults = {
        'state': lambda *args: '1',
        'solicitante_id': lambda self, cr, uid, context: uid,
        'data_solicitacao': lambda *args: strftime('%Y-%m-%d %H:%M:%S'),
        'quantidade': lambda *args: 1,
        'wmstit_tipo_movimentacao': lambda *args: 1  # Movimentacao de pulmao para pulmao
    }

    _constraints = [(wmstit_constraint_validar_quantidade, 'O valor deve ser maior que zero.',
                     ['Quantidade'])]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
