# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class wizard_transferencia_automatica(osv.TransientModel):
    """
    Transferencia automatica.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.transferencia'  # wiztrf
    _inherit = 'wizard.transferencia'  # wms/transferencia/wizard/transferencia_data

    def wiztrf_button_transferencia_automatica(self, cr, uid, ids, context=None):
        """
        Transferencia automatica.
        Realiza a mudanca de item de um endereco virtual para outro nao vitual.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = self.wiztrf_transferencia(cr, uid, ids, context)

        return result

    def wiztrf_transferencia(self, cr, uid, ids, context=None):
        """
        Realiza a transferencia.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista de IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {list} lista com o ID dos enderecos que sofreram transferencia.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_transferencia_itens = self.pool.get('wms.transferencia.itens')
        obj_transferencia_os = self.pool.get('transferencia.os')

        # Enderecos virtuais ocupados
        virtual_ids = obj_wms_endereco.wmsend_virtuais_ocupados(cr, uid, context)

        if not virtual_ids:
            msg = _('Nao existem enderecos virtuais para realizar a transferencia.')

            raise osv.except_osv(_('Atencao!'), msg)

        # Enderecos livres
        livre_ids = obj_wms_endereco.wmsend_pulmao_livre(cr, uid, context)

        if not livre_ids:
            msg = _('Nao existem enderecos livres para realizar a transferencia.')

            raise osv.except_osv(_('Atencao!'), msg)

        result = []  # Lista com ID de enderecos utilizados na transferencia

        # Cria a ordem de servico
        dados = {'descricao': 'Transferencia automatica.'}
        os_id = obj_transferencia_os.create(cr, uid, dados, context)

        for brw_virtual in obj_wms_endereco.browse(cr, uid, virtual_ids):
            if not livre_ids:  # Nao ha mais enderecos livres
                break

            # Verifica o tipo de estrutura do picking do endereco
            estrutura_tipo_id = brw_virtual.grade_id.endereco_produto_id.estrutura_tipo_id.id

            # Filtra os enderecos livres de acordo com a estrutura
            args = [('id', 'in', livre_ids)]

            if estrutura_tipo_id:
                args.append(('estrutura_tipo_id', '=', estrutura_tipo_id))

            endereco_ids = obj_wms_endereco.search(cr, uid, args, context=context)
            endereco_id = endereco_ids[0] if endereco_ids else []  # Primeiro endereco livre

            # Descarta o endereco livre ja utilizado
            if endereco_id:
                livre_ids.pop(livre_ids.index(endereco_id))

                result.append(endereco_id)

                # Transferencias automaticas transformadas em movimentacao para que a ordem de
                # servico possa ser criada.
                dados_item = {'transferencia_id': os_id,
                              'endereco_origem_id': brw_virtual.id,
                              'endereco_destino_id': endereco_id,
                              'produto_id': brw_virtual.produto_id.id,
                              'grade_id': brw_virtual.grade_id.id,
                              'quantidade': brw_virtual.quantidade,
                              'validade': brw_virtual.validade}

                # Cria o item de transferencia
                obj_transferencia_itens.create(cr, uid, dados_item, context)

        # Efetiva a criacao da ordem de servico
        os_numero = '** S/N **'
        if result:
            obj_transferencia_os.osvosv_criar_os(cr, uid, os_id, context)

            campos = ['numero']
            dados_os = obj_transferencia_os.read(cr, uid, os_id, campos, context)

            if dados_os and isinstance(dados_os, list):
                dados_os = dados_os[0]

            if dados_os and isinstance(dados_os, dict):
                os_numero = dados_os.get('numero')

        msg = _('Aviso. A ordem de servico Nr. %s foi criada para a movimentacao de ' % os_numero)

        if len(result) == 1:
            msg += _('um endereco.')
        elif len(result) > 1:
            msg += _('%s enderecos.' % len(result))
        else:
            msg = _('Aviso. Nao foi possivel realizar a transferencia. O tipo de estrutura dos '
                    'enderecos nao sao compativeis.')

        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms', 'view_wizard_transferencia_form')
        result = {'view_type': 'form', 'view_mode': 'form', 'view_id': [view and view[1] or False],
                  'res_model': 'wizard.transferencia', 'type': 'ir.actions.act_window',
                  'name': _('Transferencia automatica'),  'target': 'new', 'context': context}

        return result

    # _________________________ ORM Methods __________________________________
    def default_get(self, cr, uid, fields, context=None):
        """
        Recebe o texto da mensagem.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas estruturas de
        construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return:

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(wizard_transferencia_automatica, self).default_get(cr, uid, fields, context)

        if context.get('mensagem'):
            result['wiztrf_mensagem'] = context.get('mensagem')

        return result

    def fields_get(self, cr, uid, fields=None, context=None):
        """
        Acrescenta campos extras necessarios na transferencia.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param fields: {dict} dicionario com o nome dos campos e respectivas estruturas de
        construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: novos campos criados

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(wizard_transferencia_automatica, self).fields_get(cr, uid, fields, context)
        result['wiztrf_mensagem'] = {'selectable': True, 'type': 'char', 'size': 255,
                                     'string': 'Mensagem'}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
