# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools.translate import _


class wizard_wms_consulta_produto_grade(osv.TransientModel):
    """
    Realiza consultas a produto/grade.

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.consulta.produto.grade'  # wizcpg
    _inherit = 'wizard.wms.consulta.produto.grade'  # wms/wizard/consulta_produto_grade_data

    def wizcpg_onchange_codigo(self, cr, uid, ids, codigo, context=None):
        """
        Consulta o produto pelo codigo de barras.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param codigo: {char}: codigo de barras que representa o produto
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} dicionario com os dados do produto.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        result = self.wizcpg_consulta_produto_grade(cr, uid, codigo, context)

        return result

    def wizcpg_consulta_produto_grade(self, cr, uid, codigo, context):
        """
        Consulta de produto em funcao do codigo de barras.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param codigo: {char}: codigo de barras que representa o produto
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} dicionario com os dados do produto.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        # ___________________________ Objects ________________________________
        obj_automacao_codigo_barra = self.pool.get('automacao.codigo.barra')

        args = [('codigo_barra', '=', codigo)]
        automacao_ids = obj_automacao_codigo_barra.search(cr, uid, args, context=context)

        if automacao_ids:
            automacao_id = automacao_ids[0]
            brw_automacao = obj_automacao_codigo_barra.browse(cr, uid, automacao_id, context)

            result = {'value': {
                'wizcpg_produto_id': brw_automacao.produto_id.id,
                'wizcpg_grade_id': brw_automacao.grade_id.id,
                'wizcpg_endereco_id': brw_automacao.grade_id.endereco_produto_id.id}
            }
        else:  # Nao encontrou o codigo de barras. Tenta pelo codigo interno
            result = {'value': {'wizcpg_produto_id': False, 'wizcpg_grade_id': False,
                                'wizcpg_endereco_id': False},
                      'warning': {'title': _('Aviso.'),
                                  'message': _('Nenhum produto encontrado para o codigo de barras '
                                               'informado.')}
            }

        return result

    def wizcpg_button_imprimir_etiqueta(self, cr, uid, ids, context=None):
        """
        Imprissao da etiqueta de picking.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {object} o formulario de consulta de produtos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]
        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_ir_model_data = self.pool.get('ir.model.data')

        campos = ['wizcpg_endereco_id']
        dados_produto = self.read(cr, uid, ids, campos, context, load='_classic_write')

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            endereco_id = dados_produto.get('wizcpg_endereco_id')
            obj_wms_endereco.wmsend_imprimir_etiqueta(cr, uid, endereco_id, context)

        view = obj_ir_model_data.get_object_reference(
            cr, uid, 'wms', 'view_wizard_wms_consulta_produto_grade_form')

        return {'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view and view[1] or False],
                'res_model': self._name,
                'type': 'ir.actions.act_window',
                'name': 'Consulta de produtos',
                'res_id': ids[0],
                'target': 'new',
                'context': context}

    def wizcpg_button_atualizar_produto(self, cr, uid, ids, context=None):
        """
        Atualiza dados do produto/grade. Grava o endereco de picking.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista dos IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {object} o formulario de consulta de produtos.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # ___________________________ Objects ________________________________
        obj_estoque_produto_grade = self.pool.get('estoque.produto.grade')
        obj_ir_model_data = self.pool.get('ir.model.data')

        campos = ['wizcpg_grade_id', 'wizcpg_endereco_id']
        dados_produto = self.read(cr, uid, ids, campos, context, load='_classic_write')

        if dados_produto and isinstance(dados_produto, list):
            dados_produto = dados_produto[0]

        if dados_produto and isinstance(dados_produto, dict):
            grade_id = dados_produto.get('wizcpg_grade_id')
            endereco_id = dados_produto.get('wizcpg_endereco_id')

            obj_estoque_produto_grade.write(cr, uid, grade_id, {'endereco_produto_id': endereco_id}, context)

        view = obj_ir_model_data.get_object_reference(
            cr, uid, 'wms', 'view_wizard_wms_consulta_produto_grade_form')

        return {'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view and view[1] or False],
                'res_model': self._name,
                'type': 'ir.actions.act_window',
                'name': 'Consulta de produtos',
                'res_id': ids[0],
                'target': 'new',
                'context': context}

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
