# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _
import time


# noinspection PyPep8Naming
class wizard_wms_ressuprimento_manual(osv.TransientModel):
    """
    Realiza ressuprimento preventivo do picking. Atualiza o estoque quando o saldo esta abaixo do
    ponto de reposicao.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.endereco.ressuprimento.manual'  # wizerm
    _inherit = 'wizard.wms.endereco.ressuprimento.manual'  # wms/wizard/endereco_ressuprimento_manual_data

    def wizerm_button_ressuprimento_manual(self, cr, uid, ids, context=None):
        """
        Realiza o ressuprimento. Reposicao da quantidade no picking de acordo com a capacidade.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista dos IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: formulario com mensagem do numero da ordem de servico de ressuprimento.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        os_id = False  # Ordem de servico de movimentacao
        wms_ressuprimento_manual_dados = self.read(cr, uid, ids, ['wizerm_endereco_ids'], context)

        if wms_ressuprimento_manual_dados and isinstance(wms_ressuprimento_manual_dados, list):
            wms_ressuprimento_manual_dados = wms_ressuprimento_manual_dados[0]
            endereco_ids = wms_ressuprimento_manual_dados.get('wizerm_endereco_ids')

            os_id = self.wizerm_ressuprimento_preventivo(cr, uid, endereco_ids, context)

        if os_id:
            msg = _('A ordem de servico de movimentacao nr %s foi criada.')
            msg %= time.strftime('%Y') + '-' + str(os_id).zfill(6)
        else:
            msg = _('Nenhuma movimentacao pode ser efetuada. Nao ha saldo suficiente.')

        # Por contexto passa a mensagem
        context['mensagem'] = msg

        view = self.pool.get('ir.model.data').get_object_reference(
            cr, uid, 'wms', 'view_wizard_wms_endereco_ressuprimento_manual_form')

        result = dict(view_type='form', view_mode='form', view_id=[view and view[1] or False],
                      res_model=self._name, type='ir.actions.act_window',
                      name='Ressuprimento preventivo', target='new', context=context)

        return result

    def wizerm_ressuprimento_preventivo(self, cr, uid, ids, context=None):
        """
        Realiza o ressuprimento. Cria a ordem de servico com a movimentacao necessaria para a
        reposicao da quantidade.

        :param cr: a linha atual do cursor do banco de dados
        :param uid: ID do usuario utilizado para a verificacao de perfil
        :param ids: lista com ID dos enderecos
        :param context: um dicionario padrao para valores contextuais
        :type cr: cursor
        :type uid: int
        :type ids: long|list[long]
        :type context: dict
        :return: ID da ordem de servico de movimentacao (ressuprimento).
        :rtype: long

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')
        obj_transferencia_os = self.pool.get('transferencia.os')

        item_movimentacao_ids = []  # Itens de movimentacao da ordem de servico de ressuprimento

        for brw_picking in obj_wms_endereco.browse(
                cr, uid, ids if isinstance(ids, list) else [ids], context):

            # Calculo da quantidade a ser reposta
            endereco_id = brw_picking.id
            master = brw_picking.grade_id.embalagem_master_id
            qtde_master = master.embalagem_id.quantidade_na_embalagem
            capacidade = brw_picking.capacidade * qtde_master
            saldo = brw_picking.saldo
            produto_id = brw_picking.produto_id.id
            grade_id = brw_picking.grade_id.id
            qtde_reposta = capacidade - saldo

            # Verifica se ha necessidade de reposicao no picking
            if not qtde_reposta:
                continue

            # Busca de pulmoes com saldo
            pulmao_ids = self._wizerm_busca_pulmao(cr, uid, grade_id, False, context)
            brw_pulmao = obj_wms_endereco.browse(cr, uid, pulmao_ids)

            for pulmao in brw_pulmao:
                if pulmao.saldo < qtde_reposta:
                    quantidade = pulmao.saldo
                else:
                    quantidade = qtde_reposta

                if not quantidade:
                    continue

                # Item de movimentacao (ressuprimento)
                dados = dict(endereco_origem_id=pulmao.id, endereco_destino_id=endereco_id,
                             produto_id=produto_id, grade_id=grade_id, quantidade=quantidade)

                item_movimentacao_ids.append((0, 0, dados))

                qtde_reposta -= quantidade

                # Toda a quantidade foi reposta
                if not qtde_reposta:
                    break

        # Cria a ordem de servico de movimentacao
        if item_movimentacao_ids:
            dados = dict(descricao='Ressuprimento preventivo.', state='criada',
                         data_criacao=time.strftime('%Y-%m-%d %H:%M:%S'),
                         transferencia_item_ids=item_movimentacao_ids)
            result = obj_transferencia_os.create(cr, uid, dados, context)
        else:
            result = 0

        return result

    def _wizerm_busca_pulmao(self, cr, uid, grade_id, saldo=0, context=None):
        """
        Busca por pulmoes para reabastecimento.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param grade_id: o ID da grade,
        :param saldo: pulmoes com saldo igual ou maior,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type grade_id: long
        :type saldo: float
        :type context: dict
        :return: uma lista com o ID dos pulmoes que foram encontrados.
        :rtype: list

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        result = []

        # Nao contabiliza avaria
        args = [('tipo_nivel', '!=', 'PK'), ('nivel', '!=', 98), ('grade_id', '=', grade_id),
                ('state', '=', 'O'), ('bloqueado', '=', False)]
        ordem = 'validade, quantidade, name'

        # Pulmoes com saldo
        pulmao_ids = obj_wms_endereco.search(cr, uid, args, order=ordem, context=context)

        campos = ['saldo']
        dados_pulmao = obj_wms_endereco.read(cr, uid, pulmao_ids, campos, context)

        if dados_pulmao and isinstance(dados_pulmao, list):
            result = map(lambda i: i.get('id'),
                         filter(lambda d: d.get('saldo') >= saldo, dados_pulmao))
        elif dados_pulmao and isinstance(dados_pulmao, dict):
            if dados_pulmao.get('saldo') >= saldo:
                result = [dados_pulmao.get('id', False)]
            else:
                result = []

        return result

    # _________________________ ORM Methods __________________________________
    def default_get(self, cr, uid, fields, context=None):
        """
        Manipula a mensagem a ser mostrada.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param fields: dicionario com o nome dos campos e respectivas estruturas de construcao,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: cursor
        :type uid: int
        :type fields: list[str]
        :type context: dict
        :return: novos campos criados.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(wizard_wms_ressuprimento_manual, self).default_get(cr, uid, fields, context)

        if context.get('mensagem'):
            result['wizerm_mensagem'] = 'Aviso. ' + context.get('mensagem')

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
