# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class wizard_wms_enderecofaixa(osv.TransientModel):
    """
    Manutencao de endereco de armazenamento de produtos.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.enderecofaixa'  # wizwef
    _inherit = 'wizard.wms.enderecofaixa'  # wms/transferencia/wizard/enderecofiaxa_data

    # noinspection PyMethodMayBeStatic
    def wizwef_onchange_acao(self, cr, uid, ids, acao, tipo_nivel, context=None):
        """
        Alteracao na execucao de uma acao. Qualquer mudanca do tipo de acao ira apagar a mensagem
        de resultado da acao.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param acao: Tipo da acao a ser executada,
        :param tipo_nivel: Nivel do endereco (VT = Virtual, 'PK' = Picking e 'PU' = Pulmao),
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type acao: str
        :type tipo_nivel: str
        :type context: dict
        :return: se nenhuma acao foi selecionada, o valor padrao da acao.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = {'value': {'acao': acao}}

        if tipo_nivel == 'VT' and acao in ['1', '2']:
            result['warning'] = {'title':  _('Aviso.'),
                                 'message': _('Tipo de estrutura e tipo de endereco nao sao '
                                              'utilizados por enderecos virtuais.')}
        return result

    # noinspection PyMethodMayBeStatic
    def wizwef_onchange_swap(self, cr, uid, ids, nome, inicial, final, context=None):
        """
        Troca o valor final pelo valor inicial se o valor final for menor que o inicial.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param nome: nome do campo,
        :param inicial: valor inicial,
        :param final: valor final,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type nome: str
        :type final: int
        :type context: dict
        :return: valor inicial e final ajustado.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if inicial > final:
            final = inicial

        return {
            'value': {
                '%s_inicial' % nome: inicial,
                '%s_final' % nome: final
            }
        }

    # noinspection PyMethodMayBeStatic
    def wizwef_onchange_tiponivel(self, cr, uid, ids, tipo_nivel, deposito, context=None):
        """
        Troca o valor final pelo valor inicial se o valor final for menor que inicial.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param tipo_nivel: tipo do nivel: PK=Piking, PU=Pulmao e VT=Virtual,
        :param deposito: numero do deposito,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type tipo_nivel: str
        :type deposito: int
        :type context: dict
        :return: valor inicial e final ajustado.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = {}

        if tipo_nivel == 'VT':
            values = {'deposito': 99, 'rua_inicial': 99, 'rua_final': 99, 'predio_inicial': 99,
                      'predio_final': 99, 'nivel_inicial': 99, 'nivel_final': 99,
                      'strutura_tipo_id': False, 'endereco_tipo_id': False,
                      'wmswef_estrutura_tipo_id': False, 'wmswef_endereco_tipo_id': False}

            result = {'value': values,
                      'warning': {'title':  _('Aviso.'),
                                  'message': _('Tipo de estrutura e tipo de endereco nao sao '
                                               'utilizados por enderecos virtuais.')}}

        return result

    def wizwef_button_criar_faixa(self, cr, uid, ids, context=None):
        """
        Cria uma faixa de endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Cria um ou uma faixa de endereco
        enderecos_criados = len(self.wizwef_criar_endereco(cr, uid, ids, context))

        if not enderecos_criados:  # Nenhum endereco foi criado (faixa existente)
            msg = 'Nenhum endereco foi criado.'
        elif enderecos_criados == 1:  # Apenas um endereco foi criado
            msg = 'Um novo endereco foi criado.'
        else:  # Varios enderecos criados
            msg = 'Foram criados %s novos enderecos. Os enderecos ja criados ' \
                  'pela sobreposicao da faixa nao foram alterados.' % \
                  enderecos_criados

        # Por contexto passa o numero de novos enderecos criados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms', 'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def wizwef_button_alterar_faixa(self, cr, uid, ids, context=None):
        """
        Alteracao de uma determinada faixa de endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Altera um ou uma faixa de endereco
        enderecos_alterados = len(self.wizwef_alterar_endereco(
            cr, uid, ids if isinstance(ids, list) else [ids], context))

        if not enderecos_alterados:  # Nenhum endereco alterado
            msg = 'Nenhum endereco foi alterado.'
        elif enderecos_alterados == 1:  # Apenas um endereco foi alterado
            msg = 'Um unico endereco foi alterado.'
        else:
            # Varios enderecos foram alterados
            msg = '%s enderecos foram alterados.' % enderecos_alterados

        # Por contexto passa a mensagem com o numero de enderecos alterados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms', 'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def wizwef_button_reservar_faixa(self, cr, uid, ids, context=None):
        """
        Reserva uma determinada faixa de endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Reserva uma faixa de endereco
        enderecos_reservados = len(self.wizwef_reservar_enderecos(
            cr, uid, ids if isinstance(ids, list) else [ids], context))

        if not enderecos_reservados:  # Nenhum endereco reservado
            msg = 'Nenhum endereco foi reservado.'
        elif enderecos_reservados == 1:  # Apenas um endereco foi reservado
            msg = 'Um unico endereco foi reservado.'
        else:
            # Varios enderecos foram reservados
            msg = '%s enderecos foram reservados.' % enderecos_reservados

        # Por contexto passa a mensagem com o numero de enderecos reservados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms', 'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def wizwef_button_liberar_faixa(self, cr, uid, ids, context=None):
        """
        Libera uma determinada faixa de endereco.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # Libera uma faixa de endereco
        enderecos_liberados = len(self.wizwef_liberar_faixa_enderecos(
            cr, uid, ids if isinstance(ids, list) else [ids], context))

        if not enderecos_liberados:  # Nenhum endereco liberado
            msg = 'Nenhum endereco foi liberado.'
        elif enderecos_liberados == 1:  # Apenas um endereco foi liberado
            msg = 'Um unico endereco foi liberado.'
        else:
            # Varios enderecos foram liberados
            msg = '%s enderecos foram liberados.' % enderecos_liberados

        # Por contexto passa a mensagem com o numero de enderecos liberados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms',  'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def wizwef_button_alterar_tipo_endereco(self, cr, uid, ids, context=None):
        """
        Atera o tipo de endereco para todos os enderecos da faixa informada.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        endereco_ids = self._wizwef_faixa(cr, uid, ids if isinstance(ids, list) else [ids], context)
        campos = ['wizwef_endereco_tipo_id']
        dados_faixa = self.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context,
            load='_classic_write')

        if dados_faixa and isinstance(dados_faixa, list):
            dados_faixa = dados_faixa[0]

        if dados_faixa and isinstance(dados_faixa, dict):
            endereco_tipo_id = dados_faixa.get('wizwef_endereco_tipo_id')
            obj_wms_endereco.wmsend_alterar_tipo_endereco(
                cr, uid, endereco_ids, endereco_tipo_id, context)

        if not endereco_ids:  # Nenhum endereco alterado
            msg = 'Nenhum endereco foi alterado.'
        elif len(endereco_ids) == 1:  # Apenas um endereco foi alterado
            msg = 'Um unico endereco foi alterado.'
        else:
            # Varios enderecos foram alterados
            msg = '%s enderecos foram alterados.' % len(endereco_ids)

        # Por contexto passa a mensagem com o numero de enderecos alterados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms',  'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def wizwef_button_alterar_tipo_estrutura(self, cr, uid, ids, context=None):
        """
        Atera o tipo de estrutura para todos os enderecos da faixa informada.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: o formulario de manutencao de enderecos.
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_wms_endereco = self.pool.get('wms.endereco')

        endereco_ids = self._wizwef_faixa(cr, uid, ids if isinstance(ids, list) else [ids], context)
        campos = ['wizwef_estrutura_tipo_id']
        dados_faixa = self.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context,
            load='_classic_write')

        if dados_faixa and isinstance(dados_faixa, list):
            dados_faixa = dados_faixa[0]

        if dados_faixa and isinstance(dados_faixa, dict):
            estrutura_tipo_id = dados_faixa.get('wizwef_estrutura_tipo_id')
            obj_wms_endereco.wmsend_alterar_tipo_estrutura(
                cr, uid, endereco_ids, estrutura_tipo_id, context)

        if not endereco_ids:  # Nenhum endereco alterado
            msg = 'Nenhum endereco foi alterado.'
        elif len(endereco_ids) == 1:  # Apenas um endereco foi alterado
            msg = 'Um unico endereco foi alterado.'
        else:
            # Varios enderecos foram alterados
            msg = '%s enderecos foram alterados.' % len(endereco_ids)

        # Por contexto passa a mensagem com o numero de enderecos alterados
        context['mensagem'] = msg

        model = self.pool.get('ir.model.data')
        view = model.get_object_reference(cr, uid, 'wms',  'view_wizard_wms_enderecofaixa_form')
        result = {
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [view and view[1] or False],
            'res_model': 'wizard.wms.enderecofaixa',
            'type': 'ir.actions.act_window',
            'name': 'Manutencao de enderecos',
            'target': 'new',
            'context': context
        }

        return result

    def _wizwef_faixa(self, cr, uid, ids, context=None):
        """
        Faixa de enderecos a ser utilizada.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com o ID dos enderecos correspondente a faixa.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_endereco = self.pool.get('wms.endereco')

        result = []
        campos = ['deposito', 'rua_inicial', 'rua_final', 'predio_inicial', 'predio_final', 'lado',
                  'nivel_inicial', 'nivel_final', 'tipo_nivel', 'apartamento_inicial',
                  'apartamento_final', 'picking_dinamico', 'estrutura_tipo_id', 'endereco_tipo_id',
                  'wizwef_bloqueado']

        dados_faixa = self.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context, '_classic_write')

        if dados_faixa and isinstance(dados_faixa, list):
            dados_faixa = dados_faixa[0]

        if dados_faixa and isinstance(dados_faixa, dict):
            args = [('deposito', '=', dados_faixa.get('deposito')),
                    ('rua', '>=', dados_faixa.get('rua_inicial')),
                    ('rua', '<=', dados_faixa.get('rua_final')),
                    ('predio', '>=', dados_faixa.get('predio_inicial')),
                    ('predio', '<=', dados_faixa.get('predio_final')),
                    ('nivel', '>=', dados_faixa.get('nivel_inicial')),
                    ('nivel', '<=', dados_faixa.get('nivel_final')),
                    ('apartamento', '>=', dados_faixa.get('apartamento_inicial')),
                    ('apartamento', '<=', dados_faixa.get('apartamento_final'))]

            if dados_faixa.get('estrutura_tipo_id'):
                args.append(('estrutura_tipo_id', '=', dados_faixa.get('estrutura_tipo_id')))

            if dados_faixa.get('endereco_tipo_id'):
                args.append(('endereco_tipo_id', '=', dados_faixa.get('endereco_tipo_id')))

            # Considera ou nao os enderecos bloqueados
            if not dados_faixa.get('wizwef_bloqueado'):
                args.append(('bloqueado', '=', False))

            # Enderecos da faixa
            result = obj_endereco.search(cr, uid, args, context=context)

        return result

    # TODO: Alteracao de endereco deve ser um metodo da classe wms_endereco
    def wizwef_alterar_endereco(self, cr, uid, ids, context=None):
        """
        Altera os enderecos de acordo com as faixas especificadas.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com o ID dos enderecos alterados.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_endereco = self.pool.get('wms.endereco')

        campos = ['deposito', 'rua_inicial', 'rua_final', 'predio_inicial', 'predio_final', 'lado',
                  'nivel_inicial', 'nivel_final', 'tipo_nivel', 'apartamento_inicial',
                  'apartamento_final', 'picking_dinamico', 'estrutura_tipo_id', 'endereco_tipo_id']

        dados_endereco = self.read(
            cr, uid, ids[0] if isinstance(ids, list) else ids, campos, context=context)

        args = [('deposito', '=', dados_endereco.get('deposito')),
                ('rua', '>=', dados_endereco.get('rua_inicial')),
                ('rua', '<=', dados_endereco.get('rua_final')),
                ('predio', '>=', dados_endereco.get('predio_inicial')),
                ('predio', '<=', dados_endereco.get('predio_final')),
                ('nivel', '>=', dados_endereco.get('nivel_inicial')),
                ('nivel', '<=', dados_endereco.get('nivel_final')),
                ('apartamento', '>=', dados_endereco.get('apartamento_inicial')),
                ('apartamento', '<=', dados_endereco.get('apartamento_final')),
                ('bloqueado', '=', False), ('state', '=', 'L')]

        # Procura pelos enderecos
        endereco_ids = obj_endereco.search(cr, uid, args, context=context)

        # Verifica e filtra pelo lado
        lado = dados_endereco.get('lado', False)

        if lado:
            enderecos = obj_endereco.browse(cr, uid, endereco_ids)
            endereco_ids = map(lambda e: e.id, filter(lambda e: str(2 - e.predio % 2) == lado,
                                                      enderecos))

        if endereco_ids:
            dados = {'tipo_nivel': dados_endereco.get('tipo_nivel'),
                     'estrutura_tipo_id': dados_endereco.get('estrutura_tipo_id')[0],
                     'endereco_tipo_id': dados_endereco.get('endereco_tipo_id')[0],
                     'picking_dinamico': dados_endereco.get('picking_dinamico')}

            obj_endereco.write(cr, uid, endereco_ids, dados, context)

        return endereco_ids

    # TODO: A criacao de novos enderecos deve ser um metodo da classe wms_endereco
    def wizwef_criar_endereco(self, cr, uid, ids, context=None):
        """
        Cria os enderecos de acordo com as faixas especificadas.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com o ID dos enderecos que foram criados.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_endereco = self.pool.get('wms.endereco')

        campos = ['deposito', 'rua_inicial', 'rua_final', 'predio_inicial', 'predio_final', 'lado',
                  'nivel_inicial', 'nivel_final', 'tipo_nivel', 'apartamento_inicial',
                  'apartamento_final', 'picking_dinamico', 'estrutura_tipo_id', 'endereco_tipo_id']

        dados_endereco = self.read(cr, uid, ids[0], campos, context=context)

        # Cria novos enderecos apartir de uma faixa pre-estabelecida
        deposito = dados_endereco.get('deposito')
        rua_inicial = dados_endereco.get('rua_inicial')
        rua_final = dados_endereco.get('rua_final')
        predio_inicial = dados_endereco.get('predio_inicial')
        predio_final = dados_endereco.get('predio_final')
        nivel_inicial = dados_endereco.get('nivel_inicial')
        nivel_final = dados_endereco.get('nivel_final')
        tipo_nivel = dados_endereco.get('tipo_nivel')
        apto_inicial = dados_endereco.get('apartamento_inicial')
        apto_final = dados_endereco.get('apartamento_final')

        result = []

        for rua in range(rua_inicial, rua_final + 1):
            for predio in range(predio_inicial, predio_final + 1):
                for nivel in range(nivel_inicial, nivel_final + 1):
                    for apartamento in range(apto_inicial, apto_final + 1):
                        lado = 2 - predio % 2  # Define se lado par ou impar
                        if dados_endereco.get('lado'):
                            if int(dados_endereco.get('lado')) != lado:
                                continue
                        lado = str(lado)

                        dinamico = dados_endereco.get('picking_dinamico')
                        estrutura_tipo = dados_endereco.get('estrutura_tipo_id')
                        endereco_tipo = dados_endereco.get('endereco_tipo_id')

                        # Verifica faixa virtural
                        if tipo_nivel == 'VT':
                            dinamico = False
                            estrutura_tipo = [None]
                            endereco_tipo = [None]

                        # Verifica se o endereco nao existe para criar
                        args = [('deposito', '=', deposito), ('rua', '=', rua),
                                ('predio', '=', predio), ('nivel', '=', nivel),
                                ('apartamento', '=', apartamento)]

                        if not obj_endereco.search(cr, uid, args, context=context):
                            dados = {'deposito': deposito, 'rua': rua, 'lado': lado,
                                     'predio': predio, 'nivel': nivel, 'apartamento': apartamento,
                                     'tipo_nivel': tipo_nivel, 'picking_dinamico': dinamico,
                                     'estrutura_tipo_id': estrutura_tipo[0],
                                     'endereco_tipo_id': endereco_tipo[0],
                                     }

                            context['endereco'] = True
                            endereco_id = obj_endereco.create(cr, uid, dados, context)

                            result.append(endereco_id)

        return result

    # TODO: A reserva de enderecos deve ser um metodo da classe wms_endereco
    def wizwef_reservar_enderecos(self, cr, uid, ids, context=None):
        """
        Reserva os enderecos de acordo com as faixas especificadas.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com o ID dos enderecos reservados.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_endereco = self.pool.get('wms.endereco')

        campos = ['deposito', 'rua_inicial', 'rua_final', 'predio_inicial', 'predio_final', 'lado',
                  'nivel_inicial', 'nivel_final', 'tipo_nivel', 'apartamento_inicial',
                  'apartamento_final']

        dados_endereco = self.read(cr, uid, ids[0], campos, context=context)

        args = [('deposito', '=', dados_endereco.get('deposito')),
                ('rua', '>=', dados_endereco.get('rua_inicial')),
                ('rua', '<=', dados_endereco.get('rua_final')),
                ('predio', '>=', dados_endereco.get('predio_inicial')),
                ('predio', '<=', dados_endereco.get('predio_final')),
                ('nivel', '>=', dados_endereco.get('nivel_inicial')),
                ('nivel', '<=', dados_endereco.get('nivel_final')),
                ('apartamento', '>=', dados_endereco.get('apartamento_inicial')),
                ('apartamento', '<=', dados_endereco.get('apartamento_final')),
                ('bloqueado', '=', False),
                ('state', '=', 'L')]

        # Procura pelos enderecos
        endereco_ids = obj_endereco.search(cr, uid, args, context=context)

        # Verifica e filtra pelo lado
        lado = dados_endereco.get('lado', False)

        if lado:
            enderecos = obj_endereco.browse(cr, uid, endereco_ids)
            endereco_ids = map(lambda e: e.id, filter(lambda e: str(2 - e.predio % 2) == lado,
                                                      enderecos))
        if endereco_ids:
            obj_endereco.write(cr, uid, endereco_ids, {'state': 'R'}, context)   # Reserva os enderecos

        return endereco_ids

    def wizwef_liberar_faixa_enderecos(self, cr, uid, ids, context=None):
        """
        Libera os enderecos de acordo com as faixas especificadas.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type ids: int|list[int|long]
        :type context: dict
        :return: lista com o ID dos enderecos liberados.
        :rtype: list[int|long]

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        # ___________________________ Objects ________________________________
        obj_endereco = self.pool.get('wms.endereco')

        dados_endereco = self.read(cr, uid, ids if isinstance(ids, list) else ids, context=context)

        if dados_endereco and isinstance(dados_endereco, list):
            dados_endereco = dados_endereco[0]

        args = [('deposito', '=', dados_endereco.get('deposito')),
                ('rua', '>=', dados_endereco.get('rua_inicial')),
                ('rua', '<=', dados_endereco.get('rua_final')),
                ('predio', '>=', dados_endereco.get('predio_inicial')),
                ('predio', '<=', dados_endereco.get('predio_final')),
                ('nivel', '>=', dados_endereco.get('nivel_inicial')),
                ('nivel', '<=', dados_endereco.get('nivel_final')),
                ('apartamento', '>=', dados_endereco.get('apartamento_inicial')),
                ('apartamento', '<=', dados_endereco.get('apartamento_final')),
                ('bloqueado', '=', False),
                ('state', '=', 'R')]

        # Procura pelos enderecos
        endereco_ids = obj_endereco.search(cr, uid, args, context=context)

        # Verifica e filtra pelo lado
        lado = dados_endereco.get('lado', False)

        if lado:
            enderecos = obj_endereco.browse(cr, uid, endereco_ids)
            endereco_ids = map(lambda e: e.id, filter(lambda f: f.lado == lado, enderecos))

        if endereco_ids:
            obj_endereco.wmsend_liberar(cr, uid, endereco_ids, context)  # Libera os enderecos

        return endereco_ids

    # _________________________ ORM Methods __________________________________
    def default_get(self, cr, uid, fields, context=None):
        """
        Atribui o valor padrao para o campo ´acao´. E mostra mensagem se existir.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param fields: lista com o nome dos campos das estruturas de construcao,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: object
        :type uid: int
        :type fields: list
        :type context: dict
        :return: novos campos criados.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        result = super(wizard_wms_enderecofaixa, self).default_get(cr, uid, fields, context)
        result['acao'] = 'C'

        if context.get('mensagem'):
            result['mensagem'] = 'Aviso. ' + context.get('mensagem')

        return result

    def fields_get(self, cr, uid, allfields=None, context=None, write_access=True):
        """
        Acrescenta os campos para selecao da acao a ser executada: criar, excluir, etc.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param allfields: lista com o nome dos campos das estruturas de construcao,
        :param context: um dicionario padrao para valores contextuais,
        :param write_access: nao se aplica.
        :type cr: object
        :type uid: int
        :type allfields: None|list
        :type context: dict
        :type write_access: bool
        :return: novos campos criados.
        :rtype: dict

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        result = super(wizard_wms_enderecofaixa, self).fields_get(cr, uid, allfields, context)

        # Acoes
        tipo_acao = [('C', 'Criar faixa'), ('A', 'Alterar faixa'), ('E', 'Excluir faixa'),
                     ('R', 'Reservar faixa'), ('L', 'Liberar faixa'),
                     ('1', 'Alterar tipo de estrutura'), ('2', 'Alterar tipo de endereco')]
        result['acao'] = {'selectable': True, 'type': 'selection', 'size': 1,
                          'selection': tipo_acao, 'required': True, 'string': 'Acao'}
        result['mensagem'] = {'selectable': False, 'type': 'char', 'size': 200,
                              'string': 'Mensagem'}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
