# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class wizard_wms_estoque_picking(osv.TransientModel):
    """
    Relatorio de estoque (picking).

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.estoque.picking'  # wizepk
    _inherit = 'wizard.wms.estoque.picking'  # wms/wizard/estoque_picking_data

    def wizepk_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Impressao do relatorio de estoque (picking).

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type context: dict
        :return: o relatorio de estoque (picking)
        :rtype: object

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        args = endereco_ids = []
        wms_estoque_picking_dados = self.read(cr, uid, ids, [], context)

        if wms_estoque_picking_dados and isinstance(wms_estoque_picking_dados, list):
            wms_estoque_picking_dados = wms_estoque_picking_dados[0]
            deposito = wms_estoque_picking_dados.get('wizepk_deposito') or False
            ruas = [wms_estoque_picking_dados.get('wizepk_rua_inicio') or 0,
                    wms_estoque_picking_dados.get('wizepk_rua_fim') or 0]
            ruas = list(set(sorted(ruas))) if ruas[0] else False
            predios = [wms_estoque_picking_dados.get('wizepk_predio_inicio') or 0,
                       wms_estoque_picking_dados.get('wizepk_predio_fim') or 0]
            predios = list(set(sorted(predios))) if predios[0] else False
            niveis = [wms_estoque_picking_dados.get('wizepk_nivel_inicio') or 0,
                      wms_estoque_picking_dados.get('wizepk_nivel_fim') or 0]
            niveis = list(set(sorted(niveis)))
            aptos = [wms_estoque_picking_dados.get('wizepk_apartamento_inicio') or 0,
                     wms_estoque_picking_dados.get('wizepk_apartamento_fim') or 0]
            aptos = list(set(sorted(aptos))) if aptos[0] else False
            situacao = wms_estoque_picking_dados.get('wizepk_situacao')
            reposicao = wms_estoque_picking_dados.get('wizepk_reposicao_situacao')
            bloqueado = wms_estoque_picking_dados.get('wizepk_bloqueado')
            produto = wms_estoque_picking_dados.get('wizepk_produto_id')

            # Filtros
            args += [('wmsepk_deposito', '=', deposito)] if deposito else []

            if ruas:
                if len(ruas) > 1:
                    args += [('wmsepk_rua', 'in', range(ruas[0], ruas[1] + 1))]
                else:
                    args += [('wmsepk_rua', '=', ruas[0])]

            if predios:
                if len(predios) > 1:
                    args += [('wmsepk_predio', 'in', range(predios[0], predios[1] + 1))]
                else:
                    args += [('wmsepk_predio', '=', predios[0])]

            if niveis:
                if len(niveis) > 1:
                    args += [('wmsepk_nivel', 'in', range(niveis[0], niveis[1] + 1))]
                elif niveis[0]:
                    args += [('wmsepk_nivel', '=', niveis[0])]

            if aptos:
                if len(aptos) > 1:
                    args += [('wmsepk_apartamento', 'in', range(aptos[0], aptos[1] + 1))]
                else:
                    args += [('wmsepk_apartamento', '=', aptos[0])]

            args += [('wmsepk_situacao', '=', situacao)] if situacao else []
            args += [('wmsepk_bloqueado', '=', False)] if not bloqueado else []
            args += [('wmsepk_codigo', '=', produto[1].split()[0])] if produto else []
            args += [('wmsepk_reposicao_situacao', '=', reposicao)] if reposicao else []

            endereco_ids = self.pool.get('view.wms.estoque.picking').search(
                cr, uid, args, context=context)

        if not endereco_ids:
            # Por contexto passa a mensagem
            msg = _('Nenhum registro encontrado para o filtro selecionado.')
            context['mensagem'] = msg
            model = self.pool.get('ir.model.data')
            view = model.get_object_reference(
                cr, uid, 'wms', 'view_wizard_wms_estoque_picking_form')

            result = {
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view and view[1] or False],
                'res_model': 'wizard.wms.estoque.picking',
                'type': 'ir.actions.act_window',
                'res_id': ids[0],
                'name': 'Relatorio de estoque (picking)',
                'target': 'new',
                'context': context
            }
        else:
            result = {
                'type': 'ir.actions.report.xml',
                'report_name': 'wms.estoque.picking',
                'name': 'wms_estoque_picking',
                'datas': dict(endereco_ids=endereco_ids, args=args)
            }

        return result

    # _________________________ ORM Methods __________________________________
    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Mostra mensagem se existir.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param fields: dicionario com o nome dos campos e estruturas de construcao,
        :param context: um dicionario padrao para valores contextuais,
        :type cr: Cursor
        :type uid: long
        :type ids: long|list[long]
        :type fields: list
        :type context: dict
        :return: mensagem de aviso, erro ou advertencia.
        :rtype: str

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        results = super(wizard_wms_estoque_picking, self).read(
            cr, uid, ids if isinstance(ids, list) else [ids], fields, context, load)

        if context.get('mensagem'):
            for result in results:
                result['wizepk_mensagem'] = context.get('mensagem')

        return results

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
