# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2014 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2014, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools import drop_view_if_exists
# noinspection PyProtectedMember
from openerp.tools.translate import _


# noinspection PyPep8Naming
class wizard_wms_posicao_nf(osv.TransientModel):
    """
    Formulario. Filtra periodo das notas fiscais para o relatorio de posicao de notas fiscais

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.posicao.nf'  # wizpnf
    _inherit = 'wizard.wms.posicao.nf'  # wms/wizard/posicao_nf_data

    def wizpnf_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Impressao do relatorio de posicao de notas fiscais.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param context: um dicionario padrao para valores contextuais,

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}
        where = ''
        periodo_inical = periodo_final = None
        campos = ['wizpnf_periodo_inicio', 'wizpnf_periodo_fim']
        wizard_wms_posicao_nf_dados = self.read(
            cr, uid, ids if isinstance(ids, list) else [ids], campos, context)

        if wizard_wms_posicao_nf_dados and isinstance(wizard_wms_posicao_nf_dados, list):
            wizard_wms_posicao_nf_dados = wizard_wms_posicao_nf_dados[0]

        if wizard_wms_posicao_nf_dados and isinstance(wizard_wms_posicao_nf_dados, dict):
            periodo_inical = wizard_wms_posicao_nf_dados.get('wizpnf_periodo_inicio')
            periodo_final = wizard_wms_posicao_nf_dados.get('wizpnf_periodo_fim')

        if periodo_inical:
            where += " AND fnf.data_emissao >= '%s'" % periodo_inical

        if periodo_final:
            where += " AND fnf.data_emissao <= '%s'" % periodo_final

        drop_view_if_exists(cr, 'view_wms_posicao_nf')
        cr.execute(
            """
            CREATE VIEW view_wms_posicao_nf AS (
                SELECT	        ROW_NUMBER() OVER()         AS id,
                                fnf.numero_nf               AS wmspnf_nf_numero,
                                fnf.data_emissao            AS wmspnf_nf_data,
                                fnf.vr_total_nf             AS wmspnf_nf_valor,
                                ros.id                      AS wmspnf_rec_numero,
                                ros.data_abertura           AS wmspnf_rec_data_abertura,
                                ros.data_fechamento         AS wmspnf_rec_data_fechamento
                FROM            fiscal_nf_capa              AS fnf
                    LEFT JOIN   recebimento_os              AS ros  ON fnf.recebimento_id = ros.id
                WHERE           fnf.state                   IN ('2', '3')
                    AND         fnf.tipo                    = 'E'
                    AND         fnf.fisnfc_tipo_servico	    IS FALSE
                    AND         fnf.vr_total_nf             >  0
                    %s
            );
            """ % where
        )

        item_ids = self.pool.get('view.wms.posicao.nf').search(cr, uid, [], context=context)

        if not item_ids:
            # Por contexto passa a mensagem com o numero de enderecos alterados
            msg = _('Nenhum registro encontrado para o filtro selecionado.')
            context['mensagem'] = msg
            model = self.pool.get('ir.model.data')
            view = model.get_object_reference(
                cr, uid, 'wms',  'view_wizard_wms_posicao_nf_form')

            result = {
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view and view[1] or False],
                'res_model': 'wizard.wms.posicao.nf',
                'type': 'ir.actions.act_window',
                'res_id': ids[0],
                'name': 'Posicao de notas fiscais',
                'target': 'new',
                'context': context
            }
        else:
            result = {
                'type': 'ir.actions.report.xml',
                'report_name': 'wms.posicao.nf',
                'name': 'wms_posicao_nf',
                'datas': {'ids': item_ids}
            }

        return result

    # _________________________ ORM Methods __________________________________
    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Mostra mensagem se existir.

        :param cr: a linha atual do cursor do banco de dados,
        :param uid: ID do usuario utilizado para a verificacao de perfil,
        :param ids: lista com os IDs,
        :param fields: dicionario com o nome dos campos e estruturas de construcao,
        :param context: um dicionario padrao para valores contextuais,
        :return: novos campos criados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        results = super(wizard_wms_posicao_nf, self).read(
            cr, uid, ids if isinstance(ids, list) else [ids], fields, context, load)

        if context.get('mensagem'):
            for result in results:
                result['wizpnf_mensagem'] = context.get('mensagem')

        return results

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
