# -*- encoding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz:siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from openerp.tools import drop_view_if_exists
from openerp.tools.translate import _


class wizard_wms_produto_sem_movimentacao(osv.TransientModel):
    """
    Produtos sem movimentacao no estoque.

    :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'wizard.wms.produto.sem.movimentacao'  # wizpsm
    _inherit = 'wizard.wms.produto.sem.movimentacao'  # wms/wizard/produto_sem_movimentacao_data

    def wizpsm_button_imprimir_relatorio(self, cr, uid, ids, context=None):
        """
        Impressao do relatorio de produtos sem movimentacao no estoque.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista com os IDs,
        :param context: {dict} um dicionario padrao para valores contextuais,

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        where1 = "ros.state NOT IN ('esboco', 'cancelada')"
        where2 = "sos.state NOT IN ('esboco', 'cancelada')"
        periodo_inical = periodo_final = None
        campos = ['wizpsm_periodo_inicio', 'wizpsm_periodo_fim']
        wizard_wms_produto_sem_movimentacao_dados = self.read(cr, uid, ids, campos, context)

        if wizard_wms_produto_sem_movimentacao_dados and \
            isinstance(wizard_wms_produto_sem_movimentacao_dados, list):
            wizard_wms_produto_sem_movimentacao_dados = wizard_wms_produto_sem_movimentacao_dados[0]

        if wizard_wms_produto_sem_movimentacao_dados and \
            isinstance(wizard_wms_produto_sem_movimentacao_dados, dict):

            periodo_inical = wizard_wms_produto_sem_movimentacao_dados.get('wizpsm_periodo_inicio')
            periodo_final = wizard_wms_produto_sem_movimentacao_dados.get('wizpsm_periodo_fim')

        if periodo_inical:
            where1 += " AND ros.data_criacao >= '%s'" % periodo_inical
            where2 += " AND sos.data_criacao >= '%s'" % periodo_inical

        if periodo_final:
            where1 += " AND ros.data_criacao <= '%s'" % periodo_final
            where2 += " AND sos.data_criacao <= '%s'" % periodo_final

        drop_view_if_exists(cr, 'view_wms_produto_sem_movimentacao')
        cr.execute(
            """
            CREATE VIEW view_wms_produto_sem_movimentacao AS (
                SELECT  epg.id                              AS id,
                        ep.codigo                           AS wmspsm_codigo,
                        ep.name                             AS wmspsm_produto,
                        epg.grade                           AS wmspsm_grade,
                        wms.name                            As wmspsm_endereco,
                        wms.tipo_nivel                      AS wmspsm_nivel,
                        wms.picking_dinamico                AS wmspsm_dinamico,
                        epg.capacidade_picking              AS wmspsm_capacidade_picking,
                        epg.lastro                          AS wmspsm_lastro,
                        epg.camada                          AS wmspsm_camada,
                        epg.lastro * epg.camada             AS wmspsm_norma_palete
                FROM    estoque_produto_grade               AS epg
                    INNER JOIN wms_endereco                 AS wms ON epg.endereco_produto_id = wms.id
                    INNER JOIN estoque_produto              AS ep  ON wms.produto_id = ep.id
                WHERE   epg.id NOT IN
                    (
                        SELECT  DISTINCT ri.grade_id
                        FROM    recebimento_itens           AS ri
                            INNER JOIN recebimento_os       AS ros ON ri.recebimento_id = ros.id
                        WHERE   %s
                    )
                AND      epg.id NOT IN
                    (
                        SELECT DISTINCT si.grade_id
                        FROM    wms_separacao_itens         AS si
                            INNER JOIN separacao_os         AS sos ON si.os_id = sos.id
                        WHERE   %s
                    )
                ORDER BY wms.name
            );
            """ % (where1, where2)
        )

        movimentacao_ids = self.pool.get('view.wms.produto.sem.movimentacao').search(cr, uid, [], context=context)

        if not movimentacao_ids:
            # Por contexto passa a mensagem com o numero de enderecos alterados
            msg = _('Nenhum registro encontrado para o filtro selecionado.')
            context['mensagem'] = msg
            model = self.pool.get('ir.model.data')
            view = model.get_object_reference(
                cr, uid, 'wms',  'view_wizard_wms_produto_sem_movimentacao_form')

            result = {
                'view_type': 'form',
                'view_mode': 'form',
                'view_id': [view and view[1] or False],
                'res_model': 'wizard.wms.produto.sem.movimentacao',
                'type': 'ir.actions.act_window',
                'res_id': ids[0],
                'name': 'Produtos sem movimentacao',
                'target': 'new',
                'context': context
            }
        else:
            result = {
                'type': 'ir.actions.report.xml',
                'report_name': 'produto.sem.movimentacao',
                'name': 'produto_sem_movimentacao',
                'datas': {'ids': movimentacao_ids}
            }

        return result

    # _________________________ ORM Methods __________________________________
    def read(self, cr, uid, ids, fields=None, context=None, load='_classic_read'):
        """
        Mostra mensagem se existir.

        :param cr: {cursor} a linha atual do cursor do banco de dados,
        :param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        :param ids: {list} lista com os IDs,
        :param fields: {dict} dicionario com o nome dos campos e estruturas de construcao,
        :param context: {dict} um dicionario padrao para valores contextuais,
        :return: {dict} novos campos criados.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        results = super(wizard_wms_produto_sem_movimentacao, self).read(
            cr, uid, ids, fields, context, load)

        if context.get('mensagem'):
            for result in results:
                result['wizpsm_mensagem'] = context.get('mensagem')

        return results

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
