#!/usr/bin/python
# -*- encoding: utf-8 -*-

from re import sub as re_sub
from datetime import datetime, date
from traceback import format_exc as traceback_format_exc
from SigUtils.datas import sigDatas
from SigUtils import numero
from dateutil.parser import parse
from psycopg2.extensions import TransactionRollbackError
from SigUtils.formatastring import formataString
from ..exceptions import EstoqueParametroError
from ..funcoes.integ_funcoes import funct_retira_acentos, funct_get_categoria_pai, \
    funct_fis_validade_value, funct_converte_csosn, funct_get_empresa
from ..funcoes.integ_funcoes import CalcTempo
from ....estoque.model.funcoes import _estpgr_get_name


def est_format_produto_codigo(db_registro, produto_codigo, context, company_com_nf_id):
    estpar_company_id = funct_get_empresa(db_registro, 'estoque.parametro', company_com_nf_id)
    estpar_dados = db_registro.select(table='estoque_parametro', fields=['estpar_tamanho_codigo'],
                                      where=[('company_id', '=', estpar_company_id)])
# ##########################################################################
#     obj_estoque_parametro = context.get('ctx_est_obj_estoque_parametro')
#     estpar_dados = obj_estoque_parametro.estpar_get_fields(
#         db_registro.cr, db_registro.uid, fields_to_read=['estpar_tamanho_codigo'],
#         context=context
#     )
    if isinstance(estpar_dados, list):
        estpar_dados = estpar_dados[0]
    tamanho_codigo_produto = estpar_dados['estpar_tamanho_codigo']
    return formataString(produto_codigo.lstrip('0').rstrip(), tamanho_codigo_produto, '0', 'E')

# PRODUTO
def gerarEst01(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = re_sub('#', ' ', dados).replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _chave = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                               context=context, company_com_nf_id=company_com_nf_id)
        else:
            _chave = lista_dados[0].strip()
        _digito = lista_dados[1]
        # _unidade = lista_dados[2].strip()
        _chaveGrupos = lista_dados[3]
        _margemIdeal = lista_dados[4]
        if '.' in _margemIdeal:
            _margemIdeal = numero.getNumToStr(_margemIdeal, 3, 2, 'N')
        _margemMin = lista_dados[5]
        if '.' in _margemMin:
            _margemMin = numero.getNumToStr(_margemMin, 3, 2, 'N')
        _margemBruta = lista_dados[6]
        if '.' in _margemBruta:
            _margemBruta = numero.getNumToStr(_margemBruta, 3, 2, 'N')
        _mva = lista_dados[7][:3] + '.' + lista_dados[7][3:]
        _cstNoEstado = lista_dados[8].replace(' ', '0')
        # _cstForaEstado = lista_dados[9].replace(' ', '0')
        # _cstProdutorRural = lista_dados[10].replace(' ', '0')
        _cstUltimaEntrada = lista_dados[11].replace(' ', '0')
        _codTributConsFinal = lista_dados[12]
        _codTributContribuinte = lista_dados[13]
        _codTributProdutor = lista_dados[14]
        _codTributEntrada = lista_dados[15]
        # _codTributSubst = lista_dados[16]
        # _codTributEntradaSub = lista_dados[17]
        _aliqIpi = lista_dados[18][:3] + '.' + lista_dados[18][3:]
        # _classifFiscal = lista_dados[19].strip()
        _classifNcm = re_sub('[^0-9]', '', lista_dados[20].strip())
        if len(_classifNcm) > 8:
            _classifNcm = _classifNcm[:8]
        fiscal_ncm_dados = db_registro.select(table='fiscal_ncm', fields=['id'], where=[('codigo', '=', _classifNcm)])
        if not fiscal_ncm_dados:
            fiscal_ncm_id = db_registro.insert(table='fiscal_ncm', vals={'codigo': _classifNcm, 'descricao': _classifNcm})
        else:
            fiscal_ncm_id = fiscal_ncm_dados[0]['id']
        _pis = lista_dados[21]
        if _pis == 'Y':
            _pis = True
        else:
            _pis = False
        _pisCstEntrada = lista_dados[22]
        _pisCstSaida = lista_dados[23]
        _pisNatRec = lista_dados[24]
        _cofins = lista_dados[25]
        if _cofins == 'Y':
            _cofins = True
        else:
            _cofins = False
        _cofinsCstEntrada = lista_dados[26]
        _cofinsCstSaida = lista_dados[27]
        _cofinsNatRec = lista_dados[28]
        _fornecedorPrincipal = lista_dados[29]
        _representante = lista_dados[30]
        _tempoReposicao = lista_dados[31]
        _temGrade = lista_dados[32]
        if _temGrade == 'S':
            _temGrade = True
        else:
            _temGrade = False
        _comprador = lista_dados[33]
        # _temSerie = lista_dados[34]
        # _politicaPreco = lista_dados[35]
        # _contaCustos = lista_dados[36]
        # _servicoAgregado = lista_dados[37]
        _kitMercadorias = lista_dados[38]
        if _kitMercadorias == 'Y':
            _kitMercadorias = True
        else:
            _kitMercadorias = False
        _kitTipoPreco = lista_dados[39]

        _kitMontagem = lista_dados[40]

        _codigoExIpi = lista_dados[41].strip()
        _codigoEnqIpi = lista_dados[42].strip()
        _codGenero = lista_dados[43]
        _codTabServicos = lista_dados[44]
        if _codTabServicos == '0000':
            _codTabServicos = ''
        fiscal_tabela_servico_dados = db_registro.select(table='fiscal_tabela_servico', fields=['id'], where=[('codigo', '=', _codTabServicos)])
        if fiscal_tabela_servico_dados:
            _codTabServicos_id = fiscal_tabela_servico_dados[0]['id']
        else:
            _codTabServicos_id = False
        # _ultimaPauta = lista_dados[45]
        _producao = lista_dados[46] or ''
        if _producao.upper() == 'Y':
            _producao = True
        else:
            _producao = False
        dados_tipo_item = lista_dados[47] or '00'
        dados_codigo_anp = (lista_dados[48] or '')[:9].strip() or '0'
        dados_codigo_cest = (lista_dados[49] or '')[:7].strip() or '0'

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        try:
            int(dados_codigo_anp)
        except:
            codigo_anp_id = 'null'
        else:
            fiscal_anp_dados = db_registro.select(
                'fiscal_cadastro_anp', fields=['id'], where=[('fisanp_codigo', '=', dados_codigo_anp)]
            )
            if fiscal_anp_dados:
                codigo_anp_id = fiscal_anp_dados[0]['id']
            else:
                vals_fiscal_anp = {
                    'fisanp_codigo': dados_codigo_anp,
                    'name': dados_codigo_anp,
                }
                codigo_anp_id = db_registro.insert(table='fiscal_cadastro_anp', vals=vals_fiscal_anp)

        if dados_codigo_cest:
            fiscal_cest_dados = db_registro.select(
                'fiscal_cest', fields=['id'], where=[('fiscest_codigo', '=', dados_codigo_cest)]
            )
            if fiscal_cest_dados:
                codigo_cest_id = fiscal_cest_dados[0]['id']
            else:
                vals_fiscal_cest = {
                    'fiscest_codigo': dados_codigo_cest,
                    'name': dados_codigo_cest,
                }
                codigo_cest_id = db_registro.insert(table='fiscal_cest', vals=vals_fiscal_cest)
        else:
            codigo_cest_id = 'null'


        # achar tributacao/Encargo
        TributacaoConsFinal_ids = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_codTributConsFinal,
            reg_tabela_legado='est14',
            reg_many_ids=True,
            reg_check_table='fiscal_tributacao'
        )
        if not TributacaoConsFinal_ids:
            raise Exception('Est01: Tributacao nao cadastrada na legado_controle - %s' % _codTributConsFinal)

        _regTributacao = db_registro.select(table='fiscal_tributacao', fields=['name', 'encargo_id', 'cst_id'], where=[('id', 'in', TributacaoConsFinal_ids)])

        if not _regTributacao:
            raise Exception('Est01: Tributacao nao cadastrada - %s' % _codTributConsFinal)

        tributacao_encargo_id = _regTributacao[0]['encargo_id']
        tributacao_encargo_name = _regTributacao[0]['name']
        _regEncargo = db_registro.select(table='fiscal_encargo', fields=['id', 'sigla'], where=[('id', '=', tributacao_encargo_id)])
        if not _regEncargo:
            raise Exception('Est01: Encargo nao cadastrado - %s' % tributacao_encargo_name)

        if _codTributProdutor != '000':
            TributacaoProdutor_ids = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_codTributProdutor,
                reg_tabela_legado='est14',
                reg_many_ids=True,
                reg_check_table='fiscal_tributacao'
            )

            if not TributacaoProdutor_ids:
                raise Exception('Est01 - Tributacao(Produtor) nao cadastrada na legado_controle - %s' % _codTributProdutor)

            _regTributacao = db_registro.select('fiscal_tributacao', fields=['encargo_id', 'name'], where=[('id', 'in', TributacaoProdutor_ids)])
            if not _regTributacao:
                raise Exception('Est01 - Tributacao nao cadastrada na tabela fiscal_tributacao - %s' % _codTributProdutor)

            tribut_produtor_encargo_id = _regTributacao[0]['encargo_id']
            tribut_produtor_name = _regTributacao[0]['name']
            _regEncargo = db_registro.select(table='fiscal_encargo', fields=['id', 'sigla'], where=[('id', '=', tribut_produtor_encargo_id)])
            if not _regEncargo:
                raise Exception('Est01 - Encargo(Produtor) nao cadastrado. Tributacao - %s' % tribut_produtor_name)

        if _codTributContribuinte != '000':
            TributacaoContribuinte_ids = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_codTributContribuinte,
                reg_tabela_legado='est14',
                reg_many_ids=True,
                reg_check_table='fiscal_tributacao'
            )

            if not TributacaoContribuinte_ids:
                raise Exception('Est01 - Tributacao nao cadastrada na legado_controle - %s' % _codTributContribuinte)

            _regTributacao = db_registro.select(table='fiscal_tributacao', fields=['encargo_id', 'name'], where=[('id', 'in', TributacaoContribuinte_ids)])
            if not _regTributacao:
                raise Exception('Est01 - Tributacao(Contribuinte) nao cadastrada - %s' % _codTributContribuinte)

            tribut_contribuinte_encargo_id = _regTributacao[0]['encargo_id']
            tribut_contribuinte_name = _regTributacao[0]['name']
            _regEncargo = db_registro.select(table='fiscal_encargo', fields=['id', 'sigla'], where=[('id', '=', tribut_contribuinte_encargo_id)])
            if not _regEncargo:
                raise Exception('Est01 - Encargo(Contribuinte) nao cadastrado - %s' % tribut_contribuinte_name)

        if _codTributEntrada != '000':
            idsTributacaoEntrada = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_codTributEntrada,
                reg_tabela_legado='est14',
                reg_check_table='fiscal_tributacao'
            )

            if not idsTributacaoEntrada:
                raise Exception('Est01 - Tributacao(Entrada) nao cadastrada na legado_controle - %s' % _codTributEntrada)

            _regTributacao = db_registro.select(table='fiscal_tributacao', fields=['encargo_id', 'name'], where=[('id', '=', idsTributacaoEntrada)])
            if not _regTributacao:
                raise Exception('Est01 - Tributacao(Entrada) nao cadastrada - %s' % _codTributEntrada)

            tribut_entrada_encargo_id = _regTributacao[0]['encargo_id']
            tribut_entrada_name = _regTributacao[0]['name']
            _regEncargo = db_registro.select(table='fiscal_encargo', fields=['id', 'sigla'], where=[('id', '=', tribut_entrada_encargo_id)])
            if not _regEncargo:
                raise Exception('Est01 - Encargo(Entrada) nao cadastrado - %s' % tribut_entrada_name)

        # achar categoria
        idsCategoria = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chaveGrupos,
            reg_tabela_legado='est04',
            reg_check_table='estoque_categoria',
        )

        if not idsCategoria:
            # BUSCA A CATEGORIA SUPERIOR PARA A CATEGORIA NAO ENCONTRADA
            categorias_acima = funct_get_categoria_pai(_chaveGrupos)
            if categorias_acima:
                args_categ = [
                    ('tabela_legado', '=', 'est04'), ('chave_legado', 'in', categorias_acima), ('empresa', '=', empresa_legado)
                ]

                categ_dados = db_registro.select_legado_controle(
                    reg_fields=['chave_sigerp'],
                    reg_args=args_categ,
                    reg_order_by=['chave_legado desc'],
                    reg_check_table='estoque_categoria',
                )

                if categ_dados:
                    idsCategoria = categ_dados[0]['chave_sigerp']

        if not idsCategoria:
            raise Exception('Est01 - Categoria(est04) nao cadastrada na legado_controle. Chave_legado - %s' % _chaveGrupos)

        if _regEncargo[0]['sigla'] == 'ICMS':
            _produtoServico = 'P'
        else:
            _produtoServico = 'S'
        #achar fabricante
        _fabricante_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_fornecedorPrincipal,
            reg_tabela_legado='cpg01',
            reg_check_table='adm_pessoa',
        )

        #achar comprador
        _comprador_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_comprador,
            reg_tabela_legado='est16',
            reg_check_table='compras_comprador',
        )

        valsProduto = {
            'tipo': _produtoServico,
            'name': 'Item ' + str(_chave),
            'company_id': company_id,
            'margem_ideal': _margemIdeal[:2] + '.' + _margemIdeal[2:],
            'margem_minima': _margemMin[:2] + '.' + _margemMin[2:],
            'margem_bruta': _margemBruta[:4] + '.' + _margemBruta[4:],
            'tempo_reposicao': _tempoReposicao,
            'codigo': _chave,
            'active': True,
            'grade_usar': _temGrade,
            'digito': _digito,
            'kit_de_produtos': _kitMercadorias,
            'codigo_ex_ncm': _codigoExIpi,
            'codigo_enq_ipi': _codigoEnqIpi,
            'codigo_genero': _codGenero,
            'tipo_item': dados_tipo_item,
            'origem_mercadoria': '0',
            'estpro_pis_nat_rec': _pisNatRec,
            'estpro_cofins_nat_rec': _cofinsNatRec,
            'producao_propria': _producao,
            'estpro_codigo_anp_id': codigo_anp_id,
            'estpro_cest_id': codigo_cest_id,
        }
        if fiscal_ncm_id:
            valsProduto['ncm_id'] = fiscal_ncm_id
        if _codTabServicos_id:
            valsProduto['codigo_tab_servicos_id'] = _codTabServicos
        if _comprador_id:
            valsProduto['comprador_id'] = _comprador_id
        if _fabricante_id:
            valsProduto['fabricante_id'] = _fabricante_id
        if idsCategoria:
            valsProduto['categoria_id'] = idsCategoria
        _balancaInstalado = db_registro.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'balanca'),
                                                            ('state', '=', 'installed')]
        )
        if _balancaInstalado:
            if operacao == 'w':
                valsProduto['peso_unidade'] = 'N'
            else:
                peso_unidade = db_registro.select(table='estoque_produto',
                                                  fields=['peso_unidade'],
                                                  where=[('company_id', '=', company_id),
                                                         ('codigo', '=', _chave)])
                if peso_unidade:
                    pass
                else:
                    valsProduto['peso_unidade'] = 'N'
        if _kitMercadorias:

            if _kitMontagem == 'Y':
                _kitMontagem = True
            else:
                _kitMontagem = False

            if _kitTipoPreco != 'K':
                _kitTipoPreco = 'S'

            valsProduto['preco_kit'] = _kitTipoPreco
            valsProduto['tipo_kit'] = 'I'

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _chave)])
        if operacao == 'w':  # Gravar
            if _regProduto:
                return True
            else:
                idProduto = db_registro.insert(table='estoque_produto', vals=valsProduto)

            if _codNumerico:
                _numero = int(_chave)
                __code = 'estoque.produto'
                __name = 'Codigo de Produto/Servico'
                _dados_sequence = db_registro.select(table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', __code), ('name', '=', __name)])
                if not _dados_sequence:
                    vals_sequence = {
                        'code': __code,
                        'name': __name,
                        'number_next': 1,
                        'implementation': 'no_gap',
                        'company_id': company_id,
                        'padding': 6,
                        'active': True,
                        'number_increment': 1
                    }
                    _sequence_id = db_registro.insert(table='ir_sequence', vals=vals_sequence)
                    _ultimo = 1
                else:
                    _sequence_id = _dados_sequence[0]['id']
                    _ultimo = _dados_sequence[0]['number_next']

                if _numero > _ultimo:
                    db_registro.update(table='ir_sequence', values={'number_next': _numero + 1}, where=[('id', '=', _sequence_id)])

            if _representante:
                cpg11_chave = '%s%s' % (_fornecedorPrincipal, _representante)
                representante_id = db_registro.controle_get_chave_sigerp(
                    reg_empresa_legado=empresa_legado,
                    reg_chave=cpg11_chave,
                    reg_tabela_legado='cpg11',
                    reg_check_table='financeiro_representante',
                )
                if _fabricante_id and representante_id:

                    financ_fornecedor_dados = db_registro.select(
                        table='financeiro_fornecedor', fields=['id'], where=[('pessoa_id', '=', _fabricante_id)]
                    )
                    if financ_fornecedor_dados:
                        fornecedor_id = financ_fornecedor_dados[0]['id']
                        args_estoque_fornecedor = [
                            ('fornecedor_id', '=', fornecedor_id), ('representante_id', '=', representante_id),
                            ('produto_id', '=', idProduto)
                        ]
                        estoque_fornecedor_dados = db_registro.select(
                            table='estoque_fornecedor', fields=['id'], where=args_estoque_fornecedor
                        )
                        if not estoque_fornecedor_dados:
                            vals_representante = {
                                'fornecedor_id': fornecedor_id,
                                'representante_id': representante_id,
                                'produto_id': idProduto
                            }

                            db_registro.insert(table='estoque_fornecedor', vals=vals_representante)

            if not _temGrade:
                vals_grade = {
                    'produto_id': idProduto,
                    'grade': 'UN',
                    'descricao': 'UNICA',
                    'active': True,
                }
                db_registro.insert(table='estoque_produto_grade', vals=vals_grade)

            _pessoa_id = db_registro.select(table='res_company', fields=['rescon_pessoa_id'], where=[('id', '=', company_id)])
            if _pessoa_id:
                _pessoa_id = _pessoa_id[0]['rescon_pessoa_id']
            _regPessoa = db_registro.select(table='adm_pessoa', fields=['admpes_jur_tipo_atividade', 'admpes_jur_regime_tributario'], where=[('id', '=', _pessoa_id)])
            _tipoAtividade = _regPessoa[0]['admpes_jur_tipo_atividade']
            _regimeTributario = _regPessoa[0]['admpes_jur_regime_tributario']
            if _produtoServico == 'P':
                encargosLista = ['ICMS', 'PIS', 'COFINS']
            else:
                encargosLista = ['ICMS', 'ISS', 'PIS', 'COFINS']
            if _tipoAtividade == 'I':
                encargosLista.append('IPI')
            for _encargo in encargosLista:
                res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', _encargo)])
                if res:
                    idFiscalEncargo = res[0]['id']
                else:
                    vals = {'name': _encargo, 'sigla': _encargo}
                    if _encargo in ['COFINS', 'PIS', 'IPI']:
                        vals['tipo_encargo'] = 'F'
                    if _encargo == 'ICMS':
                        vals['tipo_encargo'] = 'E'
                    if _encargo == 'ISS':
                        vals['tipo_encargo'] = 'M'
                    idFiscalEncargo = db_registro.insert(table='fiscal_encargo', vals=vals)
                if _encargo == 'ICMS':
                    if _regimeTributario == 'S':
                        _csosn_id = db_registro.select(table='fiscal_csosn', fields=['id'],
                                                      where=[('codigo', '=', _cstNoEstado)])
                        if not _csosn_id:
                            raise Exception("Est01 - CSOSN invalido ou nao cadastrado - %s" % _cstNoEstado)
                        else:
                            _cst = funct_converte_csosn(_cstNoEstado)
                            _tipo = 'T'
                            _csosn_id = _csosn_id[0]['id']
                    else:
                        _cst = _cstNoEstado[1:]
                        _tipo = 'T'
                elif _encargo == 'PIS':
                    _cst = _pisCstSaida
                    _tipo = 'T'
                elif _encargo == 'COFINS':
                    _cst = _cofinsCstSaida
                    _tipo = 'T'
                elif _encargo == 'ISS':
                    _cst = '00'
                    _tipo = 'T'
                elif _encargo == 'IPI':
                    if float(_aliqIpi) == 0:
                        _cst = '53'
                        _tipo = 'O'
                    else:
                        _cst = '50'
                        _tipo = 'T'
                fieldsCst = ['id', 'tributo_simples_nac', 'fiscst_csosn_contribuinte_id']
                res = db_registro.select(table='fiscal_cst', fields=fieldsCst,
                                     where=[('codigo', '=', _cst),
                                            ('encargo_id', '=', idFiscalEncargo)])
                if res:
                    idCST = res[0]['id']
                    #Verifica se e simples e cria um cst correlacionado ao csosn caso nao exista
                    if _regimeTributario == 'S' and _encargo == 'ICMS':
                        simples_nacional = res[0]['tributo_simples_nac']
                        fiscst_csosn_id = res[0]['fiscst_csosn_contribuinte_id']
                        if not fiscst_csosn_id:
                            db_registro.update(table='fiscal_cst',
                                               values={'tributo_simples_nac': True,
                                                       'fiscst_csosn_contribuinte_id': _csosn_id},
                                               where=[('id', '=', idCST)])
                        else:
                            pass
                else:
                    if _regimeTributario == 'S':
                        vals = {'codigo': _cst,
                                'name': _cst,
                                'tipo': _tipo,
                                'encargo_id': idFiscalEncargo,
                                'tributo_simples_nac': True,
                                'fiscst_csosn_contribuinte_id': _csosn_id}
                    else:
                        vals = {'codigo': _cst,
                                'name': _cst,
                                'tipo': _tipo,
                                'encargo_id': idFiscalEncargo}
                    idCST = db_registro.insert(table='fiscal_cst', vals=vals)



                _idTributacao = False
                if _encargo == 'ICMS' or _encargo == 'ISS':
                    if len(TributacaoConsFinal_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoConsFinal_ids))]
                    else:
                        args = [('id', '=', TributacaoConsFinal_ids[0])]

                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)

                if _encargo == 'PIS' or _encargo == 'COFINS':
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                if _encargo == 'IPI':
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=[('cst_id', '=', idCST)])

                for a in regTributacao:
                    if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                        _idTributacao = a['id']
                if _idTributacao:
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'T',
                    }
                    if _encargo == 'ICMS':
                        vals['mva'] = _mva
                        if float(_mva) != 0 and _tipo == 'T':
                            vals['modbcst'] = '4'
                    else:
                        vals['mva'] = 0

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _encargo == 'PIS' and _pisCstEntrada != _pisCstSaida:
                    _cst = _pisCstEntrada
                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cst), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cst, 'name': _cst, 'tipo': _tipo, 'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)

                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                    else:
                        for a in regTributacao:
                            _idTributacao = a['id']
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'E',
                    }

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _encargo == 'COFINS' and _cofinsCstEntrada != _cofinsCstSaida:
                    _cst = _cofinsCstEntrada
                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cst), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cst, 'name': _cst, 'tipo': _tipo, 'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                    else:
                        for a in regTributacao:
                            _idTributacao = a['id']
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'E',
                    }

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _codTributContribuinte != '000' and _encargo == 'ICMS' and _codTributContribuinte != _codTributConsFinal:

                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:
                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'C',
                        }
                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )
                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _codTributProdutor != '000' and _encargo == 'ICMS' and _codTributProdutor != _codTributConsFinal:
                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:
                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'P',
                        }
                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )
                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _codTributEntrada != '000' and _encargo == 'ICMS' and _codTributEntrada != _codTributConsFinal:
                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'cst_id', 'encargo_id'], where=args)

                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cstUltimaEntrada[1:]), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cstUltimaEntrada[1:], 'name': _cstUltimaEntrada[1:], 'tipo': _tipo,
                                'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:

                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'E',
                        }

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )
                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0
                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

        elif operacao == 'r':  # Regravar
            if not _regProduto:
                idProduto = db_registro.insert(table='estoque_produto', vals=valsProduto)
            else:
                idProduto = _regProduto[0]['id']
                valsProduto.pop('name')
                db_registro.update(table='estoque_produto', values=valsProduto, where=[('id', '=', idProduto)])

            # ESTOQUE_FORNECEDOR
            if _representante:
                cpg11_chave = '%s%s' % (_fornecedorPrincipal, _representante)
                representante_id = db_registro.controle_get_chave_sigerp(
                    reg_empresa_legado=empresa_legado,
                    reg_chave=cpg11_chave,
                    reg_tabela_legado='cpg11',
                    reg_check_table='financeiro_representante',
                )
                if _fabricante_id and representante_id:

                    financ_fornecedor_dados = db_registro.select(
                        table='financeiro_fornecedor', fields=['id'], where=[('pessoa_id', '=', _fabricante_id)]
                    )
                    if financ_fornecedor_dados:
                        fornecedor_id = financ_fornecedor_dados[0]['id']
                        args_estoque_fornecedor = [
                            ('fornecedor_id', '=', fornecedor_id), ('representante_id', '=', representante_id),
                            ('produto_id', '=', idProduto)
                        ]
                        estoque_fornecedor_dados = db_registro.select(
                            table='estoque_fornecedor', fields=['id'], where=args_estoque_fornecedor
                        )
                        if not estoque_fornecedor_dados:
                            vals_representante = {
                                'fornecedor_id': fornecedor_id,
                                'representante_id': representante_id,
                                'produto_id': idProduto
                            }

                            db_registro.insert(table='estoque_fornecedor', vals=vals_representante)

            _pessoa_id = db_registro.select(table='res_company', fields=['rescon_pessoa_id'], where=[('id', '=', company_id)])
            if _pessoa_id:
                _pessoa_id = _pessoa_id[0]['rescon_pessoa_id']
            _regPessoa = db_registro.select(table='adm_pessoa', fields=['admpes_jur_tipo_atividade', 'admpes_jur_regime_tributario'], where=[('id', '=', _pessoa_id)])
            _tipoAtividade = _regPessoa[0]['admpes_jur_tipo_atividade']
            _regimeTributario = _regPessoa[0]['admpes_jur_regime_tributario']
            if _produtoServico == 'P':
                encargosLista = ['ICMS', 'PIS', 'COFINS']
            else:
                encargosLista = ['ICMS', 'ISS', 'PIS', 'COFINS']
            if _tipoAtividade == 'I':
                encargosLista.append('IPI')
            # APAGA A TRIBUTACAO DO ITEM
            db_registro.delete(table='fiscal_tributacao_encargos', where=[('produto_id', '=', idProduto)])

            for _encargo in encargosLista:
                res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', _encargo)])
                if res:
                    idFiscalEncargo = res[0]['id']
                else:
                    vals = {'name': _encargo, 'sigla': _encargo}
                    if _encargo in ['COFINS', 'PIS', 'IPI']:
                        vals['tipo_encargo'] = 'F'
                    if _encargo == 'ICMS':
                        vals['tipo_encargo'] = 'E'
                    if _encargo == 'ISS':
                        vals['tipo_encargo'] = 'M'
                    idFiscalEncargo = db_registro.insert(table='fiscal_encargo', vals=vals)
                if _encargo == 'ICMS':
                    if _regimeTributario == 'S':
                        _csosn_id = db_registro.select(table='fiscal_csosn', fields=['id'],
                                                      where=[('codigo', '=', _cstNoEstado)])
                        if not _csosn_id:
                            raise Exception("Est01 - CSOSN invalido ou nao cadastrado - %s" % _cstNoEstado)
                        else:
                            _cst = funct_converte_csosn(_cstNoEstado)
                            _tipo = 'T'
                            _csosn_id = _csosn_id[0]['id']
                    else:
                        _cst = _cstNoEstado[1:]
                        _tipo = 'T'
                elif _encargo == 'PIS':
                    _cst = _pisCstSaida
                    _tipo = 'T'
                elif _encargo == 'COFINS':
                    _cst = _cofinsCstSaida
                    _tipo = 'T'
                elif _encargo == 'ISS':
                    _cst = '00'
                    _tipo = 'T'
                elif _encargo == 'IPI':
                    if float(_aliqIpi) == 0:
                        _cst = '53'
                        _tipo = 'O'
                    else:
                        _cst = '50'
                        _tipo = 'T'
                fieldsCst = ['id', 'tributo_simples_nac', 'fiscst_csosn_contribuinte_id']
                res = db_registro.select(table='fiscal_cst', fields=fieldsCst,
                                         where=[('codigo', '=', _cst),
                                                ('encargo_id', '=', idFiscalEncargo)])
                if res:
                    idCST = res[0]['id']
                    #Verifica se e simples e cria um cst correlacionado ao csosn caso nao exista
                    if _regimeTributario == 'S' and _encargo == 'ICMS':
                        simples_nacional = res[0]['tributo_simples_nac']
                        fiscst_csosn_id = res[0]['fiscst_csosn_contribuinte_id']
                        if not fiscst_csosn_id:
                            db_registro.update(table='fiscal_cst',
                                               values={'tributo_simples_nac': True,
                                                       'fiscst_csosn_contribuinte_id': _csosn_id},
                                               where=[('id', '=', idCST)])
                        else:
                            pass
                else:
                    if _regimeTributario == 'S':
                        vals = {'codigo': _cst,
                                'name': _cst,
                                'tipo': _tipo,
                                'encargo_id': idFiscalEncargo,
                                'tributo_simples_nac': True,
                                'fiscst_csosn_contribuinte_id': _csosn_id}
                    else:
                        vals = {'codigo': _cst,
                                'name': _cst,
                                'tipo': _tipo,
                                'encargo_id': idFiscalEncargo}
                    idCST = db_registro.insert(table='fiscal_cst', vals=vals)

                _idTributacao = False

                if _encargo == 'ICMS' or _encargo == 'ISS':
                    if len(TributacaoConsFinal_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoConsFinal_ids))]
                    else:
                        args = [('id', '=', TributacaoConsFinal_ids[0])]

                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)
                elif _encargo == 'PIS' or _encargo == 'COFINS':
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                elif _encargo == 'IPI':
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=[('cst_id', '=', idCST)])

                for a in regTributacao:
                    if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                        _idTributacao = a['id']

                if _idTributacao:
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'T',
                    }
                    if _encargo == 'ICMS':
                        vals['mva'] = _mva
                        if float(_mva) != 0 and _tipo == 'T':
                            vals['modbcst'] = '4'
                    else:
                        vals['mva'] = 0

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _encargo == 'PIS' and _pisCstEntrada != _pisCstSaida:
                    _cst = _pisCstEntrada
                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cst), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cst, 'name': _cst, 'tipo': _tipo, 'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)

                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                    else:
                        for a in regTributacao:
                            _idTributacao = a['id']
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'E',
                    }

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _encargo == 'COFINS' and _cofinsCstEntrada != _cofinsCstSaida:
                    _cst = _cofinsCstEntrada
                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cst), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cst, 'name': _cst, 'tipo': _tipo, 'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id'], where=[('encargo_id', '=', idFiscalEncargo), ('cst_id', '=', idCST)])
                    if not regTributacao:
                        vals = {
                            'active': True,
                            'aliquota': 0,
                            'base': 100,
                            'cst_id': idCST,
                            'encargo_id': idFiscalEncargo,
                            'name': 'Tributacao CST ' + str(_cst),
                        }
                        _idTributacao = db_registro.insert(table='fiscal_tributacao', vals=vals)
                    else:
                        for a in regTributacao:
                            _idTributacao = a['id']
                    vals = {
                        'produto_id': idProduto,
                        'encargo_id': idFiscalEncargo,
                        'cst_id': idCST,
                        'tributacao_id': _idTributacao,
                        'tipo_tributacao': 'E',
                    }

                    # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                    tributacao_dados = db_registro.select(
                        table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                    )
                    if tributacao_dados:
                        aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                        vals['fisten_aliquota_venda'] = aliquota_venda

                    db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)
                if _codTributContribuinte != '000' and _encargo == 'ICMS' and _codTributContribuinte != _codTributConsFinal:

                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:
                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'C',
                        }
                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )
                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _codTributProdutor != '000' and _encargo == 'ICMS' and _codTributProdutor != _codTributConsFinal:
                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'encargo_id', 'cst_id'], where=args)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:
                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'P',
                        }
                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )
                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

                if _codTributEntrada != '000' and _encargo == 'ICMS' and _codTributEntrada != _codTributConsFinal:
                    if len(TributacaoContribuinte_ids) > 1:
                        args = [('id', 'in', tuple(TributacaoContribuinte_ids))]
                    else:
                        args = [('id', '=', TributacaoContribuinte_ids[0])]
                    regTributacao = db_registro.select(table='fiscal_tributacao', fields=['id', 'cst_id', 'encargo_id'], where=args)

                    res = db_registro.select(table='fiscal_cst', fields=['id'], where=[('codigo', '=', _cstUltimaEntrada[1:]), ('encargo_id', '=', idFiscalEncargo)])
                    if res:
                        idCST = res[0]['id']
                    else:
                        vals = {'codigo': _cstUltimaEntrada[1:], 'name': _cstUltimaEntrada[1:], 'tipo': _tipo,
                                'encargo_id': idFiscalEncargo}
                        idCST = db_registro.insert(table='fiscal_cst', vals=vals)
                    for a in regTributacao:
                        if a['encargo_id'] == idFiscalEncargo and a['cst_id'] == idCST:
                            _idTributacao = a['id']
                    if _idTributacao:

                        # BUSCA A ALIQUOTA DE VENDA PARA O PRODUTO
                        tributacao_dados = db_registro.select(
                            table='fiscal_tributacao', fields=['aliquota'], where=[('id', '=', _idTributacao)]
                        )

                        vals = {
                            'produto_id': idProduto,
                            'encargo_id': idFiscalEncargo,
                            'cst_id': idCST,
                            'tributacao_id': _idTributacao,
                            'tipo_tributacao': 'E',
                        }

                        if tributacao_dados:
                            aliquota_venda = tributacao_dados[0]['aliquota'] or 0
                            vals['fisten_aliquota_venda'] = aliquota_venda

                        if _encargo == 'ICMS':
                            vals['mva'] = _mva
                            if float(_mva) != 0 and _tipo == 'T':
                                vals['modbcst'] = '4'
                        else:
                            vals['mva'] = 0
                        db_registro.insert(table='fiscal_tributacao_encargos', vals=vals)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# MOVIMENTO
def gerarEst03(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    """
    TIPO DE MOVIMENTO (POSICAO 4)
    A - ACERTO
    D - DEVOLUCAO
    T - TRANSFERENCIA
    N - NORMAL (COMPRA OU VENDA)
    E - ESTORNO
    R - REQUISICAO
    O - OUTROS
    """

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _local = lista_dados[2]
        _entrada_saida = lista_dados[3]
        _tipoMov = lista_dados[4]
        _tipo_mov = _tipoMov
        if _tipoMov == 'T':
            _tipo_mov = 'N'
        if _tipoMov in ['E', 'O']:
            _tipo_mov = 'A'
        _tipo_programa = lista_dados[5]
        _cli_for = lista_dados[6]
        _serie = lista_dados[7].strip()
        _documento = lista_dados[8]
        _sequencia = lista_dados[9]
        _data_movimento = lista_dados[10]
        _data_documento = lista_dados[11]
        _quantidade = lista_dados[12]
        # _departamento = list_dados[13]
        # _vendedor = list_dados[14]
        # _vendedor_externo = list_dados[15]
        # _serie_conhec = list_dados[16]
        # _conhec_frete = list_dados[17]
        # _fornec_frete = list_dados[18]
        # _valor_frete = funct_fis_validade_value(list_dados[19].replace(' ', '0'))
        # _sit_tributaria = list_dados[20]
        _cst = lista_dados[21]
        _cod_tributacao = lista_dados[22]
        # _cod_tributacao_entrada = list_dados[23]
        _cfop = lista_dados[24].replace(' ', '0')

        #Implementacao quando movimento for troca
        if _cfop == '0000' and _serie == 'TR':
            if _tipo_mov == 'N':
                _cfop = '5102'
            else:
                _cfop = '1202'
        _aliquota_icms = lista_dados[25].replace(' ', '0')
        _base_icms = lista_dados[26].replace(' ', '0')
        _valor_icms = funct_fis_validade_value(lista_dados[27].replace(' ', '0'))
        # _valor_ipi = funct_fis_validade_value(list_dados[28].replace(' ', '0'))
        _valor_icms_sub_nf = funct_fis_validade_value(lista_dados[29].replace(' ', '0'))
        # _valor_icms_sub_gu = funct_fis_validade_value(list_dados[30].replace(' ', '0'))
        _mva = lista_dados[31].replace(' ', '0')
        # _aliquota_ipi = list_dados[32].replace(' ', '0')
        # _valor_pis = list_dados[33].replace(' ', '0')
        # _valor_cofins = list_dados[34].replace(' ', '0')
        # _valor_outros_extra = list_dados[35].replace(' ', '0') ## esta gerando o campo com {
        # _valor_outros = list_dados[36].replace(' ', '0')
        # _valor = list_dados[37].replace(' ', '0')
        _preco_medio = funct_fis_validade_value(lista_dados[38].replace(' ', '0'))
        _custo_corr = funct_fis_validade_value(lista_dados[39].replace(' ', '0'))
        _ultimo_custo = funct_fis_validade_value(lista_dados[40].replace(' ', '0'))
        _preco_reposicao = funct_fis_validade_value(lista_dados[41].replace(' ', '0'))
        # _p_venda_deflac = list_dados[42].replace(' ', '0')
        _preco_na_nf = funct_fis_validade_value(lista_dados[43].replace(' ', '0'))
        if float(_base_icms) == 0:
            _base_icms = _preco_na_nf
        # _pedido = list_dados[44]
        _usuario = lista_dados[45].strip()
        _desconto = funct_fis_validade_value(lista_dados[46].replace(' ', '0'))
        # _loja = list_dados[47]
        # _caixa = list_dados[48]
        # _base_ipi = list_dados[49].replace(' ', '0')
        _c_s_nf = lista_dados[50].upper()

        if not float(_quantidade):
            # A PRIMEIRA EXPORTACAO DO CUPOM COMO W A QUANTIDADE EH ZERO.
            # E DEPOIS É EXPORTADO UM R COM A QUANTIDADE REAL DO ITEM.
            # ASSIM O W FICAVA COMO ERRO.
            if _tipo_mov == 'N' and _tipo_programa == 'M':  # TIPO CUPOM
                return True

            else:
                raise Exception(
                    'Est03: A quantidade do produto informada deve ser maior que zero (Posicao 12). Quantidade - %s'
                    % str(_quantidade)
                )

        _chaveGrade = _item + '-' + _grade
        _chaveEst03C = _entrada_saida + '-' + _tipo_mov + '-' + _tipo_programa + '-' + _cli_for + '-' + _serie + '-' + _documento + '-' + _data_documento
        _chaveEst03I = _chaveEst03C + '-' + _item + '-' + _grade + '-' + _sequencia

        legado_mov_item_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chaveEst03I,
            reg_tabela_legado='est03I',
            reg_check_table='estoque_movimento_itens',
            reg_empresa_tipo=_c_s_nf
        )

        # achar grade
        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _item)])
        if _regProduto:
            _produto_id = _regProduto[0]['id']
        else:
            raise Exception('Est03: Item nao cadastrado. Codigo - %s, Company_id - %s' % (str(_item), str(company_id)))

        args_grade = [('produto_id', '=', _produto_id), ('grade', '=', _grade)]
        _regGrade = db_registro.select(table='estoque_produto_grade', fields=['id'], where=args_grade)
        if not _regGrade:
            raise Exception('Est03: Grade nao cadastrada - %s' % str(_chaveGrade))

        _grade_id = _regGrade[0]['id']
        #achar local_id
        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        _regLocal = db_registro.select(table='estoque_local', fields=['id'], where=[('numero', '=', _local), ('company_id', '=', company_id)])
        if not _regLocal:
            raise Exception('Est03: Local nao cadastrado. Numero - %s, Company_id - %s' % (str(_local), str(company_id)))

        _local_id = _regLocal[0]['id']
        _regPessoa = db_registro.select(table='adm_pessoa', fields=['id'], where=[('nome', '=', _usuario)])
        if _regPessoa:
            _pessoa_id = _regPessoa[0]['id']
        else:
            vals = {'nome': _usuario, 'pessoa_tipo': 'F', 'active': True}
            regCodigo = db_registro.select(table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')])
            vals['codigo'] = regCodigo[0]['number_next']
            codigo = int(regCodigo[0]['number_next']) + 1
            partner_id = db_registro.insert(table='res_partner', vals={'name': _usuario, 'notification_email_send': 'comment'})
            vals['partner_id'] = partner_id
            db_registro.update(table='ir_sequence', values={'number_next': codigo}, where=[('id', '=', regCodigo[0]['id'])])
            _pessoa_id = db_registro.insert(table='adm_pessoa', vals=vals)
        if _c_s_nf == 'S':
            company_id = funct_get_empresa(db_registro, 'rh.funcionario', company_sem_nf_id)
        else:
            company_id = funct_get_empresa(db_registro, 'rh.funcionario', company_com_nf_id)

        args_rh_funcionario = [('company_id', '=', company_id), ('pessoa', '=', _pessoa_id)]
        _regFuncionario = db_registro.select(table='rh_funcionario', fields=['id'], where=args_rh_funcionario)
        if _regFuncionario:
            _funcionario_id = _regFuncionario[0]['id']
        else:
            vals = {'company_id': company_id, 'pessoa': _pessoa_id}
            _funcionario_id = db_registro.insert(table='rh_funcionario', vals=vals)
        _destinatario_id = False
        _endereco_nf_id = False
        if _cli_for != '00000':
            if _entrada_saida == 'E':
                if _tipoMov == 'D':
                    _tabela = 'cre01'
                else:
                    _tabela = 'cpg01'

            else:
                if _tipoMov == 'D':
                    _tabela = 'cpg01'
                else:
                    _tabela = 'cre01'

            _destinatario_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_cli_for,
                reg_tabela_legado=_tabela,
                reg_check_table='adm_pessoa',
                reg_empresa_tipo=_c_s_nf
            )

            if not _destinatario_id:
                raise Exception(
                    'Est03: Cliente/Fornecedor nao encontrado na tabela legado_controle. Chave_legado - %s' %
                    _cli_for
                )

            _regEndereco = db_registro.select(table='adm_pessoa', fields=['endereco_nf_id'], where=[('id', '=', _destinatario_id)])
            if _regEndereco:
                _endereco_nf_id = _regEndereco[0]['endereco_nf_id']

        pis_id = cofins_id = icms_id = False
        res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', 'ICMS')])
        if res:
            icms_id = res[0]['id']
        res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', 'PIS')])
        if res:
            pis_id = res[0]['id']
        res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', 'COFINS')])
        if res:
            cofins_id = res[0]['id']
        if _entrada_saida == 'E':
            if _tipo_mov == 'N':
                _name = 'Compras'
            elif _tipo_mov == 'R':
                _name = 'Ressuprimento Entranda'
            elif _tipo_mov in ('D', 'E'):
                _name = 'Devolucao Entrando'
            elif _tipo_mov == 'A':
                _name = 'Inventario Entrada'
            elif _tipo_mov == 'T':
                _name = 'Transferencia Entrando'
            elif _tipo_mov == 'O':
                _name = 'Outras entradas'
        elif _entrada_saida == 'S':
            if _tipo_mov == 'N':
                _name = 'Vendas'
            elif _tipo_mov == 'R':
                _name = 'Ressuprimento Saindo'
            elif _tipo_mov in ('D', 'E'):
                _name = 'Devolucao Saindo'
            elif _tipo_mov == 'A':
                _name = 'Inventario Saida'
            elif _tipo_mov == 'T':
                _name = 'Transferencia Saindo'
            elif _tipo_mov == 'O':
                _name = 'Outras saidas'

        if _tipo_mov == 'N' and _tipo_programa == 'M':  # TIPO CUPOM
            _tipo_mov = 'C'

        _estoque_operacao_dados = db_registro.select(table='estoque_natureza_operacao', fields=['id'], where=[('name', '=', _name), ('active', '=', True)])
        if not _estoque_operacao_dados:
            estmog_codigo = _tipo_mov
            if _tipo_programa == 'K': # Requisicao de almoxarifado
                estmog_codigo = 'R'
            elif _tipo_mov == 'R': # Requisicao de ressuprimento
                estmog_codigo = 'S'

            origem = db_registro.select(table='estoque_movimento_origem', fields=['id'], where=[('estmog_codigo', '=', estmog_codigo)])
            vals = {
                'active': True,
                'estnto_atualizar_saldo': True,
                'estnto_gerar_estatistica': True,
                'estnto_gerar_financeiro': True,
                'estnto_tipo_operacao': _entrada_saida,
                'name': _name
            }
            if origem:
                vals['estnto_movimento_origem_id'] = origem[0]['id']

            _estoque_operacao_id = db_registro.insert(table='estoque_natureza_operacao', vals=vals)
        else:
            _estoque_operacao_id = _estoque_operacao_dados[0]['id']

        if operacao in ('w', 'r'):  #Gravar

            if operacao == 'w' and legado_mov_item_id:
                return True

            # Verificar se já foi importado(Capa)
            _fiscal_nf_capa_id = False
            _requisicao_id = False
            if _cfop != '0000':
                _regCfop = db_registro.select(table='fiscal_cfop', fields=['id', 'movimento_tipo'], where=[('codigo', '=', _cfop)])
                if not _regCfop:
                    raise Exception('Est03: CFOP nao cadastrado - %s', _cfop)

                args_cfop_natureza = [('cfop_id', '=', _regCfop[0]['id']), ('fisnat_operacao_id', '=', _estoque_operacao_id)]
                _regNatureza = db_registro.select(table='fiscal_cfop_natureza', fields=['id'], where=args_cfop_natureza)
                if _regNatureza:
                    _natureza_id = _regNatureza[0]['id']
                else:
                    valsNatureza = {
                        'cfop_id': _regCfop[0]['id'],
                        'name': 'A definir',
                        'active': True,
                        'tipo': _entrada_saida,
                        'fisnat_operacao_id': _estoque_operacao_id,
                    }
                    _natureza_id = db_registro.insert(table='fiscal_cfop_natureza', vals=valsNatureza)
                _movimento_tipo = _regCfop[0]['movimento_tipo']
                valsNatEncargos = {}
                if _movimento_tipo != 'T':
                    valsNatEncargos = {
                        'natureza_id': _natureza_id,
                    }
                    if pis_id:
                        valsNatEncargos['encargo_id'] = pis_id
                        end_id = db_registro.select(
                            table='fiscal_cfop_natureza_encargos', fields=['id'], where=[('encargo_id', '=', pis_id),
                                                                                         ('natureza_id', '=', _natureza_id)])
                        if not end_id:
                            db_registro.insert(table='fiscal_cfop_natureza_encargos', vals=valsNatEncargos)

                    if cofins_id:
                        valsNatEncargos['encargo_id'] = cofins_id
                        end_id = db_registro.select(
                            table='fiscal_cfop_natureza_encargos', fields=['id'], where=[('encargo_id', '=', cofins_id),
                                                                                         ('natureza_id', '=', _natureza_id)])
                        if not end_id:
                            db_registro.insert(table='fiscal_cfop_natureza_encargos', vals=valsNatEncargos)
                    if icms_id:
                        valsNatEncargos['encargo_id'] = icms_id
                        end_id = db_registro.select(table='fiscal_cfop_natureza_encargos', fields=['id'],
                                                    where=[('encargo_id', '=', icms_id), ('natureza_id', '=', _natureza_id)])
                        if not end_id:
                            db_registro.insert(table='fiscal_cfop_natureza_encargos', vals=valsNatEncargos)
                if _c_s_nf == 'S':
                    company_id = funct_get_empresa(db_registro, 'fiscal.nf.capa', company_sem_nf_id)
                else:
                    company_id = funct_get_empresa(db_registro, 'fiscal.nf.capa', company_com_nf_id)

                if _tipo_programa != 'M':  # Nota fiscal, nao cupom

                    args_nf = [
                        ('company_id', '=', company_id), ('numero_nf', '=', _documento), ('serie_nf', '=', _serie)
                    ]

                    if _cli_for != '00000':
                        args_nf.append(('destinatario_id', '=', _destinatario_id))

                    reg_NF = db_registro.select(table='fiscal_nf_capa', fields=['id'], where=args_nf)
                    if reg_NF:
                        _fiscal_nf_capa_id = reg_NF[0]['id']
                    else:
                        valsFiscal = {
                            'company_id': company_id,
                            'data_emissao': sigDatas(_data_documento).getDataNoFormato('N', 'S', '/'),
                            'data_saida_entrada': sigDatas(_data_movimento).getDataNoFormato('N', 'S', '/'),
                            'numero_nf': _documento,
                            'serie_nf': _serie,
                            'natureza_id': _natureza_id,
                            'state': '3',
                            'tipo': _entrada_saida,
                            'finalidade_emissao': '1',
                        }
                        if _destinatario_id:
                            valsFiscal['destinatario_id'] = _destinatario_id
                        elif not _destinatario_id and _serie == 'TR':
                            pass
                        else:
                            raise Exception('Est03: Destinatario nao cadastrado - %s' % _cli_for)

                        if _endereco_nf_id:
                            valsFiscal['destinatario_endereco_id'] = _endereco_nf_id

                        _fiscal_nf_capa_id = db_registro.insert(table='fiscal_nf_capa', vals=valsFiscal)
            if _tipo_mov == 'R': #Requisicao
                if _c_s_nf == 'S':
                    company_id = funct_get_empresa(db_registro, 'estoqueressuprimento.requisicao.capa', company_sem_nf_id)
                else:
                    company_id = funct_get_empresa(db_registro, 'estoqueressuprimento.requisicao.capa', company_com_nf_id)

                regRequisicao = db_registro.select(
                    table='estoqueressuprimento_requisicao_capa', fields=['id'], where=[('company_id', '=', company_id),
                                                                                        ('numero', '=', _documento)])
                if regRequisicao:
                    _requisicao_id = regRequisicao[0]['id']
                else:
                    valsRequisicao = {
                        'company_id': company_id,
                        'numero': _documento,
                        'data': sigDatas(_data_documento).getDataNoFormato('N', 'S', '/'),
                        'state': 'O',
                        'tipo': 'QR',
                    }
                    if _entrada_saida == 'E':
                        valsRequisicao['local_entrada_id'] = _local_id
                    else:
                        valsRequisicao['local_saida_id'] = _local_id
                    _requisicao_id = db_registro.insert(table='estoqueressuprimento_requisicao_capa', vals=valsRequisicao)
            if _c_s_nf == 'S':
                company_id = funct_get_empresa(db_registro, 'estoque.movimento.capa', company_sem_nf_id)
            else:
                company_id = funct_get_empresa(db_registro, 'estoque.movimento.capa', company_com_nf_id)


            vals = {
                'company_id': company_id,
                'data': sigDatas(_data_movimento).getDataNoFormato('N', 'S', '/'),
                'estmvc_operacao_id': _estoque_operacao_id,
                'funcionario_id': _funcionario_id,
                'numero_documento': _documento,
                'state': 'F', # Movimento Efetivado
            }
            args = []
            if _fiscal_nf_capa_id:
                vals['nf_capa_id'] = _fiscal_nf_capa_id
                args = [('nf_capa_id', '=', _fiscal_nf_capa_id)]
            if _requisicao_id:
                vals['requisicaoressuprimento_id'] = _requisicao_id
                args = [('requisicaoressuprimento_id', '=', _requisicao_id)]
            if args:
                _regMovimCapa = db_registro.select(table='estoque_movimento_capa', fields=['id'], where=args)
                if _regMovimCapa:
                    _movim_capa_id = _regMovimCapa[0]['id']
                else:
                    _movim_capa_id = db_registro.insert(table='estoque_movimento_capa', vals=vals)
            else:
                _movim_capa_id = db_registro.insert(table='estoque_movimento_capa', vals=vals)
            company_id = funct_get_empresa(db_registro, 'estoque.local.locacao', company_com_nf_id)
            _embalagens = db_registro.select(table='estoque_local_locacao', fields=['estllc_produto_embalagem_id'],
                                             where=[('produto_id', '=', _produto_id), ('active', '=', True),
                                                    ('local_id', '=', _local_id), ('grade_id', '=', _grade_id),
                                                    ('company_id', '=', company_id)])
            if not _embalagens:
                raise Exception('Est03: Nao ha embalagens cadastrada para o produto - %s' % _item)

            _produto_embalagem_id = _embalagens[0]['estllc_produto_embalagem_id']
            _cfops = db_registro.select(table='fiscal_cfop', fields=['id'], where=[('codigo', '=', _cfop)])
            if _cfops:
                _cfop_id = _cfops[0]['id']
            else:
                _cfop_id = False

            valor_total_item = float(_preco_na_nf) - float(_valor_icms_sub_nf)
            vals = {
                'capa_id': _movim_capa_id,
                'produto_id': _produto_id,
                'grade_id': _grade_id,
                'local_id': _local_id,
                'quantidade': _quantidade,
                'sequencia': _sequencia,
                'vr_unitario': (float(_preco_na_nf) - float(_valor_icms_sub_nf)) / float(_quantidade),
                'vr_total_item': valor_total_item,
                # O VALOR DO ITEM RATEADO É INSERIDO PORQUE NA CONSULTA DE MOVIMENTO DO ESTOQUE,
                # ESTE CAMPO É O VALOR TOTAL
                'vr_total_item_rateado': valor_total_item,
                'vr_desconto': _desconto,
                'estmvi_saidas_custo_corrigido_total': _custo_corr,
                'estmvi_saidas_custo_medio_contabil_total': _preco_medio,
                'estmvi_saidas_custo_reposicao_total': _preco_reposicao,
                'estmvi_saidas_ultimo_custo_total': _ultimo_custo,
            }
            if _produto_embalagem_id:
                vals['produto_embalagem_id'] = _produto_embalagem_id
                embalagem_id = db_registro.select(table='estoque_produto_embalagem', fields=['embalagem_id'],
                                                  where=[('id', '=', _produto_embalagem_id), ('active', '=', True)])
                if embalagem_id:
                    quantidade_embalagem = db_registro.select(table='estoque_embalagem',
                                                              fields=['quantidade_na_embalagem'],
                                                              where=[('id', '=', embalagem_id[0]['embalagem_id'])])
                    vals['vr_unitario'] *= quantidade_embalagem[0]['quantidade_na_embalagem']
            if _cfop_id:
                vals['cfop_id'] = _cfop_id
                cfop_natureza_dados = db_registro.select(
                    table='fiscal_cfop_natureza',
                    fields=['id'],
                    where=[('tipo', '=', _entrada_saida), ('active', '=', True), ('cfop_id', '=', _cfop_id)]
                )
                if cfop_natureza_dados:
                    cfop_natureza_id = cfop_natureza_dados[0]['id']
                    vals['natureza_id'] = cfop_natureza_id
            if _fiscal_nf_capa_id:
                vals['nf_id'] = _fiscal_nf_capa_id

            if operacao == 'w':
                _movim_item_id = db_registro.insert(table='estoque_movimento_itens', vals=vals)

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est03I',
                    reg_chave_legado=_chaveEst03I,
                    reg_chave_sigerp=_movim_item_id,
                    reg_check_table='estoque_movimento_itens',
                    reg_empresa_tipo=_c_s_nf
                )
            else:
                # SE O REGISTRO DO MOVIMENTO EXISTIR, É FEITO A ATUALIZACAO.
                if legado_mov_item_id:
                    _movim_item_id = legado_mov_item_id
                    db_registro.update(
                        table='estoque_movimento_itens', values=vals, where=[('id', '=', legado_mov_item_id)]
                    )
                # SE O REGISTRO SO MOVIMENTO NAO EXISTIR, É FEITO A INCLUSAO.
                else:
                    _movim_item_id = db_registro.insert(table='estoque_movimento_itens', vals=vals)

                    db_registro.create_legado_controle(
                        reg_empresa_legado=empresa_legado,
                        reg_tabela_legado='est03I',
                        reg_chave_legado=_chaveEst03I,
                        reg_chave_sigerp=_movim_item_id,
                        reg_check_table='estoque_movimento_itens',
                        reg_empresa_tipo=_c_s_nf
                    )

            _tributacao_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_cod_tributacao,
                reg_tabela_legado='est14',
                reg_check_table='fiscal_tributacao',
                reg_empresa_tipo=_c_s_nf
            )

            if _tipo_mov == 'C' and _tipo_programa == 'M':  # tipo cupom
                cupom_cfop_id = False

                if _tributacao_id:
                    # PROCURA O ID DO ENCARGO QUE TENHA A SIGLA IGUAL A ICMS
                    encargos_icms_dados = db_registro.select(
                        table='fiscal_encargo', fields=['id'], where=[('sigla', '=', 'ICMS')]
                    )
                    if encargos_icms_dados:
                        encargo_icms_id = encargos_icms_dados[0]['id']

                        args_cupom_tributacao_encargo = [
                            ('encargo_id', '=', encargo_icms_id), ('produto_id', '=', _produto_id),
                            ('tributacao_id', '=', _tributacao_id)
                        ]
                        # PROCURA A TRIBUTACAO DO ITEM QUE TENHA ENCARGO DE ICMS
                        cupom_tributacao_encargo_dados = db_registro.select(
                            table='fiscal_tributacao_encargos', fields=['cst_id'], where=args_cupom_tributacao_encargo
                        )
                        if cupom_tributacao_encargo_dados:
                            cupom_cst_id = cupom_tributacao_encargo_dados[0]['cst_id']

                            cst_subst_dados = db_registro.select(
                                table='fiscal_cst', fields=['id'], where=[('tipo', '=', 'U'), ('id', '=', cupom_cst_id)]
                            )
                            if cst_subst_dados:
                                cfop_dados = db_registro.select(
                                    table='fiscal_cfop', fields=['id'], where=[('codigo', '=', '5403')]
                                )
                                if cfop_dados:
                                    cupom_cfop_id = cfop_dados[0]['id']

                if not cupom_cfop_id:
                    cfop_dados = db_registro.select(
                        table='fiscal_cfop', fields=['id'], where=[('codigo', '=', '5102')]
                    )
                    cupom_cfop_id = cfop_dados[0]['id']

                values_mov_item = {
                    'cfop_id': cupom_cfop_id
                }
                db_registro.update(
                    table='estoque_movimento_itens',
                    values=values_mov_item,
                    where=[('id', '=', _movim_item_id)]
                )

            if _tributacao_id:
                args = [('codigo', '=', _cst[1:]), ('encargo_id', '=', icms_id)]
                _reg_cst = db_registro.select(table='fiscal_cst', fields=['id'], where=args)
                if _reg_cst:
                    _cst_id = _reg_cst[0]['id']
                else:
                    _cst_id = False
                vals = {
                    'aliquota': _aliquota_icms,
                    'movimento_itens_id': _movim_item_id,
                    'percentual_mva': _mva,
                    'tributacao_id': _tributacao_id,
                    'vr_base_calculo': _base_icms,
                    'vr_encargo': _valor_icms,
                }
                if _fiscal_nf_capa_id:
                    vals['fisnit_capa_id'] = _fiscal_nf_capa_id
                if _cst_id:
                    vals['cst_id'] = _cst_id
                if icms_id:
                    vals['encargo_id'] = icms_id
                # CRIACAO OU ATUALIZACAO DA TRIBUTACAO UTILIZADA NO ITEM
                if operacao == 'w':
                    db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)
                elif operacao == 'r' and vals:
                    args_nf_item_tributacao = map(lambda i: (i[0], '=', i[1]), vals.items())
                    nf_item_tributacao_dados = db_registro.select(
                        table='fiscal_nf_item_tributacao', fields=['id'], where=args_nf_item_tributacao
                    )
                    if nf_item_tributacao_dados:
                        nf_item_tributacao_id = nf_item_tributacao_dados[0]['id']
                        db_registro.update(
                            table='fiscal_nf_item_tributacao', values=vals, where=[('id', '=', nf_item_tributacao_id)]
                        )
                    else:
                        db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)

                if pis_id:
                    args_tribut_encargo = [('encargo_id', '=', pis_id), ('produto_id', '=', _produto_id),
                                           ('tipo_tributacao', '=', 'T')]

                    _dadosEncargos = db_registro.select(
                        table='fiscal_tributacao_encargos', fields=['tributacao_id'], where=args_tribut_encargo
                    )
                    if _dadosEncargos:
                        tributacao_id = _dadosEncargos[0]['tributacao_id']
                        _dadosTributacao = db_registro.select(table='fiscal_tributacao', fields=['cst_id', 'aliquota'], where=[('id', '=', tributacao_id)])
                        if _dadosTributacao:
                            _dadosCst = db_registro.select(table='fiscal_cst', fields=['tipo'], where=[('id', '=', _dadosTributacao[0]['cst_id'])])
                            if _dadosCst:
                                vals = {
                                    'aliquota': _dadosTributacao[0]['aliquota'],
                                    'cst_id': _dadosTributacao[0]['cst_id'],
                                    'encargo_id': pis_id,
                                    'movimento_itens_id': _movim_item_id,
                                    'tributacao_id': _dadosEncargos[0]['tributacao_id'],
                                    'vr_base_calculo': float(_preco_na_nf),
                                    'vr_encargo': 0,
                                }
                                if _fiscal_nf_capa_id:
                                    vals['fisnit_capa_id'] = _fiscal_nf_capa_id
                                if _dadosCst[0]['tipo'] == 'T':
                                    vals['vr_base_calculo'] = float(_preco_na_nf) - float(_valor_icms_sub_nf)
                                    vals['vr_encargo'] = (float(_preco_na_nf) - float(_valor_icms_sub_nf)) * float(
                                        _dadosTributacao[0]['aliquota']) / 100
                                # CRIACAO OU ATUALIZACAO DA TRIBUTACAO UTILIZADA NO ITEM
                                if operacao == 'w':
                                    db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)
                                elif operacao == 'r' and vals:
                                    args_nf_item_tributacao_pis = map(lambda i: (i[0], '=', i[1]), vals.items())
                                    nf_item_tributacao_pis_dados = db_registro.select(
                                        table='fiscal_nf_item_tributacao', fields=['id'], where=args_nf_item_tributacao_pis
                                    )
                                    if nf_item_tributacao_pis_dados:
                                        nf_item_tributacao_id = nf_item_tributacao_pis_dados[0]['id']
                                        db_registro.update(
                                            table='fiscal_nf_item_tributacao', values=vals, where=[('id', '=', nf_item_tributacao_id)]
                                        )
                                    else:
                                        db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)
                if cofins_id:
                    args_tribut_encargo = [('encargo_id', '=', cofins_id), ('produto_id', '=', _produto_id), ('tipo_tributacao', '=', 'T')]
                    _dadosEncargos = db_registro.select(table='fiscal_tributacao_encargos', fields=['tributacao_id'], where=args_tribut_encargo)
                    if _dadosEncargos:
                        _dadosTributacao = db_registro.select(table='fiscal_tributacao', fields=['cst_id', 'aliquota'], where=[('id', '=', _dadosEncargos[0]['tributacao_id'])])
                        if _dadosTributacao:
                            _dadosCst = db_registro.select(table='fiscal_cst', fields=['tipo'], where=[('id', '=', _dadosTributacao[0]['cst_id'])])
                            if _dadosCst:
                                vals = {
                                    'aliquota': _dadosTributacao[0]['aliquota'],
                                    'cst_id': _dadosTributacao[0]['cst_id'],
                                    'encargo_id': cofins_id,
                                    'movimento_itens_id': _movim_item_id,
                                    'tributacao_id': _dadosEncargos[0]['tributacao_id'],
                                    'vr_base_calculo': float(_preco_na_nf),
                                    'vr_encargo': 0,
                                }
                                if _fiscal_nf_capa_id:
                                    vals['fisnit_capa_id'] = _fiscal_nf_capa_id
                                if _dadosCst[0]['tipo'] == 'T':
                                    vals['vr_base_calculo'] = float(_preco_na_nf) - float(_valor_icms_sub_nf)
                                    vals['vr_encargo'] = (float(_preco_na_nf) - float(_valor_icms_sub_nf)) * float(
                                        _dadosTributacao[0]['aliquota']) / 100
                                # CRIACAO OU ATUALIZACAO DA TRIBUTACAO UTILIZADA NO ITEM
                                if operacao == 'w':
                                    db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)
                                elif operacao == 'r' and vals:
                                    args_nf_item_tributacao_cofins = map(lambda i: (i[0], '=', i[1]), vals.items())
                                    nf_item_tributacao_cofins_dados = db_registro.select(
                                        table='fiscal_nf_item_tributacao', fields=['id'], where=args_nf_item_tributacao_cofins
                                    )
                                    if nf_item_tributacao_cofins_dados:
                                        nf_item_tributacao_id = nf_item_tributacao_cofins_dados[0]['id']
                                        db_registro.update(
                                            table='fiscal_nf_item_tributacao', values=vals, where=[('id', '=', nf_item_tributacao_id)]
                                        )
                                    else:
                                        db_registro.insert(table='fiscal_nf_item_tributacao', vals=vals)
        elif operacao == 'd':  # deletar
            args_leg_controle = [
                ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est03I'), ('chave_legado', '=', _chaveEst03I)
            ]

            _regLegado = db_registro.select_legado_controle(
                reg_fields=['id', 'chave_sigerp'],
                reg_args=args_leg_controle,
                reg_check_table='estoque_movimento_itens',
                reg_empresa_tipo=_c_s_nf
            )
            if not _regLegado:
                raise Exception(
                    'Est03: Movimento nao foi encontrado na legado controle. Chave_legado - %s, Empresa - %s' %
                    (_chaveEst03I, str(empresa_legado))
                )

            _movim_item_id = _regLegado[0]['chave_sigerp']
            legado_controle_id = _regLegado[0]['id']

            db_registro.delete(table='estoque_movimento_itens', where=[('id', '=', _movim_item_id)])

            db_registro.delete_legado_controle(
                reg_args=[('id', '=', legado_controle_id)],
                reg_check_table='estoque_movimento_itens',
                reg_empresa_tipo=_c_s_nf
            )
        else:
            raise Exception('Est03: Operacao nao identificada: %s' % operacao)
        return True
    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# CATEGORIAS
def gerarEst04(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _chave1 = _chave[:3]
        _chave2 = _chave[3:6]
        _chave3 = _chave[6:9]
        _chave4 = _chave[9:]
        _nome = funct_retira_acentos(lista_dados[1].strip())
        # _tipo = lista_dados[2]
        # _contaCustos = lista_dados[3]
        # _contaCtb = lista_dados[4]

        legado_categoria_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est04',
            reg_check_table='estoque_categoria',
        )

        _chavePai = False
        if _chave2 != '000':
            if _chave3 == '000':
                _chavePai = _chave1 + '000000000'
            else:
                if _chave4 == '000':
                    _chavePai = _chave1 + _chave2 + '000000'
                else:
                    _chavePai = _chave1 + _chave2 + _chave3 + '000'

        if operacao == 'w' or (operacao == 'r' and not legado_categoria_id):  # Gravar
            if not legado_categoria_id:
                vals = {'name': _nome}
                if _chavePai:
                    legado_categoria_pai_id = db_registro.controle_get_chave_sigerp(
                        reg_empresa_legado=empresa_legado,
                        reg_chave=_chavePai,
                        reg_tabela_legado='est04',
                        reg_check_table='estoque_categoria',
                    )

                    if legado_categoria_pai_id:
                        vals['parent_id'] = legado_categoria_pai_id
                obj_estoque_categoria = context.get('ctx_est_obj_estoque_categoria')
                # CRIA UMA NOVA CATEGORIA
                categoria_id = obj_estoque_categoria.create(db_registro.cr, db_registro.uid, vals, context)

                # CRIA A LEGADO CONTROLE PARA A CATEGORIA
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est04',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=categoria_id,
                    reg_check_table='estoque_categoria',
                )

        elif operacao == 'r':  # Regravar

            if legado_categoria_id:
                categoria_id = legado_categoria_id
            else:
                categoria_id = False

            # CRIA A LEGADO CONTROLE CASO NAO EXISTA
            if not legado_categoria_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est04',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=categoria_id,
                    reg_check_table='estoque_categoria',
                )
            # ATUALIZA A CHAVE_SIGERP
            else:
                args_update_legado = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est04'), ('chave_legado', '=', _chave)
                ]
                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': categoria_id},
                    reg_args=args_update_legado,
                    reg_check_table='estoque_categoria',
                )

            vals = {'name': _nome}
            db_registro.update(table='estoque_categoria', values=vals, where=[('id', '=', categoria_id)])

        elif operacao == 'd':  # Deletar
            if not legado_categoria_id:
                raise Exception(
                    'Est04: Delecao nao realziada. Registro nao encontrado na legado_controle. Chave_legado - %s'
                    % _chave
                )

            db_registro.delete(table='estoque_categoria', where=[('id', '=',  legado_categoria_id)])

            args_legado_controle = [
                ('empresa', '=', empresa_legado), ('chave_sigerp', '=', legado_categoria_id), ('tabela_legado', '=', 'est04')
            ]

            db_registro.delete_legado_controle(
                reg_args=args_legado_controle,
                reg_check_table='estoque_categoria'
            )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# GRADE
def gerarEst05(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:

        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        # _chave = _item + '-' + _grade
        _nome = funct_retira_acentos(lista_dados[2].strip())
        _nomeResumido = funct_retira_acentos(lista_dados[3].strip())
        if len(_nomeResumido) > 25:
            _nomeResumido = _nomeResumido[:25]
        _minimo = lista_dados[4]
        _maximo = lista_dados[5]
        # _listaPesquisa = lista_dados[11]
        # _qtdEntrada = lista_dados[12]
        # _dataEntrada = lista_dados[13]
        # _qtdSaida = lista_dados[14]
        # _dataSaida = lista_dados[15]
        _itemAtivo = lista_dados[20]
        if _itemAtivo == 'N':
            _itemAtivo = False
        else:
            _itemAtivo = True
        # _dataInclusao = sigDatas(lista_dados[21]).getDataNoFormato('N', 'S', '/')
        # if _codNumerico:
            # _codigoParaBaixa = formataString(lista_dados[22].lstrip('0').rstrip(), 6, '0', 'E')
        # else:
            # _codigoParaBaixa = lista_dados[22].strip()
        # _gradeParaBaixa = lista_dados[23].strip()
        #Verificar se modulo concorrencia está instalado
        _concorrenciaInstalado = db_registro.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'concorrencia'), ('state', '=', 'installed')]
        )
        #Verificar se modulo balanca está instalado
        _balancaInstalado = db_registro.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'balanca'), ('state', '=', 'installed')]
        )
        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(
            table='estoque_produto', fields=['id', 'codigo'], where=[('company_id', '=', company_id), ('codigo', '=', _item)]
        )
        if not _regProduto:
            raise Exception('Est05: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _codigo = _regProduto[0]['codigo']
        valsProduto = {'name': _nome}
        if _concorrenciaInstalado:
            _periodoDia = lista_dados[16]
            if _periodoDia == 'S':
                _periodoDia = True
            else:
                _periodoDia = False
            _periodoSemana = lista_dados[17]
            if _periodoSemana == 'S':
                _periodoSemana = True
            else:
                _periodoSemana = False
            _periodoQuinzena = lista_dados[18]
            if _periodoQuinzena == 'S':
                _periodoQuinzena = True
            else:
                _periodoQuinzena = False
            _periodoMes = lista_dados[19]
            if _periodoMes == 'S':
                _periodoMes = True
            else:
                _periodoMes = False
            valsProduto['pesquisa_diaria'] = _periodoDia
            valsProduto['pesquisa_semanal'] = _periodoSemana
            valsProduto['pesquisa_quinzenal'] = _periodoQuinzena
            valsProduto['pesquisa_mensal'] = _periodoMes
            if _periodoDia or _periodoSemana or _periodoQuinzena or _periodoMes:
                valsProduto['acompanhar_precos_concorrentes'] = True
            else:
                valsProduto['acompanhar_precos_concorrentes'] = False
        if _balancaInstalado:
            _balanca = lista_dados[6]
            if _balanca == 'S':
                _balanca = True
            else:
                _balanca = False
            _pesoUnidade = lista_dados[7]
            _validade = lista_dados[8].replace(' ', '0')
            _tecla = re_sub('[^0-9]', '', lista_dados[9].strip())
            if not _tecla:
                _tecla = 0
            _setor = lista_dados[10]
            valsProduto['produto_de_balanca'] = _balanca
            if _balanca:
                valsProduto['peso_unidade'] = _pesoUnidade
                valsProduto['validade'] = _validade
            else:
                valsProduto['peso_unidade'] = 'N'
            valsProduto['tecla'] = _tecla
            # achar setor
            legado = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_setor,
                reg_tabela_legado='est21',
                reg_check_table='balanca_setor',
            )
            if not legado:
                _setor = False
            else:
                _setor = legado
                valsProduto['setor_id'] = _setor
        result = db_registro.select(table='estoque_produto', fields=['id'], where=[('name', '=', _nome)]) or []
        if result:
            nome_produto_id = result[0]['id']
            if _produto_id != nome_produto_id or result.__len__() > 1:
                valsProduto['name'] = _nome + '-' + str(_codigo)
        if _grade == 'UN' and not _itemAtivo:
            valsProduto['active'] = False
        result = db_registro.update(table='estoque_produto', values=valsProduto, where=[('id', '=', _produto_id)])
        if not result:
            raise Exception('Est05: Produto nao foi alterado. ID - %s, Codigo - %s' % (str(_produto_id)), str(_item))

        valsGrade = {
            'produto_id': _produto_id,
            'grade': _grade,
            'active': _itemAtivo,
            'descricao': _nome,
            'minimo': _minimo,
            'maximo': _maximo,
        }

        #Verificar se modulo ecf está instalado
        _ecfInstalado = db_registro.select(
            table='ir_module_module', fields=['id'],
            where=[('name', '=', 'ecf'), ('state', '=', 'installed')]
        )

        if _ecfInstalado:
            valsGrade['descricao_resumida'] = _nomeResumido

        grade_dados = db_registro.select(
            table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)]
        )
        if grade_dados:
            _idGrade = grade_dados[0]['id']
        else:
            _idGrade = False
        if operacao == 'w':  # Gravar
            if _idGrade:
                db_registro.update(table='estoque_produto_grade', values=valsGrade, where=[('id', '=', _idGrade)])
                grade_id = _idGrade
            else:
                grade_id = db_registro.insert(table='estoque_produto_grade', vals=valsGrade)
            selfGrade = context.get('ctx_est_obj_estoque_produto_grade')
            name = _estpgr_get_name(selfGrade, db_registro.cr, db_registro.uid, [grade_id], [], [], context=None)
            name = name.get(grade_id)
            db_registro.update(table='estoque_produto_grade', values={'name': name},
                               where=[('id', '=', grade_id)])
        elif operacao == 'r':  # Regravar
            if not _idGrade:
                grade_id = db_registro.insert(table='estoque_produto_grade', vals=valsGrade)
            else:
                db_registro.update(table='estoque_produto_grade', values=valsGrade, where=[('id', '=', _idGrade)])
                grade_id = _idGrade
            selfGrade = context.get('ctx_est_obj_estoque_produto_grade')
            name = _estpgr_get_name(selfGrade, db_registro.cr, db_registro.uid, [grade_id], [], [], context=None)
            name = name.get(grade_id)
            db_registro.update(table='estoque_produto_grade', values={'name': name},
                               where=[('id', '=', grade_id)])
        elif operacao == 'd':  # deletar
            if _idGrade:
                db_registro.delete(table='estoque_produto_grade', where=[('id', '=', _idGrade)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False

# SALDO
def gerarEst10(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        # inicio = CalcTempo()
        # a = CalcTempo()
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _chave = _item + '-' + _grade
        _local = lista_dados[2]
        # Nao existe local 00
        if _local == '00':
            return True
        _unidade = lista_dados[3].strip().upper()
        _qtdEmbalagem = lista_dados[4][:5] + '.' + lista_dados[4][5:]
        _lastro = lista_dados[5]
        # _capacPalete = lista_dados[6]
        _locacao = lista_dados[7].strip()
        _saldoC = lista_dados[8][:10] + '.' + lista_dados[8][10:]
        _requisitadoC = lista_dados[9][:10] + '.' + lista_dados[9][10:]
        _saldoS = lista_dados[10][:10] + '.' + lista_dados[10][10:]
        _requisitadoS = lista_dados[11][:10] + '.' + lista_dados[11][10:]
        _percentualLastro = lista_dados[12]
        if not _percentualLastro.__len__():
            _percentualLastro = False
        # a.apurar('Recebimento')
        # a = CalcTempo()
        # achar grade
        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(
            table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _item)]
        )
        if not _regProduto:
            raise Exception('Est10: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        regsGrade = db_registro.select(
            table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)]
        )
        if not regsGrade:
            raise Exception(
                'Est10: Grade nao encontrada para o item. Produto_id - %s, Grade - %s' % (str(_produto_id), _grade)
            )

        _grade_id = regsGrade[0]['id']
        # a.apurar('Achar grade')
        # a = CalcTempo()
        #achar local_id
        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        regsLocal = db_registro.select(
            table='estoque_local', fields=['id'], where=[('numero', '=', _local), ('company_id', '=', company_id)]
        )
        if not regsLocal:
            raise Exception('Est10: Local nao encontrado. Numero - %s' % _local)

        _local_id = regsLocal[0]['id']
        # a.apurar('Achar local')
        # a = CalcTempo()
        # achar repositor_id
        _chaveRepositor = '%s-%s' % (_local, _locacao)

        _pessoa_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chaveRepositor,
            reg_tabela_legado='est22L',
            reg_check_table='adm_pessoa',
        )

        _repositor_id = False
        if _pessoa_id:
            _dados_funcionario = db_registro.select(table='rh_funcionario', fields=['id'], where=[('pessoa', '=', _pessoa_id)])
            if _dados_funcionario:
                _repositor_id = _dados_funcionario[0]['id']
        # a.apurar('Achar repositor')
        if operacao == 'd':  #Deletar
            return True

        #achar a unidade, se nao tiver criar
        # a = CalcTempo()
        regsUnidade = db_registro.select(table='estoque_embalagem_unidade', fields=['id', 'name'], where=[('unidade', '=', _unidade)])
        if not regsUnidade:
            valsUnidade = {
                'unidade': _unidade,
                'name': _unidade + '-1',
                'fracao': False,
            }

            db_registro.insert(table='estoque_embalagem_unidade', vals=valsUnidade)

            regsUnidade = db_registro.select(table='estoque_embalagem_unidade', fields=['id', 'name'], where=[('unidade', '=', _unidade)])
            if not regsUnidade:
                raise Exception('Est10: Unidade nao cadastrada. Unidade - %s' % _unidade)

        _unidade_id = regsUnidade[0]['id']
        _unidade_name = regsUnidade[0]['name']
        # a.apurar('Achar unidade')
        # a = CalcTempo()
        #achar a embalagem, se nao tiver criar
        if float(_qtdEmbalagem) == 1:
            _sigla = _unidade.upper()
        else:
            if float(_qtdEmbalagem).is_integer():
                _qtd = str(int(float(_qtdEmbalagem)))
            else:
                _qtd = str(float(_qtdEmbalagem))
            if len(_qtd) == 1:
                _sigla = _unidade.upper() + '0' + _qtd
            else:
                _sigla = _unidade.upper() + _qtd
        if len(_sigla) > 7:
            _sigla = _sigla[:7]
        regsUnidade = db_registro.select(table='estoque_embalagem', fields=['id'], where=[('sigla', '=', _sigla)])
        if regsUnidade:
            _embalagem_id = regsUnidade[0]['id']
        else:
            valsEmbalagem = {
                'name': _unidade_name,
                'sigla': _sigla,
                'unidade_id': _unidade_id,
                'quantidade_na_embalagem': _qtdEmbalagem,
            }
            if _qtdEmbalagem > 1:
                if float(_qtdEmbalagem).is_integer():
                    valsEmbalagem['name'] = _unidade_name + ' com ' + str(int(float(_qtdEmbalagem)))
                else:
                    valsEmbalagem['name'] = _unidade_name + ' com ' + str(float(_qtdEmbalagem))
            _embalagem_id = db_registro.insert(table='estoque_embalagem', vals=valsEmbalagem)

        args_embalagem = [('produto_id', '=', _produto_id), ('embalagem_id', '=', _embalagem_id),
                          ('active', '=', True)]
        regsProdEmb = db_registro.select(table='estoque_produto_embalagem', fields=['id'], where=args_embalagem)
        if not regsProdEmb:
            valsProdEmb = {
                'produto_id': _produto_id,
                'embalagem_id': _embalagem_id,
                'active': True,
            }
            _prodEmb_id = db_registro.insert(table='estoque_produto_embalagem', vals=valsProdEmb)
        else:
            _prodEmb_id = regsProdEmb[0]['id']
        # a.apurar('Achar embalagem')
        # a = CalcTempo()

        #achar o produto no local, se nao existir, criar locacao
        company_id = funct_get_empresa(db_registro, 'estoque.local.locacao', company_com_nf_id)
        args_locacao = [('company_id', '=', company_id), ('local_id', '=', _local_id),
                        ('produto_id', '=', _produto_id), ('grade_id', '=', _grade_id),
                        ('active', '=', True)]

        _local_locacao_dados = db_registro.select(table='estoque_local_locacao', fields=['id'], where=args_locacao)
        valsLocacao = {
            'company_id': company_id,
            'locacao': _locacao,
            'local_id': _local_id,
            'produto_id': _produto_id,
            'grade_id': _grade_id,
            'active': True
        }
        _ressuprimentoInstalado = db_registro.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'estoqueressuprimento'),
                                                            ('state', '=', 'installed')]
        )
        if _ressuprimentoInstalado:
            valsLocacao['estllc_lastro'] = int(_lastro)  # COLUNA CRIADA PELO ESTOQUERESSUPRIMENTO
            if _percentualLastro:
                valsLocacao['estllc_percentual_lastro'] = int(_percentualLastro)
        if _repositor_id:
            valsLocacao['repositor_id'] = _repositor_id
        if _prodEmb_id:
            valsLocacao['estllc_produto_embalagem_id'] = _prodEmb_id
        if not _local_locacao_dados:
            local_locacao_id = db_registro.insert(table='estoque_local_locacao', vals=valsLocacao)
        else:
            local_locacao_id = _local_locacao_dados[0]['id']
            db_registro.update(table='estoque_local_locacao', values=valsLocacao, where=[('id', '=', local_locacao_id)])
        # a.apurar('Achar incluir locacao')
        # a = CalcTempo()
        #Verificar se existe o saldo NO dia, se nao existir criar
        company_id = funct_get_empresa(db_registro, 'estoque.produto.saldo', company_com_nf_id)
        hoje = date.today().strftime('%Y-%m-%d')
        valsSaldo = {
            'data_movimento': hoje,
            'saldo': float(_saldoC) + float(_saldoS) + float(_requisitadoC) + float(_requisitadoS),
            'local_id': _local_id,
            'company_id': company_id,
            'produto_id': _produto_id,
            'grade_id': _grade_id,
        }

        args_produto_saldo = [
            ('company_id', '=', company_id), ('local_id', '=', _local_id), ('produto_id', '=', _produto_id),
            ('grade_id', '=', _grade_id), ('data_movimento', '=', hoje)
        ]
        _dadosSaldo = db_registro.select(table='estoque_produto_saldo', fields=['id'], where=args_produto_saldo)
        if not _dadosSaldo:
            db_registro.insert(table='estoque_produto_saldo', vals=valsSaldo)
        else:
            values_saldo = {
                'saldo': float(_saldoC) + float(_saldoS),
            }
            saldo_id = _dadosSaldo[0]['id']
            db_registro.update(table='estoque_produto_saldo', values=values_saldo, where=[('id', '=', saldo_id)])

        # a.apurar('Atualizar saldo')
        # a = CalcTempo()
        _wmsInstalado = db_registro.select(
            table='ir_module_module', fields=['id'], where=[('name', '=', 'wms'),
                                                            ('state', '=', 'installed')]
        )
        if _wmsInstalado:
            """
            Atualiza embalagem_expedicao_id do produto qual a alteracao eh na empresa e no local
            informados na aba wms do parametro do estoque
            """
            if _grade_id and _produto_id:
                campos = ['estpar_local_id', 'estpar_empresa_id']
                obj_adm_utils = context.get('ctx_obj_adm_utils')
                est_par_dados = obj_adm_utils.admutl_get_parametros(db_registro.cr, db_registro.uid,
                                                                    'estoque', campos, context=context)
                estpar_local_id = est_par_dados.get('estpar_local_id')
                estpar_empresa_id = est_par_dados.get('estpar_empresa_id')
                if _local_id == estpar_local_id and company_id == estpar_empresa_id:
                    values_grade = {'embalagem_expedicao_id': _prodEmb_id}
                    args_grade = [('id', '=', _grade_id)]
                    db_registro.update(table='estoque_produto_grade',
                                       values=values_grade,
                                       where=args_grade)
        # a.apurar('Atualizar produto')
        # inicio.apurar('Tempo total')
        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# VENDEDOR
def gerarEst11(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _nome = lista_dados[1].strip()
        # _departamento = lista_dados[2]

        company_id = funct_get_empresa(db_registro, 'vendas.vendedor', company_com_nf_id)

        pessoa_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est11P',
            reg_check_table='adm_pessoa',
        )

        idVendedor = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est11',
            reg_check_table='vendas_vendedor',
        )

        vals_hr_funcionario = {
            'company_id': company_id,
            'active': True
        }

        if operacao == 'w': #Gravar
            if idVendedor:
                return True
            if pessoa_id:
                return True
            valsPessoa = {'nome': _nome, 'pessoa_tipo': 'F', 'active': True}

            ir_sequence_dados = db_registro.select(
                table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
            )

            sequence_id = ir_sequence_dados[0]['id']
            sequence_codigo = ir_sequence_dados[0]['number_next']

            valsPessoa['codigo'] = sequence_codigo

            proximo_codigo = int(sequence_codigo) + 1

            db_registro.update(table='ir_sequence', values={'number_next': proximo_codigo}, where=[('id', '=', sequence_id)])
            # CRIA O PARTNER
            partner_id = db_registro.insert(table='res_partner', vals={'name': _nome, 'notification_email_send': 'comment'})
            valsPessoa['partner_id'] = partner_id
            # CRIA PESSOA
            pessoa_id = db_registro.insert(table='adm_pessoa', vals=valsPessoa)
            # CRIA O LEGADO CONTROLE PARA A PESSOA
            db_registro.create_legado_controle(
                reg_empresa_legado=empresa_legado,
                reg_tabela_legado='est11P',
                reg_chave_legado=_chave,
                reg_chave_sigerp=pessoa_id,
                reg_check_table='adm_pessoa',
            )

            valsVendedor = {
                'pessoa_id': pessoa_id,
                'company_id': company_id,
                'active': True,
                'tipo': 'I',
            }
            # CRIA O VENDEDOR
            idVendedor = db_registro.insert(table='vendas_vendedor', vals=valsVendedor)
            # CRIA O LEGADO CONTROLE PARA O VENDEDOR
            db_registro.create_legado_controle(
                reg_empresa_legado=empresa_legado,
                reg_tabela_legado='est11',
                reg_chave_legado=_chave,
                reg_chave_sigerp=idVendedor,
                reg_check_table='vendas_vendedor',
            )

            rh_funcionario_dados = db_registro.select(
                table='rh_funcionario', fields=['id'], where=[('company_id', '=', company_id), ('pessoa', '=', pessoa_id)]
            )

            vals_hr_funcionario.update({'pessoa': pessoa_id})

            if not rh_funcionario_dados:
                # CRIA O VENDEDOR COMO FUNCIONARIO
                db_registro.insert(table='rh_funcionario', vals=vals_hr_funcionario)
            else:
                rh_funcionario_id = rh_funcionario_dados[0]['id']
                db_registro.update(table='rh_funcionario', values=vals_hr_funcionario, where=[('id', '=', rh_funcionario_id)])

        elif operacao == 'r':  # Regravar
            if not pessoa_id:
                raise Exception('Est11: Vendedor nao encontrado na legado_controle para regravacao. Chave - %s' % _chave)

            valsPessoa = {'nome': _nome}
            db_registro.update(table='adm_pessoa', values=valsPessoa, where=[('id', '=', pessoa_id)])
            _pessoa_dados = db_registro.select(table='adm_pessoa', fields=['partner_id'], where=[('id', '=', pessoa_id)])
            if not _pessoa_dados:
                raise Exception('Est11: Erro na regravacao - Pessoa nao cadastrada. ID - %s' % str(pessoa_id))

            partner_id = _pessoa_dados[0]['partner_id']
            db_registro.update(table='res_partner', values={'name': _nome}, where=[('id', '=', partner_id)])

            rh_funcionario_dados = db_registro.select(
                table='rh_funcionario', fields=['id'], where=[('company_id', '=', company_id), ('pessoa', '=', pessoa_id)]
            )

            vals_hr_funcionario.update({'pessoa': pessoa_id})

            if not rh_funcionario_dados:
                db_registro.insert(table='rh_funcionario', vals={'company_id': company_id, 'pessoa': pessoa_id})
            else:
                rh_funcionario_id = rh_funcionario_dados[0]['id']
                db_registro.update(table='rh_funcionario', values=vals_hr_funcionario, where=[('id', '=', rh_funcionario_id)])

        elif operacao == 'd':  # deletar
            if idVendedor:
                db_registro.delete(table='vendas_vendedor', where=[('id', '=', idVendedor)])
                args_est11 = [
                    ('chave_legado', '=', _chave), ('tabela_legado', '=', 'est11'), ('empresa', '=', empresa_legado)
                ]
                db_registro.delete_legado_controle(reg_args=args_est11, reg_check_table='vendas_vendedor')

            if pessoa_id:
                db_registro.delete(table='adm_pessoa', where=[('id', '=', pessoa_id)])
                args_est11P = [
                    ('chave_sigerp', '=', _chave), ('tabela_legado', '=', 'est11P'), ('empresa', '=', empresa_legado)
                ]
                db_registro.delete_legado_controle(reg_args=args_est11P, reg_check_table='adm_pessoa')

            rh_funcionario_dados = db_registro.select(
                table='rh_funcionario', fields=['id'], where=[('company_id', '=', company_id), ('pessoa', '=', pessoa_id)]
            )

            if rh_funcionario_dados:
                rh_funcionario_id = rh_funcionario_dados[0]['id']
                db_registro.delete(table='rh_funcionario', where=[('id', '=', rh_funcionario_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# REQUISICAO
def gerarEst13(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _locacao = lista_dados[0]
        _requisicao = lista_dados[1].strip()

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[2],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[2].strip()
        _grade = lista_dados[3].strip()
        _local_entrada = lista_dados[4]
        _local_saida = lista_dados[5]
        _quantidade = lista_dados[6][:9] + '.' + lista_dados[6][9:]
        _data = sigDatas(lista_dados[7]).getDataNoFormato('N', 'S', '/')
        _qtdConfirmada = lista_dados[8][:9] + '.' + lista_dados[8][9:]
        _empresaSaida = lista_dados[9]
        _jaSeparado = lista_dados[10]
        _jaExportadoCd = lista_dados[11]
        _chave = _requisicao

        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        _regLocal = db_registro.select(
            table='estoque_local', fields=['id'], where=[('numero', '=', _local_entrada), ('company_id', '=', company_id)]
        )
        if not _regLocal:
            raise Exception('Est13: Local de entrada nao encontrado. Numero - %s' % _local_entrada)

        _local_entrada_id = _regLocal[0]['id']

        _regLocal = db_registro.select(
            table='estoque_local', fields=['id'], where=[('numero', '=', _local_saida), ('company_id', '=', company_id)]
        )
        if not _regLocal:
            raise Exception('Est13: Local de saida nao encontrado. Numero - %s' % _local_saida)

        _local_saida_id = _regLocal[0]['id']

        company_id = funct_get_empresa(db_registro, 'estoqueressuprimento.requisicao.capa', company_com_nf_id)

        saida_company_id = False
        if _empresaSaida and int(_empresaSaida):
            legado_empresa_dados = db_registro.select(
                table='legado_empresa', fields=['company_id'], where=[('empresa_legado', '=', _empresaSaida)]
            )
            if not legado_empresa_dados:
                raise Exception('Est13: Empresa nao cadastrada na correlacao de empresas (legado.empresa) - %s' % _empresaSaida)
            else:
                saida_company_id = legado_empresa_dados[0]['company_id']

        vals = {
            'company_id': company_id,
            'numero': _requisicao,
            'data': _data,
            'local_entrada_id': _local_entrada_id,
            'local_saida_id': _local_saida_id,
            'state': 'O',
            'tipo': 'QR',
        }
        if saida_company_id:
            vals['company_id'] = saida_company_id

        _regRequisicao = db_registro.select(
            table='estoqueressuprimento_requisicao_capa', fields=['id'], where=[('resrca_empresa_entrada_id', '=', company_id),
                                                                                ('numero', '=', _requisicao)]
        )

        if operacao == 'w':  # Gravar

            if _regRequisicao:
                _requisicao_id = _regRequisicao[0]['id']
                db_registro.update(
                    table='estoqueressuprimento_requisicao_capa', values=vals, where=[('id', '=', _requisicao_id)]
                )
            else:
                db_registro.insert(table='estoqueressuprimento_requisicao_capa', vals=vals)

        elif operacao == 'r':  # Regravar
            if not _regRequisicao:
                raise Exception('Est13: Requisicao nao encontrada. Numero - %s' % _requisicao)
            else:
                _requisicao_id = _regRequisicao[0]['id']
                db_registro.update(
                    table='estoqueressuprimento_requisicao_capa', values=vals, where=[('id', '=', _requisicao_id)]
                )

        elif operacao == 'd':  # deletar
            if not _regRequisicao:
                raise Exception('Est13: Requisicao nao encontrada. Numero - %s' % _requisicao)
            else:
                _requisicao_id = _regRequisicao[0]['id']
                db_registro.delete(table='estoqueressuprimento_requisicao_capa', where=[('id', '=', _requisicao_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# TRIBUTACAO
def gerarEst14(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        _chave = lista_dados[0]
        _nome = lista_dados[1].strip()
        _aliquota = lista_dados[2]
        _base = lista_dados[3]
        if int(_base) == 0:
            _base = '100000000'
        _dispositivo1 = lista_dados[4].strip()
        _dispositivo2 = lista_dados[5].strip()
        _dispositivo3 = lista_dados[6].strip()
        _tipo = lista_dados[7]
        if lista_dados[8] == '2':
            _imposto = 'ISS'
        else:
            _imposto = 'ICMS'
        # _tributa_icms_sobre_ipi = lista_dados[9]
        # _base_reduzida_desc = lista_dados[10]
        # _base_subst_integral = lista_dados[11]

        company_id = funct_get_empresa(db_registro, 'fiscal.tributacao', company_com_nf_id)

        _regCompany = db_registro.select(table='res_company', fields=['rescon_pessoa_id'], where=[('id', '=', company_id)])
        if _regCompany:
            _pessoa_id = _regCompany[0]['rescon_pessoa_id']
        else:
            raise Exception('Est14: Empresa nao cadastrada. ID - %s' % str(company_id))

        if not _pessoa_id:
            raise Exception('Est14: Nenhum Parceiro encontra-se cadastrado para a Empresa - ID - %s' % str(company_id))

        _regPessoa = db_registro.select(
            table='adm_pessoa', fields=['admpes_jur_tipo_atividade', 'admpes_jur_regime_tributario'], where=[('id', '=', _pessoa_id)]
        )
        if not _regPessoa:
            raise Exception('Est14: Empresa cadastrada nao possui pessoa associada. ID - %s' % company_id)

        _tipoAtividade = _regPessoa[0]['admpes_jur_tipo_atividade']
        _regimeTributario = _regPessoa[0]['admpes_jur_regime_tributario']

        res = db_registro.select(table='fiscal_encargo', fields=['id'], where=[('sigla', '=', _imposto)])
        if res:
            idFiscalEncargo = res[0]['id']
        else:
            idFiscalEncargo = False
        cstLista = []
        if _imposto == "ISS":
            cstLista = ['41']
        else:
            if _tipo == 'I':
                cstLista = ['30', '40']
            if _tipo == 'D':
                cstLista = ['51']
            if _tipo == 'O':
                cstLista = ['41', '90']
            if _tipo == 'U':
                cstLista = ['10', '30', '60', '70']
            if _tipo == 'T':
                if _tipoAtividade == 'C' and 'S':  # Comercio e servico
                    cstLista = ['00']
                else:
                    cstLista = ['00', '10']
        for a in cstLista:
            args = [('codigo', '=', a), ('encargo_id', '=', idFiscalEncargo)]

            cst_dados = db_registro.select(table='fiscal_cst', fields=['id'], where=args)
            if cst_dados:
                cst_id = cst_dados[0]['id']
            else:
                cst_id = False

            if not cst_id:
                vals = {
                    'codigo': a,
                    'name': 'Alterar',
                    'tipo': _tipo,
                    'encargo_id': idFiscalEncargo,
                    'active': True,
                }
                cst_id = db_registro.insert(table='fiscal_cst', vals=vals)

            base = _base[:3] + '.' + _base[3:]
            aliquota = _aliquota[:2] + '.' + _aliquota[2:]

            valsTributacao = {
                'name': _nome,
                'encargo_id': idFiscalEncargo,
                'cst_id': cst_id,
                'base': base,
                'aliquota': aliquota,
                'active': True,
                'dispositivo_fiscal': _dispositivo1.strip() + ' ' + _dispositivo2.strip() + ' ' + _dispositivo3.strip(),
            }

            legado_controle_tributacao_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_chave,
                reg_tabela_legado='est14',
                reg_many_ids=False,
                reg_check_table='fiscal_tributacao',
            )

            tributacao_ja_cadastrada_id = False
            args_tribut = [
                ('aliquota', '=', aliquota), ('name', 'ilike', _nome), ('cst_id', '=', cst_id), ('base', '=', base),
                ('encargo_id', '=', idFiscalEncargo)
            ]
            tributacao_dados = db_registro.select(table='fiscal_tributacao', fields=['id'], where=args_tribut)
            if tributacao_dados:
                tributacao_ja_cadastrada_id = tributacao_dados[0]['id']

            if operacao == 'w':  # Gravar

                if tributacao_ja_cadastrada_id:
                    # TRIBUTACAO JA CADASTRADA NO SISTEMA
                    tributacao_id = tributacao_ja_cadastrada_id
                else:
                    # CRIA A TRIBUTACAO
                    tributacao_id = db_registro.insert(table='fiscal_tributacao', vals=valsTributacao)

                # CRIA A LEGADO CONTROLE
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est14',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=tributacao_id,
                    reg_check_table='fiscal_tributacao',
                )

            elif operacao == 'r':  # Regravar

                if not tributacao_ja_cadastrada_id:

                    rotulos = 'Aliquota - %s, Name - %s, Cst_id - %s, Base - %s, Encargo_id - %s'
                    valores = tuple(map(str, [aliquota, _nome, cst_id, base, idFiscalEncargo]))

                    label = rotulos % valores

                    raise Exception(
                        'Tributacao com os seguintes dados nao cadastrada. %s' % label
                    )
                # ATUALIZA A TRIBUTACAO DO ITEM
                db_registro.update(
                    table='fiscal_tributacao', values=valsTributacao, where=[('id', '=', tributacao_ja_cadastrada_id)]
                )

            elif operacao == 'd':  # Deletar
                if not legado_controle_tributacao_id:
                    raise Exception(
                        'Est14: Delecao nao realizada. Tributacao nao encontrada na legado_controle. Chave_legado - %s'
                        % _chave
                    )

                db_registro.delete(table='fiscal_tributacao', where=[('id', '=', legado_controle_tributacao_id)])

                args_legado_controle = [
                    ('chave_legado', '=', _chave), ('tabela_legado', '=', 'est14'), ('empresa', '=', empresa_legado)
                ]

                db_registro.delete_legado_controle(
                    reg_args=args_legado_controle,
                    reg_check_table='fiscal_tributacao',
                )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# LOCAL
def gerarEst15(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _nome = lista_dados[1].strip()
        if lista_dados[2] == 'N':
            _exclue = False
        else:
            _exclue = True
        if lista_dados[3] == 'N':
            _saldoNegativo = False
        else:
            _saldoNegativo = True
        # _contaCustos = lista_dados[4]

        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        legado_local_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est15',
            reg_check_table='estoque_local',
        )

        company_id = funct_get_empresa(db_registro, 'adm.empresa.centrocusto', company_com_nf_id)
        args = [('company_id', '=', company_id)]
        regsCentroCusto = db_registro.select(table='adm_empresa_centrocusto', fields=['id'], where=args)
        if not regsCentroCusto:
            raise Exception('Est15: Centro de custo (adm_empresa_centrocusto). Company_id - %s' % str(company_id))

        _idCentroCusto = regsCentroCusto[0]['id']

        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)
        valsLocais = {
            'company_id': company_id,
            'name': _nome,
            'excluir_consultas': _exclue,
            'aceita_saldo_negativo': _saldoNegativo,
            'numero': _chave,
            'centrocusto_id': _idCentroCusto,
            'active': True,
        }

        local_ja_cadastrado_id = False
        local_dados = db_registro.select(
            table='estoque_local', fields=['id'], where=[('numero', '=', _chave), ('company_id', '=', company_id)]
        )
        if local_dados:
            local_ja_cadastrado_id = local_dados[0]['id']

        if operacao == 'w':  # Gravar
            if not legado_local_id:
                if local_ja_cadastrado_id:
                    local_id = local_ja_cadastrado_id
                    db_registro.update(
                        table='estoque_local', values=valsLocais, where=[('id', '=', local_ja_cadastrado_id)]
                    )
                else:
                    local_id = db_registro.insert(table='estoque_local', vals=valsLocais)

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est15',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=local_id,
                    reg_check_table='estoque_local',
                )

        elif operacao == 'r':  # Regravar
            if legado_local_id:
                local_id = legado_local_id
            elif local_ja_cadastrado_id:
                local_id = local_ja_cadastrado_id
            else:
                local_id = False

            if not local_id:
                raise Exception('Est15: Local nao encontrado na legado_controle para alteracao. Chave - %s' % _chave)

            db_registro.update(table='estoque_local', values=valsLocais, where=[('id', '=', local_id)])
            if not legado_local_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est15',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=local_id,
                    reg_check_table='estoque_local',
                )
            else:
                args_update_legado = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est15'), ('chave_legado', '=', _chave)
                ]
                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': local_id},
                    reg_args=args_update_legado,
                    reg_check_table='estoque_local',
                )

        elif operacao == 'd':  # Deletar
            if not legado_local_id:
                raise Exception('Est15: Local nao encontrado na legado_controle para ser excluido. Chave - %s' % _chave)

            db_registro.delete(table='estoque_local', where=[('id', '=', legado_local_id)])
            args_legado_controle = [
                ('empresa_legado', '=', empresa_legado), ('chave_legado', '=', _chave), ('tabela_legado', '=', 'est15')
            ]
            db_registro.delete_legado_controle(reg_args=args_legado_controle, reg_check_table='estoque_local')

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# COMPRADOR
def gerarEst16(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _nome = lista_dados[1].strip()

        legado_pessoa_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est16P',
            reg_check_table='adm_pessoa',
        )

        legado_comprador_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est16',
            reg_check_table='compras_comprador',
        )

        company_id = funct_get_empresa(db_registro, 'compras.comprador', company_com_nf_id)

        if operacao == 'w':  # Gravar

            if legado_comprador_id or legado_pessoa_id:
                return True

            valsPessoa = {'nome': _nome, 'pessoa_tipo': 'F', 'active': True}

            ir_sequence_dados = db_registro.select(
                table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
            )

            sequence_id = ir_sequence_dados[0]['id']
            sequence_codigo = ir_sequence_dados[0]['number_next']

            valsPessoa['codigo'] = sequence_codigo

            proximo_codigo = int(sequence_codigo) + 1

            db_registro.update(table='ir_sequence', values={'number_next': proximo_codigo}, where=[('id', '=', sequence_id)])

            partner_id = db_registro.insert(table='res_partner', vals={'name': _nome, 'notification_email_send': 'comment'})
            valsPessoa['partner_id'] = partner_id
            pessoa_id = db_registro.insert(table='adm_pessoa', vals=valsPessoa)

            db_registro.create_legado_controle(
                reg_empresa_legado=empresa_legado,
                reg_tabela_legado='est16P',
                reg_chave_legado=_chave,
                reg_chave_sigerp=pessoa_id,
                reg_check_table='adm_pessoa',
            )

            valsComprador = {
                'pessoa_id': pessoa_id,
                'company_id': company_id,
            }

            idComprador = db_registro.insert(table='compras_comprador', vals=valsComprador)

            db_registro.create_legado_controle(
                reg_empresa_legado=empresa_legado,
                reg_tabela_legado='est16',
                reg_chave_legado=_chave,
                reg_chave_sigerp=idComprador,
                reg_check_table='compras_comprador',
            )

        elif operacao == 'r':  # regravar
            if not legado_pessoa_id:
                raise Exception('Est16: Pessoa na encontrada na legado_controle. Chave - %s' % _chave)

            valsPessoa = {'nome': _nome}

            db_registro.update(table='adm_pessoa', values=valsPessoa, where=[('id', '=', legado_pessoa_id)])
            pessoa_dados = db_registro.select(table='adm_pessoa', fields=['partner_id'], where=[('id', '=', legado_pessoa_id)])
            partner_id = pessoa_dados[0]['partner_id']

            db_registro.update(table='res_partner', values={'name': _nome}, where=[('id', '=', partner_id)])

        elif operacao == 'd':  # deletar
            if legado_comprador_id:
                db_registro.delete(table='compras_comprador', where=[('id', '=', legado_comprador_id)])

                args_legado_control_comprador = [
                    ('chave_legado', '=', _chave), ('tabela_legado', '=', 'est16'), ('empresa', '=', empresa_legado)
                ]
                db_registro.delete_legado_controle(
                    reg_args=args_legado_control_comprador,
                    reg_check_table='compras_comprador',
                )

            if legado_pessoa_id:
                db_registro.delete(table='adm_pessoa', where=[('id', '=', legado_pessoa_id)])

                args_legado_control_pessoa = [
                    ('chave_sigerp', '=', _chave), ('tabela_legado', '=', 'est16P'), ('empresa', '=', empresa_legado)
                ]
                db_registro.delete_legado_controle(reg_args=args_legado_control_pessoa, reg_check_table='adm_pessoa')

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# CODIGO DE BARRAS
def gerarEst17(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _codBarra = lista_dados[2]
        # _chave = _codBarra
        # _chaveGrade = _item + '-' + _grade
        _qtdEmbalagem = lista_dados[3][:5] + '.' + lista_dados[3][5:]
        if _qtdEmbalagem == '.':
            _qtdEmbalagem = '00000.000'
        _unidade = lista_dados[4].strip().upper()
        _emite_etiqueta = lista_dados[5]
        if _emite_etiqueta in ('S', 'Y'):
            _emite_etiqueta = True
        else:
            _emite_etiqueta = False
        # _modelo_etiqueta = lista_dados[6]
        _ultimaData = sigDatas(lista_dados[7]).getDataNoFormato('N', 'S', '/')
        _pesoEmbalagem = lista_dados[8][:1] + '.' + lista_dados[8][1:]
        if _pesoEmbalagem.strip() == '.':
            _pesoEmbalagem = '0.000'

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _item)])
        if not _regProduto:
            raise Exception('Est17: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _gradeDados = db_registro.select(table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)])
        if not _gradeDados:
            raise Exception('Est17: Grade nao cadastrada. Grade - %s, Produto_id - %s' % (_grade, str(_produto_id)))

        _grade_id = _gradeDados[0]['id']
        _regUnidade = db_registro.select(table='estoque_embalagem_unidade', fields=['id', 'name'], where=[('unidade', '=', _unidade)])
        if not _regUnidade:
            valsUnidade = {
                'unidade': _unidade,
                'name': _unidade + '-1',
                'fracao': 'I',
            }

            obj_embalagem_unidade = context.get('ctx_est_obj_embalagem_unidade')
            obj_embalagem_unidade.create(db_registro.cr, db_registro.uid, valsUnidade, context)

            _regUnidade = db_registro.select(
                table='estoque_embalagem_unidade', fields=['id', 'name'], where=[('unidade', '=', _unidade)]
            )
            if not _regUnidade:
                raise Exception('Est10: Unidade nao cadastrada. Unidade - %s' % _unidade)

        _unidade_id = _regUnidade[0]['id']
        if float(_qtdEmbalagem) == 1:
            _sigla = _unidade
        else:
            if float(_qtdEmbalagem).is_integer():
                _qtd = str(int(float(_qtdEmbalagem)))
            else:
                _qtd = str(float(_qtdEmbalagem))
            if len(_qtd) == 1:
                _sigla = _unidade.upper() + '0' + _qtd
            else:
                _sigla = _unidade.upper() + _qtd
        if len(_sigla) > 7:
            _sigla = _sigla[:7]
        _regEmbalagem = db_registro.select(table='estoque_embalagem', fields=['id'], where=[('sigla',  '=', _sigla)])
        if not _regEmbalagem:
            valsEmbalagem = {
                'name': _regUnidade[0]['name'],
                'sigla': _sigla,
                'unidade_id': _unidade_id,
                'quantidade_na_embalagem': _qtdEmbalagem,
            }
            if float(_qtdEmbalagem) > 1:
                if float(_qtdEmbalagem).is_integer():
                    valsEmbalagem['name'] = _regUnidade[0]['name'] + ' com ' + str(int(
                        float(_qtdEmbalagem)))
                else:
                    valsEmbalagem['name'] = _regUnidade[0]['name'] + ' com ' + str(float(
                        _qtdEmbalagem))
            _embalagem_id = db_registro.insert(table='estoque_embalagem', vals=valsEmbalagem)
        else:
            _embalagem_id = _regEmbalagem[0]['id']
        _regEmbProduto = db_registro.select(
            table='estoque_produto_embalagem',
            fields=['id'],
            where=[('produto_id', '=', _produto_id), ('embalagem_id', '=', _embalagem_id),
                   ('active', '=', True)]
        )
        if not _regEmbProduto:
            valsProdEmb = {
                'produto_id': _produto_id,
                'embalagem_id': _embalagem_id,
                'active': True,
            }
            _embProduto_id = db_registro.insert(table='estoque_produto_embalagem', vals=valsProdEmb)
        else:
            _embProduto_id = _regEmbProduto[0]['id']

        company_id = funct_get_empresa(db_registro, 'automacao.codigo.barra', company_com_nf_id)
        valsBarra = {
            'company_id': company_id,
            'produto_id': _produto_id,
            'grade_id': _grade_id,
            'produto_embalagem_id': _embProduto_id,
            'emite_etiqueta': _emite_etiqueta,
            'codigo_barra': _codBarra,
            'peso_embalagem': _pesoEmbalagem,
            'active': True,
            'autcba_tipo_barra': '13',
        }
        if _ultimaData != '00/00/0000':
            valsBarra['ultima_data_movimentacao'] = _ultimaData
        codbarra_dados = db_registro.select(table='automacao_codigo_barra', fields=['id'], where=[('company_id', '=', company_id), ('codigo_barra', '=',_codBarra)])
        if codbarra_dados:
            _idBarra = codbarra_dados[0]['id']
        else:
            _idBarra = False

        if operacao == 'w':  # Gravar

            if _idBarra:
                return True
            else:
                db_registro.insert(table='automacao_codigo_barra', vals=valsBarra)

        elif operacao == 'r':  # Regravar
            if not _idBarra:
                db_registro.insert(table='automacao_codigo_barra', vals=valsBarra)
            else:
                db_registro.update(table='automacao_codigo_barra', values=valsBarra, where=[('id', '=', _idBarra)])

        elif operacao == 'd':  # Deletar
            if _idBarra:
                db_registro.delete(table='automacao_codigo_barra', where=[('id', '=', _idBarra)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# GRADE - DESCRICAO COMPLEMENTAR
def gerarEst18(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _chave = _item + '-' + _grade
        _descricao = funct_retira_acentos(lista_dados[2].strip())

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _item)])
        if not _regProduto:
            raise Exception('Est18: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _gradeDados = db_registro.select(table='estoque_produto_grade', where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)])
        if not _gradeDados:
            raise Exception('Est18: Grade nao cadastrada. Grade - %s, Produto_id - %s' % (_grade, str(_produto_id)))

        _grade_id = _gradeDados[0]['id']

        if operacao == 'd':
            _descricao = ''

        valsGrade = {
            'descricao_complementar': _descricao,
        }

        db_registro.update(table='estoque_produto_grade', values=valsGrade, where=[('id', '=', _grade_id)])

        return True

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# UNIDADE
def gerarEst19(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0].strip().upper()
        _nome = lista_dados[1].strip()
        if lista_dados[2].upper() == 'I':
            _aceitarFracao = False
        else:
            _aceitarFracao = True

        obj_embalagem_unidade = context.get('ctx_est_obj_embalagem_unidade')
        _regUnidade = db_registro.select(table='estoque_embalagem_unidade', fields=['id'], where=[('unidade', 'ilike', _chave)])

        if operacao in ('w', 'r'):

            valsUnidade = {
                'unidade': _chave,
                'name': _nome,
                'fracao': _aceitarFracao,
            }

            if not _regUnidade:
                obj_embalagem_unidade.create(db_registro.cr, db_registro.uid, valsUnidade, context)
            else:
                unidade_id = _regUnidade[0]['id']
                obj_embalagem_unidade.write(db_registro.cr, db_registro.uid, [unidade_id], valsUnidade, context)

        elif operacao == 'd':
            if not _regUnidade:
                raise Exception('Est19: Unidade nao encontrada para delecao. Unidade - %s' % _chave)
            else:
                unidade_id = _regUnidade[0]['id']
                obj_embalagem_unidade.unlink(db_registro.cr, db_registro.uid, [unidade_id], context)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# SETOR
def gerarEst21(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _nome = lista_dados[1].strip()

        legado_setor_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave,
            reg_tabela_legado='est21',
            reg_check_table='balanca_setor',
        )

        balanca_setor_dados = db_registro.select(
            table='balanca_setor', fields=['id'], where=[('name', 'ilike', _nome)]
        )
        if balanca_setor_dados:
            setor_ja_cadastrado_id = balanca_setor_dados[0]['id']
        else:
            setor_ja_cadastrado_id = False

        valsSetor = {'name': _nome}

        if operacao == 'w':  # Gravar

            if not legado_setor_id:
                if setor_ja_cadastrado_id:
                    balanca_setor_id = setor_ja_cadastrado_id
                else:
                    balanca_setor_id = db_registro.insert(table='balanca_setor', vals=valsSetor)

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est21',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=balanca_setor_id,
                    reg_check_table='balanca_setor',
                )

        elif operacao == 'r':  # Regravar
            if legado_setor_id:
                balanca_setor_id = legado_setor_id
            elif setor_ja_cadastrado_id:
                balanca_setor_id = setor_ja_cadastrado_id
            else:
                balanca_setor_id = False

            if not balanca_setor_id:
                raise Exception(
                    'Est21: Atualizacao nao realizada. Nenhum registro encontrado na legado_controle. Chave - %s' %
                    _chave
                )

            db_registro.update(table='balanca_setor', values=valsSetor, where=[('id', '=', balanca_setor_id)])

            if not legado_setor_id:
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est21',
                    reg_chave_legado=_chave,
                    reg_chave_sigerp=balanca_setor_id,
                    reg_check_table='balanca_setor'
                )
            else:
                args_est21 = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est21'), ('chave_legado', '=', _chave)
                ]
                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': balanca_setor_id},
                    reg_args=args_est21,
                    reg_check_table='balanca_setor'
                )

        elif operacao == 'd':  # Deletar
            args_legado = [
                ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est21'), ('chave_legado', '=', _chave)
            ]

            db_registro.delete(table='balanca_setor', where=[('id', '=', legado_setor_id)])

            db_registro.delete_legado_controle(
                reg_args=args_legado,
                reg_check_table='balanca_setor',
            )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# FUNCIONARIO
def gerarEst22(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _local = lista_dados[0]
        _codRepositor = lista_dados[1].strip()
        _nome = lista_dados[2].strip()

        _chave_local_nome = '%s-%s' % (_local, _nome)
        _chave_local_repositor = '%s-%s' % (_local, _codRepositor)

        legado_pessoa_est22P_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave_local_nome,
            reg_tabela_legado='est22P',
            reg_check_table='adm_pessoa',
        )

        legado_est22L_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave_local_repositor,
            reg_tabela_legado='est22L',
            reg_check_table='adm_pessoa',
        )

        legado_funcionario_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_chave_local_nome,
            reg_tabela_legado='est22',
            reg_check_table='rh_funcionario',
        )

        company_id = funct_get_empresa(db_registro, 'rh.funcionario', company_com_nf_id)

        if operacao == 'w':

            if not legado_pessoa_est22P_id:
                # EST22P
                vals_pessoa_est22P = {'nome': _nome, 'pessoa_tipo': 'F', 'active': True}
                ir_sequence_dados = db_registro.select(
                    table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
                )
                sequence_id = ir_sequence_dados[0]['id']
                sequence_codigo = ir_sequence_dados[0]['number_next']
                proximo_codigo = int(sequence_codigo) + 1
                db_registro.update(table='ir_sequence', values={'number_next': proximo_codigo}, where=[('id', '=', sequence_id)])

                vals_pessoa_est22P['codigo'] = sequence_codigo

                partner_id = db_registro.insert(
                    table='res_partner', vals={'name': _nome, 'notification_email_send': 'comment'}
                )
                vals_pessoa_est22P['partner_id'] = partner_id
                # CRIA UMA PESSOA
                pessoa_id = db_registro.insert(table='adm_pessoa', vals=vals_pessoa_est22P)
                legado_pessoa_est22P_id = pessoa_id

                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est22P',
                    reg_chave_legado=_chave_local_nome,
                    reg_chave_sigerp=pessoa_id,
                    reg_check_table='adm_pessoa',
                )

            if not legado_est22L_id:
                # EST22L
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est22L',
                    reg_chave_legado=_chave_local_repositor,
                    reg_chave_sigerp=legado_pessoa_est22P_id,
                    reg_check_table='adm_pessoa',
                )

            # EST22
            if legado_funcionario_id:
                funcionario_id = legado_funcionario_id
            else:
                # CRIA UM FUNCIONARIO
                funcionario_id = db_registro.insert(
                    table='rh_funcionario', vals={'company_id': company_id, 'pessoa': legado_pessoa_est22P_id}
                )

            db_registro.create_legado_controle(
                reg_empresa_legado=empresa_legado,
                reg_tabela_legado='est22',
                reg_chave_legado=_chave_local_nome,
                reg_chave_sigerp=funcionario_id,
                reg_check_table='rh_funcionario',
            )

        elif operacao == 'r':
            if not legado_funcionario_id:
                raise Exception(
                    'Est22: Alteracao nao realizada. Funcionario nao encontrado na legado_controle. '
                    'Chave_legado - %s, Empresa - %s' % (_chave_local_nome, str(empresa_legado))
                )

            if not legado_pessoa_est22P_id:
                raise Exception(
                    'Est22: Pessoa nao encontrada na legado_controle referente ao funcionario. '
                    'Chave_legado - %s, Tabela_legado - est22P, Empresa - %s' % (_chave_local_nome, str(empresa_legado))
                )

            db_registro.update(table='adm_pessoa', values={'nome': _nome}, where=[('id', '=', legado_pessoa_est22P_id)])
            pessoa_dados = db_registro.select(table='adm_pessoa', fields=['partner_id'], where=[('id', '=', legado_pessoa_est22P_id)])
            if pessoa_dados:
                pessoa_partner_id = pessoa_dados[0]['partner_id']
                db_registro.update(table='res_partner', values={'name': _nome}, where=[('id', '=', pessoa_partner_id)])

        elif operacao == 'd':
            if legado_funcionario_id:
                db_registro.delete(table='rh_funcionario', where=[('id', '=', legado_funcionario_id)])
                args_est22 = [
                    ('empresa', '=', empresa_legado), ('chave_legado', '=', _chave_local_nome),
                    ('tabela_legado', '=', 'est22')
                ]
                db_registro.delete_legado_controle(reg_args=args_est22, reg_check_table='rh_funcionario')

            # EST22P
            args_est22P = [
                ('empresa', '=', empresa_legado), ('chave_legado', '=', _chave_local_nome),
                ('tabela_legado', '=', 'est22P')
            ]

            db_registro.delete_legado_controle(reg_args=args_est22P, reg_check_table='adm_pessoa')

            # EST22L
            args_est22L = [
                ('empresa', '=', empresa_legado), ('chave_legado', '=', _chave_local_repositor), ('tabela_legado', '=', 'est22L')]

            db_registro.delete_legado_controle(reg_args=args_est22L, reg_check_table='adm_pessoa')

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# ECF
def gerarEst24(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _chave = lista_dados[0]
        _sequencia = lista_dados[1]
        _local = lista_dados[2]
        _cSNF = lista_dados[3]
        _departamento = lista_dados[4]
        _nroSerie = lista_dados[5].strip()
        _cro = lista_dados[6]
        _situacao = lista_dados[7]
        if _situacao == 'I':
            _situacao = False
        else:
            _situacao = True
        _fabricante = lista_dados[8].strip()
        _modelo = lista_dados[9].strip()

        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        regsLocal = db_registro.select(table='estoque_local', fields=['id'], where=[('numero', '=', _local), ('company_id', '=', company_id)])
        if not regsLocal:
            raise Exception('Est24: Local nao cadastrado na tabela estoque_local. Numero - %s' % _local)

        ecf_company_id = funct_get_empresa(db_registro, 'ecf.ecf', company_com_nf_id)
        _local_id = regsLocal[0]['id']

        _regPessoa = db_registro.select(table='adm_pessoa', fields=['id'], where=[('nome', 'ilike', _fabricante)])
        if _regPessoa:
            _pessoa_id = _regPessoa[0]['id']
        else:
            vals = {'nome': _fabricante, 'pessoa_tipo': 'J', 'active': True}

            ir_sequence_dados = db_registro.select(table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')])
            sequence_id = ir_sequence_dados[0]['id']
            sequence_codigo = ir_sequence_dados[0]['number_next']
            proximo_codigo = int(sequence_codigo) + 1

            partner_id = db_registro.insert(table='res_partner', vals={'name': _fabricante, 'notification_email_send': 'comment'})

            vals['codigo'] = sequence_codigo
            vals['partner_id'] = partner_id

            _pessoa_id = db_registro.insert(table='adm_pessoa', vals=vals)
            db_registro.update(table='ir_sequence', values={'number_next': proximo_codigo}, where=[('id', '=', sequence_id)])

        _regEcf = db_registro.select(table='ecf_ecf', fields=['id'], where=[('caixa', '=', _chave)])

        if operacao in ('w', 'r'):  # Gravar e regravar

            valsEcf = {
                'company_id': ecf_company_id,
                'local_id': _local_id,
                'fabricante_id': _pessoa_id,
                'nro_serie': _nroSerie,
                'ativo': _situacao,
                'modelo': _modelo,
                'loja': 1,
                'caixa': _chave,
            }

            if _regEcf:
                ecf_id = _regEcf[0]['id']
                db_registro.update(table='ecf_ecf', values=valsEcf, where=[('id', '=', ecf_id)])
            else:
                db_registro.insert(table='ecf_ecf', vals=valsEcf)

        if operacao == 'd':  # Deletar
            if not _regEcf:
                raise Exception('Est24: ECF nao encontrado na tabela ecf_ecf. Caixa - %s' % _chave)

            ecf_id = _regEcf[0]['id']
            db_registro.delete(table='ecf_ecf', where=[('id', '=', ecf_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# CONCORRENTE ITENS
def gerarEst29(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        dados_concorrente_codigo = lista_dados[0]

        dados_produto_codigo_original = lista_dados[1].strip()

        if _codNumerico:
            dados_produto_codigo = est_format_produto_codigo(
                db_registro, produto_codigo=dados_produto_codigo_original, context=context,
                company_com_nf_id=company_com_nf_id
            )
        else:
            dados_produto_codigo = lista_dados[1].strip()

        dados_grade = lista_dados[2].strip() or 'UN'
        dados_data_coleta = lista_dados[3]
        dados_preco = '%s.%s' % (lista_dados[4][:-2], lista_dados[4][-2:])

        chave_est29 = '%s-%s-%s-%s' % \
                      (dados_concorrente_codigo, dados_produto_codigo_original, dados_grade, dados_data_coleta)

        legado_est29_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_est29,
            reg_tabela_legado='est29',
            reg_check_table='concorrencia_concorrente_item',
        )

        if operacao in ('w', 'r'):

            # ATUALIZAR O VALOR NOS ITENS DO CONCORRENTE
            if legado_est29_id:  # R
                db_registro.update(
                    table='concorrencia_concorrente_item',
                    values={'preco': dados_preco},
                    where=[('id', '=', legado_est29_id)]
                )

                args_est29 = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est29'),
                    ('chave_legado', '=', chave_est29)
                ]

                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': legado_est29_id},
                    reg_args=args_est29,
                    reg_m2m=False,
                    reg_check_table='',
                    reg_empresa_tipo=''
                )

            # INSERIR OS DADOS NOS ITENS DO CONCORRENTE
            else:  # W
                try:
                    data_coleta = parse(dados_data_coleta, yearfirst=True).strftime('%Y-%m-%d')
                except:
                    raise Exception('Data de Coleta dos precos invalida. Data - %s' % dados_data_coleta)

                # COMPANY_ID A SER USADA NOS OBJETOS
                concorrente_company_id = funct_get_empresa(db_registro, 'concorrencia.concorrente', company_com_nf_id)
                concorrente_item_company_id = funct_get_empresa(db_registro, 'concorrencia.concorrente.item', company_com_nf_id)
                produto_company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

                # BUSCAR O CONCORRENTE
                legado_est30_pessoa_id = db_registro.controle_get_chave_sigerp(
                    reg_empresa_legado=empresa_legado,
                    reg_chave=dados_concorrente_codigo,
                    reg_tabela_legado='est30',
                    reg_check_table='concorrencia_concorrente',
                )
                if not legado_est30_pessoa_id:
                    raise Exception(
                        'Est29 - Corrente nao encontrado na legado controle. Codigo do Concorrente - %s' %
                        dados_concorrente_codigo
                    )
                args_concorrente = [
                    ('pessoa_id', '=', legado_est30_pessoa_id), ('company_id', '=', concorrente_company_id)
                ]
                concorrente_dados = db_registro.select(
                    table='concorrencia_concorrente', fields=['id'], where=args_concorrente
                )
                if not concorrente_dados:
                     raise Exception(
                         'Est29 - Pessoa nao cadastrada como concorrente. Pessoa_id - %s' % str(legado_est30_pessoa_id)
                     )

                concorrente_id = concorrente_dados[0]['id']

                # BUSCAR O PRODUTO
                args_produto = [
                    ('codigo', '=', dados_produto_codigo), ('company_id', '=', produto_company_id)
                ]
                produto_dados = db_registro.select(table='estoque_produto', fields=['id'], where=args_produto)
                if not produto_dados:
                    raise Exception(
                        'Est29 - Produto nao encontrado. Codigo - %s, Company_id - %s' %
                        (str(dados_produto_codigo), str(produto_company_id))
                    )
                produto_id = produto_dados[0]['id']

                # BUSCAR A GRADE
                args_grade = [('produto_id', '=', produto_id), ('grade', '=', dados_grade)]
                grade_dados = db_registro.select(table='estoque_produto_grade', fields=['id'], where=args_grade)
                if not grade_dados:
                    raise Exception(
                        'Est29 - Grade nao encontadra. Produto_id - %s (Codigo=%s), Grade - %s' %
                        (str(produto_id), dados_produto_codigo, dados_grade)
                    )
                grade_id = grade_dados[0]['id']

                vals_concorrente_item = {
                    'company_id': concorrente_item_company_id,
                    'data': data_coleta,
                    'preco': dados_preco,
                    'produto_id': produto_id,
                    'grade_id': grade_id,
                    'concorrente_id': concorrente_id,
                    'usa_grade_ou_nao': True,
                }
                corrente_item_id = db_registro.insert(table='concorrencia_concorrente_item', vals=vals_concorrente_item)

                # CRICAO DO LEGADO PARA o ITEM.
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est29',
                    reg_chave_legado=chave_est29,
                    reg_chave_sigerp=corrente_item_id,
                    reg_check_table='concorrencia_concorrente_item',
                )
        elif operacao == 'd':
            if not legado_est29_id:
                raise Exception(
                    'Est29 - Chave na legado controle nao encontrada para o est29. Chave_legado - %s' % chave_est29
                )
            # DELETA O REGISTRO NA TABELA DE ITEM DO CONCORRENTE
            db_registro.delete(table='concorrencia_concorrente_item', where=[('id', '=', legado_est29_id)])

            # DELETE O REGISTRO DA LEGADO CONTROLE
            args_est29 = [
                ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est29'),
                ('chave_legado', '=', chave_est29)
            ]

            db_registro.delete_legado_controle(
                reg_args=args_est29,
                reg_check_table='concorrencia_concorrente_item',
            )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        msg_erro += ' \n%s' % 'OBS - MODULO CONCORRENCIA DEVE ESTAR INSTALADO.'
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# CONCORRENTES
def gerarEst30(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dados_concorrente_codigo = lista_dados[0]
        dados_concorrente_nome = funct_retira_acentos(lista_dados[1].strip().upper())

        pessoa_dados = db_registro.select(
            table='adm_pessoa', fields=['id'], where=[('nome', 'ilike', dados_concorrente_nome)]
        )
        if pessoa_dados:
            pessoa_id = pessoa_dados[0]['id']
        else:
            pessoa_id = False

        # COMPANY_ID A SER USADA NO OBJETO CONCORRENTE
        concorrente_company_id = funct_get_empresa(db_registro, 'concorrencia.concorrente', company_com_nf_id)

        if operacao in ('w', 'r'):

            vals_pessoa = {
                'nome': dados_concorrente_nome,
            }

            if pessoa_id:
                db_registro.update(table='adm_pessoa', values=vals_pessoa, where=[('id', '=', pessoa_id)])
                # ATUALIZACAO DA LEGADO CONTROLE
                args_est30 = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est30'),
                    ('chave_legado', '=', dados_concorrente_codigo)
                ]

                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': pessoa_id},
                    reg_args=args_est30,
                    reg_m2m=False,
                    reg_check_table='',
                    reg_empresa_tipo=''
                )

            else:
                vals_pessoa['codigo'] = dados_concorrente_codigo
                vals_pessoa['active'] = True
                vals_pessoa['pessoa_tipo'] = 'J'

                partner_id = db_registro.insert(
                    table='res_partner', vals={'name': dados_concorrente_nome, 'notification_email_send': 'comment'}
                )
                if partner_id:
                    vals_pessoa['partner_id'] = partner_id
                    sequence_dados = db_registro.select(
                        table='ir_sequence', fields=['id', 'number_next'], where=[('code', '=', 'pessoa.codigo')]
                    )
                    if sequence_dados:
                        sequence_id = sequence_dados[0]['id']
                        sequence_codigo = int(sequence_dados[0]['number_next'])
                        sequence_proximo_codigo = sequence_codigo + 1
                        db_registro.update(
                            table='ir_sequence',
                            values={'number_next': sequence_proximo_codigo},
                            where=[('id', '=', sequence_id)]
                        )
                        vals_pessoa['codigo'] = sequence_codigo
                pessoa_id = db_registro.insert(table='adm_pessoa', vals=vals_pessoa)

            args_concorrente = [
                ('pessoa_id', '=', pessoa_id), ('company_id', '=', concorrente_company_id)
            ]
            concorrente_dados = db_registro.select(
                table='concorrencia_concorrente', fields=['id'], where=args_concorrente
            )
            if not concorrente_dados:
                vals_concorrente = {
                    'pessoa_id': pessoa_id,
                    'company_id': concorrente_company_id,
                }
                # CRIA O CONCORRENTE
                db_registro.insert(table='concorrencia_concorrente', vals=vals_concorrente)

                # CRICAO DO LEGADO PARA A PESSOA.
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est30',
                    reg_chave_legado=dados_concorrente_codigo,
                    reg_chave_sigerp=pessoa_id,
                    reg_check_table='concorrencia_concorrente',
                )

        elif operacao == 'd':
            if not pessoa_id:
                raise Exception(
                    'Est30: Pessoa id nao encontrada para a excluir o concorrente. Pessoa_id - %s, Codigo - %s' %
                    (str(pessoa_id), dados_concorrente_codigo)
                )
            else:
                db_registro.delete(
                    table='concorrencia_concorrente',
                    where=[('pessoa_id', '=', pessoa_id), ('company_id', '=', concorrente_company_id)]
                )

                args_est30 = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est30'),
                    ('chave_legado', '=', dados_concorrente_codigo)
                ]
                db_registro.delete_legado_controle(
                    reg_args=args_est30,
                    reg_check_table='concorrencia_concorrente',
                )

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        msg_erro += ' \n%s' % 'OBS - MODULO CONCORRENCIA DEVE ESTAR INSTALADO.'
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# POLITICA ESPECIAL
def gerarEst33(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _chave = _item + '-' + _grade
        _inicio = sigDatas(lista_dados[2]).getDataNoFormato('N', 'S', '/')
        _fim = sigDatas(lista_dados[3]).getDataNoFormato('N', 'S', '/')

        if _inicio == '00/00/0000':
            raise Exception('Est33: Data de inicio invalida - %s' % _inicio)

        if _fim == '00/00/0000':
            raise Exception('Est33: Data de final invalida - %s' % _fim)

        _valor = lista_dados[4][:9] + '.' + lista_dados[4][9:]
        _usuario = lista_dados[5].strip()

        produto_company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(
            table='estoque_produto', fields=['id'], where=[('company_id', '=', produto_company_id), ('codigo', '=', _item)]
        )
        if not _regProduto:
            raise Exception('Est33: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']

        automacao_company_id = funct_get_empresa(db_registro, 'automacao.precos.promocao', company_com_nf_id)
        valsPreco = {
            'produto_id': _produto_id,
            'company_id': automacao_company_id,
            'data_inicio': _inicio,
            'data_termino': _fim,
            'preco_venda': _valor
        }

        args_promocao = [('produto_id', '=', _produto_id),
                         ('company_id', '=', automacao_company_id)]

        _regGrade = db_registro.select(
            table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id),
                                                                 ('grade', '=', _grade)]
        )
        if _regGrade:
            _grade_id = _regGrade[0]['id']
            valsPreco['grade_id'] = _grade_id
            args_promocao.append(('grade_id', '=', _grade_id))

        regPreco = db_registro.select(
            table='automacao_precos_promocao', fields=['id'], where=args_promocao)
        if operacao in ('w', 'r'):  # Gravar e regravar
            if regPreco:
                preco_id = regPreco[0]['id']
                db_registro.update(table='automacao_precos_promocao', values=valsPreco, where=[('id', '=', preco_id)])
            else:
                db_registro.insert(table='automacao_precos_promocao', vals=valsPreco)

        elif operacao == 'd':  # Deletar
            if regPreco:
                preco_id = regPreco[0]['id']
                db_registro.delete(table='automacao_precos_promocao', where=[('id', '=', preco_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# PEDIDO DE FORNECEDOR (CAPA)
def gerarEst38(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _pedido = lista_dados[0]
        # _chave = _pedido
        _fornecedor = lista_dados[1]
        # _representante = lista_dados[2]
        _data_pedido = lista_dados[3]
        _data_entrega = lista_dados[4]
        _transportador = lista_dados[5]
        _vencimento1 = sigDatas(lista_dados[6]).getDataNoFormato('N', 'S', '/')
        _vencimento2 = sigDatas(lista_dados[7]).getDataNoFormato('N', 'S', '/')
        _vencimento3 = sigDatas(lista_dados[8]).getDataNoFormato('N', 'S', '/')
        _vencimento4 = sigDatas(lista_dados[9]).getDataNoFormato('N', 'S', '/')
        _vencimento5 = sigDatas(lista_dados[10]).getDataNoFormato('N', 'S', '/')
        _vencimento6 = sigDatas(lista_dados[11]).getDataNoFormato('N', 'S', '/')
        _obs1 = lista_dados[12].strip()
        _obs2 = lista_dados[13].strip()
        _obs3 = lista_dados[14].strip()
        _obs4 = lista_dados[15].strip()
        # _data_faturamento = sigDatas(lista_dados[16]).getDataNoFormato('N', 'S', '/')
        _obs5 = lista_dados[17].strip()
        _obs6 = lista_dados[18].strip()
        # _fim = sigDatas(lista_dados[3]).getDataNoFormato('N', 'S', '/')
        _observacao = (_obs1 + ' ' + _obs2 + ' ' + _obs3 + ' ' + _obs4 + ' ' + _obs5 + ' ' + _obs6).strip()

        _observacao = funct_retira_acentos(_observacao)

        company_id = funct_get_empresa(db_registro, 'estoquepedidos.pedido.capa', company_com_nf_id)

        # fixme: pega a primeira condicao de pagamento. Esta certo???
        condicao_pgto_dados = db_registro(table='adm_condicao', fields=['id'])
        _condicao_pgto_id = condicao_pgto_dados[0]['id']

        _fornecedor_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_fornecedor,
            reg_tabela_legado='cpg01',
            reg_check_table='adm_pessoa',
        )

        valsPedido = {
            'company_id': company_id,
            'numero_pedido': _pedido,
            'data_emissao': _data_pedido,
            'condicao_pgto_id': _condicao_pgto_id,
            'observacao': _observacao,
            'tipo': 'C',
            'pessoa_id': _fornecedor_id,
            'state': 'V'
        }

        if _data_entrega != '00000000':
            valsPedido['data_entrega'] = _data_entrega

        if _transportador != '00000':
            legado_cpg01_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=_transportador,
                reg_tabela_legado='cpg01',
                reg_check_table='adm_pessoa',
            )
            if legado_cpg01_id:
                _regTransportador = db_registro.select(table='adm_transportador', fields=['id'], where=[('pessoa_id', '=', legado_cpg01_id)])
                if _regTransportador:
                    _transportador_id = _regTransportador[0]['id']
                else:
                    _transportador_id = db_registro.insert(table='adm_transportador', vals={'pessoa_id': legado_cpg01_id})
                valsPedido['transportador_id'] = _transportador_id

        _regPedido = db_registro.select(table='estoquepedidos_pedido_capa', fields=['id'], where=[('numero_pedido', '=', _pedido)])

        if operacao in ('w', 'r'):  # Gravar e regravar
            if _regPedido:
                pedido_id = _regPedido[0]['id']
                db_registro.update(table='estoquepedidos_pedido_capa', values=valsPedido, where=[('id', '=', pedido_id)])
            else:
                _idPedido = db_registro.insert(table='estoquepedidos_pedido_capa', vals=valsPedido)
                listVencimento = [_vencimento1, _vencimento2, _vencimento3, _vencimento4, _vencimento5, _vencimento6]
                for venc in listVencimento:
                    if venc != '00/00/0000':
                        valsParcela = {
                            'data_vencimento': venc,
                            'pedido_id': _idPedido,
                            'vr_parcela': 0.01
                        }
                        db_registro.insert(table='estoquepedidos_pedido_parcela', vals=valsParcela)

        elif operacao == 'd':  # Deletar
            if _regPedido:
                pedido_id = _regPedido[0]['id']
                db_registro.delete(table='estoquepedidos_pedido_capa', where=[('id', '=', pedido_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# PEDIDO DE FORNECEDOR (ITENS)
def gerarEst39(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _pedido = lista_dados[0]
        # _sequencia = lista_dados[1]

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[2],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[2].strip()
        _grade = lista_dados[3].strip()
        if _grade == '':
            _grade = 'UN'
        _quantidade = lista_dados[4]
        _valor = lista_dados[5]
        _qtd_entregue = lista_dados[6]
        # _ipi = lista_dados[7]
        _local = lista_dados[8]
        # _bonificacao = lista_dados[9].strip()
        # _saldo_na_hora = lista_dados[10].strip()

        _regPedido = db_registro.select(table='estoquepedidos_pedido_capa', fields=['id'], where=[('numero_pedido', '=', _pedido)])
        if not _regPedido:
            raise Exception('Est39: Pedido nao cadastrado - Numero_pedido - %s' % _pedido)

        _pedido_id = _regPedido[0]['id']

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _item)])
        if not _regProduto:
            raise Exception('Est39: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _chave = '%s-%s' % (_item, _grade)
        _gradeDados = db_registro.select(table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)])
        if not _gradeDados:
            raise Exception('Est39: Item sem grade. Produto_id - %s, Grade - %s' % (str(_produto_id), _grade))

        _grade_id = _gradeDados[0]['id']

        # achar local_id
        company_id = funct_get_empresa(db_registro, 'estoque.local', company_com_nf_id)

        regsLocal = db_registro.select(table='estoque_local', fields=['id'], where=[('numero', '=', _local), ('company_id', '=', company_id)])
        if not regsLocal:
            raise Exception('Est39 - Local nao cadastrado. Numero - %s' % _local)

        _local_id = regsLocal[0]['id']

        valsPedido = {
            'grade_id': _grade_id,
            'local_id': _local_id,
            'pedido_id': _pedido_id,
            'produto_id': _produto_id,
            'qtde_pedida': _quantidade,
            'valor_total': _valor,
            'vr_unitario': float(_valor) / float(_quantidade),
        }
        if _qtd_entregue != '000000000.000':
            valsPedido['qtde_atendida'] = _qtd_entregue
        args_pedido_item = [
            ('pedido_id', '=', _pedido_id), ('produto_id', '=', _produto_id), ('grade_id', '=', _grade_id)
        ]
        _regPedidoItem = db_registro.select(table='estoquepedidos_pedido_item', fields=['id'], where=args_pedido_item)

        if operacao in ('w', 'r'):  # Gravar e regravar
            if _regPedidoItem:
                _pedido_item_id = _regPedidoItem[0]['id']
                db_registro.update(table='estoquepedidos_pedido_item', values=valsPedido, where=[('id', '=', _pedido_item_id)])
            else:
                db_registro.insert(table='estoquepedidos_pedido_item', vals=valsPedido)

            valor = 0
            regItens = db_registro.select(table='estoquepedidos_pedido_item', where=[('pedido_id', '=', _pedido_id)])
            for reg in regItens:
                valor += reg['valor_total']

            db_registro.update(table='estoquepedidos_pedido_capa', values={'valor_total': valor}, where=[('id', '=', _pedido_id)])
            regParcelas = db_registro.select(table='estoquepedidos_pedido_parcela', fields=['id'], where=[('pedido_id', '=', _pedido_id)])
            if regParcelas:
                _ids = []
                for reg in regParcelas:
                    _ids.append(reg['id'])
                qtde = len(regParcelas)
                if qtde > 0:
                    _valorParcela = valor / qtde
                    db_registro.update(
                        table='estoquepedidos_pedido_parcela', values={'vr_parcela': _valorParcela}, where=[('id', 'in', tuple(_ids))])

        elif operacao == 'd':  # Deletar
            if _regPedidoItem:
                _pedido_item_id = _regPedidoItem[0]['id']
                db_registro.delete(table='estoquepedidos_pedido_capa', where=[('id', '=', _pedido_item_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# INVENTARIO
def gerarEst53(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')
        _ano = lista_dados[0]
        _mes = lista_dados[1]
        _local = lista_dados[2]

        if not int(_local):  # se o local for 00 nao irá cadastrar nada pois nao existe
            return True

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[3],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[3].strip()
        _grade = lista_dados[4].strip()
        if _grade == '':
            _grade = 'UN'
        chave_est53i_parcial = _item + '-' + _grade
        # _unidade = lista_dados[5]
        # _qtdEmbalagem = lista_dados[6][:5] + '.' + lista_dados[6][5:]
        _contado = lista_dados[7][:10] + '.' + lista_dados[7][10:]
        _saldoCadastroCNF = lista_dados[8][:10] + '.' + lista_dados[8][10:]
        _saldoCadastroSNF = lista_dados[9][:10] + '.' + lista_dados[9][10:]
        _custoMedio = lista_dados[10][:11] + '.' + lista_dados[10][11:]
        _precoCusto = lista_dados[11][:11] + '.' + lista_dados[11][11:]
        _jaAtualizou = lista_dados[12]
        if _jaAtualizou in ('Y', 'S'):
            _jaAtualizou = True
        else:
            _jaAtualizou = False
            # _quemContou = lista_dados[13].strip()
        _emAndamento = lista_dados[14][:10] + '.' + lista_dados[14][10:]
        # _codTributacao = lista_dados[15]

        inv_capa_company_id = funct_get_empresa(db_registro, 'estoque.inventario.capa', company_com_nf_id)
        produto_company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(
            table='estoque_produto', where=[('company_id', '=', produto_company_id), ('codigo', '=', _item)]
        )
        if not _regProduto:
            raise Exception('Est53 - Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _gradeDados = db_registro.select(
            table='estoque_produto_grade', fields=['id'], where=[('produto_id', '=', _produto_id), ('grade', '=', _grade)]
        )
        if not _gradeDados:
            raise Exception('Est53: Grade nao cadastrada. Grade - %s' % _grade)

        _grade_id = _gradeDados[0]['id']

        # CHAVE SIGERP NA LEGADO CONTROLE PARA O EST53C
        chave_est53c = _ano + _mes
        legado_est53c_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=chave_est53c,
            reg_tabela_legado='est53C',
            reg_check_table='estoque_inventario_capa',
        )

        if legado_est53c_id:
            chave_est53i = str(legado_est53c_id) + chave_est53i_parcial + _local
            # CHAVE SIGERP NA LEGADO CONTROLE PARA O EST53I
            legado_est53i_id = db_registro.controle_get_chave_sigerp(
                reg_empresa_legado=empresa_legado,
                reg_chave=chave_est53i,
                reg_tabela_legado='est53I',
                reg_check_table='estoque_inventario_contagem',
            )

        else:
            legado_est53i_id = False

        local_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_local,
            reg_tabela_legado='est15',
            reg_check_table='estoque_local',
        )

        if not local_id:
            raise Exception(
                'Est53: Locao nao encontrado na legado_controle. Chave_legado - %s, Tabela_legado - est15, Empresa - %s'
                % (_local, str(empresa_legado))
            )

        if operacao in ('w', 'r'):

            if not legado_est53c_id:
                vals_inv_capa = {
                    'descricao': '%s-%s' % (_ano, _mes),
                    'company_id': inv_capa_company_id,
                    'ano': _ano,
                    'mes': _mes,
                    'create_date': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                    'create_uid': db_registro.uid,
                    'atualizado': _jaAtualizou
                }
                # CRICACAO DA CAPA DO INVENTARIO
                inv_capa_id = db_registro.insert(table='estoque_inventario_capa', vals=vals_inv_capa)
            else:
                # ATUALIZACAO DA CAPA DO INVENTARIO
                inv_capa_id = legado_est53c_id
                values_inv_capa = {'atualizado': _jaAtualizou}
                args_update_inv_capa = [('id', '=', inv_capa_id)]
                db_registro.update(table='estoque_inventario_capa', values=values_inv_capa, where=args_update_inv_capa)

            # ITEMS DO INVENTARIO
            vals_inv_item = {
                'inventario_id': inv_capa_id,
                'produto_id': _produto_id,
                'grade_id': _grade_id,
                'local_id': local_id,
                'estict_custo_medio_contabil': _custoMedio,
                'estict_ultimo_custo': _precoCusto,
                'conferido': _jaAtualizou,
                'saldo_cadastro': float(_saldoCadastroCNF) + float(_saldoCadastroSNF),
                'saldo_contado': _contado,
                'requisitado': _emAndamento,
            }

            if not legado_est53i_id:
                inv_item_id = db_registro.insert(table='estoque_inventario_contagem', vals=vals_inv_item)
            else:
                inv_item_id = legado_est53i_id
                db_registro.update(
                    table='estoque_inventario_contagem', values=vals_inv_item, where=[('id', '=', inv_item_id)]
                )

            # INVENTARIO CAPA
            if not legado_est53c_id:
                # CRIACAO DA LEGADO CONTROLE
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est53C',
                    reg_chave_legado=chave_est53c,
                    reg_chave_sigerp=inv_capa_id,
                    reg_check_table='estoque_inventario_capa',
                )
            else:
                # ATUALIZACAO DA LEGADO CONTROLE
                args_est53c = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est53C'),
                    ('chave_legado', '=', chave_est53c)
                ]
                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': inv_capa_id},
                    reg_args=args_est53c,
                    reg_check_table='estoque_inventario_capa',
                )

            # INVENTARIO ITEM
            chave_est53i = str(inv_capa_id) + chave_est53i_parcial + _local
            if not legado_est53i_id:
                # CRIACAO DA LEGADO CONTROLE
                db_registro.create_legado_controle(
                    reg_empresa_legado=empresa_legado,
                    reg_tabela_legado='est53I',
                    reg_chave_legado=chave_est53i,
                    reg_chave_sigerp=inv_item_id,
                    reg_check_table='estoque_inventario_contagem',
                )
            else:
                # ATUALIZACAO DA LEGADO CONTROLE
                args_est53c = [
                    ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est53I'),
                    ('chave_legado', '=', chave_est53i)
                ]
                db_registro.update_legado_controle(
                    reg_values={'chave_sigerp': inv_item_id},
                    reg_args=args_est53c,
                    reg_check_table='estoque_inventario_contagem',
                )

        elif operacao == 'd':  # Deletar
            chave_est53i = str(legado_est53c_id) + chave_est53i_parcial + _local
            if not legado_est53i_id:
                raise Exception(
                    'Est53: Inventario(item) nao localizado na legado_controle para ser alterado. '
                    'Chave_legado - %s, Empresa - %s' %
                    (chave_est53i, str(empresa_legado))
                )

            db_registro.delete(table='estoque_inventario_contagem', where=[('id', '=',  legado_est53i_id)])
            args_legado_controle = [
                ('empresa', '=', empresa_legado), ('tabela_legado', '=', 'est53I'),
                ('chave_legado', '=', chave_est53i)
            ]
            db_registro.delete_legado_controle(reg_args=args_legado_controle, reg_check_table='args_legado_controle')

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# PRECO DE CUSTO E VENDA
def gerarEst55(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _chave = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                               context=context, company_com_nf_id=company_com_nf_id)
        else:
            _chave = lista_dados[0].strip()
        _pVenda1 = lista_dados[1][:11] + '.' + lista_dados[1][11:]
        # _pVenda2 = lista_dados[2][:11] + '.' + lista_dados[2][11:]
        # _tabelaCorrecao = lista_dados[3].strip().upper()
        #_indexaPVenda = lista_dados[4]
        #if _indexaPVenda == 'Y':
        #    _indexaPVenda = True
        #else:
        #    _indexaPVenda = False
        # _nomeIndexador = lista_dados[5].strip().upper()
        _pCusto = lista_dados[6][:10] + '.' + lista_dados[6][10:]
        _pMedio = lista_dados[7][:11] + '.' + lista_dados[7][11:]
        _pReposicao = lista_dados[8][:11] + '.' + lista_dados[8][11:]
        _custoCorr = lista_dados[9][:9] + '.' + lista_dados[9][9:]
        # _ultimoFornec = lista_dados[10]
        # _penulPCusto = lista_dados[11][:10] + '.' + lista_dados[11][10:]
        # _penulPMedio = lista_dados[12][:11] + '.' + lista_dados[12][11:]
        # _penulCustoCorr = lista_dados[13][:9] + '.' + lista_dados[13][9:]
        # _penulPReposicao = lista_dados[14][:11] + '.' + lista_dados[14][11:]
        #_penulFornec = lista_dados[15]
        dados_usuario = lista_dados[16]

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(table='estoque_produto', fields=['id'], where=[('company_id', '=', company_id), ('codigo', '=', _chave)])
        if not _regProduto:
            raise Exception('Est55: Item nao cadastrado. Codigo - %s' % _chave)

        _produto_id = _regProduto[0]['id']

        preco_custo_company_id = funct_get_empresa(db_registro, 'estoque.preco.custo', company_com_nf_id)

        agora = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        args_preco_custo = [
            ('produto_id', '=', _produto_id), ('create_date', '<=', agora), ('company_id', '=', preco_custo_company_id)
        ]
        regCusto = db_registro.select(table='estoque_preco_custo',
                                      fields=['id', 'estprc_custo_corrigido', 'estprc_ultimo_custo',
                                              'estprc_custo_medio_contabil', 'estprc_custo_reposicao'],
                                      where=args_preco_custo,
                                      order_by=['create_date desc'])

        preco_venda_company_id = funct_get_empresa(db_registro, 'estoque.preco.venda', company_com_nf_id)

        # FIXME: VERIFICAR A HORA
        args_prevo_venda = [
            ('produto_id', '=', _produto_id), ('create_date', '<=', agora), ('company_id', '=', preco_venda_company_id)
        ]

        regvenda = db_registro.select(
            table='estoque_preco_venda',
            fields=['id', 'preco_venda'],
            where=args_prevo_venda,
            order_by=['create_date desc']
        )
        try:
            _custoCorr = float(_custoCorr)
        except:
            _custoCorr = 0.0

        try:
            _pMedio = float(_pMedio)
        except:
            _pMedio = 0.0

        try:
            _pCusto = float(_pCusto)
        except:
            _pCusto = 0.0

        try:
            _pReposicao = float(_pReposicao)
        except:
            _pReposicao = 0.0
        try:
            dados_usuario = int(dados_usuario)
            erp_user_id = int(dados_usuario)
        except:
            dados_usuario = 0
            erp_user_id = 0

        if erp_user_id:
            user_dados = db_registro.select(table='res_users', fields=['id'], where=[('id', '=', erp_user_id)])
            if not user_dados:
                raise Exception('Est55: Usuario informado nao esta cadastrado no ERP. Usuario - %s' % str(erp_user_id))
        else:
            erp_user_id = 1

        # PRECO DE CUSTO
        vals_preco_custo = {
            'produto_id': _produto_id,
            'company_id': preco_custo_company_id,
            'estprc_custo_corrigido': _custoCorr,
            'estprc_custo_medio_contabil': _pMedio,
            'estprc_ultimo_custo': _pCusto,
            'estprc_custo_reposicao': _pReposicao,
            'active': True,
        }

        if regCusto:
            if (regCusto[0]['estprc_ultimo_custo'] != float(_pCusto) or
                regCusto[0]['estprc_custo_medio_contabil'] != float(_pMedio) or
                regCusto[0]['estprc_custo_corrigido'] != float(_custoCorr) or
                regCusto[0]['estprc_custo_reposicao'] != float(_pReposicao)):
                  db_registro.insert(table='estoque_preco_custo', vals=vals_preco_custo)
        else:
            db_registro.insert(table='estoque_preco_custo', vals=vals_preco_custo)

        # PRECO DE VENDA
        vals_preco_venda = {
            'produto_id': _produto_id,
            'company_id': preco_venda_company_id,
            'write_date': agora,
            'preco_venda': _pVenda1,
            'create_uid': erp_user_id
        }

        if regvenda:
            preco_venda_id = regvenda[0]['id']
            preco_venda_valor = regvenda[0]['preco_venda']

            if preco_venda_valor != float(_pVenda1):
                db_registro.insert(table='estoque_preco_venda', vals=vals_preco_venda)
            elif dados_usuario:
                today = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                db_registro.update(
                    table='estoque_preco_venda',
                    values={'create_date': today, 'create_uid': erp_user_id},
                    where=[('id', '=', preco_venda_id)]
                )
        else:
            db_registro.insert(table='estoque_preco_venda', vals=vals_preco_venda)

        obj_estoque_produto = context.get('ctx_est_obj_estoque_produto')
        #Sempre commitar a conexao da integracao antes de chamar o metodo write do openerp
        db_registro.cr_integracao.connection.commit()
        obj_estoque_produto.write(db_registro.cr, db_registro.uid, [_produto_id], {'write_date': agora}, context)

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False


# CODIGOS DE PRODUTOS PARA OS FORNECEDORES
def gerarEst59(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):
    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            _item = est_format_produto_codigo(db_registro, produto_codigo=lista_dados[0],
                                              context=context, company_com_nf_id=company_com_nf_id)
        else:
            _item = lista_dados[0].strip()
        _grade = lista_dados[1].strip()
        if _grade == '':
            _grade = 'UN'
        _fornecedor = lista_dados[2].strip()
        _codFornecedor = lista_dados[3].strip()

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        _regProduto = db_registro.select(
            table='estoque_produto',
            fields=['id', 'fabricante_id'],
            where=[('company_id', '=', company_id), ('codigo', '=', _item)]
        )
        if not _regProduto:
            raise Exception('Est59: Item nao cadastrado. Codigo - %s' % _item)

        _produto_id = _regProduto[0]['id']
        _fabricante_id = _regProduto[0]['fabricante_id']
        args_produto_grade = [('produto_id', '=', _produto_id), ('grade', '=', _grade)]

        _regGrade = db_registro.select(table='estoque_produto_grade', fields=['id'], where=args_produto_grade)
        if not _regGrade:
            raise Exception('Est59: Grade nao cadastrada. Grade - %s' % _grade)

        _grade_id = _regGrade[0]['id']

        _pessoa_id = db_registro.controle_get_chave_sigerp(
            reg_empresa_legado=empresa_legado,
            reg_chave=_fornecedor,
            reg_tabela_legado='cpg01',
            reg_check_table='adm_pessoa',
        )

        if not _pessoa_id:
            raise Exception(
                'Est59: Pessoa nao cadastrada na legado_controle. Chave_legado - %s, Tabela_legado - cpg01, Empresa - %s'
                % (_fornecedor, str(empresa_legado))
            )

        company_id = funct_get_empresa(db_registro, 'financeiro.fornecedor', company_com_nf_id)

        _regFornecedor = db_registro.select(
            table='financeiro_fornecedor', fields=['id'], where=[('company_id', '=', company_id), ('pessoa_id', '=', _pessoa_id)]
        )
        if _regFornecedor:
            _fornecedor_id = _regFornecedor[0]['id']
        else:
            raise Exception('Est59: Fornecedor nao cadastrado. Pessoa_id - %s' % _pessoa_id)

        vals = {
            'codigo_fornecedor': _codFornecedor,
            'fornecedor_id': _fornecedor_id,
            'grade_id': _grade_id,
            'produto_id': _produto_id,
        }

        args_produto_codigo_fornecedor = [
            ('produto_id', '=', _produto_id), ('grade_id', '=', _grade_id), ('codigo_fornecedor', '=', _codFornecedor)
        ]
        regCodFornecedor = db_registro.select(
            table='estoque_produto_codigo_fornecedor', fields=['id'], where=args_produto_codigo_fornecedor
        )
        if operacao in ('w', 'r'):  # Gravar e Regravar
            if regCodFornecedor:
                codfornec_id = regCodFornecedor[0]['id']
                db_registro.update(table='estoque_produto_codigo_fornecedor', values=vals, where=[('id', '=', codfornec_id)])
            else:
                db_registro.insert(table='estoque_produto_codigo_fornecedor', vals=vals)

            if _fabricante_id and _grade_id:
                db_registro.update(
                    table='estoque_produto_grade',
                    values={'codigo_no_fabricante': _codFornecedor},
                    where=[('id', '=', _grade_id)]
                )

        elif operacao == 'd':  # Deletar
            if regCodFornecedor:
                codfornec_id = regCodFornecedor[0]['id']
                db_registro.delete(table='estoque_produto_codigo_fornecedor', where=[('id', '=', codfornec_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False

# KIT
def gerarEst62(db_registro, legado_log_id, dados, company_com_nf_id, company_sem_nf_id, empresa_legado, operacao, context=None):

    try:
        lista_dados = dados.replace('#', ' ').replace('\n', '').split('*|*')

        dict_company_utilizar_cod_numerico = context.get('ctx_est_utilizar_codigo_numerico')
        try:
            _codNumerico = dict_company_utilizar_cod_numerico[company_com_nf_id]
        except KeyError:
            raise EstoqueParametroError(value=empresa_legado)

        if _codNumerico:
            dados_produto_codigo = est_format_produto_codigo(db_registro,
                                                             produto_codigo=lista_dados[0],
                                                             context=context,
                                                             company_com_nf_id=company_com_nf_id)
            dados_produto_composicao_codigo = est_format_produto_codigo(db_registro,
                                                                        produto_codigo=lista_dados[1],
                                                                        context=context,
                                                                        company_com_nf_id=company_com_nf_id)
        else:
            dados_produto_codigo = lista_dados[0].strip()
            dados_produto_composicao_codigo = lista_dados[1].strip()

        dados_grade = lista_dados[2].strip()

        if not dados_grade:
            dados_grade = 'UN'

        dados_quantidade = '%s.%s' % (lista_dados[3][:8], lista_dados[3][8:11])

        company_id = funct_get_empresa(db_registro, 'estoque.produto', company_com_nf_id)

        def est62_get_produto(codigo):
            args_produto = [
                ('codigo', '=', codigo), ('company_id', '=', company_id)
            ]
            produto_dados = db_registro.select(table='estoque_produto', fields=['id'], where=args_produto)
            if not produto_dados:
                res = False
            else:
                res = produto_dados[0]['id']
            return res

        # PRODUTO FINAL (KIT)
        produto_final_id = est62_get_produto(codigo=dados_produto_codigo)
        if not produto_final_id:
            raise Exception(
                'Est62: Produto kit nao cadastrado. Codigo - %s, Company_id - %s' %
                (str(dados_produto_codigo), str(company_id))
            )
        # VERIFICA SE O PRODUTO ENCONTRADO ESTA MARCADO COMO KIT
        produto_kit = db_registro.select(
            table='estoque_produto', fields=['id'], where=[('id', '=', produto_final_id), ('kit_de_produtos', '=', True)]
        )
        if not produto_kit:
            raise Exception('Est62: Produto nao cadastrado como kit. Codigo - %s' % dados_produto_codigo)

            # PRODUTO QUE COMPONHE O KIT
        produto_composicao_id = est62_get_produto(codigo=dados_produto_composicao_codigo)
        if not produto_composicao_id:
            raise Exception(
                'Est62: Produto que componhe o kit nao encontrado. Codigo - %s, Company_id - %s' %
                (str(dados_produto_composicao_codigo), str(company_id))
            )
        # GRADE DO ITEM QUE COMPONHE O KIT
        args_grade = [('produto_id', '=', produto_composicao_id), ('grade', '=', dados_grade)]
        grade_dados = db_registro.select(table='estoque_produto_grade', fields=['id'], where=args_grade)
        if not grade_dados:
            raise Exception(
                'Est62: Grade nao encontrada para o produto que componhe o kit. Grade - %s, Produto_id - %s' %
                (str(dados_grade), str(produto_composicao_id))
            )
        grade_id = grade_dados[0]['id']

        args_kit_produto = [
            ('kitpro_produto_id', '=', produto_final_id), ('kitpro_item_produto_id', '=', produto_composicao_id),
            ('kitpro_grade_id', '=', grade_id)
        ]

        kit_dados = db_registro.select(table='kit_produto', fields=['id'], where=args_kit_produto)

        if operacao in ('w', 'r'):
            if kit_dados:
                # ATUALIZA O KIT JA EXISTENTE
                kit_id = kit_dados[0]['id']
                db_registro.update(
                    table='kit_produto', values={'kitpro_quantidade': dados_quantidade}, where=[('id', '=', kit_id)]
                )
            else:
                dados_automacao = db_registro.select(table='automacao_codigo_barra',
                                                     fields=['produto_embalagem_id'],
                    where=[('produto_id', '=', produto_composicao_id), ('grade_id', '=', grade_id),
                           ('active', '=', True)])
                if dados_automacao:
                    prod_embalagem_id = dados_automacao[0].get('produto_embalagem_id')
                else:
                    dados_embalagem = db_registro.select(table='estoque_produto_embalagem',
                                                         fields=['id'],
                    where=[('produto_id', '=', produto_composicao_id),
                           ('grade_id', '=', grade_id),
                           ('active', '=', True)])
                    if dados_embalagem:
                        prod_embalagem_id = dados_embalagem[0].get('id')

                # RELACIONA OS ITENS QUE COMPONHE O KIT
                vals_kit = {
                    'kitpro_produto_id': produto_final_id,
                    'kitpro_item_produto_id': produto_composicao_id,
                    'kitpro_grade_id': grade_id,
                    'kitpro_quantidade': dados_quantidade,
                    'kitpro_produto_embalagem_id': prod_embalagem_id,
                }
                db_registro.insert(table='kit_produto', vals=vals_kit)

        elif operacao == 'd':
            if not kit_dados:
                raise Exception(
                    'Est62: Delecao nao realizada. Kit nao encontrado. '
                    'Produto_kit_id - %s, Produto_id - %s, Grade_id - %s' %
                    (str(produto_final_id), str(produto_composicao_id), str(grade_id))
                )
            else:
                kit_id = kit_dados[0]['id']
                db_registro.delete(table='kit_produto', where=[('id', '=', kit_id)])

        return True

    except TransactionRollbackError:
        return False

    except:
        db_registro.cr_integracao.connection.rollback()
        msg_erro = traceback_format_exc().replace("'", '"')
        db_registro.registro_gravar_erro(legado_log_id, msg_erro)
        return False
